<?php

/**
 * General Class
 * For more information contact  support@alenet.com
 *
 * @author       $Author: cramirez $
 * @copyright    Alenet, Inc.
 * @since        $Date: 2018-10-09 11:22:02 -0400 (Tue, 09 Oct 2018) $
 * @version        $Rev: 10222 $
 * @package		Clientix.Classes.Network
 * @abstract		General Class
 * CHANGE LOG
 *  Included label
 *  Class renamed to clx_ftp
 */
class clx_ftp {

    var $cnx;
    var $address;
    var $username;
    var $password;
    var $port = CLX_NUM_21;
    var $timeout = CLX_NUM_30; //seconds
    var $status;
    var $error;
    var $errornum;
    var $errormsg;
    var $verbose = true;

    /**
     * Contructor
     *
     * @param string $name
     * @param string $address
     * @param string $username
     * @param string $password
     */
    function __construct($address, $username, $password, $port = CLX_NUM_21, $verbose = false) {
        $this->address = $address;
        $this->username = $username;
        $this->password = $password;
        $this->cnx = CLX_INIT_ZERO;
        $this->status = 'IDLE';
        $this->error = false;
        $this->errornum = CLX_INIT_ZERO;
        $this->errormsg = '';
        $this->port = $port;
        $this->verbose = $verbose;
    }

    /**
     * Connects to the server
     */
    function connect() {
        $contenidoHTML = '';
        if ($this->verbose === true) {
            $contenidoHTML.= 'Connecting to server: ' . $this->address . '<br>n';
        }
        if (($this->cnx = ftp_connect($this->address, $this->port, $this->timeout)) === true) {
            if (ftp_login($this->cnx, $this->username, $this->password)) {
                $this->status = 'CONNECTED';
            } else {
                $this->setError(CLX_NUM_ONE, 'Can' . CLX_CHAR_SINGLE_QUOTE . 't login to server ' . $this->address);
            }
        } else {
            $this->setError(CLX_NUM_ONE, 'Can' . CLX_CHAR_SINGLE_QUOTE . 't connect to server ' . $this->address);
        }
        echo $contenidoHTML;
    }

    /**
     * Close connection to ftp server
     */
    function close() {
        $contenidoHTML = '';
        if ($this->verbose === true) {
            $contenidoHTML.= 'Disconnecting to server: ' . $this->address . '<br>' . CLX_CHAR_NL;
        }
        if ($this->status === 'CONNECTED') {
            ftp_close($this->cnx);
        } else {
            $this->setError(CLX_NUM_ONE, 'Can' . CLX_CHAR_SINGLE_QUOTE . 't close, not connected');
        }
        echo $contenidoHTML;
    }

    /**
     * return current working directory
     *
     * @return string
     */
    function pwd() {
        if ($this->cnx === true) {
            return ftp_pwd($this->cnx);
        } else {
            $this->setError(CLX_NUM_ONE, 'Can' . CLX_CHAR_SINGLE_QUOTE . 't get current working directory, not connected');
        }
    }

    /**
     * Changes or creates a new directory
     *
     * @param string $newdirectorypath
     * @param boolean $create_if_not_exists
     */
    function change_directory($newdirectorypath, $create_if_not_exists = false) {
        $contenidoHTML = '';
        $pos = strpos($newdirectorypath, '/');
        if (false === $pos) {
            $newdirectory = $newdirectorypath;
            $newdirectorypath = '';
        } else {
            $newdirectory = substr($newdirectorypath, CLX_NUM_ZERO, $pos);
            $newdirectorypath = substr($newdirectorypath, $pos + CLX_NUM_ONE);
        }
        if (!empty($this->cnx)) {
            if (ftp_chdir($this->cnx, $newdirectory)) {
                if (!empty($this->verbose))
                    $contenidoHTML.= 'Changed directory to ' . $newdirectory . '<br>' . CLX_CHAR_NL;
            } elseif ($create_if_not_exists === true) {
                $this->create_directory($newdirectory);
                $this->change_directory($newdirectory);
            } else {
                $this->setError(CLX_NUM_ONE, 'Can' . CLX_CHAR_SINGLE_QUOTE . 't change current working directory to ' . $newdirectory);
            }
        } else {
            $this->setError(CLX_NUM_ONE, 'Can' . CLX_CHAR_SINGLE_QUOTE . 't change directory, not connected');
        }
        if (empty($this->error) and strlen($newdirectorypath) > CLX_NUM_ZERO) {
            $this->change_directory($newdirectorypath, $create_if_not_exists);
        }
        echo $contenidoHTML;
    }

    /**
     * Creates a new directory
     *
     * @param string $newdirectory
     */
    function create_directory($newdirectory) {
        $contenidoHTML = '';
        if ($this->cnx === true) {
            if (ftp_mkdir($this->cnx, $newdirectory)) {
                if ($this->verbose === true)
                    $contenidoHTML.='Created directory: ' . $newdirectory . '<br>' . CLX_CHAR_NL;
            } else {
                $this->setError(CLX_NUM_ONE, 'Can' . CLX_CHAR_SINGLE_QUOTE . 't create directory: ' . $newdirectory);
            }
        } else {
            $this->setError(CLX_NUM_ONE, 'Can' . CLX_CHAR_SINGLE_QUOTE . 't create directory, not connected');
        }
        echo $contenidoHTML;
    }

    /**
     * Return to parent directory
     *
     * @param integer $levels
     */
    function parent_directory($levels = CLX_NUM_ONE) {
        if ($this->cnx === true) {
            if (ftp_cdup($this->cnx)) {
                $levels --;
                if ($levels > CLX_NUM_ZERO) {
                    $this->parent_directory($levels);
                }
            } else {
                $this->setError(CLX_NUM_ONE, 'Can' . CLX_CHAR_SINGLE_QUOTE . 't move to parent directory');
            }
        } else {
            $this->setError(CLX_NUM_ONE, 'Can' . CLX_CHAR_SINGLE_QUOTE . 't move to parent directory, not connected');
        }
    }

    /**
     * Uploads file to server
     *
     * @param string $local_pathname
     * @param string $filename
     */
    function upload($local_pathname, $filename, $binary_mode = true) {
        $contenidoHTML = '';
        $mode = ($binary_mode) ? FTP_BINARY : FTP_ASCII;
        if ($this->verbose === true)
            $contenidoHTML.= '<br>Uploading file $local_pathname as ' . $filename . '<br>' . CLX_CHAR_NL;
        if ($this->cnx === true) {
            if (ftp_put($this->cnx, $filename, $local_pathname, $mode)) {
                if ($this->verbose === true) {
                    $contenidoHTML.= '<br>Uploaded file ' . $local_pathname . ' as ' . $filename . '<br>n';
                }
            } else {
                $this->setError(CLX_NUM_ONE, 'Can' . CLX_CHAR_SINGLE_QUOTE . 't upload file: ' . $local_pathname);
            }
        } else {
            $this->setError(CLX_NUM_ONE, 'Can' . CLX_CHAR_SINGLE_QUOTE . 't upload file, not connected');
        }
        echo $contenidoHTML;
    }

    /**
     * Return a list of files, folders and permitions on the files
     *
     * @param boolean $raw
     * @return unknown multitype:multitype:string
     */
    function catalog($raw = false) {
        if ($this->cnx === true) {
            $raw_catalog = ftp_rawlist($this->cnx, '.');
            if (empty($raw_catalog)) {
                $this->setError(CLX_NUM_ONE, 'Can' . CLX_CHAR_SINGLE_QUOTE . 't get catalog from directory');
            } else {
                if (!empty($raw)) {
                    return $raw_catalog;
                } else {
                    $catalog = array();
                    foreach ($raw_catalog as $raw_file) {
                        $file = array();
                        $file ['type'] = substr($raw_file, CLX_NUM_ZERO, CLX_NUM_ONE);
                        $file ['owner_access'] = substr($raw_file, CLX_NUM_ONE, CLX_NUM_THREE);
                        $file ['group_access'] = substr($raw_file, CLX_NUM_FOUR, CLX_NUM_THREE);
                        $file ['other_access'] = substr($raw_file, CLX_NUM_7, CLX_NUM_THREE);
                        $file ['owner_name'] = trim(substr($raw_file, CLX_NUM_15, CLX_NUM_8));
                        $file ['group_name'] = trim(substr($raw_file, CLX_NUM_24, CLX_NUM_8));
                        $file ['size'] = trim(substr($raw_file, CLX_NUM_33, CLX_NUM_11));
                        $file ['date'] = trim(substr($raw_file, CLX_NUM_46, CLX_NUM_SIX));
                        $file ['time'] = trim(substr($raw_file, CLX_NUM_53, CLX_NUM_FIVE));
                        $file ['name'] = trim(substr($raw_file, CLX_NUM_59));
                        $catalog [] = $file;
                    }
                    return $catalog;
                }
            }
        } else {
            $this->setError(CLX_NUM_ONE, 'Can' . CLX_CHAR_SINGLE_QUOTE . 't get catalog, not connected');
        }
    }

    /**
     * Deletes file on server
     *
     * @param unknown_type $filename
     */
    function delete($filename) {
        $contenidoHTML = '';
        if (!empty($this->cnx)) {
            if (ftp_delete($this->cnx, $filename)) {
                if (!empty($this->verbose)) {
                    $contenidoHTML.= 'Deleted file ' . $filename . '<br>' . CLX_CHAR_NL;
                }
            } else {
                $this->setError(CLX_NUM_ONE, 'Can' . CLX_CHAR_SINGLE_QUOTE . 't delete file: ' . $filename);
            }
        } else {
            $this->setError(CLX_NUM_ONE, 'Can' . CLX_CHAR_SINGLE_QUOTE . 't upload file, not connected');
        }
        echo $contenidoHTML;
    }

    /**
     * Set pasive mode on ftp connection
     * @param unknown_type $status
     */
    function setPasiveMode($status = true) {
        if (!empty($this->cnx)) {
            ftp_pasv($this->cnx, $status);
        }
    }

    /**
     * Sets ftp error
     *
     * @param unknown_type $errornum
     * @param unknown_type $errormsg
     */
    private function setError($errornum, $errormsg) {
        $this->error = true;
        $this->errornum = $errornum;
        $this->errormsg = $errormsg;
    }

}
