<?php

/**
 * General Class
 * For more information contact  supportalenet.com
 *
 * author       $Author: cramirez $
 * copyright    Alenet, Inc.
 * since        $Date: 2018-10-09 11:22:02 -0400 (Tue, 09 Oct 2018) $
 * version        $Rev: 10222 $
 * package		Clientix.Classes.Network
 * abstract		Abstract Class
 * CHANGE LOG
 *  Included label
 * 	: 'a_last_error_msg' property and 'last_error_msg()' method added to be able to report a more clear transmission fail reason to the end user
 */
class clx_socket {

    var $address; // Remote ip address 0.0.0.0
    var $port;  // Remote Port
    var $pointer; // Pointer
    var $debug;  // Debug value
    var $html;  // Set true to display debug on html format
    var $status; // Status of the connection
    var $timeout;   // Socket Connection time out in seconds
    var $read_type; // Read types: PHP_BINARY_READ  or PHP_NORMAL_READ
    var $a_last_error_msg; // text of last error message occurred

    /**
     * Constructor
     * param: address, Socket ip address
     * param: port, Socket port
     * param: html, Allow html output.
     */

    function __construct($address, $port, $html = false, $debug = false) {
        $this->address = (string) $address;
        $this->port = (int) $port;
        $this->pointer = -CLX_NUM_ONE;
        $this->debug = $debug;
        $this->status = (bool) true;
        $this->html = $html;
        $this->timeout = (integer) CLX_NUM_THREE; //Seconds
        $this->read_type = PHP_BINARY_READ;
        ClxFreeReturn($this->create());
        set_error_handler(array($this, 'warningHandler'), E_WARNING);
    }

    /**
     * string getAddress(void)
     * Returns the current socket ip address
     */
    function getAddress() {
        return $this->address;
    }

    /**
     * int getPort(void)
     * Returns the current socket port number
     */
    function getPort() {
        return $this->port;
    }

    /**
     * bool existsPointer(void)
     * Returns true if resource is defined
     */
    function existsPointer() {
        return is_resource($this->pointer);
    }

    /**
     * Warning Handler
     * @param type $errno
     * @param type $errstr
     */
    public function warningHandler($errno, $errstr) {
        if($this->debug){
            echo 'Warning:'.$errno.' '.$errstr.'<br>';
        }
    }

    /**
     * void debug($status = false, $msg = '')
     * Outputs debug information to screen
     */
    function debug($status = false, $msg = '') {
        $contenidoHTML = '';
        if (empty($status)) {
            $this->a_last_error_msg = $this->last_error_msg(true);
        }
        if (!empty($this->debug) && $this->html === true) {
            if (empty($status))
                $contenidoHTML.= $msg . ' <font color="red">Fail </font><i>-' . $this->errorMsg() . '</i><br>';
            else
                $contenidoHTML.= $msg . ' <font color="green">OK</font>' . '<br>';
        }elseif (!empty($this->debug)) {
            if (empty($status))
                $contenidoHTML.= $msg . ' Fail -' . $this->errorMsg() . CLX_CHAR_CR . CLX_CHAR_NL;
            else
                $contenidoHTML.= $msg . ' OK' . CLX_CHAR_CR . CLX_CHAR_NL;
        }
        echo $contenidoHTML;
    }

    /**
     * resource create(void)
     * Creates new sockets and returns socket resource.
     */
    function create() {
        $result = false;
		$this->pointer = socket_create(AF_INET, SOCK_STREAM, SOL_TCP);
        if ($this->existsPointer()) {
            $result = true;
        } else {
            $this->status = false;
        }
        $this->debug($result, 'Create Socket:');
        return $result;
    }

    /**
     * resource connect(void)
     * Creates new connection to the socket.
     */
    function connect() {
        $contenidoHTML = '';
        $result = false;
        if ($this->status === true) {
			$result = socket_connect($this->pointer, $this->address, $this->port);
            if (empty($result))
                $this->status = false;
            $this->debug($result, 'Socket Connection:');
        }
        if (!empty($this->debug)) {
            if ($this->html === true) {
                $contenidoHTML.= 'Socket Timeout:' . $this->timeout . '<Br>';
            } else {
                $contenidoHTML.= 'Socket Timeout:' . $this->timeout . CLX_CHAR_CR . CLX_CHAR_NL;
            }
        }
        echo $contenidoHTML;
        return $result;
    }

    /**
     * bool nonblock(void)
     * Set non socket blocking
     */
    function nonblock() {
        $result = false;
        if ($this->status === true) {
            $result = socket_set_nonblock($this->pointer);
            $this->debug($result, 'Non blocking Socket:');
        }
        return $result;
    }

    /**
     * bool block(void)
     * Set socket blocking
     */
    function block() {
        $result = false;
        if ($this->status === true) {
            $result = socket_set_block($this->pointer);
            $this->debug($result, 'Blocking Socket:');
        }
        return $result;
    }

    function send($request) {
        $result = false;
        if ($this->status === true) {
			$result = socket_send($this->pointer, $request, strlen($request), CLX_NUM_0x1);
            $this->debug($result, 'Socket Send:');
        }
        return $result;
    }

    /**
     * bool write(string $request)
     * Writes request to socket.
     */
    function write($request, $bytes = false) {
        $contenidoHTML = '';
        $result = false;
		if (!empty($this->status)) {
            if (empty($bytes)) {
                if (!empty($this->debug)){
					$contenidoHTML.= 'Request:<br><textarea cols=60 rows=10>start->' . $request . '<-end</textarea><br>';
					$tta=fopen(kPathToWebSite."/".kPathToTemp."socket.txt","w+");
					fwrite($tta,'start->' . $request . '<-end'.chr(13).chr(10));
					fclose($tta); 
				}	
                $result = socket_write($this->pointer, $request, strlen($request)) or ( $this->status = false);
            }else {
                if (!empty($this->debug)){
					$contenidoHTML.= 'Request:<br><textarea cols=60 rows=10>start->' . $request . '<-end</textarea><br>';
				    $tta=fopen(kPathToWebSite."/".kPathToTemp."socket.txt","w+");
					fwrite($tta,'start->' . $request . '<-end'.chr(13).chr(10));
					fclose($tta); 
				}	
                $result = socket_write($this->pointer, $request, $bytes) or ( $this->status = false);
            }
            $this->debug($result, 'Write Total Bytes: ' . $result . ' Socket Write Result:');
        }
        echo $contenidoHTML;
        return $result;
    }

    /**
     * string read(integer $buffer = '2048', string $type = 'nomal')
     * Reads socket contents and return string.
     * param: type normal or binary
     */
    function read($buffer = CLX_NUM_1024, $timeout = CLX_NUM_THREE) {
        $result = '';
        $contenidoHTML = '';
		ClxFreeReturn($this->set_readtimeout(CLX_NUM_THREE));
        while ($out = socket_read($this->pointer, $buffer, $this->read_type)) {
            $result .= $out;
        }

        if (!empty($this->debug)) {
            $contenidoHTML.= '<br><br>Respose:<br><textarea cols=60 rows=10>start->KKKKKK' . $result . '<-end</textarea><br>' . '<br>Read Length:' . strlen($result) . '<br>';
        }
		$this->debug((strlen($result) > CLX_NUM_ZERO), 'Socket Read:');

        if (!empty($this->debug)) {
            echo $contenidoHTML;
        }

        return $result;
    }

    /**
     * 	Method socket read
     */
    function readcv($buffer = '1024') {
        $result = '';
        if ($this->status === true) {
            while ((socket_recv($this->pointer, $out, $buffer, CLX_NUM_ZERO)) === true) {
                $out = ord(substr($out, CLX_NUM_MINUS_1));
                if ($out == CLX_NUM_ZERO) {
                    $result .= substr($out, CLX_NUM_ZERO, CLX_NUM_MINUS_1);
                    break;
                } else {
                    $result .= $out;
                }
            }//end while
            $this->debug((strlen($result) > CLX_NUM_ZERO), 'Socket Read:');
        }
        return $result;
    }

    /**
     *  Method socket close
     */
    function close() {
        $result = false;
        if ($this->status === true) {
            $result = socket_close($this->pointer) or ( $this->status = false);
            $this->debug(true, 'Socket Close:');
        }
        return $result;
    }

    function errorNum() {
        $e = (int) socket_last_error($this->pointer);
        socket_clear_error($this->pointer);
        return $e;
    }

    function errorMsg() {
        $errNum = $this->errorNum();
        return socket_strerror($errNum) . ' (Err Num: ' . $errNum . ')';
    }

    /**
     * array last_error_msg(boolean $status)
     * Returns the last error message
     * param: $status = estatus of last socket operation
     */
    function last_error_msg($status) {
        $e = (int) socket_last_error($this->pointer);
        $a_last_error_msg = array('error' => $status, 'errorNum' => $e, 'errorMsg' => socket_strerror($e));
        return $a_last_error_msg;
    }

    /**
     * bool set_readtimeout(integer $timeout = 10)
     * Set the read time out for a socket
     * param: Integer timeout in seconds
     */
    function set_readtimeout($timeout = CLX_NUM_7) {
        $to = array('sec' => $timeout, 'usec' => CLX_NUM_ZERO);
        return socket_set_option($this->pointer, SOL_SOCKET, SO_RCVTIMEO, $to);
    }

    /**
     * bytes request length
     * Returns length of request string represented in bytes.
     * param: String Request
     */
    function stringByteLength($request) {
        return pack('n', strlen($request));
    }

}
