<?php

/**
 * General Class
 * For more information contact  support@alenet.com
 *
 * @author       $Author: imrsnik $
 * @copyright    Alenet, Inc.
 * @since        $Date: 2015-10-18 17:52:00 +0000 (Sun, 18 Oct 2015) $
 * @version        $Rev: 7353 $
 * @package		Clientix.Classes.Xml
 * @abstract		General Class
 * CHANGE LOG
 *  Included label
 */
class xmlObject {

    var $tag;
    var $properties;
    var $declarations;
    var $objects;
    var $contents;
    var $uppercase_tags;
    var $die_on_error = true;

    function __construct($tagname, $uppercase_tag = true) {
        $this->uppercase_tags = $uppercase_tag;
        if (!empty($this->uppercase_tag))
            $tagname = uppercase($tagname);
        $this->tag = $tagname;
        $this->properties = array();
        $this->declarations = '';
        $this->objects = array();
        $this->contents = '';
        $qty = func_num_args();
        for ($i = CLX_NUM_TWO; $i < $qty; $i = $i + CLX_NUM_TWO) {
            $key = uppercase(func_get_arg($i));
            $value = func_get_arg($i + CLX_NUM_TWO);
            $this->setProp($key, $value);
        }
    }

    function setTagCase($uppercase = true) {
        $this->uppercase_tags = $uppercase;
    }

    function getTag() {
        return (string) $this->tag;
    }

    function setProp($key, $value) {
        if (!empty($this->uppercase_tag))
            $key = uppercase($key);
        $this->properties[$key] = $value;
    }

    function getProp($key) {
        if (!empty($this->uppercase_tag))
            $key = uppercase($key);
        return $this->properties[$key];
    }

    function addObject($obj) {
        if (!is_object($obj))
            $this->error('XML::addObject() = This is not an object: ' . $obj);
        $this->objects[] = $obj;
    }

    function getObjects() {
        return $this->objects;
    }

    function getObjectXml($tag) {
        if (!empty($this->uppercase_tag))
            $tag = uppercase($tag);
        $qty = count($this->objects);
        $i = CLX_INIT_ZERO;
        while (($i < $qty)
        ) {
            if ($this->objects[$i]->tag == $tag) {
                return $this->objects[$i];
            }
            $i++;
        }
    }

    function hasObject($tag) {
        if (!empty($this->uppercase_tag))
            $tag = uppercase($tag);
        $found = false;
        $qty = count($this->objects);
        $i = CLX_INIT_ZERO;
        while (($i < $qty) and ( !$found)) {
            $o = $this->objects[$i];
            if ($o->tag == $tag) {
                $found = true;
            }
            $i++;
        }
        return $found;
    }

    function hasSubObject($tag) {
        if (!empty($this->uppercase_tag)) {
            $tag = uppercase($tag);
        }
        return $this->getObjectXml($tag);
    }

    function getSubObjectContents($tag) {
        if (!empty($this->uppercase_tag))
            $tag = uppercase($tag);
        if ($o = $this->getObjectXml($tag)) {
            return $o->getContents();
        } else {
            return CLX_CHAR_EMPTY_STRING;
        }
    }

    function setContents($contents) {
        $this->contents = $contents;
    }

    function addContents($contents) {
        $this->contents .= $contents;
    }

    function getContents() {
        return $this->contents;
    }

    function indent($level) {
        return str_repeat(CLX_CHAR_TAB, $level);
    }

    function openTag($level) {
        $buffer = '';
        $buffer .= $this->indent($level);
        $buffer .= '<' . $this->tag;
        foreach ($this->properties as $key => $value) {
            $buffer .= ' ' . $key . '=' . quote($value);
        }
        $buffer .= '>';
        if (count($this->objects) > CLX_NUM_ZERO) {
            $buffer .= CLX_CHAR_NL;
        }
        return $buffer;
    }

    function closeTag($level) {
        $buffer = '';
        if (count($this->objects) > CLX_NUM_ZERO) {
            $buffer .= $this->indent($level);
        }
        $buffer .= '</' . $this->tag . '>';
        return $buffer;
    }

    function source_($parameter) {
        $parameters = array_merge(array('level' => CLX_NUM_ZERO), $parameter);
        foreach ($parameters as $key => $value) {
            $$key = $value;
        }
        $buffer = '';
        $buffer .= $this->openTag($level);
        if (count($this->objects) > CLX_NUM_ZERO) {
            foreach ($this->objects as $obj) {
                $buffer .= $obj->source_(array('level' => $level + CLX_NUM_ONE));
                $buffer .= CLX_CHAR_NL;
            }
        } else {
            $buffer .= $this->contents;
        }
        $buffer .= $this->closeTag($level);
        return $buffer;
    }

    function load($filename) {
        if (!file_exists($filename) && strpos($filename, 'http') === false)
            $this->error('File does not exist: ' . $filename);
        $fd = fopen($filename, 'r');
        if (empty($fd))
            $this->error('Cannot open file: ' . $filename);
        $size = CLX_NUM_8192;
        $buffer = '';
        while (!feof($fd)) {
            $buffer .= fread($fd, $size);
        }
        fclose($fd);
        $this->parse($buffer);
    }

    function parse($buffer) {
        $tagname = null;
        $properties = null;
        $contents = null;
        $this->parseDeclarations($buffer);
        ClxFreeReturn($this->getObjectInfo($buffer, $tagname, $properties, $contents));

        $this->tag = $tagname;
        $this->properties = array();
        $this->objects = array();
        $this->contents = '';
        $this->parseProperties($properties);
        $this->parseContents($contents);
    }

    function save($filename) {

        $file = new clx_file($filename, 'w');
        $file->open();
        if (!$file->error) {
            $file->write($this->source_(array('level' => CLX_NUM_ZERO)));
        } else {
            addMessage($file->getErrorMsg() . ': ' . $file->fileGetPathFileName());
        }
        $file->close();
    }

    function parseContents($buffer) {
        $tagname = null;
        $properties = null;
        $contents = null;
        if (strpos($buffer, '<') === false) {
            $this->contents = $buffer;
        } else {
            while ($this->getObjectInfo($buffer, $tagname, $properties, $contents)) {
                $n = new xmlObject($tagname);
                $n->parseProperties($properties);
                $n->parseContents($contents);
                $this->addObject($n);
            }
        }
    }

    function getObjectInfo(&$buffer, &$tagname, &$properties, &$contents) {
        $openpos = strpos($buffer, '<');
        if (false === $openpos) {
            $more = false;
            $tagname = '';
            $newproperties = '';
            $newcontents = '';
            $newbuffer = $buffer;
        } else {
            $closepos = strpos($buffer, '>', $openpos);
            if (false === $closepos) {
                $this->error('No closing bracket near: ' . htmlspecialchars(substr($buffer, $openpos, CLX_NUM_HUNDRED)) . '<br>');
                return false;
            }
            $taginfo = substr($buffer, $openpos + CLX_NUM_ONE, $closepos - $openpos - CLX_NUM_ONE);
            $spacepos = strpos($taginfo, ' ');
            if (false === $spacepos) {
                $tagname = $taginfo;
                $newproperties = '';
            } else {
                $tagname = substr($taginfo, CLX_NUM_ZERO, $spacepos);
                $newproperties = substr($taginfo, $spacepos + CLX_NUM_ONE);
            }
            $x = $closepos + CLX_NUM_ONE;
            $qty = CLX_INIT_ZERO; // the number of found opening tags in $buffer
            $done = false;
            //Modifyed to allow dictionary entries on tool packages
            //$recursionlimit = 4;
            $recursionlimit = CLX_NUM_NINE;
            while (empty($done)) {
                $i = strpos($buffer, '<' . $tagname, $x);
                $j = strpos($buffer, '</' . $tagname, $x - CLX_NUM_ONE);
                if (false === $j) {
                    $this->error('No closing tag for <br>' . htmlspecialchars($tagname) . '<br> near: ' . htmlspecialchars($buffer) . '<br>');
                    return false;
                } elseif (($i !== false) && ($i < $j)) {
                    $x = $i + strlen($tagname) + CLX_NUM_THREE;
                    $qty++;
                } else { // this is the closing
                    if ($qty == CLX_NUM_ZERO) {
                        $done = true;
                    } else {
                        $qty--;
                        $x = $j + strlen($tagname) + CLX_NUM_FOUR;
                    }
                }
                if (($qty > $recursionlimit) or ( $qty < -$recursionlimit)) {
                    $this->error('Too much recursion...<br>');
                    return false;
                }
            }
            $newcontents = substr($buffer, $closepos + CLX_NUM_ONE, $j - $closepos - CLX_NUM_ONE);
            $newbuffer = substr($buffer, $j + strlen($tagname) + CLX_NUM_THREE);
            $more = true;
        }
        $buffer = $newbuffer;
        $properties = $newproperties;
        $contents = $newcontents;
        return $more;
    }

    function parseProperties($buffer) {

        $equalpos = strpos($buffer, '=');
        if (false !== $equalpos) {

            $propname = trim(substr($buffer, CLX_NUM_ZERO, $equalpos));
            $quoteopen = strpos($buffer, CLX_CHAR_DOUBLE_QUOTE);
            if (false === $quoteopen)
                $this->error('Syntax error while searching for opening ');
            $quoteclose = strpos($buffer, CLX_CHAR_DOUBLE_QUOTE, $quoteopen + CLX_NUM_ONE);
            if (false === $quoteclose)
                $this->error('Syntax error while searching for closing ');
            $propvalue = substr($buffer, $quoteopen + CLX_NUM_ONE, $quoteclose - $quoteopen - CLX_NUM_ONE);
            $this->setProp($propname, $propvalue);
            $this->parseProperties(substr($buffer, $quoteclose + CLX_NUM_ONE));
        }
    }

    function parseDeclarations(&$buffer) {
        $openpos = strpos(lowercase($buffer), '<?xml');
        if (false !== $openpos) {
            $closepos = strpos(lowercase($buffer), '?>');
            $this->declarations = substr($buffer, $openpos, $closepos + CLX_NUM_TWO);
            $buffer = substr($buffer, $closepos + CLX_NUM_TWO, strlen($buffer) - strlen($this->declarations));
        }
    }

    function dump() {
        $contenidoHTML = '<pre>' . htmlspecialchars($this->source_(array('level' => CLX_NUM_ZERO))) . '</pre>';
        echo $contenidoHTML;
    }

    function error($msg) {
        ob_flush();
        if (!empty($this->die_on_error)) {
            salir($msg);
        }
    }

    /**
     * Inserts new node to parent
     * @param type $key
     * @param type $value
     * @param type $parentnode
     */
    public static function addNewObjectToNode($key, $value, &$parentnode) {
        $e = new xmlObject($key);
        $e->setContents($value);
        $parentnode->addObject($e);
    }
	function source(){
		return $this->source2((array)$this);
	}
	function source2($array) {
		$xml = new SimpleXMLElement('<xml/>');
		
		$node = $xml->addChild('returns');

		$this->array_to_xml($array['objects'], $node);

		return $xml->asXML();

		
		
	}
	function array_to_xml($array, &$xml) {
		
		foreach($array as $key => $value) {
			if (is_object($value))
				$value=(array) $value;
			if(is_array($value)) {
				if(!is_numeric($key)){
					$subnode = $xml->addChild($key);
					$this->array_to_xml($value, $xml);
				} else {
					$this->array_to_xml($value, $xml);
				}
			} else {
				if ( !empty($key)) {
					$xml->addChild($key,$value);
					
				}
			}
		}
	}


}
