<?php

/**
 * Database Import Extended Tool
 * For more information contact  support@alenet.com
 *
 * @author       $Author: cramirez $
 * @copyright    Alenet, Inc.
 * @since        $Date: 2018-04-20 08:37:18 -0400 (Fri, 20 Apr 2018) $
 * @version        $Rev: 9795 $
 *
 * CHANGE LOG
 *  Included svn prop
 */
// LIBRARY CODE: CIET
class clx_import_extended_tool {

    var $s_username;
    var $s_import_def_code;
    var $o_clx_file;
    var $o_proc_eng_params;
    var $s_main_table_name;
    var $a_usr_date_fields_existance;
    var $b_header_line_read = false;
    var $a_flatfile_headers = null;
    var $a_import_header;
    var $a_import_mappings;
    var $b_sw_w_creation_data;
    var $s_record_data_to_show;
    var $s_title;
    var $i_line_counter = CLX_INIT_ZERO;
    var $o_sw_include_ignored_cols = false;
    var $line;

    /*
     * Constructor
     * @param array
     * @return void
     */

    function __construct($parameters = array(), $s_import_def_code = null) {
        // Loads required classes
        if (!class_exists('proc_eng_params'))
            includeCodeLib('PROC_2');
        if (!function_exists('setCreationData'))
            includeCodeLib('GEN_REC_UTIL');
        if (!class_exists('clx_file_delimited'))
            require_once kPathToWebSite . 'classes/clx_file_delimited.php';
        // Inicializa los parametros del process engine
        if(!array_key_exists('USERNAME',$parameters)) 
            $parameters['USERNAME'] = $_SESSION['user_record']['username'];
        $this->o_proc_eng_params = new proc_eng_params($parameters);
        // Username
        $this->s_username = $this->o_proc_eng_params->process_parameters['USERNAME'];
        // Reads the import definition header
        if ($s_import_def_code != null) {
            $this->read_import_header($s_import_def_code);
        }
    }

    /*
     * Verifies the existance of columns: CDATE (creation date), CREATEDBY (user who created the record), MDATE (modification date) and MODIFIEDBY (user who modified the record
     * @param void
     * @return void
     */

    function verify_usr_date_fields_existance() {
        $this->a_usr_date_fields_existance = array();
        $a_usr_date_flds = array('CDATE', 'CREATEDBY', 'MDATE', 'MODIFIEDBY');
		$rs = ClxCommon::$db->fieldList($this->s_main_table_name);
		while (($result = ClxCommon::$db->getNextRecord($rs, ASSOCIATIVE))) {
            if (!array_search($result['Field'], $a_usr_date_flds) === false) {
                $this->a_usr_date_fields_existance[] = $result['Field'];
                break;
            }
        }
    }

    /*
     * Verifies the input file existance and opens it for reading
     * @param string
     * @return boolean
     */

    function report_file_existance_and_openit($filename) {
        $a_response = array('error' => false,
            'errormsg' => '',
            'response' => ''
        );
        // Verifies the input file existance
        if (file_exists($filename)) {
            /*
             * GEN_ERR_FILE_FOUND
             * File found: ^1
             * Archivo encontrado: ^1
             */
            $a_response['response'] = (CLX_CHAR_NL . dict('GEN_ERR_FILE_FOUND', $filename) . '<br>');
            // Opens input file
            $this->o_clx_file = new clx_file_delimited($filename, CLX_CHAR_NL, chr(CLX_NUM_NINE));
            if ($this->o_clx_file->error === true) {
                $msg = dict('GEN_ERR_FILE_OPENERROR',"($filename)") . ' (' . $this->o_clx_file->errorMsg . ')';
                $this->proc_log_write($msg);
                $a_response['error'] = true;
                $a_response['errormsg'] = $msg;
            }
        } else {
            /*
             * GEN_ERR_FILE_NOTFOUND
             * File not found: ^1
             * Archivo no fue encontrado: ^1
             */
            $msg = dict('GEN_ERR_FILE_NOTFOUND', $filename);
            $this->proc_log_write($msg);
            $a_response['error'] = true;
            $a_response['errormsg'] = $msg;
        }
        return $a_response;
    }

// Method End
    /*
     * Reads next plain text file line
     * @param void
     * @return string
     */
    function read_next_line() {
        // reads next plain text file line
        $line = $this->o_clx_file->getNextRecord2();
        if (empty($line))
            return $line;
        // internal plain text file line counter
        $this->i_line_counter++;
        // Cleans the line
        $line = $this->o_clx_file->delete_ending_crlf($line);
        if (trim($line) === '') {
            return $this->read_next_line();
        }
        if (empty($this->b_header_line_read)) {
            // Set the first line read flag
            $this->b_header_line_read = true;
            // Read the first line into array of plain text file headers
            $this->a_flatfile_headers = $this->read_header_into_record_array($line);
            // Reads next line again, to return the real line to the parent process
            return $this->read_next_line();
        }
        return $line;
    }

    /*
     * Write the process engine exceptions records
     * @param string, booblean, array
     * @return void
     */

    function proc_log_write($msg, $error = true, $a_fields = array()) {
        if ($error)
            $this->o_proc_eng_params->report_exceptions($msg, $a_fields);
    }

// Method End
    /**
     * Returns record array from the FIRST line (HEADER)
     * @param string
     * @return array
     */
    function read_header_into_record_array($line) {
        $line = $this->o_clx_file->toArray($line);
        $record = array();
        $total = count($line);
        for ($i = CLX_INIT_ZERO; $i < $total; $i++) {
            // To know the relationship between column names and it's indexes #
            $record[uppercase(trim($line[$i]))] = $i;
        }
        return $record;
    }

// Method End
    /*
     * Returns record array from the line according to import definitions
     * @param string
     * @return array
     */
    function read_line_into_record_array($line, $a_predef_frmla = array()) {
        $this->line = $this->o_clx_file->toArray($line);
        $record = array();
        $this->s_record_data_to_show = '<tr><td>%i_imported_rec_counter%</td>';
        $total = count($this->a_import_mappings);
        for ($i = CLX_INIT_ZERO; $i < $total; $i++) {
            // Assigns column value depending of import definition mapping
            switch ($this->a_import_mappings[$i]['TYPE']) {
                case 'FILE':
                    $record[$this->a_import_mappings[$i]['TABLE_FIELDNAME']] = $this->line[$this->a_flatfile_headers[$this->a_import_mappings[$i]['FILE_FIELDNAME']]];
                    $record[$this->a_import_mappings[$i]['TABLE_FIELDNAME']] = encode_string($record[$this->a_import_mappings[$i]['TABLE_FIELDNAME']]);
                    break;
                case 'CALC':
                case 'FORM':
                    $record[$this->a_import_mappings[$i]['TABLE_FIELDNAME']] = $this->executeFormula($this->a_import_mappings[$i]['FORMULA'], $a_predef_frmla);
                    break;
                default :
                    break;
            } // End Switch
            // Do eventual translations and reporting(only if it isn') . 't ignored)
            if ($this->a_import_mappings[$i]['TYPE'] !== 'IGNR') {
                // Do the eventual translation
                $this->translate($record[$this->a_import_mappings[$i]['TABLE_FIELDNAME']], $this->a_import_mappings[$i]['TRANSLATION']);
                // Adds column to detail line for reporting
                $this->s_record_data_to_show .= '<td>' . htmlentities(decode_string($record[$this->a_import_mappings[$i]['TABLE_FIELDNAME']])) . '</td>';
            } // End If
        } // End For
        $this->s_record_data_to_show .= '%extra_info%';
        // Assigns creation/modification date-time and username
        if ($this->b_sw_w_creation_data === true) {
            $record['CREATEDBY'] = $this->s_username;
            $record['MODIFIEDBY'] = $this->s_username;
            $record['CDATE'] = '';
            $record['MDATE'] = '';
            $record = setCreationData($record);
        } // End If
        return $record;
    }

// Method End
    /*
     * Reads import header definition
     * @param string
     * @return void
     */
    function read_import_header($s_import_def_code) {
        $this->s_import_def_code = $s_import_def_code;
        $sql = 'SELECT IDNAME, DESCRIPTION, TABLECODE, FIELD_DELIM, RECORD_DELIM, TYPE, RECORD_LENGTH FROM CLX_DB_IMPORTS WHERE IDCODE = ' . singleQuote($this->s_import_def_code);
        $rs = ClxCommon::$db->executeQuery($sql);
        // Import header record data
        $this->a_import_header = ClxCommon::$db->getNextRecord($rs, ASSOCIATIVE);
        // Main table name
        $this->s_main_table_name = tableName($this->a_import_header['TABLECODE']);
        // Verifies record audit data existance
        $this->verify_usr_date_fields_existance();
        // Reads import definitions columns mappings
        $this->read_import_mappings();
        // Data import definition title - it could be used on screen or activity report
        $this->s_title = $this->a_import_header['IDNAME'];
        // resources release
        unset($results);
    }

    /*
     * Reads import definition coorrespondence between table columns and flat file columns
     * @param void
     * @return void
     */

    function read_import_mappings() {
        $this->b_sw_w_creation_data = false;
        $this->a_import_mappings = array();
        $this->s_record_data_to_show = '<td>###</td>';
        $sql = 'SELECT ID, TABLE_FIELDNAME,  TYPE,  FILE_FIELDNAME,  FORMULA FROM CLX_DB_IMPORT_MAPPINGS WHERE IMPORTCODE = ' . singleQuote($this->s_import_def_code) . ' ORDER BY SEQNUM';
        $rs = ClxCommon::$db->executeQuery($sql);
        while ($record = ClxCommon::$db->getNextRecord($rs, ASSOCIATIVE)) {
            if (empty($this->b_sw_w_creation_data)) {
                switch ($record['TABLE_FIELDNAME']) {
                    case 'CREATEDBY':
                    case 'MODIFIEDBY':
                    case 'CDATE':
                    case 'MDATE':
                        $this->b_sw_w_creation_data = true;
                        break;
                    default :
                        break;
                }
            } // end if
            $a_item = array('TABLE_FIELDNAME' => $record['TABLE_FIELDNAME'],
                'TYPE' => $record['TYPE'],
                'FILE_FIELDNAME' => $record['FILE_FIELDNAME'],
                'FORMULA' => $record['FORMULA'],
                'TRANSLATION' => $this->get_import_mappings_translations($record['ID']));
            if ($record['TYPE'] !== 'IGNR') {
                // Don't include ignored (disabled) columns
                if ($record['TYPE'] === 'FILE') {
                    $this->s_record_data_to_show .= '<td>' . $record['FILE_FIELDNAME'] . '</td>';
                } else {
                    $a_item['FILE_FIELDNAME'] = $record['TABLE_FIELDNAME'] . '-' . $record['TYPE'];
                    $this->s_record_data_to_show .= '<td>' . $record['TABLE_FIELDNAME'] . '<br>[' . $record['TYPE'] . ']</td>';
                }
                $this->a_import_mappings[] = $a_item;
            } elseif ($this->o_sw_include_ignored_cols === true) {
                // Include ignored (disabled) columns anyway
                $this->a_import_mappings[] = $a_item;
            }
        } // end while
        unset($results);
    }

// Method End
    /*
     * Import Mapping Translation definitions read (for one specified column mapping)
     * @param integer
     * @return array
     */
    function get_import_mappings_translations($n_id) {
        $a_reponse = array();
        $sql = 'SELECT ORIGINAL_VALUE, REPLACE_VALUE, CASE_SENSITIVE FROM CLX_DB_IMPORT_TRANSLATIONS WHERE MAPID = ' . $n_id . ' ORDER BY SEQNUM';
        $rs = ClxCommon::$db->executeQuery($sql);
        while ($record = ClxCommon::$db->getNextRecord($rs, ASSOCIATIVE)) {
            $a_reponse[$record['ORIGINAL_VALUE']] = array('REPLACE_VALUE' => $record['REPLACE_VALUE'],
                'CASE_SENSITIVE' => $record['CASE_SENSITIVE']
            );
        }
        unset($results);
        return $a_reponse;
    }

// Method End
    /*
     * Excecutes and returns resulting value of import definition mapping translation
     * @param string, array
     * @return void
     */
    function translate(&$field_value, $translations) {
        if (array_key_exists($field_value, $translations)) {
            $info = $translations[$field_value];
            if ($info['case_sensitive'] === true) {
                if (strcmp($field_value, $info['ORIGINAL_VALUE']) == CLX_NUM_ZERO) {
                    $field_value = $info['REPLACE_VALUE'];
                }
            } else {
                $field_value = $info['REPLACE_VALUE'];
            }
        }
    }

    /*
     * Excecutes and returns resulting value of import definition mapping formula
     * @param string
     * @return string
     */

    function executeFormula($s_formula, $a_predef_frmla = array()) {
        $formula = trim(strval($s_formula));
        if ($s_formula <> '') {
			if(isset($a_predef_frmla[$s_formula])) {
				$field_value = $a_predef_frmla[$s_formula];
			} else {
				$cal = new ClxExpression();
				$field_value = $cal->parseRecordFormula($formula);
			}
        } else {
            $field_value = '';
        }
        return $field_value;
    }

// Method End
    /*
     * Process end
     * @param string
     * @return string
     */
    function terminates_process() {
        // Closes input file
        $this->o_clx_file->close();
    }

// Method End
    /*
     * Initial report formatting secuence
     * @param void
     * @return string
     */
    function report_init() {
        $s_response = '<table border="1" cellspacing="1" cellpadding="1">';
        return $s_response;
    }

    /*
     * Ending report formatting secuence
     * @param void
     * @return string
     */

    function report_ending() {
        $s_response = '</table>';
        return $s_response;
    }

}
