<?php
/**
 * General Code File
 * For more information contact  support@alenet.com
 *
 * @author       $Author: cramirez $
 * @copyright    Alenet, Inc.
 * @since        $Date: 2018-02-19 14:22:19 -0500 (Mon, 19 Feb 2018) $
 * @version        $Rev: 9732 $
 *
 * Depende de:
 * 	tools/eb_sockets.php
 * 	data/code/EIBS_utilities.php
 * 	data/code/PROC_class_aid.php
 * 	data/code/EIBS_util_ibiq.php
 */
if (!class_exists('ibs_check_interface_commons'))
    includeCodeLib('EIBS_CHK');

/**
 * e-IBS Accounting Interface common properties and methods
 * @author Francisco Santana
 */
class ibs_accounting_interface_commons extends ibs_check_interface_commons {

    var $s_jeex_trsfr_output_dir;
    var $s_jeex_trsfr_output_dir_html;
    var $b_jeex_output_to_file = true;

    /**
     * Constructor
     * @param text, array
     * @return void
     */
    function __construct($s_comp_code = null, $parameters = array()) {
        //parent::ibs_check_interface_commons($this->s_comp_code, $parameters);
        parent::__construct($s_comp_code, $parameters);
        multiTableName($this, array('GLTT', 'GLCP'));
        // Lenght of account descriptions in the CLX_INPCLS socket definition
        if(!defined('EIBS_JOURNAL_ENTRY_EXPORT_IDNAME_MAXLEN')) 
            define('EIBS_JOURNAL_ENTRY_EXPORT_IDNAME_MAXLEN', companyParameterDef('EIBS_JOURNAL_ENTRY_EXPORT_IDNAME_MAXLEN', CLX_NUM_30, $this->s_comp_code));
        if(!defined('EIBS_JOURNAL_ENTRY_EXPORT_FILENAME')) 
            define('EIBS_JOURNAL_ENTRY_EXPORT_FILENAME', companyParameterDef('EIBS_JOURNAL_ENTRY_EXPORT_FILENAME', '%s_idcode%', $this->s_comp_code));
        if(!defined('EIBS_JOURNAL_ENTRY_EXPORT_FILE_EXT')) 
            define('EIBS_JOURNAL_ENTRY_EXPORT_FILE_EXT', companyParameterDef('EIBS_JOURNAL_ENTRY_EXPORT_FILE_EXT', '.txt', $this->s_comp_code));
        if(!defined('EIBS_JOURNAL_ENTRY_EXPORT_ERRFILE_EXT')) 
            define('EIBS_JOURNAL_ENTRY_EXPORT_ERRFILE_EXT', companyParameterDef('EIBS_JOURNAL_ENTRY_EXPORT_ERRFILE_EXT', '.log', $this->s_comp_code));
        if(!defined('EIBS_JOURNAL_ENTRY_EXPORT_ERRFILE_OLD_EXT')) 
            define('EIBS_JOURNAL_ENTRY_EXPORT_ERRFILE_OLD_EXT', companyParameterDef('EIBS_JOURNAL_ENTRY_EXPORT_ERRFILE_OLD_EXT', '.old', $this->s_comp_code));
        // String witch identifies successfull internal transfer
        if(!defined('EIBS_JOURNAL_ENTRY_EXPORT_LOGFILE_STATUS_SUCCESS')) 
            define('EIBS_JOURNAL_ENTRY_EXPORT_LOGFILE_STATUS_SUCCESS', companyParameterDef('EIBS_JOURNAL_ENTRY_EXPORT_LOGFILE_STATUS_SUCCESS', 'OK', $this->s_comp_code));
        // Repository for interchange files ()
        /* /home/fideicomiso/fide/cont_repository [O POR DEFECTO ASUME] data/files */
        $this->s_jeex_trsfr_output_dir = def_dir_ass(array('par_value' => trim(companyParameter('EIBS_JEEX_TRSFR_INTRNL_EXPORT_DIR', $this->s_comp_code))), 'par_value', kPathToWebSite . kPathToData);
        /* /home/fideicomiso/fide/cont_repository [O POR DEFECTO ASUME] data/files */
        $this->s_jeex_trsfr_output_dir_html = def_dir_ass(array('par_value' => trim(companyParameter('EIBS_JEEX_TRSFR_HTML_EXPORT_DIR', $this->s_comp_code))), 'par_value', kPathToData);
        if (!empty($this->debug)) {
            echoLog('EIBS_JOURNAL_ENTRY_EXPORT_IDNAME_MAXLEN=' . EIBS_JOURNAL_ENTRY_EXPORT_IDNAME_MAXLEN);
            echoLog('EIBS_JOURNAL_ENTRY_EXPORT_FILENAME=' . EIBS_JOURNAL_ENTRY_EXPORT_FILENAME);
            echoLog('EIBS_JOURNAL_ENTRY_EXPORT_FILE_EXT=' . EIBS_JOURNAL_ENTRY_EXPORT_FILE_EXT);
            echoLog('EIBS_JOURNAL_ENTRY_EXPORT_ERRFILE_EXT=' . EIBS_JOURNAL_ENTRY_EXPORT_ERRFILE_EXT);
            echoLog('EIBS_JOURNAL_ENTRY_EXPORT_LOGFILE_STATUS_SUCCESS=' . EIBS_JOURNAL_ENTRY_EXPORT_LOGFILE_STATUS_SUCCESS);
            echoLog('this->s_jeex_trsfr_output_dir=' . $this->s_jeex_trsfr_output_dir);
            echoLog('this->s_jeex_trsfr_output_dir_html=' . $this->s_jeex_trsfr_output_dir_html);
        }
    }

    /**
     * Read Parameters of the accounting/internal transfer socket
     * definition to know the amounts lenght
     */
    function ibs_je_readpar_amtlen(&$EBSK, &$s_msg_code, &$ar_mappings) {
        if ($ar_mappings == null) {
            $ar_mappings = $EBSK->getMappings($s_msg_code);
        }
        $ar_amount_pars = $this->eibs_mapping_array_all_pars($ar_mappings);
        $ar_amount_pars['DEBITS'] = $ar_amount_pars['INPDAM'];
        $ar_amount_pars['CREDITS'] = $ar_amount_pars['INPCAM'];
        if (!empty($this->debug)) {
            echoLog('ibs_je_readpar_amtlen<br>' .
                    debug_dump('ibs_journal_export-mappings', $ar_mappings) . '<br>' .
                    debug_dump('ar_amount_pars', $ar_amount_pars) . '<br>');
        }
        return $ar_amount_pars;
    }

    /**
     * Check if the number has decimal point character. If true, leave it
     * as is, if not, add '.00'. This is just to avoid problems
     * with the EIBS tostring() method.
     * @param float
     * @return float
     */
    function integer_amt_fix_before_trsnfr($n_amount) {
        $s_amount = $n_amount;
        if (strpos($s_amount, '.') === false)
            $s_amount .= '.00';
        if (!empty($this->debug))
            echoLog('<br>integer_amt_fix_before_trsnfr(' . $n_amount . ') = ' . $s_amount);
        return $s_amount;
    }

    /**
     * Split debit and credit amounts to respect the limit of
     * accounting/internar transfer socket definition debit/credit number
     * lenght
     * @return array with one or more lines for one account debit and credit
     * amounts. The number of lines is the same for both CR & DB.
     * If one of the DB/CR doesn't have enough linea, it creates additional
     * lines with amount 0
     */
    function ibs_je_decr_split_amt($ar_record_acum, $ar_amount_pars) {
        $ar_amt = array();
        $ar_amt['DEBITS'] = $this->amt_splitter($ar_record_acum['DEBITS'], $ar_amount_pars['DEBITS']['FIELD_LENGTH'], $ar_amount_pars['DEBITS']['FIELD_DECIMALS']);
        $ar_amt['CREDITS'] = $this->amt_splitter($ar_record_acum['CREDITS'], $ar_amount_pars['CREDITS']['FIELD_LENGTH'], $ar_amount_pars['CREDITS']['FIELD_DECIMALS']);
        if (count($ar_amt['DEBITS']) > count($ar_amt['CREDITS'])) {
            $desde = count($ar_amt['CREDITS']);
            $hasta = count($ar_amt['DEBITS']);
            for ($ik = $desde; $ik < $hasta; $ik++) {
                $ar_amt['CREDITS'][] = CLX_INIT_ZERO;
            }
        } elseif (count($ar_amt['DEBITS']) < count($ar_amt['CREDITS'])) {
            $desde = count($ar_amt['DEBITS']);
            $hasta = count($ar_amt['CREDITS']);
            for ($ik = $desde; $ik < $hasta; $ik++) {
                $ar_amt['DEBITS'][] = CLX_INIT_ZERO;
            }
        }
        if (!empty($this->debug))
            echoLog(debug_dump('ibs_je_decr_split_amt: ar_amt', $ar_amt));
        return $ar_amt;
    }

    /**
     * Returns an array of amounts splitted from a given amount according
     * to a maximun amount lenght
     */
   function amt_splitter($n_number, $n_maxnumlen, $n_maxnumdec) {
        $ar_response = array();
        if($n_maxnumlen == 0) {
            // Si el tamaño del campo es cero, acepta cualquier tamaño de monto, asi que devuelve uno solo
            if($n_number != 0) {
                // Solo agrega la linea si el monto es distinto a cero, REDONDEANDO a la cantidad de decimales definida
                $ar_response[] = bcadd($n_number, 0, $n_maxnumdec);
            }
        } else {
            $n_number = "".$n_number;
            $n_max_amount = str_repeat('9', $n_maxnumlen - $n_maxnumdec) . '.' . str_repeat('9', $n_maxnumdec);
            if (!empty($this->debug)) {
                echoLog('amt_splitter = n_number=' . $n_number . ', n_maxnumlen=' . $n_maxnumlen . ', n_maxnumdec=' . $n_maxnumdec . ', n_max_amount=' . $n_max_amount);
            }
            // Pica el monto en varias lineas cuyos montos individuales no superen el tamaño maximo del campo
            while (bccomp($n_number, '0', $n_maxnumdec) == CLX_NUM_ONE) {
                if (bccomp($n_number, $n_max_amount, $n_maxnumdec) == CLX_NUM_ONE) {
                    $ar_response[] = $n_max_amount;
                    $n_number = bcsub($n_number, $n_max_amount, $n_maxnumdec);
                } else {
                    $ar_response[] = bcsub($n_number, '0', $n_maxnumdec);
                    $n_number = '0';
                }
            }
        }
        if (!empty($this->debug))
            echoLog(debug_dump('amt_splitter-resultado=', $ar_response));
        return $ar_response;
    }

    /*
     * Check if there's a .log file response to any INTERNAL TRANSFER or JOURNAL ENTRY
     * @param void
     * @return void
     */

    function ibs_trsf_je_logfile_check($s_check_type) {

        echo "<br>\n";
        
        if ($this->sw_old_acco_struct === false) {
            // New structure / version: FIBT doesn't exists anymore... data of FIBT is in FIAT
            $this->fibtname = $this->fiatname;
        }
        $b_different_IO_dir = false;
        switch ($s_check_type) {
            case 'ITRF':
                // Internal Transfer
                if (defined('EIBS_TRSFR_IMPORT_DIR') && EIBS_TRSFR_IMPORT_DIR !== '') {
                    // IBS 408 or greater
                    $dir = EIBS_TRSFR_IMPORT_DIR;
                    $b_different_IO_dir = (EIBS_TRSFR_EXPORT_DIR <> EIBS_TRSFR_IMPORT_DIR);
                } else {
                    $dir = EIBS_TRSFR_EXPORT_DIR;
                }
                break;
            default:
                // Journal entry
                if (defined('EIBS_JEEX_IMPORT_DIR') && EIBS_JEEX_IMPORT_DIR !== '') {
                    // IBS 408 or greater
                    $dir = EIBS_JEEX_IMPORT_DIR;
                    $b_different_IO_dir = (EIBS_JEEX_IMPORT_DIR <> EIBS_JEEX_EXPORT_DIR);
                } else {
                    $dir = EIBS_JEEX_EXPORT_DIR;
                }
                break;
        }
        if (substr($dir, CLX_NUM_MINUS_1) != '/' && substr($dir, CLX_NUM_MINUS_1) != '\\')
            $dir .= DIRECTORY_SEPARATOR;
        if (!empty($this->debug))
            echoLog('Dir: $dir');
        // Verify if old directorio exists, to move IBS response files after processed
        //   - Caso 1002774 - CAR-Interfaces IBS - BEGIN
        $old_files_dir = $dir . 'old';
        $old_files_dir_exists = (is_dir($old_files_dir));
        $old_files_dir .= DIRECTORY_SEPARATOR;
        //   - Caso 1002774 - CAR-Interfaces IBS - END
        // Open a known directory, and proceed to read its contents
        //   - Caso 1002774 - CAR-Interfaces IBS - BEGIN
        // En el caso de Caruna, tiene un servidor Linux y siempre da el mensaje de error: CODE_ERR_FILE_INVALIDDIR
        // Por estar inclu�do el separador de directorios al final, Linux no lo reconoce como directorio...
        // SOLUCION: quitar el separador de directorios al final cuando se verifica si es un directorio
        $tmp_dir = $dir;
        if (substr($tmp_dir, CLX_NUM_MINUS_1) == DIRECTORY_SEPARATOR)
            $tmp_dir = mb_substr($tmp_dir, CLX_NUM_ZERO, mb_strlen($tmp_dir) - CLX_NUM_ONE);
        if (!is_dir($tmp_dir)) {
            /*
             * CODE_ERR_FILE_INVALIDDIR
             * Invalid directory: ^1
             * Directorio no es v�lido: ^1
             */
            //   - Caso 1002774 - CAR-Interfaces IBS
            //e choLog(dict('CODE_ERR_FILE_INVALIDDIR', $dir));
            echoLog('[*1*] ' . dict('CODE_ERR_FILE_INVALIDDIR', $tmp_dir));
            //   - Caso 1002774 - CAR-Interfaces IBS - END
        } elseif ($dh = opendir($dir)) {
            // For each file in the export directory...
            while (($file = readdir($dh)) !== false) {
                $filespec = $dir . $file;
                $fexten = substr($file, -strlen(EIBS_JOURNAL_ENTRY_EXPORT_ERRFILE_EXT));
                if (!empty($this->debug))
                    echoLog('FILE: ' . $file . ', FEXTEN: ' . $fexten . ', filetype: ' . filetype($filespec) . ', FILESPEC: ' . $filespec);
                // Only files (no dirs) wich file extension is .LOG
                if (!(filetype($filespec) !== 'dir' && lowercase($fexten) == EIBS_JOURNAL_ENTRY_EXPORT_ERRFILE_EXT)) {
                    if (!empty($this->debug))
                        echoLog('NO cumple con la condicion porque es un directorio o no tiene la extension .LOG');
                    continue;
                }
                if (!empty($this->debug))
                    echoLog('cumple con la condicion, es un .LOG' . '<br>' . $file);
                // a log file response for a export already made
                // The filename (without extension) is the record IDCODE
                // Reads the log plain text file
                $o_clx_file = new clx_file($filespec, 'r');
                $o_clx_file->open();
                if ($o_clx_file->error === true) {
                    /*
                     * GEN_ERR_FILE_OPENERROR
                     * Unable to open file: ^1
                     * No se pudo abrir el archivo: ^1
                     */
                    echoLog(dict('GEN_ERR_FILE_OPENERROR', $filespec . '.<br>' . CLX_CHAR_NL . $o_clx_file . '->errorMsg'));
                    continue;
                    //return false;
                }
                if (!empty($this->debug))
                    echoLog('abrio el archivo');
                // The first line is the record IDCODe
                $s_record_key = $o_clx_file->read_line();
                $s_record_key = $o_clx_file->delete_ending_crlf($s_record_key);
                $s_record_key = trim($s_record_key);
                // The rest is the error log contents...
                $s_contenido = $o_clx_file->read($o_clx_file->size() - strlen($s_record_key));
                if (!empty($this->debug))
                    echoLog('s_record_key: ' . $s_record_key . ', s_contenido: ' . $s_contenido . ', asccidump(s_record_key): ' . asccidump($s_record_key));
                if ($o_clx_file->error === true) {
                    /*
                     * GEN_ERR_FILE_READERROR
                     * Error reading file: ^1
                     * Error al leer el archivo: ^1
                     */
                    echoLog(dict('GEN_ERR_FILE_READERROR', $filespec . '.<br>' . CLX_CHAR_NL . $o_clx_file->errorMsg));
                    $o_clx_file->close();
                    continue;
                    //return false;
                }
                $o_clx_file->close();
                // Removes eventual undesired ending CR+LFs
                $s_contenido = $o_clx_file->delete_ending_crlf($s_contenido);
                // Search the corresponding record
              //  echo ' * ', $file, ' | ', $s_record_key . ' - ';
                $s_tmp_var_01 = '(JEEX_STATUS IS NULL OR JEEX_STATUS = ' . singleQuote(CLX_CHAR_EMPTY_STRING) . '  OR JEEX_STATUS = ' . singleQuote('IDLE') . ' OR JEEX_STATUS = ' . singleQuote('INPROG') . ') AND ';
                $ar_record = ClxCommon::$db->getRecord($this->jeexname, $s_tmp_var_01 . 'IDCODE = ' . singleQuote($s_record_key));
                //   = new version of CLX002P deletes first character of .txt filename, why? we don�t know but FS command me to take this in account...
                if (empty($ar_record)) {
                    $ar_record = ClxCommon::$db->getRecord($this->jeexname, $s_tmp_var_01 . 'SUBSTR(IDCODE,-' . strlen($s_record_key) . ') = ' . singleQuote($s_record_key));
                }
                if (empty($ar_record)) {
                    echo 'Skipped';
                } else {
                    // JEEX RECORD FOUND!
                    $i_record_id = $ar_record['ID'];
                    // IBS Process status check and assign
                    // Example of successful processing:
                    // 		00000085
                    //		OK
                    // Example of unsuccess processing:
                    // 		00000085
                    //		14-Error en cuenta de Credito
                    //		14-Error en monto de Debito
                    if ($s_contenido == EIBS_JOURNAL_ENTRY_EXPORT_LOGFILE_STATUS_SUCCESS) {
                        $s_record_status = 'OK';
                    } else {
                        // Verify new IBS 408 response format (BDA-Panama)
                        // Example of successful processing:
                        // 		00000085
                        //		7891- Lote procesado OK
                        // Example of unsuccess processing:
                        // 		00000085
                        //		14-Error en cuenta de Credito
                        //		14-Error en monto de Debito
                        //		7891- Lote procesado OK
                        $a_tmp_resp_recs = explode(CLX_CHAR_CR . CLX_CHAR_NL, $s_contenido);
                        // Caso 1002901 - BEGIN
                        // Se toma en cuenta que el nombre del archivo se repita en la 1era y segunda linea
                        $s_st = 'OK';
                        if (count($a_tmp_resp_recs) > CLX_NUM_ONE) {
                            foreach ($a_tmp_resp_recs as $kkey1 => $vvalue1) {
                                $a_tmp_resp_recs[$kkey1] = $o_clx_file->delete_ending_crlf($a_tmp_resp_recs[$kkey1]);
                                $a_tmp_resp_recs[$kkey1] = trim($a_tmp_resp_recs[$kkey1]);
                                if($a_tmp_resp_recs[$kkey1] === $s_record_key) {
                                    // Skip filename repetition
                                    continue;
                                }
                                // Verify if it's the 1st error line
                                $s_st = uppercase(substr($a_tmp_resp_recs[$kkey1], (strlen($a_tmp_resp_recs[$kkey1]) - strlen(EIBS_JOURNAL_ENTRY_EXPORT_LOGFILE_STATUS_SUCCESS))));
                                if ($s_st != EIBS_JOURNAL_ENTRY_EXPORT_LOGFILE_STATUS_SUCCESS) {
                                    $s_st = 'ERROR';
                                }
                                break;
                            }
                        }
                        if($s_st === 'OK') {
                            // There was lines with filename repetition and the last line has the processing status
                            $n_pos_from = (strlen($a_tmp_resp_recs[count($a_tmp_resp_recs)-1]) - strlen(EIBS_JOURNAL_ENTRY_EXPORT_LOGFILE_STATUS_SUCCESS));
                            $s_st = uppercase(substr($a_tmp_resp_recs[count($a_tmp_resp_recs)-1], $n_pos_from));
                        }
                        // Caso 1002901 - END
                        if ($s_st == EIBS_JOURNAL_ENTRY_EXPORT_LOGFILE_STATUS_SUCCESS) {
                            $s_record_status = 'OK';
                        } else {
                            // The's error lines
                            $s_record_status = 'ERROR';
                        }
                    }
                    // Response output reporting for this line...
                    echo '[', $ar_record['IDCODE'], ' | ', $ar_record['IDNAME'], '] - ', $s_record_status;
                    // Check the record type: ITRF = internal transfer, JEEX = journal entry
                    if ($ar_record['JEEX_REC_TYPE'] === 'ITRF') {
                        // ONLY FOR INTERNAL TRANSFERS!
                        $ar_other_info = explode(CLX_CHAR_NL, $ar_record['JEEX_OTHERINFO']);
                        if (!empty($this->debug))
                            echoLog(debug_dump('ar_other_info', $ar_other_info));
                        $s_tmp_var_01 = '';
                        //   para que tome como procesado OK con la nueva 71
                        // o cuando se el OK viene seguido de otra l�nea que indica el numero de lote
                        if ($s_record_status === 'OK') {
                            // Transfer is OK. All the lines were processed so we've to mark it up
                            // to block'em in order to avoid re-transmitting
                            $total = count($ar_other_info);
                            for ($j = CLX_INIT_ZERO; $j < $total; $j++) {
                                list($s_trans_code, $s_fibt_id) = explode(',', $ar_other_info[$j]);
                                $ar_payee_data = $this->get_special_payee_record_data($s_fibt_id);
                                if (!empty($this->debug))
                                    echoLog('Line: ' . $j . ', TRANSCODE=' . $s_trans_code . ', ID=' . $s_fibt_id . ', OK');
                                // Transfer of line $j is OK, so block the corresponding record in FIBT
                                if (!empty($this->debug))
                                    ClxCommon::$db->debug = true;

                                // Agrega estatus de procesado OK al mensaje de la transacci�n
                                $s_interface_msg = ClxCommon::$db->getField($this->fibtname, 'INTERFACE_MSG', 'ID = ' . $s_fibt_id);
                                $s_interface_msg .= (CLX_CHAR_NL . dict('GEN_LBL_STATUS') . ': ' . $s_contenido);
                                ClxCommon::$db->updateRecord($this->fibtname, array('RECORD_BLOCKING' => '1', 'INTERFACE_MSG' => $s_interface_msg), 'ID = ' . singleQuote($s_fibt_id));
                                if (!empty($this->debug))
                                    ClxCommon::$db->debug = false;
                                /*
                                 * CODE_LBL_ITRANSFEREMISSION_RESOK
                                 * Transfer completed successfully
                                 * Transferencia realizada con �xito
                                 */
                                $s_tmp_var_01 .= dict('CODE_LBL_ITRANSFEREMISSION_RESLTST', $j + CLX_NUM_ONE, $s_trans_code, $ar_payee_data['BENEFICIARY_NAME'], $ar_payee_data['TRANS_AMT'], dict('CODE_LBL_ITRANSFEREMISSION_RESOK'));
                            }
                        } else {
                            // Verify each line to report the corresponding errors
                            $ar_result_cont = explode(CLX_CHAR_NL, $s_contenido);
                            if (!empty($this->debug))
                                echoLog(debug_dump('ar_result_cont', $ar_result_cont));
                            // Caso 1002901 - BEGIN
                            //$total = count($ar_result_cont);
                            $total = count($ar_result_cont) - 1; // Skip last line, which has the whole lot status
                            // Caso 1002901 - END
                            for ($j = CLX_INIT_ZERO; $j < $total; $j++) {
                                list($s_linenum, $s_errmsg) = explode('-', $ar_result_cont[$j]);
                                $n_linenum = (CLX_NUM_ZERO + $s_linenum) - CLX_NUM_ONE;
                                if (isset($ar_other_info[$n_linenum])) {
                                    list($s_trans_code, $s_fibt_id) = explode(',', $ar_other_info[$n_linenum]);
                                    $ar_payee_data = $this->get_special_payee_record_data($s_fibt_id);
                                    if (!empty($this->debug))
                                        echoLog('s_linenum=' . $s_linenum . ', [n_linenum=' . $n_linenum . '], s_errmsg=' . $s_errmsg . ', s_trans_code=' . $s_trans_code . ', s_fibt_id=' . $s_fibt_id . ', ' . debug_dump('ar_payee_data', $ar_payee_data));
                                    // Transfer of line $s_linenum is wrong, so unblock the corresponding record in FIBT so it could be re-transmitted
                                    if (!empty($this->debug))
                                        ClxCommon::$db->debug = true;
                                    //   para que tome como procesado OK con la nueva 71
                                    $s_interface_msg = ClxCommon::$db->getField($this->fibtname, 'INTERFACE_MSG', 'ID = ' . $s_fibt_id);
                                    $s_interface_msg .= (CLX_CHAR_NL . dict('GEN_LBL_ERROR') . ': ' . $s_errmsg);
                                    ClxCommon::$db->updateRecord($this->fibtname, array('RECORD_BLOCKING' => '4', 'INTERFACE_MSG' => $s_interface_msg), 'ID = ' . $s_fibt_id);
                                    if (!empty($this->debug))
                                        ClxCommon::$db->debug = false;
                                } else {
                                    // Not an specific line error, just general error...
                                    $ar_payee_data = array();
                                    $ar_payee_data['BENEFICIARY_NAME'] = 'N/A';
                                    $ar_payee_data['TRANS_AMT'] = CLX_INIT_ZERO;
                                }
                                /*
                                 * CODE_LBL_ITRANSFEREMISSION_RESLTST
                                 * Line # ^1, Transaction # ^2, Payee ^3, Amount ^4, Status: ^5
                                 * L�nea # ^1, Transacci�n # ^2, Beneficiario ^3, Monto ^4, Estatus: ^5
                                 */
                                $s_tmp_var_01 .= dict('CODE_LBL_ITRANSFEREMISSION_RESLTST', $s_linenum, $s_trans_code, $ar_payee_data['BENEFICIARY_NAME'], $ar_payee_data['TRANS_AMT'], $s_errmsg);
                            }
                        }
                        // Add calculated xtra info to response
                        $s_contenido .= '<br><br>' . $s_tmp_var_01;
                    }
                    // Update the log record with error info
                    if (!empty($this->debug))
                        echoLog('ACTUALIZANDO EL RECORD.<br>' . CLX_CHAR_NL . $s_contenido);
                    if (!empty($this->debug))
                        ClxCommon::$db->debug = true;
                    ClxCommon::$db->updateRecord($this->jeexname, array('JEEX_COFIERR' => $s_contenido, 'JEEX_STATUS' => $s_record_status), 'ID = ' . $i_record_id);
                    if (!empty($this->debug))
                        ClxCommon::$db->debug = false;
                } // jeex record found...
                
                echo "<br>\n";
                
                // deletes the log file (only if itsn't in debugging mode)
                if (empty($this->debug)) {
                    if ($old_files_dir_exists === true) {
                        $this->move_to_old($file, $dir, $old_files_dir, $b_different_IO_dir);
                    } else {
                        if ($b_different_IO_dir === false) {
                            // OLD ACCOUNTING INTERFACE VERSION
                            // Same directory for I/O, so delete .log file
                            chmod($filespec, CLX_NUM_0775);
                            unlink($filespec);
                            if (file_exists($filespec)) {
                                /*
                                 * GEN_ERR_FILE_DELETEERROR
                                 * Error deleting the file: ^1
                                 * Error eliminando el archivo: ^1
                                 */
                                echoLog(dict('GEN_ERR_FILE_DELETEERROR', $filespec));
                            }
                        } else {
                            // Different directory for I/O, so RENAME .log file
                            $filespec = correct_dir_sep($filespec);
                            chmod($filespec, CLX_NUM_0775);
                            $s_new_name = substr($filespec, CLX_NUM_ZERO, strlen($filespec) - strlen(EIBS_JOURNAL_ENTRY_EXPORT_ERRFILE_EXT)) . EIBS_JOURNAL_ENTRY_EXPORT_ERRFILE_OLD_EXT;
                            if (!rename($filespec, $s_new_name)) {
                                // Trouble renaming file...
                                echoLog(dict('GEN_ERR_FILE_DELETEERROR', ('RENAME ' . $filespec . ' ' . $s_new_name)));
                                // Try to copy instead... Delete target first...
                                chmod($s_new_name, CLX_NUM_0775);
                                unlink($s_new_name);
                                if(!copy($filespec, $s_new_name)) {
                                    // Copy couldn´t be done, so report it...
                                    echoLog(dict('GEN_ERR_FILE_DELETEERROR', ('COPY ' . $filespec . ' ' . $s_new_name)));
                                } 
                                unlink($filespec); // ... and delete source .LOG
                                if (file_exists($filespec)) {
                                    echoLog(dict('GEN_ERR_FILE_DELETEERROR', $filespec)); // Deletion couldn´t be done
                                }
                            }
                        }
                    }
                }
            }
            closedir($dh);
        }
    }

    function move_to_old($file, $dir, $old_files_dir, $b_different_IO_dir) {
        $filename_only = substr($file, CLX_NUM_ZERO, strlen($filespec) - strlen(EIBS_JOURNAL_ENTRY_EXPORT_ERRFILE_EXT));
        $a_file_list = array($file);
        if ($b_different_IO_dir === true) {
            // If different directories, multiple response files generated by IBS could be there...
            $a_file_list[] = $filename_only . '.REP';
            $a_file_list[] = $filename_only . '.acp';
            $a_file_list[] = $filename_only . '.dec';
        }
        // Move files from source dir to old bkp dir
        foreach ($a_file_list as $key => $value) {
            $source_filespec = $dir . $value;
            $target_filespec = $old_files_dir . $value;
            if (!empty($this->debug))
                echoLog('<BR>COPY [' . $source_filespec . '] TO [' . $target_filespec . ']');
            if (copy($source_filespec, $target_filespec)) {
                if (!empty($this->debug))
                    echoLog('<BR>DEL ' . $source_filespec);
                chmod($source_filespec, CLX_NUM_0775);
                unlink($source_filespec);
                if (file_exists($source_filespec)) {
                    /*
                     * GEN_ERR_FILE_DELETEERROR
                     * Error deleting the file: ^1
                     * Error eliminando el archivo: ^1
                     */
                    echoLog('<br>' . dict('GEN_ERR_FILE_DELETEERROR', $source_filespec));
                }
            } else {
                /*
                 * GEN_ERR_FILESYS_COPYFAIL
                 * Error copying from file ^1 to file ^2
                 * Error realizando la copia del archivo ^1 al archivo ^2
                 */
                echoLog('<br>' . dict('GEN_ERR_FILESYS_COPYFAIL', $source_filespec, $target_filespec));
            }
        }
    }

}

/**
 * e-IBS Batch Accounting Interface
 * @author Francisco Santana
 *
 */
class ibs_accounting_batch_interface extends ibs_accounting_interface_commons {

    var $ibs_socket_disable = false;

    /**
     * Constructor
     * @param text, array
     * @return void
     */
    function __construct($s_comp_code = null, $parameters = array()) {
        parent::__construct($s_comp_code, $parameters);
        // /home/fideicomiso/fide/fide/cont
        if(!defined('EIBS_JEEX_EXPORT_DIR')) define('EIBS_JEEX_EXPORT_DIR', trim(companyParameter('EIBS_JEEX_EXPORT_DIR', $this->s_comp_code)));
        // /home/fideicomiso/fide/fide/cont_ibs_resp
        if(!defined('EIBS_JEEX_IMPORT_DIR')) define('EIBS_JEEX_IMPORT_DIR', trim(companyParameter('EIBS_JEEX_IMPORT_DIR', $this->s_comp_code)));
        // Filename prefix for files in the repository
        if(!defined('EIBS_JEEX_FILENAME_PREFIX')) define('EIBS_JEEX_FILENAME_PREFIX', 'EIBS_JEEX_');
        echoLog('EIBS_JEEX_EXPORT_DIR=' . EIBS_JEEX_EXPORT_DIR);
        echoLog('EIBS_JEEX_FILENAME_PREFIX=' . EIBS_JEEX_FILENAME_PREFIX);
    }

    /**
     * Builds a file for journal entry batch export
     * @param text, text
     * @return text = result of operation
     */
    function ibs_journal_export($s_hoy = null, $s_fecha_hasta = null, $s_comp_code = null) {
        $contents = '';
        if ($s_comp_code == null)
            $s_comp_code = $this->s_comp_code;
        // current date calc
        if ($s_hoy == null) {
            $s_hoy = adodb_next_day($this->get_comp_acc_date($s_comp_code));
        }
        echoLog(dict('GEN_LBL_DATE') . ' 1: ' . $s_hoy . ($s_fecha_hasta == null ? '' : ' - ' . dict('GEN_LBL_DATE') . ' 2: ' . $s_fecha_hasta));
        //  CDA IBS 7
        // Verify existance of BDA's budget fields... (Panama)
        $sw_fico_budget_exists = ClxCommon::$db->fieldExists($this->ficoname, 'EXT_BUDGET_ITEM');
        //ECHO 'OTRO EXISTE EXT_BUDGET_ITEM';
		//PRE('SELECT * FROM '.$this->ficoname .' WHERE 1=1 ');
		//Capturando nombres de cuentas contables para facilitar luego los sql.
        $glacs = array();
        $sql = 'SELECT GLAC.IDCODE, MAX(GLAC.IDNAME) IDNAME, MAX(GLAC.GL_TYPE) GL_TYPE, MAX(GLAC.PROVISION) PROVISION FROM ' . $this->glacname . ' GLAC GROUP BY GLAC.IDCODE';
        $rs = ClxCommon::$db->executeQuery($sql);
		//pre($sql);
        while ($r = ClxCommon::$db->getNextRecord($rs, ASSOCIATIVE)) {
            //Determinando signo de cuenta para ver si crece por debe o haber.
            $sign = CLX_NUM_ONE;
            if ($r['GL_TYPE'] === 'LI' || $r['GL_TYPE'] === 'IN' || $r['GL_TYPE'] === 'EQ')
                $sign = -CLX_NUM_ONE;
            if ($r['PROVISION'] == CLX_NUM_ONE)
                $sign *= -CLX_NUM_ONE;
            $glacs[$r['IDCODE']] = array('IDNAME' => $r['IDNAME'], 'SIGN' => $sign);
        }
        // Socket definition
        $EBSK = getTool('EBSK');
        $EBSK->debug = $this->debug;
        $EBSK->tfields_only = true;
        $EBSK->remove_decimal_point = true;
        $msg_code = EBSK_IBS_DEF_INPCLX_MSG;
        // Read Parameters to know the amounts lenght
        $mappings = null;
        $ar_amount_pars = $this->ibs_je_readpar_amtlen($EBSK, $msg_code, $mappings);
        // Prepares record creation
        $o_seq_name_file_generation = new seq_name_file_generation();
        $o_seq_name_file_generation->debug = $this->debug;
        $ar_pars = array();
        $ar_pars['export_dir'] = EIBS_JEEX_EXPORT_DIR; // Export server directory
        $ar_pars['rec_type'] = 'JEEX'; // record type: Fund transfer
        $ar_pars['EBSK'] = $EBSK;
        $ar_pars['msg_code'] = $msg_code;
        $ar_pars['efftdate'] = $s_hoy;
        $ar_pars['filename_prefix'] = EIBS_JEEX_FILENAME_PREFIX;
        $ar_pars['output_to_file'] = $this->b_jeex_output_to_file;
        $ar_pars['output_dir'] = $this->s_jeex_trsfr_output_dir;
        $ar_pars['output_dir_html'] = $this->s_jeex_trsfr_output_dir_html;
        $o_seq_name_file_generation->def_pars_set($ar_pars);
		///pre($ar_pars);
        // Prepares the column headers for the human readable
        // version of the export file
        $o_seq_name_file_generation->s_cofiex_html = $o_seq_name_file_generation->cofiex_linegen_header();
        // Creates record in the log table
        $ar_seq_response = $o_seq_name_file_generation->create_record();
        if ($ar_seq_response['error'] === true) {
            $contents .= '- ' . $ar_seq_response['errormsg'] . '<BR>' . CLX_CHAR_NL;
            echoLog('<B>ERROR</B> ' . $contents);
			//pre('error retorno '. $ar_seq_response['errormsg']);
            return $contents;
        }
        $s_sql_extra_pars_select_local = '';
        // Verify GLAC.INC_ORIG_ACC column existance
        if (ClxCommon::$db->fieldExists($this->glacname, 'INC_ORIG_ACC')) {
            $s_sql_extra_pars_select_local = 'GLAC.INC_ORIG_ACC,';
        } else {
            $s_sql_extra_pars_select_local = '1 INC_ORIG_ACC,';
        }
        if ($this->sw_old_acco_struct === false) {
            $sql = 'SELECT JEHD.EFFTDATE, JELN.CURRENCY_CODE, JELN.EXCHANGE_RT, MAX(FICO.CONTRACT_NUMBER) CHART_FIELD01, MAX(FICO.IDCODE) FICO_IDCODE, JELN.GLACCOUNTCODE, GLAC.IDNAME, ' . $s_sql_extra_pars_select_local . ' JEHD.JE_SOURCE, SUM(JELN.DEBIT_AMT) DEBITS, SUM(JELN.CREDIT_AMT) CREDITS FROM ' . $this->ficoname . ' FICO, ' . $this->jelnname . ' JELN, ' . $this->jehdname . ' JEHD, ' . $this->glacname . ' GLAC, ' . $this->glcpname . ' GLCP WHERE JEHD.IDCODE = JELN.JEHDR_CODE AND FICO.IDCODE = JELN.CHART_FIELD01 AND GLAC.IDCODE = JELN.GLACCOUNTCODE AND GLCP.COMP_CODE = JEHD.COMP_CODE AND GLCP.GLGROUP_CODE = GLAC.GLGROUP_CODE AND JEHD.JE_STATUS IN (' . singleQuoteArray(array('AP', 'CLOS')) . ') {DATE_FILTER} AND JEHD.COMP_CODE = ' . singleQuote($s_comp_code) . ' GROUP BY JELN.CURRENCY_CODE,JELN.EXCHANGE_RT, JELN.GLACCOUNTCODE, GLAC.IDNAME, JEHD.EFFTDATE, JEHD.JE_SOURCE ORDER BY JELN.CURRENCY_CODE, JELN.EXCHANGE_RT, JELN.GLACCOUNTCODE, JEHD.EFFTDATE';
			
			if ($s_fecha_hasta != null && $s_fecha_hasta !== '') {
                $sql = str_replace('{DATE_FILTER}', 'AND JEHD.REAL_DATE<' . singleQuote($s_fecha_hasta) . ' AND JEHD.REAL_DATE >= ' . singleQuote($s_hoy), $sql);
            } else {
                $sql = str_replace('{DATE_FILTER}', ' AND JEHD.REAL_DATE = ' . singleQuote($s_hoy), $sql);
            }
        } else {//DEPRECATED VERSION 50
            $sql = 'SELECT {ADDITIONAL_COLUMNS} JELN.GLACCOUNTCODE, GLAC.IDNAME, JEHD.JE_SOURCE, SUM(JELN.CREDIT_AMT) CREDITS, SUM(JELN.DEBIT_AMT) DEBITS FROM ' . $this->jelnname . ' JELN LEFT JOIN ' . $this->glacname . ' GLAC ON GLAC.IDCODE = JELN.GLACCOUNTCODE, ' . $this->jehdname . ' JEHD WHERE JEHD.IDCODE = JELN.JEHDR_CODE AND JEHD.JE_STATUS IN (' . singleQuoteArray(array('AP', 'CLOS')) . ') ';
            if ($s_fecha_hasta != null && $s_fecha_hasta !== '') {
                $sql .= ' AND JEHD.REAL_DATE >= ' . singleQuote($s_hoy) . ' AND JEHD.REAL_DATE <= ' . singleQuote($s_fecha_hasta) . ' ';
            } else {
                $sql .= ' AND JEHD.REAL_DATE = ' . singleQuote($s_hoy) . ' ';
            }
            $sql .= '{CMD_TAIL} ORDER BY {ADDITIONAL_ORDERBY_COLS}';
            $sql = str_replace('{ADDITIONAL_COLUMNS}', '', $sql);
            $sql = str_replace('{ADDITIONAL_ORDERBY_COLS}', 'JELN.GLACCOUNTCODE', $sql);
            $sql = str_replace('{CMD_TAIL}', '', $sql);
        }
		if (!empty($this->debug)) {
            echoLog(pre('<B>JEHD sql</B>=' . $sql));
        }
        $i_proc_je_counter = CLX_INIT_ZERO; // Line counter
        // To accomulate totals of debits and credits
        $ar_amt_sum = array();
        $ar_amt_sum['DEBITS'] = '0';
        $ar_amt_sum['CREDITS'] = '0';
        $i_num_rows = CLX_INIT_ZERO;
        // Por cada registro contable...
        ///pre($sql);
		$rs = ClxCommon::$db->executeQuery($sql);
        while ($r = ClxCommon::$db->getNextRecord($rs, ASSOCIATIVE)) {
            $r['IDNAME'] = $r['JE_SOURCE'] . '-' . $r['IDNAME'];
            //  BDA - Budget code from FICO added...
            //echo 'otrogetfield EXT_BUDGET_ITEM';
			if ($sw_fico_budget_exists === true) {
                $r['EXT_BUDGET_ITEM'] = ClxCommon::$db->getField($this->ficoname, 'EXT_BUDGET_ITEM', ' IDCODE = ' . singleQuote($r['CHART_FIELD01']));
				//pre("select * ".$this->ficoname. ' IDCODE = ' . singleQuote($r['CHART_FIELD01']));
			} else {
                $r['EXT_BUDGET_ITEM'] = '';
            }
            $currenct_account = $r['GLACCOUNTCODE'];
            $r['GLACCOUNTCODE_DB'] = $currenct_account;
            $r['GLACCOUNTCODE_CR'] = $currenct_account;
            $i_num_rows++; //fjsh keep from original code, may be used latter
            //  CR-BNP CASE 1002191
            // this new column (GLAC.INC_ORIG_ACC) set inclusion or exclusion of account on export
            // This is needed for those cases in which the account has transalation (see GLTT later in this method source code) and include a line for the original account is not suitable
            // Default Value = Include original account (CDA Case). BNP Case: exclude original account
            if ($r['INC_ORIG_ACC'] != CLX_NUM_ZERO) {
                $this->ibs_je_cb_processing($i_proc_je_counter, $ar_amt_sum, $r, //ex $ar_record_acum
                        $ar_amount_pars, $r, $i_num_rows, $EBSK, $msg_code, $o_seq_name_file_generation, $s_hoy);
            }
            //Adding interfase balance Translation SEE TABLE GLTT
            $sql = 'SELECT DISTINCT GLTT.JE_SOURCE, GLTT.DEBIT_GL_CODE, GLTT.CREDIT_GL_CODE, GLAC1.IDNAME DEBIT_GL_NAME, GLAC2.IDNAME CREDIT_GL_NAME FROM ' . $this->glttname . ' GLTT, ' . $this->glacname . ' GLAC1, ' . $this->glacname . ' GLAC2 WHERE GLTT.INTERFACE_CODE = ' . singleQuote('IBS_GLBAL') . ' AND GLTT.DEBIT_GL_CODE = GLAC1.IDCODE AND GLTT.CREDIT_GL_CODE = GLAC2.IDCODE AND GLTT.DIRECTION = 0 AND GLTT.GLACCOUNTCODE = ' . singleQuote($currenct_account) . ' AND (GLTT.JE_SOURCE = ' . singleQuote($r['JE_SOURCE']) . ' OR GLTT.JE_SOURCE = ' . singleQuote(CLX_CHAR_EMPTY_STRING) . ' OR GLTT.JE_SOURCE IS NULL)';
			
			if (!empty($this->debug)) {
            	echoLog(pre('<B>Interface sql</B>=' . $sql));
        	}
			//pre($sql);
            $rs1 = ClxCommon::$db->executeQuery($sql);
            while (($r1 = ClxCommon::$db->getNextRecord($rs1))) {
                $r2 = $r;
                $r3 = $r;
                $r2['GLACCOUNTCODE'] = $r1['DEBIT_GL_CODE'];
                $r2['GLACCOUNTCODE_DB'] = $r1['DEBIT_GL_CODE'];
                $r2['GLACCOUNTCODE_CR'] = $r1['DEBIT_GL_CODE'];
                $r2['BANK_ACCOUNT_NO_DB'] = '';
                $r3['GLACCOUNTCODE'] = $r1['CREDIT_GL_CODE'];
                $r3['GLACCOUNTCODE_DB'] = $r1['CREDIT_GL_CODE'];
                $r3['GLACCOUNTCODE_CR'] = $r1['CREDIT_GL_CODE'];
                $r2['BANK_ACCOUNT_NO_CR'] = '';
				//echo "r2";
				//pre($r2);
				//echo "r3";
				//pre($r3);
                $bal = bcsub($r['DEBITS'], $r['CREDITS']);
                //   - Debit/credit amount placement inverted
                // When balance amount (original account Debit-Credit) is greater than zero (positive value), balance in 1st Journal Line goes to Debit and 2nd goes to Credit
                // When balance amount is less than zero (negative value), absolute value of balance in 1st Journal Line goes to Credit and 2nd goes to Credit
                // When balance amount is zero, no Journal Line is generated
                if (floatVal($bal) > CLX_NUM_ZERO) {
                    $r2['DEBITS'] = $bal;
                    $r2['CREDITS'] = CLX_INIT_ZERO;
                    $r3['DEBITS'] = CLX_INIT_ZERO;
                    $r3['CREDITS'] = $bal;
                } elseif (floatVal($bal) < CLX_NUM_ZERO) {
                    $r2['DEBITS'] = CLX_INIT_ZERO;
                    $r2['CREDITS'] = bcabs($bal);
                    $r3['DEBITS'] = bcabs($bal);
                    $r3['CREDITS'] = CLX_INIT_ZERO;
                } else {
					//echo "continue es 0";
                    continue; //no balance
                }
                $r2['IDNAME'] = substr($r1['DEBIT_GL_NAME'] . '*', CLX_NUM_ZERO, CLX_NUM_64);
                $r3['IDNAME'] = substr($r1['CREDIT_GL_NAME'] . '*', CLX_NUM_ZERO, CLX_NUM_64);
                $i_num_rows++;
                $this->ibs_je_cb_processing($i_proc_je_counter, $ar_amt_sum, $r2, //ex $ar_record_acum
                        $ar_amount_pars, $r2, $i_num_rows, $EBSK, $msg_code, $o_seq_name_file_generation, $s_hoy);
                $i_num_rows++;
                $this->ibs_je_cb_processing($i_proc_je_counter, $ar_amt_sum, $r3, //ex $ar_record_acum
                        $ar_amount_pars, $r3, $i_num_rows, $EBSK, $msg_code, $o_seq_name_file_generation, $s_hoy);
                unset($r2, $r3);
                $o_seq_name_file_generation->i_cont_tot_reg++;  // Caso 1002901 - To know if there was records generated...
            }
        } // end while
        //FJS - Interfaz contable tipo OTH
        $sql = 'SELECT JEHD.IDCODE, JELN.CURRENCY_CODE, JELN.EXCHANGE_RT, FICO.CONTRACT_NUMBER CHART_FIELD01, FICO.IDCODE FICO_IDCODE, JELN.GLACCOUNTCODE, JEHD.JE_SOURCE, GLTT.ITEM_CODE, (SELECT MAX(JEIT.IDNAME) FROM GL_ITEM_TBL JEIT WHERE JEIT.IDCODE = GLTT.ITEM_CODE) IDNAME, GLTT.DEBIT_GL_CODE, GLTT.CREDIT_GL_CODE, case GLTT.BANK_DR_TYPE WHEN ' . singleQuote('FACC') . ' THEN FIAT.FIDU_BANKACT_CODE WHEN ' . singleQuote('BACC') . ' THEN IF(FIAT.BANK_ACCOUNT_NO IS NULL, FIAT.BANKAC_CODE, FIAT.BANK_ACCOUNT_NO) ELSE null END BANK_DB, case GLTT.BANK_CR_TYPE WHEN ' . singleQuote('FACC') . ' THEN FIAT.FIDU_BANKACT_CODE WHEN ' . singleQuote('BACC') . ' THEN IF(FIAT.BANK_ACCOUNT_NO IS NULL, FIAT.BANKAC_CODE, FIAT.BANK_ACCOUNT_NO) ELSE null END BANK_CR, (JELN.DEBIT_AMT-JELN.CREDIT_AMT) BALANCE FROM ' . $this->fitbname . ' FITB, ' . $this->ficoname . ' FICO, ' . $this->fiatname . ' FIAT, ' . $this->jelnname . ' JELN, ' . $this->jehdname . ' JEHD, ' . $this->glttname . ' GLTT WHERE JEHD.IDCODE = JELN.JEHDR_CODE AND FITB.IDCODE = FIAT.TRANSTBL_CODE AND FITB.HAS_GLTT_OTH = 1 AND FIAT.IDCODE = JEHD.TRANS_CODE AND FICO.IDCODE = JELN.CHART_FIELD01 AND JEHD.JE_STATUS IN (' . singleQuoteArray(array('AP', 'CLOS')) . ') ' . (($s_fecha_hasta != null && $s_fecha_hasta !== '') ? 'AND JEHD.REAL_DATE<' . singleQuote($s_fecha_hasta) . ' AND JEHD.REAL_DATE >= ' . singleQuote($s_hoy) : 'AND JEHD.REAL_DATE = ' . singleQuote($s_hoy) ) . '  AND JEHD.COMP_CODE = ' . singleQuote($s_comp_code) . '  AND GLTT.INTERFACE_CODE = ' . singleQuote('OTH') . '  AND GLTT.DIRECTION = 0 AND GLTT.GLACCOUNTCODE = JELN.GLACCOUNTCODE AND (GLTT.JE_SOURCE IN (JEHD.JE_SOURCE,' . singleQuote(CLX_CHAR_EMPTY_STRING) . ') OR GLTT.JE_SOURCE IS NULL) AND (JELN.DEBIT_AMT-JELN.CREDIT_AMT) <> 0 ORDER BY JELN.CURRENCY_CODE, JELN.EXCHANGE_RT, JELN.GLACCOUNTCODE';
		
		if (!empty($this->debug)) {
            echoLog(pre('<B>Interfaz contable tipo OTH sql</B>=' . $sql));
        }
		//pre($sql);
        $rs = ClxCommon::$db->executeQuery($sql);
        while ($r = ClxCommon::$db->getNextRecord($rs, ASSOCIATIVE)) {
            //  BDA - Budget code from FICO added...
			//echo 'otro2 getfield EXT_BUDGET_ITEM';
	
			if ($sw_fico_budget_exists === true) {
                $r['EXT_BUDGET_ITEM'] = ClxCommon::$db->getField($this->ficoname, 'EXT_BUDGET_ITEM', 'IDCODE = ' . singleQuote($r['CHART_FIELD01']));
 
    			//pre("select * ".$this->ficoname. ' IDCODE = ' . singleQuote($r['CHART_FIELD01']));

			} else {
                $r['EXT_BUDGET_ITEM'] = '';
            }
            //Montando una linea con debitos y creditos la cuenta cambia segun signo
            //Se busca el signo de la cuenta para manterne la logica de que debe ir por
            //credito y que debe ir por debito. Ejemplo si la cuenta es pasivo crece por el haber
            //lo que la hace negativa. El signo de la cuenta se determina al inicio del metodo en $glacs
            $r['DEBITS'] = bcabs($r['BALANCE']);
            $r['CREDITS'] = bcabs($r['BALANCE']);
            $bal = floatVal($r['BALANCE']);
            if ($bal > CLX_NUM_ZERO) {
                $r['GLACCOUNTCODE_DB'] = $r['DEBIT_GL_CODE'];
                $r['GLACCOUNTCODE_CR'] = $r['CREDIT_GL_CODE'];
                $r['BANK_ACCOUNT_NO_DB'] = $r['BANK_DB'];
                $r['BANK_ACCOUNT_NO_CR'] = $r['BANK_CR'];
                if (empty($r['IDNAME'])) {
                    $r['IDNAME'] = substr('OTH -' . $r['JE_SOURCE'] . '-' . $glacs[$r['GLACCOUNTCODE_DB']]['IDNAME'], CLX_NUM_ZERO, CLX_NUM_64);
                    $r['IDNAME_2'] = substr('OTH -' . $r['JE_SOURCE'] . '-' . $glacs[$r['GLACCOUNTCODE_CR']]['IDNAME'], CLX_NUM_ZERO, CLX_NUM_64);
                } else {
                    $r['IDNAME_2'] = $r['IDNAME'];
                }
            } else {
                $r['GLACCOUNTCODE_DB'] = $r['CREDIT_GL_CODE'];
                $r['GLACCOUNTCODE_CR'] = $r['DEBIT_GL_CODE'];
                if (empty($r['IDNAME'])) {
                    $r['IDNAME'] = substr('OTH -' . $r['JE_SOURCE'] . '-' . $glacs[$r['GLACCOUNTCODE_CR']]['IDNAME'], CLX_NUM_ZERO, CLX_NUM_64);
                    $r['IDNAME_2'] = substr('OTH -' . $r['JE_SOURCE'] . '-' . $glacs[$r['GLACCOUNTCODE_DB']]['IDNAME'], CLX_NUM_ZERO, CLX_NUM_64);
                } else {
                    $r['IDNAME_2'] = $r['IDNAME'];
                }
                //Invierte asiento ya que operacion es contraria
                $r['BANK_ACCOUNT_NO_DB'] = $r['BANK_CR'];
                $r['BANK_ACCOUNT_NO_CR'] = $r['BANK_DB'];
            }
            $r['IDNAME'] = $r['IDCODE'] . ' ' . $r['IDNAME'];
            $r['IDNAME_2'] = $r['IDCODE'] . ' ' . $r['IDNAME_2'];
            $i_num_rows++; //fjsh keep from original code, may be used latter
            $this->ibs_je_cb_processing($i_proc_je_counter, $ar_amt_sum, $r, $ar_amount_pars, $r, $i_num_rows, $EBSK, $msg_code, $o_seq_name_file_generation, $s_hoy);
            $o_seq_name_file_generation->i_cont_tot_reg++;  // Caso 1002901 - To know if there was records generated...
        }//end while
        // Flush rest of pending content, Transfers the file, Traps errors and save it to database
        $o_seq_name_file_generation->s_line_footer_html = str_replace('%DEBITS%', xFormat::number($ar_amt_sum['DEBITS'], CLX_NUM_TWO), $o_seq_name_file_generation->s_line_footer_html);
        $o_seq_name_file_generation->s_line_footer_html = str_replace('%CREDITS%', xFormat::number($ar_amt_sum['CREDITS'], CLX_NUM_TWO), $o_seq_name_file_generation->s_line_footer_html);
        $o_seq_name_file_generation->transfer_output_file(null, $o_seq_name_file_generation->s_line_footer_html . '</TABLE>');
        /*
          The IBS import module process the file and eventually returns a .LOG file with a ramdomdly generated filename. The first line will contain the IDCODE of the record generated by this process, just to be able to identify the
          correspondence with the .log file and attach the result to the generated record The rest of the content are the error messages and it could be like this:
          1-Error Agencia Debito 1-Error en Codigo de Centro de Costo Debito 1-Error Agencia Credito 1-Error en Codigo de Centro de Costo  Credito 2-Error Agencia Debito */
        if (!empty($this->debug)) {
            echoLog('debit=' . $ar_amt_sum['DEBITS'] . ', credit=' . $ar_amt_sum['CREDITS']);
        }
        if ($o_seq_name_file_generation->b_stop_file_generation === true) {
            $contents .= '<br>' . CLX_CHAR_NL . $o_seq_name_file_generation->b_stop_file_gen_msg;
        } else {
            // Load the sockets interface (just in case)
            if (empty($this->ibs_socket_disable)) {
                $this->massive_socket_ondemand_loader();
            }
        }
        return $contents;
    }

    /**
     * After sumarize of one account's lines ()ust after detect the control
     * break or at the end) , writes de line(s) to the output file
     * Depending of the amout, it create one or more lines
     * @param $i_proc_je_counter
     * @param $ar_amt_sum
     * @param $ar_record_acum
     * @param $ar_amount_pars
     * @param $record
     * @param $i_num_rows
     * @param $EBSK
     * @param $msg_code
     * @param $o_seq_name_file_generation
     * @param $s_hoy
     */
    function ibs_je_cb_processing(&$i_proc_je_counter, &$ar_amt_sum, &$ar_record_acum, &$ar_amount_pars, &$record, &$i_num_rows, &$EBSK, &$msg_code, &$o_seq_name_file_generation, &$s_hoy) {
        $o_ibs_check_socket_interface = new ibs_check_socket_interface();
        $s_hoy_yy = substr($s_hoy, CLX_NUM_ZERO, CLX_NUM_FOUR);
        //$s_hoy_yy = substr($s_hoy_yy, -2); // NOTE: must be only the last two digits of the year
        $s_hoy_yy = substr($s_hoy_yy, -$ar_amount_pars['INPVDY']['FIELD_LENGTH']); // NOTE: must be only the last two digits of the year	//  > CDA - IBS 7
        $s_hoy_mm = substr($s_hoy, CLX_NUM_FIVE, CLX_NUM_TWO);
        $s_hoy_dd = substr($s_hoy, CLX_NUM_8, CLX_NUM_TWO);
        // Output filename
        $s_output_filename0 = $o_seq_name_file_generation->s_idcode . $o_seq_name_file_generation->s_export_file_ext; //  > CDA - IBS 7
        if (isset($ar_amount_pars['INPFIL']['FIELD_LENGTH'])) {
            $s_output_filename0 = fija($s_output_filename0, $ar_amount_pars['INPFIL']['FIELD_LENGTH']);
        }
        // Blank values for certain fields
        // banco
        $s_blk_INPCBK = str_repeat(' ', $ar_amount_pars['INPCBK']['FIELD_LENGTH']); //  > CDA - IBS 7
        // sucursal
        $s_blk_INPCBR = str_repeat('0', $ar_amount_pars['INPCBR']['FIELD_LENGTH']);
        // moneda
        $s_blk_INPCCY = str_repeat(' ', $ar_amount_pars['INPCCY']['FIELD_LENGTH']);
        // cod. cta
        $s_blk_INPCGL = str_repeat('0', $ar_amount_pars['INPCGL']['FIELD_LENGTH']);
        // centro de costo
        $s_blk_INPCCC = str_repeat('0', $ar_amount_pars['INPCCC']['FIELD_LENGTH']);
        // descripcion de la cta.
        $s_blk_INPCNR = str_repeat(' ', $ar_amount_pars['INPCNR']['FIELD_LENGTH']);
        // Sum total debit and credit with bcadd
        $ar_amt_sum['DEBITS'] = bcadd($ar_amt_sum['DEBITS'], $ar_record_acum['DEBITS'], $ar_amount_pars['DEBITS']['FIELD_DECIMALS']);
        $ar_amt_sum['CREDITS'] = bcadd($ar_amt_sum['CREDITS'], $ar_record_acum['CREDITS'], $ar_amount_pars['CREDITS']['FIELD_DECIMALS']);
        // Split debit and credit amounts to respect the limit of accounting definition format number lenght
        $ar_amt = $this->ibs_je_decr_split_amt($ar_record_acum, $ar_amount_pars);
        // For each amount line, insert a separate record line to the output file
        $total = count($ar_amt['DEBITS']);
        for ($ik = CLX_INIT_ZERO; $ik < $total; $ik++) {
            $i_proc_je_counter++;
            $record['DEBITS'] = $this->integer_amt_fix_before_trsnfr($ar_amt['DEBITS'][$ik]);
            $record['CREDITS'] = $this->integer_amt_fix_before_trsnfr($ar_amt['CREDITS'][$ik]);
            $record['IDCODE'] = $o_seq_name_file_generation->s_idcode;
            $record['DD'] = $s_hoy_dd;
            $record['MM'] = $s_hoy_mm;
            $record['YY'] = $s_hoy_yy;
            $record['USERNAME'] = $this->username;

            // Para resolver problema con los acentos que descuadran el formato, NO PUEDEN HABER ACENTOS AQUI
            $record['IDNAME'] = $o_ibs_check_socket_interface->replace_accents($record['IDNAME']);

            $s_idname0 = $record['IDNAME']; //  > CDA - IBS 7
            if (array_key_exists('IDNAME_2', $record)) {//fjsh estaba colocando el mismo mensaje en ambas cuentas
                $s_idname1 = $o_ibs_check_socket_interface->replace_accents($record['IDNAME_2']);
            } else {
                $s_idname1 = $s_idname0;
            }
            $record['IDNAME'] = fija($s_idname0, $ar_amount_pars['INPDNR']['FIELD_LENGTH']); //  > CDA - IBS 7
            $record['IDNAME_2'] = fija($s_idname1, $ar_amount_pars['INPCNR']['FIELD_LENGTH']); //  > CDA - IBS 7
            $record['CURRENCY_CODE'] = $this->ar_comp_record['CURRENCY_CODE'];
            $record['INPOBK'] = EIBS_JOURNAL_ENTRY_EXPORT_BANK;
            $record['INPDBK'] = EIBS_JOURNAL_ENTRY_EXPORT_BANK;
            $record['INPCBK'] = EIBS_JOURNAL_ENTRY_EXPORT_BANK;
            $record['INPOBR'] = EIBS_JOURNAL_ENTRY_EXPORT_BRANCH;
            $record['INPDBR'] = EIBS_JOURNAL_ENTRY_EXPORT_BRANCH;
            $record['INPCBR'] = EIBS_JOURNAL_ENTRY_EXPORT_BRANCH;
            $record['EXCHANGE_RT'] = (isset($record['EXCHANGE_RT']) ? $record['EXCHANGE_RT'] : CLX_NUM_ONE);
            $record['INPFIL'] = $s_output_filename0;
            $record['FILENAME'] = $s_output_filename0;
            /*
             * fjsh - El campo cuenta contable esta alineando a la derecha las
             * 2 posiciones que faltan. Incorpore ceros para prevenir error
             * en alineacion
             */
            if ($this->sw_old_acco_struct === false) {
                // Cost center = Contracto Code, vers. 5.5.
                $record['CONTRACT_CODE'] = $record['FICO_IDCODE'];
                if (defined('EIBS_JEEX_EXPORT_COSTCNT_CHR') && EIBS_JEEX_EXPORT_COSTCNT_CHR === '1') {
                    $record['CONTRACT_CODE'] = $record['CHART_FIELD01'];
                }
                if (defined('EIBS_JEEX_EXPORT_COSTCENTER') && EIBS_JEEX_EXPORT_COSTCENTER !== '') {
                    $record['CONTRACT_CODE'] = EIBS_JEEX_EXPORT_COSTCENTER;
                }
            }
            if(mb_strlen($record['CONTRACT_CODE']) > $ar_amount_pars['INPDCC']['FIELD_LENGTH']) {
                // Global SAAS IBS 4.8 PTY - If contract code lenght is greater than cost center field lenght, asssign zero
                $record['CONTRACT_CODE'] = '0'; 
            }
			//echo 'create message ibs_je_cb_processing record';
			//pre($record);
            $ar_socket_line = $EBSK->createMessage($msg_code, $record);
            //e cho debug_dump('ibs_je_cb_processing - <B>$i_proc_je_counter</B>', $ar_socket_line);
            //   Socket line generation Error handling
            $a_error_response = $this->send_field_error_verification($EBSK);
            if ($this->eibs_send_debug === true) {
                echoLog('<hr>EIBS Socket Debug Info<br>' .
                        $this->sdi_debug_dump($this->ar_last_sent_skt) . '<hr>');
            }
            $s_obs = '';
            if ($a_error_response['error'] === true) {
                // fields error... socket record could not be send
                $s_obs = $a_error_response['errormsg']; // The error msg
                // stop file generation
                $o_seq_name_file_generation->b_stop_file_generation = true;
            }
			
			//echo 'record ibs_je_cb_processing 2';
			//PRE($record);
            if ($record['DEBITS'] == CLX_NUM_ZERO) {
                // Si el monto del debe es cero, no envia...
                // banco
                $i_i = $this->referential_array_search('INPDBK', $ar_socket_line, 'FILEFIELD');
                $ar_socket_line[$i_i]['VAL'] = $s_blk_INPCBK;
                // sucursal
                $i_i = $this->referential_array_search('INPDBR', $ar_socket_line, 'FILEFIELD');
                $ar_socket_line[$i_i]['VAL'] = $s_blk_INPCBR;
                // moneda
                $i_i = $this->referential_array_search('INPDCY', $ar_socket_line, 'FILEFIELD');
                $ar_socket_line[$i_i]['VAL'] = $s_blk_INPCCY;
                // cod. cta
                $i_i = $this->referential_array_search('INPDGL', $ar_socket_line, 'FILEFIELD');
                $ar_socket_line[$i_i]['VAL'] = $s_blk_INPCGL;
                // centro de costo
                $i_i = $this->referential_array_search('INPDCC', $ar_socket_line, 'FILEFIELD');
                $ar_socket_line[$i_i]['VAL'] = $s_blk_INPCCC;
                // descripcion de la cta.
                $i_i = $this->referential_array_search('INPDNR', $ar_socket_line, 'FILEFIELD');
                $ar_socket_line[$i_i]['VAL'] = $s_blk_INPCNR;
            }
            if ($record['CREDITS'] == CLX_NUM_ZERO) {
                // Si el monto del debe es cero, no envia...
                // banco
                $i_i = $this->referential_array_search('INPCBK', $ar_socket_line, 'FILEFIELD');
                $ar_socket_line[$i_i]['VAL'] = $s_blk_INPCBK;
                // sucursal
                $i_i = $this->referential_array_search('INPCBR', $ar_socket_line, 'FILEFIELD');
                $ar_socket_line[$i_i]['VAL'] = $s_blk_INPCBR;
                // moneda
                $i_i = $this->referential_array_search('INPCCY', $ar_socket_line, 'FILEFIELD');
                $ar_socket_line[$i_i]['VAL'] = $s_blk_INPCCY;
                // cod. cta
                $i_i = $this->referential_array_search('INPCGL', $ar_socket_line, 'FILEFIELD');
                $ar_socket_line[$i_i]['VAL'] = $s_blk_INPCGL;
                // centro de costo
                $i_i = $this->referential_array_search('INPCCC', $ar_socket_line, 'FILEFIELD');
                $ar_socket_line[$i_i]['VAL'] = $s_blk_INPCCC;
                // descripcion de la cta.
                $i_i = $this->referential_array_search('INPCNR', $ar_socket_line, 'FILEFIELD');
                $ar_socket_line[$i_i]['VAL'] = $s_blk_INPCNR;
            }
            $EBSK->msgrecord = $ar_socket_line;
            $s_socket_line = $EBSK->toString_();
            if (!empty($this->debug))
                echoLog('<br>' . $s_socket_line);
			//pre($ar_socket_line);
            if ($i_proc_je_counter > CLX_NUM_ONE) {
                $s_socket_line = CLX_CHAR_CR . CLX_CHAR_NL . $s_socket_line; // primera linea sin enter delante
            }
            // Html version of the content
            $s_line_html = $o_seq_name_file_generation->cofiex_linegen_detail($i_proc_je_counter, $ar_socket_line, $s_obs);
            // Writes record data
            $o_seq_name_file_generation->insert_record_data($s_socket_line, $s_line_html, null);
        } // end for
    }

    /**
     * method to be called by the button at the JEEX Listing to
     * resend the export file
     */
    function ibs_journal_export_send_outfile($s_dir = null) {
        if ($s_dir == null)
            $s_dir = EIBS_JEEX_EXPORT_DIR;
        $s_record_key = getIdlist();
        $total = count($s_record_key);
        for ($i = CLX_INIT_ZERO; $i < $total; $i++) {
            if (!empty($this->debug))
                echoLog($this->jeexname . ', ID = ' . singleQuote($s_record_key[$i]));
            $ar_record = ClxCommon::$db->getRecord($this->jeexname, 'ID = ' . singleQuote($s_record_key[$i]));
            if ($ar_record === true) {
                /*
                 * CODE_MSG_JEEXSOF_RECINFO
                 * Re-sending... record # ^1, file: ^2
                 * Reenviando... registro # ^1, archivo: ^2
                 */
                echoLog(dict('CODE_MSG_JEEXSOF_RECINFO', $s_record_key[$i], $ar_record['IDCODE'] . EIBS_JOURNAL_ENTRY_EXPORT_FILE_EXT));
                if (!empty($this->debug))
                    echoLog($ar_record['JEEX_COFIEX']);

                // No funciona la vieja forma de chequear el status OK. Se usa la columna JEEX_STATUS que es mas efectivo
                //if(substr($ar_record['JEEX_COFIERR'], 0, strlen(EIBS_JOURNAL_ENTRY_EXPORT_LOGFILE_STATUS_SUCCESS))
                //				== EIBS_JOURNAL_ENTRY_EXPORT_LOGFILE_STATUS_SUCCESS) {
                if ($ar_record['JEEX_STATUS'] === 'OK') {
                    /*
                     * CODE_ERR_JEEXSOF_CANNOTRETRANSMIT
                     * Successfull transmition. Re-transmition isn') . 't permitted
                     * El resultado de esta transmisi�n fue completado con �xito. No est� permitido retransmitirlo
                     */
                    echoLog(dict('CODE_ERR_JEEXSOF_CANNOTRETRANSMIT'));
                } elseif ($this->ibs_journal_export_outfile_tnsfr($ar_record['IDCODE'], $ar_record['JEEX_COFIEX'], $s_dir)) {
                    /*
                     * GEN_MSG_READY
                     * Ready
                     * Listo
                     */
                    echoLog(dict('GEN_MSG_READY'));
                    // Load the sockets interface (just in case)
                    $this->massive_socket_ondemand_loader();
                } else {
                    /*
                     * GEN_MSG_TRANSMITION_FAIL
                     * Transmition failed
                     * Fall� el env�o
                     */
                    echoLog(dict('GEN_MSG_TRANSMITION_FAIL'));
                }
            } else {
                /*
                 * GEN_MSG_RECORD_NOTFOUND
                 * Record not found: ^1
                 * No se encontr� el registro: ^1
                 */
                echoLog(dict('GEN_MSG_RECORD_NOTFOUND', $s_record_key[$i]));
            }
        }
    }

    /**
     * Transfers the export result to the server filesystem
     * @param $s_output_filename_par
     * @param $s_contenido
     * @param $s_dir
     */
    function ibs_journal_export_outfile_tnsfr($s_output_filename_par, $s_contenido, $s_dir = null) {
        $b_response = true;
        $dir = '';
        if ($s_dir == null)
            $s_dir = EIBS_JEEX_EXPORT_DIR;
        if (substr($dir, CLX_NUM_MINUS_1) != DIRECTORY_SEPARATOR)
            $dir .= DIRECTORY_SEPARATOR;
        $s_output_filename = $s_dir . $s_output_filename_par . EIBS_JOURNAL_ENTRY_EXPORT_FILE_EXT;
        if (!empty($this->debug))
            echoLog('output filename=' . $s_output_filename);
        if ($this->b_jeex_output_to_file === true) {
            $s_src_filename = kPathToWebSite . $this->s_jeex_trsfr_output_dir . EIBS_JEEX_FILENAME_PREFIX . $s_output_filename_par . EIBS_JOURNAL_ENTRY_EXPORT_FILE_EXT;
            if (!empty($this->debug))
                echoLog('source filename=' . $s_src_filename);
            if (!copy($s_src_filename, $s_output_filename))
                $b_response = false;
        } else {
           
            $o_clx_file = new clx_file($s_output_filename, 'w');
            $o_clx_file->open();
            if ($o_clx_file->error === true) {
                echoLog($o_clx_file->errorMsg . ': ' . $s_output_filename);
                $b_response = false;
            } else {
                $o_clx_file->write($s_contenido);
                if ($o_clx_file->error === true) {
                    echoLog($o_clx_file->errorMsg . ': ' . $s_output_filename);
                    $b_response = false;
                } else {
                    $o_clx_file->close();
                }
            }
        }
        return $b_response;
    }

    /**
     * Check if there's a .log file response to any export made
     */
    function ibs_journal_export_logfile_check() {
        return $this->ibs_trsf_je_logfile_check('JEEX');
    }

    // GL ONLINE Accounting Methods
    /**
     * Accounting Interfase, Send transaction msg clxcl6001
     * @return array
     */
    function sendAccTrans() {
        $record = array();
        $record['USERNAME'] = $this->username;
        $result = $this->send('CLXCL6001', $record);
        return $result;
    }
}
