<?php

/**
 * EIBS Class containing required base tools to connect with IBS
 * For more information contact  support@alenet.com
 *
 * @author       $Author: cramirez $
 * @copyright    Alenet, Inc.
 * @since        $Date: 2018-12-28 19:00:08 -0500 (Fri, 28 Dec 2018) $
 * @version    	$Rev: 10429 $
 */
class ibs {

    var $code = 'EIBS';
    var $acconame;  // Accounts
    var $aceaname;  // Participants e-mails
    var $acloname;  // Participants locations
    var $acphname;  // Participants phones
    var $coloname; // Companies' locations
    var $compname; // Companies
    var $contname;  // Contacts
    var $dyconame; // Dynamic Contact
    var $fiasname; // Contract Assets
    var $fiatname; // Contract Transactions
    var $fibkname; // Banck accounts
    var $fibtname; // Beneficiaries
    var $ficoname;  // Contracts
    var $fipaname;  // Participants
    var $fitbname;
    var $glacname; // Cuentas contables
    var $glcpname; // Cuentas contables
    var $infuname; // Portfolio
    var $jeexname; // journal_export
    var $jehdname; // Journal Entries - Header
    var $jelnname; // Journal Entries - Detail lines
    var $prmtname;
    var $sw_old_acco_struct;
    var $eibs_send_debug;
    var $proc_eng_params;
    var $username; //Username
    var $html_output; //Html output true or false
    var $debug = false; //Debug Option
    var $ar_last_sent_skt; // array result of last sent socket msg (return value of $EBSK->createMessage function)
    var $b_massive_socket_ondemand_loader = true;
    var $ar_comp_record;
    var $s_comp_code;
    var $o_response_rpt;
    var $s_fld_attrib = null;
    var $s_eibs_interface_active;
    var $b_eibs_cuscun_trim_zeroes = false;
    var $o_createClient_outputFileHandle = false;
    var $EBSK_direction = CLX_INIT_ZERO; // translation default value: 0=in (normal) 1=out (reverse)
    //var $EBSK_timeout = null;	// Specific time out
    var $eibs_utilities_last_send_response; // Last $EBSK-send response
    var $fixed_output_filespec = ''; /* 2018-02-23 Caso 1003630 - PBL-Interfaz contable, habilita posiiblidad de tener un nombre de archivo fijo para el archivo de salida */

    /**
     * Class Constructor
     */

    function __construct($s_comp_code = null, $parameters = array()) {
//e cho '<br>IBS class($s_comp_code)';
        //if(!class_exists('ibs_interface_queue')) includeCodeLib('EIBS_4');
        if (!class_exists('ibs_interface_queue'))
            includeCodeLib('EIBS_IBIQ');
        if (!function_exists('clx_vers_check'))
            includeCodeLib('EIBS_CLXV');
        if (!class_exists('proc_eng_params'))
            includeCodeLib('PROC_2');
        if (!function_exists('setCreationData'))
            includeCodeLib('GEN_REC_UTIL');


        // Init EBSK System Parameters (it's necessarry, on purpose, please do not delete this line, p-p-p-pleasssse)
        if (defined('EBSK_IBS_DEBUG') && EBSK_IBS_DEBUG === '1') {
            $this->debug = true;
        }
		
        //true indicates to send debug info of transmission data to the output screen
        //$s_eibs_send_debug = trim(companyParameter('EIBS_SOCKET_DEBUG', $this->s_comp_code));
        $s_eibs_send_debug = (defined('EBSK_IBS_SOCKET_DEBUG') ? EBSK_IBS_SOCKET_DEBUG : '');
        if (empty($s_eibs_send_debug)) {
            $this->eibs_send_debug = $this->debug;
        } else {
            $this->eibs_send_debug = ( $s_eibs_send_debug === '1');
        }
        if (!empty($this->debug)) {
            echoLog('IBS Debug Mode [' . EBSK_IBS_DEBUG . ']');
        }
        if ($this->eibs_send_debug === true) {
            echoLog('IBS SEND Debug Mode [' . $s_eibs_send_debug . ']');
        }
        multiTableName($this, array('ACCO', 'ACEA', 'ACLO', 'ACPH', 'COLO', 'COMP', 'CONT', 'DYCO', 'FIAS', 'FIAT', 'FIBK', 'FIBT', 'FICO', 'FIPA', 'FITB', 'GLAC', 'GLCP', 'INFU', 'JEEX', 'JEHD', 'JELN', 'PRMT'));
        $ar_response = clx_vers_check();
        // indicates old or new acco/cont tables structure version
        $this->sw_old_acco_struct = $ar_response['sw_old_acco_struct'];
        ////////////////////////////////////////////////////////////////////////
        // EIBS interface active: '1' = active, '0' = inactive
		if (isset($GLOBALS['EIBS_INTERFACE_ACTIVE'])) {
			 $this->s_eibs_interface_active =$GLOBALS['EIBS_INTERFACE_ACTIVE'];
		}
		else {
			$GLOBALS['EIBS_INTERFACE_ACTIVE']=$this->s_eibs_interface_active = companyParameter('EIBS_INTERFACE_ACTIVE');
        }
		if (!$this->set_comp_params($s_comp_code)) {
            return;
        }
        if (!defined('DIRECTORY_SEPARATOR')) {
            define('DIRECTORY_SEPARATOR', uppercase(substr(PHP_OS, CLX_NUM_ZERO, CLX_NUM_THREE) === 'WIN') ? '\\' : '/');
        }
        // Mensaje: OK
        if (!defined('EIBS_LBL_OK')) {
            define('EIBS_LBL_OK', dict('GEN_LBL_OK'));
        }
        if (isset($_SESSION['user_record'])) {
            $this->username = $_SESSION['user_record']['username']; // registered IBS username
        } else {
            $this->username = ''; // login time maybe...
        }
        $this->html_output = false;
        if (!empty($this->debug)) {
            echoLog('s_comp_code=' . $this->s_comp_code);
            echoLog('EIBS_LBL_OK=' . EIBS_LBL_OK);
            echoLog('this->username=' . $this->username);
            echoLog('this->html_output=' . $this->html_output);
        }
        // Inicializa los parametros del process engine
        $this->proc_eng_params = new proc_eng_params($parameters);
    }

// Constructor End
    /*
     * Company dependent values assigment
     * @param text, array
     * @return boolean
     */
    function set_comp_params($s_comp_code) {
        $contenidoHTML = '';
//e cho '<br>set_comp_params($s_comp_code)';
        // Gets default company
        if ($s_comp_code == null || trim($s_comp_code) === '') {
            if (isset($GLOBALS['s_comp_code'])) {
				$s_comp_code =$GLOBALS['s_comp_code'];
			}
			else {
				$GLOBALS['s_comp_code']=$s_comp_code=companyParameter('DEFAULT_COMPANY');
			}			
		}
        // Cannot continue without company code
        if (empty($s_comp_code)) {
            $contenidoHTML.= '<h1>COMPANY NOT SET. CHECK ' . singleQuote('DEFAULT_COMPANY') . ' GENERAL PARAMETER</h1>';
            echo $contenidoHTML;
            return false;
        }
        // Global company code property
        $this->s_comp_code = $s_comp_code;
        // Datos de la compa?ia/banco corriente
		if (isset($GLOBALS['ar_comp_record'])) {
			 $this->ar_comp_record =$GLOBALS['ar_comp_record'];
		}
		else {
			$GLOBALS['ar_comp_record']=$this->ar_comp_record = ClxCommon::$db->getRecord($this->compname, 'IDCODE = ' . singleQuote($this->s_comp_code));
        }
        if (!empty($this->debug)) {
            echoLog('set_comp_params--ar_comp_record', $this->ar_comp_record);
        }
        echo $contenidoHTML;
        return true;
    }

    /**
     * Gets and returns current accounting date from accounting parameters table for a given company
     * @param text $s_comp_code
     * @return text
     */
    function get_comp_acc_date($s_comp_code = null) {
        // Gets default company
        if ($s_comp_code == null)
            $s_comp_code = $this->s_comp_code;
        // Gets current accounting date from accounting parameters table
        $glcpname = tableName('GLCP');
        $s_response = ClxCommon::$db->getField($glcpname, 'ACCOUNTING_DATE', 'COMP_CODE = ' . singleQuote($s_comp_code));
        return $s_response;
    }

    /**
     * Sets the property that tells whether an IBS User is authenticated or not
     * @param text
     * @return text
     */
    function set_eibs_auth_user($s_new_val) {
        /*
         * e_IBS autheticated user flag:
         * 	'1' = authenticated
         * 	'', null, !isset() = not authenticated
         */
        $_SESSION['others']['eibs_auth_user'] = $s_new_val;
        return $s_new_val;
    }

    /**
     * Get the property that tells whether an IBS User is authenticated or not
     * @param void
     * @return text
     */
    function get_eibs_auth_user() {
        $ret = '';
        if (isset($_SESSION['others']['eibs_auth_user'])) {
            $ret = $_SESSION['others']['eibs_auth_user'];
        }
        return $ret;
    }

    /**
     * Now if the IBS interface is active and the IBS user is authenticated
     * @param void
     * @return bool
     */
    function verify_eibs_auth_user() {
        // $this->s_eibs_interface_active = '2' : IBS interface active, no IBS authentication required
        // $this->s_eibs_interface_active = '1' : IBS interface active,  IBS authentication required
        return ( $this->s_eibs_interface_active === '1' && $this->get_eibs_auth_user() === '1' ) || $this->s_eibs_interface_active === '2';
    }

    /**
     * Opens output file for the report of interface file generation
     * @param text $s_ibs_cuscun
     * @return text
     */
    function ibs_cuscun_code_fix($s_ibs_cuscun) {
        if ($this->b_eibs_cuscun_trim_zeroes) {
            $s_ibs_cuscun = '' . (integer) $s_ibs_cuscun; //  eliminacion de zeros del cuscun
        }
        return $s_ibs_cuscun;
    }

    /**
     * Opens output file for the report of interface file generation
     * @param text
     * @return text
     */
    function open_rpt_process_run($s_output_filename) {
        $this->o_response_rpt = new clx_file($s_output_filename, 'w');
        $this->o_response_rpt->open();
        if ($this->o_response_rpt->error === true) {
            return $this->o_response_rpt->errorMsg;
        }
        return CLX_CHAR_EMPTY_STRING;
    }

    /*
     * Get field size of a socket definition. It could return any other field by the way, specifying the $s_data2ret
     * @param text, array, text, text
     * @return variant
     */

    function get_sktdef_size($s_fldname, $ar_mappings, $s_data2ret = 'FIELD_LENGTH', $s_data2seach = 'FILE_FIELDNAME') {
        $v_response = null;
        foreach ($ar_mappings as $key => $field) {
            if ($field[$s_data2seach] == $s_fldname) {
                if ($s_data2ret === '*') {
                    $v_response = $field; //  returns complete definition array
                } else {
                    $v_response = $field[$s_data2ret];
                }
            }
        }
        return $v_response;
    }

    /**
     * Extracts eventual errors for each socket field, put it on the last sent socket property
     * and returns an array with an error flag and a text with all the errors text
     * @param $EBSK
     * @return array
     */
    function send_field_error_verification($EBSK) {
        $b_error = false;
        $s_error = '';
        $s_sep = '';
        if (count($EBSK->errors) > CLX_NUM_ZERO) {
            foreach ($EBSK->msgrecord as $i => $val) {
                if (isset($EBSK->errors[$val['FILEFIELD']])) {
                    $b_error = true;
                    $s_error .= $s_sep . $val['FILEFIELD'] . ' [' . $val['FIELD'] . ']: ' . $EBSK->errors[$val['FILEFIELD']];
                    $this->ar_last_sent_skt[$i]['errormsg'] = $EBSK->errors[$val['FILEFIELD']];
                    $s_sep = '<br>';
                }
            }
            $EBSK->errors = array(); // Clean error array to avoid confussion on further transmissions
        }
        return array('error' => $b_error, 'errormsg' => $s_error);
    }

    /*
     * Send message
     * @param text, array
     * @return array
     */

    var $b_allow_error_0 = false;

    function send($msg_code, $record, $skt_action = 'CODE') {
        // Comunicating with IBS via sockets
        $EBSK = getTool('EBSK');
        $EBSK->debug = ($this->eibs_send_debug || $this->debug);
        /*
          $EBSK->tfields_only = true;
         */
        $EBSK->direction = $this->EBSK_direction;
        if ($this->eibs_send_debug === true)
            echo debug_dump('EIBS_utilities-send-record', $record);
        $msg_record = ClxCommon::$db->getRecord($EBSK->tablename, 'IDCODE = ' . singleQuote($msg_code));
        if (empty($msg_record)) {
            $v_result = $EBSK->build_ibs_style_error_array(array('SKT_NOTFOUND' => dict('GEN_LBL_SOCK_INVALID', $msg_code))
            );
        } else {
            $this->ar_last_sent_skt = $EBSK->createMessage($msg_record['IDCODE'], $record, $skt_action);
            $a_error_response = $this->send_field_error_verification($EBSK);
            if ($a_error_response['error'] === true) {
                // fields error... socket record coluld not be send
                $v_result = $EBSK->build_ibs_style_error_array(array('REC_ERR' => $a_error_response['errormsg'])
                );
            } elseif ($this->o_createClient_outputFileHandle == null) {
                $v_result = $EBSK->sendMessage($msg_code, $record, $skt_action);
                $this->eibs_utilities_last_send_response = $v_result;
                if ($this->eibs_send_debug === true)
                    echoLog('<hr><br>');
                if (!is_array($v_result)) {
                    /*
                     * GEN_ERR_IBS_COMM_BREAKDOWN
                     * IBS comunication breakdown
                     * No hay comunicaci?n con IBS
                     */
                    $v_result = $EBSK->build_ibs_style_error_array(array('NOCOMM' => dict('GEN_ERR_IBS_COMM_BREAKDOWN')
                            )
                    );
                } elseif (empty($v_result[CLX_NUM_ZERO])) {
                    /*
                     * GEN_ERR_IBS_COMM_NORESPONSE
                     * No response from IBS
                     * No hubo respuesta de IBS
                     */
                    $v_result = $EBSK->build_ibs_style_error_array(array('NORESPONSE' => dict('GEN_ERR_IBS_COMM_NORESPONSE')
                            )
                    );
                } else {
                    if (empty($v_result[CLX_NUM_ZERO]['HEADERMSG']['ERRNUM'])) {//No Error
                        $v_result = $EBSK->build_ibs_style_error_array(array()); // OK
                    }
                }
            } else {
                // If the parameter $this->o_createClient_outputFileHandle is set, it has an openned file handle
                // and means that it must write the record content to that file
                $v_result = $EBSK->build_ibs_style_error_array(array()); // OK
                $s_socket_line = $EBSK->toString_();
                $this->o_createClient_outputFileHandle->write($s_socket_line . CLX_CHAR_NL);
                if ($this->o_createClient_outputFileHandle->error === true) {
                    // Output FILE Writting ERror
                    $v_result = $EBSK->build_ibs_style_error_array(array('OFILEWER' => $this->o_createClient_outputFileHandle->errorMsg)
                    );
                    //return false;	//  cjrd ????????
                }
            }
        }
        $this->EBSK_direction = CLX_INIT_ZERO; // finished... sets translation default value
        return $v_result;
    }

    /**
     * Builds result Page
     * @param text
     * @return text
     */
    function display($msg) {
        $contents = '';
        if ($this->html_output === true) {
            //Some Output

            $table = new htmlTable(CLX_WIDTH_ONE);
            $table->setProps(array(CLX_PROP_CLASS => CLX_CLASS_LIST, CLX_PROP_WIDTH => CLX_PROP_100PCT));

            $table->addCell(dict('GEN_LBL_IBSEXCHANGE'), array(CLX_PROP_CLASS => CLX_CLASS_TITLE, CLX_PROP_COLSPAN => CLX_NUM_ONE));
            $table->addCell(dict('GEN_LBL_STATUS'), array(CLX_PROP_CLASS => CLX_CLASS_SUBTITLE, CLX_PROP_COLSPAN => CLX_NUM_ONE));
            //$table->addCell($msg,array( CLX_PROP_CLASS=> ($i%2==0)? CLX_CLASS_DATA : CLX_CLASS_DATAALT));
            $table->addCell($msg, array(CLX_PROP_CLASS => CLX_CLASS_DATA));
            $c = new htmlButtonCloseWindow(dict('GEN_LBL_CLOSE'));
            $table->addCell($c->source(), array(CLX_PROP_CLASS => 'footer'));
            $contents .= $table->source();
        } else {
            $contents = $msg;
        }
        return $contents;
    }

    /**
     * Returns EIBS record data to be send formatted tabullary
     * @param array
     * @return text
     */
    function sdi_debug_dump($ar_last_sent_skt) {
        $s_response = '<TABLE width="100%" border="1">' . CLX_CHAR_NL . '<TR>';
        foreach ($ar_last_sent_skt[CLX_NUM_ZERO] as $name => $val) {
            $s_response .= '<TD>$name</TD>';
        }
        $s_response .= '</TR>' . CLX_CHAR_NL;
        $total = count($ar_last_sent_skt);
        for ($i = CLX_NUM_ONE; $i < $total; $i++) {
            $s_response .= '<TR>';
            foreach ($ar_last_sent_skt[$i] as $name => $val) {
                $val = ($val === '' ? CLX_CHAR_HTML_NBSP : $val);
                $s_response .= '<TD>' . $val . '</TD>';
            }
            $s_response .= '</TR>' . CLX_CHAR_NL;
        }
        $s_response .= '</TABLE>' . CLX_CHAR_NL;
        return $s_response;
    }

    /**
     * Returns an array with the errors returned by IBS in a understandable format
     * @param array, array
     * @return array
     */
    function formatIbsErrorMsg($error_rec) {
        $return = array();
        /* ERROR ARRAY FORMAT:
         * ** CUANDO HAY ERRORES:
          Array[CLX_NUM_ZERO] => Array
          ( [HEADER] => ELEERR
          [ERRNUM] => 1
          [ERNU01] => 0086
          [ERDS01] => Este usuario no es Reconocido en el Sistema
          [ERDR01] => 0
          [ERDR02] => 0
          [ERDR03] => 0
          [ERDR04] => 0
          [ERDR05] => 0
          [ERDR06] => 0
          [ERDR07] => 0
          [ERDR08] => 0
          [ERDR09] => 0
          [ERDR10] => 0
          )
         * ** CUANDO NO HAY ERROR ES:
          Array[CLX_NUM_ZERO] => Array
          ( [HEADER] => ELEERR
          [ERRNUM] => 0
          [ERDR01] => 0
          [ERDR02] => 0
          [ERDR03] => 0
          [ERDR04] => 0
          [ERDR05] => 0
          [ERDR06] => 0
          [ERDR07] => 0
          [ERDR08] => 0
          [ERDR09] => 0
          [ERDR10] => 0
          )
         */
        for ($i = CLX_NUM_ONE; $i <= $error_rec['HEADERMSG']['ERRNUM']; $i++) {
            $s_i = $i;
            if ($i < CLX_NUM_TEN)
                $s_i = '0' . $s_i;
            if (!empty($error_rec['HEADERMSG']['ERDS' . $s_i])) {
                $s_valor_campo = '';
                if (isset($error_rec['HEADERMSG']['ERDF' . $s_i])) {
                    $s_valor_campo = ' (' . uppercase($error_rec['HEADERMSG']['ERDF' . $s_i]) . ' = ' . singleQuote($error_rec[uppercase(trim($error_rec['HEADERMSG']['ERDF' . $s_i]))]) . ')';
                }
                $return['ERDS' . $s_i] = $error_rec['HEADERMSG']['ERNU' . $s_i] . ' ' . $error_rec['HEADERMSG']['ERDS' . $s_i] . $s_valor_campo;
            }
        }
        return $return;
    }

    /**
     * Searchs a value in a referential array. Returns null if the element's value does'nt exist or the specified value otherwise
     * @param text, array, text, text
     * @return variant
     */
    function search_fld_value_in_sent_result($s_codigo_campo_trama, $ar_last_sent_skt, $s_return_element = 'VAL', $s_element_name = 'FILEFIELD') {
        $s_codigo_campo_trama = uppercase($s_codigo_campo_trama);
        $i_i = $this->referential_array_search($s_codigo_campo_trama, $ar_last_sent_skt, $s_element_name);
        $v_ret = null;
        if ($i_i != -CLX_NUM_ONE) {
            $v_ret = $ar_last_sent_skt[$i_i][$s_return_element];
        }
        return $v_ret;
    }

    /**
     * Search a value in a referential array. Returns -1 if the element's value does'nt exist, otherwise returns the found element's index
     * @param text, array, text
     * @return integer
     */
    function referential_array_search($s_value_to_search, $ar_subject, $s_element_name) {
        foreach ($ar_subject as $i_i => $val) {
            if (isset($val[$s_element_name]) && $val[$s_element_name] == $s_value_to_search) {
                return $i_i;
            }
        }
        return -CLX_NUM_ONE;
    }

    /**
     * Returns an text elements array formatted in order to show it up in a bullet style
     * @param array
     * @return text
     */
    function ar_error_display_out($ar_var) {
        return '<br>' . CLX_CHAR_NL . '* ' . implode('<br>' . CLX_CHAR_NL . '* ', $ar_var) . '<br>' . CLX_CHAR_NL;
    }

    /**
     * Assign an array with the values returned by the IBS_sockets functions, only for those witch has FIELD element specified
     * @param array
     * @return array
     */
    function eibs_def_array_directval_assing($ar_record_original) {
        $ar_record = array();
        $total = count($ar_record_original);
        for ($i = CLX_INIT_ZERO; $i < $total; $i++) {
            if ($ar_record_original[$i]['FIELD'] !== '') {
                $ar_record[$ar_record_original[$i]['FIELD']] = $ar_record_original[$i]['VAL'];
            }
        }
        return $ar_record;
    }

    /**
     * Returns line of and multidimensional array according to the value of one of its elements
     * @param text, array, text
     * @return array
     */
    function eibs_mapping_array_entry_pars($s_entry_name, $ar_record_original, $s_fieldname = 'FILE_FIELDNAME') {
        $ar_record = null;
        foreach ($ar_record_original as $key => $field) {
            if ($field[$s_fieldname] == $s_entry_name) {
                $ar_record = $field;
                break;
            }
        }
        return $ar_record;
    }

    /**
     * Returns  multidimensional array with all of its elements with element's ket name as 'FILE_FIELDname).' OR given definition element value.
     * @param array
     * @return array
     */
    function eibs_mapping_array_all_pars($ar_record_original, $s_fieldname = 'FILE_FIELDNAME') {
        $ar_records = array();
        foreach ($ar_record_original as $key => $field) {
            $ar_records[$field[$s_fieldname]] = $field;
        }
        return $ar_records;
    }

    /**
     * Look for an entry in the definition array and returns its parameters
     * @param array, text
     * @return array
     */
    function eibs_def_array_entry_pars($ar_record_original, $s_entry_name) {
        $ar_record = null;
        $total = count($ar_record_original);
        for ($i = CLX_INIT_ZERO; $i < $total; $i++) {
            if ($ar_record_original[$i]['FIELD'] == $s_entry_name) {
                $ar_record = $ar_record_original[$i];
            }
        }
        return $ar_record;
    }

    /**
     * Returns the sending result message of a items sent.
     * This result may be OK or not, case in which shows the list of denial reasons
     * Note: previously was the line:
     * 	$msg =($result[CLX_NUM_ZERO]['ERRNUM'] === '0')?dict('GEN_LBL_OK'):dict('GEN_LBL_FAIL') . ', <li>'.implode('</li><li>',$this->formatIbsErrorMsg($result[CLX_NUM_ZERO], $this->ar_last_sent_skt)).'</li>';
     * @param array, array
     * @return text
     */
    function print_error_out($result, $ar_last_sent_skt) {
        if (!empty($this->debug))
            echoLog(debug_dump('print_error_out - result', $result));
        if (empty($result[CLX_NUM_ZERO]['HEADERMSG']['ERRNUM'])) {//No error
            $msg = EIBS_LBL_OK;
        } else {
            $msg = dict('GEN_LBL_FAIL') . ', ' .
                    $this->ar_error_display_out($this->formatIbsErrorMsg($result[CLX_NUM_ZERO]));
        }
        return $msg;
    }

    function err_msg_report_det($result) {
        $msg = '';
        foreach ($result as $s_name => $s_value) {
            if (is_array($s_value)) {
                $msg .= $this->err_msg_report_det($s_value);
            } else {
                $msg .= '<br>' . CLX_CHAR_NL . '* ' . $s_name . ' = ' . $s_value;
            }
        }
        return $msg;
    }

    /*
     * Includes de JS library of general functions
     * @param void
     * @return void
     */

    function include_js_ultilities() {
        $contenidoHTML = CLX_CHAR_NL . '<script language="JavaScript">' . CLX_CHAR_NL;
        echo $contenidoHTML;
        include kPathToWebSite . 'js/standard/util.js';
        $contenidoHTML = CLX_CHAR_NL . '</script>' . CLX_CHAR_NL;
        $contenidoHTML;
    }

    /*
     * To remove leading zeroes from the IBS CUSCUN
     * @param text, text
     * @return text
     */
    function remove_leading_zeroes($s_str, $s_char_to_remove = '0') {
        while (substr($s_str, CLX_NUM_ZERO, CLX_NUM_ONE) == $s_char_to_remove) {
            $s_str = substr($s_str, CLX_NUM_ONE);
            if ($s_str === '')
                break;
        }
        return $s_str;
    }

    /*
     * Alters Process Schedule data to run it inmediatelly
     * @param text, text, array, text
     * @return text
     */

    function create_proc_sched_sched($s_title, $s_sched_process, $ar_pars, $s_recurrence = null) {
        return $this->proc_eng_params->create_proc_sched($s_title, $ar_pars, $s_recurrence, $s_sched_process);
    }

    /**
     * End dialog
     * @param text, text
     * @return text
     */
    function done_dialog($title, $msg) {
        $table = new htmlTable(CLX_WIDTH_TWO);
        $table->setProps(array(CLX_PROP_CLASS => CLX_CLASS_FORM, CLX_PROP_ALIGN => 'center'));

        $table->addCell($title, array(CLX_PROP_CLASS => CLX_CLASS_TITLE, CLX_PROP_COLSPAN => CLX_NUM_ONE));
        $table->addCell(CLX_CHAR_HTML_NBSP, array(CLX_PROP_COLSPAN => CLX_NUM_ONE));
        $table->addCell($msg, array(CLX_PROP_COLSPAN => CLX_NUM_ONE));
        $table->addCell(CLX_CHAR_HTML_NBSP, array(CLX_PROP_COLSPAN => CLX_NUM_ONE));
        return $table->source();
    }

// End method
    /*
     * Makes the call to load the massive operations socket on IBS. This is necessary because thos interfaces are normally down cause it use a lot of server load percent
     * @param void
     * @return array
     */
    function massive_socket_ondemand_loader() {
        if ($this->b_massive_socket_ondemand_loader != true)
            return array();
        $record = array();
        $record['USERNAME'] = $this->username;
        $EBSK = getTool('EBSK');
        $EBSK->debug = $this->debug;
        $EBSK->createMessage('QCMDEXC', $record);
        $ibsuser_data = $EBSK->send();
        return $ibsuser_data;
    }

// End method
    /**
     * Ejecutes (put on schedule) the IBS interfaces queue monitor
     * @param void
     * @return text
     */
    function ibiq_schedule_4_exec() {
        $a = new ibs_interface_queue();
        $a->verif_queue();
        ClxFreeReturn($this->create_proc_sched_sched('', 'IBS_IQ_01', array()));
    }

    /**
     * Reporta los errores en las excepciones del proceso y en el reporte de la cola de interfaces
     * @param text, boolean, array
     * @return void
     */
    function report_error($msg, $error = true, $a_fields = array()) {
        if (!empty($this->debug))
            echoLog('<br>report_error(' . $msg . ', ' . ($error ? 'true' : 'false'));
        if (!empty($error))
            $this->proc_eng_params->report_exceptions($msg, $a_fields);
        $this->o_response_rpt->write($msg . '<br>' . CLX_CHAR_NL);
    }

    /*
     * Get the specified field's attributes
     * Maintains its own property variable as a buffer of already taken fields' attributes
     * @param text, text
     * @return variant
     */

    function get_fld_attrib($s_fld_name, $s_fld_attrib = 'FIELD_SIZE') {
        if ($this->s_fld_attrib == null)
            $this->s_fld_attrib = array();
        if (!isset($this->s_fld_attrib[$s_fld_name])) {
            $this->s_fld_attrib[$s_fld_name] = array();
        }
        if (!isset($this->s_fld_attrib[$s_fld_name][$s_fld_attrib])) {
            $this->s_fld_attrib[$s_fld_name][$s_fld_attrib] = ClxCommon::$db->getField('CLX_DB_FIELDS', $s_fld_attrib, 'FIELDNAME = ' . singleQuote($s_fld_name));
        }
        return $this->s_fld_attrib[$s_fld_name][$s_fld_attrib];
    }

}

// ibs() Class End
/*
  ----------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------   Vista de Proceso ---- Proceso: PARA PRUEBAS DE LA IMPORTACION DE ACTIVIDADES DE INVERSION
  ---- O CUANDO ES UN PROCESO AUTOMATICO  C?digo 	IBS_INAT_IMP Nombre Corto 	IBS : Investment Interfase
  Estado 	Activo Modulo Caja 	FIDU Descripci?n Nivel de Prioridad 	100  ------------------------  Secciones del Proceso  Nombre Corto: Main Section
  Is Main Section: S?
  Estado: Activo
  ------------------------  Paso del Proceso  Secci?n 	BAVS05, Main Section Nombre Corto 	PHP Code N?mero de Sequencia 	1 Code Type 	C?digo PHP Do Secci?n Do Secci?n Tipo 	NOT Biblioteca de C?digo 	INUT Code, Library Call, or Do Section Condition 	$a = new fidu_investments_interfase(); ; $a->inat_import($parameters['ANYFILE']); echo $a->contents();
  // Nota: Si no se le pasa el par?metro, muestra una pantalla que pide el archivo de entrada
  ------------------------  Parametro del Proceso:
  Nombre Corto: Investments file
  Campo: ANYFILE
  ------------------------------------------------------------------------  Cronograma del Proceso  C?digo 	IBS_INAT_IMP Nombre Corto 	IBS : Investment Interfase
  Proceso 	IBS_INAT_IMP, IBS : Investment Interfase
  Estado 	Activo Notas  Opciones de Cronograma Recurrencia 	Nunca  Opciones Adicionales Dependency Process Mostrar 	S? Debug 	S?  Par?metros de Ejecuci?n Campo: ANYFILE
  Valor del Parametro: clxftp/INAT_IMPORT.txt
  ---------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------- ---- Opci?n de Men?: IMPORTACION DE ACTIVIDADES DE INVERSION
  Item de Men? Nombre: 	Import Actividad Inversi?n
  Descripci?n:
  Versi?n: 	1.0.0-1
  C?digo de M?dulo: 	FIDU
  Detalles de Items de Men?:
  Idioma: 	PHP
  Biblioteca: 	EIBS
  Ventana: 	Misma Ventana
  Llamada: $a = new fidu_investments_2(); $a->inat_import(); $contents = $a->contents();
  ---------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------- */
/*
  ---------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------- Lista: ACCO_FIDU
  ---- Bot?n: TRANSMISION DE CLIENTES DEL CLX AL IBS
  Nombre: 	Transmitir a IBS
  Prendido: 	Registros m?ltiples seleccionados
  Acceder: 	No Restringido
  Llamada: 	$a = new ibs(); echo $a->createClient();
  Biblioteca: 	EIBS
  Ejecutar: 	Ejecutar una vez para todos los registros?
  Ventana: 	Ventana Pop
  Continuar: 	No
  ---------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------- Lista: FICO
  ---- Bot?n: TRANSMISION DE CONTRATOS DEL CLX AL IBS
  Nombre: 	Transmitir a IBS
  Prendido: 	Registros m?ltiples seleccionados
  Acceder: 	No Restringido
  Llamada: 	$a = new ibs(); echo $a->createFiduContracts();
  Biblioteca: 	EIBS
  Ejecutar: 	Ejecutar una vez para todos los registros?
  Ventana: 	Ventana Pop
  Continuar: 	No
  ----------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------  Proceso ---- Proceso: PARA PRUEBAS DE LA IMPORTACION DE CLIENTES DE IBS A CLX
  C?digo 	IBS_CLI_IMP Nombre Corto 	IBS : Clients data import from IBS
  Estado 	Activo Modulo Caja 	FIDU Descripci?n Nivel de Prioridad 	100  ------------- Secci?n del Proceso  C?digo 	BAVS04 Nombre Corto 	Main Section Is Main Section 	S? Estado 	Activo Proceso 	IBS_CLI_IMP, IBS : Clients data import from IBS
  Descripci?n  -------------- Paso del Proceso  Secci?n 	BAVS04, Main Section Nombre Corto 	PHP Code N?mero de Sequencia 	1 Code Type 	C?digo PHP Do Secci?n Do Secci?n Tipo 	NOT Biblioteca de C?digo 	EIBS Code, Library Call, or Do Section Condition 	$a = new ibs(); $a->clientImport(parameters['ANYFILE']);
  --------------  Vista de Parametros del Proceso Proceso 	IBS_CLI_IMP, IBS : Clients data import from IBS
  Nombre Corto 	Clients file Campo 	ANYFILE  --------------  Cronograma del Proceso  C?digo 	IBS_CLI_IMP Nombre Corto 	IBS : Clients data import from IBS
  Proceso 	IBS_CLI_IMP, IBS : Clients data import from IBS
  Estado 	Activo Notas  Opciones de Cronograma Fecha de Inicio 	Oct 31, 06 14:53:00
  Recurrencia 	Nunca Pr?xima Ejecuci?n 	Oct 31, 06 Hora pr?xima ejecuci?n 	14:53:18
  Opciones Adicionales Dependency Process Mostrar 	S? Debug 	S?  Par?metros de Ejecuci?n Campo: ANYFILE
  Valor del Parametro: clxftp/PERSON.TXT o JURIDIC.TXT
  ----------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------  Proceso ---- PROCESO DE PRUEBAS DEL EXPORT JEEX  C?digo 	IBS_JE_EX Nombre Corto 	IBS : Journal Entry Export file generation
  Estado 	Activo Modulo Caja 	EIBS Descripci?n 	Prueba del modulo de exportacion diaria de resumen de comprobantes Nivel de Prioridad 	100  --------------------------  Secci?n del Proceso C?digo 	BAVS03 Nombre Corto 	Main Section Is Main Section 	S? Estado 	Activo Proceso 	IBS_JE_EX, IBS : Journal Entry Export file generation
  Descripci?n  -----------------------------  Secci?n 	BAVS03, Main Section Nombre Corto 	PHP Code N?mero de Sequencia 	1 Code Type 	C?digo PHP Do Secci?n Do Secci?n Tipo 	NOT Biblioteca de C?digo 	EIBS Code, Library Call, or Do Section Condition 	$a = new ibs(); $a->ibs_journal_export('BAV', '');
  ----------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------  Proceso ---- PROCESO DE MONITOREO DE RESPUESTAS  DEL EXPORT JEEX  C?digo 	IBS_JEEX_MONITOR Nombre Corto 	IBS : Journal export error monitor
  Estado 	Activo Modulo Caja 	FIDU Descripci?n Nivel de Prioridad 	100  --------------  C?digo 	BAVS04 Nombre Corto 	Main Section Is Main Section 	S? Estado 	Activo Proceso 	IBS_JEEX_MONITOR, IBS : Journal export error monitor
  Descripci?n  --------------  Paso del Proceso Secci?n 	BAVS04, Main Section Nombre Corto 	PHP Code N?mero de Sequencia 	1 Code Type 	C?digo PHP Do Secci?n Do Secci?n Tipo 	NOT Biblioteca de C?digo 	EIBS Code, Library Call, or Do Section Condition 	$a = new ibs(); $a->ibs_journal_export_logfile_check();
  --------------  Cronograma del Proceso C?digo 	IBS_JEEX_MONITOR Nombre Corto 	IBS : Journal export error monitor
  Proceso 	IBS_JEEX_MONITOR, IBS : Journal export error monitor
  Estado 	Activo Notas  Opciones de Cronograma Fecha de Inicio 	Nov 09, 06 09:59:00
  Recurrencia 	Nunca Pr?xima Ejecuci?n 	Nov 09, 06 Hora pr?xima ejecuci?n 	09:59:22
  Opciones Adicionales Dependency Process Mostrar 	S? Debug 	S?  ----------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------  LISTADO PARA MONITOREAR RESULTADO DE LA EXPORTACION CONTABLE A IBS  Listado:
  C?digo: 	JEEX
  Nombre: 	JOURNAL_EXPORT
  Descripci?n: 	Auto generated default listing
  Este listado pertenece al m?dulo: 	GL
  Este listado trabaja con la tabla: 	JEEX
  Consulta: 	JEEX
  T?tulo: 	Exportaciones
  Subt?tulo:
  ------  Formas en listado actual:
  Crear Forma:
  Modificar Forma:
  Mostrar Forma: 	JEEX
  Duplicar Forma:
  Borrar Forma:
  Buscar Forma:
  ------  Botones Extra  Nombre: Retransferir
  Prendido: Registros M?ltiples Seleccionados
  Acceder: No restringido
  Llamada: $a = new ibs(); $a->ibs_journal_export_send_outfile();
  Biblioteca: EIBS
  Ejecutar: 	Ejecutar una vez para todos los registros
  Ventana: 	Ventana Pop
  Continuar:	No
  ------  Nombre: Verificar Respuestas
  Prendido: Siempre prendido
  Acceder: No restringido
  Llamada: $a = new ibs(); $a->ibs_journal_export_logfile_check();
  Biblioteca: EIBS
  Ejecutar: 	Ejecutar una vez para todos los registros
  Ventana: 	Ventana Pop
  Continuar:	No
  ----------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------  MENU PARA MONITOREAR RESULTADO DE LA EXPORTACION CONTABLE A IBS  Nombre: Monitor de Transferencias
  Descripci?n: Permite verificar el estatus de las transaferencias realizadas mediante el bot?n 'Emisi?n de Cheques/Transferencias' en las Transacciones de Contratos de Fideicomiso
  C?digo de M?dulo: FIDU
  Tipo: List Records
  Listado: JEEX1
  ----------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------  BOTON PARA IMPRIMIR CHEQUES Y HACER TRANSFERENCIAS  Listado: FIAT_FICO
  Botones Extra:
  Nombre: 	Emitir Cheques/Transferencias | Transfers/Checks Print
  Prendido: 	Registros m?ltiples seleccionados
  Acceder: 	No restringido
  Llamada: 	$a = new ibs(); $a->debug = false; $contents = $a->printChecks();
  Biblioteca:	EIBS
  Ejecutar: 	Ejecutar una vez para todos los registros
  Ventana: 	Venetana POP
  Continuar:	No
  ----------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------  C?digos  C?digo de Tabla		Formato		?ltimo ID	Incremento JEEX				########	000000	1
  ----------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------  MONITOR DE TRANSFERENCIAS  Menu  Nombre: 	Monitor de Transferencias
  Descripci?n: 	Permite verificar el estatus de las transaferencias realizadas mediante el bot?n 'Emisi?n de Cheques/Transferencias' en las Transacciones de Contratos de Fideicomiso
  C?digo de M?dulo: 	FIDU
  Detalles de Items de Men?:
  Listado: 	JEEX1
  ----------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------  LISTADO PARA MONITOREAR RESULTADO DE LA EXPORTACION CONTABLE A IBS  Listado:
  C?digo: 	JEEX1
  Nombre: 	JOURNAL_EXPORT_ITRF
  Descripci?n: 	Auto generated default listing
  Este listado pertenece al m?dulo: 	GL
  Este listado trabaja con la tabla: 	JEEX
  Consulta: 	JEEX_ITRF
  T?tulo: 	Transferencias Internas
  Subt?tulo:
  ------  Formas en listado actual:
  Crear Forma:
  Modificar Forma:
  Mostrar Forma: 	JEEX
  Duplicar Forma:
  Borrar Forma:
  Buscar Forma:
  ------  Botones Extra  Nombre: Retransferir
  Prendido: Registros M?ltiples Seleccionados
  Acceder: No restringido
  Llamada: $a = new ibs(); $a->debug=false; $a->ibs_journal_export_send_outfile();
  Biblioteca: EIBS
  Ejecutar: 	Ejecutar una vez para todos los registros
  Ventana: 	Ventana Pop
  Continuar:	No
  ------  Nombre: Verificar Respuestas
  Prendido: Siempre prendido
  Acceder: No restringido
  Llamada: $a = new ibs(); $a->debug=false; $a->ibs_transfer_logfile_check();
  Biblioteca: EIBS
  Ejecutar: 	Ejecutar una vez para todos los registros
  Ventana: 	Ventana Pop
  Continuar:	No
  ----------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------  PROCESO PARA CHEQUEAR EL STATUS DE LAS TRANSFERENCIAS  ------  Vista de Proceso C?digo IBS_ITRF_MONITOR  Nombre Corto: IBS : Internal Money Transfers status monitor
  Estado: Activo
  Modulo Caja: FIDU
  Descripci?n: Process the file IBS returns in response to the transfer request sent by the Transfers/Checks print option in FIDU transactions
  ------  Vista de Secci?n del Proceso  C?digo: IBS_ITRF_MONITOR
  Nombre Corto: Main Section
  Is Main Section: S?
  Estado: Activo
  Proceso: IBS_ITRF_MONITOR, IBS : Internal Money Transfers status monitor
  Descripci?n:
  ------  Vista de Paso del Proceso  Secci?n: IBS_ITRF_MONITOR, Main Section
  Nombre Corto: PHP Code
  N?mero de Sequencia: 0
  Code Type: C?digo PHP
  Do Secci?n:
  Do Secci?n Tipo: NOT
  Biblioteca de C?digo: EIBS
  Code, Library Call, or Do Section Condition: $a = new ibs(); $a->debug=false; $a->ibs_transfer_logfile_check();
  ------  Vista de Cronograma del Proceso C?digo: IBS_ITRF_MONITOR
  Nombre Corto: IBS : Internal Money Transfers status monitor
  Proceso: IBS_ITRF_MONITOR, IBS : Internal Money Transfers status monitor
  Estado: Activo
  Notas:
  Opciones de Cronograma Fecha de Inicio: Dec 29, 06 00:00:00
  Recurrencia: Nunca ---- OJO: DEBERIA ESTAR ACTIVO CADA HORA EN SITUACIONES NORMALES
  Pr?xima Ejecuci?n: Dec 29, 06
  Hora pr?xima ejecuci?n: 09:27:49
  Opciones Adicionales Proceso de Dependencia:
  Mostrar: S?
  Depurador: S?
  ----------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------  PROCESO PARA CHEQUEAR EL STATUS DEL MASIVO DE CREACION DE CLIENTES  Menu  Nombre: Verificaci?n Creaci?n Cliente IBS
  Descripcion: Verifica la respuesta de IBS a la solicitud de creaci?n masiva de Clientes
  C?digo de M?dulo: FIDU
  Tipo: Code Lib
  Venta: Misma
  Lenguaje: PHP
  Biblioteca: EIBS
  Llamada: $a = new ibs(); $contents .= $a->mass_client_creation_response_checker();
  ------  ----------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------  Notas:
  En el Formato CLX_INPCLX  INPDAC, BANK_ACCOUNT_NO_DB, Nro. de Cuenta (Producto) a debitar INPCAC, BANK_ACCOUNT_NO_CR, Nro. de Cta. (Producto) a acreditar IDCODE, GL_JOURNAL_EXPORT, C?digo de la entrada en el LogFile, para poder dar el nombre al archivo .txt que se env?a y saber en el Log que se recibe a cual de los .txt se refiere la respuesta */
/*
 * ***************************
 * Class to manage processes that creates a file with secuential names and is necesary to keep track the its generation info
 * ****************************
 */
class seq_name_file_generation {

    var $s_export_file_basename;
    var $s_export_dir;
    var $s_rec_type;
    var $s_effdate;
    var $s_cofiex;
    var $s_cofiex_html;
    var $s_cofierr;
    var $s_otherinfo;
    var $s_export_file_ext = '.txt';
    var $s_export_html_file_ext = '.html';
    var $s_tablealias = 'JEEX';
    var $i_max_buffer_records = CLX_NUM_TEN;
    var $s_export_filename_token = '%s_idcode%';
    var $s_output_filename;
    var $s_output_dir;
    var $s_output_dir_html;
    var $b_output_to_file = true;
    var $o_clx_file_html;
    var $o_clx_file_txt;
    var $s_output_filename_txt;
    var $s_output_filename_html;
    var $s_filename_prefix = '';
    var $s_line_footer_html;
    var $n_id;
    var $s_idcode;
    var $debug = true;
    var $s_tablename;
    var $i_cont_aux_reg;
    var $i_cont_tot_reg;
    var $mappings;
    var $EBSK;
    var $msg_code;
    var $b_stop_file_generation = false; //   to handle flat file lines socket generation errors
    var $b_stop_file_gen_msg = '';

    /*
     * Class Constructor
     * @param array
     * @return void
     */
    function __construct($ar_pars = null) {
        if ($ar_pars != null)
            $this->def_pars_set($ar_pars);
        if (!function_exists('setCreationData'))
            includeCodeLib('GEN_REC_UTIL');
       
    }

    /*
     * Default paramenters values set
     * @param array
     * @return void
     */
    function def_pars_set($ar_pars) {
        $this->s_export_filename_token = (isset($ar_pars['export_filename_token'])) ? $ar_pars['export_filename_token'] : $this->s_export_filename_token;
        $this->i_max_buffer_records = (isset($ar_pars['max_buffer_records'])) ? $ar_pars['max_buffer_records'] : $this->i_max_buffer_records;
        $this->s_tablealias = (isset($ar_pars['tablealias'])) ? $ar_pars['tablealias'] : $this->s_tablealias;
        $this->s_export_file_ext = (isset($ar_pars['export_file_ext'])) ? $ar_pars['export_file_ext'] : $this->s_export_file_ext;
        $this->s_export_file_basename = (isset($ar_pars['export_file_basename'])) ? $ar_pars['export_file_basename'] : $this->s_export_filename_token;
        $this->s_export_dir = (isset($ar_pars['export_dir'])) ? $ar_pars['export_dir'] : '';
        $this->s_rec_type = (isset($ar_pars['rec_type'])) ? $ar_pars['rec_type'] : '';
        $this->s_effdate = (isset($ar_pars['efftdate'])) ? $ar_pars['efftdate'] : getNow('Y-m-d');
        $this->s_cofiex = (isset($ar_pars['cofiex'])) ? $ar_pars['cofiex'] : '';
        $this->s_cofiex_html = (isset($ar_pars['cofiex_html'])) ? $ar_pars['cofiex_html'] : '';
        $this->s_cofierr = (isset($ar_pars['cofierr'])) ? $ar_pars['cofierr'] : '';
        $this->s_otherinfo = (isset($ar_pars['otherinfo'])) ? $ar_pars['otherinfo'] : '';
        $this->EBSK = (isset($ar_pars['EBSK'])) ? $ar_pars['EBSK'] : null;
        $this->msg_code = (isset($ar_pars['msg_code'])) ? $ar_pars['msg_code'] : '';
        $this->mappings = $this->EBSK->getMappings($this->msg_code);
		//pre($this->msg_code);
		//echo '----------------------';
        $this->s_tablename = tableName($this->s_tablealias);
        $this->b_output_to_file = (isset($ar_pars['output_to_file'])) ? $ar_pars['output_to_file'] : $this->b_output_to_file;
        $this->s_filename_prefix = (isset($ar_pars['filename_prefix'])) ? $ar_pars['filename_prefix'] : $this->s_filename_prefix;
        $this->s_export_html_file_ext = (isset($ar_pars['export_html_file_ext'])) ? $ar_pars['export_html_file_ext'] : $this->s_export_html_file_ext;
        $this->s_output_dir = def_dir_ass($ar_pars, 'output_dir', kPathToWebSite . kPathToData);
        $this->s_output_dir_html = def_dir_ass($ar_pars, 'output_dir_html', kPathToData);
    }

    /*
     * Prepares record creation
     * @param void
     * @return array
     */

    function create_record() {
        $ar_response = array();
        $ar_response['error'] = false;
        $ar_response['errormsg'] = '';
        //   - Caso 1002774 - CAR-Interfaces IBS
        // En el caso de Caruna, tiene un servidor Linux y siempre da el mensaje de error: CODE_ERR_FILE_INVALIDDIR
        // Por estar inclu�do el separador de directorios al final, Linux no lo reconoce como directorio...
        // SOLUCION: quitar el separador de directorios al final cuando se verifica si es un directorio
        $tmp_dir = $this->s_export_dir;
        if (substr($tmp_dir, CLX_NUM_MINUS_1) == DIRECTORY_SEPARATOR)
            $tmp_dir = mb_substr($tmp_dir, CLX_NUM_ZERO, mb_strlen($tmp_dir) - CLX_NUM_ONE);
        if ($this->s_export_dir !== '' && !is_dir($tmp_dir)) {
            /*
             * CODE_ERR_FILE_INVALIDDIR
             * Invalid directory: ^1
             * Directorio no es v�lido: ^1
             */
            $ar_response['error'] = true;
            $ar_response['errormsg'] = '[*2*] ' . dict('CODE_ERR_FILE_INVALIDDIR', $tmp_dir);
            return $ar_response;
        }
        $output_record = array();
        $output_record['CREATEDBY'] = '';
        $output_record['MODIFIEDBY'] = '';
        $output_record['CDATE'] = '';
        $output_record['MDATE'] = '';
        $output_record = setCreationData($output_record);
        $output_record['IDCODE'] = 'AUTO';
        $output_record = setCreationIdCode($output_record, $this->s_tablealias);
        $this->s_idcode = $output_record['IDCODE'];
        $this->s_output_filename = $this->s_export_file_basename;
        $this->s_output_filename = str_replace($this->s_export_filename_token, $output_record['IDCODE'], $this->s_output_filename);
        $output_record['IDNAME'] = $this->s_output_filename . $this->s_export_file_ext;
        $output_record['JEEX_REC_TYPE'] = $this->s_rec_type;
        $output_record['EFFTDATE'] = $this->s_effdate;
        $output_record['JEEX_COFIERR'] = addslashes($this->s_cofierr);
        $output_record['JEEX_COFIEX'] = addslashes($this->s_cofiex);
        $output_record['JEEX_COFIEX_HTML'] = addslashes($this->s_cofiex_html);
        $output_record['JEEX_OTHERINFO'] = addslashes($this->s_otherinfo);
        // File export process status verification
        $output_record['JEEX_STATUS'] = (empty($this->s_cofierr) ? 'IDLE' : 'ERRGEN');
        if ($this->b_output_to_file === true) {
            $this->s_output_filename_txt = $this->s_output_dir . $this->s_filename_prefix . $this->s_output_filename . $this->s_export_file_ext;
            
			//pre('abriendo para escribir txt');
			//pre($this->s_output_filename_txt);
			$this->o_clx_file_txt = new clx_file($this->s_output_filename_txt, 'w');
            $this->o_clx_file_txt->open();
          //  pre($this->o_clx_file_txt->errorMsg);
            if ($this->o_clx_file_txt->error === true) {
                $ar_response['error'] = true;
			
                $ar_response['errormsg'] = $this->o_clx_file_txt->errorMsg . ': ' . $this->s_output_filename_txt;
                //pre('saliendo error');
				//pre($a_response);
				return $ar_response;
            }
			//pre('abriendo para escribir html');
            $this->s_output_filename_html = kPathToWebSite . $this->s_output_dir_html . $this->s_filename_prefix . $this->s_output_filename . $this->s_export_html_file_ext;
            //pre( $this->s_output_filename_html);
			$this->o_clx_file_html = new clx_file($this->s_output_filename_html, 'w');
            $this->o_clx_file_html->open();
			
            if ($this->o_clx_file_html->error === true) {
                $ar_response['error'] = true;
                $ar_response['errormsg'] = $this->o_clx_file_html->errorMsg . ': ' . $this->s_output_filename_html;
                //pre('saliendo error');
				//pre($a_response);
				return $ar_response;
            }
            $this->o_clx_file_html->write($output_record['JEEX_COFIEX_HTML']);
            if ($this->o_clx_file_html->error === true) {
                $ar_response['error'] = true;
				//pre('saliendo error html 2');
				//pre($a_response);
                $ar_response['errormsg'] = $this->o_clx_file_html->errorMsg . ': ' . $this->s_output_filename_html;
                return $ar_response;
            }
            /*
             * GEN_MSG_IBS_EEX_CLICTOSEE_REPORT
             * Click here to see this field contents
             * Haga clic aqu? para ver el contenido de este campo
             */
            $s_url = $this->s_output_dir_html . $this->s_filename_prefix . $this->s_output_filename . $this->s_export_html_file_ext;
            //$output_record['JEEX_COFIEX_HTML'] = '<A HREF=' . singleQuote($s_url) . ' TARGET=' . singleQuote('_new') . '>' . dict('GEN_MSG_IBS_JEEX_CLICTOSEE_REPORT') . '</A>';
            $output_record['JEEX_COFIEX_HTML'] = '<A HREF="' . $s_url . '" TARGET="_new">' . dict('GEN_MSG_IBS_JEEX_CLICTOSEE_REPORT') . '</A>';
            /*
             * GEN_MSG_IBS_JEEX_CLICTOSEE_EXPRTFILE
             * Click here to review the exported file
             * Haga clic aqu? para ver el archivo exportado
             */
        }
        ClxCommon::$db->debug = $this->debug;
        $this->n_id = ClxCommon::$db->createRecord($this->s_tablename, $output_record, true);
        $output_record['ID'] = $this->n_id;
        ClxCommon::$db->debug = false;
        $this->s_cofierr = '';
        $this->s_cofiex = '';
        $this->s_cofiex_html = '';
        $this->s_otherinfo = '';
        $this->i_cont_aux_reg = CLX_INIT_ZERO;
        $this->i_cont_tot_reg = CLX_INIT_ZERO;
        $ar_response['output_record'] = $output_record;
		//pre('output record');
		//pre($ar_response);
        unset($output_record);
        return $ar_response;
    }

    /*
     * JEEX_COFIEX_HTML column html content - header
     * @param void
     * @return text
     */
    function cofiex_linegen_header() {
        $this->s_line_footer_html = '<TR>';
        $n_skip_cols = CLX_NUM_ONE;
        $s_line_html = '<TABLE BORDER=1><TR><TD>###</TD>';
		//pre('.........>');
		//pre($this->mappings);	
        foreach ($this->mappings as $key => $field) {
			
            if ($field['TABLE_FIELDNAME'] === 'DEBITS' || $field['TABLE_FIELDNAME'] === 'CREDITS') {
                if ($n_skip_cols > CLX_NUM_ZERO) {
                    $this->s_line_footer_html .= '<TD COLSPAN=' . singleQuote(($n_skip_cols)) . '>&nbsp;</TD>';
                }
                $this->s_line_footer_html .= '<TD>%' . $field['TABLE_FIELDNAME'] . '%</TD>';
                $n_skip_cols = CLX_INIT_ZERO;
            } else {
                $n_skip_cols++;
            }
            $s_line_html .= '<TD>' . strblank($field['DESCRIPTION'], '--') . '</TD>';
        }
        /*
         * OBSERVATIONS
         * Observations
         * Observationes
         */
        $s_line_html .= '<TD>' . dict('OBSERVATIONS') ."xxx". '</TD>';
        $s_line_html .= '</TR>';
        if ($n_skip_cols > CLX_NUM_ZERO) {
            $this->s_line_footer_html .= '<TD COLSPAN=' . singleQuote(($n_skip_cols)) . '>&nbsp;</TD>';
        }
        $this->s_line_footer_html .= '</TR>';
        return $s_line_html;
    }

    /*
     * JEEX_COFIEX_HTML column html content - detail lines
     * @param integer, array
     * @return text
     */
    function cofiex_linegen_detail($i_cont_reg, $ar_socket_line, $s_obs = '') {
        $s_line_html = '<TR><TD>' . $i_cont_reg . '</TD>';
        foreach ($ar_socket_line as $key => $field) {
            $val = $field['VAL'];
            if ($this->EBSK->remove_decimal_point === true && $field['TYPE'] === 'DECN' && $field['DECIMALS'] > CLX_NUM_ZERO) {
                $val = substr($val, CLX_NUM_ZERO, strlen($val) - $field['DECIMALS']) . '.' . substr($val, -$field['DECIMALS']);
            }
            $s_line_html .= '<TD>' . strblank($val, '--') . '</TD>';
        }
        $s_line_html .= '<TD>' . strblank($s_obs, '--') . '</TD>';
        $s_line_html .= '</TR>';
        return $s_line_html;
    }

    /*
     * Inserts record BLOB columns data. When the defined buffer max. record counter is reached, flush data to avoid strings memory overflow
     * @param text, text, text
     * @return void
     */
    function insert_record_data($s_cofiex = null, $s_cofiex_html = null, $s_otherinfo = null) {
        if (!empty($this->debug))
            echoLog('insert_record_data<br>$s_cofiex<br>$s_cofiex_html<br>$s_otherinfo<br><br>');
        // Export file content
        $this->s_cofiex .= ($s_cofiex == null) ? '' : $s_cofiex;
        // Export HTML report
        $this->s_cofiex_html .= ($s_cofiex_html == null) ? '' : $s_cofiex_html;
        // Other info
        $this->s_otherinfo .= ($s_otherinfo == null) ? '' : $s_otherinfo;
        // buffer record counter
        $this->i_cont_aux_reg++;
        if ($this->i_cont_aux_reg >= $this->i_max_buffer_records) {
            // Flush records data to the database
            $this->flush_record_buffer();
            // Initialize buffer containers and counters
            $this->s_cofiex = '';
            $this->s_cofiex_html = '';
            $this->s_otherinfo = '';
            $this->i_cont_aux_reg = CLX_INIT_ZERO;
        }
    }

    /*
     * Flush records data to the database
     * @param void
     * @return void
     */
    function flush_record_buffer() {
        if ($this->b_output_to_file === true) {
            $this->o_clx_file_txt->write($this->s_cofiex);
            if ($this->o_clx_file_txt->error === true) {
                echoLog($this->o_clx_file_txt->errorMsg . ':' . $this->s_output_filename_txt);
            } else {
                $this->o_clx_file_html->write($this->s_cofiex_html);
                if ($this->o_clx_file_html->error === true) {
                    echoLog($this->o_clx_file_html->errorMsg . ':' . $this->s_output_filename_html);
                } else {
                    $sql = 'UPDATE ' . $this->s_tablename . ' SET JEEX_OTHERINFO = concat(JEEX_OTHERINFO, ' . singleQuote(addslashes($this->s_otherinfo)) . ') WHERE IDCODE = ' . singleQuote($this->s_idcode) . ' LIMIT 1';
                    if (!empty($this->debug))
                        echoLog('flush_record_buffer - ' . $sql . '<br>');
                    ClxCommon::$db->executeQuery($sql);
                }
            }
        } else {
			
            $sql = 'UPDATE ' . $this->s_tablename . ' SET JEEX_COFIEX = concat(JEEX_COFIEX, ' . singleQuote(
                    ($this->s_cofiex)) . '), JEEX_COFIEX_HTML = concat(JEEX_COFIEX_HTML, ' . singleQuote(addslashes($this->s_cofiex_html)) . '), JEEX_OTHERINFO = concat(JEEX_OTHERINFO, ' . singleQuote(addslashes($this->s_otherinfo)) . ') WHERE IDCODE = ' . singleQuote($this->s_idcode) . ' LIMIT 1';
            //pre($sql);
			if (!empty($this->debug))
                echoLog('flush_record_buffer - ' . $sql . '<br>');
            ClxCommon::$db->executeQuery($sql);
        }
    }

    /*
     * Transfers the export result to the output file. If there`s an error, report it in the JEEX_COFIERR column
     * @param variant, text, variant
     * @return void
     */

    function transfer_output_file($v_cofiex = null, $s_cofiex_html = '', $v_otherinfo = null) {
		//echo 'transfer_output_file';
	   // Flush pending data
        $this->i_cont_aux_reg = $this->i_max_buffer_records + CLX_NUM_ONE; // to force the flush of pending content
        //$this->insert_record_data(null, '</TABLE>', null);	// To close the HTML version of the exported file
        $this->insert_record_data($v_cofiex, $s_cofiex_html, $v_otherinfo); // To close the HTML version of the exported file
        if ($this->b_output_to_file == true) {
			//echo 'output file';
            $this->o_clx_file_txt->close();
            $this->o_clx_file_html->close();
            $s_jeex_cofiex = '';
        } else {
			///echo 'not output file';
            // Extract file contents
            $s_jeex_cofiex = ClxCommon::$db->getField($this->s_tablename, 'JEEX_COFIEX', 'IDCODE = ' . singleQuote($this->s_idcode));
			
		}
        // Copy the exporting file to target directory
        if ($this->b_stop_file_generation == false) {
			//echo 'stop generation';
            if (!$this->generate_output_file($s_jeex_cofiex)) {
                /*
                 * GEN_ERR_IBS_JEEX_OUTFILEERR
                 * Error writting output file
                 * Error escribiendo el archivo de salida
                 */
                ClxCommon::$db->updateRecord($this->s_tablename, array('JEEX_COFIERR' => dict('GEN_ERR_IBS_JEEX_OUTFILEERR'), 'JEEX_STATUS' => 'ERRGEN'), 'IDCODE = ' . singleQuote($this->s_idcode));
                //   Reporta error en la copia al directorio final
                $this->b_stop_file_gen_msg = dict('GEN_ERR_IBS_JEEX_OUTFILEERR');
            } else {
                // Everything goes Ok, so update record status. JEEX_STATUS column status options: INPROG = Waiting for IBS to process, NOREC = Empty batch file
                ClxCommon::$db->updateRecord($this->s_tablename, array('JEEX_STATUS' => ($this->i_cont_tot_reg > CLX_INIT_ZERO ? 'INPROG' : 'NOREC')), 'IDCODE = ' . singleQuote($this->s_idcode));
            }
        } else {
			//echo 'else not generation';
            // Error generation flat file...
            if ($this->b_stop_file_gen_msg === '') {
                /*
                 * GEN_ERR_IBS_JEEX_ERGENFILE
                 * Error(s) generating output file
                 * Error(es) generando el archivo de salida
                 */
                $this->b_stop_file_gen_msg = dict('GEN_ERR_IBS_JEEX_ERGENFILE');
            }
            ClxCommon::$db->updateRecord($this->s_tablename, array('JEEX_COFIERR' => $this->b_stop_file_gen_msg, 'JEEX_STATUS' => 'ERRGEN'), 'IDCODE = ' . singleQuote($this->s_idcode));
        }
        unset($s_jeex_cofiex);
    }

    /*
     * Generates the export file on the filesystem
     * @param text
     * @return boolean
     */
    function generate_output_file($s_contenido) {
		//pre('output_file');
        if (!empty($this->debug))
            echoLog('<br>this->s_export_dir=' . $this->s_export_dir);
        if (substr($this->s_export_dir, CLX_NUM_MINUS_1) != DIRECTORY_SEPARATOR)
            $this->s_export_dir .= DIRECTORY_SEPARATOR;
        if(empty($this->fixed_output_filespec)) {
            $s_output_filename = $this->s_export_dir . $this->s_output_filename . $this->s_export_file_ext;
        } else {
            $s_output_filename = $this->fixed_output_filespec;
        }
		//pre($s_output_filename);
        if ($this->b_output_to_file == true) {
			//pre('true output');
            if (!empty($this->debug))
                echoLog('<br>copy $this->s_output_filename_txt="' . $this->s_output_filename_txt . '" to $s_output_filename="' . $s_output_filename . '"<br>');
            if (!copy($this->s_output_filename_txt, $s_output_filename)) {
                echoLog('<br>copy ERROR: '.$this->s_output_filename_txt . ' to ' . $s_output_filename.'<br>');
                return false;
            }
        } else {
			//pre('false output');
            if (!empty($this->debug))
                echoLog('<br>Writting s_output_filename=' . $s_output_filename);
            $o_clx_file = new clx_file($s_output_filename, 'w');
            $o_clx_file->open();
            if ($o_clx_file->error === true) {
                echoLog($o_clx_file->errorMsg . ':' . $s_output_filename);
                return false;
            }
            $o_clx_file->write($s_contenido);
            if ($o_clx_file->error === true) {
                echoLog($o_clx_file->errorMsg . ':' . $s_output_filename);
                return false;
            }
            $o_clx_file->close();
        }
        return true;
    }
}
// jeex Class End

/*
 * Checks for two different levels of default directory value. Arranges the resulting path to include the final slash
 * Parameters:
 * 	 $ar_pars = array of parameters values
 * 	$s_outdir_varname = parameter entry name for the first level of value
 * 	$s_outdir_defval = default value just in case the parameters isn?t specified in $ar_pars
 * @param array, text, text
 * @return text
 */
function def_dir_ass($ar_pars, $s_outdir_varname, $s_outdir_defval) {
    $s_dir = '';
    if (isset($ar_pars[$s_outdir_varname]))
        $s_dir = $ar_pars[$s_outdir_varname];
    if ($s_dir === '' || is_null($s_dir))
        $s_output_dir = $s_outdir_defval;
    else
        $s_output_dir = $s_dir;
    $s_output_dir = trim($s_output_dir);
    if (substr($s_output_dir, CLX_NUM_MINUS_1) != '/' && substr($s_output_dir, CLX_NUM_MINUS_1) != '\\')
        $s_output_dir .= DIRECTORY_SEPARATOR;
    return $s_output_dir;
}

function correct_dir_sep($filespec) {
    // Always uses OS's own directorio separator to avoid command line conflicts
    if(DIRECTORY_SEPARATOR === '\\') {
        $filespec = str_replace ('/', DIRECTORY_SEPARATOR, $filespec);  
    } else {
        $filespec = str_replace ('\\', DIRECTORY_SEPARATOR, $filespec);  
    }
    return $filespec;
}
// End Function
