<?php
/**
 * ETL utilities
 * For more information contact  support@alenet.com
 *
 * @author       $Author: cramirez $
 * @copyright    Alenet, Inc.
 * @since        $Date: 2018-09-29 17:46:00 -0500 (Thu, 20 Sep 2018) $
 * @version    	$Rev: 10045 $
 */

class ETL_utilities {

    private $etl_param = array();
    
    /**
     * Constructor
     * @param array $procParameters
     * @return void
     */
    function __construct($procParameters = array()) {
        /*
         * IMCF = CLX_ETL_IMPORT_CFG | Import Configurator Table
         * IMFT = CLX_ETL_IMPFILES_TMP | ETL - Import Files to be processed
         * IMOQ = CLX_ETL_OP_QUEUE | ETL - Processing Queue
         */
        multiTableName($this, array('IMCF', 'IMFT', 'IMOQ'));
        // Parametros generales
        $this->etl_param = $procParameters;
        $this->etl_param['debug'] = (!empty($this->etl_param['DEBUG'])) ? $this->etl_param['DEBUG'] == '1' : false;
$this->etl_param['debug'] = true;
        $this->etl_param['BATCH_ID'] = (!empty($this->etl_param['BATCH_ID'])) ? $this->etl_param['BATCH_ID'] : batch_id();
        $this->etl_param['FILE_NAME_SEP'] = (!empty($this->etl_param['FILE_NAME_SEP'])) ? $this->etl_param['FILE_NAME_SEP'] : '.-.';
        $this->etl_param['username'] = $_SESSION['user_record']['username'];
        $this->etl_param['DBIM_test'] = (!empty($this->etl_param['TEST'])) ? $this->etl_param['TEST'] : '0';
        $this->etl_param['DBIM_show_file'] = (!empty($this->etl_param['SHOW_FILE'])) ? $this->etl_param['SHOW_FILE'] : '0';
        $this->etl_param['DBIM_error_limit'] = (!empty($this->etl_param['ERROR_LIMIT'])) ? $this->etl_param['ERROR_LIMIT'] : '100';
        // Variables generales
        $this->etl_param['error'] = array();
        $this->etl_param['qty_proc'] = CLX_INIT_ZERO;
        $this->etl_param['qty_err'] = CLX_INIT_ZERO;
        // Parametros de directorios
        $this->etl_param['COMP_CODE'] = (!empty($this->etl_param['COMP_CODE'])) ? trim($this->etl_param['COMP_CODE']) : null;
        $this->etl_param['ETL_IMPORT_PEND_DIR'] = companyParameterDef('ETL_IMPORT_PEND_DIR', null, $this->etl_param['COMP_CODE']);
        $this->etl_param['ETL_IMPORT_REJ_DIR'] = companyParameterDef('ETL_IMPORT_REJ_DIR', null, $this->etl_param['COMP_CODE']);
        $this->etl_param['ETL_IMPORT_PROC_DIR'] = companyParameterDef('ETL_IMPORT_PROC_DIR', null, $this->etl_param['COMP_CODE']);
		//IVAN; AGREGAR PARAMETRO PARA CONFIGURAR ALERTA ETL_001
        $this->etl_param['ETL_ALERT_CODE'] = companyParameterDef('ETL_ALERT_CODE', 'ETL_001', $this->etl_param['COMP_CODE']);
		$this->etl_param['ETL_HOUR_ALERT_FILE_NOT_FOUND'] = companyParameterDef('ETL_HOUR_ALERT_FILE_NOT_FOUND', '00:01', $this->etl_param['COMP_CODE']);
    }

    /**
     * Proceso principal de ETL
     * @param void
     * @return boolean
     */
    function main_process() {
        // Verifica parametros de directorios
        if(is_null($this->etl_param['ETL_IMPORT_PEND_DIR'])) {
            /*
             * GEN_ETL_ERR_010
             * General parameter "^1" not defined
             * Parmetro general "^1" no definido
             */
            $this->etl_param['error'][] = dict('GEN_ETL_ERR_010', 'ETL_IMPORT_PEND_DIR');
        }
        if(is_null($this->etl_param['ETL_IMPORT_REJ_DIR'])) {
            $this->etl_param['error'][] = dict('GEN_ETL_ERR_010', 'ETL_IMPORT_REJ_DIR');
        }
        if(is_null($this->etl_param['ETL_IMPORT_PROC_DIR'])) {
            $this->etl_param['error'][] = dict('GEN_ETL_ERR_010', 'ETL_IMPORT_PROC_DIR');
        }
        if(count($this->etl_param['error'])) {
            // Si falta algun directorio, registra error y termina
            $this->log_error($this->etl_param['error']);
            return false;
        }
        // Lee directorio y registra archivos en la tabla temporal
        $this->dir_process($this->etl_param['ETL_IMPORT_PEND_DIR']);
        // Procesa cada importador registrado
        $this->etl_process();
        // Para los archivos temporales que no encuentra importador, los mueve al directorio de archivos con error
        $this->etl_no_processed_moving();
	////////////////////////////////////////////////////////////////////////////////////////////////////////////
	///////////////////////////////////////////////////////////////////////////////////////////////////////////////	
		// Para procesos sin archivos
        $this->etl_no_found_file();
		
        // Si hubo errores, los reporta
        $this->etl_param['qty_err'] = count($this->etl_param['error']);
        if($this->etl_param['qty_err'] > 0) {
            // Si falta algun directorio, registra error y termina
            $this->log_error($this->etl_param['error']);
        }
        /*
         * GEN_ETL_MSG_010
         * ETL Process finished. ^1 files processed, ^2 errors
         * Proceso ETL terminado. ^1 archivos procesados, ^2 errores
         */
        $this->log_normal(dict('GEN_ETL_MSG_010', $this->etl_param['qty_proc'], $this->etl_param['qty_err']));
        return ($this->etl_param['qty_err'] == CLX_INIT_ZERO);
    }
    
    /**
     * Lee directorio y registra archivos en la tabla temporal
     * @param varchar $dir
     * @return void
     */
    function dir_process($dir) {
        $dir_files = scandir($dir);
        //$this->etl_param['qty_proc'] = CLX_INIT_ZERO;
        foreach($dir_files as $filename) {
            if (in_array($filename,array('.','..'))) { 
                // Marcas de directorios especiales
                continue;
            }
            if (is_dir($dir . DIRECTORY_SEPARATOR . $filename)) { 
                // Es un subdirectorio
                $this->dir_process($dir . DIRECTORY_SEPARATOR . $filename);
                continue;
            }
            // Crea fila en tabla temporal de archivos por procesar [IMFT]
            $filespec_from = $dir . DIRECTORY_SEPARATOR . $filename;
            $filespec_data = pathinfo($filespec_from);
            $IMFT_record = array();
            $IMFT_record['ETL_FILENAME'] = $filename;
            $IMFT_record['ETL_FILEPATH'] = $filespec_data['dirname'];
            $IMFT_record['ETL_FILEDATE'] = date('Y-m-d H:i:s', filemtime($filespec_from));
            $IMFT_record['ETL_FILEEXT'] = $filespec_data['extension'];
            $IMFT_record['ETL_FILEID'] = '';
            $IMFT_record['ETL_FILESEQ'] = CLX_INIT_ZERO;
            $IMFT_record['RUNID'] = $this->etl_param['BATCH_ID'];
            $IMFT_record['CDATE'] = xFormat::getNow('Y-m-d H:i:s');
            $IMFT_id = ClxCommon::$db->createRecord($this->imftname, $IMFT_record, false);
            // Mueve archivo al directorio de pendientes de proceso
            $filespec_dest = $this->etl_param['ETL_IMPORT_PROC_DIR'] . DIRECTORY_SEPARATOR . $IMFT_id . $this->etl_param['FILE_NAME_SEP'] . $IMFT_record['ETL_FILENAME'];
            if(!rename($filespec_from, $filespec_dest)) {
                /*
                 * GEN_ETL_ERR_020
                 * Could not move "^1" to "^2"
                 * No se pudo mover "^1" a "^2"
                 */
                $this->etl_param['error'][] = dict('GEN_ETL_ERR_020', $filespec_from, $filespec_dest) . ' [010]';
                // Elimina el registro de la tabla temporal
                ClxCommon::$db->deleteRecords($this->imftname, 'ID='.$IMFT_id);
            } else {
                // Actualiza directorio actual del archivo, porque lo pudo mover
                $record = array();
                $record['ETL_FILEPATH'] = $this->etl_param['ETL_IMPORT_PROC_DIR'];
if($this->etl_param['debug'] === true) { ClxCommon::$db->debug=true; }
                ClxCommon::$db->updateRecord($this->imftname, $record, 'ID='.$IMFT_id);
if($this->etl_param['debug'] === true) { ClxCommon::$db->debug=false; }
                $this->etl_param['qty_proc']++;
            }
        }
    }
	function etl_no_found_file() {
   
	}
		
    /*
     * Procesa cada importador registrado, comparando los registros 
     * @param void
     * @return void
     */
    function etl_process() {
        $sql_where_extra = '';
        $binds = array();
        $binds['STATUS'] = 'A';
        if(!is_null($this->etl_param['COMP_CODE'])) {
            $binds['COMP_CODE'] = $this->etl_param['COMP_CODE'];
            $sql_where_extra .= ' AND COMP_CODE = :COMP_CODE';
        }
        // Tabla IMCF : $this->imcfname : CLX_ETL_IMPORT_CFG
        $sql = 'SELECT * FROM ' . $this->imcfname . ' WHERE STATUS=:STATUS' . $sql_where_extra;
	    $IMCF_results = ClxCommon::$db->executeQueryUniversal($sql, $binds);
        if($this->etl_param['debug'] === true) { pre($sql); pre($binds); }
        while (($IMCF_record = ClxCommon::$db->getNextRecord($IMCF_results, ASSOCIATIVE))) {
            // Busca el archivo por patron de expresin regular
			
			
	        $sql_where_extra = '';
			$binds = array($IMCF_record);
            $binds['FILENAME_MASK'] = $IMCF_record['FILENAME_MASK'];
            $binds['RUNID'] = $this->etl_param['BATCH_ID'];
            // Tabla IMFT : $this->imftname : CLX_ETL_IMPFILES_TMP
			if (kDatabaseType === 'oci') {
                $sql = 'SELECT * FROM ' . $this->imftname . ' WHERE (ETL_FILESEQ = 0 OR RUNID = :RUNID) AND REGEXP_LIKE (ETL_FILENAME, :FILENAME_MASK, ' . singleQuote('i') . ')' . $sql_where_extra;
            } else {
                $sql = 'SELECT * FROM ' . $this->imftname . ' WHERE (ETL_FILESEQ = 0 OR RUNID = :RUNID) AND ETL_FILENAME REGEXP :FILENAME_MASK' . $sql_where_extra;
            }

            if($this->etl_param['debug'] === true) { pre($sql); pre($binds); }
            $IMFT_results = ClxCommon::$db->executeQueryUniversal($sql, $binds);
            // Si la encuentra, hace entrada en la cola de procesamiento
            while (($IMFT_record = ClxCommon::$db->getNextRecord($IMFT_results, ASSOCIATIVE))) {
			
			/*	        * ['R1'] => Recurrence Type General Type of recurrence
         * ['R2'] => Day Value
         * ['R3'] => Day Type  if r2=5 semanal  if r2=numero que coloques dias xxx and r3=1 diario es
         * ['R4'] => Every week repetitions  if r4=numero que coloques semanas and r7 es el dia de la semana  r7=1 lunes domingo 7 
         * ['R5'] => Every Month r5    es mes
         * ['R6'] => Month type
         * ['R7'] => Week days for month            r1=MNTH
         * ['R8'] => Every Year                     r1 ="YEAR"  and r8  lleno en annios
         * ['R9'] => End date                         fecha 
		 
		 solo 1 una vez      r1=ONE       r2 r4 r6 r8 =0
		 
		 recurmode=ONE MUL NEV
		*/
				//IVAN: VALIDACIONES VAN AQUI
				$error = FALSE;
				if (!empty($IMCF_record['RECURRENT_R9']) && $IMCF_record['RECURRENT_R9']<date("Y-m-d")) {
					$this->etl_param['error'][] = dict('GEN_ERR_ETL', dict('GEN_ERR_ETL_EXPIRE_DATE'),  $IMFT_record['ETL_FILENAME'], $IMCF_record['IMPORT_CODE']) . ' [020]';
					sendAlert($this->etl_param['ETL_ALERT_CODE']);
					$error=true;
				}
				if ( file_exists($IMFT_record['ETL_FILEPATH']."/".$IMFT_record['ETL_FILENAME']) && filesize($IMFT_record['ETL_FILEPATH']."/".$IMFT_record['ETL_FILENAME'])==0){
					$this->etl_param['error'][] = dict('GEN_ERR_ETL', dict('GEN_ERR_ETL_SIZE_FILE'),  $filespec_from, $IMCF_record['IMPORT_CODE']) . ' [021]';
					sendAlert($this->etl_param['ETL_ALERT_CODE']);
					$error=true;	
				}
				if (!empty($IMCF_record['NEXT_RUN_DATI']) && substr($IMCF_record['NEXT_RUN_DATI'],0,10)<>date("Y-m-d")) {
					$this->etl_param['error'][] = dict('GEN_ERR_ETL', dict('GEN_ERR_ETL_FILE_NOT_DATE'),  $IMFT_record['ETL_FILENAME'], $IMCF_record['IMPORT_CODE']) . ' [022]';
					sendAlert($this->etl_param['ETL_ALERT_CODE']);
					$error=true;
				}
				if ($IMCF_record['RECURMODE']=='ONE'){
					if(ClxCommon::$db->getOne('SELECT count(*) FROM CLX_ETL_OP_QUEUE WHERE IMPORT_CODE='.singleQuote($IMCF_record['IMPORT_CODE']).' and CLX_EFFTDATE='.singleQuote(date("Y-m-d")))>0)
						$this->etl_param['error'][] = dict('GEN_ERR_ETL', dict('GEN_ERR_ETL_TWO_FILE_PROCESS'),  $filespec_from, $IMCF_record['IMPORT_CODE']) . ' [023]';
					sendAlert($this->etl_param['ETL_ALERT_CODE']);
					$error=true;	
				}
				
				
				if($error==true) {
					continue;
				}
                // Tabla IMOQ : $this->imoqname : CLX_ETL_OP_QUEUE
/*
                $IMOQ_record = $IMCF_record;
                unset($IMOQ_record['ID']);
                unset($IMOQ_record['FILENAME_MASK']);
                unset($IMOQ_record['CLX_EFFTDATE']);
                unset($IMOQ_record['CLX_AUTH_STATUS']); */
                $IMOQ_record['FILE_NAME'] = $IMFT_record['ETL_FILENAME'];
                unset($IMOQ_record['CLX_AUTH_STATUS']);
                if($this->etl_param['debug'] === true) { pre('$IMCF_record'); pre($IMCF_record); }
                //
                $imoq_filename = $IMFT_record['ID'] . $this->etl_param['FILE_NAME_SEP'] . $IMFT_record['ETL_FILENAME'];
                $IMOQ_record['FILE_NAME'] = $imoq_filename;
                //$IMOQ_record['FILE_NAME'] = $IMFT_record['ETL_FILENAME'];
                $IMOQ_record['FILEDATE'] = $IMFT_record['ETL_FILEDATE'];
                $IMOQ_record['CLX_EFFTDATE'] = $IMFT_record['ETL_FILEDATE'];
                $IMOQ_record['FILE_EXT'] = $IMFT_record['ETL_FILEEXT'];
                $IMOQ_record['IMFT_ID'] = $IMFT_record['ID'];
                $IMOQ_record['CDATE'] = xFormat::getNow('Y-m-d H:i:s');
                $IMOQ_record['MDATE'] = xFormat::getNow('Y-m-d H:i:s');
                $IMOQ_record['CREATEDBY'] = $this->etl_param['username'];
                $IMOQ_record['MODIFIEDBY'] = $this->etl_param['username'];
                $IMOQ_record['REC_STATUS'] = 'IDLE';
                $IMOQ_record['DBIM_ID'] = CLX_INIT_ZERO;
				$IMOQ_record['COMP_CODE']=$IMFT_record['COMP_CODE'];
				$IMOQ_record['IMPORT_CODE']=$IMCF_record['IMPORT_CODE'];
				$IMOQ_record['RECURRENT_R2']=$IMCF_record['RECURRENT_R2'];
				$IMOQ_record['RECURRENT_R3']=$IMCF_record['RECURRENT_R3'];
				$IMOQ_record['RECURRENT_R4']=$IMCF_record['RECURRENT_R4'];
				$IMOQ_record['RECURRENT_R5']=$IMCF_record['RECURRENT_R5'];
				$IMOQ_record['RECURRENT_R6']=$IMCF_record['RECURRENT_R6'];
				$IMOQ_record['RECURRENT_R7']=$IMCF_record['RECURRENT_R7'];
				$IMOQ_record['RECURRENT_R8']=$IMCF_record['RECURRENT_R8'];
				$IMOQ_record['RECURRENT_R9']=$IMCF_record['RECURRENT_R9'];
				$IMOQ_record['RECURRENT_R0']=$IMCF_record['RECURRENT_R0'];
				$IMOQ_record['RECURRENT_R1']=CLX_CHAR_EMPTY_STRING;
				$IMOQ_record['RECURMODE']=$IMCF_record['RECURMODE'];
				$IMOQ_record['NEXT_RUN_DATI']=$IMCF_record['NEXT_RUN_DATI'];
                $IMOQ_id = ClxCommon::$db->createRecord($this->imoqname, $IMOQ_record, false);
                // Marca registro temporal como asignado a cola de procesamiento
                $record = array();
                $record['ETL_FILESEQ'] = $IMOQ_id;
if($this->etl_param['debug'] === true) { ClxCommon::$db->debug=true; }
                ClxCommon::$db->updateRecord($this->imftname, $record, 'ID='.$IMFT_record['ID']);
if($this->etl_param['debug'] === true) { ClxCommon::$db->debug=false; }
            }
        }
    }
	
    /*
     * Para los archivos temporales que no encuentra importador, los mueve al directorio de archivos con error
     * @param void
     * @return void
     */
    function etl_no_processed_moving() {
        $binds = array();
        $binds['RUNID'] = $this->etl_param['BATCH_ID'];
        $binds['CDATE'] = xFormat::getNow('Y-m-d');
        // Busca los procesados en esta sesin o lo que estn en el aire desde ayer hacia atras...
        $sql = 'SELECT * FROM ' . $this->imftname . ' WHERE ETL_FILESEQ = 0 AND (RUNID = :RUNID OR CDATE < :CDATE)';
        if (kDatabaseType === 'oci') {
            $sql = str_replace(':CDATE', ClxCommon::$db->db_to_date(':CDATE'), $subject);
        }
        $IMFT_results = ClxCommon::$db->executeQueryUniversal($sql, $binds);
        if($this->etl_param['debug'] === true) { pre($sql); pre($binds); }
        while (($IMFT_record = ClxCommon::$db->getNextRecord($IMFT_results, ASSOCIATIVE))) {
            $filespec_from = $this->etl_param['ETL_IMPORT_PROC_DIR'] . DIRECTORY_SEPARATOR . $IMFT_record['ID'] . $this->etl_param['FILE_NAME_SEP'] . $IMFT_record['ETL_FILENAME'];
            $filespec_dest = $this->etl_param['ETL_IMPORT_REJ_DIR'] . DIRECTORY_SEPARATOR . $IMFT_record['ID'] . $this->etl_param['FILE_NAME_SEP'] . $IMFT_record['ETL_FILENAME'];
            $op_err = false;
            if(file_exists($filespec_from)) {
                if(!file_exists($filespec_dest)) {
                    // Mueve archivo al directorio de archivos rechazados
                    if(!rename($filespec_from, $filespec_dest)) {
                        // No lo pudo mover
                        $this->etl_param['error'][] = dict('GEN_ETL_ERR_020', $filespec_from, $filespec_dest) . ' [020]';
                        $op_err = true;
                    }
                } else {
                    // Inconsistencia: existe en el origen y en el destino tambien
                    $this->etl_param['error'][] = dict('GEN_ETL_ERR_020', $filespec_from, $filespec_dest) . ' [025-EXIST-DEST]';
                    $op_err = true;
                }
            } else {
                if(!file_exists($filespec_dest)) {
                    // Inconsistencia: No existe en el origen ni en el destino
                    $this->etl_param['error'][] = dict('GEN_ETL_ERR_020', $filespec_from, $filespec_dest) . ' [027-NO-EXIST-ORIG]';
                    $op_err = true;
                }
                // SI EXISTE EL DESTINO, ES QUE LO PROCESO, LO MOVIO PERO NO MARCO EL REGISTRO... ASI QUE LO MARCA...
            }
            if($op_err === false) {
                // Actualiza directorio actual del archivo, porque lo pudo mover
                $record = array();
                $record['ETL_FILEPATH'] = $this->etl_param['ETL_IMPORT_REJ_DIR'];
                $record['ETL_FILESEQ'] = 'null';
if($this->etl_param['debug'] === true) { ClxCommon::$db->debug=true; }
                ClxCommon::$db->updateRecord($this->imftname, $record, 'ID='.$IMFT_record['ID']);
if($this->etl_param['debug'] === true) { ClxCommon::$db->debug=false; }
            }
        }
    }
    
    /*
     * Procesa cola de operaciones de ETL, buscando los registros pendientes de procesar y verificando los que hayan termina de procesarse.
     * @param void
     * @return void
     */
    function imoq_process() {
        $this->imoq_sched_imports();
        $this->imoq_ver_ended_imps();
    }
    
    /*
     * Cola de operaciones de ETL: manda a ejecutar en batch los Imports que estn agendados y pendientes por ejecutar (IDLE)
     * @param void
     * @return void
     */
    function imoq_sched_imports() {
        $DBIX = &getTool('DBIX');
        $sql_where_extra = '';
        $binds = array();
        $binds['STATUS'] = 'A';
        $binds['REC_STATUS'] = 'IDLE';
        if(!is_null($this->etl_param['COMP_CODE'])) {
            $binds['COMP_CODE'] = $this->etl_param['COMP_CODE'];
            $sql_where_extra .= ' AND COMP_CODE = :COMP_CODE';
        }
        $sql = 'SELECT * FROM ' . $this->imoqname . ' WHERE STATUS=:STATUS AND REC_STATUS = :REC_STATUS' . $sql_where_extra . ' ORDER BY MDATE ASC';
        $IMOQ_results = ClxCommon::$db->executeQueryUniversal($sql, $binds);
        if($this->etl_param['debug'] === true) { pre($sql); pre($binds); }
        while (($IMOQ_record = ClxCommon::$db->getNextRecord($IMOQ_results, ASSOCIATIVE))) {
            $IMOQ_id = $IMOQ_record['ID'];
            // FileSpec (directorio + nombre) del archivo de entrada
            //$original_filename = $IMOQ_record['IMFT_ID'] . $this->etl_param['FILE_NAME_SEP'] . $IMOQ_record['FILE_NAME'];
            $original_filename = $IMOQ_record['FILE_NAME'];
            $filespec_source = $this->etl_param['ETL_IMPORT_PROC_DIR'] . DIRECTORY_SEPARATOR . $original_filename;
//            $filespec_dest = kPathToWebSite . 'import' . DIRECTORY_SEPARATOR . $original_filename;
            $line_count = CLX_INIT_ZERO;
            // Copia archivo al directorio de importacion
/*
            if(!copy($filespec_source, $filespec_dest)) {
                // Error en copia
                /*
                 * GEN_ETL_ERR_030
                 * Could not copy "^1" to "^2"
                 * No se pudo copiar "^1" a "^2"
                 * /
                $this->etl_param['error'][] = dict('GEN_ETL_ERR_030', $filespec_from, $filespec_dest) . ' [030]';
                continue;
            }
*/
            // asigna valores a ser serializados y pasados en la columna BATCH_PRMT
            $dbim_files = array();
            $dbim_files['DBIM_id'] = ClxCommon::$db->getField('CLX_DB_IMPORTS', 'ID', 'IDCODE='.singleQuote($IMOQ_record['IMPORT_CODE']));
            $dbim_files['DBIM_test'] = $this->etl_param['DBIM_test'];
            $dbim_files['DBIM_show_file'] = $this->etl_param['DBIM_show_file'];
            $dbim_files['DBIM_error_limit'] = $this->etl_param['DBIM_error_limit'];
            $dbim_files['DBIM_description'] = '[IMOQ-' . $IMOQ_record['ID'] . ']';
            $dbim_files['$_FILES'] = array();
            $dbim_files['$_FILES']['DBIM_file'] = array();
            $dbim_files['$_FILES']['DBIM_file']['name'] = $original_filename;
            $dbim_files['$_FILES']['DBIM_file']['tmp_name'] = $filespec_source;
			$dbim_files['DBIM_test'] = $this->etl_param['DBIM_test'];
            // Ejecuta importador en batch
            $DBIX_rec = $DBIX->defaultTableRecord();
            unset($DBIX_rec['ID']);
            $DBIX_rec['IMPORTCODE'] = $IMOQ_record['IMPORT_CODE'];
            $DBIX_rec['RECORD_QTY'] = $line_count;
            $DBIX_rec['CDATE'] = xFormat::getNow('Y-m-d H:i:s');
            $DBIX_rec['OWNERCODE'] = $this->etl_param['username'];
            $DBIX_rec['DESCRIPTION'] = $dbim_files['DBIM_description'];
            $DBIX_rec['FILENAME'] = $original_filename;
            $DBIX_rec['RUN_BATCH'] = CLX_NUM_ONE;
            $DBIX_rec['BATCH_STATUS'] = 'IDLE';
            $DBIX_rec['BATCH_PID'] = CLX_INIT_ZERO;
            $DBIX_rec['BATCH_SDATE'] = null;
            $DBIX_rec['BATCH_EDATE'] = null;
            $DBIX_rec['BATCH_PRMT'] = serialize($dbim_files);
            // Verifica si ejecucin en batch del importador ya esta agendada
            if (ClxCommon::$db->oneExistsRecords($DBIX->tablename, 'FILENAME = ' . singleQuote($original_filename) . '  AND BATCH_STATUS =' . singleQuote('IDLE') . ' AND RUN_BATCH = 1') == CLX_NUM_ZERO) {
                // Agenda ejecucin en batch del importador
                $DBIX_id = ClxCommon::$db->createRecord($DBIX->tablename, $DBIX_rec, false);
                // Marca registro IMOQ como "EN PROCESO"
                $record = array();
                $record['REC_STATUS'] = 'RUN';
                $record['DBIM_ID'] = $DBIX_id;
                ClxCommon::$db->updateRecord($this->imoqname, $record, 'ID='.$IMOQ_id);
            } else {
                // Ya se est procesando ????? >> INCONSISTENCIA...
                /*
                 * GEN_ERR_CONSISTENCY
                 * Consistency Error: ^1
                 * Error de Consistencia: ^1
                 */
                $this->etl_param['error'][] = dict('GEN_ERR_CONSISTENCY', '[IMOQ-040-IMP-REPRO-NOT-ALLOW] ' . $original_filename);
            }
        }
    }

    /*
     * Cola de operaciones de ETL: busca registros en ejecucin "RUN" cuyos importadores hayan terminado de ejecutarse
     * @param void
     * @return void
     */
    function imoq_ver_ended_imps() {
        $DBIX = &getTool('DBIX');
        $sql_where_extra = '';
        $binds = array();
        $binds['STATUS'] = 'A';
        $binds['REC_STATUS'] = 'RUN';
        if(!is_null($this->etl_param['COMP_CODE'])) {
            $binds['COMP_CODE'] = $this->etl_param['COMP_CODE'];
            $sql_where_extra .= ' AND COMP_CODE = :COMP_CODE';
        }
        $sql = 'SELECT * FROM ' . $this->imoqname . ' WHERE STATUS=:STATUS AND REC_STATUS = :REC_STATUS' . $sql_where_extra . ' ORDER BY MDATE ASC';
        if($this->etl_param['debug'] === true) { pre($sql); pre($binds); }
        $IMOQ_results = ClxCommon::$db->executeQueryUniversal($sql, $binds);
        while (($IMOQ_record = ClxCommon::$db->getNextRecord($IMOQ_results, ASSOCIATIVE))) {
            if (ClxCommon::$db->oneExistsRecords($DBIX->tablename, 'ID = ' . singleQuote($IMOQ_record['DBIM_ID']) . '  AND BATCH_STATUS =' . singleQuote('END') . ' AND RUN_BATCH = 1') == CLX_NUM_ZERO) {
                // No ha terminado de ejecutarse
                continue;
            }
            // Si lo consigue, ya termin de ejecutarse... Marca registro IMOQ como procesado
            $IMOQ_id = $IMOQ_record['ID'];
            $record = array();
            $record['REC_STATUS'] = 'END';
            ClxCommon::$db->updateRecord($this->imoqname, $record, 'ID='.$IMOQ_id);
        }
    }

    /*
     * Registra mensaje de error en el Log de operaciones
     * @param varchar/array $msg
     * @return void
     */
    function log_error($msg) {
        if(!is_array($msg)) {
            $msg = array($msg);
        }
        foreach($msg as $key => $val) {
            $msg[$key] = 'ERROR-'.$val;
        }
        $this->log_general($msg);
    }

    /*
     * Registra mensaje en el Log de operaciones
     * @param varchar/array $msg
     * @return void
     */
    function log_normal($msg) {
        $this->log_general($msg);
    }
    
    /*
     * Registra mensaje en el Log de operaciones
     * @param varchar/array $msg
     * @return void
     */
    function log_general($msg) {
        if(!is_array($msg)) {
            $msg = array($msg);
        }
        foreach($msg as $msg_val) {
            echo CLX_CHAR_NL . '<br>'.$msg_val;
        }
    }
}