<?php

/**
 * General Code File
 * For more information contact  support@alenet.com
 *
 * @author       $Author: cramirez $
 * @copyright    Alenet, Inc.
 * @since        $Date: 2018-08-23 18:18:58 -0400 (Thu, 23 Aug 2018) $
 * @version        $Rev: 9902 $
 *
 * CHANGE LOG
 *  Included svn prop
 */
class fidu_flexformula {

    var $code = 'FIFF';
    var $version = '1.0.0.0';
    var $flexcode;  //Formula Flex Code
    var $datestamp;
    var $data;  // Source Record
    var $ficoname;  // Contracts
    var $fiatname;  // Contract Transactions
    var $fiffname; // Flex Formula Table
    var $fifename; // Flex Formula Elements
    var $results; // Result Accumulators
    var $elements; // Elements
    var $contents; // Contents Variables
    var $contentTable = false; //Table of contents
    var $debug = false;

    /**
     * Constructor
     */
    function __construct() {
        $this->flexcode = '';
        $this->datestamp = date('Y-m-d H:i:s');
        $this->ficoname = tableName('FICO');
        $this->fiatname = tableName('FIAT');
        $this->fiffname = tableName('FIFF');
        $this->fifename = tableName('FIFE');
        $this->contents = '';
        $this->record = array();
        $this->results = array();
        $this->accumulators = array();
        $this->elements = array();
    }

// End contructor
    /**
     * Add Contents value
     */
    function addContent($msg, $class = CLX_CLASS_DATA) {
        if (empty($this->contentTable)) {
            //Log table
            $this->contentTable = new htmlTable(CLX_WIDTH_ONE);
            $this->contentTable->setProps(array(CLX_PROP_CLASS => CLX_CLASS_FORM, CLX_PROP_ALIGN => 'center'));
        }
        $this->contentTable->addcell($msg, array(CLX_PROP_CLASS => $class));
    }

// End method
    /**
     * Get Contents value
     */
    function getContent($msg = array(), $class = CLX_CLASS_DATA) {
        return $this->contentTable->source();
    }

// End method
    /**
     * Init Accumulators
     */
    function init($flexcode, $record) {
        $this->flexcode = $flexcode;
        $this->results[$flexcode] = CLX_INIT_ZERO;
        $this->record = $record;
        if (array_key_exists('TRANS_CODE', $record)) {
            $transrec = ClxCommon::$db->getRecord($this->fiatname, 'IDCODE = ' . singleQuote($record['TRANS_CODE']));
            foreach ($transrec as $key => $val) {
                if (!array_key_exists($key, $this->record)) {
                    $this->record[$key] = $val;
                }
            }
        }

        //Prepare Formula Elements
        $sql = 'SELECT FIFE.* FROM ' . $this->fiffname . ' FIFF, ' . $this->fifename . ' FIFE WHERE FIFF.IDCODE = FIFE.FIDUFORMULA_CODE AND FIFF.IDCODE = ' . singleQuote($flexcode) . ' ORDER BY FIFE.SEQNUM ASC';
        $results = ClxCommon::$db->executeQuery($sql);
        while ($result = ClxCommon::$db->getNextRecord($results, ASSOCIATIVE)) {
            //Init Results
            $this->elements[] = $result;
            $this->results[$result['IDNAME']] = CLX_INIT_ZERO;
        }
    }

    /**
     * Start Calculations
     */
    function start() {

        $cal = new ClxExpression();

	foreach ($this->elements as $key => $val) {
            if (!empty($this->debug)) {
                $this->addContent(dict('TOOL_CODE_FIFF_NEXTELEMENT^1', $val['SEQNUM'] . '. ' . $val['IDNAME']), CLX_CLASS_SUBTITLE);
            }


	    if ($val['LOGICAL_OPERATOR'] <> '') {
                if (!empty($this->debug)) {
                    $this->addContent(dict('TOOL_CODE_LOGICALTSTARTED'));
                }
                $logfail = false;
                $left = '';
                if ($val['LOGIC_FIELD1'] <> '') {
                    $left = $this->record[$val['LOGIC_FIELD1']];
                } elseif ($val['LOGIC_VALUE1'] <> '') {
                    $left = $val['LOGIC_VALUE1'];
                } else {
                    $logfail = true;
                }
                if ($val['LOGIC_FIELD2'] <> '')
                    $right = $this->record[$val['LOGIC_FIELD2']];
                elseif ($val['LOGIC_VALUE2'] <> '')
                    $right = $val['LOGIC_VALUE2'];
                else
                    $logfail = true;


                if (empty($logfail)) {
		    switch ($val['LOGICAL_OPERATOR']) {
			case '<':
			    $check =  $left < $right;
			    $checkt = " $left < $right";
			    break;
			case '>':
			    $check =  $left > $right;
			    $checkt = " $left > $right";
			    break;
			case '>=':
			    $check =  $left >= $right;
			    $checkt = " $left >= $right";
			    break;
			case '<=':
			    $check =  $left <= $right;
			    $checkt = " $left <= $right";
			    break;
			case '<>':
			case '!=':
			    $check =  $left <> $right;
			    $checkt = " $left <> $right";
			    break;
			case '==':
			    $check =  $left == $right;
			    $checkt = " $left == $right";
			    break;
			default:
			    break;
			}
                    if (!empty($this->debug))
                        $this->addContent(dict('TOOL_CODE_FIFF_LOGTEST', $checkt));
                    $cal->parseAndCalculate($check);
                    if (!empty($this->debug)) {
                        if (!empty($check)) {
                            $this->addContent(dict('TOOL_CODE_FIFF_LOGTESTRESOK'));
                        } else {
                            $this->addContent(dict('TOOL_CODE_FIFF_LOGTESTRESFAIL'));
                            $this->addContent(CLX_CHAR_HTML_NBSP, 'caption');
                        }
                    }
                    unset($right, $left, $logfail);
		    if (!$check)
                        continue;
                }else {
                    if (!empty($this->debug))
                        $this->addContent(dict('TOOL_CODE_FIFF_LOGFAILNOCOND'));
                    unset($right, $left, $logfail);
                    continue;
                }
                unset($right, $left, $logfail);
            }else {
                if (!empty($this->debug))
                    $this->addContent(dict('TOOL_CODE_FIFF_NOLOGTEST'));
            }
            //Perform Calculation
            $op = $this->operator($val['MATH_OPERATOR']);
            $op_bc = $this->operator($val['MATH_OPERATOR'], true);
            if (!empty($this->debug))
                $this->addContent(dict('TOOL_CODE_FIFF_MATHOP^1', $val['MATH_OPERATOR']));
            $eq = (string) '';
            $eq_bc = (string) '';

            if ($val['ARGUMENT_VALUE'] <> CLX_NUM_ZERO) {
                $eq = $val['ARGUMENT_VALUE'];
                $eq_bc = $val['ARGUMENT_VALUE'];
            }
            if (array_key_exists($val['FORMULAFIELD_1'], $this->record) && strlen($this->record[$val['FORMULAFIELD_1']]) > CLX_NUM_ZERO) {
                if(strlen($eq)>0) $eq = $this->record[$val['FORMULAFIELD_1']].$op.$eq;
		else $eq = $eq.$this->record[$val['FORMULAFIELD_1']];

		//Using bcmath lib
                if (strlen($eq_bc) > CLX_NUM_ZERO) {
		    $eq_bc = $this->formula_calculo($op_bc, trim($this->record[$val['FORMULAFIELD_1']]) ,  $eq_bc );
		}
                else
                    $eq_bc = $this->record[$val['FORMULAFIELD_1']];
            }

            if (array_key_exists($val['FORMULAFIELD_2'], $this->record) && strlen($this->record[$val['FORMULAFIELD_2']]) > CLX_NUM_ZERO) {
		/*if(strlen($eq)>0) $eq = $this->accumulators[$val['FLEXACCU_CODE']].$op.$eq;
		else $eq = $eq.$this->accumulators[$val['FLEXACCU_CODE']];*/

		if (strlen($eq_bc) > CLX_NUM_ZERO)
                    //$eq_bc = $op_bc . '(' . singleQuote($this->record[$val['FORMULAFIELD_2']]) . ',' . $eq_bc . ')';
		    $eq_bc = $this->formula_calculo($op_bc, trim($this->record[$val['FORMULAFIELD_2']]) ,  $eq_bc );
                else
                    $eq_bc = $this->record[$val['FORMULAFIELD_2']];
            }
            //if(array_key_exists($val['FORMELMENT_NAME'], $this->record) && strlen($this->results[$val['FORMELMENT_NAME']]) > 0){
            //print_r($this->results);
            if (strlen($val['FORMELMENT_NAME']) > CLX_NUM_ZERO && strlen($this->results[$val['FORMELMENT_NAME']]) > CLX_NUM_ZERO) {
		if(strlen($eq)>0) $eq = $this->results[$val['FORMELMENT_NAME']].$op.$eq;
		else $eq = $this->results[$val['FORMELMENT_NAME']];


		if (strlen($eq_bc) > CLX_NUM_ZERO)
                    //$eq_bc = $op_bc . '(' . singleQuote($this->results[$val['FORMELMENT_NAME']]) . ',' . $eq_bc . ')';
		    $eq_bc = $this->formula_calculo($op_bc, $this->results[$val['FORMELMENT_NAME']] ,  $eq_bc );
                else
                    $eq_bc = $this->results[$val['FORMELMENT_NAME']];
            }
            //if(array_key_exists($val['FORMELMENT_NAME2'], $this->record) && strlen($this->results[$val['FORMELMENT_NAME2']]) > 0){
            if (strlen($val['FORMELMENT_NAME2']) > CLX_NUM_ZERO && strlen($this->results[$val['FORMELMENT_NAME2']]) > CLX_NUM_ZERO) {
                if(strlen($eq)>0) $eq = $this->results[$val['FORMELMENT_NAME2']].$op.$eq;
		else $eq = $this->results[$val['FORMELMENT_NAME2']];

		if (strlen($eq_bc) > CLX_NUM_ZERO)
                  //  $eq_bc = $op_bc . '(' . singleQuote($this->results[$val['FORMELMENT_NAME2']]) . ',' . $eq_bc . ')';
		      $eq_bc = $this->formula_calculo($op_bc, trim($this->results[$val['FORMELMENT_NAME2']]) ,  $eq_bc );
                else
                    $eq_bc = $this->results[$val['FORMELMENT_NAME2']];
            }
            if ($val['MATH_OPERATOR'] === '%') {
		$eq = '('.$eq.')/100';
               // $eq_bc = 'bcdiv(' . $eq_bc . ',' . singleQuote(CLX_NUM_HUNDRED) . ')';
		$eq_bc = $this->formula_calculo('bcdiv', $eq_bc  ,  CLX_NUM_HUNDRED );
            }
            if ($val['MATH_OPERATOR'] === 'R') {
                $eq = explode(',', $eq);
         //       $eq_bc = 'round(' . $eq['0'] . ',' . rounder($eq[CLX_NUM_ONE], CLX_NUM_ZERO) . ')';
		$eq_bc = $this->formula_calculo('round', $eq[CLX_INIT_ZERO]  ,  $eq_bc[1]);
		$eq = $eq_bc;
            }
	    if($this->debug){
		$this->addContent(dict('TOOL_CODE_FIFF_ELEMOPERA^1',$eq));
	    }

	    $eq = $eq_bc;
	    if($this->debug){
		$this->addContent(dict('TOOL_CODE_FIFF_ELEMRESULT^1',$eq), "subtitle");
		$this->addContent("&nbsp;", "caption");
	    }

	    $this->results[$val['IDNAME']] = $eq;
            $this->results[$this->flexcode] = $this->results[$val['IDNAME']];
        }
    }
    function formula_calculo($function_operador,$valor1,$valor2) {
	$_return='0';
	switch ($function_operador){
	    case 'bcmul':
		$_return=bcmul($valor1,$valor2);
		break;
	    case 'bcdiv':
		$_return=bcdiv($valor1,$valor2);
		break;
	    case 'bcadd':
		$_return=bcadd($valor1,$valor2);
		break;
	    case 'bcdiv':
		$_return=bcdiv($valor1,$valor2);
		break;
	    case 'bcsub':
		$_return=bcsub($valor1,$valor2);
		break;
	    case 'bcpow':
		$_return=bcpow($valor1,$valor2);
		break;
	    case 'round':
		$_return=round( $valor1, rounder($valor2, CLX_NUM_ZERO) );
		break;
	    default:
		break;
	}
	return $_return;

    }
    function result() {
        return $this->results[$this->flexcode];
    }

    /**
     * Return logical operator
     */
    function operator($operator, $bcmath = false) {
        $return = '';
        switch ($operator) {
            case '+':
                $return = ($bcmath) ? 'bcadd' : '+';
                break;
            case '-':
                $return = ($bcmath) ? 'bcsub' : '-';
                break;
            case '*':
                $return = ($bcmath) ? 'bcmul' : '*';
                break;
            case '/':
                $return = ($bcmath) ? 'bcdiv' : '/';
                break;
            case '%':
                $return = ($bcmath) ? 'bcmul' : '*';
                break;
            case 'R':
                $return = ',';
                break;
            default:
                $return = '';
                break;
        }// End switch
        return $return;
    }

    /**
     * Performs a test calculation
     *
     * @return string;
     */
    function test() {

        $this->debug = true;
        $this->addContent(dict('TOOL_CODE_FIFF_TITLETEST'), CLX_CLASS_TITLE);
        $fiff = UserRequest::$request['idlist'];
        $fiffrec = ClxCommon::$db->getRecord($this->fiffname, 'ID = ' . singleQuote($fiff));
        $this->addContent(dict('TOOL_CODE_FIFF_FORMULACODE^1', $fiffrec['IDNAME'] . ' (' . $fiffrec['IDCODE'] . ')'));
        $this->addContent(dict('TOOL_CODE_FIFF_TESTPARAMETERS^1', $fiffrec['SAMPLE_DATA']));
        $this->addContent(CLX_CHAR_HTML_NBSP, 'caption');
        $record = array();
        $flexcode = $fiffrec['IDCODE'];
        $sp = trim($fiffrec['SAMPLE_DATA']);
        if ($sp <> '') {
            $sp = str_replace(' ', '', $sp);
            $sp = str_replace(CLX_CHAR_NL, '', $sp);
            $sp = str_replace(CLX_CHAR_CR, '', $sp);
            $sp = str_replace('!', '', $sp);
            $a = explode(',', $sp);
            foreach ($a as $val) {
                $b = explode('=', $val);
                if (!array_key_exists(CLX_NUM_ONE, $b)) {
                    $this->addContent(smsg('MSG-214', ''), CLX_CLASS_TITLE);
                    return $this->getContent();
                }
                $record[$b[CLX_NUM_ZERO]] = $b[CLX_NUM_ONE];
            }
            unset($b, $a);
        }
        $this->init($flexcode, $record);
        $this->start();
        $this->addContent(dict('TOOL_CODE_FIFF_TOTAL^1', $this->result()), CLX_CLASS_TITLE);
        return $this->getContent();
    }

    /**
     * Flex formula
     */
    function calculate($flexcode, $record) {
        $this->init($flexcode, $record);
        $this->start();
        return $this->result();
    }

    /**
     * Check if formula exists
     */
    function formulaExists($flexcode) {
        $count = ClxCommon::$db->oneExistsRecords($this->fiffname, 'IDCODE = ' . singleQuote($flexcode));
        return $count > CLX_NUM_ZERO;
    }

}
