<?php

/**
 * General Code File
 * For more information contact  support@alenet.com
 *
 * @author       $Author: jclerma $
 * @copyright    Alenet, Inc.
 * @since        $Date: 2018-02-18 13:23:58 -0500 (Sun, 18 Feb 2018) $
 * @version        $Rev: 9729 $
 *
 * CHANGE LOG
 *  Included svn prop
 */
if (!class_exists('fidu_settlement')) {

    /**
     * Settlement process
     */
    class fidu_settlement {

        var $rounding = CLX_NUM_FIVE;
        var $debug = false;
        var $trace = false;
        var $batch_id = '';
        var $je_source = '';
        var $contents = ''; //content to be dump
        var $seqnum = CLX_INIT_ZERO; //Sequence number for transaction lines
        var $start_time = '';
        var $efftdate;
        var $lggroup_code;
        var $fiscal_period;
        var $process_parameters = array();

        /**
         * Constructor
         */
        function __construct() {
            if (!empty($this->debug))
                echolog('Process Status Initiated');
            $this->now = date('Y-m-d H:i:s');
            $this->username = $_SESSION['user_record']['username'];
            $this->today = date('Y-m-d');
            $this->modulecode = 'FIDU'; // Module
            $this->username = $_SESSION['user_record']['username'];
            $this->batch_id = uniqid();
            $this->start_time = microtime_float();
            $this->je_source = '';
            multiTableName($this, array('GLCP', 'FTMP', 'JTMP', 'ITMP', 'FICO', 'FIAI', 'TTPA', 'INDT', 'FIAF', 'FIGN', 'FIAL', 'FIPA', 'FIRT', 'FIAC', 'PTMP', 'IAT3', 'IAT4', 'FIAT', 'GLAC', 'INPR', 'IETB', 'FIBK', 'FITB', 'INFV', 'INDV', 'PRAT', 'COMP', 'ACCO', 'FIGA', 'JEHD', 'JELN', 'FIAC', 'JCBV', 'PROD', 'SETV', 'FIGN', 'INPR', 'PRAT', 'INFA', 'INFU', 'ITMP', 'PTMP', 'FIAS', 'PRAT', 'JABV', 'GDVW'));
        }

        /**
         * Sets the default parameters for this process
         */
        function set_comp_parameters($comp_code) {
            if (!class_exists('Comp'))
                includeCodeLib('comp');
            $comp = new Comp();
            $rs = $comp->comp_operation_parameters($comp_code);
            $this->operation_date = $rs['OPERATION_DATE'];
            $this->lggroup_code = $rs['LGGROUP_CODE'];
            $this->efftdate = adodb_next_day($this->operation_date);
            $this->fiscal_period = $comp->comp_fiscal_period($comp_code, $this->efftdate);
        }

        /**
         * Send exceptions to process schedule
         */
        function report_exceptions($msg) {
            $PRPR = getTool('PRPR');
            $PRPR->report_exception($this->process_parameters['SCHEDULECODE'], $this->process_parameters['START_TIME'], $this->process_parameters['RUNID'], $msg, 'FIPA', 'ERRO');
        }

        // Settlement Transaction Creation
        /**
         * Starts settlement process.
         *
         * @param array $parameters
         */
        function start($parameters) {
            UserResponse::setIgnoreUserAbort(CLX_NUM_ONE);
            UserResponse::setRunTimeLimit(CLX_NUM_24 * CLX_NUM_60 * CLX_NUM_60);
            $this->process_parameters = $parameters;
            $this->je_source = 'SETT';
            if (!empty($parameters['RUNID'])) {
                $this->batch_id = $parameters['RUNID'];
            }
            echoLog('Settlement process started: ' . $this->je_source);
            echoLog('Batch id: ' . $this->batch_id);
            echoLog('Company: ' . $parameters['COMP_CODE']);
            if (empty($parameters['COMP_CODE'])) {
                echoLog(dict('GEN_LBL_COMPMISSING'));
                $this->report_exceptions(dict('GEN_LBL_COMPMISSING'));
                return;
            }
            $this->set_comp_parameters($parameters['COMP_CODE']);
            //Drop temporary table just in case exists
            $this->dropTempTables(); //$this->batch_id);
            //0. Inicializar tablas temporales
            $this->settlement_reject_pending($parameters['COMP_CODE']);
            //1. Load temporary information and list all participants to settle
            //currency exchanges
            $this->initCurrTempTable($parameters['COMP_CODE']);
            //Setting candidates and balances
            $this->initFipaTempTable($parameters['COMP_CODE']);
            $this->initGlBalTempTable($parameters['COMP_CODE']);

            //Marcando como cancelado contratos sin movimientos
            $this->setClosedEmptyContracts($parameters['COMP_CODE']);

            //Building Operation Lines
            $this->settOperationLines();
            //Creating transaction
            $this->build_transaction();
            //Cleanup tables.
            $this->dropTempTables(); //$this->batch_id);
            echoLog('Settlement process ended. Elapse: ' . (microtime_float() - $this->start_time));
            echoLog('Settlement process ended<hr>');
        }

        /**
         * Validate if participant can be settle
         * @param: String contract_code
         * @param: String participant_id
         * @return: Boolean
         */
        function validate($comp_code, &$msg) {
            $status = true;

            $sql = 'DROP TABLE IF EXISTS SET_TEMP_LIST';
            ClxCommon::$db->executeQuery($sql);

            $sql = 'CREATE TEMPORARY TABLE SET_TEMP_LIST SELECT FIPA.ID, GLCP.OPERATION_DATE END_DATE, FICO.COMP_CODE, FIPA.CONTRACT_CODE, FIPA.ACCOUNTCODE, FICO.CONTRACT_NUMBER, CASE FICO.CANCEL_DATE WHEN GLCP.OPERATION_DATE THEN 1 ELSE 0 END IS_FICO_SETT FROM  FIDU_PARTICIPANTS FIPA,  FIDU_CONTRACTS FICO,  GL_PARAMETERS_TBL GLCP, CRM_ACCOUNTS ACCO WHERE FIPA.CONTRACT_CODE = FICO.IDCODE AND ACCO.IDCODE = FIPA.ACCOUNTCODE  AND FICO.COMP_CODE = GLCP.COMP_CODE AND FIPA.STATUS = ' . singleQuote('A') . ' AND FICO.COMP_CODE = ' . singleQuote($comp_code) . ' AND FICO.CONTRACT_STATUS = ' . singleQuote('ACTI') . ' AND FIPA.ISPRIMARY=1 AND (FIPA.END_DATE = ADDDATE(GLCP.OPERATION_DATE,INTERVAL 1 DAY) OR FICO.CANCEL_DATE = ADDDATE(GLCP.OPERATION_DATE,INTERVAL 1 DAY))';
            ClxCommon::$db->executeQuery($sql);

            //Check if settle account has more than one bank account
            //$sql = 'SELECT COUNT(A.BANK_ACCOUNT_NO) QTY, A.CHART_FIELD01, A.CHART_FIELD02 FROM (SELECT FIBK.BANK_ACCOUNT_NO, JABV.CHART_FIELD01, JABV.CHART_FIELD02, SUM(JABV.BALANCE_AMT) BALANCE_AMT FROM ' . $this->fiasname . ' FIAS, ' . $this->pratname . ' PRAT, ' . $this->fibkname . ' FIBK, ' . $this->jabvname . ' JABV, ' . $this->setvname . ' SETV WHERE PRAT.PRODUCT_CODE = FIBK.BANK_PROD_CODE AND JABV.CHART_FIELD03 = FIAS.IDCODE AND FIAS.FIDU_BANKACT_CODE = FIBK.BANK_ACCOUNT_NO AND PRAT.PRO_GL_CODE = JABV.GLACCOUNTCODE AND FIAS.CONTRACT_CODE = JABV.CHART_FIELD01 AND JABV.CHART_FIELD01 = SETV.CONTRACT_CODE AND JABV.CHART_FIELD02 = SETV.ACCOUNTCODE AND JABV.TRUST_ITEM_TYPE = ' . singleQuote('CA') . ' GROUP BY JABV.CHART_FIELD01, JABV.CHART_FIELD02, FIBK.BANK_ACCOUNT_NO HAVING SUM(JABV.BALANCE_AMT)<>0) A GROUP BY A.CHART_FIELD01, A.CHART_FIELD02 HAVING COUNT(A.BANK_ACCOUNT_NO)>1';
            //$sql = 'SELECT COUNT(A.BANK_ACCOUNT_NO) QTY, A.CONTRACT_NUMBER CHART_FIELD01, A.CHART_FIELD02 FROM (SELECT FIBK.BANK_ACCOUNT_NO, SETV.CONTRACT_NUMBER, SETV.CONTRACT_CODE CHART_FIELD01, SETV.ACCOUNTCODE CHART_FIELD02, FIBK.BALANCE_AMT FROM FIDU_BANK_ACCOUNTS FIBK, FIDU_ITEMS FIAS, SET_TEMP_LIST SETV  WHERE FIAS.FIDU_BANKACT_CODE = FIBK.BANK_ACCOUNT_NO AND FIAS.CONTRACT_CODE = SETV.CONTRACT_CODE  AND (FIAS.ACCOUNTCODE = SETV.ACCOUNTCODE OR FIAS.ACCOUNTCODE IS NULL OR FIAS.ACCOUNTCODE = "") AND FIBK.BALANCE_AMT<>0 AND FIBK.IS_CDAC = 0 AND FIBK.COMP_CODE = ' . singleQuote($comp_code) . ') A GROUP BY A.CONTRACT_NUMBER, A.CHART_FIELD02 HAVING QTY>1';
            $sql = 'SELECT COUNT(A.BANK_ACCOUNT_NO) QTY, A.CHART_FIELD01, A.CHART_FIELD02 FROM (SELECT FIBK.BANK_ACCOUNT_NO, JABV.CHART_FIELD01, JABV.CHART_FIELD02, SUM(JABV.BALANCE_AMT) BALANCE_AMT FROM ' . $this->fiasname . ' FIAS, ' . $this->pratname . ' PRAT, ' . $this->fibkname . ' FIBK, ' . $this->jabvname . ' JABV, SET_TEMP_LIST SETV WHERE PRAT.PRODUCT_CODE = FIBK.BANK_PROD_CODE AND JABV.CHART_FIELD03 = FIAS.IDCODE AND FIAS.FIDU_BANKACT_CODE = FIBK.BANK_ACCOUNT_NO AND FIBK.IS_CDAC = 0 AND PRAT.PRO_GL_CODE = JABV.GLACCOUNTCODE AND FIAS.CONTRACT_CODE = JABV.CHART_FIELD01 AND JABV.CHART_FIELD01 = SETV.CONTRACT_CODE AND JABV.CHART_FIELD02 = SETV.ACCOUNTCODE AND JABV.TRUST_ITEM_TYPE = ' . singleQuote('CA') . ' GROUP BY JABV.CHART_FIELD01, JABV.CHART_FIELD02, FIBK.BANK_ACCOUNT_NO HAVING SUM(JABV.BALANCE_AMT)<>0) A GROUP BY A.CHART_FIELD01, A.CHART_FIELD02 HAVING COUNT(A.BANK_ACCOUNT_NO)>1';
            $rs = ClxCommon::$db->executeQuery($sql);
            while ($r = ClxCommon::$db->getNextRecord($rs, ASSOCIATIVE)) {
                if (empty($msg['GLREP_SDATE']))
                    $msg['GLREP_SDATE'] = '';
                $msg['GLREP_SDATE'] .= '<br>- ' . smsg('MSG-251', $r['CHART_FIELD01'], $r['CHART_FIELD02']);
                $status = false;
            }

            $sql = 'SELECT T.CONTRACT_NUMBER, JCBV.CHART_FIELD01, JCBV.CHART_FIELD02, SUM(JCBV.BALANCE_AMT) BALANCE_AMT FROM GL_CUR_BALANCE_VW JCBV, SET_TEMP_LIST T WHERE JCBV.CHART_FIELD01 = T.CONTRACT_CODE AND JCBV.CHART_FIELD02 = T.ACCOUNTCODE AND JCBV.GL_TYPE = '.singleQuote('LI').' GROUP BY T.CONTRACT_NUMBER, JCBV.CHART_FIELD01, JCBV.CHART_FIELD02 HAVING SUM(JCBV.BALANCE_AMT)<>0';
            $rs = ClxCommon::$db->executeQuery($sql);
            while ($r = ClxCommon::$db->getNextRecord($rs, ASSOCIATIVE)) {
                if (empty($msg['GLREP_SDATE']))
                    $msg['GLREP_SDATE'] = '';
                $msg['GLREP_SDATE'] .= '<br>- ' . smsg('MSG-504', $r['CONTRACT_NUMBER'], $r['CHART_FIELD02']);
                $status = false;
            }

            //used on gl_closings
            //if investment directed not = 0 then stop and alert to sell or transfer first
            //check if directed transactions
            //$sql = 'SELECT COUNT(INDV.INVESTMENT_CODE) QTY FROM ' . $this->infuname . ' INFU, ' . $this->indvname . ' INDV, ' . $this->setvname . ' SETV WHERE INFU.IDCODE = INDV.PORTFOLIO_CODE AND INFU.DISCRETIONARY_PORT = 0 AND INDV.INVEST_AMT <> 0 AND INDV.CHART_FIELD01 = SETV.CONTRACT_CODE AND INDV.CHART_FIELD02 = SETV.ACCOUNTCODE AND SETV.COMP_CODE = INFU.COMP_CODE AND SETV.COMP_CODE = ' . singleQuote($comp_code);
            $sql = 'SELECT COUNT(INDV.INVESTMENT_CODE) QTY FROM INVE_PORTFOLIOS INFU, INVE_INVESTMENTS INDV, SET_TEMP_LIST SETV WHERE INFU.COMP_CODE = ' . singleQuote($comp_code) . ' AND INFU.IDCODE = INDV.PORTFOLIO_CODE AND INFU.DISCRETIONARY_PORT = 0 AND INDV.INVEST_AMT <> 0 AND INDV.CONTRACT_CODE = SETV.CONTRACT_CODE AND INDV.PARTICIPANT_ID = SETV.ACCOUNTCODE AND SETV.COMP_CODE = INFU.COMP_CODE AND SETV.COMP_CODE = ' . singleQuote($comp_code) . ' HAVING QTY>0';
            if (ClxCommon::$db->getOne($sql) > CLX_NUM_ZERO) {
                if (empty($msg['GLREP_SDATE']))
                    $msg['GLREP_SDATE'] = '';
                $msg['GLREP_SDATE'] .= '<br>- ' . smsg('MSG-142');
                $status = false;
            }
            $sql = 'SELECT COUNT(FILO.ID) QTY FROM FIDU_ITEMS FIAS, CRED_CREDITS FILO, SET_TEMP_LIST SETV  WHERE FILO.COMP_CODE = ' . singleQuote($comp_code) . ' AND FIAS.CONTRACT_CODE = SETV.CONTRACT_CODE AND FILO.IDCODE = FIAS.LOAN_CODE AND FILO.CLX_EFFTDATE = (SELECT MAX(FILO.CLX_EFFTDATE) FROM CRED_CREDITS FILOX WHERE FILOX.IDCODE = FILO.IDCODE) AND FILO.PRINCIPAL_BAL <> 0 AND SETV.COMP_CODE = ' . singleQuote($comp_code);
            if (ClxCommon::$db->getOne($sql) > CLX_NUM_ZERO) {
                if (empty($msg['GLREP_SDATE']))
                    $msg['GLREP_SDATE'] = '';
                $msg['GLREP_SDATE'] .= '<br>- ' . smsg('MSG-502', '');
                $status = false;
            }
            //Validar actividades no contabilizadas para portafolios no dirigidos
            //$sql = 'SELECT COUNT(INDV.INVESTMENT_CODE) QTY FROM ' . $this->infuname . ' INFU, ' . $this->indvname . ' INDV, ' . $this->setvname . ' SETV WHERE INFU.IDCODE = INDV.PORTFOLIO_CODE AND INFU.DISCRETIONARY_PORT = 0 AND INDV.INVEST_AMT <> 0 AND INDV.CHART_FIELD01 = SETV.CONTRACT_CODE AND INDV.CHART_FIELD02 = SETV.ACCOUNTCODE AND SETV.COMP_CODE = INFU.COMP_CODE AND SETV.COMP_CODE = ' . singleQuote($comp_code);
            //$sql = 'SELECT COUNT(INDV.INVESTMENT_CODE) QTY FROM ' . $this->infuname . ' INFU, ' . $this->indvname . ' INDV, SET_TEMP_LIST SETV WHERE INFU.IDCODE = INDV.PORTFOLIO_CODE AND INFU.DISCRETIONARY_PORT = 0 AND INDV.INVEST_AMT <> 0 AND INDV.CHART_FIELD01 = SETV.CONTRACT_CODE AND INDV.CHART_FIELD02 = SETV.ACCOUNTCODE AND SETV.COMP_CODE = INFU.COMP_CODE AND SETV.COMP_CODE = ' . singleQuote($comp_code);
            /* if(ClxCommon::$db->getOne($sql)>0){
              $msg['GLREP_SDATE'] = smsg('MSG-142');
              $status = false;
              } */

            $sql = 'DROP TABLE IF EXISTS SET_TEMP_LIST';
            ClxCommon::$db->executeQuery($sql);

            return $status;
        }

        /**
         * Builds lines for settlement transaction from temporary table
         * @param type $jehdr_code
         * @param type $currency_code
         * @param type $contract_code
         * @param type $participant_id
         */
        private function settJelnLines($jehdr_code, $currency_code, $contract_code, $participant_id) {
            $sql = 'INSERT INTO ' . $this->jelnname . '  (JEHDR_CODE, CURRENCY_CODE, EXCHANGE_RT, CUR_OPERATOR, GLACCOUNTCODE, ITEM_CODE, ITEM_REF_NO, CHART_FIELD01, CHART_FIELD02, CHART_FIELD03, CHART_FIELD04, DEBIT_TRANS_AMT, CREDIT_TRANS_AMT, DEBIT_AMT, CREDIT_AMT, RECONCILIED, SEQNUM) SELECT ' . singleQuote($jehdr_code) . ' JEHDR_CODE, CURRENCY_CODE, EXCHANGE_RT, CUR_OPERATOR, GLACCOUNTCODE, ITEM_CODE, nullIF(ITEM_REF_NO, ' . singleQuote('NA') . ') ITEM_REF_NO, CHART_FIELD01, CHART_FIELD02, nullIF(CHART_FIELD03, ' . singleQuote('NA') . ') CHART_FIELD03, nullIF(CHART_FIELD04, ' . singleQuote('NA') . ') CHART_FIELD04, SUM(DEBIT_TRANS_AMT) DEBIT_TRANS_AMT, SUM(CREDIT_TRANS_AMT) CREDIT_TRANS_AMT, SUM(DEBIT_AMT) DEBIT_AMT, SUM(CREDIT_AMT) CREDIT_AMT, 0 RECONCILIED, SEQNUM FROM SETT_OPERATION_TMP WHERE CHART_FIELD01 = ' . singleQuote($contract_code) . ' AND CHART_FIELD02 = ' . singleQuote($participant_id) . ' AND CURRENCY_CODE = ' . singleQuote($currency_code) . ' GROUP BY CURRENCY_CODE, EXCHANGE_RT, CUR_OPERATOR, GLACCOUNTCODE, ITEM_CODE, CHART_FIELD01, CHART_FIELD02, CHART_FIELD03, CHART_FIELD04, SEQNUM, nullIF(ITEM_REF_NO, ' . singleQuote('NA') . ')';
            ClxCommon::$db->executeQuery($sql);
        }

        /**
         * Construye la transaccion de finiquito SYS_DESIN
         */
        private function build_transaction() {
            $idname = dict('GEN_LBL_SETTLEMENT');
            if (!class_exists('Jehd')) {
                includeCodeLib('JEHD');
            }
            $jehd = new Jehd();
            if (!class_exists('Fiat')) {
                includeCodeLib('FIAT');
            }
            $fiat = new Fiat();
            /*
            //Build JEHD
            ClxCommon::$db->executeQuery('DROP TABLE IF EXISTS DEBUG_TMP');
            ClxCommon::$db->executeQuery('CREATE TABLE DEBUG_TMP SELECT * FROM SETT_OPERATION_TMP');

            ClxCommon::$db->executeQuery('DROP TABLE IF EXISTS DEBUG_TMP2');
            ClxCommon::$db->executeQuery('CREATE TABLE DEBUG_TMP2 SELECT * FROM FIDU_FIPA_TMP');

            ClxCommon::$db->executeQuery('DROP TABLE IF EXISTS DEBUG_TMP3');
            ClxCommon::$db->executeQuery('CREATE TABLE DEBUG_TMP3 SELECT * FROM GL_BALANCES_TMP');
            */
            $sql = 'SELECT DISTINCT ' . singleQuote('AUTO') . ' IDCODE, ' . singleQuote($idname) . ' IDNAME, PA.COMP_CODE, PA.OPERATION_DATE EFFTDATE, PA.OPERATION_DATE REAL_DATE, PA.OPERATION_DATE AVAILABILITY_DATE, ' . singleQuote($this->je_source) . ' JE_SOURCE, PA.CHART_FIELD01, PA.CHART_FIELD02, OP.CURRENCY_CODE, OP.CUR_OPERATOR, OP.EXCHANGE_RT, PA.BATCH_ID, ' . singleQuote($this->username) . ' CREATEDBY, ' . singleQuote($this->username) . ' MODIFIEDBY, ' . singleQuote($this->now) . ' CDATE, ' . singleQuote($this->now) . ' MDATE FROM SETT_OPERATION_TMP OP, FIDU_FIPA_TMP PA WHERE PA.CHART_FIELD01 = OP.CHART_FIELD01 AND PA.CHART_FIELD02 = OP.CHART_FIELD02 AND PA.BATCH_ID = ' . singleQuote($this->batch_id);
            $rs = ClxCommon::$db->executeQuery($sql);
            while ($r = ClxCommon::$db->getNextRecord($rs, ASSOCIATIVE)) {
                $fiatrec = $r;
                $currency_code = $r['CURRENCY_CODE'];
                $chart_field01 = $r['CHART_FIELD01'];
                $chart_field02 = $r['CHART_FIELD02'];
                //ojo que secuencia 7 tiene que ver con el efectivo que sale.
                $sql = 'SELECT FIAS.IDCODE CHART_FIELD03, FIAS.FIDU_BANKACT_CODE, -1*IFNULL(SUM(DEBIT_TRANS_AMT-CREDIT_TRANS_AMT),0) TRANS_AMT FROM SETT_OPERATION_TMP O, FIDU_ITEMS FIAS WHERE FIAS.IDCODE = O.CHART_FIELD03 AND O.CHART_FIELD01 = ' . singleQuote($chart_field01) . ' AND O.CHART_FIELD02 = ' . singleQuote($chart_field02) . ' AND O.CURRENCY_CODE = ' . singleQuote($currency_code) . ' AND O.SEQNUM = 7 GROUP BY FIAS.IDCODE, FIAS.FIDU_BANKACT_CODE';

                //Selecting bank account for disbursement
                $cashinfo = ClxCommon::$db->getOneRecord($sql);
                $trans_amt = $cashinfo['TRANS_AMT'];
                $fiatrec['FIDU_BANKACT_CODE'] = $cashinfo['FIDU_BANKACT_CODE'];
                $fiatrec['CHART_FIELD03'] = $cashinfo['CHART_FIELD03'];
                 //  				$fiatrec['DISBURSEMENT_TYPE'] = 'CH';
				 $defaultDisbursement=companyParameter('FIAT_DISBURSEMENT_TYPE_DEFAULT');
                $fiatrec['DISBURSEMENT_TYPE'] = (EMPTY($defaultDisbursement)?'	':$defaultDisbursement);
                $fiatrec['BANKAC_CODE'] = $fiatrec['FIDU_BANKACT_CODE'];
                $fiatrec['BENEFICIARY_CODE'] = $chart_field02;
                $r['TRANS_CODE'] = $fiat->fiat_build_fiat_only($r['COMP_CODE'], 'SYS_DESIN', $chart_field01, $chart_field02, $trans_amt, $fiatrec);
                unset($r['CURRENCY_CODE'], $r['CHART_FIELD01'], $r['CHART_FIELD02'], $r['CUR_OPERATOR'], $r['EXCHANGE_RT'], $r['AVAILABILITY_DATE'], $r['OPERATION_DATE'], $fiatrec, $cashinfo, $trans_amt);
                //Generar encabezado para portafolio
                $jehd->jehd_build($r);
                $jehdr_code = $r['IDCODE'];
                //dump all invesment account for portfolio into cash
                //distribute all investment based on cash amounts
                $this->settJelnLines($jehdr_code, $currency_code, $chart_field01, $chart_field02);
                //Cambiar estatus de asientos generados por el proceso
            }//end while
            return;
        }

        /**
         * Reject pending settlement transactions
         * @param: String contract_code
         * @param: String participant_id
         * @return: void
         */
        private function settlement_reject_pending($comp_code) {
            $sql = 'UPDATE ' . $this->fiatname . ' FIAT SET FIAT.CLX_AUTH_STATUS = ' . singleQuote('REJ') . ', FIAT.TRANS_STATUS = ' . singleQuote('REJ') . ' WHERE FIAT.JE_SOURCE = ' . singleQuote($this->je_source) . ' AND FIAT.TRANS_STATUS = ' . singleQuote('PEND') . ' AND EXISTS(SELECT FICO.IDCODE FROM ' . $this->ficoname . ' FICO WHERE FICO.IDCODE = FIAT.CHART_FIELD01 AND FICO.COMP_CODE = ' . singleQuote($comp_code) . ')';
            ClxCommon::$db->executeQuery($sql);
            $sql = 'UPDATE ' . $this->jehdname . ' JEHD SET JEHD.CLX_AUTH_STATUS = ' . singleQuote('REJ') . ', JEHD.JE_STATUS = ' . singleQuote('REJ') . ' WHERE JEHD.JE_SOURCE = ' . singleQuote($this->je_source) . '  AND JEHD.JE_STATUS = ' . singleQuote('PEND') . ' AND JEHD.COMP_CODE = ' . singleQuote($comp_code);
            ClxCommon::$db->executeQuery($sql);
        }

        // Settlement Reports
        /**
         * Reporte de desincorporacion. La idea es analizar
         * como quedan las cuenta tras que la transaccion es creada
         */
        function settlement_report_start() {
            $idlist = toString(getIdlist());
            $sql = 'SELECT FIAT.IDCODE, FIAT.CONTRACT_CODE, FIAT.PARTICIPANT_ID FROM ' . $this->fiatname . ' FIAT, ' . $this->ficoname . ' FICO, ' . $this->fipaname . ' FIPA WHERE FIAT.CONTRACT_CODE = FICO.IDCODE AND FIPA.CONTRACT_CODE = FIAT.CONTRACT_CODE AND FIPA.ACCOUNTCODE = FIAT.PARTICIPANT_ID AND FIAT.TRANSTBL_CODE = ' . singleQuote('SYS_DESIN') . ' AND FIAT.TRANS_STATUS = ' . singleQuote('PEND') . ' AND FIPA.ID = ' . $idlist . ' ORDER BY FIAT.ID DESC ';
            $trans_code = ClxCommon::$db->getOneRecord($sql);
            ClxFreeReturn($this->settlement_report($trans_code['IDCODE'], $trans_code['CONTRACT_CODE'], $trans_code['PARTICIPANT_ID']));
        }

        /**
         * Test if transaction its on balance
         */
        function settlement_report() {
            $idlist = toString(getIdlist());
            $fiparec = ClxCommon::$db->getFields($this->fipaname, 'CONTRACT_CODE, ACCOUNTCODE', 'ID=' . $idlist);
            if (empty($fiparec))
                return false;
            $contract_code = $fiparec['CONTRACT_CODE'];
            $participant_id = $fiparec['ACCOUNTCODE'];
            $status = false;

            $contract_no = ClxCommon::$db->getField($this->ficoname, 'CONTRACT_NUMBER', 'IDCODE = ' . singleQuote($contract_code));
            $participant_name = ClxCommon::$db->getField($this->acconame, 'IDNAME', 'IDCODE = ' . singleQuote($participant_id));
            $this->contents = '';
            $table = new htmlTable(CLX_WIDTH_FIVE);
            $table->setProps(array(CLX_PROP_CLASS => 'buttons', CLX_PROP_WIDTH => CLX_PROP_100PCT));

            $table->addCell(dict('GEN_LBL_ACCDETAILS'), array(CLX_PROP_CLASS => CLX_CLASS_TITLE, CLX_PROP_COLSPAN => $table->width));
            $table->addCell(dict('GEN_LBL_CONTRACT_NAME') . ': ' . $participant_name . ' (' . $contract_no . ')', array(CLX_PROP_CLASS => CLX_CLASS_DATA, CLX_PROP_COLSPAN => $table->width));
            $table->addCell(dict('GEN_LBL_PARTICIPANT') . ': ' . $participant_name . ' (' . $participant_id . ')', array(CLX_PROP_CLASS => CLX_CLASS_DATA, CLX_PROP_COLSPAN => $table->width));
            $table->addCell(dict('GEN_LBL_ACCOUNT'), array(CLX_PROP_CLASS => CLX_CLASS_HEADERS));
            //$table->addCell(dict('GEN_LBL_CURRENCY_CODE'),array( CLX_PROP_CLASS=> CLX_CLASS_HEADERS));
            $table->addCell(dict('GEN_LBL_INITBALANCE'), array(CLX_PROP_CLASS => CLX_CLASS_HEADERS));
            $table->addCell(dict('GEN_LBL_DEBITS'), array(CLX_PROP_CLASS => CLX_CLASS_HEADERS));
            $table->addCell(dict('GEN_LBL_CREDITS'), array(CLX_PROP_CLASS => CLX_CLASS_HEADERS));
            $table->addCell(dict('GEN_LBL_BALANCE'), array(CLX_PROP_CLASS => CLX_CLASS_HEADERS));
            //Pending operations
            $sql = 'SELECT GLAC.IDCODE GLACCOUNTCODE, GLAC.GL_TYPE, GLAC.IDNAME, SUM(IF(JEHD.JE_STATUS = ' . singleQuote('AP') . ', JELN.DEBIT_AMT-JELN.CREDIT_AMT, 0)) BALANCE_AMT, SUM(IF(JEHD.JE_STATUS = ' . singleQuote('PEND') . ', JELN.DEBIT_AMT, 0)) DEBIT_AMT, SUM(IF(JEHD.JE_STATUS = ' . singleQuote('PEND') . ', JELN.CREDIT_AMT, 0)) CREDIT_AMT, SUM(JELN.DEBIT_AMT-JELN.CREDIT_AMT) FINAL_BALANCE_AMT FROM GL_PARAMETERS_TBL GLCP, GL_JOURNAL_HDR JEHD, GL_JOURNAL_LINES JELN, GL_CODES_TBL GLAC WHERE GLAC.GLGROUP_CODE = GLCP.GLGROUP_CODE AND GLAC.IDCODE = JELN.GLACCOUNTCODE AND GLCP.COMP_CODE = JEHD.COMP_CODE AND JELN.JEHDR_CODE = JEHD.IDCODE AND JEHD.JE_STATUS IN (' . singleQuote('AP') . ',' . singleQuote('PEND') . ') AND JELN.CHART_FIELD01 = ' . singleQuote($contract_code) . ' AND JELN.CHART_FIELD02 = ' . singleQuote($participant_id) . ' GROUP BY GLAC.IDCODE, GLAC.GL_TYPE, GLAC.IDNAME HAVING BALANCE_AMT<>0 OR DEBIT_AMT <>0 OR CREDIT_AMT <>0 ORDER BY GLAC.IDCODE ASC';
            $total = array(CLX_NUM_ZERO, CLX_NUM_ZERO, CLX_NUM_ZERO, CLX_NUM_ZERO);
            $balances = ClxCommon::$db->executeQuery($sql);
            while (($balance = ClxCommon::$db->getNextRecord($balances))) {
                $table->addCell($balance['GLACCOUNTCODE'] . ' - ' . $balance['GL_TYPE'] . ' - ' . $balance['IDNAME'], array(CLX_PROP_CLASS => CLX_CLASS_DATA));
                //$table->addCell($balance['CURRENCY_CODE'],array( CLX_PROP_CLASS=> CLX_CLASS_DATA));
                $table->addCell(xFormat::number($balance['BALANCE_AMT']), array(CLX_PROP_CLASS => CLX_CLASS_DATA, CLX_PROP_ALIGN => CLX_PROP_RIGHT));
                $balance['FINAL_BALANCE_AMT'] = bcadd($balance['BALANCE_AMT'], bcsub($balance['DEBIT_AMT'], $balance['CREDIT_AMT']));
                $total[CLX_NUM_ZERO] = bcadd($balance['BALANCE_AMT'], $total[CLX_NUM_ZERO]);
                $total[CLX_NUM_ONE] = bcadd($balance['DEBIT_AMT'], $total[CLX_NUM_ONE]);
                $total[CLX_NUM_TWO] = bcadd($balance['CREDIT_AMT'], $total[CLX_NUM_TWO]);
                $total[CLX_NUM_THREE] = bcadd($balance['FINAL_BALANCE_AMT'], $total[CLX_NUM_THREE]);
                $table->addCell(xFormat::number($balance['DEBIT_AMT']), array(CLX_PROP_CLASS => CLX_CLASS_DATA, CLX_PROP_ALIGN => CLX_PROP_RIGHT));
                $table->addCell(xFormat::number($balance['CREDIT_AMT']), array(CLX_PROP_CLASS => CLX_CLASS_DATA, CLX_PROP_ALIGN => CLX_PROP_RIGHT));
                $table->addCell(xFormat::number($balance['FINAL_BALANCE_AMT']), array(CLX_PROP_CLASS => CLX_CLASS_DATA, CLX_PROP_ALIGN => CLX_PROP_RIGHT));
            }
            //Displaying totals
            $table->addCell(dict('GEN_LBL_TOTAL'), array(CLX_PROP_CLASS => CLX_CLASS_HEADERS));
            $table->addCell(xFormat::number($total[CLX_NUM_ZERO]), array(CLX_PROP_CLASS => CLX_CLASS_HEADERS, CLX_PROP_ALIGN => CLX_PROP_RIGHT));
            $table->addCell(xFormat::number($total[CLX_NUM_ONE]), array(CLX_PROP_CLASS => CLX_CLASS_HEADERS, CLX_PROP_ALIGN => CLX_PROP_RIGHT));
            $table->addCell(xFormat::number($total[CLX_NUM_TWO]), array(CLX_PROP_CLASS => CLX_CLASS_HEADERS, CLX_PROP_ALIGN => CLX_PROP_RIGHT));
            $table->addCell(xFormat::number($total[CLX_NUM_THREE]), array(CLX_PROP_CLASS => CLX_CLASS_HEADERS, CLX_PROP_ALIGN => CLX_PROP_RIGHT));
            $this->contents = $table->source();
            return $status;
        }

        /**
         * Returns content source
         */
        function source() {
            return $this->contents;
        }

        // Settlement Approval Method
        /**
         * Process settlement transaction
         */
        function settlement_approval($trans_code) {
            // get transaction record
            $transaction = ClxCommon::$db->getRecord($this->fiatname, 'IDCODE = ' . singleQuote($trans_code));
            // change participant status
            $this->settlement_chance_participant_status($transaction['CHART_FIELD01'], $transaction['CHART_FIELD02']);
        }

        /**
         * Validates if transaction can be approved
         */
        function settlement_approval_validate(&$transaction) {
            $status = true;
            //Check participant status
            if (ClxCommon::$db->oneExistsRecords($this->fipaname, 'STATUS = ' . singleQuote('A') . ' AND CONTRACT_CODE = ' . singleQuote($transaction['CONTRACT_CODE']) . ' AND ACCOUNTCODE = ' . singleQuote($transaction['PARTICIPANT_ID'])) == CLX_NUM_ZERO) {
                addMessage('A1' . dict('CODE_ERR_TRANSINVALIDSETTLEAGAIN') . ' ' . $transaction['IDCODE']);
                $status = false;
            }
            //check transaction status
            if (!$this->settlement_report($transaction['IDCODE'], $transaction['CONTRACT_CODE'], $transaction['PARTICIPANT_ID'])) {
                addMessage('A2' . dict('CODE_ERR_TRANSINVALIDSETTLEAGAIN') . ' ' . $transaction['IDCODE']);
                $status = false;
            }
            //if investment directed not = 0 then stop and alert to sell or transfer first
            $sql = 'SELECT COUNT(INFU.ID) FROM ' . $this->infuname . ' INFU, ' . $this->fiasname . ' FIAS, $this->fidaname FIDA WHERE FIAS.IDCODE = FIDA.ASSET_CODE AND INFU.IDCODE = FIAS.PORTFOLIO_CODE AND INFU.DISCRETIONARY_PORT = 0 AND ROUND(FIDA.BALANCE_AMT, 2) <> 0 AND FIDA.CONTRACT_CODE = ' . singleQuote($transaction['CONTRACT_CODE']) . ' AND FIDA.PARTICIPANT_ID = ' . singleQuote($transaction['PARTICIPANT_ID']) . ' ';
            if (ClxCommon::$db->getOne($sql) > CLX_NUM_ZERO) {
                /*
                 * GEN_ERR_INVESTMENTPENDING
                 * Investments must be sell or transfer before participant closing, please sell or transfer investments before continuing
                 * Participante posee inversiones que no pueden ser colocadas automaticamente, Favor resolver inversiones antes de finiquitar.
                 */
                addMessage('A4 ' . dict('GEN_ERR_INVESTMENTPENDING'));
                $status = false;
            }
            if (!empty($this->debug)) {
                addMessage('A3 ' . 'Debug enabled, no transacction will be approved.');
                $status = false; // Just for debug options
            }
            return $status;
        }

        /**
         * Validates if transaction can be approved
         */
        function settlement_chance_participant_status($contract_code, $participant_id, $STATUS = 'I') {
            ClxCommon::$db->updateRecord($this->fipaname, array('STATUS' => $status), 'ACCOUNTCODE = ' . singleQuote($participant_id) . ' AND CONTRACT_CODE = ' . singleQuote($contract_code) . ' AND STATUS = ' . singleQuote('A') . '');
        }

        // Settlement Internal Sale On Closing After settlements are approved
        function sett_int_sales_button() {
            $idlist = getIdlist();
            $idlist = toString(getIdlist());
            $sql = 'SELECT COMP_CODE FROM ' . $this->ficoname . '  WHERE ID IN (' . $idlist . ')';
            $rs = ClxCommon::$db->executeQuery($sql);
            while (($r = ClxCommon::$db->getNextRecord($rs, NUMERIC))) {
                $this->settlement_interest_internal_sales($r[CLX_NUM_ZERO]);
            }
        }

        // Sett new version methods
        /**
         * Initializes
         * @param type $batch_id
         */
        function dropTempTables() {
            ClxCommon::$db->executeQuery('DROP TABLE IF EXISTS FIDU_FIPA_TMP');
            ClxCommon::$db->executeQuery('DROP TABLE IF EXISTS GL_BALANCES_TMP');
            ClxCommon::$db->executeQuery('DROP TABLE IF EXISTS SETT_OPERATION_TMP');
            ClxCommon::$db->executeQuery('DROP TABLE IF EXISTS SETT_CURR_TMP');
        }

        /**
         * Currency exchanges
         *
         */
        private function initCurrTempTable($comp_code) {
            $sql = 'CREATE TEMPORARY TABLE SETT_CURR_TMP SELECT CURR.* FROM BASE_CURRENCY_CONVERSIONS CURR, GL_PARAMETERS_TBL GLCP, BASE_COMPANIES COMP WHERE GLCP.COMP_CODE = ' . singleQuote($comp_code) . '  AND COMP.IDCODE = GLCP.COMP_CODE AND CURR.FOREIGN_CURRENCY <> CURR.BASE_CURRENCY AND CURR.FOREIGN_CURRENCY = COMP.CURRENCY_CODE AND CURR.CLX_EFFTDATE = (SELECT MAX(CURRX.CLX_EFFTDATE) FROM BASE_CURRENCY_CONVERSIONS CURRX WHERE CURRX.IDCODE = CURR.IDCODE AND CURRX.CLX_EFFTDATE<=GLCP.OPERATION_DATE)';
            ClxCommon::$db->executeQuery($sql);
        }

        /**
         * Fipa temp table
         * @param type $batch_id
         * @param type $comp_code
         * @param type $contract_code
         */
        private function initFipaTempTable($comp_code) {
            $sql = 'CREATE TEMPORARY TABLE FIDU_FIPA_TMP SELECT FICO.COMP_CODE, COMP.CURRENCY_CODE, GLCP.OPERATION_DATE, GLCP.GLGROUP_CODE, CAST(1 AS DECIMAL (20, 6)) EXCHANGE_RT, CAST(1 AS SIGNED) CUR_OPERATOR, ADDDATE(GLCP.OPERATION_DATE, INTERVAL 1 DAY) EFFTDATE, FICO.IDCODE CHART_FIELD01, FIPA.ACCOUNTCODE CHART_FIELD02, TTPA.EQ_GL_CODE, TTPA.EQF_GL_CODE, ' . singleQuote($this->batch_id) . ' BATCH_ID FROM BASE_COMPANIES COMP, FIDU_PARTICIPANTS FIPA, FIDU_CONTRACTS FICO, GL_PARAMETERS_TBL GLCP, FIDU_PARAMETERS_TBL TTPA WHERE TTPA.GLGROUP_CODE = GLCP.GLGROUP_CODE AND FICO.IDCODE = FIPA.CONTRACT_CODE AND TTPA.FI_TYPE = FICO.FI_TYPE AND TTPA.CONTRACT_TYPE = FICO.CONTRACT_TYPE AND TTPA.TRUST_FUND_SRC = FICO.TRUST_FUND_SRC AND FICO.COMP_CODE = GLCP.COMP_CODE AND COMP.IDCODE = FICO.COMP_CODE AND FIPA.STATUS = ' . singleQuote('A') . ' AND FICO.COMP_CODE = ' . singleQuote($comp_code) . ' AND (FIPA.END_DATE = GLCP.OPERATION_DATE OR FICO.CANCEL_DATE = GLCP.OPERATION_DATE)';
            ClxCommon::$db->executeQuery($sql);
        }

        /**
         *
         * @param type $batch_id
         * @param type $comp_code
         */
        private function initGlBalTempTable($comp_code) {
            $sql = 'CREATE TEMPORARY TABLE GL_BALANCES_TMP SELECT FTMP.EFFTDATE, FTMP.CURRENCY_CODE BASE_CURRENCY, JELN.CURRENCY_CODE, CAST(FTMP.EXCHANGE_RT AS DECIMAL (20, 6)) EXCHANGE_RT, CAST(FTMP.CUR_OPERATOR AS SIGNED) CUR_OPERATOR, FTMP.EQ_GL_CODE, FTMP.EQF_GL_CODE, JEHD.COMP_CODE, MAX(IF(JEHD.JE_SOURCE=' . singleQuote('EXVL') . ', ' . singleQuote('EXVL') . ',' . singleQuote('USR') . ')) JE_SOURCE, JELN.GLACCOUNTCODE, ' . singleQuote('SETT') . ' ITEM_CODE, IF(JEHD.JE_SOURCE=' . singleQuote('EXVL') . ', ' . singleQuote('NA') . ', IFNULL(IF(GLAC.GL_TYPE IN (' . singleQuote('AS') . ',' . singleQuote('LI') . '), JELN.ITEM_REF_NO,' . singleQuote('NA') . '),' . singleQuote('NA') . ') ) ITEM_REF_NO, GLAC.GL_TYPE, GLAC.TRUST_ITEM_TYPE, JELN.CHART_FIELD01, JELN.CHART_FIELD02, IFNULL(JELN.CHART_FIELD03, ' . singleQuote('NA') . ') CHART_FIELD03, IFNULL(JELN.CHART_FIELD04, ' . singleQuote('NA') . ') CHART_FIELD04, CAST(SUM(JELN.DEBIT_TRANS_AMT-CREDIT_TRANS_AMT) AS DECIMAL (16, 2)) DEBITS, CAST(0 AS DECIMAL (16, 2)) CREDITS, CAST(SUM(JELN.DEBIT_TRANS_AMT-JELN.CREDIT_TRANS_AMT) AS DECIMAL (16, 2)) DEBIT_TRANS_AMT, CAST(0 AS DECIMAL (16, 2)) CREDIT_TRANS_AMT, CAST(SUM(JELN.DEBIT_AMT-CREDIT_AMT) AS DECIMAL (16, 2)) DEBITS_ORG, CAST(0 AS DECIMAL (16, 2)) CREDITS_ORG, FTMP.BATCH_ID FROM GL_JOURNAL_HDR JEHD, GL_JOURNAL_LINES JELN, GL_CODES_TBL GLAC, FIDU_FIPA_TMP FTMP WHERE 1=1 AND JEHD.IDCODE = JELN.JEHDR_CODE AND GLAC.IDCODE = JELN.GLACCOUNTCODE AND GLAC.GLGROUP_CODE = FTMP.GLGROUP_CODE AND JEHD.COMP_CODE = FTMP.COMP_CODE AND JELN.CHART_FIELD01 = FTMP.CHART_FIELD01 AND JELN.CHART_FIELD02 = FTMP.CHART_FIELD02 AND FTMP.BATCH_ID = ' . singleQuote($this->batch_id) . ' AND JEHD.COMP_CODE = ' . singleQuote($comp_code) . ' AND JEHD.JE_STATUS = ' . singleQuote('AP') . ' GROUP BY FTMP.EFFTDATE, JELN.CURRENCY_CODE, JEHD.COMP_CODE, JELN.GLACCOUNTCODE, IFNULL(IF(GLAC.GL_TYPE IN (' . singleQuote('AS') . ',' . singleQuote('LI') . '), JELN.ITEM_REF_NO,' . singleQuote('NA') . '),' . singleQuote('NA') . '), GLAC.GL_TYPE, GLAC.TRUST_ITEM_TYPE, JELN.CHART_FIELD01, JELN.CHART_FIELD02, JELN.CHART_FIELD03, JELN.CHART_FIELD04, FTMP.EQ_GL_CODE, FTMP.EQF_GL_CODE, FTMP.BATCH_ID';
            ClxCommon::$db->executeQuery($sql);
            /* Resolviendo conversiones de moneda en movimientos */
            $sql = 'UPDATE GL_BALANCES_TMP T, SETT_CURR_TMP C SET T.EXCHANGE_RT = C.EXCHANGE_RT, T.CUR_OPERATOR = C.CUR_OPERATOR WHERE T.CURRENCY_CODE = C.BASE_CURRENCY';
            ClxCommon::$db->executeQuery($sql);
            /* Setting todays exchange */
            $sql = 'UPDATE GL_BALANCES_TMP SET DEBITS = ROUND(DEBITS*POW(EXCHANGE_RT, CUR_OPERATOR), 2) WHERE EXCHANGE_RT <> 1';
            ClxCommon::$db->executeQuery($sql);
            /* Moving balances to the correct abs position */
            $sql = 'UPDATE GL_BALANCES_TMP SET CREDITS = -1*DEBITS, DEBITS = 0 WHERE DEBITS < 0';
            $sql = 'UPDATE GL_BALANCES_TMP SET CREDIT_TRANS_AMT = -1*DEBIT_TRANS_AMT, DEBIT_TRANS_AMT = 0 WHERE DEBIT_TRANS_AMT < 0';
            ClxCommon::$db->executeQuery($sql);
            $sql = 'UPDATE GL_BALANCES_TMP SET CREDITS_ORG = -1*DEBITS_ORG, DEBITS_ORG = 0 WHERE DEBITS_ORG < 0';
            ClxCommon::$db->executeQuery($sql);
            $sql = 'UPDATE GL_BALANCES_TMP SET ITEM_REF_NO = ' . singleQuote('NA') . ' WHERE ITEM_REF_NO = ' . "''";
            ClxCommon::$db->executeQuery($sql);
        }

        /**
         * Mark as closed empty contracts
         * @param type $comp_code
         */
        private function setClosedEmptyContracts($comp_code) {
            $sql = 'SELECT DISTINCT P.CHART_FIELD01, P.CHART_FIELD02 FROM FIDU_FIPA_TMP P LEFT JOIN GL_BALANCES_TMP B ON B.CHART_FIELD01 = P.CHART_FIELD01/* WHERE B.CHART_FIELD01 IS NULL*/';
            $rs = ClxCommon::$db->executeQuery($sql);
            $flag = false;
            while ($r = ClxCommon::$db->getNextRecord($rs)) {
                if (false === $flag) {
                    includeCodeLib('FIPA');
                    $fipa = new Fipa();
                    $flag = true;
                }
                $fipa->fipa_settle_status($r['CHART_FIELD01'], $r['CHART_FIELD02']);
            }
        }

        /**
         * Reversal of operations
         */
        private function settOperationLines() {
            /*
             * Estrategia implica
             * 0. Reversar saldos contables por cuentas
             * 1. Colocar diferencia por cambio para que coincidan movimientos de cuenta
             * 2. Resolviendo cuentas para saldos de acuerdo al tipo de cuenta
             * 3. Generando salida de efectivo contra patrimonio
             */
            //Reversando cuentas con saldo actual al monto de conversion del dia
            $sql = 'CREATE TEMPORARY TABLE SETT_OPERATION_TMP SELECT CAST(' . singleQuote('REV') . ' AS CHAR(10)) TARGET, 0 LINE_TYPE, 0 SEQNUM, JE_SOURCE, GLACCOUNTCODE, GL_TYPE, TRUST_ITEM_TYPE, ITEM_CODE, ITEM_REF_NO, CURRENCY_CODE, EXCHANGE_RT EXCHANGE_RT, CUR_OPERATOR CUR_OPERATOR, CHART_FIELD01, CHART_FIELD02, CHART_FIELD03, CHART_FIELD04, CAST(- 1 *SUM(DEBIT_TRANS_AMT-CREDIT_TRANS_AMT) AS DECIMAL (16, 2))  DEBIT_TRANS_AMT, CAST(0 AS DECIMAL (16, 2))  CREDIT_TRANS_AMT, CAST(- 1 *SUM(DEBITS-CREDITS) AS DECIMAL (16, 2))  DEBIT_AMT, CAST(0 AS DECIMAL (16, 2)) CREDIT_AMT FROM GL_BALANCES_TMP WHERE BATCH_ID = ' . singleQuote($this->batch_id) . ' GROUP BY JE_SOURCE, GLACCOUNTCODE, GL_TYPE, TRUST_ITEM_TYPE, ITEM_CODE, ITEM_REF_NO, CURRENCY_CODE, EXCHANGE_RT, CUR_OPERATOR, CHART_FIELD01, CHART_FIELD02, CHART_FIELD03, CHART_FIELD04';
            ClxCommon::$db->executeQuery($sql);
            /*
              Incorporando reverso de diferencias en cuenta por
              diferencial de moneda al cambio del dia contra cambio
              en contabilidad */
            $sql = 'INSERT INTO SETT_OPERATION_TMP (TARGET, LINE_TYPE, SEQNUM, JE_SOURCE, GLACCOUNTCODE, GL_TYPE, TRUST_ITEM_TYPE, ITEM_CODE, ITEM_REF_NO, CURRENCY_CODE, EXCHANGE_RT, CUR_OPERATOR, CHART_FIELD01, CHART_FIELD02, CHART_FIELD03, CHART_FIELD04, DEBIT_TRANS_AMT, CREDIT_TRANS_AMT, DEBIT_AMT, CREDIT_AMT) SELECT ' . singleQuote('REV') . ' TARGET, 0 LINE_TYPE, 1 SEQNUM, JE_SOURCE, GLACCOUNTCODE, GL_TYPE, TRUST_ITEM_TYPE, ITEM_CODE, ITEM_REF_NO, BASE_CURRENCY CURRENCY_CODE, 1 EXCHANGE_RT, 1 CUR_OPERATOR, CHART_FIELD01, CHART_FIELD02, CHART_FIELD03, CHART_FIELD04, SUM((DEBITS-CREDITS)-(DEBITS_ORG-CREDITS_ORG)) DEBIT_TRANS_AMT, 0 CREDIT_TRANS_AMT, SUM((DEBITS-CREDITS)-(DEBITS_ORG-CREDITS_ORG)) DEBIT_AMT, 0 CREDIT_AMT FROM GL_BALANCES_TMP WHERE BATCH_ID = ' . singleQuote($this->batch_id) . ' AND EXCHANGE_RT <> 1 GROUP BY JE_SOURCE, GLACCOUNTCODE, GL_TYPE, TRUST_ITEM_TYPE, ITEM_CODE, ITEM_REF_NO, BASE_CURRENCY, CHART_FIELD01, CHART_FIELD02, CHART_FIELD03, CHART_FIELD04';
            ClxCommon::$db->executeQuery($sql);

            /* Solventando Inversiones y cuentas por cobrar contra efectivo. En  DEBE EFECTIVO VS HABER AR y INV InicialmenteAND ((JTMP.GL_TYPE ='AS' AND JTMP.TRUST_ITEM_TYPE IN ('AR','ARI','IN','CR')) OR JTMP.GL_TYPE = 'LI') */
            $sql = 'INSERT INTO SETT_OPERATION_TMP (TARGET, LINE_TYPE, SEQNUM, JE_SOURCE, GLACCOUNTCODE, GL_TYPE, TRUST_ITEM_TYPE, ITEM_CODE, ITEM_REF_NO, CURRENCY_CODE, EXCHANGE_RT, CUR_OPERATOR, CHART_FIELD01, CHART_FIELD02, CHART_FIELD03, CHART_FIELD04, DEBIT_TRANS_AMT, CREDIT_TRANS_AMT, DEBIT_AMT, CREDIT_AMT) SELECT ' . singleQuote('CA') . ' TARGET, 0 LINE_TYPE, 2 SEQNUM, ' . singleQuote('USR') . ' JE_SOURCE, ' . singleQuote('CA') . ' GLACCOUNTCODE, ' . singleQuote('AS') . ' GL_TYPE, ' . singleQuote('CA') . ' TRUST_ITEM_TYPE, ' . singleQuote('SETT') . ' ITEM_CODE, ' . singleQuote('NA') . ' ITEM_REF_NO, CURRENCY_CODE, EXCHANGE_RT, CUR_OPERATOR, CHART_FIELD01, CHART_FIELD02, ' . singleQuote('NA') . ' CHART_FIELD03, ' . singleQuote('NA') . ' CHART_FIELD04, SUM(DEBIT_TRANS_AMT-CREDIT_TRANS_AMT) DEBIT_TRANS_AMT, 0  CREDIT_TRANS_AMT, SUM(DEBITS-CREDITS) DEBIT_AMT, 0 CREDIT_AMT FROM GL_BALANCES_TMP WHERE BATCH_ID = ' . singleQuote($this->batch_id) . ' AND GL_TYPE = ' . singleQuote('AS') . ' AND TRUST_ITEM_TYPE IN (' . singleQuoteArray(array('AR', 'ARI', 'IN', 'CR', 'CA','INDV','CD')) . ')  GROUP BY CURRENCY_CODE, EXCHANGE_RT, CUR_OPERATOR, CHART_FIELD01, CHART_FIELD02';
            ClxCommon::$db->executeQuery($sql);
            //Eliminando pasivos contra efectivo
            $sql = 'INSERT INTO SETT_OPERATION_TMP (TARGET, LINE_TYPE, SEQNUM, JE_SOURCE, GLACCOUNTCODE, GL_TYPE, TRUST_ITEM_TYPE, ITEM_CODE, ITEM_REF_NO, CURRENCY_CODE, EXCHANGE_RT, CUR_OPERATOR, CHART_FIELD01, CHART_FIELD02, CHART_FIELD03, CHART_FIELD04, DEBIT_TRANS_AMT, CREDIT_TRANS_AMT, DEBIT_AMT, CREDIT_AMT) SELECT ' . singleQuote('CA') . ' TARGET, 0 LINE_TYPE, 3 SEQNUM, ' . singleQuote('USR') . ' JE_SOURCE, ' . singleQuote('CA') . ' GLACCOUNTCODE, ' . singleQuote('AS') . ' GL_TYPE, ' . singleQuote('CA') . ' TRUST_ITEM_TYPE, ' . singleQuote('SETT') . ' ITEM_CODE, ' . singleQuote('NA') . ' ITEM_REF_NO, CURRENCY_CODE, EXCHANGE_RT, CUR_OPERATOR, CHART_FIELD01, CHART_FIELD02, ' . singleQuote('NA') . ' CHART_FIELD03, ' . singleQuote('NA') . ' CHART_FIELD04, SUM(DEBIT_TRANS_AMT-CREDIT_TRANS_AMT) DEBIT_TRANS_AMT, 0  CREDIT_TRANS_AMT, SUM(DEBITS-CREDITS) DEBIT_AMT, 0 CREDIT_AMT FROM GL_BALANCES_TMP WHERE BATCH_ID = ' . singleQuote($this->batch_id) . ' AND GL_TYPE = ' . singleQuote('LI') . ' GROUP BY CURRENCY_CODE, EXCHANGE_RT, CUR_OPERATOR, CHART_FIELD01, CHART_FIELD02';
            ClxCommon::$db->executeQuery($sql);
            //Resolviendo otros activos no incluidos en efectivo contra patrimonio.
            $sql = 'INSERT INTO SETT_OPERATION_TMP (TARGET, LINE_TYPE, SEQNUM, JE_SOURCE, GLACCOUNTCODE, GL_TYPE, TRUST_ITEM_TYPE, ITEM_CODE, ITEM_REF_NO, CURRENCY_CODE, EXCHANGE_RT, CUR_OPERATOR, CHART_FIELD01, CHART_FIELD02, CHART_FIELD03, CHART_FIELD04, DEBIT_TRANS_AMT, CREDIT_TRANS_AMT, DEBIT_AMT, CREDIT_AMT) SELECT ' . singleQuote('EQ') . ' TARGET, 0 LINE_TYPE, 4 SEQNUM, ' . singleQuote('USR') . ' JE_SOURCE, ' . singleQuote('EQ') . ' GLACCOUNTCODE, ' . singleQuote('EQ') . ' GL_TYPE, ' . singleQuote('EQ') . ' TRUST_ITEM_TYPE, ' . singleQuote('SETT') . ' ITEM_CODE, ' . singleQuote('NA') . ' ITEM_REF_NO, CURRENCY_CODE, 1 EXCHANGE_RT, 1 CUR_OPERATOR, CHART_FIELD01, CHART_FIELD02, ' . singleQuote('NA') . ' CHART_FIELD03, ' . singleQuote('NA') . ' CHART_FIELD04, SUM(DEBIT_TRANS_AMT-CREDIT_TRANS_AMT) DEBIT_TRANS_AMT, 0 CREDIT_TRANS_AMT, SUM(DEBITS-CREDITS) DEBIT_AMT, 0 CREDIT_AMT FROM GL_BALANCES_TMP WHERE BATCH_ID = ' . singleQuote($this->batch_id) . ' AND GL_TYPE = ' . singleQuote('AS') . ' AND TRUST_ITEM_TYPE NOT IN (' . singleQuoteArray(array('AR', 'ARI', 'IN', 'CR', 'CA','INDV','CD')) . ') GROUP BY CURRENCY_CODE, CHART_FIELD01, CHART_FIELD02';
            ClxCommon::$db->executeQuery($sql);

            /* Ingresos-Gastos se resuelven contra patrimonio. */
            $sql = 'INSERT INTO SETT_OPERATION_TMP (TARGET, LINE_TYPE, SEQNUM, JE_SOURCE, GLACCOUNTCODE, GL_TYPE, TRUST_ITEM_TYPE, ITEM_CODE, ITEM_REF_NO, CURRENCY_CODE, EXCHANGE_RT, CUR_OPERATOR, CHART_FIELD01, CHART_FIELD02, CHART_FIELD03, CHART_FIELD04, DEBIT_TRANS_AMT, CREDIT_TRANS_AMT, DEBIT_AMT, CREDIT_AMT) SELECT ' . singleQuote('EQ') . ' TARGET, 0 LINE_TYPE, 5 SEQNUM, ' . singleQuote('USR') . ' JE_SOURCE, ' . singleQuote('EQ') . ' GLACCOUNTCODE, ' . singleQuote('EQ') . ' GL_TYPE, ' . singleQuote('EQ') . ' TRUST_ITEM_TYPE, ' . singleQuote('SETT') . ' ITEM_CODE, ' . singleQuote('NA') . ' ITEM_REF_NO, CURRENCY_CODE, EXCHANGE_RT, CUR_OPERATOR, CHART_FIELD01, CHART_FIELD02, ' . singleQuote('NA') . ' CHART_FIELD03, ' . singleQuote('NA') . ' CHART_FIELD04, SUM(DEBIT_TRANS_AMT-CREDIT_TRANS_AMT) DEBIT_TRANS_AMT, 0 CREDIT_TRANS_AMT, SUM(DEBITS-CREDITS) DEBIT_AMT, 0 CREDIT_AMT FROM GL_BALANCES_TMP WHERE BATCH_ID = ' . singleQuote($this->batch_id) . ' AND GL_TYPE IN (' . singleQuote('IN') . ',' . singleQuote('EX') . ') GROUP BY EXCHANGE_RT, CUR_OPERATOR, CURRENCY_CODE, CHART_FIELD01, CHART_FIELD02';
            ClxCommon::$db->executeQuery($sql);
            /* Incorporando cuentas de patrimonio para que se incluyan en la salida. */
            $sql = 'INSERT INTO SETT_OPERATION_TMP (TARGET, LINE_TYPE, SEQNUM, JE_SOURCE, GLACCOUNTCODE, GL_TYPE, TRUST_ITEM_TYPE, ITEM_CODE, ITEM_REF_NO, CURRENCY_CODE, EXCHANGE_RT, CUR_OPERATOR, CHART_FIELD01, CHART_FIELD02, CHART_FIELD03, CHART_FIELD04, DEBIT_TRANS_AMT, CREDIT_TRANS_AMT, DEBIT_AMT, CREDIT_AMT) SELECT ' . singleQuote('EQ') . ' TARGET, 0 LINE_TYPE, 6 SEQNUM, ' . singleQuote('USR') . ' JE_SOURCE, ' . singleQuote('EQ') . ' GLACCOUNTCODE, ' . singleQuote('EQ') . ' GL_TYPE, ' . singleQuote('EQ') . ' TRUST_ITEM_TYPE, ' . singleQuote('SETT') . ' ITEM_CODE, ' . singleQuote('NA') . ' ITEM_REF_NO, CURRENCY_CODE, 1 EXCHANGE_RT, 1 CUR_OPERATOR, CHART_FIELD01, CHART_FIELD02, ' . singleQuote('NA') . ' CHART_FIELD03, ' . singleQuote('NA') . ' CHART_FIELD04, SUM(DEBIT_TRANS_AMT-CREDIT_TRANS_AMT) DEBIT_TRANS_AMT, 0 CREDIT_TRANS_AMT, SUM(DEBITS-CREDITS) DEBIT_AMT, 0 CREDIT_AMT FROM GL_BALANCES_TMP WHERE BATCH_ID = ' . singleQuote($this->batch_id) . ' AND GL_TYPE = ' . singleQuote('EQ') . ' GROUP BY CURRENCY_CODE, CHART_FIELD01, CHART_FIELD02';
            ClxCommon::$db->executeQuery($sql);
            //4 Completando transacciones de finiquito------------------------------
            //Creando movimiento para salida de efectivo final
            $sql = 'INSERT INTO SETT_OPERATION_TMP (TARGET, LINE_TYPE, SEQNUM, JE_SOURCE, GLACCOUNTCODE, GL_TYPE, TRUST_ITEM_TYPE, ITEM_CODE, ITEM_REF_NO, CURRENCY_CODE, EXCHANGE_RT, CUR_OPERATOR, CHART_FIELD01, CHART_FIELD02, CHART_FIELD03, CHART_FIELD04, DEBIT_TRANS_AMT, CREDIT_TRANS_AMT, DEBIT_AMT, CREDIT_AMT) SELECT ' . singleQuote('CA') . ' TARGET, 0 LINE_TYPE, 7 SEQNUM, ' . singleQuote('USR') . ' JE_SOURCE, ' . singleQuote('CA') . ' GLACCOUNTCODE, ' . singleQuote('AS') . ' GL_TYPE, ' . singleQuote('CA') . ' TRUST_ITEM_TYPE, ' . singleQuote('SETT') . ' ITEM_CODE, ' . singleQuote('NA') . ' ITEM_REF_NO, CURRENCY_CODE, EXCHANGE_RT, CUR_OPERATOR, CHART_FIELD01, CHART_FIELD02, ' . singleQuote('NA') . ' CHART_FIELD03, ' . singleQuote('NA') . ' CHART_FIELD04, -1*SUM(DEBIT_TRANS_AMT-CREDIT_TRANS_AMT) DEBIT_TRANS_AMT, 0  CREDIT_TRANS_AMT, -1*SUM(DEBITS-CREDITS) DEBIT_AMT, 0 CREDIT_AMT FROM GL_BALANCES_TMP WHERE BATCH_ID = ' . singleQuote($this->batch_id) . ' AND ((GL_TYPE = ' . singleQuote('AS') . ' AND TRUST_ITEM_TYPE IN (' . singleQuoteArray(array('AR', 'ARI', 'IN', 'CR', 'CA', 'CA','INDV','CD')) . ')) OR GL_TYPE = ' . singleQuote('LI') . ') AND JE_SOURCE = ' . singleQuote('USR') . ' GROUP BY CURRENCY_CODE, EXCHANGE_RT, CUR_OPERATOR, CHART_FIELD01, CHART_FIELD02';
            ClxCommon::$db->executeQuery($sql);
            /* Creando diferencia de patrimonio */
            $sql = 'INSERT INTO SETT_OPERATION_TMP (TARGET, LINE_TYPE, SEQNUM, JE_SOURCE, GLACCOUNTCODE, GL_TYPE, TRUST_ITEM_TYPE, ITEM_CODE, ITEM_REF_NO, CURRENCY_CODE, EXCHANGE_RT, CUR_OPERATOR, CHART_FIELD01, CHART_FIELD02, CHART_FIELD03, CHART_FIELD04, DEBIT_TRANS_AMT, CREDIT_TRANS_AMT, DEBIT_AMT, CREDIT_AMT) SELECT ' . singleQuote('EQ') . ' TARGET, 0 LINE_TYPE, 8 SEQNUM, ' . singleQuote('USR') . ' JE_SOURCE, ' . singleQuote('EQ') . ' GLACCOUNTCODE, ' . singleQuote('EQ') . ' GL_TYPE, ' . singleQuote('EQ') . ' TRUST_ITEM_TYPE, ' . singleQuote('SETT') . ' ITEM_CODE, ' . singleQuote('NA') . ' ITEM_REF_NO, CURRENCY_CODE, EXCHANGE_RT, CUR_OPERATOR, CHART_FIELD01, CHART_FIELD02, ' . singleQuote('NA') . ' CHART_FIELD03, ' . singleQuote('NA') . ' CHART_FIELD04, -1*SUM(DEBIT_TRANS_AMT-CREDIT_TRANS_AMT) DEBIT_TRANS_AMT, 0 CREDIT_TRANS_AMT, -1*SUM(DEBITS-CREDITS) DEBIT_AMT, 0 CREDIT_AMT FROM GL_BALANCES_TMP WHERE BATCH_ID = ' . singleQuote($this->batch_id) . ' AND ((GL_TYPE = ' . singleQuote('AS') . ' AND TRUST_ITEM_TYPE NOT IN (' . singleQuoteArray(array('CA', 'AR', 'ARI', 'IN', 'CR', 'CA','INDV','CD')) . ')) OR GL_TYPE IN (' . singleQuote('IN') . ',' . singleQuote('EX') . ', ' . singleQuote('EQ') . ')) GROUP BY EXCHANGE_RT, CUR_OPERATOR, CURRENCY_CODE, CHART_FIELD01, CHART_FIELD02';
            ClxCommon::$db->executeQuery($sql);
            // Insertando lineas valorizacion ESTO EN REALIZADA AFECTA LA MONEDA NACIONAL
            $sql = 'INSERT INTO SETT_OPERATION_TMP (TARGET, LINE_TYPE, SEQNUM, JE_SOURCE, GLACCOUNTCODE, GL_TYPE, TRUST_ITEM_TYPE, ITEM_CODE, ITEM_REF_NO, CURRENCY_CODE, EXCHANGE_RT, CUR_OPERATOR, CHART_FIELD01, CHART_FIELD02, CHART_FIELD03, CHART_FIELD04, DEBIT_TRANS_AMT, CREDIT_TRANS_AMT, DEBIT_AMT, CREDIT_AMT) SELECT ' . singleQuote('VAL') . ' TARGET, 0 LINE_TYPE, 9 SEQNUM, JE_SOURCE, ' . singleQuote('CA') . ' GLACCOUNTCODE, ' . singleQuote('EQ') . ' GL_TYPE, ' . singleQuote('EQ') . ' TRUST_ITEM_TYPE, ' . singleQuote('SETT') . ' ITEM_CODE, ' . singleQuote('NA') . ' ITEM_REF_NO, BASE_CURRENCY CURRENCY_CODE, 1 EXCHANGE_RT, 1 CUR_OPERATOR, CHART_FIELD01, CHART_FIELD02, ' . singleQuote('NA') . ' CHART_FIELD03, ' . singleQuote('NA') . ' CHART_FIELD04, -1*SUM(DEBIT_TRANS_AMT-CREDIT_TRANS_AMT) DEBIT_TRANS_AMT, 0 CREDIT_TRANS_AMT, -1*SUM(DEBITS-CREDITS) DEBIT_AMT, 0 CREDIT_AMT FROM GL_BALANCES_TMP WHERE BATCH_ID = ' . singleQuote($this->batch_id) . ' AND JE_SOURCE = ' . singleQuote('EXVL') . ' GROUP BY JE_SOURCE, BASE_CURRENCY, CHART_FIELD01, CHART_FIELD02';
            ClxCommon::$db->executeQuery($sql);
            //Incorporando diferencia entre activo y pasivo contra capital para balancear moneda extrangera
            $sql = 'INSERT INTO SETT_OPERATION_TMP (TARGET, LINE_TYPE, SEQNUM, JE_SOURCE, GLACCOUNTCODE, GL_TYPE, TRUST_ITEM_TYPE, ITEM_CODE, ITEM_REF_NO, CURRENCY_CODE, EXCHANGE_RT, CUR_OPERATOR, CHART_FIELD01, CHART_FIELD02, CHART_FIELD03, CHART_FIELD04, DEBIT_TRANS_AMT, CREDIT_TRANS_AMT, DEBIT_AMT, CREDIT_AMT) SELECT ' . singleQuote('EQ') . ' TARGET, 0 LINE_TYPE, 10 SEQNUM, JE_SOURCE, ' . singleQuote('EQ') . ' GLACCOUNTCODE, ' . singleQuote('EQ') . ' GL_TYPE, ' . singleQuote('EQ') . ' TRUST_ITEM_TYPE, ' . singleQuote('SETT') . ' ITEM_CODE, ' . singleQuote('NA') . ' ITEM_REF_NO, CURRENCY_CODE, EXCHANGE_RT, CUR_OPERATOR, CHART_FIELD01, CHART_FIELD02, ' . singleQuote('NA') . ' CHART_FIELD03, ' . singleQuote('NA') . ' CHART_FIELD04, SUM(DEBIT_TRANS_AMT-CREDIT_TRANS_AMT) DEBIT_TRANS_AMT, 0 CREDIT_TRANS_AMT, SUM(DEBITS-CREDITS) DEBIT_AMT, 0 CREDIT_AMT FROM GL_BALANCES_TMP WHERE BATCH_ID = ' . singleQuote($this->batch_id) . ' AND CURRENCY_CODE <> BASE_CURRENCY AND GL_TYPE IN (' . singleQuote('AS') . ',' . singleQuote('LI') . ',' . singleQuote('IN') . ',' . singleQuote('EX') . ',' . singleQuote('EQ') . ') AND JE_SOURCE = ' . singleQuote('USR') . ' GROUP BY JE_SOURCE, CURRENCY_CODE, EXCHANGE_RT, CUR_OPERATOR, CHART_FIELD01, CHART_FIELD02';
            ClxCommon::$db->executeQuery($sql);
            $sql = 'INSERT INTO SETT_OPERATION_TMP (TARGET, LINE_TYPE, SEQNUM, JE_SOURCE, GLACCOUNTCODE, GL_TYPE, TRUST_ITEM_TYPE, ITEM_CODE, ITEM_REF_NO, CURRENCY_CODE, EXCHANGE_RT, CUR_OPERATOR, CHART_FIELD01, CHART_FIELD02, CHART_FIELD03, CHART_FIELD04, DEBIT_TRANS_AMT, CREDIT_TRANS_AMT, DEBIT_AMT, CREDIT_AMT) SELECT ' . singleQuote('EQ') . ' TARGET, 0 LINE_TYPE, 11 SEQNUM, ' . singleQuote('USR') . ' JE_SOURCE, ' . singleQuote('EQ') . ' GLACCOUNTCODE, ' . singleQuote('EQ') . ' GL_TYPE, ' . singleQuote('EQ') . ' TRUST_ITEM_TYPE, ' . singleQuote('SETT') . ' ITEM_CODE, ' . singleQuote('NA') . ' ITEM_REF_NO, BASE_CURRENCY CURRENCY_CODE, 1 EXCHANGE_RT, 1 CUR_OPERATOR, CHART_FIELD01, CHART_FIELD02, ' . singleQuote('NA') . ' CHART_FIELD03, ' . singleQuote('NA') . ' CHART_FIELD04, -1*SUM(DEBITS-CREDITS) DEBIT_TRANS_AMT, 0 CREDIT_TRANS_AMT, -1*SUM(DEBITS-CREDITS) DEBIT_AMT, 0 CREDIT_AMT FROM GL_BALANCES_TMP WHERE 1=1 AND BATCH_ID = ' . singleQuote($this->batch_id) . ' AND CURRENCY_CODE <> BASE_CURRENCY AND GL_TYPE IN (' . singleQuote('AS') . ',' . singleQuote('LI') . ',' . singleQuote('IN') . ',' . singleQuote('EX') . ',' . singleQuote('EQ') . ') AND JE_SOURCE = ' . singleQuote('USR') . ' GROUP BY BASE_CURRENCY, CHART_FIELD01, CHART_FIELD02';
            ClxCommon::$db->executeQuery($sql);
            //Coloando movimiento en debit o credito de acuerdo a la cuenta contable
            $sql = 'UPDATE SETT_OPERATION_TMP SET CREDIT_AMT = -1*DEBIT_AMT, DEBIT_AMT = 0 WHERE DEBIT_AMT < 0';
            ClxCommon::$db->executeQuery($sql);
            $sql = 'UPDATE SETT_OPERATION_TMP SET CREDIT_TRANS_AMT = -1*DEBIT_TRANS_AMT, DEBIT_TRANS_AMT = 0 WHERE DEBIT_TRANS_AMT < 0';
            ClxCommon::$db->executeQuery($sql);
            //Asignando cuenta de patrimonio para salida final
            $sql = 'UPDATE SETT_OPERATION_TMP OT, FIDU_FIPA_TMP PT SET OT.GLACCOUNTCODE = PT.EQ_GL_CODE WHERE OT.CHART_FIELD01 = PT.CHART_FIELD01 AND OT.GLACCOUNTCODE = ' . singleQuote('EQ') . '';
            ClxCommon::$db->executeQuery($sql);
            //Asignando cuenta de banco por defecto para salida final
            $sql = 'UPDATE SETT_OPERATION_TMP OT, (SELECT FIAS.CONTRACT_CODE CHART_FIELD01, FIBK.CURRENCY_CODE, PRAT.PRO_GL_CODE GLACCOUNTCODE, FIAS.IDCODE CHART_FIELD03 FROM CRM_PRODUCTS PROD, GL_PROD_PRMTR_TBL PRAT, FIDU_ITEMS FIAS, FIDU_BANK_ACCOUNTS FIBK, FIDU_FIPA_TMP PT WHERE FIAS.CONTRACT_CODE = PT.CHART_FIELD01 AND PROD.IDCODE = PRAT.PRODUCT_CODE AND PRAT.PRODUCT_CODE = FIBK.BANK_PROD_CODE AND PRAT.PROD_STATUS = ' . singleQuote('VIG') . ' AND FIAS.FIDU_BANKACT_CODE = FIBK.BANK_ACCOUNT_NO AND PROD.MODULECODE = ' . singleQuote('CASH') . ') PT SET OT.GLACCOUNTCODE = PT.GLACCOUNTCODE, OT.CHART_FIELD03 = PT.CHART_FIELD03 WHERE OT.CHART_FIELD01 = PT.CHART_FIELD01 AND OT.CURRENCY_CODE = PT.CURRENCY_CODE AND OT.GLACCOUNTCODE = ' . singleQuote('CA') . '';
            ClxCommon::$db->executeQuery($sql);
            /* JUST IN case CLEAR ALL IN 0 ENTRIES */
            $sql = 'DELETE FROM SETT_OPERATION_TMP WHERE DEBIT_AMT = 0 AND CREDIT_AMT = 0 AND DEBIT_TRANS_AMT = 0 AND CREDIT_TRANS_AMT = 0';
            ClxCommon::$db->executeQuery($sql);
        }

    }

}