<?php

/**
 * General Code File
 * For more information contact  support@alenet.com
 *
 * @author       $Author: fsantana $
 * @copyright    Alenet, Inc.
 * @since        $Date: 2015-09-25 18:47:34 +0000 (Fri, 25 Sep 2015) $
 * @version        $Rev: 7014 $
 *
 * CHANGE LOG
 *  Included svn prop
 */
class Amds {

    /**
     * Constructor
     * @param array [$procParameters]
     */
    function __construct($procParameters = array()) {
        multiTableName($this, array('AMDS', 'FIAE', 'PROD', 'PRAT', 'GLAC'));
        $this->batch_id = (!empty($procParameters['BATCH_ID'])) ? $procParameters['BATCH_ID'] : batch_id();
        $this->username = $_SESSION['user_record']['username'];
        $this->now = xFormat::getNow('Y-m-d H:i:s');
        $this->today = xFormat::getNow('Y-m-d');
        $this->je_source = '';
    }

    /**
     * Initialice fiat form - Executed on init event
     *
     * @param unknown_type $record
     */
    function init_form(&$record) {
        $r = $this->get_asset_record($record['AMDS_ASSETCODE'], $this->today);
        if (!empty($r)) {
            $record['AMDS_DEPRECIATION_METHOD'] = $r['DEPRECIATION_METHOD'];
            $record['AMDS_SALVAGE_VALUE'] = $r['SALVAGE_VALUE'];
            $record['AMDS_EFFTDATE'] = $r['CLX_EFFTDATE'];
        }
    }

    /**
     * Validate record information
     * Executed on table validate
     *
     * @param array $record
     * @param array $msg
     * @return boolean
     */
    function validate_form(&$record, &$msg) {
        //check that only manual depreciation could be modified
        $r = $this->get_asset_record($record['AMDS_ASSETCODE'], $this->today);
        if ($r['DEPRECIATION_METHOD'] <> '99') {
            $msg['AMDS_ASSETCODE'] = smsg('MSG-259');
        }
    }

    /**
     * Build Asset depreciation schedule
     * @param $asset_code
     * @param $efftdate
     */
    function build_depreciation($asset_code, $efftdate) {
        //first copy previous is no current amortization exists
        $qty = ClxCommon::$db->oneExistsRecords($this->amdsname, 'ASSETCODE = ' . singleQuote($asset_code) . ' AND EFFTDATE = ' . singleQuote($efftdate));
        if ($qty == CLX_NUM_ZERO) {
            $sql = 'SELECT * FROM ' . $this->amdsname . ' AMDS WHERE AMDS.ASSETCODE = ' . singleQuote($asset_code) . ' AND AMDS.EFFTDATE = (SELECT MAX(X.EFFTDATE) FROM ' . $this->amdsname . ' X WHERE X.ASSETCODE = AMDS.ASSETCODE AND X.EFFTDATE <' . singleQuote($efftdate) . ') ORDER BY AMDS.SEQNUM';
            $rs = ClxCommon::$db->executeQuery($sql);
            while ($r = ClxCommon::$db->getNextRecord($rs, ASSOCIATIVE)) {
                unset($r['ID']);
                $r['EFFTDATE'] = $efftdate;
                ClxCommon::$db->loadDbRecord($this->amdsname, $r, false, 'ASSETCODE = ' . singleQuote($r['ASSETCODE']) . ' AND EFFTDATE = ' . singleQuote($r['EFFTDATE']) . ' AND SEQNUM = ' . singleQuote($r['SEQNUM']));
            }
        }
        /* mark all current */
        $sql = 'UPDATE ' . $this->amdsname . '  SET IS_REMOVE = 1 WHERE ASSETCODE = ' . singleQuote($asset_code) . ' AND EFFTDATE = ' . singleQuote($efftdate);
        ClxCommon::$db->executeQuery($sql);
        //Calculate formula and update balances.
        $fiaerec = $this->get_asset_record($asset_code, $efftdate);
        if ($fiaerec['USEFUL_LIFE'] > CLX_NUM_ZERO) {
            $DBTB = &getTool('DBTB');
            $amdsfls = $DBTB->tableDefaultRecord('AMDS');
            $amdsfls['CREATEDBY'] = $this->username;
            $amdsfls['CDATE'] = $this->now;
            $amdsfls['EFFTDATE'] = $efftdate;
            $amdsfls['ASSETCODE'] = $asset_code;
            $bv = bcsub($fiaerec['PURCHASE_AMT'], $fiaerec['INIT_ACC_DEPRECIATION'], CLX_NUM_TWO);
            $sdate = (!empty($fiaerec['DEPRECIATION_SDATE'])) ? $fiaerec['DEPRECIATION_SDATE'] : $fiaerec['PURCHASE_DATE'];
            $per = $this->get_no_periods($fiaerec['USEFUL_LIFE'], $fiaerec['USEFUL_LIFE_UNIT'], $fiaerec['DEPRECIATION_FREQ']);
            $balamt = CLX_INIT_ZERO;
            for ($i = CLX_NUM_ONE; $i <= $per; $i++) {
                $r = ClxCommon::$db->getRecord($this->amdsname, 'ASSETCODE = ' . singleQuote($asset_code) . ' AND EFFTDATE = ' . singleQuote($efftdate) . ' AND SEQNUM = ' . $i);
                if (empty($r))
                    $r = $amdsfls;
                $r['PERIOD_START_DATE'] = $sdate;
                //dates
                switch ($fiaerec['DEPRECIATION_FREQ']) {
                    case '12.0':
                        if ($i == CLX_NUM_ONE) {//first month partial
                            $sdate = adodb_next_day(adodb_month_lastday($sdate, 'Y-m-d'));
                        } else {
                            $sdate = adodb_period_date($sdate, 'M', CLX_NUM_ONE, 'Y-m-d');
                        }
                        break;
                    case '12'://Monthly
                        $sdate = adodb_period_date($sdate, 'M', CLX_NUM_ONE, 'Y-m-d');
                        break;
                    case '6'://Bimonthly(6)
                        $sdate = adodb_period_date($sdate, 'M', CLX_NUM_TWO, 'Y-m-d');
                        break;
                    case '4'://Quarterly(4)
                        $sdate = adodb_period_date($sdate, 'M', CLX_NUM_THREE, 'Y-m-d');
                        break;
                    case '3'://Four-monthly 1/3(3)
                        $sdate = adodb_period_date($sdate, 'M', CLX_NUM_FOUR, 'Y-m-d');
                        break;
                    case '2'://Semiannually(2)
                        $sdate = adodb_period_date($sdate, 'M', CLX_NUM_SIX, 'Y-m-d');
                        break;
                    case '1'://Annually(1)
                        $sdate = adodb_next_date($sdate, 'Y');
                        break;
                    default:
                        $sdate = 'null';
                        break;
                }
                if ($fiaerec['DEPRECIATION_AMT'] > CLX_NUM_ZERO) {
                    $dpramt = $fiaerec['DEPRECIATION_AMT'];
                } elseif ($fiaerec['DEPRECIATION_RT'] > CLX_NUM_ZERO) {
                    $dpramt = round($bv * ($fiaerec['DEPRECIATION_RT'] / CLX_NUM_HUNDRED), CLX_NUM_TWO);
                } else {
                    $dpramt = CLX_INIT_ZERO;
                }
                //Calculate partial depresiation
                if ($i == CLX_NUM_ONE && $fiaerec['DEPRECIATION_FREQ'] === '12.0') {
                    $lastday = adodb_month_lastday($r['PERIOD_START_DATE'], 'Y-m-d');
                    $days_of_month = day_diff($lastday, adodb_month_lastday($r['PERIOD_START_DATE'], 'Y-m-01')) + CLX_NUM_ONE;
                    $no_of_days = day_diff($lastday, $r['PERIOD_START_DATE']) + CLX_NUM_ONE;
                    $dpramt = round(($dpramt / $days_of_month) * $no_of_days, CLX_NUM_TWO);
                }
                if (($bv - $fiaerec['SALVAGE_VALUE']) < $dpramt) {
                    $dpramt = $bv - $fiaerec['SALVAGE_VALUE'];
                }
                if ($bv == CLX_NUM_ZERO)
                    continue;
                $balamt += $dpramt;
                $r['MODIFIEDBY'] = $this->username;
                $r['MDATE'] = $this->now;
                $r['IS_REMOVE'] = CLX_INIT_ZERO;
                $r['SEQNUM'] = $i;
                $r['PERIOD_END_DATE'] = adodb_prev_day($sdate);
                $r['BOOK_VALUE'] = $bv;
                $r['DEPRECIATION_AMT'] = $dpramt;
                $r['DEPRECIATION_RT'] = $fiaerec['DEPRECIATION_RT'];
                $r['ACCU_DEPRTION_AMT'] = $balamt;
                $bv -= $dpramt;
                if ($i == $per) {
                    $dif = round($bv - $fiaerec['SALVAGE_VALUE'], CLX_NUM_TWO);
                    if ($dif <> CLX_NUM_ZERO) {
                        $r['DEPRECIATION_AMT'] += $dif;
                        $r['ACCU_DEPRTION_AMT'] += $dif;
                    }
                }
                ClxCommon::$db->loadDbRecord($this->amdsname, $r, false, 'ASSETCODE = ' . singleQuote($r['ASSETCODE']) . ' AND EFFTDATE = ' . singleQuote($r['EFFTDATE']) . ' AND SEQNUM = ' . singleQuote($r['SEQNUM']));
                if ($bv == $fiaerec['SALVAGE_VALUE']) {
                    break;
                }
            }
        }
        $sql = 'DELETE FROM ' . $this->amdsname . ' WHERE IS_REMOVE = 1 AND ASSETCODE = ' . singleQuote($asset_code) . ' AND EFFTDATE = ' . singleQuote($efftdate);
        ClxCommon::$db->executeQuery($sql);
    }

    /**
     * Calculates the number of period based on depreciation paramteres
     * @param int $useful_life
     * @param string $useful_life_unit
     * @param int $depreciation_freq
     */
    private function get_no_periods($useful_life, $useful_life_unit, $depreciation_freq) {
        $per = CLX_INIT_ZERO;
        $periodfactor = CLX_INIT_ZERO;
        if (!is_numeric($useful_life))
            $useful_life = CLX_INIT_ZERO;
        switch ($useful_life_unit) {
            case 'D': $periodfactor = CLX_360_DAYS;
                break;
            case 'W': $periodfactor = CLX_NUM_52;
                break;
            case 'M': $periodfactor = CLX_NUM_12;
                break;
            case 'Q': $periodfactor = CLX_NUM_FOUR;
                break;
            case 'S': $periodfactor = CLX_NUM_TWO;
                break;
            case 'Y': $periodfactor = CLX_NUM_ONE;
                break;
            default: $periodfactor = CLX_INIT_ZERO;
                break;
        }
        if ($periodfactor > CLX_NUM_ZERO) {
            $per = $useful_life * ($depreciation_freq / $periodfactor);
        }
        if ($depreciation_freq == CLX_NUM_99)
            $per = CLX_NUM_ONE;
        return $per;
    }

    /**
     * Returns current asset record if no efftdate is set today will be assumed
     * @param $asset_code
     * @param $efftdate
     */
    function get_asset_record($asset_code, $efftdate = '') {
        if ($efftdate === '')
            $efftdate = $this->today;
        $sql = 'SELECT FIAE.* FROM ' . $this->fiaename . ' FIAE WHERE FIAE.IDCODE = ' . singleQuote($asset_code) . ' AND FIAE.CLX_EFFTDATE = (SELECT MAX(CLX_EFFTDATE) FROM ' . $this->fiaename . ' X WHERE FIAE.IDCODE = X.IDCODE AND  X.CLX_EFFTDATE<=' . singleQuote($efftdate) . ')';
        return ClxCommon::$db->getOneRecord($sql);
    }

}
