<?php

/**
 * General Code File
 * For more information contact  support@alenet.com
 *
 * @author       $Author: jclerma $
 * @copyright    Alenet, Inc.
 * @since        $Date: 2018-02-05 07:57:46 -0500 (Mon, 05 Feb 2018) $
 * @version        $Rev: 9713 $
 *
 * CHANGE LOG
 *  Included svn prop
 */
class Comp {

    function __construct($procParameters = array()) {
        multiTableName($this, array('COMP', 'GLCP', 'JACP', 'CURR', 'RUCO'));
        $this->batch_id = (!empty($procParameters ['BATCH_ID'])) ? $procParameters ['BATCH_ID'] : batch_id();
        $this->username = $_SESSION ['user_record'] ['username'];
        $this->now = xFormat::getNow('Y-m-d H:i:s');
    }

    /**
     * Initialice comp form
     * Executed on init event
     */
    function comp_init_form(&$record) {
	ClxFreeReturn($record);
    }

// end function
    /**
     * Validate record information
     * Executed on table validate
     */
    function comp_validate_form(&$record, &$msg) {
	ClxFreeReturn($record);
	ClxFreeReturn($msg);
    }

// end function
    /**
     * Parametros iniciales de compania utilizados por diferentes funciones de
     * inicializacion
     */
    function comp_operation_parameters($comp_code = '') {
        if ($comp_code === '' && !empty($_SESSION ['prefs'] ['UIFF_COMP_CODE'])) {
            $comp_code = $_SESSION ['prefs'] ['UIFF_COMP_CODE'];
        }
        $sql = 'SELECT COMP.LEGAL_NAME, COMP.CURRENCY_CODE, COMP.IDCODE, GLCP.DEF_LGGROUP_CODE LGGROUP_CODE, GLCP.DEF_LEDGER_CODE LEDGER_CODE, GLCP.*, COMP.COUNTRY FROM ' . $this->compname . ' COMP, ' . $this->glcpname . ' GLCP WHERE COMP.IDCODE = GLCP.COMP_CODE ';
        if ($comp_code !== '')
            $sql .= ' AND COMP.IDCODE = ' . singleQuote($comp_code);
        $return = ClxCommon::$db->getOneRecord($sql);
        if (!empty($return)) {
            if (!empty($_SESSION ['prefs'] ['UIFF_CURRENCY_CODE'])) {
                $return ['CURRENCY_CODE'] = $_SESSION ['prefs'] ['UIFF_CURRENCY_CODE'];
            }
            $return ['FISCAL_PERIOD'] = $this->comp_fiscal_period($return ['IDCODE'], $return ['OPERATION_DATE']);
        }
        return $return;
    }

// end function
    /**
     * Returns default ledger group for company
     *
     * @param string $comp_code
     * @return string
     */
    function comp_lggroup_code($comp_code = '') {
        $sql = 'SELECT DEF_LGGROUP_CODE FROM ' . $this->glcpname;
        if ($comp_code !== '')
            $sql .= 'WHERE COMP_CODE = ' . singleQuote($comp_code);
        $return = ClxCommon::$db->getOne($sql);
        if (empty($return))
            $return = 'AC';
        return $return;
    }

    /**
     * Returns default ledger for company
     *
     * @param string $comp_code
     * @return string
     */
    function comp_ledger_code($comp_code = '') {
        $sql = 'SELECT DEF_LEDGER_CODE FROM ' . $this->glcpname;
        if ($comp_code !== '')
            $sql .= 'WHERE COMP_CODE = ' . singleQuote($comp_code);
        $return = ClxCommon::$db->getOne($sql);
        if (empty($return))
            $return = 'AC';
        return $return;
    }

    /**
     * Returns company operation date
     *
     * @param string $comp_code
     * @return string
     */
    function comp_operation_date($comp_code = '') {
        $comprec = $this->comp_operation_parameters($comp_code);
        return $comprec ['OPERATION_DATE'];
    }

    function comp_fiscal_period($comp_code, $efftdate) {
        $sql = 'SELECT IDCODE FROM ' . $this->jacpname . ' WHERE COMP_CODE = ' . singleQuote($comp_code) . ' AND FROM_DATE <=' . singleQuote($efftdate . ' ') . ' AND TO_DATE >= ' . singleQuote($efftdate);
        return ClxCommon::$db->getOne($sql);
    }

    /**
     * Return company current exchange rate
     *
     * @param string $comp_code
     * @param string $currency_code
     * @param string $efftdate
     * @return array
     */
    function comp_current_exchange($comp_code, $currency_code, $efftdate, $tobase = true) {
        $sql = 'SELECT COMP.CURRENCY_CODE FROM ' . $this->compname . ' COMP WHERE COMP.IDCODE = ' . singleQuote($comp_code);
        $comp_currency = ClxCommon::$db->getOne($sql);
        if ($comp_currency == $currency_code) {
            $return = array('EFFTDATE' => $efftdate, 'BASE_CURRENCY' => $currency_code, 'FOREIGN_CURRENCY' => $currency_code, 'EXCHANGE_RT' => CLX_NUM_ONE, 'CUR_OPERATOR' => CLX_NUM_ONE,  'COMP_CURRENCY' => $comp_currency );
        } else {
            $crit = ' AND CURR.BASE_CURRENCY = COMP.CURRENCY_CODE AND CURR.FOREIGN_CURRENCY = ' . singleQuote($currency_code) . ' ';
            if ($tobase === true) {
                $crit = 'AND CURR.BASE_CURRENCY = ' . singleQuote($currency_code) . ' AND CURR.FOREIGN_CURRENCY = COMP.CURRENCY_CODE';
            }
            $sql = 'SELECT MAX(CURR.CLX_EFFTDATE) EFFTDATE, CURR.BASE_CURRENCY, CURR.FOREIGN_CURRENCY, CURR.EXCHANGE_RT, CURR.CUR_OPERATOR, COMP.CURRENCY_CODE COMP_CURRENCY FROM ' . $this->currname . ' CURR, ' . $this->compname . ' COMP WHERE COMP.IDCODE = ' . singleQuote($comp_code) . ' ' . $crit . ' AND CURR.CLX_EFFTDATE=(SELECT MAX(CURRX.CLX_EFFTDATE) FROM ' . $this->currname . ' CURRX WHERE CURRX.IDCODE = CURR.IDCODE AND CURRX.CLX_EFFTDATE <=' . singleQuote($efftdate) . ') GROUP BY CURR.BASE_CURRENCY, CURR.FOREIGN_CURRENCY, CURR.EXCHANGE_RT, CURR.CUR_OPERATOR';
            $return = ClxCommon::$db->getOneRecord($sql);
            if (empty($return)) {
                $return = array('EFFTDATE' => $efftdate, 'BASE_CURRENCY' => $currency_code, 'FOREIGN_CURRENCY' => $comp_currency, 'EXCHANGE_RT' => CLX_NUM_ZERO, 'CUR_OPERATOR' => CLX_NUM_ONE,  'COMP_CURRENCY' => $comp_currency );
            }
        }
        return $return;
    }

    /**
     * Enter description here...
     *
     * @param unknown_type $comp_code
     * @return unknown
     */
    function logo($comp_code = '') {
        if ($comp_code === '') {
            $comp_code = ClxCommon::$db->getField($this->compname, 'COMP_LOGO');
        }
        $OBJS = &getTool('OBJS');
        $fieldinstance = &$OBJS->getObject_('DBFT', 'FILE');
        $fieldrec = ClxCommon::$db->getRecord('CLX_DB_FIELDS', 'FIELDNAME=' . singleQuote('COMP_LOGO') . '');
        $comprec = ClxCommon::$db->getRecord($this->compname, 'IDCODE = ' . singleQuote($comp_code));
        return $fieldinstance->formatField('COMP_LOGO', $fieldrec, $comprec);
    }

    /**
     * Check Run Control status for selected process
     * @param string $comp_code
     * @param string $je_source
     */
    function ruco_status($comp_code, $je_source) {
        $sql = 'SELECT COUNT(ID) FROM ' . $this->ruconame . ' WHERE COMP_CODE = ' . singleQuote($comp_code) . ' AND JE_SOURCE = ' . singleQuote($je_source) . ' AND STATUS = ' . singleQuote('I') . '';
        return (ClxCommon::$db->getOne($sql) == CLX_NUM_ZERO) ? true : false;
    }

    /**
     * List Companies
     * @param string $field
     * @param string $creteria
     */
    function list_companies($field = "IDCODE", $criteria = "") {
        $comp = array();

        $companies = ClxCommon::$db->getRecords($this->compname, $criteria);
        while (($company = ClxCommon::$db->getNextRecord($companies))) {
            $comp[] = $company[$field];
        }

        return $comp;
    }
    
}