<?php

/**
 * General Code File
 * For more information contact  support@alenet.com
 *
 * @author       $Author: jclerma $
 * @copyright    Alenet, Inc.
 * @since        $Date: 2017-12-01 09:21:47 -0500 (vie, 01 dic 2017) $
 * @version        $Rev: 9645 $
 *
 * CHANGE LOG
 *  Included svn prop
 */
class Curr {

    /**
     * Contructor
     *
     * @param array $procParameters
     * @return void
     */
    function __construct($procParameters = array()) {
        multiTableName($this, array('CURR', 'COMP', 'GLCP'));
        $this->batch_id = (!empty($procParameters['BATCH_ID'])) ? $procParameters['BATCH_ID'] : batch_id();
        $this->username = $_SESSION['user_record']['username'];
        $this->now = xFormat::getNow('Y-m-d H:i:s');
        $this->today = xFormat::getNow('Y-m-d');
    }

    /**
     * Initialice CURR form
     * Executed on init event
     */
    function curr_init_form(&$record) {
        if (UserRequest::$request['ac'] === 'CREATE' || UserRequest::$request['ac'] === 'DUPLICATE') {
            $record ['CURR_IDCODE'] = 'AUTO';
            $record ['CURR_CLX_EFFTDATE'] = $this->today;
        }
    }

    /**
     * Return currency exchage information
     *
     * @param string $currency_code
     * @param string $base_currency
     * @param string $efftdate
     * @return array
     */
    function get_convertion_info($currency_code, $base_currency, $efftdate) {
        $sql = 'SELECT MAX(CLX_EFFTDATE) EFFTDATE, BASE_CURRENCY, FOREIGN_CURRENCY, EXCHANGE_RT, CUR_OPERATOR FROM ' . $this->currname . ' WHERE BASE_CURRENCY= ' . singleQuote($base_currency) . ' AND FOREIGN_CURRENCY= ' . singleQuote($currency_code) . ' AND CLX_EFFTDATE<=' . singleQuote($efftdate) . ' GROUP BY BASE_CURRENCY, FOREIGN_CURRENCY, EXCHANGE_RT, CUR_OPERATOR';
        return ClxCommon::$db->getOneRecord($sql);
    }

    /**
     * Base Conversion utilities
     * version: 1.0.0
     * author: Alenet, Inc.
     * date: 6/10/2005
     */
    function convertion($from, $to = 'usa', $amount = CLX_NUM_ONE) {
        if (strlen($amount) == CLX_NUM_ZERO)
            $amount = 'NAN';
        $result = CLX_INIT_ZERO;
        // Array of parameters
        $service = 'http://www.xmethods.net/sd/CurrencyExchangeService.wsdl';
        $parameters = array('country1' => $from, 'country2' => $to);
        if (file($service) !== false) {
            // Define new object and specify location of wsdl file.
            $soapclient = new SoapClient($service, 'wsdl');
            // call the method and get the result.
            $result = $soapclient->__soapCall('getRate', $parameters);
            if (!is_array($result))
                $result = ($amount * $result);
            else
                $result = CLX_INIT_ZERO;
        }
        return $result;
    }

    /**
     * Get Rates from socket.
     */
    function get_rates() {
        $tableinfo = ClxCommon::$db->getRecord('CLX_DB_TABLES', 'IDCODE=' . singleQuote('CURR') . '');
        $tablename = $tableinfo['MODULECODE'] . '_' . $tableinfo['IDNAME'];
        $conversions = array();
        $conversions[] = array('FROM' => 'usa', 'TO' => 'venezuela', 'IDCODE' => 'VEB-USD', 'IDNAME' => 'Bolivares - Dollar', 'F' => 'VEB', 'T' => 'USD');
        $conversions[] = array('FROM' => 'usa', 'TO' => 'euro', 'IDCODE' => 'EUR-USD', 'IDNAME' => 'Euro - Dollar', 'F' => 'EUR', 'T' => 'USD');
        $conversions[] = array('FROM' => 'usa', 'TO' => 'mexico', 'IDCODE' => 'MXN-USD', 'IDNAME' => 'Peso Mexicano - Dollar', 'F' => 'MXN', 'T' => 'USD');
        $d = date('Y-m-d');
        reset($conversions);
        foreach ($conversions as $key => $val) {
            if (ClxCommon::$db->countRecords($tablename, 'IDCODE=' . singleQuote($val['IDCODE']) . ' AND CLX_EFFTDATE=' . singleQuote($d)) == CLX_NUM_ZERO) {
                $record = array();
                $record['IDCODE'] = $val['IDCODE'];
                $record['CLX_EFFTDATE'] = $d;
                $record['IDNAME'] = $val['IDNAME'];
                $record['BASE_CURRENCY'] = $val['F'];
                $record['FOREIGN_CURRENCY'] = $val['T'];
                $record['EXCHANGE_RT'] = $this->convertion($val['FROM'], $val['TO'], CLX_NUM_ONE);
                ClxCommon::$db->createRecord($tablename, $record, false);
            }
        }
    }

    function prepareCurrRecord(&$record) {
        $record['IDCODE'] = $record['BASE_CURRENCY'] . '-' . $record['FOREIGN_CURRENCY'];
    }

    /**
     * IBS Currency Exchange File Loader
     * Function will extract the flat file and load it into
     */
    function IBS_import($filename) {
        $contenidoHTML = '';
        $contenidoHTML.= 'File:' . $filename . '<br>';
        if (file_exists($filename))
            $contenidoHTML.= 'File Found<br>';
        else
            $contenidoHTML.= 'File Invalid<br>';
        $currname = tableName('CURR');
        $lines = file($filename);
        foreach ($lines as $line_num => $line) {
            $contenidoHTML.= '<br>' . CLX_CHAR_NL . 'Line #<b>{$line_num}</b> : ' . htmlspecialchars($line) . '<br />' . CLX_CHAR_NL;
            if (empty($line))
                continue;
            if (substr($line, CLX_NUM_ZERO, CLX_NUM_TWO) <> '01') {
                $contenidoHTML.= 'Currency not from bank 01, Skeeping';
                continue;
            }
            $record = array('IDCODE' => substr($line, CLX_NUM_27, CLX_NUM_THREE) . '-' . substr($line, CLX_NUM_30, CLX_NUM_THREE),
                'CLX_EFFTDATE' => substr($line, CLX_NUM_FIVE, CLX_NUM_TEN),
                'IDNAME' => substr($line, CLX_NUM_15, CLX_NUM_12),
                'BASE_CURRENCY' => substr($line, CLX_NUM_27, CLX_NUM_THREE),
                'FOREIGN_CURRENCY' => substr($line, CLX_NUM_30, CLX_NUM_THREE),
                'EXCHANGE_RT' => substr($line, CLX_NUM_34, CLX_NUM_11),
                'CUR_OPERATOR' => substr($line, CLX_NUM_45, CLX_NUM_ONE));
            $count = ClxCommon::$db->countRecords($currname, 'IDCODE =' . singleQuote($record['IDCODE']) . ' AND CLX_EFFTDATE =' . singleQuote($record['CLX_EFFTDATE']));
            if ($count == CLX_NUM_ZERO) {
                $contenidoHTML.= 'Currency Added: ' . $record['IDCODE'] . ' ' . $record['CLX_EFFTDATE'];
                ClxCommon::$db->createRecord($currname, $record, false);
            } else {
                $contenidoHTML.= 'Currency Already Exists: ' . $record['IDCODE'] . ' ' . $record['CLX_EFFTDATE'];
            }
        }
        echo $contenidoHTML;
    }

}
