<?php

/**
 * Operation queue class
 * For more information contact  support@alenet.com
 *
 * @author       $Author: cramirez $
 * @copyright    Alenet, Inc.
 * @since        $Date: 2018-08-25 14:48:10 -0400 (Sat, 25 Aug 2018) $
 * @version     $Rev: 9909 $
 */
class Gloq {

    var $code = 'GLOP';
    var $efftdate;
    var $contract_code;
    var $participant_id;
    var $rebuild;
    var $today_only;
    var $batch_id;

    function __construct() {
        $this->datestamp = date('Y-m-d H:i:s');
        multiTableName($this, array('GLOQ', 'FIAT', 'FICO', 'INAT', 'COMP', 'INFU', 'PRSP', 'PRSC', 'FPCT', 'PRLO', 'JEHD', 'GLCP', 'FIMT'));
        $this->contract_code = '';
        $this->participant_id = '';
        $this->efftdate = '';
        $this->batch_id = batch_id();
    }

    /**
     * Adds operation for execution
     *
     * @param string $gloq_type
     * @param string $comp_code
     * @param string $closing_date
     * @param string $next_oper_date
     * @param string $trans_code
     * @param string $invtrans_code
     * @return bool
     */
    function pushQueue($gloq_type, $comp_code, $closing_date, $next_oper_date, $operation_code, $batch_id, $record_id = 0) {
        $status = false;
        $rec = array('GLOQ_TYPE' => $gloq_type, 'COMP_CODE' => $comp_code, 'CLOSING_DATE' => $closing_date, 'NEXT_OPR_DATE' => $next_oper_date, 'BATCH_ID' => $batch_id, 'BATCH_PID' => CLX_NUM_ZERO, 'CREATEDBY' => $_SESSION['user_record']['username'], 'CDATE' => xFormat::getNow('Y-m-d H:i:s'), 'REC_STATUS' => 'IDLE', 'REC_ID'=>$record_id);
        switch ($gloq_type) {
            case 'INAT':
                $rec['INVTRANS_CODE'] = $operation_code;
                break;
            case 'FIAT':
                $rec['TRANS_CODE'] = $operation_code;
                break;
            case 'FIMT':
                $rec['MASS_TRANS_CODE'] = $operation_code;
                break;
            case 'JEHD':
                $rec['JEHDR_CODE'] = $operation_code;
                break;
            default:
                break;
        }
        $flag = true;
        if($gloq_type=='GLAP' && ClxCommon::$db->oneExistsRecords($this->gloqname, 'GLOQ_TYPE = '.singleQuote($gloq_type).' AND COMP_CODE = '.singleQuote($comp_code).' AND CLOSING_DATE = '.singleQuote($closing_date).' AND REC_STATUS IN (' . singleQuote('PEND') . ', ' . singleQuote('RUN') . ') ' )>0){
            $flag = false;
        }
        if ($flag && ClxCommon::$db->createRecord($this->gloqname, $rec, false)) {
            $status = true;
        }
        return $status;
    }

    /**
     * Adds transaction for excecution
     *
     * @return void
     */
    function pushJehdListing($jehdr_code, $force = false) {
        $idlist = getIdlist();
        $status = false;
        //Al parecer esto previene que se llame si no se le da autorizar
        if (!$force && count($idlist) == CLX_NUM_ZERO)
            return $status;
        //Load queue object
        //Adding entry to queue
        $sql = 'SELECT JEHD.IDCODE, JEHD.EFFTDATE, JEHD.COMP_CODE FROM ' . $this->jehdname . ' JEHD WHERE JEHD.IDCODE = ' . singleQuote($jehdr_code) . ' ORDER BY JEHD.IDCODE ASC';
        $results = ClxCommon::$db->executeQuery($sql);
        while ($result = ClxCommon::$db->getNextRecord($results, ASSOCIATIVE)) {
            //set status to queue
            ClxCommon::$db->updateRecord($this->jehdname, array('JE_STATUS' => 'OPQU', 'CLX_AUTH_STATUS' => 'PEAP'), 'IDCODE = ' . singleQuote($result['IDCODE']));
            //add queue entry
            ClxFreeReturn($this->pushQueue('JEHD', $result['COMP_CODE'], 'null', 'null', $result['IDCODE'], 'null'));
        }
        return $status;
    }

    /**
     * Adds transaction for excecution
     *
     * @return void
     */
    function pushFimtListing($trans_code, $force = false) {
        $idlist = getIdlist();
        $status = false;
        if (!$force && count($idlist) == CLX_NUM_ZERO)
            return $status;
        //Load queue object
        //Adding entry to queue
        $sql = 'SELECT FIMT.IDCODE, FIMT.EFFTDATE, FICO.COMP_CODE FROM ' . $this->fimtname . ' FIMT, ' . $this->ficoname . ' FICO WHERE FICO.IDCODE = FIMT.CHART_FIELD01 AND FIMT.IDCODE = ' . singleQuote($trans_code) . ' ORDER BY FIMT.IDCODE ASC';
        $results = ClxCommon::$db->executeQuery($sql);
        while ($result = ClxCommon::$db->getNextRecord($results, ASSOCIATIVE)) {
            //set status to queue
            ClxCommon::$db->updateRecord($this->fimtname, array('TRANS_STATUS' => 'OPQU', 'CLX_AUTH_STATUS' => 'PEAP'), 'IDCODE = ' . singleQuote($result['IDCODE']));
            //add queue entry
            ClxFreeReturn($this->pushQueue('FIMT', $result['COMP_CODE'], 'null', 'null', $result['IDCODE'], 'null'));
        }
        return $status;
    }

    /**
     * Adds transaction for excecution
     *
     * @return void
     */
    function pushFiatListing($trans_code, $force = false) {
        $idlist = getIdlist();
        $status = false;
        if (!$force && count($idlist) == CLX_NUM_ZERO)
            return $status;
        //Load queue object
        //Adding entry to queue
        $sql = 'SELECT FIAT.IDCODE, FIAT.EFFTDATE, FICO.COMP_CODE FROM ' . $this->fiatname . ' FIAT, ' . $this->ficoname . ' FICO WHERE FICO.IDCODE = FIAT.CHART_FIELD01 AND FIAT.IDCODE = ' . singleQuote($trans_code) . ' AND FIAT.TRANS_STATUS NOT IN (' . singleQuoteArray(array('AP', 'REJ')) . ') ORDER BY FIAT.IDCODE ASC';

        $results = ClxCommon::$db->executeQuery($sql);
        while ($result = ClxCommon::$db->getNextRecord($results, ASSOCIATIVE)) {
            //set status to queue
            ClxCommon::$db->updateRecord($this->fiatname, array('TRANS_STATUS' => 'OPQU', 'CLX_AUTH_STATUS' => 'PEAP'), 'IDCODE = ' . singleQuote($result['IDCODE']));
            //add queue entry
            ClxFreeReturn($this->pushQueue('FIAT', $result['COMP_CODE'], 'null', 'null', $result['IDCODE'], 'null'));
        }
        return $status;
    }

    /**
     * Adds investment activity for excecution
     *
     * @return void
     */
    function pushInatListing() {
        $idlist = getIdlist();
        $status = false;
        if (count($idlist) == CLX_NUM_ZERO)
            return $status;
        //Load queue object
        //Adding entry to queue
        $sql = 'SELECT INAT.IDCODE, INAT.EFFTDATE, INFU.COMP_CODE FROM ' . $this->inatname . ' INAT, ' . $this->infuname . ' INFU WHERE INFU.IDCODE = INAT.PORTFOLIO_CODE AND INAT.TRANS_STATUS = ' . singleQuote('PEND') . ' AND INAT.ID IN (' . implode(',', $idlist) . ') ORDER BY INAT.IDCODE ASC';
        $results = ClxCommon::$db->executeQuery($sql);
        while ($result = ClxCommon::$db->getNextRecord($results, ASSOCIATIVE)) {
            //set status to queue
            ClxCommon::$db->updateRecord($this->inatname, array('TRANS_STATUS' => 'OPQU', 'CLX_AUTH_STATUS' => 'PEAP'), 'IDCODE = ' . singleQuote($result['IDCODE']));
            //add queue entry
            ClxFreeReturn($this->pushQueue('INAT', $result['COMP_CODE'], 'null', 'null', $result['IDCODE'], 'null'));
        }
        return $status;
    }

    /**
     * Adds closing activity for excecution on daily closing
     *
     * @return void
     */
    function pushDacl($comp_code, $sdate, $edate) {
        //add queue entry
        return $this->pushQueue('DACL', $comp_code, $sdate, $edate, 'null', 'null');
    }

    /**
     * Adds massive desincorporations to queue
     *
     */
    function pushFpctListing(&$contents) {
        $idlist = getIdlist();
        $status = false;
        if (count($idlist) > CLX_NUM_ZERO) {
            //Load queue object
            //Adding entry to queue
            $sql = 'SELECT DISTINCT BATCH_ID FROM ' . $this->fpctname . ' WHERE ID IN (' . implode(',', $idlist) . ') AND REC_STATUS IN (' . singleQuoteArray(array('ERRO', 'IDLE')) . ')';
            $results = ClxCommon::$db->executeQuery($sql);
            while ($result = ClxCommon::$db->getNextRecord($results, ASSOCIATIVE)) {
                //add queue entry
                $status = $this->pushQueue('FPCT', 'null', 'null', 'null', 'null', $result['BATCH_ID']);
                $status = true;
            }
            if (!empty($status))
                $contents .= note('<p aling="left"> ' . dict('GEN_LBL_SCHED_ACTIVITY') . '</p>');
        }
        return $status;
    }

    function pushFibrListing($comp_code) {
        if (empty($comp_code))
            $comp_code = 'n/a';
        return $this->pushQueue('FIBR', $comp_code, 'null', 'null', 'null', 'null');
    }

    /**
     * Process queue
     *
     * @param array $queue
     */
    function processQueue(&$queue) {
        //setting user to run queue
        $_SESSION['user_record']['username'] = $queue['CREATEDBY'];
        $status = false;
        switch ($queue['GLOQ_TYPE']) {
            case 'FIBN':
                if (!class_exists('Fibn')){
                    includeCodeLib('FIBN');
                }
                $a = new Fibn();
                $status = $a->bv_run_gloq($queue['REC_ID']);
                unset($a);
                break;
            case 'INAT':
                if (!class_exists('fidu_investments'))
                    includeCodeLib('INUT');
                $a = new fidu_investments();
                $status = $a->approveStatusByTranscode($queue['INVTRANS_CODE']);
                unset($a);
                break;
            case 'FIAT':
                if (!class_exists('Fiat'))
                    includeCodeLib('FIAT');
                $a = new Fiat();
                $status = $a->fiat_approve_by_transcode($queue['TRANS_CODE']);
                unset($a);
                break;
            case 'FIMT':
                if (!class_exists('Fimt'))
                    includeCodeLib('FIMT');
                $a = new Fimt();
                $status = $a->fimt_approve_by_transcode($queue['MASS_TRANS_CODE']);
                unset($a);
                break;
            case 'JEHD':
                if (!class_exists('Jehd'))
                    includeCodeLib('JEHD');
                $a = new Jehd();
                $status = $a->jehd_approve_by_code($queue['JEHDR_CODE']);
                unset($a);
                break;
            case 'DACL'://Proceso de cierre diarios
                runProcess('DACL', array('COMP_CODE' => $queue['COMP_CODE'], 'EFFTDATE' => $queue['CLOSING_DATE'], 'EDATE' => $queue['NEXT_OPR_DATE']));
                $status = (ClxCommon::$db->oneExistsRecords($this->glcpname, 'COMP_CODE = ' . singleQuote($queue['COMP_CODE']) . ' AND OPERATION_DATE  = ' . singleQuote($queue['NEXT_OPR_DATE'])) > CLX_NUM_ZERO) ? true : false;
                break;
            case 'GLAP'://Proceso de cierre diarios
                runProcess('GLAP', array('COMP_CODE' => $queue['COMP_CODE'], 'OPERATION_DATE' => $queue['CLOSING_DATE'], 'CONTRACT_CODE' =>'', 'PARTICIPANT_ID'=>''));
                $status = true;
                break;
            case 'FPCT':
                //Cierre contable
                runProcess('FPCT', array('BATCH_ID' => $queue['BATCH_ID']));
                $status = true;
                break;
            case 'FIBR':
                runProcess('FIBK_PAY_INT');
                $status = true;
                break;
            default:
                //e cho 'nada';
                break;
        }//end switch
        return $status;
    }

    /**
     * Test queue and executes next pending entry
     *
     */
    function checkQueue() {
        $this->batch_id = getmypid();
        switch (kDatabaseType) {
            case 'oci':
                ClxCommon::$db->commit(); // Para solucionar error "ORA-01453: SET TRANSACTION must be first statement of transaction (1453 )"
                ClxCommon::$db->executeQuery('SET TRANSACTION NAME '.singleQuote($this->batch_id.'_trn'));
                break;
            default:
                ClxCommon::$db->executeQuery('START TRANSACTION WITH CONSISTENT SNAPSHOT');
                break;
        }
        if (ClxCommon::$db->oneExistsRecords($this->gloqname, 'REC_STATUS = ' . singleQuote('RUN')) > CLX_NUM_ZERO) {
			if ( isset($_SESSION["_dEBUG"]) and $_SESSION["_dEBUG"]==99) { $tta=fopen(kPathToWebSite."/".kPathToTemp."archivo.txt","a+");	fwrite($tta,"queue process:".chr(13).chr(10));fclose($tta);}
			
            ClxCommon::$db->commit();
            return;
        }
        $sql = 'UPDATE ' . $this->gloqname . ' GLOQ SET GLOQ.BATCH_PID = ' . singleQuote($this->batch_id) . ', GLOQ.REC_STATUS = ' . singleQuote('RUN') . ', GLOQ.SDATE = ' . singleQuote(xFormat::getNow('Y-m-d H:i:s')) . ' WHERE GLOQ.REC_STATUS = ' . singleQuote('IDLE') . '';
        ClxCommon::$db->executeQuery($sql);
        ClxCommon::$db->commit();
        $sql = 'SELECT * FROM ' . $this->gloqname . ' WHERE REC_STATUS = ' . singleQuote('RUN') . ' AND BATCH_PID = ' . singleQuote($this->batch_id);
        $queues = ClxCommon::$db->executeQuery($sql);
        while (($queue = ClxCommon::$db->getNextRecord($queues))) {
            if ($this->processQueue($queue))
                $rec_status = 'END';
            else
                $rec_status = 'ENDR';
            //closing queue
            ClxCommon::$db->updateRecord($this->gloqname, array('REC_STATUS' => $rec_status, 'EDATE' => xFormat::getNow('Y-m-d H:i:s')), 'ID = ' . $queue['ID']);
            ClxCommon::$db->commit();
        }//end while
    }

}
