<?php

/**
 * General Code File
 * For more information contact  support@alenet.com
 *
 * @author       $Author: jclerma $
 * @copyright    Alenet, Inc.
 * @since        $Date: 2017-08-09 07:53:27 -0400 (Wed, 09 Aug 2017) $
 * @version        $Rev: 9456 $
 *
 * CHANGE LOG
 *  Included svn prop
 */
if (!class_exists('Glrc')) {

    class Glrc {

        /**
         * Contructor
         *
         * @param array $procParameters
         * @return void
         */
        function __construct($procParameters = array()) {
            multiTableName($this, array('GLRC', 'GLRL', 'FIBK', 'JEHD', 'JELN', 'PRAT', 'PROD', 'FIAS', 'FIAT'));
            $this->batch_id = (!empty($procParameters['BATCH_ID'])) ? $procParameters['BATCH_ID'] : batch_id();
            $this->username = $_SESSION['user_record']['username'];
            $this->now = xFormat::getNow('Y-m-d H:i:s');
        }

        /**
         * Validate record information
         * Executed on table validate
         */
        function glrc_validate_form(&$record, &$msg) {
            getLookupValue('GLRC_FIDU_BANKACT_CODE', 'FIDU_BANKACT_CODE', $record);
            $create = (UserRequest::$post['ac'] === 'CREATE') ? true : false;
            $sdate = xFormat::date($record['GLRC_FROM_DATE'], 'Y-m-d');
            $edate = xFormat::date($record['GLRC_TO_DATE'], 'Y-m-d');
            if ($edate < $sdate) {
                $msg['GLRC_FROM_DATE'] = smsg('MSG-70');
            }
            if (UserRequest::$post['ac'] == 'CREATE' || UserRequest::$post['ac'] == 'MODIFY' ){
                $sql = 'SELECT COUNT(PRAT.PRO_GL_CODE) FROM ' . $this->pratname . ' PRAT, ' . $this->fibkname . ' FIBK WHERE PRAT.PRODUCT_CODE = FIBK.BANK_PROD_CODE AND FIBK.BANK_ACCOUNT_NO = ' . singleQuote($record['GLRC_FIDU_BANKACT_CODE']);
                $qty = ClxCommon::$db->getOne($sql);
                if ($qty == CLX_NUM_ZERO) {
                    $msg['GLRC_FIDU_BANKACT_CODE'] = smsg('MSG-71');
                }
            }    
            if (!empty($create)) {
                $qty = ClxCommon::$db->oneExistsRecords($this->glrcname, 'FIDU_BANKACT_CODE = ' . singleQuote($record['GLRC_FIDU_BANKACT_CODE']) . ' AND FROM_DATE = ' . singleQuote($sdate) . ' AND TO_DATE = ' . singleQuote($edate));
                if ($qty <> CLX_NUM_ZERO) {
                    $msg['GLRC_IDNAME'] = smsg('MSG-72');
                }
            }
        }

        /**
         * Preparar registro a con informacion de conciliacion
         *
         * @param array $record
         */
        function glrc_pre_commit(&$record) {
            $sql = 'SELECT PRAT.PRO_GL_CODE FROM ' . $this->pratname . ' PRAT, ' . $this->fibkname . ' FIBK WHERE PRAT.PRODUCT_CODE = FIBK.BANK_PROD_CODE AND FIBK.BANK_ACCOUNT_NO = ' . singleQuote($record['FIDU_BANKACT_CODE']);
            $record['GLACCOUNTCODE'] = ClxCommon::$db->getOne($sql);
            if (UserRequest::$request['ac'] === 'MODIFY') {
                $sql = 'SELECT TO_DATE FROM GL_CONCILIATIONS WHERE IDCODE = ' . singleQuote($record['IDCODE']);
                $old_date = ClxCommon::$db->getOne($sql);
                if ($old_date <> $record['TO_DATE']) {
                    //actualiza jeln para que fecha de conciliacion cambie a TO_DATE en registros con old_date
                    $sql = 'UPDATE ' . $this->jelnname . ' JELN, ' . $this->fibkname . ' FIBK, ' . $this->jehdname . ' JEHD, ' . $this->fiasname . ' FIAS SET JELN.RECONCILIED_DT = ' . singleQuote($record['TO_DATE']) . ' WHERE JELN.JEHDR_CODE = JEHD.IDCODE AND FIBK.COMP_CODE = JEHD.COMP_CODE AND JEHD.JE_STATUS IN (' . singleQuoteArray(array('AP', 'VER', 'CLOS')) . ') AND FIAS.IDCODE = JELN.CHART_FIELD03 AND JELN.GLACCOUNTCODE = ' . singleQuote($record['GLACCOUNTCODE']) . ' AND FIBK.BANK_ACCOUNT_NO = ' . singleQuote($record['FIDU_BANKACT_CODE']) . ' AND JELN.RECONCILIED_DT = ' . singleQuote($old_date) . ' AND FIAS.FIDU_BANKACT_CODE = FIBK.BANK_ACCOUNT_NO';
                    ClxCommon::$db->executeQuery($sql);
                }
            }
        }

        /**
         * Set conciliation totals
         *
         * @param array $record
         */
        function glrc_post_commit(&$record) {
            $this->setGlrcAmt($record['IDCODE']);
        }

        /**
         * Executes conciliations in case operation is from especific date
         * @param type $jehd_code
         */
        function setGlrcAmtByJehd($jehd_code) {
            $sql = 'SELECT DISTINCT GLRC.IDCODE FROM FIDU_ITEMS FIAS, FIDU_BANK_ACCOUNTS FIBK, GL_CONCILIATIONS GLRC, GL_JOURNAL_HDR JEHD, GL_JOURNAL_LINES JELN WHERE JEHD.IDCODE = JELN.JEHDR_CODE AND  JELN.CHART_FIELD03 = FIAS.IDCODE AND  FIAS.FIDU_BANKACT_CODE = FIBK.BANK_ACCOUNT_NO AND  JELN.CURRENCY_CODE = FIBK.CURRENCY_CODE AND  GLRC.FIDU_BANKACT_CODE = FIBK.BANK_ACCOUNT_NO AND  JEHD.EFFTDATE <= GLRC.TO_DATE AND  JEHD.EFFTDATE >= CAST(DATE_FORMAT(GLRC.TO_DATE,\'%Y-%m-01\') AS DATE) AND  EXISTS(SELECT PRAT.ID FROM GL_PROD_PRMTR_TBL PRAT WHERE PRAT.PRO_GL_CODE = JELN.GLACCOUNTCODE AND PRAT.PRODUCT_CODE = FIBK.BANK_PROD_CODE) AND JEHD.IDCODE = ' . singleQuote($jehd_code);
            $rs = ClxCommon::$db->executeQuery($sql);
            while ($r = ClxCommon::$db->getNextRecord($rs, ASSOCIATIVE)) {
                $this->setGlrcAmt($r['IDCODE']);
            }
        }

        function setGlrcAmt($glrc_code) {
            $record = ClxCommon::$db->getFields('GL_CONCILIATIONS', 'IDCODE,TO_DATE,FIDU_BANKACT_CODE', 'IDCODE = ' . singleQuote($glrc_code));
            if (empty($record))
                return;
            $date_a = adodb_prev_month_lastday($record['TO_DATE']);
            $sql = 'SELECT SUM(GLDV.DEBIT_TRANS_AMT-GLDV.CREDIT_TRANS_AMT) BALANCE_AMT FROM GL_JOURNAL_DET_VW GLDV, FIDU_ITEMS FIAS, FIDU_BANK_ACCOUNTS FIBK WHERE GLDV.CHART_FIELD03 = FIAS.IDCODE AND FIAS.FIDU_BANKACT_CODE = FIBK.BANK_ACCOUNT_NO AND GLDV.CURRENCY_CODE = FIBK.CURRENCY_CODE AND FIBK.BANK_ACCOUNT_NO = ' . singleQuote($record['FIDU_BANKACT_CODE']) . ' AND GLDV.EFFTDATE<=' . singleQuote($date_a) . ' AND EXISTS(SELECT PRAT.ID FROM GL_PROD_PRMTR_TBL PRAT WHERE PRAT.PRO_GL_CODE = GLDV.GLACCOUNTCODE AND PRAT.PRODUCT_CODE = FIBK.BANK_PROD_CODE)';
            $r = ClxCommon::$db->getOne($sql);
            $begin_bal = (!empty($r)) ? $r : 0;

            $sql = 'SELECT SUM(JELN.DEBIT_TRANS_AMT) DEPOSITOS, SUM(JELN.CREDIT_TRANS_AMT) RETIROS FROM FIDU_ITEMS FIAS, FIDU_BANK_ACCOUNTS FIBK, GL_CONCILIATIONS GLRC, GL_JOURNAL_HDR JEHD, GL_JOURNAL_LINES JELN WHERE JEHD.IDCODE = JELN.JEHDR_CODE AND  JELN.CHART_FIELD03 = FIAS.IDCODE AND  FIAS.FIDU_BANKACT_CODE = FIBK.BANK_ACCOUNT_NO AND  JELN.CURRENCY_CODE = FIBK.CURRENCY_CODE AND  JEHD.JE_STATUS = ' . singleQuote('AP') . ' AND  GLRC.FIDU_BANKACT_CODE = FIBK.BANK_ACCOUNT_NO AND  JEHD.REVERSED = 0 AND  JEHD.EFFTDATE <= GLRC.TO_DATE AND JEHD.EFFTDATE > ' . singleQuote($date_a) . '  AND (JELN.RECONCILIED_DT IS NULL OR JELN.RECONCILIED_DT = GLRC.TO_DATE) AND  EXISTS(SELECT PRAT.ID FROM GL_PROD_PRMTR_TBL PRAT WHERE PRAT.PRO_GL_CODE = JELN.GLACCOUNTCODE AND PRAT.PRODUCT_CODE = FIBK.BANK_PROD_CODE) AND GLRC.IDCODE = ' . singleQuote($record['IDCODE']);
            $results = ClxCommon::$db->getOneRecord($sql);
            $depositos = (empty($results['DEPOSITOS'])) ? 0.00 : $results['DEPOSITOS'];
            $retiros = (empty($results['RETIROS'])) ? 0.00 : $results['RETIROS'];

            //SUMARIZANDO ENCABEZADO
            $sql = 'UPDATE GL_CONCILIATIONS GLRC SET GLRC.BEGIN_BALANCE_AMT = ' . $begin_bal . ', GLRC.WITHDRAWALS_AMT = ' . $retiros . ', GLRC.DEPOSITS_AMT = ' . $depositos . ', GLRC.REC_STATUS = ' . singleQuote('END') . ' WHERE GLRC.IDCODE = ' . singleQuote($record['IDCODE']);
            ClxCommon::$db->executeQuery($sql);
        }

        function change_conciliations() {
            $idlist = getIdlist();
            foreach ($idlist as $value) {
                $callingpage = peekHistory();
                $dt = 'null';
                if (!empty($callingpage['parentrecord']['GLRC_TO_DATE'])) {
                    $dt = singleQuote($callingpage['parentrecord']['GLRC_TO_DATE']);
                }
                $recon = ($dt === 'null') ? 0 : 1;
                $sql = 'UPDATE ' . $this->jelnname . ' JELN, ' . $this->jehdname . ' JEHD SET JELN.RECONCILIED = ' . $recon . ', JELN.RECONCILIED_DT = ' . $dt . ', JEHD.MDATE = NOW() WHERE JEHD.IDCODE = JELN.JEHDR_CODE AND JELN.ID = ' . singleQuote($value);
                ClxCommon::$db->executeQuery($sql);
            }
        }
        
        function load_form(&$record) {
            if (UserRequest::$request['ac'] === 'DUPLICATE' && empty(UserRequest::$post['confirm'])) {
                $record['GLRC_CREATEBY'] = '';
                $record['GLRC_MODIFIEDBY'] = '';
                $record['GLRC_CDATE'] = '';
                $record['GLRC_MDATE'] = '';
                $record['GLRC_IDCODE'] = 'AUTO';
            }
            
        }

    }

}