<?php

/**
 * General Code File
 * For more information contact support@alenet.com
 *
 * @author $Author: fsantana $
 * @copyright Alenet, Inc.
 * @since $Date: 2016-05-10 16:22:02 -0400 (Tue, 10 May 2016) $
 * @version $Rev: 8198 $
 *
 * CHANGE LOG
 *  Included svn prop
 */
class Jebl {

    var $code = 'JEBL';
    var $operation_date;
    var $datestamp;
    var $comp_code;
    var $batch_id;

    function __construct() {
        $this->datestamp = date('Y-m-d H:i:s');
        multiTableName($this, array('JEBL', 'JEHD', 'JELN', 'COMP', 'GLAC', 'GLCP', 'LEGI', 'CURR'));
        $this->efftdate = '';
        $this->batch_id = batch_id();
    }

    /**
     * Builds balances
     *
     * @param string $comp_code
     * @param string $operation_date
     */
    function jebl_build_if_not_exists($comp_code, $operation_date) {
        $sql = 'SELECT COUNT(ID) FROM ' . $this->jeblname . ' WHERE COMP_CODE = ' . singleQuote($comp_code) . ' AND EFFTDATE = ' . singleQuote($operation_date);
        if (ClxCommon::$db->getOne($sql) == CLX_NUM_ZERO) {
            ClxFreeReturn($this->jebl_build($comp_code, $operation_date));
        }
    }

    /**
     * Rebuilds all ledger balances since start date
     * @param $comp_code
     * @param $start_date
     */
    function rebuild_balances_since($comp_code, $start_date) {
        if (empty($comp_code) || empty($start_date)) {
            echoLog('<br>Rebuilding balances JEBL invalid date or company');
            return false;
        }
        $end_date = ClxCommon::$db->getField($this->glcpname, 'OPERATION_DATE', 'COMP_CODE = ' . singleQuote($comp_code));
		$start_date = adodb_get_month_lastday($start_date, 'Y-m-d');
        echoLog('<br>Rebuilding balances JEBL for: ' . $comp_code);
        echoLog('<br>Start date: ' . $start_date);
        echoLog('<br>End date: ' . $end_date);
        if ($start_date >= $end_date || $end_date === '') {
            echoLog('<br>Start date >= End date. Proc. completed');
            return;
        }
        //Delete all balances
        $sql = 'DELETE FROM ' . $this->jeblname . ' WHERE COMP_CODE = ' . singleQuote($comp_code) . ' AND EFFTDATE>=' . singleQuote($start_date);
        ClxCommon::$db->executeQuery($sql);
        $i = CLX_INIT_ZERO;
        while ($start_date < $end_date) {
            echoLog('<br>Building balances for: ' . $start_date . '<BR>');
            ClxFreeReturn($this->jebl_build($comp_code, $start_date));
            $start_date = adodb_next_month_lastday($start_date, 'Y-m-d');
            if ($i > CLX_NUM_200)
                break;
            $i++;
        }
    }

    /**
     * Builds balances
     *
     * @param string $comp_code
     * @param string $operation_date
     */
    function jebl_build($comp_code, $operation_date) {
        $flag = false;
        if (!empty($comp_code) && !empty($operation_date)) {

            $last_run = ClxCommon::$db->getOne('SELECT MAX(EFFTDATE) FROM ' . $this->jeblname . ' WHERE COMP_CODE = ' . singleQuote($comp_code).' AND EFFTDATE <= '. singleQuote($operation_date));
			$ini_date = ClxCommon::$db->getField($this->compname, 'EFFTDATE', 'IDCODE = ' . singleQuote($comp_code));

			if(!$last_run){
			$last_run = $ini_date ;
			}

            $timestamp = strtotime($last_run);
            $last_run_day = date("d", $timestamp);
            $op = ($last_run_day == '01')?'=':'';
			
            $sql = 'DELETE FROM ' . $this->jeblname . ' WHERE COMP_CODE = ' . singleQuote($comp_code) . ' AND EFFTDATE = ' . singleQuote($operation_date);
            ClxCommon::$db->executeQuery($sql);

            /*$sql = 'INSERT INTO ' . $this->jeblname . ' (EFFTDATE, COMP_CODE, LEDGER_CODE, CURRENCY_CODE, GLACCOUNTCODE, GL_TYPE, TRUST_ITEM_TYPE, CHART_FIELD01, CHART_FIELD02, CHART_FIELD03, CHART_FIELD04, BALANCE_AMT, BASE_BAL_AMT) '
                    . 'SELECT ' . singleQuote($operation_date) . ' EFFTDATE, JEHD.COMP_CODE, LEGI.LEDGER_CODE, JELN.CURRENCY_CODE, JELN.GLACCOUNTCODE, GLAC.GL_TYPE, GLAC.TRUST_ITEM_TYPE, JELN.CHART_FIELD01, JELN.CHART_FIELD02, JELN.CHART_FIELD03, JELN.CHART_FIELD04, SUM(JELN.DEBIT_TRANS_AMT-JELN.CREDIT_TRANS_AMT) BALANCE_AMT, SUM(JELN.DEBIT_AMT - JELN.CREDIT_AMT) BASE_BAL_AMT FROM ' . $this->leginame . ' LEGI, ' . $this->jelnname . ' JELN, ' . $this->jehdname . ' JEHD, ' . $this->glacname . ' GLAC, ' . $this->glcpname . ' GLCP '
                    . 'WHERE LEGI.LGGROUP_CODE = JEHD.LGGROUP_CODE AND JEHD.IDCODE = JELN.JEHDR_CODE AND JELN.GLACCOUNTCODE = GLAC.IDCODE AND GLAC.GLGROUP_CODE = GLCP.GLGROUP_CODE AND GLCP.COMP_CODE = JEHD.COMP_CODE AND JEHD.COMP_CODE = ' . singleQuote($comp_code) . ' AND JEHD.JE_STATUS = ' . singleQuote('AP') . ' '
                    . 'AND JEHD.EFFTDATE <= ' . singleQuote($operation_date) . ' '
                    . 'GROUP BY JEHD.COMP_CODE, LEGI.LEDGER_CODE, JELN.CURRENCY_CODE, JELN.GLACCOUNTCODE, GLAC.GL_TYPE, GLAC.TRUST_ITEM_TYPE, JELN.CHART_FIELD01, JELN.CHART_FIELD02, JELN.CHART_FIELD03, JELN.CHART_FIELD04 '
                    ;*/

            $sql = 'INSERT INTO ' . $this->jeblname . ' (EFFTDATE, COMP_CODE, LEDGER_CODE, CURRENCY_CODE, GLACCOUNTCODE, GL_TYPE, TRUST_ITEM_TYPE, CHART_FIELD01, CHART_FIELD02, CHART_FIELD03, CHART_FIELD04, BALANCE_AMT, BASE_BAL_AMT)
                SELECT
                    A.EFFTDATE,
                    A.COMP_CODE,
                    A.LEDGER_CODE,
                    A.CURRENCY_CODE,
                    A.GLACCOUNTCODE,
                    A.GL_TYPE,
                    A.TRUST_ITEM_TYPE,
                    A.CHART_FIELD01,
                    A.CHART_FIELD02,
                    A.CHART_FIELD03,
                    A.CHART_FIELD04,
                    SUM(A.BALANCE_AMT) BALANCE_AMT,
                    SUM(A.BASE_BAL_AMT) BASE_BAL_AMT
                  FROM
                  (SELECT
                    '. singleQuote($operation_date) . ' EFFTDATE,
                    B.COMP_CODE,
                    B.LEDGER_CODE,
                    B.CURRENCY_CODE,
                    B.GLACCOUNTCODE,
                    B.GL_TYPE,
                    B.TRUST_ITEM_TYPE,
                    B.CHART_FIELD01,
                    B.CHART_FIELD02,
                    B.CHART_FIELD03,
                    B.CHART_FIELD04,
                    B.BALANCE_AMT,
                    B.BASE_BAL_AMT
                  FROM
                    GL_BALANCES B
                  WHERE
                    B.COMP_CODE = ' . singleQuote($comp_code) . '
                    AND B.EFFTDATE = ' . singleQuote($last_run) .'
                  UNION ALL
                  SELECT
                    '. singleQuote($operation_date) . ' EFFTDATE,
                    JEHD.COMP_CODE,
                    LEGI.LEDGER_CODE,
                    JELN.CURRENCY_CODE,
                    JELN.GLACCOUNTCODE,
                    GLAC.GL_TYPE,
                    GLAC.TRUST_ITEM_TYPE,
                    JELN.CHART_FIELD01,
                    JELN.CHART_FIELD02,
                    JELN.CHART_FIELD03,
                    JELN.CHART_FIELD04,
                    SUM(JELN.DEBIT_TRANS_AMT-JELN.CREDIT_TRANS_AMT) BALANCE_AMT,
                    SUM(JELN.DEBIT_AMT - JELN.CREDIT_AMT) BASE_BAL_AMT
                  FROM
                    GL_LED_GITEMS_TBL LEGI,
                    GL_JOURNAL_LINES JELN,
                    GL_JOURNAL_HDR JEHD,
                    GL_CODES_TBL GLAC,
                    GL_PARAMETERS_TBL GLCP
                  WHERE
                    LEGI.LGGROUP_CODE = JEHD.LGGROUP_CODE
                    AND JEHD.IDCODE = JELN.JEHDR_CODE
                    AND JELN.GLACCOUNTCODE = GLAC.IDCODE
                    AND GLAC.GLGROUP_CODE = GLCP.GLGROUP_CODE
                    AND GLCP.COMP_CODE = JEHD.COMP_CODE
                    AND JEHD.COMP_CODE = ' . singleQuote($comp_code) . '
                    AND JEHD.JE_STATUS = '.singleQuote('AP').'
                    AND JEHD.EFFTDATE >'.$op.' '. singleQuote($last_run) .'
                    AND JEHD.EFFTDATE <= '. singleQuote($operation_date) . '
                  GROUP BY
                    JEHD.COMP_CODE,
                    LEGI.LEDGER_CODE,
                    JELN.CURRENCY_CODE,
                    JELN.GLACCOUNTCODE,
                    GLAC.GL_TYPE,
                    GLAC.TRUST_ITEM_TYPE,
                    JELN.CHART_FIELD01,
                    JELN.CHART_FIELD02,
                    JELN.CHART_FIELD03,
                    JELN.CHART_FIELD04) A
                  GROUP BY
                    A.EFFTDATE,
                    A.COMP_CODE,
                    A.LEDGER_CODE,
                    A.CURRENCY_CODE,
                    A.GLACCOUNTCODE,
                    A.GL_TYPE,
                    A.TRUST_ITEM_TYPE,
                    A.CHART_FIELD01,
                    A.CHART_FIELD02,
                    A.CHART_FIELD03,
                    A.CHART_FIELD04';
            ClxCommon::$db->executeQuery($sql);

            $flag = true;
        }
        return $flag;
    }

}
