<?php

/**
 * Object Class
 * For more information contact  support@alenet.com
 * @package		Clientix.Tool.Event
 * @abstract		System Events Manager - Webservice Event.
 * @author       $Author: jclerma $
 * @copyright    Alenet, Inc.
 * @since        $Date: 2017-11-03 15:09:40 -0400 (Fri, 03 Nov 2017) $
 * @version        $Rev: 9615 $
 *
 * CHANGE LOG
 *  Oracle uppercase record changed
 *  Included label
 */
if (!class_exists('actn') && !defined('clxInstallProcess'))
    include_once kPathToObjects . 'actn.php';

class actn_process extends actn {

    /**
     * Constructor
     */
    function __construct() {
        $this->code = 'PRPR';
        $this->name = 'Process Event';
        $this->type = 'ACTN';
        $this->version = '5.5.0.0';
        $this->description = 'Executes process engine.';
    }

    /**
     * Build code object panel on edit event.
     * @param array $msg
     * @param array $actionrecord
     * @return string
     */
    function panel($msg, $actionrecord) {
        if ($actionrecord['EVNT_action'] != $this->code) {
            $this->defaultPanel($actionrecord);
        }
        $table = new htmlTable(CLX_WIDTH_TWO);
        $table->setProps(array(CLX_PROP_WIDTH => '90%', CLX_PROP_CLASS => CLX_CLASS_FORM));
        $table->addCell(dict('GEN_LBL_PRPR_PROPS') . ':', array(CLX_PROP_CLASS => CLX_CLASS_SUBTITLE, CLX_PROP_COLSPAN => CLX_WIDTH_TWO));
        $PRPR = &getTool('PRPR');
        if (!empty($msg['PRPR_processcode'])) {
            $table->addCell($msg['WEBS_webservice'], array(CLX_PROP_CLASS => CLX_CLASS_ALERT, CLX_PROP_COLSPAN => $table->width));
        }
        $table->addData(dict('GEN_LBL_PROCESS') . ':', $PRPR->chooseRecord('PRPR_processcode', $actionrecord['PRPR_processcode'], $actionrecord, CLX_NUM_20, CLX_NUM_16));
        $div = new htmlDiv($this->code, 'visibility:hidden; display:none', $table->source());
        return CLX_CHAR_NL . CLX_CHAR_NL . $div->source() . CLX_CHAR_NL . CLX_CHAR_NL;
    }

    /**
     * Validate Panel User Input before saving record.
     * @param array $msg
     */
    function checkPanel(&$msg) {
        if (UserRequest::$post['EVNT_language'] <> 'PHP' && UserRequest::$post['EVNT_language'] <> 'COMP') {
            $msg['EVNT_action'] = dict('EVNT_LBL_PHP_ONLY');
        } else {
            if (empty(UserRequest::$post['PRPR_processcode'])) {
                $msg['PRPR_processcode'] = dict('GEN_LBL_INVALID_PROCESS');
            } else {
                if (ClxCommon::$db->oneExistsRecords('CLX_PR_PROCESS', 'IDCODE = ' . singleQuote(UserRequest::$post['PRPR_processcode'])) == CLX_NUM_ZERO) {
                    $msg['PRPR_processcode'] = dict('GEN_LBL_INVALID_PROCESS');
                }
            }
        }
    }

    /**
     * Build object record from user post
     * @param array $actionrecord
     */
    function getPanel(&$actionrecord) {
        $actionrecord['extra1'] = CLX_INIT_ZERO;
        $actionrecord['extra2'] = UserRequest::$post['PRPR_processcode'];
    }

    /**
     * Sets edit record required on forms of user interface.
     * @param array $actionrecord
     * @param array $values
     */
    function setPanel(&$actionrecord, $values) {
        if ($values['ACTION'] == $this->code) {
            $actionrecord['PRPR_processcode'] = $values['EXTRA2'];
        } else {
            $this->defaultPanel($actionrecord);
        }
    }

    /**
     * Sets the default values for form on create event.
     * @param array $actionrecord
     */
    function defaultPanel(&$actionrecord) {
        $actionrecord['PRPR_processcode'] = '';
    }

    /**
     * Build panel for display form
     * @param obj $table
     * @param array $record
     */
    function viewForm_2(&$table, $record) {
        $table->addData(dict('GEN_LBL_ACTION') . ':', $this->name);
        $table->addData(dict('GEN_LBL_PROCESS') . ':', $record['PRPR_processcode']);
    }

    /**
     * Builds description about the object on event panel listing.
     * @param array $record
     * @return string
     */
    function describe($record) {
        return dict('GEN_LBL_PROCESS', '') . ': ' . $record['EXTRA2'];
    }

    /**
     * Execute code according object type.
     * @param string $event
     * @param array $record
     * @param array $msg
     */
    function execute3($event, &$record, &$msg) {IF (ISSET($GLOBALS['ADDRDEBUGEVENTS'])) { echo __METHOD__.' '. __FILE__ .':'.__LINE__; pre($event);pre($record); }
        //pendiente por completar
        $processcode = $event['EXTRA2'];
        $PRSC = &getTool('PRSC');
        $PRSC->runProcessNow($processcode, $record);
        return true;
    }

}
