<?php

/**
 * Object Class
 * For more information contact  support@alenet.com
 * @package		Clientix.Tool.Event
 * @abstract		System Events Manager - Record Task Event.
 * @author       $Author: jclerma $
 * @copyright    Alenet, Inc.
 * @since        $Date: 2017-11-03 15:09:40 -0400 (Fri, 03 Nov 2017) $
 * @version        $Rev: 9615 $
 *
 * CHANGE LOG
 *  Oracle uppercase record changed
 *  Included label
 */
if (!class_exists('actn') && !defined('clxInstallProcess'))
    include_once kPathToObjects . 'actn.php';

class actn_rectask extends actn {

    /**
     * Constructor
     */
    function __construct() {
        $this->code = 'TASR';
        $this->name = 'Task Scheduler from record';
        $this->type = 'ACTN';
        $this->version = '1.0.0.0';
        $this->description = 'Creates a task from record.';
    }

    /**
     * Build code object panel on edit event.
     * @param array $msg
     * @param array $actionrecord
     * @return string
     */
    function panel($msg, $actionrecord) {
        if ($actionrecord['EVNT_action'] != $this->code) {
            $this->defaultPanel($actionrecord);
        }
        $table = new htmlTable(CLX_WIDTH_TWO);
        $table->setProps(array(CLX_PROP_WIDTH => '90%', CLX_PROP_CLASS => CLX_CLASS_FORM));
        $table->addCell('Task Properties', array(CLX_PROP_CLASS => CLX_CLASS_SUBTITLE, CLX_PROP_COLSPAN => CLX_WIDTH_TWO));
        $table->addCell(dict('GEN_LBL_TASKREC'), array(CLX_PROP_CLASS => CLX_CLASS_DATA, CLX_PROP_COLSPAN => CLX_WIDTH_TWO));
        $table->addField($msg, dict('GEN_LBL_ASSIGNED_TO') . ':', 'TASR_assignedto', $actionrecord['TASR_assignedto'], CLX_NUM_30, CLX_NUM_30);
        $table->addField($msg, dict('GEN_LBL_SUBJECT') . ':', 'TASR_subject', $actionrecord['TASR_subject'], CLX_NUM_30, CLX_NUM_30);
        $table->addField($msg, dict('GEN_LBL_START_DATE') . ':', 'TASR_start_date', $actionrecord['TASR_start_date'], CLX_NUM_30, CLX_NUM_30);
        $table->addField($msg, dict('GEN_LBL_DUE_DATE') . ':', 'TASR_due_date', $actionrecord['TASR_due_date'], CLX_NUM_30, CLX_NUM_30);
        $table->addField($msg, dict('GEN_LBL_PRIORITY') . ':', 'TASR_priority', $actionrecord['TASR_priority'], CLX_NUM_30, CLX_NUM_30);
        $table->addField($msg, dict('GEN_LBL_TYPE') . ':', 'TASR_type', $actionrecord['TASR_type'], CLX_NUM_30, CLX_NUM_30);
        $table->addField($msg, dict('GEN_LBL_DESCRIPTION') . ':', 'TASR_description', $actionrecord['TASR_description'], CLX_NUM_30, CLX_NUM_30);
        $div = new htmlDiv($this->code, 'visibility:hidden; display:none', $table->source());
        return CLX_CHAR_NL . CLX_CHAR_NL . $div->source() . CLX_CHAR_NL . CLX_CHAR_NL;
    }

    /**
     * Validate Panel User Input before saving record.
     * @param array $msg
     */
    function checkPanel(&$msg) {
        if (UserRequest::$post['EVNT_language'] <> 'PHP' && UserRequest::$post['EVNT_language'] <> 'COMP') {
            $msg['EVNT_action'] = dict('EVNT_LBL_PHP_ONLY');
        } else {
            xValidate::required($msg, 'TASR_assignedto', UserRequest::$post['TASR_assignedto'], dict('GEN_LBL_ASSIGNEDTO'));
            xValidate::required($msg, 'TASR_description', UserRequest::$post['TASR_description'], dict('GEN_LBL_DESCRIPTION'));
            xValidate::required($msg, 'TASR_type', UserRequest::$post['TASR_type'], dict('GEN_LBL_TYPE'));
            xValidate::required($msg, 'TASR_subject', UserRequest::$post['TASR_subject'], dict('GEN_LBL_SUBJECT'));
            xValidate::required($msg, 'TASR_priority', UserRequest::$post['TASR_priority'], dict('GEN_LBL_PRIORITY'));
            xValidate::required($msg, 'TASR_start_date', UserRequest::$post['TASR_start_date'], dict('GEN_LBL_START_DATE'));
            xValidate::required($msg, 'TASR_due_date', UserRequest::$post['TASR_due_date'], dict('GEN_LBL_DUE_DATE'));
        }
    }

    /**
     * Build object record from user post
     * @param array $actionrecord
     */
    function getPanel(&$actionrecord) {
        $actionrecord['EXTRA1'] = CLX_INIT_ZERO;
        $actionrecord['EXTRA2'] = UserRequest::$post['TASR_assignedto'];
        $actionrecord['EXTRA3'] = UserRequest::$post['TASR_subject'];
        $actionrecord['EXTRA4'] = UserRequest::$post['TASR_start_date'];
        $actionrecord['EXTRA5'] = UserRequest::$post['TASR_due_date'];
        $actionrecord['EXTRA6'] = UserRequest::$post['TASR_priority'] . '|' . UserRequest::$post['TASR_type'] . '|' . UserRequest::$post['TASR_description'];
    }

    /**
     * Sets edit record required on forms of user interface.
     * @param array $actionrecord
     * @param array $values
     */
    function setPanel(&$actionrecord, $values) {
        if ($values['ACTION'] == $this->code) {
            $actionrecord['TASR_assignedto'] = $values['EXTRA2'];
            $actionrecord['TASR_subject'] = $values['EXTRA3'];
            $actionrecord['TASR_start_date'] = $values['EXTRA4'];
            $actionrecord['TASR_due_date'] = $values['EXTRA5'];
            $info = explode('|', $values['EXTRA6']);
            $actionrecord['TASR_priority'] = $info[CLX_NUM_ZERO];
            $actionrecord['TASR_type'] = $info[CLX_NUM_ONE];
            $actionrecord['TASR_description'] = $info[CLX_NUM_TWO];
        } else {
            $this->defaultPanel($actionrecord);
        }
    }

    /**
     * Sets the default values for form on create event.
     * @param array $actionrecord
     */
    function defaultPanel(&$actionrecord) {
        $actionrecord['TASR_assignedto'] = 'OWNERCODE';
        $actionrecord['TASR_subject'] = 'SUBJECT';
        $actionrecord['TASR_start_date'] = 'START_DATE';
        $actionrecord['TASR_due_date'] = 'DUE_DATE';
        $actionrecord['TASR_priority'] = 'PRIORITY';
        $actionrecord['TASR_type'] = 'TYPE';
        $actionrecord['TASR_description'] = 'DESCRIPTION';
    }

    /**
     * Build panel for display form
     * @param obj $table
     * @param array $record
     */
    function viewForm(&$table, $record) {
        $table->addData(dict('GEN_LBL_ACTION') . ':', $this->name);
        $table->addData(dict('GEN_LBL_ASSIGNED_TO') . ':', $record['TASR_assignedto']);
        $table->addData(dict('GEN_LBL_SUBJECT') . ':', $record['TASR_subject']);
        $table->addData(dict('GEN_LBL_START_DATE') . ':', $record['TASR_start_date']);
        $table->addData(dict('GEN_LBL_DUE_DATE') . ':', $record['TASR_due_date']);
        $table->addData(dict('GEN_LBL_DESCRIPTION') . ':', $record['TASR_description']);
    }

    /**
     * Builds description about the object on event panel listing.
     * @param array $record
     * @return string
     */
    function describe($record) {
        return dict('GEN_LBL_CREATE_TASKR_TO') . ': ' . $record['EXTRA2'];
    }

    /**
     * Execute code according object type.
     * @param string $event
     * @param array $record
     * @param array $msg
     */
    function execute3($event, &$record, &$msg) {IF (ISSET($GLOBALS['ADDRDEBUGEVENTS'])) { echo __METHOD__.' '. __FILE__ .':'.__LINE__; pre($event);pre($record); }
        //if no record nothing to do
        if (empty($record))
            return;
        $info = explode('|', $event['EXTRA6']);
        $sdate = (array_key_exists($event['EXTRA4'], $record)) ? $record[$event['EXTRA4']] : 'now';
        $ddate = (array_key_exists($event['EXTRA5'], $record)) ? $record[$event['EXTRA5']] : 'now';
        $rec = array();
        $rec['STATUS'] = 'IDLE';
        $rec['TYPE'] = (array_key_exists($info[CLX_NUM_ONE], $record)) ? $record[$info[CLX_NUM_ONE]] : $info[CLX_NUM_ONE];
        $rec['SUBJECT'] = (array_key_exists($event['EXTRA3'], $record)) ? $record[$event['EXTRA3']] : $event['EXTRA3'];
        $rec['DESCRIPTION'] = (array_key_exists($info[CLX_NUM_TWO], $record)) ? $record[$info[CLX_NUM_TWO]] : $info[CLX_NUM_TWO];
        $rec['PERCENT_COMPLETE'] = '0';
        $rec['COMPLETE_DATE'] = 'null';
        $rec['PRIORITY'] = (array_key_exists($info[CLX_NUM_ZERO], $record)) ? $record[$info[CLX_NUM_ZERO]] : $info[CLX_NUM_ZERO];
        $rec['OWNERCODE'] = $_SESSION['user_record']['username'];
        $rec['ASSIGNEDTO'] = (array_key_exists($event['EXTRA2'], $record)) ? $record[$event['EXTRA2']] : $event['EXTRA2'];
        $rec['REMINDER_DATETIME'] = 'null';
        $rec['RECURRING'] = 'NONE';
        $rec['DEPENDS'] = '0';
        $rec['START_DATE'] = xFormat::date($sdate, 'Y-m-d');
        $rec['DUE_DATE'] = xFormat::date($ddate, 'Y-m-d');
        ClxCommon::$db->createRecordVoid('CLX_US_TASKS', $rec, false);
    }

}
