<?php

/**
 * Object Class
 * For more information contact  support@alenet.com
 * @package		Clientix.Tool.Event
 * @abstract		System Events Manager - Webservice Event.
 * @author       $Author: jclerma $
 * @copyright    Alenet, Inc.
 * @since        $Date: 2017-11-03 15:09:40 -0400 (Fri, 03 Nov 2017) $
 * @version        $Rev: 9615 $
 *
 * CHANGE LOG
 *  Oracle uppercase record changed
 *  Included label
 */
if (!class_exists('actn') && !defined('clxInstallProcess'))
    include_once kPathToObjects . 'actn.php';

class actn_webservice extends actn {

    /**
     * Constructor
     */
    function __construct() {
        $this->code = 'WEBS';
        $this->name = 'Webservice Event';
        $this->type = 'ACTN';
        $this->version = '5.5.0.0';
        $this->description = 'Sends a webservice call.';
    }

    /**
     * Build code object panel on edit event.
     * @param array $msg
     * @param array $actionrecord
     * @return string
     */
    function panel($msg, $actionrecord) {
        if ($actionrecord['EVNT_action'] != $this->code) {
            $this->defaultPanel($actionrecord);
        }
        $table = new htmlTable(CLX_WIDTH_TWO);
        $table->setProps(array(CLX_PROP_WIDTH => '90%', CLX_PROP_CLASS => CLX_CLASS_FORM));
        $table->addCell(dict('GEN_LBL_WEBS_PROPS') . ':', array(CLX_PROP_CLASS => CLX_CLASS_SUBTITLE, CLX_PROP_COLSPAN => CLX_WIDTH_TWO));
        $SKSV = getTool('SKSV');
        if (!empty($msg['WEBS_webservice'])) {
            $table->addCell($msg['WEBS_webservice'], array(CLX_PROP_CLASS => CLX_CLASS_ALERT, CLX_PROP_COLSPAN => $table->width));
        }
        $table->addData(dict('GEN_LBL_WEBSERVICE') . ':', $SKSV->chooseRecord('WEBS_webservice', $actionrecord['WEBS_webservice'], $actionrecord, CLX_NUM_20, CLX_NUM_16));
        $div = new htmlDiv($this->code, 'visibility:hidden; display:none', $table->source());
        return CLX_CHAR_NL . CLX_CHAR_NL . $div->source() . CLX_CHAR_NL . CLX_CHAR_NL;
    }

    /**
     * Validate Panel User Input before saving record.
     * @param array $msg
     */
    function checkPanel(&$msg) {
        if (UserRequest::$post['EVNT_language'] <> 'PHP' && UserRequest::$post['EVNT_language'] <> 'COMP') {
            $msg['EVNT_action'] = dict('EVNT_LBL_PHP_ONLY');
        } else {
            if (empty(UserRequest::$post['WEBS_webservice'])) {
                $msg['WEBS_webservice'] = dict('GEN_LBL_INVALID_SERVICE');
            } else {
                $sql = 'SELECT COUNT(ID) FROM CLX_SK_SERVICES WHERE IDCODE = ' . singleQuote(UserRequest::$post['WEBS_webservice']);
                if (ClxCommon::$db->getOne($sql) == CLX_NUM_ZERO) {
                    $msg['WEBS_webservice'] = dict('GEN_LBL_INVALID_SERVICE');
                }
            }
        }
    }

    /**
     * Build object record from user post
     * @param array $actionrecord
     */
    function getPanel(&$actionrecord) {
        $actionrecord['extra1'] = CLX_INIT_ZERO;
        $actionrecord['extra2'] = UserRequest::$post['WEBS_webservice'];
    }

    /**
     * Sets edit record required on forms of user interface.
     * @param array $actionrecord
     * @param array $values
     */
    function setPanel(&$actionrecord, $values) {
        if ($values['ACTION'] == $this->code) {
            $actionrecord['WEBS_webservice'] = $values['EXTRA2'];
        } else {
            $this->defaultPanel($actionrecord);
        }
    }

    /**
     * Sets the default values for form on create event.
     * @param array $actionrecord
     */
    function defaultPanel(&$actionrecord) {
        $actionrecord['WEBS_webservice'] = '';
    }

    /**
     * Build panel for display form
     * @param obj $table
     * @param array $record
     */
    function viewForm(&$table, $record) {
        $table->addData(dict('GEN_LBL_ACTION') . ':', $this->name);
        $table->addData(dict('GEN_LBL_WEBSERVICE') . ':', $record['WEBS_webservice']);
    }

    /**
     * Builds description about the object on event panel listing.
     * @param array $record
     * @return string
     */
    function describe($record) {
        return dict('GEN_LBL_SENDWEBSTO', '') . ': ' . $record['EXTRA2'];
    }

    /**
     * Execute code according object type.
     * @param string $event
     * @param array $record
     * @param array $msg
     */
    function execute3($event, &$record, &$msg) {IF (ISSET($GLOBALS['ADDRDEBUGEVENTS'])) { echo __METHOD__.' '. __FILE__ .':'.__LINE__; pre($event);pre($record); }
        if ($event['EXTRA2'] === '')
            return false;
        $r = getService($event['EXTRA2'], $record);
        if (!empty($r['faultstring'])) {
            $msg[] = $r['faultstring'];
        }
        if (!empty($r['a_response'])) {
            return $r['a_response'];
        } else {
            return false;
        }
    }

}
