<?php

/**
 * Character field object class
 * For more information contact  support@alenet.com
 *
 * @author       $Author: cramirez $
 * @copyright    Alenet, Inc.
 * @since        $Date: 2018-08-23 18:18:58 -0400 (Thu, 23 Aug 2018) $
 * @version    	$Rev: 9902 $
 *
 * Change log
 *  Added Oracle on field properties
 *
 */
if (!class_exists('dbft') && !defined('clxInstallProcess'))
    include_once kPathToObjects . 'dbft.php';

class dbft_character extends dbft {

    // CONSTRUCTOR //
    function __construct() {
        $this->code = 'CHAR';
        $this->name = 'Character';
        $this->type = 'DBFT';
        $this->version = '0.2.3.2';
        $this->description = 'Field that saves any text up to 255 characters long.';
        $this->multifield = false; // Describes it field is a multi field see properties;
        $this->has_lookup = true; // Decribes if field can be set as copy field on lookup
        $this->has_question = true; // Describes if field can be used on queries criteria
        $this->yes_no = array('GEN_LBL_NO', 'GEN_LBL_YES');
        $this->formats = array('GEN_LBL_ANY', 'GEN_LBL_LOWERCASE', 'GEN_LBL_UPPERCASE', 'GEN_LBL_CAPITALIZE');
        $this->text_align = array('GEN_LBL_LEFT', 'GEN_LBL_CENTER', 'GEN_LBL_RIGHT');
        $this->operators = array('==' => 'GEN_LBL_EQUALTO', '!= ' => 'GEN_LBL_NOTEQUALTO',
            '%=' => 'GEN_LBL_STARTSWITH',
            '=%' => 'GEN_LBL_ENDWITH',
            '%%' => 'GEN_LBL_CONTEINS');
    }

    function panel($msg, $fieldrecord, &$form) {
        $c = $this->code;
        if ($fieldrecord['DBFL_fieldtype'] != $c) {
            $this->defaultPanel($fieldrecord);
        }
        $table = new htmlTable(CLX_WIDTH_TWO);
        $table->setProps(array(CLX_PROP_CLASS => CLX_CLASS_FORM));
        $table->addCell(dict('GEN_LBL_CHARFIELDPROP'), array(CLX_PROP_CLASS => CLX_CLASS_SUBTITLE, CLX_PROP_COLSPAN => CLX_WIDTH_TWO));
        $table->addField($msg, dict('GEN_LBL_SIZE') . ':', $this->code . '_size', $fieldrecord[$c . '_size'], CLX_NUM_FOUR, CLX_NUM_FOUR);
        $sel = new htmlSelect($c . '_required');
        foreach ($this->yes_no as $key => $val) {
            $sel->addItem(dict($val), $key, ($fieldrecord[$c . '_required'] == $key));
        }
        $table->addData(dict('GEN_LBL_REQUIRED') . ':', $sel->source());
        $sel = new htmlSelect($c . '_format');
        foreach ($this->formats as $key => $val) {
            $sel->addItem(dict($val), $key, ($fieldrecord[$c . '_format'] == $key));
        }
        $table->addData(dict('GEN_LBL_CAPS') . ':', $sel->source());
        $sel = new htmlSelect($c . '_align');
        foreach ($this->text_align as $key => $val) {
            $sel->addItem(dict($val), $key, ($fieldrecord[$c . '_align'] == $key));
        }
        $table->addData(dict('GEN_LBL_ALIGN') . ':', $sel->source());
        $table->addField($msg, dict('GEN_LBL_DEFAULT') . ':', $c . '_default', $fieldrecord[$c . '_default'], CLX_NUM_32, CLX_NUM_64);
        $table->addField($msg, dict('GEN_LBL_DISPLAY_SIZE') . ':', $c . '_displaysize', $fieldrecord[$c . '_displaysize'], CLX_NUM_FOUR, CLX_NUM_FOUR);
        $div = new htmlDiv($this->code, 'visibility:hidden; display:none', $table->source());
        return CLX_CHAR_NL . CLX_CHAR_NL . $div->source() . CLX_CHAR_NL . CLX_CHAR_NL;
    }

    function checkPanel(&$msg) {
        $c = $this->code;
        if (empty(UserRequest::$post['DBFL_displaysize'])) {
            UserRequest::$post['DBFL_displaysize'] = CLX_INIT_ZERO;
        }
        xValidate::integer($msg, $c . '_size', UserRequest::$post[$c . '_size'], CLX_NUM_ONE, CLX_NUM_2000,'size');
        xValidate::integer($msg, $c . '_displaysize', UserRequest::$post[$c . '_displaysize'], CLX_NUM_ONE, CLX_NUM_255, 'display size');
        if (UserRequest::$post['DBFL_displaysize'] > UserRequest::$post[$c . '_size']) {
            $msg['DBFL_displaysize'] = dict('GEN_LBL_MAX_DISPLAYSIZE', UserRequest::$post[$c . '_size']);
        }
        if (!empty(UserRequest::$post[$c . '_required'])) {
            $default = trim(UserRequest::$post[$c . '_default']);
            xValidate::required($msg, $c . '_default', $default);
        }
    }

    function savePanel(&$fieldrecord) {
        $c = $this->code;
        $fieldrecord['FIELD_SIZE'] = UserRequest::$post[$c . '_size'];
        $fieldrecord['FIELD_REQUIRED'] = UserRequest::$post[$c . '_required'];
        $fieldrecord['FIELD_FORMAT'] = UserRequest::$post[$c . '_format'];
        $fieldrecord['FIELD_ALIGN'] = UserRequest::$post[$c . '_align'];
        $fieldrecord['FIELD_DEFAULT'] = UserRequest::$post[$c . '_default'];
        $fieldrecord['FIELD_MINVALUE'] = '';
        $fieldrecord['FIELD_MAXVALUE'] = '';
        $fieldrecord['FIELD_DISPLAYSIZE'] = UserRequest::$post[$c . '_displaysize'];
        $fieldrecord['FIELD_EXTRA1'] = '';
        $fieldrecord['FIELD_EXTRA2'] = '';
        $fieldrecord['FIELD_EXTRA3'] = '';
        $fieldrecord['FIELD_EXTRA4'] = '';
        $fieldrecord['FIELD_EXTRA5'] = '';
    }

    function loadPanel(&$fieldrecord, $values) {
        $c = $this->code;
        if ($values['FIELDTYPE'] == $c) {
            $fieldrecord[$c . '_size'] = $values['FIELD_SIZE'];
            $fieldrecord[$c . '_required'] = $values['FIELD_REQUIRED'];
            $fieldrecord[$c . '_format'] = $values['FIELD_FORMAT'];
            $fieldrecord[$c . '_align'] = $values['FIELD_ALIGN'];
            $fieldrecord[$c . '_default'] = ($values['FIELD_DEFAULT'] === '') ? 'null' : $values['FIELD_DEFAULT'];
            $fieldrecord[$c . '_displaysize'] = $values['FIELD_DISPLAYSIZE'];
            $fieldrecord[$c . '_format'] = $values['FIELD_FORMAT'];
        } else {
            $this->defaultPanel($fieldrecord);
        }
    }

    function defaultPanel(&$fieldrecord) {
        $c = $this->code;
        $fieldrecord[$c . '_size'] = CLX_NUM_32;
        $fieldrecord[$c . '_required'] = CLX_INIT_ZERO;
        $fieldrecord[$c . '_format'] = CLX_INIT_ZERO;
        $fieldrecord[$c . '_align'] = CLX_INIT_ZERO;
        $fieldrecord[$c . '_default'] = 'null';
        $fieldrecord[$c . '_minvalue'] = '';
        $fieldrecord[$c . '_maxvalue'] = '';
        $fieldrecord[$c . '_displaysize'] = '32';
        $fieldrecord[$c . '_extra1'] = '';
        $fieldrecord[$c . '_extra2'] = '';
        $fieldrecord[$c . '_extra3'] = '';
        $fieldrecord[$c . '_extra4'] = '';
        $fieldrecord[$c . '_extra5'] = '';
    }

    function viewForm(&$table, $record) {
        if ($record['CHAR_format'] === '')
            $record['CHAR_format'] = CLX_INIT_ZERO;
        $table->addData(dict('GEN_LBL_TYPE') . ':', $this->name);
        $table->addData(dict('GEN_LBL_SIZE') . ':', $record['CHAR_size']);
        $table->addData(dict('GEN_LBL_REQUIRED') . ':', dict($this->yes_no[$record['CHAR_required']]));
        $table->addData(dict('GEN_LBL_ALIGN') . ':', dict($this->text_align[$record['CHAR_align']]));
        $table->addData(dict('GEN_LBL_FORMAT') . ':', dict($this->formats[$record['CHAR_format']]));
        $table->addData(dict('GEN_LBL_DEFAULT') . ':', $record['CHAR_default']);
        $table->addData(dict('GEN_LBL_DISPLAY_SIZE') . ':', $record['CHAR_displaysize']);
    }

    function fieldProperties($fieldrecord) {
        $props = array();
        // DB implementation
        switch (kDatabaseType) {
            case 'oci':
                $props['type'] = ($fieldrecord['FIELD_SIZE'] == CLX_NUM_ONE) ? 'CHAR(1)' : 'NVARCHAR2(' . $fieldrecord['FIELD_SIZE'] . ')';
                //$props['type'] = ($fieldrecord['FIELD_SIZE'] == CLX_NUM_ONE) ? 'CHAR(1)' : 'VARCHAR2(' . $fieldrecord['FIELD_SIZE'] . ')';
                break;
            default:
                $props['type'] = ($fieldrecord['FIELD_SIZE'] == CLX_NUM_ONE) ? 'CHAR(1)' : 'VARCHAR(' . $fieldrecord['FIELD_SIZE'] . ')';
                break;
        }
        if ($fieldrecord['FIELD_DEFAULT'] === 'null' || strlen($fieldrecord['FIELD_DEFAULT']) == CLX_NUM_ZERO) {
            $props['default'] = 'null';
        } else {
            $props['default_raw'] = $fieldrecord['FIELD_DEFAULT'];
            $props['default'] = singleQuote($fieldrecord['FIELD_DEFAULT']);
        }
        $props['properties'] = '';
        $props['null'] = ($fieldrecord['FIELD_REQUIRED']) ? 'NO' : 'YES';
        return $props;
    }

    function validateField(&$msg, $form_fieldname, $fieldrecord, &$record, &$table, &$form) {

        //Setting form require pref if any
        if (array_key_exists('FIELD_FORM_REQUIRED', $fieldrecord) && $fieldrecord['FIELD_FORM_REQUIRED'] != CLX_NUM_TWO) {
            $fieldrecord['FIELD_REQUIRED'] = $fieldrecord['FIELD_FORM_REQUIRED'];
        }
        // Note: !array_key_exists was added to enable lookups to validate
        if (!array_key_exists('FIELD_HIDDEN', $fieldrecord))
            $fieldrecord['FIELD_HIDDEN'] = false;
        if (empty($fieldrecord['FIELD_HIDDEN'])) {
            if (UserRequest::$request['ac']!='DELETE' &&$fieldrecord['FIELD_REQUIRED'] === '1') {
                xValidate::required($msg, $form_fieldname, $record[$form_fieldname]);
            }
        }//end if hidden
    }

    function saveField($form_fieldname, $fieldrecord, &$record, $recordid) {
        if (isset(UserRequest::$post[$form_fieldname]) && strlen(trim(UserRequest::$post[$form_fieldname])) > CLX_NUM_ZERO) {
            if (UserRequest::$post[$form_fieldname] === 'null') {
                $record[$fieldrecord['FIELDNAME']] = 'null';
            } else {
                switch ($fieldrecord['FIELD_FORMAT']) {
                    case '1'://Lower
                        $record[$fieldrecord['FIELDNAME']] = lowercase(UserRequest::$post[$form_fieldname]);
                        break;
                    case '2'://upper
                        $record[$fieldrecord['FIELDNAME']] = uppercase(UserRequest::$post[$form_fieldname]);
                        break;
                    case '3'://cap
                        $record[$fieldrecord['FIELDNAME']] = capitalize(UserRequest::$post[$form_fieldname]);
                        break;
                    default://any
                        $record[$fieldrecord['FIELDNAME']] = UserRequest::$post[$form_fieldname];
                        break;
                }
            }
        } else {
            $record[$fieldrecord['FIELDNAME']] = 'null';
        }

        $record[$fieldrecord['FIELDNAME']] = clx_html_spchrs_decode($record[$fieldrecord['FIELDNAME']]);
    }

    function formatField($form_fieldname, $fieldrecord, $record) {
        if (!array_key_exists($form_fieldname, $record)) {
            $record[$form_fieldname] = '';
        }

        if(!empty($fieldrecord['DBQF_FIELD_FORMAT']) &&   $fieldrecord['DBQF_FIELD_FORMAT'] === '1'){
            if (array_key_exists($form_fieldname, $record)) {
                $original_value = $record[$form_fieldname];
            } else {
                $original_value = '';
            }
            return $original_value;
        }
        
        //Alignment
        switch ($fieldrecord['FIELD_ALIGN']) {
            case CLX_NUM_ONE:
                $align = 'text-align: center;';
                break;
            case CLX_NUM_TWO:
                $align = 'text-align: right;right';
                break;
            default:
                $align = 'text-align: left;';
                break;
        }
             //  comentada consume memoria pero strtoupper no trabaja acentos los lookups demasiado en el export  $form_fieldname = uppercase($form_fieldname);
		$form_fieldname = strtoupper($form_fieldname);

        $record = array_change_key_case($record, CASE_UPPER);
        if (array_key_exists('FORMAT', $fieldrecord) && $fieldrecord['FORMAT'] == CLX_NUM_7) {
            $record[$form_fieldname] = clx_format_textual_char($record[$form_fieldname], getCurrentLanguage(), true);
        }

        if (!empty($fieldrecord['FIELD_PASSWORD']) && $fieldrecord['FIELD_PASSWORD']) {
            $la = strlen($record[$form_fieldname]);
            $input = '';
            return str_pad($input, $la, '*');
        } else {
			IF (isset($_GET['ac']) && ($_GET['ac'] =='BUILDCSV' OR $_GET['ac'] =='BUILDCSV_BATCH')) {
				return $record[$form_fieldname];
			}
			else {
				$p = new htmlObject('span');
				$p->setProp('style', $align);
				$p->setContents($record[$form_fieldname]);
				return $p->source();
			}
		}
    }

    /**
     * Builds form object
     * @param type $form_fieldname
     * @param type $fieldrecord
     * @param type $record
     * @return type
     */
    function fieldObject(&$form_fieldname, &$fieldrecord, $record) {
		if (!array_key_exists($form_fieldname, $record) || strlen($record[$form_fieldname]) == CLX_NUM_ZERO) {
            $record[$form_fieldname] = $fieldrecord['FIELD_DEFAULT'];
        }
        $record[$form_fieldname] = clx_html_spchrs_decode($record[$form_fieldname]);
        return parent::fieldObject($form_fieldname, $fieldrecord, $record);
    }

    /**
     * Load session filters
     * @param type $tablecode
     * @param type $fieldname
     * @param type $fieldrecord
     * @param type $record
     * @return string
     */
    function setQuickFilters($tablecode, $fieldname, $fieldrecord, $record) {
        $criteria = '';
        if (array_key_exists($tablecode . '_' . $fieldname, $record) && strlen($record[$tablecode . '_' . $fieldname]) > CLX_NUM_ZERO) {
            $value = str_replace(array(CLX_CHAR_SINGLE_QUOTE, CLX_CHAR_DOUBLE_QUOTE), '', clx_html_spchrs_decode($record[$tablecode . '_' . $fieldname]));
            if (strpos($value, '%') !== false) {
                $criteria = $tablecode . '.' . $fieldname . ' LIKE ' . singleQuote($value);
            } else {
                $criteria = $tablecode . '.' . $fieldname . ' = ' . singleQuote($value);
            }
        }
        return $criteria;
    }

}
