<?php

/**
 * Time field object class
 * For more information contact  support@alenet.com
 *
 * @author       $Author: jclerma $
 * @copyright    Alenet, Inc.
 * @since        $Date: 2017-11-03 15:09:40 -0400 (Fri, 03 Nov 2017) $
 * @version    	$Rev: 9615 $
 */
if (!class_exists('dbft') && !defined('clxInstallProcess'))
    include_once kPathToObjects . 'dbft.php';

class dbft_time extends dbft {

    // CONSTRUCTOR //
    function __construct() {
        $this->code = 'TIME';
        $this->name = 'Time';
        $this->type = 'DBFT';
        $this->version = '0.1.4.3';
        $this->description = 'Can somebody place a nice description here?';
        $this->multifield = false; // Describes it field is a multi field see properties;
        $this->has_lookup = true; // Decribes if field can be set as copy field on lookup
        $this->has_question = true; // Describes if field can be used on queries criteria
        $this->yes_no = array(dict('GEN_LBL_NO'), dict('GEN_LBL_YES'));
        $this->operators = array('===' => 'GEN_LBL_EQUALTO',
            '!== ' => 'GEN_LBL_EQUALTO',
            '>' => 'GEN_LBL_GREATERTHAN', '>=' => 'GEN_LBL_GREATERTHANOREQUEALTO', '<=' => 'GEN_LBL_LESSTHANOREQUALTO',
            '<' => 'GEN_LBL_LESSTHAN'
        );
    }

    function panel($msg, $fieldrecord, &$form) {
        $c = $this->code;
        if ($fieldrecord['DBFL_fieldtype'] != $c) {
            $this->defaultPanel($fieldrecord);
        }
        $table = new htmlTable(CLX_WIDTH_TWO);
        $table->setProps(array(CLX_PROP_CLASS => CLX_CLASS_FORM));
        $table->addCell(dict('GEN_LBL_TIMEFIELDPROP'), array(CLX_PROP_CLASS => CLX_CLASS_SUBTITLE, CLX_PROP_COLSPAN => CLX_WIDTH_TWO));
        $yesno = new htmlSelect($c . '_required');
        $yesno->addItem($this->yes_no[CLX_NUM_ZERO], '0', ($fieldrecord[$c . '_required'] == CLX_NUM_ZERO));
        $yesno->addItem($this->yes_no[CLX_NUM_ONE], '1', ($fieldrecord[$c . '_required'] == CLX_NUM_ONE));
        $table->addData(dict('GEN_LBL_REQUIRED') . ':', $yesno->source());
        $table->addField($msg, dict('GEN_LBL_FORMAT') . ':', $c . '_format', $fieldrecord[$c . '_format'], CLX_NUM_32, CLX_NUM_32);
        $table->addField($msg, dict('GEN_LBL_DEFAULT') . ':', $c . '_default', $fieldrecord[$c . '_default'], CLX_NUM_32, CLX_NUM_32);
        $table->addField($msg, dict('GEN_LBL_DISPLAY_SIZE') . ':', $c . '_displaysize', $fieldrecord[$c . '_displaysize'], CLX_NUM_FOUR, CLX_NUM_FOUR);
        $div = new htmlDiv($this->code, 'visibility:hidden; display:none', $table->source());
        return CLX_CHAR_NL . CLX_CHAR_NL . $div->source() . CLX_CHAR_NL . CLX_CHAR_NL;
    }

    function checkPanel(&$msg) {
        $c = $this->code;
        xValidate::integer($msg, $c . '_displaysize', UserRequest::$post[$c . '_displaysize'], CLX_NUM_ONE, CLX_NUM_255, 'display size');
        if (!empty(UserRequest::$post[$c . '_required'])) {
            xValidate::required($msg, $c . '_default', UserRequest::$post[$c . '_default']);
        }
    }

    function savePanel(&$fieldrecord) {
        $c = $this->code;
        //$fieldrecord['FIELD_SIZE'] = UserRequest::$post['DBFL_fielddisplaysize'];
        $fieldrecord['FIELD_SIZE'] = UserRequest::$post[$c . '_displaysize'];
        $fieldrecord['FIELD_REQUIRED'] = UserRequest::$post[$c . '_required'];
        $fieldrecord['FIELD_FORMAT'] = UserRequest::$post[$c . '_format'];
        $fieldrecord['FIELD_ALIGN'] = CLX_INIT_ZERO;
        $fieldrecord['FIELD_DEFAULT'] = UserRequest::$post[$c . '_default'];
        $fieldrecord['FIELD_MINVALUE'] = '';
        $fieldrecord['FIELD_MAXVALUE'] = '';
        $fieldrecord['FIELD_DISPLAYSIZE'] = UserRequest::$post[$c . '_displaysize'];
        $fieldrecord['FIELD_EXTRA1'] = '';
        $fieldrecord['FIELD_EXTRA2'] = '';
        $fieldrecord['FIELD_EXTRA3'] = '';
        $fieldrecord['FIELD_EXTRA4'] = '';
        $fieldrecord['FIELD_EXTRA5'] = '';
    }

    function loadPanel(&$fieldrecord, $values) {
        $c = $this->code;
        if ($values['FIELDTYPE'] == $c) {
            $fieldrecord[$c . '_required'] = $values['FIELD_REQUIRED'];
            $fieldrecord[$c . '_format'] = $values['FIELD_FORMAT'];
            $fieldrecord[$c . '_default'] = $values['FIELD_DEFAULT'];
            $fieldrecord[$c . '_displaysize'] = $values['FIELD_DISPLAYSIZE'];
        } else {
            $this->defaultPanel($fieldrecord);
        }
    }

    function defaultPanel(&$fieldrecord) {
        $c = $this->code;
        $fieldrecord[$c . '_size'] = CLX_INIT_ZERO;
        $fieldrecord[$c . '_required'] = CLX_INIT_ZERO;
        $fieldrecord[$c . '_format'] = 'H:i:s';
        $fieldrecord[$c . '_align'] = CLX_INIT_ZERO;
        $fieldrecord[$c . '_default'] = 'now';
        $fieldrecord[$c . '_minvalue'] = '';
        $fieldrecord[$c . '_maxvalue'] = '';
        $fieldrecord[$c . '_displaysize'] = '30';
        $fieldrecord[$c . '_extra1'] = '';
        $fieldrecord[$c . '_extra2'] = '';
        $fieldrecord[$c . '_extra3'] = '';
        $fieldrecord[$c . '_extra4'] = '';
        $fieldrecord[$c . '_extra5'] = '';
    }

    function viewForm(&$table, $record) {
        $table->addData(dict('GEN_LBL_TYPE') . ':', $this->name);
        $table->addData(dict('GEN_LBL_REQUIRED') . ':', $this->yes_no[$record['TIME_required']]);
        $table->addData(dict('GEN_LBL_FORMAT') . ':', $record['TIME_format']);
        $table->addData(dict('GEN_LBL_DEFAULT') . ':', $record['TIME_default']);
        $table->addData(dict('GEN_LBL_DISPLAY_SIZE') . ':', $record['TIME_displaysize']);
    }

    function fieldProperties($fieldrecord) {
        $props = array();
        $props['properties'] = '';
        $props['null'] = ($fieldrecord['FIELD_REQUIRED']) ? 'NO' : 'YES';
        // DB implementation
        switch (kDatabaseType) {
            case 'oci':
                $props['type'] = 'DATE';
                break;
            default:
                $props['type'] = 'TIME';
                break;
        }
        $def = str_replace(' ', '', str_replace(':', '', str_replace('-', '', $fieldrecord['FIELD_DEFAULT'])));
        if (is_numeric($def)) {
            if ($fieldrecord['FIELD_DEFAULT'] === 'null' || strlen($fieldrecord['FIELD_DEFAULT']) == CLX_NUM_ZERO) {
                $props['default'] = 'null';
            } else {
                $props['default'] = singleQuote($fieldrecord['FIELD_DEFAULT']);
                $props['default_raw'] = $fieldrecord['FIELD_DEFAULT'];
            }
        } else {
            $props['default'] = 'null';
        }
        return $props;
    }

    function defaultField($form_fieldname, $fieldrecord, &$record) {
        $record[$form_fieldname] = xFormat::date($fieldrecord['FIELD_DEFAULT'], 'H:i:s');
    }

    function fieldObject(&$form_fieldname, &$fieldrecord, $record) {
        $field_displaysize = $fieldrecord['FIELD_DISPLAYSIZE'];
        if (empty($field_displaysize))
            $field_displaysize = $fieldrecord['FIELD_DB_DISPLAYSIZE'];
        $field_size = $field_displaysize;

        if (!array_key_exists($form_fieldname, $record))
            $record[$form_fieldname] = '';
        $fieldobj = new htmlField($form_fieldname, xFormat::date($record[$form_fieldname], 'H:i:s'), $field_displaysize, $field_size);
        //Added instead of displaying read only files
        if (array_key_exists('FIELD_READONLY', $fieldrecord) && $fieldrecord['FIELD_READONLY']) {
            $fieldobj->setProp('readonly', 'true');
            $fieldobj->setProp(CLX_PROP_CLASS, 'readonly');
        }
        return $fieldobj;
    }

    function validateField(&$msg, $form_fieldname, $fieldrecord, &$record, &$table, &$form) {
        //Setting form require pref if any
        if (array_key_exists('FIELD_FORM_REQUIRED', $fieldrecord) && $fieldrecord['FIELD_FORM_REQUIRED'] != CLX_NUM_TWO) {
            $fieldrecord['FIELD_REQUIRED'] = $fieldrecord['FIELD_FORM_REQUIRED'];
        }
        // Note: !array_key_exists was added to enable lookups to validate
        if (!array_key_exists('FIELD_HIDDEN', $fieldrecord))
            $fieldrecord['FIELD_HIDDEN'] = false;
        //Setting hidden to override required
        if (empty($fieldrecord['FIELD_HIDDEN'])) {
            if (UserRequest::$request['ac']!='DELETE' &&(!empty($record[$form_fieldname]) or $fieldrecord['FIELD_REQUIRED'])) {
                xValidate::time($msg, $form_fieldname, $record[$form_fieldname], $fieldrecord['FIELDLISTLABEL']);
            }
        }//end hidden
    }

    function validateImportField(&$msg, $fieldrecord, $fieldvalue) {
        if ($fieldrecord['FIELD_REQUIRED'] == CLX_NUM_ZERO && $fieldvalue === 'null') {
            return;
        } else {
            xValidate::time($msg, $fieldrecord['FIELDNAME'], $fieldvalue, $fieldrecord['FIELDLISTLABEL']);
        }
    }

    function saveField($form_fieldname, $fieldrecord, &$record, $recordid = null) {
        if (strlen(UserRequest::$post[$form_fieldname]) == CLX_NUM_ZERO) {
            $record[$fieldrecord['FIELDNAME']] = 'null';
        } else {
            $record[$fieldrecord['FIELDNAME']] = xFormat::date(UserRequest::$post[$form_fieldname], 'Y-m-d H:i:s');
        }
    }

    function formatField($form_fieldname, $fieldrecord, $record) {
        $form_fieldname = uppercase($form_fieldname);
        $record = array_change_key_case($record, CASE_UPPER);
        if ($record[$form_fieldname] === '00:00:00') {
            return CLX_CHAR_EMPTY_STRING;
        } elseif (strlen($record[$form_fieldname]) == CLX_NUM_ZERO) {
            return CLX_CHAR_EMPTY_STRING;
        } else {
            return xFormat::time($record[$form_fieldname], $fieldrecord['FIELD_FORMAT']);
        }
    }

}
