<?php

/**
 * Tree field object class
 * For more information contact  support@alenet.com
 *
 * @author       $Author: jclerma $
 * @copyright    Alenet, Inc.
 * @since        $Date: 2017-11-03 15:09:40 -0400 (Fri, 03 Nov 2017) $
 * @version    	$Rev: 9615 $
 */
if (!class_exists('dbft') && !defined('clxInstallProcess'))
    include_once kPathToObjects . 'dbft.php';

class dbft_tree_lookup extends dbft {

    // CONSTRUCTOR //
    function __construct() {
        $this->code = 'TLOK';
        $this->name = 'Access Tree Look up';
        $this->type = 'DBFT';
        $this->version = '0.2.4.2';
        $this->description = 'Access Tree Look up field';
        $this->multifield = false; // Describes it field is a multi field see properties;
        $this->has_lookup = true; // Decribes if field can be set as copy field on lookup
        $this->has_question = true; // Describes if field can be used on queries criteria
        $this->operators = array('===' => 'GEN_LBL_EQUALTO', '!== ' => 'GEN_LBL_NOTEQUALTO');
		IF (ISSET($_SESSION['ADDRDEBUGEDIT']) && $_SESSION['ADDRDEBUGEDIT']==$_SERVER['REMOTE_ADDR']) {
			$this->debug=true;
		}
    }

    function panel($msg, $fieldrecord, &$form) {
        $c = $this->code;
        if ($fieldrecord['DBFL_fieldtype'] != $c) {
            $this->defaultPanel($fieldrecord);
        }
        $contents = '';
        $table = new htmlTable(CLX_WIDTH_TWO);
        $table->setProps(array(CLX_PROP_CLASS => CLX_CLASS_FORM));
        $table->addCell(dict('GEN_LBL_ACTEFIELDPROP'), array(CLX_PROP_CLASS => CLX_CLASS_SUBTITLE, CLX_PROP_COLSPAN => CLX_WIDTH_TWO));
        $ACTR = &getTool('ACTR');
        $contents .= CLX_CHAR_NL . CLX_CHAR_NL . $ACTR->placeChooseTree($msg, dict('GEN_LBL_TREECODE') . ':', $c . '_extra1', $fieldrecord[$c . '_extra1'], $table);
        $div = new htmlDiv($this->code, 'visibility:hidden; display:none', $table->source());
        $contents .= CLX_CHAR_NL . CLX_CHAR_NL . $div->source();
        return $contents . CLX_CHAR_NL . CLX_CHAR_NL;
    }

    function checkPanel(&$msg) {
        $c = $this->code;
        $ACTR = &getTool('ACTR');
        if (!$ACTR->treeExists(UserRequest::$post[$c . '_extra1'])) {
            $msg[$c . '_extra1'] = dict('DBTF_LBL_NORECFOUND');
        }
        if (!empty(UserRequest::$post[$c . '_required'])) {
            xValidate::required($msg, $c . '_default', trim(UserRequest::$post[$c . '_default']));
        }
    }

    function savePanel(&$fieldrecord) {
        $c = $this->code;
        $fieldrecord['FIELD_SIZE'] = '';
        $fieldrecord['FIELD_REQUIRED'] = '';
        $fieldrecord['FIELD_FORMAT'] = '';
        $fieldrecord['FIELD_ALIGN'] = CLX_INIT_ZERO;
        $fieldrecord['FIELD_DEFAULT'] = '';
        $fieldrecord['FIELD_MINVALUE'] = '';
        $fieldrecord['FIELD_MAXVALUE'] = '';
        $fieldrecord['FIELD_EXTRA1'] = UserRequest::$post[$c . '_extra1'];
        $fieldrecord['FIELD_EXTRA2'] = '';
        $fieldrecord['FIELD_EXTRA3'] = '';
        $fieldrecord['FIELD_EXTRA4'] = '';
        $fieldrecord['FIELD_EXTRA5'] = '';
    }

    function loadPanel(&$fieldrecord, $values) {
        $c = $this->code;
        if ($values['FIELDTYPE'] == $c) {
            $fieldrecord[$c . '_extra1'] = $values['FIELD_EXTRA1'];
        } else {
            $this->defaultPanel($fieldrecord);
        }
    }

    function defaultPanel(&$fieldrecord) {
        $c = $this->code;
        $fieldrecord[$c . '_size'] = CLX_INIT_ZERO;
        $fieldrecord[$c . '_required'] = CLX_INIT_ZERO;
        $fieldrecord[$c . '_format'] = '';
        $fieldrecord[$c . '_align'] = CLX_INIT_ZERO;
        $fieldrecord[$c . '_default'] = '';
        $fieldrecord[$c . '_minvalue'] = '';
        $fieldrecord[$c . '_maxvalue'] = '';
        $fieldrecord[$c . '_extra1'] = '';
        $fieldrecord[$c . '_extra2'] = '';
        $fieldrecord[$c . '_extra3'] = '';
        $fieldrecord[$c . '_extra4'] = '';
        $fieldrecord[$c . '_extra5'] = '';
    }

    function viewForm(&$table, $record) {
        $table->addData('Type:', $this->name);
        $table->addData('Tree Code:', $record['TLOK_extra1']);
    }

    function fieldProperties($fieldrecord) {
        $props = array();
        switch (kDatabaseType) {
            case 'oci':
                $props['type'] = 'VARCHAR2(255)';
                break;
            default:
                $props['type'] = 'VARCHAR(255)';
                break;
        }
        $props['default'] = 'null';
        $props['properties'] = '';
        $props['null'] = 'YES';
        return $props;
    }

    function placeEditField($msg, $form_fieldname, $fieldrecord, $record, &$form, $jsevents) {
        if (!empty($fieldrecord['FIELD_READONLY'])) {
            $form->addArgument($form_fieldname, $record[$form_fieldname]);
            $table = $this->placeViewField($form_fieldname, $fieldrecord, $record);
        } else {
            $table = new htmlTable(CLX_WIDTH_TWO);
            $table->setProps(array(CLX_PROP_WIDTH => CLX_PROP_100PCT));
            $treecode = $fieldrecord['FIELD_EXTRA1'];
            if ($form->arguments['ac'] === 'CREATE') {
                $criteria = 'USERNAME=' . singleQuote($_SESSION['user_record']['username']) . ' AND TREECODE=' . singleQuote($treecode) . ' AND DEFAULT_ENTRY=1 AND STATUS=1';
                if (ClxCommon::$db->oneExistsRecords('CLX_AC_TREE_USERS', $criteria) > CLX_NUM_ZERO) {
                    $defaultnode = ClxCommon::$db->getField('CLX_AC_TREE_USERS', 'TREENODE', $criteria);
                    $defaultnoderec = ClxCommon::$db->getRecord('CLX_AC_TREE_CONTENTS', 'IDCODE=' . singleQuote($defaultnode) . ' AND TREECODE=' . singleQuote($treecode));
                    $record[$form_fieldname] = $defaultnoderec['IDCODE'];
                    $record[$form_fieldname . '_name'] = $defaultnoderec['IDNAME'];
                }
            }

            $cpt = dict($fieldrecord['CAPTION']);
            if (strlen($cpt) > CLX_NUM_ZERO) {
                $table->addCell($cpt, array(CLX_PROP_CLASS => $fieldrecord['CAPTION_CLASS'], CLX_PROP_COLSPAN => CLX_WIDTH_TWO));
            }
            if (isset($msg[$form_fieldname])) {
                $table->addCell($msg[$form_fieldname], array(CLX_PROP_CLASS => CLX_CLASS_ALERT, CLX_PROP_COLSPAN => CLX_WIDTH_TWO));
            }
            if (isset($msg[$form_fieldname . '_name'])) {
                $table->addCell($msg[$form_fieldname . '_name'], array(CLX_PROP_CLASS => CLX_CLASS_ALERT, CLX_PROP_COLSPAN => CLX_WIDTH_TWO));
            }

            //$prim = new htmlField($form_fieldname, (array_key_exists($form_fieldname, $record)) ? $record[$form_fieldname] : '', CLX_NUM_30, CLX_NUM_30);
            //CLXFreeReturn(new htmlField($form_fieldname . '_name', (array_key_exists($form_fieldname . '_name', $record)) ? $record[$form_fieldname . '_name'] : '', CLX_NUM_30, CLX_NUM_30));
            /*foreach ($jsevents as $key => $value) {
                $prim->setProp($key, $value);
            }*/
            $class = CLX_CLASS_PROMPTREQUIRED;
            $pfx = dict($fieldrecord['FIELDPREFIX']);
            $fieldsource = (strlen($pfx) > CLX_NUM_ZERO) ? $pfx : '';
            $fieldobj1 = new htmlField($form_fieldname, (array_key_exists($form_fieldname, $record)) ? $record[$form_fieldname] : '', '16', '16');
            $fieldsource .= dict('GEN_LBL_CODE') . $fieldobj1->source();
            $fieldobj2 = new htmlField($form_fieldname . '_name', (array_key_exists($form_fieldname . '_name', $record)) ? clx_html_spchrs_decode($record[$form_fieldname . '_name']) : '', '32', '64');
            $fieldsource .= dict('GEN_LBL_NAME') . $fieldobj2->source();
            $sfx = dict($fieldrecord['FIELDSUFIX']);
            $fieldsource .= (strlen($sfx) > CLX_NUM_ZERO) ? $sfx : '';
            $fieldsource .= getSkinIcon('view', 'javascript:openWindow(' . singleQuote('') . ', ' . singleQuote('pop.php?to=ACTE&ac=SEARCH&treecode=' . $treecode . '&target=' . $form_fieldname) . ', 500, 300);', 'Search in a tree');
            $table->addCell(dict($fieldrecord['FIELDFORMLABEL']), array(CLX_PROP_CLASS => $class));
            $table->addCells(CLX_CLASS_DATA, array($fieldsource));
			if ( $this->debug==true &&!isset($GLOBALS['placelistcount']) && $GLOBALS['placelistcount']<200 ) {
				$tta=fopen(kPathToWebSite."/".kPathToTemp."archivo.txt","a+");
				fwrite($tta,chr(13).chr(10).$pos."_".$te[$posx]["function"]."dbft_free_lookup-hora:".date("h:i:s d/m/Y").chr(13).chr(10));
				$te=debug_backtrace();
				for ($t=0;$t<12;$t++) {
					if (isset($te[$t]["file"]))
						fwrite($tta,substr($te[$t]["file"].str_repeat(" ",80),0,80).chr(9).$te[$t]["line"].chr(9).$te[$t]["function"].chr(13).chr(10));
					
				}

				fwrite($tta,chr(13).chr(10)."placeListFieldText: ".print_r($fieldrecord,true));
				fclose($tta);
				if (!isset($GLOBALS['placelistcount']))
					$GLOBALS['placelistcount']=1;
				else
					$GLOBALS['placelistcount']++;				
			}
        }
        return $table;
    }

    function fieldObject(&$form_fieldname, &$fieldrecord, $record) {
        die(dict('GEN_LBL_METHODNOTALLOWED') . ' dbft_tree_lookup.fieldObject');
    }

    function searchFieldObject($form_fieldname, $fieldrecord, $record) {
        $copyfield = $fieldrecord['FIELD_EXTRA2'];
        $copyfieldrec = ClxCommon::$db->getRecord('CLX_DB_FIELDS', 'FIELDNAME=' . singleQuote($copyfield));
        $fieldobj = new htmlField($form_fieldname, (array_key_exists($form_fieldname, $record)) ? $record[$form_fieldname] : '', $copyfieldrec['FIELD_DISPLAYSIZE'], $copyfieldrec['FIELD_SIZE']);
        return $fieldobj;
    }

    function validateField(&$msg, $form_fieldname, $fieldrecord, &$record, &$table, &$form) {
        $something = ((!empty($record[$form_fieldname])) or ( !empty($record[$form_fieldname . '_name'])));
        $required = true;
        if ($required === true and ! $something === true) {
			IF(UserRequest::$request['ac']!='DELETE')
            	$msg[$form_fieldname] = dict('GEN_ERR_REQUIRED_FIELD');
        } elseif ($required === true or $something === true) {
            $treecode = $fieldrecord['FIELD_EXTRA1'];
            $idcode = $record[$form_fieldname];
            $idname = (array_key_exists($form_fieldname . '_name', $record)) ? clx_html_spchrs_decode($record[$form_fieldname . '_name']) : '';
            $criteria = 'TREECODE=' . singleQuote($treecode);
            $altcriteria = 'TREECODE=' . singleQuote($treecode);
            $boolean = ' AND ';
            $usedfields = CLX_INIT_ZERO;
            if (strlen($idcode) > CLX_NUM_ZERO) {
                $criteria .= $boolean . 'IDCODE = ' . singleQuote(clxCommon::$db->addslashes($idcode));
                $altcriteria .= $boolean . 'IDCODE LIKE ' . singleQuote(clxCommon::$db->addslashes($idcode) . '%');
                $usedfields++;
            }
            if (strlen(trim($idname)) > CLX_NUM_ZERO) {
                $criteria .= $boolean . 'IDNAME = ' . singleQuote(clxCommon::$db->addslashes($idname));
                $altcriteria .= $boolean . 'IDNAME LIKE ' . singleQuote(clxCommon::$db->addslashes($idname) . '%');
                $usedfields++;
            }
            $qty = ClxCommon::$db->countRecords('CLX_AC_TREE_CONTENTS', $criteria);
            $altqty = ClxCommon::$db->countRecords('CLX_AC_TREE_CONTENTS', $altcriteria);
            if ($qty >= CLX_NUM_ONE) {
                // do nothing.
            } elseif (($qty + $altqty) == CLX_NUM_ZERO) {
                $msg[$form_fieldname] = dict('DBTF_LBL_NORECFOUND');
            } elseif (($qty > CLX_NUM_ONE) or ( $altqty > CLX_NUM_ONE)) {
                $msg[$form_fieldname] = dict('DBTF_LBL_MORETHANONERECFOUND');
            }
            if (count($msg) == CLX_NUM_ZERO) {
                if ($qty == CLX_NUM_ONE) {
                    $lookrec = ClxCommon::$db->getRecord('CLX_AC_TREE_CONTENTS', $criteria);
                } else {
                    $lookrec = ClxCommon::$db->getRecord('CLX_AC_TREE_CONTENTS', $altcriteria);
                }
                $record[$form_fieldname] = $lookrec['IDCODE'];
                $record[$form_fieldname . '_name'] = $lookrec['IDNAME'];
            }
        } else {
            $record[$form_fieldname] = '';
        }
    }

    function validateImportField(&$msg, $fieldrecord, $fieldvalue) {
        if ($fieldrecord['FIELD_REQUIRED'] == CLX_NUM_ZERO && $fieldvalue === 'null') {
            return;
        } elseif ($fieldvalue <> '') {
            $treecode = $fieldrecord['FIELD_EXTRA1'];
            $lookrec = ClxCommon::$db->getRecord('CLX_AC_TREE_CONTENTS', 'IDCODE = ' . singleQuote($fieldvalue) . ' AND TREECODE=' . singleQuote($treecode));
            if (empty($lookrec)) {
                $msg[$fieldrecord['FIELDNAME']] = dict('DBTF_LBL_NORECFOUND');
            }
        }
    }

    function loadField($form_fieldname, &$fieldrecord, &$record, $values) {
        $record[$form_fieldname] = $values[$fieldrecord['FIELDNAME']];
        if (!empty($values[$fieldrecord['FIELDNAME']])) {
            $treecode = $fieldrecord['FIELD_EXTRA1'];
            $rec = ClxCommon::$db->getRecord('CLX_AC_TREE_CONTENTS', 'TREECODE=' . singleQuote($treecode) . ' AND IDCODE=' . singleQuote($values[$fieldrecord['FIELDNAME']]));
			if(gettype($rec)=='NULL')
				$record[$form_fieldname . '_name'] ='';
			else
				$record[$form_fieldname . '_name'] = $rec['IDNAME'];
        }
    }

    function formatField($form_fieldname, $fieldrecord, $record) {
        $form_fieldname = uppercase($form_fieldname);
        $record = array_change_key_case($record, CASE_UPPER);
       
		if (!isset($record[$form_fieldname]))
			return '';
		else {
			$record[$form_fieldname] = trim($record[$form_fieldname]);
			return htmlspecialchars($record[$form_fieldname]);
		}
    }

    /**
     * Returns the default tree node for a user or current user if user not
     * provided.
     *
     * @param string $fieldname
     * @param string $username
     * @return string
     */
    function getDefaultTreeByUser($fieldname, $username = '') {
        if ($username === '')
            $username = $_SESSION['user_record']['username'];
        $sql = 'SELECT  TC.IDCODE FROM  CLX_DB_FIELDS DBFL, CLX_AC_TREE_USERS TU, CLX_AC_TREE_CONTENTS TC WHERE TC.IDCODE=TU.TREENODE  AND TU.TREECODE = DBFL.FIELD_EXTRA1 AND TU.TREECODE = TC.TREECODE AND UPPER(TU.USERNAME)=' . singleQuote(uppercase($username)) . '  AND DBFL.FIELDNAME = ' . singleQuote($fieldname) . '  AND TU.DEFAULT_ENTRY=1  AND TU.STATUS=1';
        return ClxCommon::$db->getOne($sql);
    }

}
