<?php

/**
 * Object Class
 * For more information contact  support@alenet.com
 *
 * @author       $Author: jclerma $
 * @copyright    Alenet, Inc.
 * @since        $Date: 2017-11-03 15:09:40 -0400 (Fri, 03 Nov 2017) $
 * @version        $Rev: 9615 $
 *
 * CHANGE LOG
 *  Oracle uppercase record changed
 *  Included label
 */
if (!class_exists('plot') && !defined('clxInstallProcess'))
    include_once kPathToObjects . 'plot.php';

class plot_fi2gantt extends plot {

    var $operators;
    var $serie_types;
    var $total_records = CLX_INIT_ZERO; //Stores total records on database.

    /**
     * CONSTRUCTOR
     */

    function __construct() {
        $this->code = 'PLGT';
        $this->name = 'Others - Gantt Chart';
        $this->type = 'PLOT';
        $this->version = '1.0.0.1';
        $this->description = 'The Clientix Gantt Chart';
        $this->plotfile = 'FI2_Gantt.swf';
        $this->has_range = false;
        $this->period_type = array('YEAR' => dict('GEN_LBL_YEAR'), 'DAYS' => dict('GEN_LBL_DAY'));
        $this->serie_types = array('CAT' => dict('GEN_LABEL_CATEGORY'),
            'PROC' => dict('GEN_LBL_PROCESS'),
            'PKEY' => dict('GEN_LBL_PRIMARYKEY'),
            'TASK' => dict('GEN_LBL_TASK'),
            'SESD' => dict('GEN_LBL_SERIES_SDATE'),
            'SEED' => dict('GEN_LBL_SERIES_EDATE'),
            'TASA' => dict('GEN_LBL_ACTUAL_TASK'),
            'SASD' => dict('GEN_LBL_SERIES_ACTUAL_SDATE'),
            'SAED' => dict('GEN_LBL_SERIES_ACTUAL_EDATE'),
            'MILE' => dict('GEN_LBL_MILESTONE'),
            'CONN' => dict('GEN_LBL_CONNECTOR'));
    }

    function panel($msg, $plotrecord, &$form) {

        $table = new htmlTable(CLX_WIDTH_TWO);
        $table->setProps(array(CLX_PROP_CLASS => CLX_CLASS_FORM));
        $table->addField($msg, dict('GEN_LBL_DATESCALE_LBL') . ':', $this->code . '_field_extra4', $plotrecord[$this->code . '_field_extra4'], CLX_NUM_20, CLX_NUM_20);
        $div = new htmlDiv($this->code, 'visibility:hidden; display:none', $table->source());
        return CLX_CHAR_NL . CLX_CHAR_NL . $div->source() . CLX_CHAR_NL . CLX_CHAR_NL;
    }

    function createPanel(&$plotrecord) {
        $plotrecord['FIELD_EXTRA1'] = UserRequest::$post[$this->code . '_field_extra1'];
        $plotrecord['FIELD_EXTRA2'] = UserRequest::$post[$this->code . '_field_extra2'];
        $plotrecord['FIELD_EXTRA3'] = UserRequest::$post[$this->code . '_field_extra3'];
        $plotrecord['FIELD_EXTRA4'] = UserRequest::$post[$this->code . '_field_extra4'];
    }

    function savePanel(&$plotrecord) {
        $plotrecord['FIELD_EXTRA1'] = UserRequest::$post[$this->code . '_field_extra1'];
        $plotrecord['FIELD_EXTRA2'] = UserRequest::$post[$this->code . '_field_extra2'];
        $plotrecord['FIELD_EXTRA3'] = UserRequest::$post[$this->code . '_field_extra3'];
        $plotrecord['FIELD_EXTRA4'] = UserRequest::$post[$this->code . '_field_extra4'];
    }

    function loadPanel(&$plotrecord, $values) {
        $plotrecord[$this->code . '_field_extra1'] = $values['FIELD_EXTRA1'];
        $plotrecord[$this->code . '_field_extra2'] = $values['FIELD_EXTRA2'];
        $plotrecord[$this->code . '_field_extra3'] = $values['FIELD_EXTRA3'];
        $plotrecord[$this->code . '_field_extra4'] = $values['FIELD_EXTRA4'];
    }

    function deletePanel($plotrecord) {

    }

    function defaultPanel(&$plotrecord) {
        $plotrecord[$this->code . '_field_extra1'] = date('Y-01-01');
        $plotrecord[$this->code . '_field_extra2'] = date('Y-12-01');
        $plotrecord[$this->code . '_field_extra3'] = 'YEAR';
        $plotrecord[$this->code . '_field_extra4'] = '';
    }

    function viewForm(&$table, $record) {
        $table->addData(dict('GEN_LBL_DATESCALE_LBL') . ':', $record[$this->code . '_field_extra4']);
        $table->addCell(dict('GEN_PLOT_GANTINSTRUCTIONS'), array(CLX_PROP_CLASS => CLX_CLASS_PROMPT));
        $table->addCell(dict('GEN_PLOT_GANTINSTRUCTION'), array(CLX_PROP_CLASS => CLX_CLASS_DATA));
    }

    function defaultRange($plotrecord) {

    }

    /**
     * builds Final Plot
     */
    function build($plotrecord) {
        //Test if series are complete
        if (!empty($plotrecord))
            $sourceData = $this->plotData($plotrecord);
        if (!empty($sourceData)) {
            /** Chart Data start * */
            $sdate = $sourceData[CLX_NUM_ZERO]['start'];
            $edate = $sourceData[CLX_NUM_ZERO]['end'];
            $dates = $this->dateScale($sdate, $edate, 'YEAR');
            $chart_sdate = $dates[CLX_NUM_ZERO]['FROM_DATE'];
            $chart_edate = $dates[count($dates) - CLX_NUM_ONE]['TO_DATE'];
            /** XML Setup * */
            $basefont = ($plotrecord['FONTTYPE'] === '') ? 'arial' : $plotrecord['FONTTYPE'];
            $baseFontSize = $plotrecord['FONTSIZE'];
            $basefontcolor = '000000';
            /** XML Creation Chart Defaults* */
            $data = new XMLObject('Chart');
            $data->setProp('basefontsize', $baseFontSize);
            $data->setProp('basefont', $basefont);
            $data->setProp('basefontcolor', $basefontcolor);
            $data->setProp('dateFormat', 'yyyy/mm/dd');
            $data->setProp('showTaskNames', '1');
            $data->setProp('showTaskStartDate', CLX_NUM_ZERO); //Displays dates on taks
            $data->setProp('showTaskEndDate', CLX_NUM_ZERO); //Display Dates on Task
            $data->setProp('taskDatePadding', -CLX_NUM_30); //Position of the date
            $data->setProp('extendCategoryBg', CLX_NUM_ZERO);
            $data->setProp('gridBorderAlpha', CLX_NUM_HUNDRED);
            $data->setProp('canvasBorderColor', '333333');
            $data->setProp('canvasBorderThickness', '0');
            $data->setProp('hoverCapBgColor', 'FFFFFF');
            $data->setProp('hoverCapBorderColor', '333333');
            $data->setProp('extendcategoryBg', '0');
            $data->setProp('ganttLineColor', '99cc00');
            $data->setProp('ganttLineAlpha', '20');
            $data->setProp('baseFontColor', '333333');
            $data->setProp('gridBorderColor', '99cc00');
            $data->setProp('gridResizeBarThickness', CLX_NUM_TEN);
            /** categories 1-n* */
            //Load Category Query
            $categories = new XMLObject('categories');
            $categories->setProp('bgColor', '333333');
            $categories->setProp('fontColor', '99cc00');
            $categories->setProp('isBold', '1');
            $categories->setProp('fontSize', $plotrecord['FONTSIZE']);
            $category = new XMLObject('category');
            $category->setProp('start', $chart_sdate);
            $category->setProp('end', $chart_edate);
            $category->setProp('name', $plotrecord['FIELD_EXTRA4']);
            $categories->addObject($category);
            $data->addObject($categories);
            /** Build Dates Category  Scale* */
            $categories = new XMLObject('categories');
            $categories->setProp('bgColor', '333333');
            $categories->setProp('fontColor', '99cc00');
            $categories->setProp('isBold', '1');
            $categories->setProp('fontSize', $plotrecord['FONTSIZE']);
            $i = CLX_INIT_ZERO;
            foreach ($dates as $key => $val) {
                $category = new XMLObject('category');
                $category->setProp('start', $val['FROM_DATE']);
                $category->setProp('end', $val['TO_DATE']);
                $category->setProp('name', $val['NAME']);
                $categories->addObject($category);
                $i++;
            }//End is
            $data->addObject($categories);
            /** Adding Year Scale * */
            $categories = new XMLObject('categories');
            $categories->setProp('bgColor', '333333');
            $categories->setProp('fontColor', '99cc00');
            $categories->setProp('isBold', '1');
            $categories->setProp('fontSize', $plotrecord['FONTSIZE']);
            $i = CLX_INIT_ZERO;
            $dates2 = $this->dateScale($chart_sdate, $chart_edate, 'DAYS');
            foreach ($dates2 as $key => $val) {
                $category = new XMLObject('category');
                $category->setProp('start', $val['FROM_DATE']);
                $category->setProp('end', $val['TO_DATE']);
                $category->setProp('name', $val['NAME']);
                $categories->addObject($category);
                $i++;
            }//End is
            $data->addObject($categories);
            /** Processes Config* */
            $processes = new XMLObject('processes');
            $processes->setProp('positionInGrid', CLX_PROP_RIGHT);
            $processes->setProp(CLX_PROP_ALIGN, 'center');
            $processes->setProp('fontColor', '333333');
            $processes->setProp('fontSize', $plotrecord['FONTSIZE']);
            $processes->setProp('isBold', '1');
            $processes->setProp('isAnimated', '1');
            $processes->setProp('bgColor', '99cc00');
            $processes->setProp('headerbgColor', '333333');
            $processes->setProp('headerFontColor', '99cc00');
            $processes->setProp('headerFontSize', $plotrecord['FONTSIZE']);
            $processes->setProp('bgAlpha', '40');
            $processes->setProp('headerText', $sourceData[CLX_NUM_ZERO]['process']);
            //$processes->setProp(CLX_PROP_WIDTH, 100);
            /** Data Table * */
            $dataTable = new XMLObject('dataTable');
            $dataTable->setProp('fontColor', '333333');
            $dataTable->setProp('fontSize', $plotrecord['FONTSIZE']);
            $dataTable->setProp('isBold', '1');
            $dataTable->setProp('headerFontColor', '000000');
            $dataTable->setProp('headerFontSize', '11');
            /** Data Column * */
            $dataColumn = new XMLObject('dataColumn');
            $dataColumn->setProp('headerbgColor', '333333');
            $dataColumn->setProp('headerfontSize', $plotrecord['FONTSIZE']);
            $dataColumn->setProp('headerAlign', 'center');
            $dataColumn->setProp('headerfontcolor', '99cc00');
            $dataColumn->setProp('bgColor', '99cc00');
            $dataColumn->setProp(CLX_PROP_ALIGN, 'center');
            $dataColumn->setProp('bgAlpha', '65');
            $dataColumn->setProp('headerText', $sourceData[CLX_NUM_ZERO]['category']);
            //$dataColumn->setProp(CLX_PROP_WIDTH, 100);
            /** tasks * */
            $tasks = new XMLObject('tasks');
            $milestones = new XMLObject('milestones');
            $connectors = new XMLObject('connectors');

            unset($sourceData[CLX_NUM_ZERO]); //Label;
            //print_r($sourceData);
            $i = CLX_INIT_ZERO;
            //while($record = ClxCommon::$db->getNextRecord($records,ASSOCIATIVE)){
            foreach ($sourceData as $key => $val) {
                /**  Columns * */
                $text = new XMLObject('text');
                $text->setProp('label', $val['text']);
                $dataColumn->addObject($text);
                /** processes * */
                $process = new XMLObject('process');
                $process->setProp('Name', $val['process']);
                $process->setProp(CLX_PROP_ID, $key);
                $processes->addObject($process);
                //Planned
                $task = new XMLObject('task');
                $task->setProp('name', $val['task']);
                $task->setProp('hoverText', $val['task']);
                $task->setProp('processId', $key);
                $task->setProp('start', date('Y/m/d', strtotime($val['start'])));
                $task->setProp('end', date('Y/m/d', strtotime($val['end'])));
                $task->setProp(CLX_PROP_ID, $key);
                $task->setProp('color', '4567aa');
                $task->setProp('alpha', '60');
                $task->setProp('height', '13');
                $task->setProp('topPadding', '13');
                $tasks->addObject($task);
                //Actual Series
                $task = new XMLObject('task');
                $task->setProp('name', $val['actual_task']);
                $task->setProp('hoverText', $val['actual_task']);
                $task->setProp('processId', $key);
                $task->setProp('start', date('Y/m/d', strtotime($val['actual_start'])));
                $task->setProp('end', date('Y/m/d', strtotime($val['actual_end'])));
                $task->setProp(CLX_PROP_ID, 'a' . $key);
                $task->setProp('color', '99cc00');
                $task->setProp('alpha', '60');
                $task->setProp('height', '13');
                $task->setProp('topPadding', '36');
                $tasks->addObject($task);
                /** milestone * */
                if ($val['milestone'] <> '') {
                    $milestone = new XMLObject('milestone');
                    $milestone->setProp('date', date('Y/m/d', strtotime($val['end'])));
                    $milestone->setProp('taskId', $key);
                    $milestone->setProp('radius', '3');
                    $milestone->setProp('color', '333333');
                    $milestone->setProp('shape', 'Star');
                    $milestone->setProp('numSides', '3');
                    $milestone->setProp('borderThickness', '1');
                    $milestones->addObject($milestone);
                }
                /** Connectors * */
                if ($val['connector_to'] <> '') {
                    $connector = new XMLObject('connector');
                    $connector->setProp('fromTaskId', $val['connector_from']);
                    $connector->setProp('toTaskId', $val['connector_to']);
                    $connector->setProp('color', '4567aa');
                    $connector->setProp('thickness', '1');
                    $connector->setProp('fromTaskConnectStart', '1');
                    $connector->setProp('toTaskConnectStart', '0');
                    $connectors->addObject($connector);
                    $connector = new XMLObject('connector');
                    $connector->setProp('fromTaskId', 'a' . $val['connector_from']);
                    $connector->setProp('toTaskId', 'a' . $val['connector_to']);
                    $connector->setProp('color', 'ff0000');
                    $connector->setProp('thickness', '1');
                    $connector->setProp('fromTaskConnectStart', '1');
                    $connector->setProp('toTaskConnectStart', '0');
                    $connectors->addObject($connector);
                }
                $i++;
            }
            $dataTable->addObject($dataColumn);
            $data->addObject($dataTable);
            $data->addObject($processes);
            $data->addObject($tasks);
            $data->addObject($milestones);
            $data->addObject($connectors);
            $flash = array();
            $flash['TYPE'] = $this->plotfile;

            $flash['FILENAME'] = 'rep_' . $plotrecord['IDCODE'] . '_' . time() . '.xml';
            $data->save(kPathToWebSite . kPathToTemp . $flash['FILENAME']);
            $flash['WIDTH'] = $plotrecord['HSIZE'];
            $flash['HEIGHT'] = $plotrecord['VSIZE'];
            $flash['HEIGHT'] = $this->total_records * CLX_NUM_HUNDRED + CLX_NUM_HUNDRED;
            if ($flash['HEIGHT'] < CLX_NUM_3000) {
                $quality = 'high';
            } else {
                $quality = 'low';
            }
            $flash['BORDER'] = $plotrecord['BORDER'];
            $flash['SWFFILE'] = 'plot/' . $flash['TYPE'] . '?dataUrl=' . kWebSite . 'temp/' . $flash['FILENAME'];
            $flash['ID'] = str_replace('.swf', '', $flash['TYPE']);
            $flash['ALIGN'] = '';
            $flash['BGCOLOR'] = '#FFFFFF';
            $contents = '<object classid="clsid:D27CDB6E-AE6D-11cf-96B8-444553540000" codebase="http://download.macromedia.com/pub/shockwave/cabs/flash/swflash.cab#version=6,0,0,0" WIDTH="' . $plotrecord['HSIZE'] . '" HEIGHT="' . $flash['HEIGHT'] . '" VIEWASTEXT>
		<param NAME="movie" VALUE="' . $flash['SWFFILE'] . '&amp;chartWidth=' . $flash['WIDTH'] . '&amp;chartHeight=' . $flash['HEIGHT'] . '">
		<param NAME="FlashVars" VALUE="">
		<param NAME="quality" VALUE="' . $quality . '">
		<param NAME="bgcolor" VALUE="#FFFFFF">
		<embed src="' . $flash['SWFFILE'] . '&amp;chartWidth=' . $flash['WIDTH'] . '&amp;chartHeight=' . $flash['HEIGHT'] . '" FlashVars="" quality="' . $quality . '" bgcolor="#FFFFFF" NAME="FusionCharts" ALIGN TYPE="application/x-shockwave-flash" WIDTH="' . $flash['WIDTH'] . '" HEIGHT="' . $flash['HEIGHT'] . '" PLUGINSPAGE="http://www.macromedia.com/go/getflashplayer">
		</object>';
        } else {
            $contents = dict('GEN_ERROR_INVALID_CHART_OR_SEARCH');
        }
        return $contents;
    }

    /**
     * Construct Data Series
     */
    function plotData($plotrecord) {
        $sql = '';
        $series_data = array();
        $fields_cache = array();
        $series_records = array();
        $this->plotQuery($plotrecord, $series_records, $sql);
        if (count($series_records) <= CLX_NUM_FIVE) {
            return array();
        }
        $records = ClxCommon::$db->executeQuerySilent($sql);
        $this->total_records = ClxCommon::$db->numRows($records);
        if ($records === true) {
            $i = CLX_NUM_ZERO;
            $datos = array();
            while (($record = ClxCommon::$db->getNextRecord($records, ASSOCIATIVE))) {
                $pkey = '';
                $temp_arr = array('text' => '', 'process' => '', 'task' => '', 'start' => '', 'end' => '', 'actual_task' => '', 'actual_start' => '', 'actual_end' => '', 'milestone' => '', 'connector_from' => '', 'connector_to' => '');
                foreach ($series_records as $series_rec) {
                    $pos = strpos($series_rec['AS'], '_', CLX_NUM_ZERO) + CLX_NUM_ONE;
                    $steps = strlen($series_rec['AS']) - $pos;
                    $fieldname = substr($series_rec['AS'], $pos, $steps);
                    if (array_key_exists($fieldname, $fields_cache)) {
                        $fieldrec = $fields_cache[$fieldname];
                    } else {
                        $fieldrec = ClxCommon::$db->getRecord('CLX_DB_FIELDS', 'FIELDNAME=' . singleQuote($fieldname));
                        $fields_cache[$fieldname] = $fieldrec;
                    }
                    if (!empty($fieldrec)) {
                        $fieldinstance = getToolObject('DBFT', $fieldrec['FIELDTYPE']);
                        $data = array();
                        $data[$fieldrec['FIELDNAME']] = $record[$series_rec['AS']];
                        if ($fieldrec['FIELDTYPE'] === 'DECN' || $fieldrec['FIELDTYPE'] === 'INTN') {
                            $value = $data[$fieldrec['FIELDNAME']]; // Field format as string could damage the plot
                        } else {
                            if ($series_rec['FORMAT'] === true) {
                                $value = $fieldinstance->formatField($fieldrec['FIELDNAME'], $fieldrec, $data);
                                $value = strip_tags($value);
                            } else {
                                $value = $data[$fieldrec['FIELDNAME']];
                            }
                        }
                        switch ($series_rec['TYPE']) {
                            case 'PKEY':
                                $pkey = $data[$fieldrec['FIELDNAME']];
                                break;
                            case 'PROC':
                                $temp_arr['process'] = $value;
                                if ($i == CLX_NUM_ZERO)
                                    $datos[CLX_NUM_ZERO]['process'] = $series_rec['NAME'];
                                break;
                            case 'TASK':
                                $temp_arr['task'] = $value;
                                break;
                            case 'TASA':
                                $temp_arr['actual_task'] = $value;
                                break;
                            case 'CAT':
                                $temp_arr['text'] = $value;
                                if ($i == CLX_NUM_ZERO)
                                    $datos[CLX_NUM_ZERO]['category'] = $series_rec['NAME'];
                                break;
                            case 'SESD':
                                $temp_arr['start'] = substr($data[$fieldrec['FIELDNAME']], CLX_NUM_ZERO, CLX_NUM_TEN);
                                if ($datos[CLX_NUM_ZERO]['start'] > strtotime($temp_arr['start']) || $datos[CLX_NUM_ZERO]['start'] == CLX_NUM_ZERO)
                                    $datos[CLX_NUM_ZERO]['start'] = strtotime($temp_arr['start']);
                                break;
                            case 'SEED':
                                $temp_arr['end'] = substr($data[$fieldrec['FIELDNAME']], CLX_NUM_ZERO, CLX_NUM_TEN);
                                if ($datos[CLX_NUM_ZERO]['end'] < strtotime($temp_arr['end']) || $datos[CLX_NUM_ZERO]['end'] == CLX_NUM_ZERO)
                                    $datos[CLX_NUM_ZERO]['end'] = strtotime($temp_arr['end']);
                                break;
                            case 'SASD':
                                $temp_arr['actual_start'] = substr($data[$fieldrec['FIELDNAME']], CLX_NUM_ZERO, CLX_NUM_TEN);
                                if ($datos[CLX_NUM_ZERO]['start'] > strtotime($temp_arr['start']) || $datos[CLX_NUM_ZERO]['start'] == CLX_NUM_ZERO)
                                    $datos[CLX_NUM_ZERO]['start'] = strtotime($temp_arr['start']);
                                break;
                            case 'SAED':
                                $temp_arr['actual_end'] = substr($data[$fieldrec['FIELDNAME']], CLX_NUM_ZERO, CLX_NUM_TEN);
                                if ($datos[CLX_NUM_ZERO]['end'] < strtotime($temp_arr['end']) || $datos[CLX_NUM_ZERO]['end'] == CLX_NUM_ZERO)
                                    $datos[CLX_NUM_ZERO]['end'] = strtotime($temp_arr['end']);
                                break;
                            case 'MILE':
                                $temp_arr['milestone'] = (!empty($data[$fieldrec['FIELDNAME']])) ? CLX_NUM_ONE : CLX_NUM_ZERO;
                                break;
                            case 'CONN':
                                $temp_arr['connector_to'] = $data[$fieldrec['FIELDNAME']];
                                break;
                            default:
                                break;
                        }
                        $data = array('val' => $value, 'type' => $series_rec['TYPE']);
                        $series_data[$i][] = $data;
                    }
                }
                if ($temp_arr['actual_task'] === '')
                    $temp_arr['actual_task'] = $temp_arr['task'];
                $temp_arr['connector_from'] = $pkey;
                $datos[$pkey] = $temp_arr;
                $i++;
            }//end while
            $series_data = $datos;
        }else {

            $series_data = array();
        }//end if
        if ($series_data[CLX_NUM_ZERO]['start'] > CLX_NUM_ZERO)
            $series_data[CLX_NUM_ZERO]['start'] = date('Y-m-d', $series_data[CLX_NUM_ZERO]['start']);
        if ($series_data[CLX_NUM_ZERO]['end'] > CLX_NUM_ZERO)
            $series_data[CLX_NUM_ZERO]['end'] = date('Y-m-d', $series_data[CLX_NUM_ZERO]['end']);
        return $series_data;
    }

    function dateScale($sdate, $edate, $period_type) {
        /** Chart Data start * */
        $days = array();
        $days[] = 'UICA_LBL_SUNDAY_SHORT';
        $days[] = 'UICA_LBL_MONDAY_SHORT';
        $days[] = 'UICA_LBL_TUESDAY_SHORT';
        $days[] = 'UICA_LBL_WEDNESDAY_SHORT';
        $days[] = 'UICA_LBL_THRUESDAY_SHORT';
        $days[] = 'UICA_LBL_FRIDAY_SHORT';
        $days[] = 'UICA_LBL_SATURDAY_SHORT';
        $sdate_arr = explode('-', $sdate);
        $sdate_arr[CLX_NUM_TWO] = CLX_NUM_ONE;
        $dates = array();

        switch ($period_type) {
            case 'DAYS':
                $periods = day_diff($edate, $sdate);
                for ($i = CLX_INIT_ZERO; $i <= $periods; $i++) {
                    $dates[] = array(
                        'FROM_DATE' => date('Y/m/d', mktime(CLX_NUM_ZERO, CLX_NUM_ZERO, CLX_NUM_ZERO, $sdate_arr[CLX_NUM_ONE], $i, $sdate_arr[CLX_NUM_ZERO])),
                        'TO_DATE' => date('Y/m/d', mktime(CLX_NUM_ZERO, CLX_NUM_ZERO, CLX_NUM_ZERO, $sdate_arr[CLX_NUM_ONE], $i, $sdate_arr[CLX_NUM_ZERO])),
                        'NAME' => dict($days[date('w', mktime(CLX_NUM_ZERO, CLX_NUM_ZERO, CLX_NUM_ZERO, $sdate_arr[CLX_NUM_ONE], $i, $sdate_arr[CLX_NUM_ZERO]))]) . ' ' . date('d', mktime(CLX_NUM_ZERO, CLX_NUM_ZERO, CLX_NUM_ZERO, $sdate_arr[CLX_NUM_ONE], $i, $sdate_arr[CLX_NUM_ZERO])));
                }
                break;
            case 'WEEK':
                $periods = intval(round(day_diff($edate, $sdate) / CLX_NUM_7));
                $last = date('Y-m-d', mktime(CLX_NUM_ZERO, CLX_NUM_ZERO, CLX_NUM_ZERO, $sdate_arr[CLX_NUM_ONE], CLX_NUM_ZERO, $sdate_arr[CLX_NUM_ZERO]));
                $j = CLX_NUM_ONE;
                for ($i = CLX_INIT_ZERO; $i < $periods; $i++) {
                    $sdate_arr = explode('-', $last);
                    $dates[] = array(
                        'FROM_DATE' => date('Y/m/d', mktime(CLX_NUM_ZERO, CLX_NUM_ZERO, CLX_NUM_ZERO, $sdate_arr[CLX_NUM_ONE], $sdate_arr[CLX_NUM_TWO], $sdate_arr[CLX_NUM_ZERO])),
                        'TO_DATE' => date('Y/m/d', mktime(CLX_NUM_ZERO, CLX_NUM_ZERO, CLX_NUM_ZERO, $sdate_arr[CLX_NUM_ONE], $sdate_arr[CLX_NUM_TWO] + CLX_NUM_7, $sdate_arr[CLX_NUM_ZERO])),
                        'NAME' => date('W', mktime(CLX_NUM_ZERO, CLX_NUM_ZERO, CLX_NUM_ZERO, $sdate_arr[CLX_NUM_ONE], $sdate_arr[CLX_NUM_TWO] + CLX_NUM_7, $sdate_arr[CLX_NUM_ZERO])));
                    $j++;
                    if ($j > CLX_NUM_FOUR)
                        $j = CLX_NUM_ONE;
                    $last = date('Y-m-d', mktime(CLX_NUM_ZERO, CLX_NUM_ZERO, CLX_NUM_ZERO, $sdate_arr[CLX_NUM_ONE], $sdate_arr[CLX_NUM_TWO] + CLX_NUM_SIX, $sdate_arr[CLX_NUM_ZERO]));
                }
                break;
            case 'YEAR':
                $edatestr = strtotime($edate);
                $end = CLX_INIT_ZERO;
                $i = CLX_NUM_ONE;
                while ($end <= $edatestr) {
                    $end = mktime(CLX_NUM_ZERO, CLX_NUM_ZERO, CLX_NUM_ZERO, $sdate_arr[CLX_NUM_ONE] + $i, CLX_NUM_ZERO, $sdate_arr[CLX_NUM_ZERO]);
                    $dates[] = array(
                        'FROM_DATE' => date('Y/m/d', mktime(CLX_NUM_ZERO, CLX_NUM_ZERO, CLX_NUM_ZERO, $sdate_arr[CLX_NUM_ONE] + ($i - CLX_NUM_ONE), CLX_NUM_ONE, $sdate_arr[CLX_NUM_ZERO])),
                        'TO_DATE' => date('Y/m/d', $end),
                        'NAME' => dict('UICA_LBL_MONTH_' . adodb_date('m', $end)) . ' ' . adodb_date('Y', $end));
                    $i++;
                }
                break;
            default:
                break;
        }
        return $dates;
    }

}
