<?php

/**
 * Object Class
 * For more information contact  support@alenet.com
 *
 * @package		Clientix.Tool.Analytics
 * @abstract		Report Type Object
 * @author       $Author: cramirez $
 * @copyright    Alenet, Inc.
 * @since        $Date: 2018-12-10 18:11:05 -0400 (lu., 10 dic. 2018) $
 * @version        $Rev: 10412 $
 *
 * CHANGE LOG
 *  Oracle uppercase record changed
 *  Included label
 */
class rpft extends clx_object {

    /**
     * Report Code
     * @var unknown_type
     */
    var $reportcode;
    var $reportrec;
    var $groups;
    var $groups_values;
    var $functions;
    var $functions_values;
    var $columns;
    var $windowurl;
    var $editPanel;
    var $output_buffer = true;
    var $total_rows = CLX_INIT_ZERO;
    var $total_columns = CLX_INIT_ZERO;
    var $oneline_tot_template;
    var $oneline_tot_array;
    var $oneline_tot_func_cols;
	// 2018-12-10 CR Caso 1003486 - GB-Rep BATCH no arranca
	var $output_to_file = null;
	var $error = false;
	var $errormsg = '';
	const SKIN_CONT_SEP_TOKEN = '%begin_content%';
    var $contents;
	
    /**
     * Set to false to stop displaying report
     * as an option on choose report format
     * @var bool
     */
    var $show_prompt_format = true;

    /**
     * Constructor
     */
    function __construct() {
        $this->code = 'RPFT';
        $this->name = 'Report Format Generic Object';
        $this->type = '----';
        $this->version = '0.4.3.0';
        $this->description = 'Report Format Generic Object';
        $this->editPanel = array();
    }

    function build($reportid) {
        $this->getReportRecord($reportid);
        //check access
        $DBQY = &getTool('DBQY');
        $ANRP = &getTool('ANRP');
        //$qaccess = $DBQY->getUserAccess($this->reportrec['QUERYCODE']);
        $qaccess = $DBQY->getAppAccess($this->reportrec['QUERYCODE']);
        if (!$qaccess->canread()) {
            return warning(dict('GEN_ERR_NO_ACCESS') . ' [RPFT-CAN_READ]' . ' ' . $ANRP->getCode(false) . ' Query:' . $this->reportrec['QUERYCODE']);
        }
        $this->getReportColumns();
        $this->getReportGroups();
        $this->getReportFunctions();
        $this->start();
		// 2018-12-10 Caso 1003486 - GB-Rep BATCH no arranca
		if($this->error === true) {
			return 'ERROR: ' . $this->errormsg;
		}
        $records = $this->getRecords();
        $lastrecord = null;
        while ($record = ClxCommon::$db->getNextRecord($records, ASSOCIATIVE)) {
            $this->runRecordEvents($reportid, $record);
            if ($lastrecord != null) {
                $this->insertFunctions($record, $lastrecord);
            }
            $forcebreak = false;
            foreach ($this->groups as $groupid => $grouprec) {
                $value = $record[uppercase($grouprec['TABLECODE'] . '_' . $grouprec['FIELDNAME'])];
                if (($this->groups_values[$groupid] != $value) or ( $forcebreak)) {
                    $this->insertBreak($groupid, $record);
                    $this->groups_values[$groupid] = $value;
                    $forcebreak = true;
                }
            }
            $this->detail($record);
            $this->calculateFunctions($record);
            $lastrecord = $record;
        }
        // Last Group Total
        $this->insertFunctions($record, $lastrecord);
        // General Total
        if ($this->reportrec['HIDE_GRAN_TOTAL'] == CLX_NUM_ZERO) {
            $this->insertGroupFunction('OVERALLTOTAL');
        }
        return $this->finish();
    }

    function Report_Headers() {
        return;
    }

    function Report_Footers() {
        return;
    }

    /**
     * Build Report By Template Id
     * @param int $reportid
     */
    function buildTemplate($reportid) {

    }

    function start() {

    }

    function detail($record) {

    }

    function headers() {

    }

    function footers() {

    }

    function insertbreak($groupid, $record) {

    }

    function finish() {
        return CLX_CHAR_EMPTY_STRING;
    }

    /**
     * Collects queries questions answers
     * @return array
     */
    function getRequestQuestions() {
        $qs = UserRequest::$post;
        unset($qs[CLX_PROP_TO], $qs['ac'], $qs['qy'], $qs['fm'], $qs['li'], $qs['nofr'], $qs['pfid'], $qs['rf'], $qs['rfi'], $qs['ANRP_id'], $qs['ANRP_report_type'], $qs['confirm'], $qs['ANRP_rf'], $qs['submit1']);
        return $qs;
    }

    /**
     * Insert Function
     * @param unknown_type $record
     * @param unknown_type $lastrecord
     */
    function insertFunctions($record, $lastrecord) {
        end($this->groups);
        while (($group = current($this->groups)) !== false) {
            $fieldname = uppercase($group['TABLECODE'] . '_' . $group['FIELDNAME']);
			if (isset($record[$fieldname]) && isset($lastrecord[$fieldname]) && $record[$fieldname] != $lastrecord[$fieldname]) {
                $this->insertGroupFunction($group['GROUPID']);
            }
            prev($this->groups);
        }
    }

    function oneline_tot_init() {
        $this->oneline_tot_array = array('TOTALS' => array('SUM' => array('active' => CLX_NUM_ZERO, 'cols' => $this->oneline_tot_template), 'AVG' => array('active' => CLX_NUM_ZERO, 'cols' => $this->oneline_tot_template), 'MIN' => array('active' => CLX_NUM_ZERO, 'cols' => $this->oneline_tot_template), 'MAX' => array('active' => CLX_NUM_ZERO, 'cols' => $this->oneline_tot_template), 'COUNT' => array('active' => CLX_NUM_ZERO, 'cols' => $this->oneline_tot_template),), 'OTHER' => array());
    }

    function oneline_tot_add($func_name, $function, $groupid, $functionid) {
        //Si no trae resultado debe quedar en 0
        if ($groupid === 'OVERALLTOTAL' && empty($this->functions_values[$groupid][$functionid][$func_name])) {
            $this->functions_values[$groupid][$functionid][$func_name] = CLX_INIT_ZERO;
        }
        $value = $this->functions_values[$groupid][$functionid][$func_name];
        if ($value === '')
            $value = CLX_NUM_0_PT_00;
        // Verify if its a in-line total or outsider one
        $colname = $function['TABLECODE'] . '_' . $function['FIELDNAME'];
        if (!array_key_exists($colname, $this->oneline_tot_template)) {
            // it isnot a column..., so its an outsider total
            $this->oneline_tot_array['OTHER'][] = array('func_name' => $func_name, 'groupid' => $groupid, 'value' => $value, 'functionid' => $functionid);
        } else {
            // it's a column... so its in-line total
            $this->oneline_tot_array['TOTALS'][$func_name]['active'] = CLX_NUM_ONE; // mark it to activate it's output
            $this->oneline_tot_array['TOTALS'][$func_name]['cols'][$colname] = $value;
            $this->oneline_tot_array['TOTALS'][$func_name]['groupid'] = $groupid; // mark it to activate its output
        }
        // Reset totalizer
        $this->functions_values[$groupid][$functionid][$func_name] = null;
    }

    function tot_lbl_assigment($lbl, $groupid) {
        if ($groupid === 'OVERALLTOTAL') {
            /*
             * GEN_LBL_OVERALLTOTAL
             * Overall ^1
             * ^1 General
             */
            $lbl = dict('GEN_LBL_OVERALLTOTAL', $lbl);
        }
        return $lbl;
    }

    function oneline_tot_flush() {
        $first_colname = array_keys($this->oneline_tot_template);
        $first_colname = $first_colname[CLX_NUM_ZERO];
        foreach ($this->oneline_tot_array['TOTALS'] as $key => $totrec) {
            if ($this->oneline_tot_array['TOTALS'][$key]['active'] == CLX_NUM_ONE) {
                // identify total type in 1st. column
                $this->oneline_tot_array['TOTALS'][$key]['cols'][$first_colname] = $this->tot_lbl_assigment(($key === 'SUM' ? dict('GEN_LBL_TOTAL') : dict('GEN_LBL_' . $key)), $this->oneline_tot_array['TOTALS'][$key]['groupid']) . ': ' . $this->oneline_tot_array['TOTALS'][$key]['cols'][$first_colname];
                foreach ($this->oneline_tot_func_cols[$key] as $col_nam) {
                    if ($this->oneline_tot_array['TOTALS'][$key]['cols'][$col_nam] === '') {
                        $this->oneline_tot_array['TOTALS'][$key]['cols'][$col_nam] = '0.00';
                    }
                }
                $this->detail($this->oneline_tot_array['TOTALS'][$key]['cols'], true);
            }
        }
        foreach ($this->oneline_tot_array['OTHER'] as $key => $totrec) {
            if ($this->functions[$totrec['groupid']][$totrec['functionid']]['FIELDTYPE'] === 'DECN') {
                $dec = explode(',', $this->functions[$totrec['groupid']][$totrec['functionid']]['FIELD_FORMAT']);
                $dc = $dec[CLX_NUM_ONE];
                unset($dec);
            } else {
                $dc = CLX_INIT_ZERO;
            }
            $value = /*clx_*/number_format($totrec['value'], $dc);
            $value = $this->tot_lbl_assigment(dict('GEN_LBL_' . $totrec['func_name']), $totrec['groupid']) . ' (' . dict($this->functions[$totrec['groupid']][$totrec['functionid']]['FIELDLISTLABEL']) . '): ' . $value;
            $this->insertFunction($totrec['groupid'], $value, $totrec['functionid']);
        }
        $this->insertBlankLine();
    }

    function insertGroupFunction($groupid) {
        foreach ($this->functions as $key => $functions) {
            if ($key <> $groupid)
                continue;
            $this->oneline_tot_init();
            foreach ($functions as $functionid => $function) {
                if ($function['SUM'] == CLX_NUM_ONE) {
                    $this->oneline_tot_add('SUM', $function, $groupid, $functionid);
                }
                if ($function['AVG'] == CLX_NUM_ONE) {
                    $this->oneline_tot_add('AVG', $function, $groupid, $functionid);
                }
                if ($function['MIN'] == CLX_NUM_ONE) {
                    $this->oneline_tot_add('MIN', $function, $groupid, $functionid);
                }
                if ($function['MAX'] == CLX_NUM_ONE) {
                    $this->oneline_tot_add('MAX', $function, $groupid, $functionid);
                }
                if ($function['COUNT'] == CLX_NUM_ONE) {
                    $this->oneline_tot_add('COUNT', $function, $groupid, $functionid);
                }
            }//end for each function
            // Throws up totals
            $this->oneline_tot_flush();
        }
    }

    /**
     * Inserts blank line (generic)
     * @return void
     */
    function insertBlankLine() {

    }

    function calculateFunctions($record) {
        foreach ($this->functions as $groupid => $functions) {
            foreach ($functions as $functionid => $function) {
                $field = $function['TABLECODE'] . '_' . $function['FIELDNAME'];
                if (!array_key_exists($field, $record))
                    $record[$field] = '';
                $value = $record[$field];
                if ($function['SUM'] == CLX_NUM_ONE || $function['AVG'] == CLX_NUM_ONE) {
                    if (!isset($this->functions_values[$groupid][$functionid]['SUM'])) {
                        $this->functions_values[$groupid][$functionid]['SUM'] = CLX_INIT_ZERO;
                    }
                    if ($this->functions_values[$groupid][$functionid]['SUM'] == null) {
                        $this->functions_values[$groupid][$functionid]['SUM'] = $value;
                    } else {
                        $this->functions_values[$groupid][$functionid]['SUM'] = bcadd($this->functions_values[$groupid][$functionid]['SUM'],$value);
                    }
                    if ($this->functions_values['OVERALLTOTAL']['SUM'] == null) {
                        $this->functions_values['OVERALLTOTAL']['SUM'] = $value;
                    } else {
                        $this->functions_values['OVERALLTOTAL']['SUM'] = bcadd($value,$this->functions_values['OVERALLTOTAL']['SUM']);
                    }
                }
                if ($function['COUNT'] == CLX_NUM_ONE || $function['AVG'] == CLX_NUM_ONE) {
                    if (!isset($this->functions_values[$groupid][$functionid]['COUNT'])) {
                        $this->functions_values[$groupid][$functionid]['COUNT'] = CLX_INIT_ZERO;
                    }
                    if ($this->functions_values[$groupid][$functionid]['COUNT'] == null) {
                        $this->functions_values[$groupid][$functionid]['COUNT'] = CLX_NUM_ONE;
                    } else {
                        $this->functions_values[$groupid][$functionid]['COUNT'] ++;
                    }
                    if ($this->functions_values['OVERALLTOTAL']['COUNT'] == null) {
                        $this->functions_values['OVERALLTOTAL']['COUNT'] = CLX_NUM_ONE;
                    } else {
                        $this->functions_values['OVERALLTOTAL']['COUNT'] ++;
                    }
                }
                if ($function['AVG'] == CLX_NUM_ONE) {
                    if (!isset($this->functions_values[$groupid][$functionid]['AVG'])) {
                        $this->functions_values[$groupid][$functionid]['AVG'] = CLX_INIT_ZERO;
                    }
                    if ($this->functions_values[$groupid][$functionid]['AVG'] == null) {
                        $this->functions_values[$groupid][$functionid]['AVG'] = $value;
                    } else {
                        $this->functions_values[$groupid][$functionid]['AVG'] = bcdiv($this->functions_values[$groupid][$functionid]['SUM'], strval($this->functions_values[$groupid][$functionid]['COUNT']));
                    }
                    if ($this->functions_values['OVERALLTOTAL']['AVG'] == null) {
                        $this->functions_values['OVERALLTOTAL']['AVG'] = $value;
                    } else {
                        $this->functions_values['OVERALLTOTAL']['AVG'] = bcdiv($this->functions_values['OVERALLTOTAL']['SUM'], strval($this->functions_values['OVERALLTOTAL']['COUNT']));
                    }
                }
                if ($function['MIN'] == CLX_NUM_ONE) {
                    if (!isset($this->functions_values[$groupid][$functionid]['MIN'])) {
                        $this->functions_values[$groupid][$functionid]['MIN'] = CLX_INIT_ZERO;
                    }
                    if (($this->functions_values[$groupid][$functionid]['MIN'] == null) || ( $value < $this->functions_values[$groupid][$functionid]['MIN'])) {
                        $this->functions_values[$groupid][$functionid]['MIN'] = $value;
                    }
                    if (($this->functions_values['OVERALLTOTAL']['MIN'] == null) || ( $value < $this->functions_values['OVERALLTOTAL']['MIN'])) {
                        $this->functions_values['OVERALLTOTAL']['MIN'] = $value;
                    }
                }
                if ($function['MAX'] === CLX_NUM_ONE) {
                    if (!isset($this->functions_values[$groupid][$functionid]['MAX'])) {
                        $this->functions_values[$groupid][$functionid]['MAX'] = CLX_INIT_ZERO;
                    }
                    if (($this->functions_values[$groupid][$functionid]['MAX'] == null) || ( $value > $this->functions_values[$groupid][$functionid]['MAX'])) {
                        $this->functions_values[$groupid][$functionid]['MAX'] = $value;
                    }
                    if (($this->functions_values['OVERALLTOTAL']['MAX'] == null) || ( $value > $this->functions_values['OVERALLTOTAL']['MAX'])) {
                        $this->functions_values['OVERALLTOTAL']['MAX'] = $value;
                    }
                }
            }//End for each funtions
        }
    }

    /**
     * Loads report variables into object
     * @param int $reportid
     */
    function getReportRecord($reportid) {
        $this->reportrec = ClxCommon::$db->getRecord('CLX_AN_REPORTS', 'ID=' . singleQuote($reportid));
        $this->reportcode = $this->reportrec['IDCODE'];
    }

    function getReportGroups() {
        $sql = 'SELECT DBTF.ID DBTF_ID, RG.FORMAT FORMAT, RG.ID GROUPID, RG.TABLECODE TABLECODE, RG.FIELDLISTLABEL RG_FIELDLISTLABEL, RG.SEQNUM SEQNUM, F.* FROM CLX_DB_FIELDS F, CLX_AN_REPORT_GROUPS RG LEFT JOIN CLX_DB_TABLEFIELDS DBTF ON DBTF.TABLECODE = RG.TABLECODE AND DBTF.FIELDNAME = RG.FIELDNAME WHERE RG.REPORTCODE = ' . singleQuote($this->reportcode) . ' AND RG.FIELDNAME = F.FIELDNAME ORDER BY RG.SEQNUM';
        $records = ClxCommon::$db->executeQuery($sql);
        $this->groups = array();
        $this->groups_values = array();
        while (($record = ClxCommon::$db->getNextRecord($records, ASSOCIATIVE))) {
            $record['DERIVED_COL_NAME'] = $this->determine_derived_field_condition($record); // check if derived field
            $this->groups[$record['GROUPID']] = $record;
            $this->groups_values[$record['GROUPID']] = null;
        }
    }

    function getReportFunctions() {
        $sql = 'SELECT DBTF.ID DBTF_ID, GF.*, F.* FROM CLX_AN_REPORT_GROUPS RG, CLX_DB_FIELDS F, CLX_AN_REPORT_GROUP_FUNC GF LEFT JOIN CLX_DB_TABLEFIELDS DBTF ON DBTF.TABLECODE = GF.TABLECODE AND DBTF.FIELDNAME = GF.FIELDNAME WHERE RG.REPORTCODE = ' . singleQuote($this->reportcode) . ' AND GF.GROUPID = RG.ID AND GF.FIELDNAME = F.FIELDNAME';
        $records = ClxCommon::$db->executeQuery($sql);
        $this->functions = array();
        $this->functions_values = array();
        while (($record = ClxCommon::$db->getNextRecord($records, ASSOCIATIVE))) {
            $record['CELL_NUMBER'] = CLX_NUM_MINUS_1;
            foreach ($this->columns as $key => $val) {
                if ($val['FIELDNAME'] == $record['FIELDNAME']) {
                    $record['CELL_NUMBER'] = $key;
                }
            }
            $record['DERIVED_COL_NAME'] = $this->determine_derived_field_condition($record); //check if derived field
            $this->functions[$record['GROUPID']][$record['ID']] = $record;
            $this->functions_values[$record['GROUPID']][$record['ID']] = array('SUM' => null, 'COUNT' => null, 'AVG' => null, 'MIN' => null, 'MAX' => null);
            $this->functions['OVERALLTOTAL'][$record['ID']] = $record;
            $this->functions_values['OVERALLTOTAL'] = array('SUM' => null, 'COUNT' => null, 'AVG' => null, 'MIN' => null, 'MAX' => null);
        }
    }

    function getReportColumns() {
        switch (kDatabaseType) {
            case 'oci': // Oracle
                //$sql = 'SELECT ANRC.FORMAT FORMAT, ANRC.SORTING SORTING, ANRC.TABLECODE TABLECODE, ANRC.FIELDLISTLABEL ANRC_FIELDLISTLABEL, DBTF.ID DBTF_ID, DBFL.*, NVL((SELECT QF.EXPRESSION FROM CLX_DB_QUERY_FIELDS QF WHERE QF.FIELDNAME = DBFL.FIELDNAME AND QF.TABLECODE = ANRC.TABLECODE AND QF.QUERYCODE = ANRP.QUERYCODE AND ROWNUM = 1),' . singleQuote(CLX_CHAR_EMPTY_STRING) . ') EXPRESSION FROM CLX_AN_REPORTS ANRP, CLX_DB_FIELDS DBFL, CLX_AN_REPORT_COLUMNS ANRC LEFT JOIN CLX_DB_TABLEFIELDS DBTF ON DBTF.TABLECODE = ANRC.TABLECODE AND DBTF.FIELDNAME = ANRC.FIELDNAME WHERE ANRP.IDCODE = ANRC.REPORTCODE AND ANRC.REPORTCODE = ' . singleQuote($this->reportcode) . ' AND ANRC.FIELDNAME = DBFL.FIELDNAME ORDER BY ANRC.SEQNUM';
                $sql = 'SELECT ANRC.FORMAT FORMAT, ANRC.SORTING SORTING, ANRC.TABLECODE TABLECODE, ANRC.FIELDLISTLABEL ANRC_FIELDLISTLABEL, DBTF.ID DBTF_ID, DBFL.*, NVL((SELECT QF.EXPRESSION FROM CLX_DB_QUERY_FIELDS QF WHERE QF.FIELDNAME = DBFL.FIELDNAME AND QF.TABLECODE = ANRC.TABLECODE AND QF.QUERYCODE = ANRP.QUERYCODE AND ROWNUM = 1),' . singleQuote(CLX_CHAR_EMPTY_STRING) . ') EXPRESSION FROM CLX_AN_REPORTS ANRP, CLX_DB_FIELDS DBFL, CLX_AN_REPORT_COLUMNS ANRC LEFT JOIN CLX_DB_TABLEFIELDS DBTF ON DBTF.TABLECODE = ANRC.TABLECODE AND DBTF.FIELDNAME = ANRC.FIELDNAME WHERE ANRP.IDCODE = ANRC.REPORTCODE AND ANRC.REPORTCODE = ' . singleQuote($this->reportcode) . ' AND ANRC.FIELDNAME = DBFL.FIELDNAME AND ANRC.ACTIVE = ' . singleQuote('1'). ' ORDER BY ANRC.SEQNUM';
                break;
            default: // MySQL Version all others
                //$sql = 'SELECT ANRC.FORMAT FORMAT, ANRC.SORTING SORTING, ANRC.TABLECODE TABLECODE, ANRC.FIELDLISTLABEL ANRC_FIELDLISTLABEL, DBTF.ID DBTF_ID, DBFL.*, NULLIF((SELECT QF.EXPRESSION FROM CLX_DB_QUERY_FIELDS QF WHERE QF.FIELDNAME = DBFL.FIELDNAME AND QF.TABLECODE = ANRC.TABLECODE AND QF.QUERYCODE = ANRP.QUERYCODE LIMIT 1),' . singleQuote(CLX_CHAR_EMPTY_STRING) . ') EXPRESSION FROM CLX_AN_REPORTS ANRP, CLX_DB_FIELDS DBFL, CLX_AN_REPORT_COLUMNS ANRC LEFT JOIN CLX_DB_TABLEFIELDS DBTF ON DBTF.TABLECODE = ANRC.TABLECODE AND DBTF.FIELDNAME = ANRC.FIELDNAME WHERE ANRP.IDCODE = ANRC.REPORTCODE AND ANRC.REPORTCODE = ' . singleQuote($this->reportcode) . ' AND ANRC.FIELDNAME = DBFL.FIELDNAME ORDER BY ANRC.SEQNUM';
                $sql = 'SELECT ANRC.FORMAT FORMAT, ANRC.SORTING SORTING, ANRC.TABLECODE TABLECODE, ANRC.FIELDLISTLABEL ANRC_FIELDLISTLABEL, DBTF.ID DBTF_ID, DBFL.*, NULLIF((SELECT QF.EXPRESSION FROM CLX_DB_QUERY_FIELDS QF WHERE QF.FIELDNAME = DBFL.FIELDNAME AND QF.TABLECODE = ANRC.TABLECODE AND QF.QUERYCODE = ANRP.QUERYCODE LIMIT 1),' . singleQuote(CLX_CHAR_EMPTY_STRING) . ') EXPRESSION FROM CLX_AN_REPORTS ANRP, CLX_DB_FIELDS DBFL, CLX_AN_REPORT_COLUMNS ANRC LEFT JOIN CLX_DB_TABLEFIELDS DBTF ON DBTF.TABLECODE = ANRC.TABLECODE AND DBTF.FIELDNAME = ANRC.FIELDNAME WHERE ANRP.IDCODE = ANRC.REPORTCODE AND ANRC.REPORTCODE = ' . singleQuote($this->reportcode) . ' AND ANRC.FIELDNAME = DBFL.FIELDNAME AND ANRC.ACTIVE = ' . singleQuote('1'). ' ORDER BY ANRC.SEQNUM';
                break;
	}
        
        $records = ClxCommon::$db->executeQuery($sql);
        $this->columns = array();
        while (($record = ClxCommon::$db->getNextRecord($records, ASSOCIATIVE))) {
            $record['DERIVED_COL_NAME'] = $this->determine_derived_field_condition($record); //check if derived field.
            if ($record['EXPRESSION'] === '' && empty($record['DBTF_ID']) && empty($record['DERIVED_COL_NAME'])) {
                continue;
            }
            $this->columns[] = $record;
        }
        $this->total_columns = count($this->columns);
    }

    function determine_derived_field_condition($record) {
        if (!empty($record['DBTF_ID'])) {
            // Put a mark to identify normal field, in this case is blank identifies it as physical field that comes from table
            $response = '';
        } else {
            // then if derived value, search for the formula first
            // Put a mark to identify derived field, in this case is the name for derived field in the query definition
            $response = $record['TABLECODE'] . '_' . $record['FIELDNAME'] . '_WRK';
        }
        return $response;
    }

    function determine_field_name_or_expr($fieldinstance, $column, $queryrec) {
        if ($column['DERIVED_COL_NAME'] === '') {
            // normal field
            $field = $fieldinstance->queryFieldColumn($column['FIELDNAME'], $column['TABLECODE']);
            if (isset($column['EXPRESSION']) && $column['EXPRESSION'] <> '') {
                $col_name = $column['TABLECODE'] . '_' . $column['FIELDNAME'];
                $field = '(' . $column['EXPRESSION'] . ') ' . $col_name;
            }
        } else {
            // derived field
            $col_name = $column['TABLECODE'] . '_' . $column['FIELDNAME'];
            $expresion = $queryrec['COLUMNS'][$column['DERIVED_COL_NAME']]['EXPRESSION_ONLY'];
            $field = $fieldinstance->queryFieldColumn($column['FIELDNAME'], $column['TABLECODE'], '(' . $expresion . ') ' . $col_name);
        }
        return $field;
    }

    function getRecords() {
        $DBQY = &getTool('DBQY');
        // Support var for one-line totaling display
        $this->oneline_tot_template = array();
        $this->oneline_tot_func_cols = array();
        $querycode = $this->reportrec['QUERYCODE'];
        $DBQY->include_empty_answers = false; // to avoid inclusion of empty answers in query condition
        $queryrec = $DBQY->getquery('ANRP', $querycode);
        $DBQY->include_empty_answers = true; // restore defaulkt value for empty answers
        // get the groups and functions
        $selected_fields = array();
        $sort_fields = array();
		
        foreach ($this->groups as $groupid => $group) {
            $fieldinstance = getToolObject('DBFT', $group['FIELDTYPE']);
            $field = $this->determine_field_name_or_expr($fieldinstance, $group, $queryrec);
            if (!in_array($field, $selected_fields)) {
                $selected_fields[] = $field;
            }
            $sort_fields[] = $fieldinstance->queryFieldSorts($group['FIELDNAME'], $group['TABLECODE'], 'ASC');
        }
        foreach ($this->functions as $groupid => $functions) {
            foreach ($functions as $function) {
                $fieldinstance = getToolObject('DBFT', $function['FIELDTYPE']);
                $field = $this->determine_field_name_or_expr($fieldinstance, $function, $queryrec);
                if (!in_array($field, $selected_fields)) {
                    $selected_fields[] = $field;
                }
                // Memorizes column names included in each function type
                $func_type = array();
                if ($function['SUM'] == CLX_NUM_ONE) {
                    $func_type[] = 'SUM';
                }
                if ($function['AVG'] == CLX_NUM_ONE) {
                    $func_type[] = 'AVG';
                }
                if ($function['MIN'] == CLX_NUM_ONE) {
                    $func_type[] = 'MIN';
                }
                if ($function['MAX'] == CLX_NUM_ONE) {
                    $func_type[] = 'MAX';
                }
                if ($function['COUNT'] == CLX_NUM_ONE) {
                    $func_type[] = 'COUNT';
                }
                if (count($func_type) > CLX_NUM_ZERO) {
                    foreach ($func_type as $funct_t) {
                        if (!isset($this->oneline_tot_func_cols[$funct_t]))
                            $this->oneline_tot_func_cols[$funct_t] = array();
                        $this->oneline_tot_func_cols[$funct_t][] = $function['TABLECODE'] . '_' . $function['FIELDNAME'];
                    }
                }
            }
        }


        foreach ($this->columns as $column) {
            if (!array_key_exists('SORTING', $column)) {
                $column['SORTING'] = '';
            }
            $fieldinstance = getToolObject('DBFT', $column['FIELDTYPE']);
            $field = $this->determine_field_name_or_expr($fieldinstance, $column, $queryrec);
            if (!in_array($field, $selected_fields)) {
                $selected_fields[] = $field;
            }
            if ($column['SORTING'] !== 'NONE') {
                $field = $fieldinstance->queryFieldSorts($column['FIELDNAME'], $column['TABLECODE'], $column['SORTING']);
                $sort_fields[] = $field;
            }
            $this->oneline_tot_template[$column['TABLECODE'] . '_' . $column['FIELDNAME']] = '';
        }
        $selectclause = toString($selected_fields);
        $sortclause = toString($sort_fields);
        //Collect form filters
        $filter_crits = $this->getQueryFilter($this->reportrec['SEARCH_FORM']);
	//	Echo $filter_crits;
        if (strlen($filter_crits) > CLX_NUM_ZERO) {
            if (strlen($queryrec['WHERECLAUSE']) > CLX_NUM_ZERO) {
                $queryrec['WHERECLAUSE'].= ' AND ' . $filter_crits;
            } else {
                $queryrec['WHERECLAUSE'].= $filter_crits;
            }
        }
        $sql = 'SELECT COUNT(0) FROM ' . $queryrec['FROMCLAUSE'];
        if (strlen($queryrec['WHERECLAUSE']) > CLX_NUM_ZERO) {
            $sql .= ' WHERE ' . $queryrec['WHERECLAUSE'];
        }
        if (strlen($queryrec['GROUPCLAUSE']) > CLX_NUM_ZERO) {
            $sql .= ' GROUP BY ' . $queryrec['GROUPCLAUSE'];
        }
        if (strlen($sortclause) > CLX_NUM_ZERO) {
            $sql .= ' ORDER BY ' . $sortclause;
        }

        $this->total_rows = ClxCommon::$db->getOne($sql);
        if ($this->total_rows > CLX_NUM_90) {
		 	$max=$this->total_rows * CLX_NUM_THREE;
			if (ini_get('max_execution_time')<$max)
				UserResponse::setRunTimeLimit($max );
        }
        // Build the sql
		if (empty($selectclause))
			$selectclause='*';
        $sql = 'SELECT ' . $selectclause . ' FROM ' . $queryrec['FROMCLAUSE'];
        $sql .= (strlen($queryrec['WHERECLAUSE']) > CLX_NUM_ZERO) ? ' WHERE ' . $queryrec['WHERECLAUSE'] : CLX_CHAR_EMPTY_STRING;
        $sql .= (strlen($queryrec['GROUPCLAUSE']) > CLX_NUM_ZERO) ? ' GROUP BY ' . $queryrec['GROUPCLAUSE'] : CLX_CHAR_EMPTY_STRING;
        $sql .= (strlen($sortclause) > CLX_NUM_ZERO) ? ' ORDER BY ' . $sortclause : '';
        return ClxCommon::$db->executeQuery($sql);
    }

    /**
     * Loads Filter Criteria
     * @param unknown_type $search_form
     * @return string
     */
    function getQueryFilter($search_form) {
        if (strlen($search_form) === CLX_NUM_ZERO) {
            return CLX_CHAR_EMPTY_STRING;
        }
        $DBQY = &getTool('DBQY');
        $UIFM = &getTool('UIFM');
        $crits = array();
        $form_fields = array();
        $fls = $UIFM->getFormFields($search_form);
        while (($fl = ClxCommon::$db->getNextRecord($fls))) {
            $form_fields[$fl['TABLECODE'] . $fl['FIELD_TABLECODE'] . $fl['FIELDNAME']] = $fl;
        }
        $tbs = $DBQY->getQueryTables($this->reportrec['QUERYCODE']);
        while (( ClxCommon::$db->getNextRecord($tbs))) {
            foreach ($form_fields as $val) {
                $fieldinstance = getToolObject('DBFT', $val['FIELDTYPE']);
                if (strlen($val['FIELD_TABLECODE'])) {
                    $criteria = $fieldinstance->setQuickFilters($val['FIELD_TABLECODE'], $val['FIELDNAME'], $val, UserRequest::$post);
                } else {
                    $criteria = $fieldinstance->setQuickFilters($val['TABLECODE'], $val['FIELDNAME'], $val, UserRequest::$post);
                }
                if (!empty($criteria)) {
                    $crits[] = $criteria;
                }
            }
        }//end tb
        $crits = implode(' AND ', $crits);
        if (strlen($crits) > CLX_NUM_ZERO) {
            $crits = '(' . $crits . ')';
        }
        return $crits;
    }

    /**
     *
     * @param string $fieldname
     * @param array $record
     * @param array $fieldrec
     * @param bool $append_quote
     * @return string
     */
    function fieldFormat($fieldname, &$record, &$fieldrec, $append_quote = false) {

        $fieldinstance = getToolObject('DBFT', $fieldrec['FIELDTYPE']);
        $fieldrec['DBQF_FIELD_FORMAT'] = $fieldrec['FORMAT'];
		
        $val = $fieldinstance->formatField($fieldname, $fieldrec, $record);
        ClxFreeReturn($append_quote);
        return $val;
    }

    /**
     * Returns white space for groups title indent
     * @param int $level
     * @param varchar $blk_space = ' '
     * @param int $multiplier = 3
     */
    function get_indent_space($level, $blk_space = ' ', $multiplier = CLX_NUM_THREE) {
        if ($level <= CLX_NUM_ONE) {
            $response = '';
        } else {
            $response = str_repeat($blk_space, ($level - CLX_NUM_ONE) * $multiplier);
        }
        return $response;
    }

    /**
     * Executes Report Events
     * @param unknown_type $reportid
     */
    protected function runRecordEvents($reportid, &$record) {
        $msg = array();
        $EVNT = &getTool('EVNT');
        if ($EVNT->OwnerEventExists('ANRP', $reportid, 'RPBR')) {
            $EVNT->execute5('ANRP', $reportid, 'RPBR', $record, $msg); //  Listing Load Record
        }
    }

	function get_skin_html($skin_file = null) {
		if(is_null($skin_file)) {
			//$skin_file = 'pop.html';
			$skin_file = 'report.html';
		}
		$variables = array();
		$variables['PAGE_TITLE'] = $record['IDNAME'];
		$variables['JAVA_SCRIPT_1'] = '';
		$variables['JAVA_SCRIPT_2'] = '';
		$variables['JAVA_SCRIPT_3'] = '';
		$variables['JAVA_SCRIPT_4'] = '';
		$variables['JAVA_SCRIPT_5'] = '';
		$variables['STYLE_SHEET'] = sprintf('<link href=' . singleQuote('%s') . ' rel=' . singleQuote('stylesheet') . ' type=' . singleQuote('text/css') . '>', kWebSite . kPathToSkins . $_SESSION['skin'] . '/styles.css');
		$variables['PAGETIME'] = rounder((microtime_float() - $this->output_to_file['stime']), CLX_NUM_FOUR);
		$variables['PAGEPEAKSIZE'] = rounder((memory_get_peak_usage(false) / CLX_NUM_1024), CLX_NUM_TWO) . ' Kb';
		$variables['CONTENTS'] = self::SKIN_CONT_SEP_TOKEN;
		$contents = getSkinForm($skin_file, $variables);
		unset($variables);
		return $contents;
	}
	
	function get_begin_content() {
		if($this->code !== 'RPHT') {
			return CLX_CHAR_EMPTY_STRING;
		}
		$contents = $this->get_skin_html();
		return strstr($contents, self::SKIN_CONT_SEP_TOKEN, true) . '<!-- ANRJ Begin -->';
	}
	
	function get_end_content() {
		if($this->code !== 'RPHT') {
			return CLX_CHAR_EMPTY_STRING;
		}
		$contents = $this->get_skin_html();
		return '<!-- ANRJ End -->' . str_replace(self::SKIN_CONT_SEP_TOKEN, '', strstr($contents, self::SKIN_CONT_SEP_TOKEN)) ;
	}

	function output_open_file() {
		$this->errormsg = '';
		$this->output_to_file['stime'] = microtime_float();
		$this->output_to_file['fp'] = new clx_file($this->output_to_file['file_name'], 'w');
		$this->output_to_file['fp']->open();
		if ($this->output_to_file['fp']->hasError()) {
			$this->errormsg = '[RPFT-HTM-010] Cannot write directory/file: ' . $this->output_to_file['file_name'];
			// Se debe detener el procesamiento del reporte
			$this->error = true;
		}
	}

	function output_init() {
		if($this->is_output_to_file()) {//dump directly to a file
			$this->output_buffer = false;
			$this->output_open_file();
			if($this->error === false) {
				$this->output_add($this->get_begin_content());
			}
			return;
		}
		//dump directly to screen
		if($this->code === 'RPHT') {
			ob_start();
			return;
		}
		$this->contents = '';
	}

	function output_end() {
		if ($this->output_buffer === true) {
			return;
		}
		if($this->is_output_to_file()) {
			$this->output_add($this->get_end_content());
			$this->output_to_file['fp']->close();
		}
	}
	
	function output_add($out_content) {
		if ($this->output_buffer === true) {
			$this->contents .= $out_content;
			return;
		}
		if($this->is_output_to_file()) {
			//$this->output_to_file['fp']->write(decode_string($out_content));
			$this->output_to_file['fp']->write(utf8_decode($out_content));
		} else {
			if($this->code === 'RPHT') {
				echo $out_content;
			}
		}
	}
	
	function is_output_to_file() {
		return (isset($this->output_to_file['file_name']));
	}
}
