<?php

/**
 * Object Class
 * For more information contact  support@alenet.com
 * @package		Clientix.Tool.Analytics
 * @abstract		Report Type Object
 * @author       $Author: cramirez $
 * @copyright    Alenet, Inc.
 * @since        $Date: 2018-12-10 18:11:05 -0400 (lu., 10 dic. 2018) $
 * @version        $Rev: 10412 $
 *
 * CHANGE LOG
 *  Oracle uppercase record changed
 *  Included label
 * 		detail( ) method: strip_tags() added to avoid special characters and html code on excel
 * 		remove_unwanted_characters() new method to remove html entities and reduce the single quotes on alphabetic strings
 * 				this method is called from detail() and insertbreak() methods. Also method documentation has been added
 */
if (!class_exists('rpft') && !defined('clxInstallProcess'))
    include_once kPathToObjects . 'rpft.php';

class rpft_excel extends rpft {

    var $with;
    var $contents;
    var $needheaders;
    var $file_save = true;

    /**
     * Constructor
     */
    function __construct() {
	$this->code = 'RPEX';
	$this->name = 'Microsoft Excel';
	$this->type = 'RPFT';
	$this->version = '0.3.1.0';
	$this->description = 'Microsoft Excel Report Format';
	$this->windowurl = 'window.php';
	$this->editPanel = array();
    }

    /**
     * Initialices the report (only once)
     */
    function start() {
	$this->width = count($this->groups) + count($this->columns);
	// 2018-12-10 CR - Caso 1003486 - GB-Rep BATCH no arranca
	//$this->contents='';
	$this->output_init();
	if($this->error === true) {
		return;
	}
	$title = dict($this->reportrec['TITLE']);
	$title = str_replace(CLX_CHAR_NL, ' ', $title);
	$title = str_replace(CLX_CHAR_CR, ' ', $title);
	$title = str_replace(CLX_CHAR_TAB, ' ', $title);
	//$this->contents.=$this->Report_Headers();
	$this->output_add($this->Report_Headers());
	if (strlen($title) > CLX_NUM_ZERO) {
	    //$this->contents .= $title . CLX_CHAR_NL;
	    $this->output_add($title . CLX_CHAR_NL);
	}
	$subtitle = dict($this->reportrec['SUBTITLE']);
	$subtitle = str_replace(CLX_CHAR_NL, ' ', $subtitle);
	$subtitle = str_replace(CLX_CHAR_CR, ' ', $subtitle);
	$subtitle = str_replace(CLX_CHAR_TAB, ' ', $subtitle);
	if (strlen($subtitle) > CLX_NUM_ZERO) {
	    //$this->contents .= $subtitle . CLX_CHAR_NL;
	    $this->output_add($subtitle . CLX_CHAR_NL);
	}
	$this->needheaders = true;
    }

    /**
     * Inserts one line
     *
     * @param array $record
     * @return void
     */
    function detail($record, $total_line = false) {

	if ($this->reportrec['SHOW_DETAIL'] == true || $total_line == true) {


	    if (!defined('NEW_LINE_SECUENCE'))
		define('NEW_LINE_SECUENCE', '{NEW|LINE}');
	    $cur_line = array();
	    $cur_col = CLX_INIT_ZERO;
	    $max_cols = CLX_INIT_ZERO;
	    if (!empty($this->needheaders))
		$this->headers();
	    $total = count($this->groups);
	    for ($i = CLX_INIT_ZERO; $i < $total; $i++) {
		$cur_col += CLX_NUM_ONE;
	    }
	    foreach ($this->columns as $fieldrec) {
		if ($cur_col > $max_cols)
		    $max_cols = $cur_col;
		$fn = $fieldrec['TABLECODE'] . '_' . $fieldrec['FIELDNAME'];
		if ($total_line && $record[$fn] === '') {
		    // Summary line but no function on this column
		    $val = '';
		} else {
		    $val = $this->fieldFormat($fn, $record, $fieldrec, true);
		    if ($fieldrec['FIELDTYPE'] === 'INTN' || $fieldrec['FIELDTYPE'] === 'DECN') {
			$val = str_replace('(', '', $val);
			$val = str_replace(')', '', $val);
		    }
		    $val = $this->replace_newline_characters($val);
		    $val = $this->remove_unwanted_characters($val);
		    if ($fieldrec['FIELDTYPE'] == 'CHAR' and substr(ltrim($val), CLX_NUM_ZERO, CLX_NUM_ONE)=='0') {
			$val="`".$val;
		    }
		}
		// Split posible New Line delimiter into one o more lines on current column
		$a_val = explode(NEW_LINE_SECUENCE, $val);
		$total = count($a_val);
		for ($j_line = CLX_INIT_ZERO; $j_line < $total; $j_line++) {
		    if (!isset($cur_line[$j_line])) {
			$cur_line[$j_line] = array();
			for ($cur_col2 = CLX_INIT_ZERO; $cur_col2 <= $max_cols; $cur_col2++) {
			    if (!isset($cur_line[$j_line][$cur_col2]))
				$cur_line[$j_line][$cur_col2] = '';
			}
		    }
		    $val = $this->remove_unwanted_characters($a_val[$j_line]);
		    $cur_line[$j_line][$cur_col] = $val;
		}
		$cur_col ++;
	    }
	    // Flush line (maybe line has been splitted)
	    $total = count($cur_line);
	    for ($j_line = CLX_INIT_ZERO; $j_line < $total; $j_line++) {
		//$this->contents .= implode(CLX_CHAR_TAB, $cur_line[$j_line]) . CLX_CHAR_NL;
		$this->output_add(implode(CLX_CHAR_TAB, $cur_line[$j_line]) . CLX_CHAR_NL);
	    }
	}
//	$this->contents.= $this->Report_Footers();
    }

    /**
     * Replace new line secuences on alphabetic strings.
     * <BR> & <BR/> ---> {NEW|LINE}
     *
     * @param text $val
     * @return text
     */
    function replace_newline_characters($val) {
	return str_replace(str_repeat(NEW_LINE_SECUENCE,CLX_NUM_TWO), NEW_LINE_SECUENCE, str_replace(str_repeat(NEW_LINE_SECUENCE,CLX_NUM_TWO), NEW_LINE_SECUENCE, str_replace(array('<BR>','<BR/>','<BR />','<br>', '<br/>', '<br />'), NEW_LINE_SECUENCE, $val)));
    }

    /**
     * Remove HTML tags, HTML entities and reduce the single quotes on alphabetic strings
     *
     * @param text $val
     * @return text
     */
    function remove_unwanted_characters($val) {
	// Deletes HTML tags
	$val = strip_tags($val);
	//Delete line breaks on data
	$val = str_replace(array(CLX_CHAR_NL,CLX_CHAR_CR,CLX_CHAR_TAB), CLX_CHAR_EMPTY_STRING, $val);
	// Deletes HTML entities
	$val = html_entity_decode($val);
	// Delete first simple quote if the first chacarter isn\'t a number (just to avoid garbagge excess)
	if (substr($val, CLX_NUM_ZERO, CLX_NUM_ONE) == CLX_CHAR_SINGLE_QUOTE) {
	    if (!is_numeric(substr($val, CLX_NUM_ONE, CLX_NUM_ONE))) {
		$val = substr($val, CLX_NUM_ONE);
	    }
	}
        // Delete first character if it's double quote but the final character isn't and viceversa, to avoid Excel file to be broken at this point
	if ( (substr($val, CLX_NUM_ZERO, CLX_NUM_ONE) === CLX_CHAR_DOUBLE_QUOTE && substr($val, -CLX_NUM_ONE) !== CLX_CHAR_DOUBLE_QUOTE) || (substr($val, CLX_NUM_ZERO, CLX_NUM_ONE) !== CLX_CHAR_DOUBLE_QUOTE && substr($val, -CLX_NUM_ONE) === CLX_CHAR_DOUBLE_QUOTE)) {
            $val = trim($val, CLX_CHAR_DOUBLE_QUOTE);
	}
	return $val;
    }

    /**
     * Inserts header for a control break
     *
     * @param text $groupid, array $record
     * @return void
     */
    function insertbreak($groupid, $record) {
	if ($this->needheaders === true)
	    $this->headers();
	$grouprec = $this->groups[$groupid];
	$level = $grouprec['SEQNUM'];
	for ($i = CLX_NUM_ONE; $i < $level; $i++) {
	    //$this->contents .= CLX_CHAR_TAB;
	    $this->output_add(CLX_CHAR_TAB);
	}
	$fn = $grouprec['TABLECODE'] . '_' . $grouprec['FIELDNAME'];
	$linelabel_code = $grouprec['FIELDLISTLABEL'];
	if (isset($grouprec['RG_FIELDLISTLABEL']) && !is_null($grouprec['RG_FIELDLISTLABEL']) && $grouprec['RG_FIELDLISTLABEL'] !== '') {
	    $linelabel_code = $grouprec['RG_FIELDLISTLABEL'];
	}
	$linelabel = CLX_CHAR_SINGLE_QUOTE . dict($linelabel_code) . ': ';
	$val = $linelabel . $this->fieldFormat($fn, $record, $grouprec);
	$val = $this->remove_unwanted_characters($val);
	if ( substr($val,0,1)=='0') {
	    $val="`".$val;
	}
                
	//$this->contents .= CLX_CHAR_NL . $val. CLX_CHAR_NL. CLX_CHAR_NL;
        //$this->contents .= $val. CLX_CHAR_NL;// . CLX_CHAR_NL;
        $this->output_add($val. CLX_CHAR_NL);// . CLX_CHAR_NL;
    }

    /**
     * Inserts function's result value with indent spaces at left
     *
     * @param text $groupid, variant $value, integer $functionid
     * @return void
     */
    function insertFunction($groupid, $value, $functionid) {
	if (!array_key_exists($groupid, $this->groups)) {
	    $grouprec = array();
	    $level = CLX_INIT_ZERO;
	} else {
	    $grouprec = $this->groups[$groupid];
	    $level = $grouprec['SEQNUM'];
	}
	for ($i = CLX_NUM_ONE; $i < $level; $i++) {
	    //$this->contents .= CLX_CHAR_TAB;
	    $this->output_add(CLX_CHAR_TAB);
	}
	//$this->contents .= $value . CLX_CHAR_NL;
	$this->output_add($value . CLX_CHAR_NL);
    }

    /**
     * Inserts blank line
     *
     * @return void
     */
    function insertBlankLine() {
	//$this->contents .= CLX_CHAR_NL;
	$this->output_add(CLX_CHAR_NL);
    }

    /**
     * TErminates report: save it or inserts the appropiate HTML header to send results to the output stream
     *
     * @param void
     * @return text
     */
    function finish() {
        if($this->is_output_to_file()) {
			$this->output_end();
			return CLX_CHAR_EMPTY_STRING;
		}
		if ($this->file_save == true) {
			$filename = kPathToTemp . 'rep' . str_replace('.', '', microtime(true)) . '.xls';
			$fl = new clx_file($filename, 'a');
			$fl->open();
			$this->contents = utf8_decode($this->contents);
			$fl->write($this->contents);
			$fl->write($this->Report_Footers());
			$fl->close();
			$this->contents = '<html><head><link href="' . skinPath() . '/styles.css" rel="styleshee" type="text/css"></head><body>';
			$this->contents .= note('<a href="' . $filename . '" target="_blank">' . dict('GEN_LBL_DOWNLOAD_THE_FILE') . '</a><br>');
			$this->contents .= '</body></html>';
		} else {
			/*
			 * Caso 1002542 - GB-Reporte Fideicomiso Garantia Autos
			 * Caso 1002794 - GB-Reporte Saldo de Patrimonio
			 * Se verifica si el reporte es en batch para sólo hacer 'echo...' del contenido del archivo Excel,
			 * en vez de hacer las llamadas a 'header()' antes del 'echo...'
			 */
			$this->contents = utf8_decode($this->contents);
			$batchoption = (isset(UserRequest::$request['ANRP_rbat']) ? UserRequest::$request['ANRP_rbat'] : '');
			if ($batchoption !== '1') {
				header('Content-type: application/vnd.ms-excel');
				header('Content-Disposition: attachment; filename=' . $this->reportrec['IDNAME'] . '.xls');
				header('Pragma: no-cache');
				header('Expires: 0');
				header('Cache-control: private');
				echo $this->contents;
				echo $this->Report_Footers();
				salir();
			}
		}
		return $this->contents;
    }

    function Report_Headers() {
	$ANHF = &getTool('ANHF');
	$parametros = array('type_report' => 'EXCEL', 'search' => $this->reportrec['HEADERCODE'], 'position' => 'Header');
	$position = 'H';
	$value = $ANHF->search_Data($parametros);
	if (empty($value[$position . 'LEFT']))
		$value[$position . 'LEFT']='';
	if (empty($value[$position . 'CENTER']))
		$value[$position . 'CENTER']='';
	if (empty($value[$position . 'RIGHT']))
		$value[$position . 'RIGHT']='';
	return clx_change_especial_char_html($ANHF->Merge(clx_keywords_simple(strip_tags($value[$position . 'LEFT'])),UserRequest::$request)) . CLX_CHAR_TAB .
		clx_change_especial_char_html($ANHF->Merge(clx_keywords_simple(strip_tags($value[$position . 'CENTER'])),UserRequest::$request)) . CLX_CHAR_TAB .
		clx_change_especial_char_html($ANHF->Merge(clx_keywords_simple(strip_tags($value[$position . 'RIGHT'])),UserRequest::$request)) . CLX_CHAR_NL;
    }

    function Report_Footers() {

	$ANHF = &getTool('ANHF');
	$parametros = array('type_report' => 'EXCEL', 'search' => $this->reportrec['HEADERCODE'], 'position' => 'FOOTER');

	$value = $ANHF->makepanel($parametros);
	$position = 'F';
	if (empty($value[$position . 'LEFT']))
		$value[$position . 'LEFT']='';
	if (empty($value[$position . 'CENTER']))
		$value[$position . 'CENTER']='';
	if (empty($value[$position . 'RIGHT']))
		$value[$position . 'RIGHT']='';
	return clx_change_especial_char_html($ANHF->Merge(clx_keywords_simple(strip_tags($value[$position . 'LEFT'])),UserRequest::$request)) . CLX_CHAR_TAB .
		clx_change_especial_char_html($ANHF->Merge(clx_keywords_simple(strip_tags($value[$position . 'CENTER'])),UserRequest::$request)) . CLX_CHAR_TAB .
		clx_change_especial_char_html($ANHF->Merge(clx_keywords_simple(strip_tags($value[$position . 'RIGHT'])),UserRequest::$request)) . CLX_CHAR_NL;

    }

    /**
     * Inserts column headers
     *
     * @param void
     * @return void
     */
    function headers() {
	if ($this->reportrec['SHOW_DETAIL'] == true) {
	    foreach ($this->groups as $groupid => $grouprec) {
		$c = '';
		$c = str_replace(CLX_CHAR_NL, '', $c);
		$c = str_replace(CLX_CHAR_CR, '', $c);
		$c = str_replace(CLX_CHAR_TAB, '', $c);
		//$this->contents .= $c . CLX_CHAR_TAB;
		$this->output_add($c . CLX_CHAR_TAB);
	    }
	    foreach ($this->columns as $fieldindex => $fieldrec) {
		$clabel = $fieldrec['FIELDLISTLABEL'];
		if (isset($fieldrec['ANRC_FIELDLISTLABEL']) && !is_null($fieldrec['ANRC_FIELDLISTLABEL']) && $fieldrec['ANRC_FIELDLISTLABEL'] !== '') {
		    $clabel = $fieldrec['ANRC_FIELDLISTLABEL'];
		}
		$c = dict($clabel);
		$c = str_replace(CLX_CHAR_NL, '', $c);
		$c = str_replace(CLX_CHAR_CR, '', $c);
		$c = str_replace(CLX_CHAR_TAB, '', $c);
		//$this->contents .= $c . CLX_CHAR_TAB;
		$this->output_add($c . CLX_CHAR_TAB);
	    }
	    //$this->contents .= CLX_CHAR_NL;
	    $this->output_add(CLX_CHAR_NL);
	    $this->needheaders = false;
	}
    }

}
