<?php

/**
 * Object Class
 * For more information contact  support@alenet.com
 *
 * @package		Clientix.Tool.Analytics
 * @abstract		Report Type Object
 * @author       $Author: cramirez $
 * @copyright    Alenet, Inc.
 * @since        $Date: 2018-12-10 18:11:05 -0400 (lu., 10 dic. 2018) $
 * @version        $Rev: 10412 $
 *
 * CHANGE LOG
 *  Oracle uppercase record changed
 *  Included label
 */
if (!class_exists('rpft') && !defined('clxInstallProcess')) {
    include_once kPathToObjects . 'rpft.php';
}

class rpft_html extends rpft {

    var $table;
    var $width;
    var $needheaders;
    var $linecount;

    // CONSTRUCTOR //
    function __construct() {
        $this->code = 'RPHT';
        $this->name = 'HTML';
        $this->type = 'RPFT';
        $this->version = '0.3.1.0';
        $this->description = 'HTML Report Format';
        $this->windowurl = 'pop.php';
        $this->online = true;
        $this->editPanel = array();
        $this->output_buffer = false;
        $this->notInitHtml=0;
    }

    function start() {
        $contenidoHTML = '';
        $this->width = count($this->columns);
        if (!array_key_exists('TITLE', $this->reportrec)) {
            $this->reportrec['TITLE'] = '';
        }
        if (!array_key_exists('SUBTITLE', $this->reportrec)) {
            $this->reportrec['SUBTITLE'] = '';
        }
        $title = dict($this->reportrec['TITLE']);
        $subtitle = dict($this->reportrec['SUBTITLE']);

        if ($this->output_buffer === true) {
            echo $this->Report_Headers();
            $this->table = new htmlTable($this->width);
			// 2018-11-16 CR - Caso 1003486 - GB-Rep BATCH no arranca
            // $table->setProps(array(CLX_PROP_WIDTH => CLX_PROP_100PCT));
            $this->table->setProps(array(CLX_PROP_WIDTH => CLX_PROP_100PCT));
            if (strlen($title) > CLX_NUM_ZERO) {
                $this->table->addCell($title, array(CLX_PROP_CLASS => 'title', 'colspan' => $this->width));
            }
            if (strlen($subtitle) > CLX_NUM_ZERO) {
                $this->table->addCell($subtitle, array(CLX_PROP_CLASS => 'subtitle', 'colspan' => $this->width));
            }
        } else {
            if ($this->notInitHtml==0){
				$style=file_get_contents(kPathToSkins . $_SESSION['skin']. '/styles.css');
				$style='<style>'.$style.'</style>';	
                $contenidoHTML = '<html><head><title>' . $title . '</title>'. 
				'<meta charset="UTF-8">'.
				$style. 
				'</head>' .
				$this->Report_Headers() . 
//				'<script>$(\'head\').append(\'<link rel="stylesheet" type="text/css" href="../'.kPathToSkins . $_SESSION['skin']. '/styles.css'.'">\');</script>'.
				' <table width="100%"><tr><td class=' . singleQuote('title') . ' colspan=' . singleQuote($this->width) . '>' . $title . '</td></tr>';
                //$contenidoHTML = $this->Report_Headers() . ' <table width="100%"><tr><td class=' . singleQuote('title') . ' colspan=' . singleQuote($this->width) . '>' . $title . '</td></tr>';
                if (!empty($subtitle)) {
                    $contenidoHTML .= '<tr><td class=' . singleQuote('subtitle') . ' colspan=' . singleQuote($this->width) . '>' . $subtitle . '</td></tr>';
                }
            } else {
				$contenidoHTML = '<table width="100%">
    <tr><td class="title" colspan="'.$this->width.'">'.$title.'</td></tr>
    <tr><td class="subtitle" colspan="'.$this->width.'">'.$subtitle.'</td></tr>';
            }
			$this->output_init();
			if($this->error === false) {
				$this->output_add($contenidoHTML);
			}
			$contenidoHTML = '';
        }
		if($this->error === false) {
			$this->insertBlankLine();
			$this->needheaders = true;
			$this->linecount = CLX_INIT_ZERO;
		}
    }

    /**
     * Format number
     * @param string $val
     * @param type $column
     * @return string
     */
    function num_fmt($val, $column) {
        if ($column['FIELDTYPE'] === 'DECN' || $column['FIELDTYPE'] === 'INTN') {
            $val = '<div style="text-align: right">' . $val . '</div>';
        }
        return $val;
    }

    /**
     * (non-PHPdoc)
     * @see rpft::detail()
     */
    function detail($record, $total_line = false) {
        $contenidoHTML = '';
        if (!empty($this->reportrec['SHOW_DETAIL']) || !empty($total_line)) {
            if (!empty($this->needheaders)) {
                $this->headers();
            }
            if (!empty($total_line)) {
                $class = 'datatot';
            } else {
                $class = ($this->linecount % CLX_NUM_TWO == CLX_NUM_ZERO) ? CLX_CLASS_DATA : CLX_CLASS_DATAALT;
            }
            if ($this->output_buffer === true) {
                foreach ($this->columns as $fieldindex => $fieldrec) { // add formating here!!!!
                    $fn = $fieldrec['TABLECODE'] . '_' . $fieldrec['FIELDNAME'];
                    $val = ($total_line && $record[$fn] === '') ? CLX_CHAR_EMPTY_STRING : $this->num_fmt($this->fieldFormat($fn, $record, $fieldrec), $fieldrec);
                    $this->table->addCell($val, array(CLX_PROP_CLASS => $class));
                }
            } else {
                $contenidoHTML = '<tr>';
				
                foreach ($this->columns as $fieldindex => $fieldrec) {
                    $fn = $fieldrec['TABLECODE'] . '_' . $fieldrec['FIELDNAME'];
                    $val = ($total_line && $record[$fn] === '') ? CLX_CHAR_EMPTY_STRING : $this->num_fmt($this->fieldFormat($fn, $record, $fieldrec), $fieldrec);
                    $contenidoHTML.='<td class="' . $class . '">' . $val . '</td>';
                }
                $contenidoHTML.= '</tr>';
				$this->output_add($contenidoHTML);
            }
            $this->linecount++;
        }
    }

    function insertbreak($groupid, $record) {

        if ($this->needheaders) {
            $this->headers();
        }
        $grouprec = $this->groups[$groupid];
        $level = $grouprec["SEQNUM"];
        $fn = uppercase($grouprec['TABLECODE'] . '_' . $grouprec['FIELDNAME']);

        $val = $this->fieldFormat($fn, $record, $grouprec, false);
        $this->linecount = CLX_INIT_ZERO;
        $linelabel = $grouprec["FIELDLISTLABEL"];
        if (isset($grouprec["RG_FIELDLISTLABEL"]) && !is_null($grouprec["RG_FIELDLISTLABEL"]) && $grouprec["RG_FIELDLISTLABEL"] != '') {
            $linelabel = $grouprec["RG_FIELDLISTLABEL"];
        }
        $val = $this->get_indent_space($level, CLX_CHAR_HTML_NBSP) . dict($linelabel) . ': ' . $val;
        if ($this->output_buffer === true) {
            //$this->table->addCell($val, "colspan", $this->width, "class", "datahigh");
            $this->table->addCell($val, array(CLX_PROP_CLASS => 'datahigh', CLX_PROP_COLSPAN => $this->width));
        } else {
            $contenidoHTML = '';
            $contenidoHTML .= '<tr><td colspan=' . ($this->width) . ' class="datahigh">' . $val . '</td></tr>';
			$this->output_add($contenidoHTML);
        }
        $this->linecount = 0;
    }

    /**
     * Inserts blank line
     */
    function insertBlankLine() {
        $class = 'datablk';
        $value = '<br>';
        if ($this->output_buffer === true) {
            $this->table->addCell($value, array(CLX_PROP_CLASS => $class, CLX_PROP_COLSPAN => $this->width));
        } else {
            $contenidoHTML = '';
            $contenidoHTML.= '<tr><td colspan="' . $this->width . '" class="' . $class . '">' . $value . '</td></tr>';
			$this->output_add($contenidoHTML);
        }
    }

    /**
     * Inserts Function
     * @param int $groupid
     * @param string $value
     * @param int $functionid
     */
    function insertFunction($groupid, $value, $functionid) {
        $class = 'datatot';
        if (!array_key_exists($groupid, $this->groups)) {
            $grouprec = array();
            $level = CLX_INIT_ZERO;
        } else {
            $grouprec = $this->groups[$groupid];
            $level = $grouprec['SEQNUM'];
        }
        $value = $this->get_indent_space($level, CLX_CHAR_HTML_NBSP) . $value;
        if ($this->output_buffer === true) {
            $this->table->addCell($value, array(CLX_PROP_CLASS => $class, 'colspan' => $this->width));
        } else {
            $contenidoHTML = '<tr><td colspan=' . ($this->width) . ' class="' . $class . '">' . $value . '</td></tr>';
			$this->output_add($contenidoHTML);
        }
    }

    function finish() {
        $contenidoHTML = '';
        if ($this->output_buffer === true) {
            $variables = array();
            $variables['PAGE_TITLE'] = dict($this->reportrec['TITLE']);
            $variables['STYLE_SHEET'] = sprintf('<link href=' . singleQuote('%s') . ' rel=' . singleQuote('stylesheet') . ' type=' . singleQuote('text/css') . '>', kPathToSkins . $_SESSION['skin'] . '/styles.css');
            $variables['CONTENTS'] = $this->table->source() . $this->Report_Footers();
            return getSkinForm('report.html', $variables);
        } else {
            $contenidoHTML.= '</table>' . $this->Report_Footers() . '</body></html>';
			$this->output_add($contenidoHTML);
			$this->output_end();
            /*$variables = array();
            $variables['PAGE_TITLE'] = dict($this->reportrec['TITLE']);
            $variables['STYLE_SHEET'] = sprintf('<link href=' . singleQuote('%s') . ' rel=' . singleQuote('stylesheet') . ' type=' . singleQuote('text/css') . '>', kPathToSkins . $_SESSION['skin'] . '/styles.css');
            $variables['CONTENTS'] = ob_get_clean();
            return getSkinForm('report.html', $variables);*/
            return CLX_CHAR_EMPTY_STRING;
        }
    }

    function Report_Headers() {
        $ANHF = &getTool('ANHF');
        return $ANHF->makepanel(array('type_report' => 'HTML', 'search' => $this->reportrec['HEADERCODE'], 'position' => 'Header'));
    }

    function Report_Footers() {
        $ANHF = &getTool('ANHF');
        return $ANHF->makepanel(array('type_report' => 'HTML', 'search' => $this->reportrec['FOOTERCODE'], 'position' => 'Footer'));
    }

    /**
     * (non-PHPdoc)
     * @see rpft::headers()
     */
    function headers() {
        $contenidoHTML = '';
        if ($this->reportrec['SHOW_DETAIL']) {
            if ($this->output_buffer === true) {
                foreach ($this->columns as $fieldindex => $fieldrec) {
                    $clabel = $fieldrec['FIELDLISTLABEL'];
                    if (isset($fieldrec['ANRC_FIELDLISTLABEL']) && !is_null($fieldrec['ANRC_FIELDLISTLABEL']) && $fieldrec['ANRC_FIELDLISTLABEL'] !== '') {
                        $clabel = $fieldrec['ANRC_FIELDLISTLABEL'];
                    }
                    $this->table->addCell(dict($clabel), array(CLX_PROP_CLASS => 'headers'));
                }
            } else {
                $contenidoHTML.= '<tr>';
                foreach ($this->columns as $fieldindex => $fieldrec) {
                    $clabel = $fieldrec['FIELDLISTLABEL'];
                    if (isset($fieldrec['ANRC_FIELDLISTLABEL']) && !is_null($fieldrec['ANRC_FIELDLISTLABEL']) && $fieldrec['ANRC_FIELDLISTLABEL'] !== '') {
                        $clabel = $fieldrec['ANRC_FIELDLISTLABEL'];
                    }
                    $contenidoHTML.= '<td  class="headers">' . dict($clabel) . '</td>';
                }
                $contenidoHTML.= '</tr>';
            }
        }
		$this->output_add($contenidoHTML);
        $this->needheaders = false;
    }
	function output_add($out_content) {
		if ($this->output_buffer === true) {
			$this->contents .= $out_content;
			return;
		}
		if($this->is_output_to_file()) {
			//$this->output_to_file['fp']->write(decode_string($out_content));
			$this->output_to_file['fp']->write($out_content);
		} else {
			if($this->code === 'RPHT') {
				echo $out_content;
			}
		}
	}
}
