<?php

/**
 * Object Class
 * For more information contact  support@alenet.com
 *
 * @author       $Author: jclerma $
 * @copyright    Alenet, Inc.
 * @since        $Date: 2017-11-03 15:09:40 -0400 (Fri, 03 Nov 2017) $
 * @version        $Rev: 9615 $
 *
 * CHANGE LOG
 *  Oracle uppercase record changed
 *  Included label
 */
if (!class_exists('util') && !defined('clxInstallProcess'))
    include_once kPathToObjects . 'util.php';

class util_dbrebuild extends util {

    // CONSTRUCTOR //
    function __construct() {
        $this->code = 'DBRB';
        $this->name = 'Rebuild Database';
        $this->type = 'UTIL';
        $this->version = '0.1.0.0';
        $this->description = 'This utility will rebuild the database.';
        $this->haspanel = true;
    }

    function panel($msg) {
        $contents = '';
        $form = new htmlForm(CLX_PROP_ACTION, 'tool.php');
        $form->addArgument(CLX_PROP_TO, 'UTIL');
        $form->addArgument('ut', $this->code);
        $form->addArgument('ac', 'PANEL');
        $form->addArgument('confirm', '1');
        $table = new htmlTable(CLX_WIDTH_TWO);
        $table->setProps(array(CLX_PROP_CLASS => CLX_CLASS_FORM));
        $table->addCell(dict('OBJ_DBRB_TITLE'), array(CLX_PROP_CLASS => CLX_CLASS_TITLE, CLX_PROP_COLSPAN => CLX_WIDTH_TWO));
        $table->addCell(dict('OBJ_DBRB_WARNING'), array(CLX_PROP_CLASS => CLX_CLASS_ALERT, CLX_PROP_COLSPAN => CLX_WIDTH_TWO));
        $table->addCell(dict('OBJ_DBRB_EXPLAIN'), array(CLX_PROP_CLASS => CLX_CLASS_SUBTITLE, CLX_PROP_COLSPAN => CLX_WIDTH_TWO));
        $buttons = '';
        $c = new htmlFormSubmit(dict('GEN_LBL_EXECUTE'), 'PANEL', $form->getProp(CLX_PROP_ID));
        $buttons .= $c->source();
        $c = new htmlButtonCancel();
        $buttons .= $c->source();
        $table->addCell($buttons, array(CLX_PROP_CLASS => CLX_CLASS_FOOTERS, CLX_PROP_COLSPAN => CLX_WIDTH_TWO));
        $form->setContents($table->source());
        $contents = $form->source();
        return $contents;
    }

    function checkPanel(&$msg) {
        $msg = array();
        return count($msg) == CLX_NUM_ZERO;
    }

    function execute_() {
        UserResponse::setIgnoreUserAbort(CLX_NUM_ONE);
        UserResponse::setRunTimeLimit(CLX_NUM_ZERO);
        $contents = '<p>';
        $this->createModules($contents);
        $contents .= '</p>';
        return $contents;
    }

    function createModules(&$contents) {
        $modules = ClxCommon::$db->getRecords('CLX_DB_MODULES');
        while (($module = ClxCommon::$db->getNextRecord($modules))) {
            $contents .= '<b>' . $module['IDNAME'] . '</b><br>' . CLX_CHAR_NL;
            $this->createTables2($contents, $module['IDCODE']);
        }
    }

    function createTables2(&$contents, $modulecode) {
        $tables = ClxCommon::$db->getRecords('CLX_DB_TABLES', 'MODULECODE=' . singleQuote($modulecode));
        while (($table = ClxCommon::$db->getNextRecord($tables))) {
            ClxCommon::$db->dropTable($modulecode . '_' . $table['IDNAME']);
            $contents .= '-->' . $table['IDNAME'] . '<br>' . CLX_CHAR_NL;
            $field = array('type' => 'int(11) unsigned', 'properties' => 'not null auto_increment', 'index' => 'primary');
            $fields = array('ID' => $field);
            ClxCommon::$db->createTable($modulecode . '_' . $table['IDNAME'], $table['type'], $fields);
            $this->createFields($contents, $table['IDCODE'], $modulecode . '_' . $table['IDNAME']);
            $this->createIndexes($contents, $table['IDCODE'], $modulecode . '_' . $table['IDNAME']);
        }
    }

    function createFields(&$contents, $tablecode, $tablename) {
        $fields = ClxCommon::$db->getRecords('CLX_DB_TABLEFIELDS', 'TABLECODE=' . singleQuote($tablecode) . ' ORDER BY SEQNUM');
        while (($field = ClxCommon::$db->getNextRecord($fields))) {
            $fieldrec = ClxCommon::$db->getRecord('CLX_DB_FIELDS', 'FIELDNAME=' . singleQuote($field['fieldname']));
            if ($fieldrec['FIELDTYPE'] !== 'PRIM') {
                $fieldobj = getToolObject('DBFT', $fieldrec['FIELDTYPE']);
                $fieldobj->createTableField(array('tablename' => $tablename, 'fieldrecord' => $fieldrec));
            }
        }
    }

    function createIndexes(&$contents, $tablecode, $tablename) {
        $indexinfo = array();
        // get indexes information
        $sql = 'SELECT  DBTI.IDCODE,  DBTI.TYPE,  DBTC.FIELDNAME FROM  CLX_DB_TABLEINDEXES DBTI,  CLX_DB_INDEX_COLUMNS DBTC WHERE  DBTI.IDCODE = DBTC.INDEX_CODE AND DBTI.TABLECODE = ' . singleQuote($tablecode) . ' ORDER BY  DBTC.SEQNUM ASC';
        $records = ClxCommon::$db->executeQuery($sql);
        while (($index = ClxCommon::$db->getNextRecord($records, ASSOCIATIVE))) {
            $indexinfo[$index['IDCODE']]['index'] = $index['type'];
            if (empty($indexinfo[$index['IDCODE']]['fieldname'])) {
                $indexinfo[$index['IDCODE']]['fieldname'] = $index['fieldname'];
            } else {
                $indexinfo[$index['IDCODE']]['fieldname'] .= ',' . $index['fieldname'];
            }
        }
        $msg = '';
        $fix = CLX_NUM_ONE;
        foreach ($indexinfo as $key => $val) {
            ClxCommon::$db->checkFieldIndex($tablename, $key, $indexinfo, $msg, $fix);
        }
    }

}
