<?php

/**
 * Tool Class
 * For more information contact  support@alenet.com
 *
 * @author       $Author: alenetdev $
 * @copyright    Alenet, Inc.
 * @since        $Date: 2019-01-22 15:23:35 -0400 (ma., 22 ene. 2019) $
 * @version        $Rev: 10489 $
 *
 * CHANGE LOG
 *  Included label
 */
class ac_trees extends clx_tool {

    // CONSTRUCTOR //
    function __construct() {
        $this->code = 'ACTR';
        $this->name = 'Access Tree Manager';
        $this->system = 'Security';
        $this->version = '0.4.2.0';

        $this->has_access = true;
        $this->has_package = true;

        $this->tablename = 'CLX_AC_TREES';
        $this->record_name_singular = 'tree';
        $this->record_name_plural = 'trees';
        $this->primary_field = 'IDCODE';
        $this->secondary_field = 'IDNAME';

        $this->editform = array(CLX_CLASS_TITLE => 'TOOL_ACTR_TITLE_MODIFY',
            'panels' => array(
                array('type' => 'normal',
                    'entries' => array(
                        array('label' => 'GEN_LBL_CODE',
                            'fields' => array(
                                array('name' => 'idcode', 'type' => 'string', 'default' => '', 'size' => '16', 'displaysize' => '16', 'dontmodify' => '1',
                                    'validations' => array(
                                        array('type' => 'string', 'required' => '1'))
                                )
                            )
                        ), array('label' => 'GEN_LBL_NAME',
                            'fields' => array(
                                array('name' => 'idname', 'type' => 'string', 'default' => '', 'size' => '64', 'displaysize' => '48', 'dontmodify' => '1',
                                    'validations' => array(
                                        array('type' => 'string', 'required' => '1'))
                                )
                            )
                        ), array('label' => 'GEN_LBL_VERSION',
                            'fields' => array(
                                array('name' => 'version', 'type' => 'string', 'default' => '', 'size' => '32', 'displaysize' => '32', 'validations' => array(
                                        array('type' => 'string', 'required' => '1'))
                                )
                            )
                        ), array('label' => 'GEN_LBL_DESCRIPTION',
                            'fields' => array(
                                array('name' => 'description', 'type' => 'text', 'default' => '', 'rows' => '4', 'cols' => '48'))
                        ), array('label' => 'GEN_LBL_MODULECODE',
                            'fields' => array(
                                array('name' => 'modulecode', 'type' => 'lookup', 'tool' => 'DBMO', 'validations' => array(
                                        array('type' => 'lookup', 'required' => '1', 'fieldname' => 'idcode', 'tablename' => 'CLX_DB_MODULES'))
                                )
                            )
                        ), array('label' => 'GEN_LBL_FORMAT',
                            'fields' => array(
                                array('name' => 'format', 'type' => 'string', 'default' => '', 'size' => '255', 'displaysize' => '48', 'dontmodify' => '1',
                                    'validations' => array(
                                        array('type' => 'string', 'required' => '1'))
                                )
                            )
                        )
                    )
                )
            )
        );
    }

    function install() {
        $UIMI = getTool('UIMI');
        $UIMI->add($this->code, 'MENU_TOOLS:MENU_SECURITY:MENU_TREES', 'TOOL', $this->code, 'ac=LIST');
    }

    function toolTableStructure() {
        $fields = array();
        $fields['ID'] = array('type' => 'int(11) unsigned', 'null' => 'NO', 'properties' => 'auto_increment', 'index' => 'PRI');
        $fields['IDCODE'] = array('type' => 'varchar(16)', 'null' => 'NO', 'properties' => '');
        $fields['IDNAME'] = array('type' => 'varchar(64)', 'null' => 'NO', 'properties' => '');
        $fields['VERSION'] = array('type' => 'varchar(32)', 'null' => 'NO', 'properties' => '');
        $fields['VERSION_COUNTER'] = array('type' => 'int(11) unsigned', 'null' => 'NO', 'properties' => '', 'default' => CLX_NUM_ZERO);
        $fields['CDATE'] = array('type' => 'datetime', 'null' => 'YES', 'properties' => '');
        $fields['MDATE'] = array('type' => 'datetime', 'null' => 'YES', 'properties' => '');
        $fields['USERNAME'] = array('type' => 'varchar(30)', 'null' => 'NO', 'properties' => '');
        $fields['DESCRIPTION'] = array('type' => 'mediumtext', 'null' => 'YES', 'properties' => '');
        $fields['MODULECODE'] = array('type' => 'varchar(4)', 'null' => 'NO', 'properties' => '');
        $fields['FORMAT'] = array('type' => 'varchar(255)', 'null' => 'YES', 'properties' => '');
        $indexes = array();
        $ipf = 'CLX' . $this->code;
        $indexes[$ipf . '01'] = array('index' => 'PRI', 'fieldname' => 'ID');
        $indexes[$ipf . '02'] = array('index' => 'UNI', 'fieldname' => 'IDCODE');
        $indexes[$ipf . '03'] = array('index' => 'MUL', 'fieldname' => 'IDNAME');
        $indexes[$ipf . '04'] = array('index' => 'MUL', 'fieldname' => 'MODULECODE');
        return array('idcode' => $this->code, 'idname' => $this->name, 'name' => $this->tablename, 'type' => 'InnoDB',
            'fields' => $fields,
            'indexes' => $indexes);
    }

//-- A C C E S S    T R E E S
    function pack($record) {
        $o = $this->packRecord($record);
        $code = $record['IDCODE'];
        $objs = new XMLObject($this->code . '_OBJECTS');
        $this->packsub($objs, 'ACTL', 'SELECT * FROM CLX_AC_TREE_LEVELS WHERE UPPER(TREECODE)=' . singleQuote(uppercase($code)));
        $o->addObject($objs);
        return $o;
    }

    function unpack($parameters) {
        $parameters = array_values(array_merge(array('package' => null), $parameters));
        list($package) = $parameters;
        $record = $this->unpackRecord($package);
        $tablecode = $record['IDCODE']; // This may be needed
        $treecode = $record['IDCODE'];
        $exists = (ClxCommon::$db->oneExistsRecords($this->tablename, 'UPPER(IDCODE)=' . singleQuote(uppercase($tablecode))) > CLX_NUM_ZERO);
        if ($exists === true) {
            $treeid = ClxCommon::$db->getField($this->tablename, 'ID', 'UPPER(IDCODE)=' . singleQuote(uppercase($tablecode)));
            $this->deleteRecordSk($treeid);
        }
        $treeid = ClxCommon::$db->createRecord($this->tablename, $record, false);
        if ($package->hasSubObject($this->code . '_OBJECTS')) {
            //$objs = $package->getObject_($this->code . '_OBJECTS');
            $objs = $package->getObjectXml($this->code . '_OBJECTS');
            foreach ($objs->objects as $o) {
                $toolcode = $o->tag;
                $tool = &getTool($toolcode);
                $tool->unpack(array('package' => $o, 'id' => $treecode));
            }
        }
    }

    function action($ac, $idlist) {
        if ($this->getAccess()->canRead()) {
            $contents = parent::action($ac, $idlist);
        } else {
            $contents = warning(dict('GEN_ERR_NO_ACCESS') . ' [AC_TREES-CAN_READ][' . $ac . '] ' . $this->getCode(false));
        }
        return $contents;
    }

    function makelist($listingcode = null) {
        $t = time();
        $to = $this->code;
        $here = 'tool.php?to=' . $to . '&t=' . $t;
        clearHistory();
        pushHistory($this->code, 'LIST', array());
        $form = new htmlForm(CLX_PROP_ACTION, 'tool.php');
        $form->setProp(CLX_PROP_ID, 'ACTRList');
        $form->addArgument(CLX_PROP_TO, $this->code);
        $form->addArgument('ac', 'NOT_DEFINED');
        $selectclause = '*';
        $fromclause = 'CLX_AC_TREES';
        $whereclause = '';
        $sortclause = '';
        $page = new xPaging();
        $page->pageInit($form->getProp(CLX_PROP_ID), $here, $selectclause, $fromclause, $whereclause, $sortclause, '');
        $table = new htmlTable(CLX_WIDTH_SIX);
        $table->setProps(array(CLX_PROP_CLASS => CLX_CLASS_LIST, CLX_PROP_WIDTH => CLX_PROP_100PCT));
        $table->addCell(dict('TOOL_ACTR_TITLE_LIST'), array(CLX_PROP_CLASS => CLX_CLASS_TITLE, CLX_PROP_COLSPAN => CLX_WIDTH_SIX));
        $page->header($table);
        $table->addCells(CLX_CLASS_HEADERS, array(dict('GEN_LBL_SEL'), dict('GEN_LBL_MODULECODE'), dict('GEN_LBL_CODE'), dict('GEN_LBL_NAME'), dict('GEN_LBL_VERSION'), dict('GEN_LBL_DESCRIPTION')));
        $i = CLX_INIT_ZERO;
        while ($record = $page->nextRecord()) {
            $recordid = $record['ID'];
            $select = new htmlCheckbox('selector_' . $recordid, false);
            $select->setProp('onclick', 'selectButtons(' . singleQuote($form->getProp(CLX_PROP_ID)) . ');');
            $table->addCell($select->source(), array(CLX_PROP_CLASS => CLX_CLASS_FOOTERS, CLX_PROP_WIDTH => '40', CLX_PROP_ALIGN => 'center'));
            $table->addCells(($i % CLX_NUM_TWO == CLX_NUM_ZERO) ? CLX_CLASS_DATA : CLX_CLASS_DATAALT, array($record['MODULECODE'], $record['IDCODE'], $record['IDNAME'], $record['VERSION'] . '-' . $record['VERSION_COUNTER'], xFormat::text($record['DESCRIPTION'])));
            $i++;
        }
        $buttons = '';
        $buttons .= htmlButton::getButton('submit', dict('GEN_LBL_DISPLAY'), array('action' => 'DISPLAY', 'form' => $form->getProp(CLX_PROP_ID), CLX_PROP_ID => 'MULT'))->source();
        if ($this->getAccess()->canModify()) {
            $buttons .=htmlButton::getButton('submit', dict('GEN_LBL_MODIFY'), array('action' => 'MODIFY', 'form' => $form->getProp(CLX_PROP_ID), CLX_PROP_ID => 'MULT'))->source();
        }
        if ($this->getAccess()->canDelete()) {
            $buttons .=htmlButton::getButton('submit', dict('GEN_LBL_DELETE'), array('action' => 'DELETE', 'form' => $form->getProp(CLX_PROP_ID), CLX_PROP_ID => 'MULT'))->source();
        }
        if ($this->getAccess()->canDisplay()) {
            $buttons .=htmlButton::getButton('submit', dict('GEN_LBL_CREATE_SOMETHING', dict("GEN_LBL_TREE")), array('action' => 'CREATE', 'form' => $form->getProp(CLX_PROP_ID), CLX_PROP_ID => 'ON'))->source();
        }
        $buttons .=htmlButton::getButton('submit', dict('GEN_LBL_FILTER'), array('action' => 'FILTER', 'form' => $form->getProp(CLX_PROP_ID), CLX_PROP_ID => 'ON'))->source();
        $form->addContents($page->footers($table, $buttons));
        return $form->source();
    }

    function searchForm() {
        $msg = array();
        $contents = '';
        // Search Form
        $form = new htmlForm(CLX_PROP_ACTION, UserRequest::$server['PHP_SELF']);
        $form->addArgument(CLX_PROP_TO, $this->code);
        $form->addArgument('ac', 'SEARCH');
        $form->addArgument('confirm', '1');
        $form->addArgument(CLX_PROP_TARGET, UserRequest::$request[CLX_PROP_TARGET]);
        $table = new htmlTable(CLX_WIDTH_TWO);
        $table->setProps(array(CLX_PROP_CLASS => CLX_CLASS_FORM, CLX_PROP_WIDTH => CLX_PROP_100PCT));
        $table->addCell(dict('TOOL_ACTR_TITLE_SEARCH'), array(CLX_PROP_CLASS => CLX_CLASS_TITLE, CLX_PROP_COLSPAN => CLX_WIDTH_TWO));
        if (!array_key_exists('ACTR_idcode', UserRequest::$post))
            UserRequest::$post['ACTR_idcode'] = '';
        $table->addField($msg, dict('GEN_LBL_CODE') . ':', 'ACTR_idcode', UserRequest::$post['ACTR_idcode'], CLX_NUM_16, CLX_NUM_16);
        if (!array_key_exists('ACTR_idname', UserRequest::$post))
            UserRequest::$post['ACTR_idname'] = '';
        $table->addField($msg, dict('GEN_LBL_NAME') . ':', 'ACTR_idname', UserRequest::$post['ACTR_idname'], CLX_NUM_32, CLX_NUM_64);
        if (!array_key_exists('ACTR_modulecode', UserRequest::$post))
            UserRequest::$post['ACTR_modulecode'] = '';
        $table->addField($msg, dict('GEN_LBL_MODULECODE') . ':', 'ACTR_modulecode', UserRequest::$post['ACTR_modulecode'], CLX_NUM_FOUR, CLX_NUM_FOUR);
        $buttons =htmlButton::getButton('submit', dict('GEN_LBL_SEARCH'), array('action' => 'SEARCH', 'form' => $form->getProp(CLX_PROP_ID)))->source();
        $c = new htmlButtonCloseWindow(dict('GEN_LBL_CANCEL'));
        $buttons .= $c->source();
        $table->addCell($buttons, array(CLX_PROP_CLASS => CLX_CLASS_FOOTERS, CLX_PROP_COLSPAN => CLX_WIDTH_TWO));
        $form->setContents($table->source());
        $contents .= $form->source();
        // Search Procedure
        if (isset(UserRequest::$post['confirm'])) {
            $table = new htmlTable(CLX_WIDTH_TWO);
            $table->setProps(array(CLX_PROP_CLASS => CLX_CLASS_LIST, CLX_PROP_WIDTH => CLX_PROP_100PCT));
            $criteria = '';
            $boolean = '';
            if (UserRequest::$post['ACTR_idcode'] <> '') {
                $criteria .= $boolean . 'UPPER(IDCODE) LIKE ' . singleQuote(uppercase(UserRequest::$post['ACTR_idcode']) . '%');
                $boolean = ' AND ';
            }
            if (UserRequest::$post['ACTR_idname'] <> '') {
                $criteria .= $boolean . 'UPPER(IDNAME) LIKE ' . singleQuote(uppercase(clx_change_especial_char_html (UserRequest::$post['ACTR_idname'])) . '%');
                $boolean = ' AND ';
            }
            if (UserRequest::$post['ACTR_modulecode'] <> '') {
                $criteria .= $boolean . 'UPPER(MODULECODE) LIKE ' . singleQuote(uppercase(UserRequest::$post['ACTR_modulecode']) . '%');
                $boolean = ' AND ';
            }
            $qty = ClxCommon::$db->oneExistsRecords($this->tablename, $criteria);
            if ($qty == CLX_NUM_ZERO) {
                $table->addCell(dict('TOOL_GEN_ERR_NOMATCH'), array(CLX_PROP_CLASS => CLX_CLASS_ALERT, CLX_PROP_COLSPAN => CLX_WIDTH_TWO));
            } else {
                $table->addCell(dict('TOOL_GEN_MSG_FOUNDRECORDS'), array(CLX_PROP_CLASS => CLX_CLASS_SUBTITLE, CLX_PROP_COLSPAN => CLX_WIDTH_TWO));
                $records = ClxCommon::$db->getRecords($this->tablename, $criteria);
                $i = CLX_INIT_ZERO;
                while (($record = ClxCommon::$db->getNextRecord($records, ASSOCIATIVE))) {
                    $js = 'javascript:setLookupValue(' . quote(UserRequest::$request[CLX_PROP_TARGET]) . ', ' . quote($record['IDCODE']) . ');';
                    $table->addCells(($i % CLX_NUM_TWO == CLX_NUM_ZERO) ? CLX_CLASS_DATA : CLX_CLASS_DATAALT, array('<page t=ac_trees></page><a href=' . singleQuote($js) . '>' . $record['IDCODE'] . '</a>', $record['IDNAME']));
                    $i++;
                }
            }
            $contents .= $table->source();
        }
        return $contents;
    }

    function editForm($ac, $idlist, $record, $msg) {
        $modify = (UserRequest::$request['ac'] === 'MODIFY');
        $title = ($modify) ? dict('TOOL_ACTR_TITLE_MODIFY') : dict('TOOL_ACTR_TITLE_CREATE');
        $variables = array();
        $variables['FIRSTFIELD'] = ($modify) ? quote('ACTR_idname') : quote('ACTR_idcode');
        $topscript = setJavaScriptTag(getJavaScript('ACTR_editform.js', $variables));
        $btmscript = setJavaScriptTag('initForm();',false);
        $form = new htmlForm(CLX_PROP_ACTION, 'tool.php');
        $form->addArgument(CLX_PROP_TO, $this->code);
        $form->addArgument('ac', $ac);
        $form->addArgument('confirm', '1');
        $form->addArgument('ACTR_id', $record['ACTR_id']);
        $form->addArgument('idlist', toString($idlist));
        $table = new htmlTable(CLX_WIDTH_TWO);
        $table->setProps(array(CLX_PROP_CLASS => CLX_CLASS_FORM, CLX_PROP_WIDTH => CLX_PROP_100PCT));
        $table->addCell($title, array(CLX_PROP_CLASS => CLX_CLASS_TITLE, CLX_PROP_COLSPAN => CLX_WIDTH_TWO));
        if (!empty($modify)) {
            $form->addArgument('ACTR_idcode', $record['ACTR_idcode']);
            $table->addData(dict('GEN_LBL_CODE') . ':', $record['ACTR_idcode']);
        } else {
            $table->addField($msg, dict('GEN_LBL_CODE') . ':', 'ACTR_idcode', $record['ACTR_idcode'], CLX_NUM_16, CLX_NUM_16);
        }
        $table->addField($msg, dict('GEN_LBL_NAME') . ':', 'ACTR_idname', $record['ACTR_idname'], CLX_NUM_48, CLX_NUM_64);
        $table->addField($msg, dict('GEN_LBL_VERSION') . ':', 'ACTR_version', $record['ACTR_version'], CLX_NUM_16, CLX_NUM_16);
        $table->addText($msg, dict('GEN_LBL_DESCRIPTION') . ':', 'ACTR_description', $record['ACTR_description'], CLX_NUM_FOUR, CLX_NUM_48);
        $callingpage = peekHistory();
        if ($callingpage[CLX_PROP_TO] === 'DBMO') {
            $form->addArgument('ACTR_modulecode', $callingpage['modulecode']);
        } else {
            $DBMO = &getTool('DBMO');
            $DBMO->placeChooseModule($msg, dict('GEN_LBL_MODULECODE') . ':', 'ACTR_modulecode', $record['ACTR_modulecode'], $table);
        }
        $form->addContents($table->source());
        $table = new htmlTable(CLX_WIDTH_FOUR);
        $table->setProps(array(CLX_PROP_CLASS => CLX_CLASS_FORM, CLX_PROP_WIDTH => CLX_PROP_100PCT));
        $table->addCell(dict('TOOL_ACTR_TITLE_LEVELS'), array(CLX_PROP_CLASS => CLX_CLASS_SUBTITLE, CLX_PROP_COLSPAN => CLX_WIDTH_FOUR));
        $table->addCells(CLX_CLASS_HEADERS, array(CLX_CHAR_HTML_NBSP, dict('TOOL_ACTL_PROMPT_NAMESING'), dict('TOOL_ACTL_PROMPT_NAMEPLUR'), dict('GEN_LBL_FORMAT')));
        for ($i = CLX_NUM_ONE; $i <= CLX_NUM_TEN; $i++) {
            $table->addCell(dict('TOOL_ACTR_PROMPT_LEVEL', $i . ':', CLX_PROP_CLASS, CLX_CLASS_PROMPT), array());
            $h1 = new htmlField('ACTR_levels' . $i, (array_key_exists('ACTR_levels' . $i, $record)) ? $record['ACTR_levels' . $i] : '', CLX_NUM_32, CLX_NUM_48);
            $h2 = new htmlField('ACTR_levelp' . $i, (array_key_exists('ACTR_levelp' . $i, $record)) ? $record['ACTR_levelp' . $i] : '', CLX_NUM_32, CLX_NUM_48);
            $h3 = new htmlField('ACTR_format' . $i, (array_key_exists('ACTR_format' . $i, $record)) ? $record['ACTR_format' . $i] : '', CLX_NUM_TEN, CLX_NUM_TEN);
            $table->addCells(CLX_CLASS_DATA, array($h1->source(), $h2->source(), $h3->source()));
        }
        $form->addContents($table->source());
        $table = new htmlTable(CLX_WIDTH_FOUR);
        $table->setProps(array(CLX_PROP_CLASS => CLX_CLASS_FORM, CLX_PROP_WIDTH => CLX_PROP_100PCT));
        $buttons = '';
        if (!empty($modify)) {
            $buttons .= htmlButton::getButton('submit', dict('GEN_LBL_SAVE'), array('action' => 'MODIFY', 'form' => $form->getProp(CLX_PROP_ID)))->source();
        } else {
            $buttons .=htmlButton::getButton('submit', dict('GEN_LBL_CREATE'), array('action' => 'CREATE', 'form' => $form->getProp(CLX_PROP_ID)))->source();
        }
        $c = new htmlButtonCancel();
        $buttons .= $c->source();
        $table->addCell($buttons, array(CLX_PROP_CLASS => CLX_CLASS_FOOTERS, CLX_PROP_COLSPAN => CLX_WIDTH_FOUR));
        $form->addContents($table->source());
        return $topscript . $form->source() . $btmscript;
    }

    function checkForm(&$msg) {
        $msg = array();
        $action = UserRequest::$post['ac'];
        if ($action === 'CREATE') {
            xValidate::unique($msg, 'ACTR_idcode', 'code', $this->tablename, 'UPPER(IDCODE)=' . singleQuote(uppercase(UserRequest::$post['ACTR_idcode'])));
        }
        if (empty(UserRequest::$post['ACTR_idname'])) {
            $msg['ACTR_idname'] = 'You have to write a name here.';
        }
        $DBMO = &getTool('DBMO');
        if (!$DBMO->moduleExists(UserRequest::$post['ACTR_modulecode'])) {
            $msg['ACTR_modulecode'] = 'There is no module with this code.';
        }
        return count($msg) == CLX_NUM_ZERO;
    }

    function confirmDeleteFill(&$table, $record) {
        $table->addData(dict('GEN_LBL_CODE') . ':', $record['ACTR_idcode']);
        $table->addData(dict('GEN_LBL_NAME') . ':', $record['ACTR_idname']);
    }

    function viewFormFill($record) {
        addHistory('treecode', $record['ACTR_idcode']);
        $contents = '';
        $table = new htmlTable(CLX_WIDTH_TWO);
        $table->setProps(array(CLX_PROP_CLASS => CLX_CLASS_FORM, CLX_PROP_WIDTH => CLX_PROP_100PCT));
        $table->addCell(dict('TOOL_ACTR_TITLE_VIEW') . ': ' . $record['ACTR_idname'], array(CLX_PROP_CLASS => CLX_CLASS_TITLE, CLX_PROP_COLSPAN => CLX_WIDTH_TWO));
        $table->addData(dict('GEN_LBL_CODE') . ':', $record['ACTR_idcode']);
        $table->addData(dict('GEN_LBL_NAME') . ':', $record['ACTR_idname']);
        $table->addData(dict('GEN_LBL_VERSION') . ':', $record['ACTR_version'] . '-' . $record['ACTR_version_counter']);
        $table->addData(dict('GEN_LBL_DESCRIPTION') . ':', xFormat::text($record['ACTR_description']));
        $table->addData(dict('GEN_LBL_MODULECODE') . ':', $record['ACTR_modulecode']);
        $table->addData(dict('GEN_LBL_FORMAT') . ':', $record['ACTR_format']);
        $contents .= $table->source();
        $ACTL = &getTool('ACTL');
        $contents .= $ACTL->makepanel($record['ACTR_idcode']);
        $table = new htmlTable(CLX_WIDTH_TWO);
        $table->setProps(array(CLX_PROP_CLASS => CLX_CLASS_FORM, CLX_PROP_WIDTH => CLX_PROP_100PCT));
        $table->addData(dict('GEN_LBL_CDATE') . ':', $record['ACTR_cdate']);
        $table->addData(dict('GEN_LBL_MDATE') . ':', $record['ACTR_mdate']);
        $table->addData(dict('GEN_LBL_USERNAME') . ':', $record['ACTR_username']);
        $contents .= buildPanelInterface(dict('GEN_TTL_SYSTEMINFO'), 'SYSTEMINFO', $table->source(), CLX_NUM_MINUS_1);
        return $contents;
    }

    function filterFormFill($msg, &$table) {
        if (!array_key_exists('ACTR_idcode', UserRequest::$post))
            UserRequest::$post['ACTR_idcode'] = '';
        if (!array_key_exists('ACTR_idname', UserRequest::$post))
            UserRequest::$post['ACTR_idname'] = '';
        if (!array_key_exists('ACTR_modulecode', UserRequest::$post))
            UserRequest::$post['ACTR_modulecode'] = '';
        $table->addField($msg, dict('GEN_LBL_CODE') . ':', 'ACTR_idcode', UserRequest::$post['ACTR_idcode'], CLX_NUM_FOUR, CLX_NUM_FOUR);
        $table->addField($msg, dict('GEN_LBL_NAME') . ':', 'ACTR_idname', UserRequest::$post['ACTR_idname'], CLX_NUM_32, CLX_NUM_64);
        $table->addField($msg, dict('GEN_LBL_MODULECODE') . ':', 'ACTR_modulecode', UserRequest::$post['ACTR_modulecode'], CLX_NUM_FOUR, CLX_NUM_FOUR);
        $sort = new HtmlSelect('ACTR_sort');
        $sort->addItem('', '', true);
        $sort->addItem('Module Code', 'modulecode', false);
        $sort->addItem('Code', 'idcode', false);
        $sort->addItem('Name', 'idname', false);
        $table->addData(dict('GEN_LBL_ORDERBY') . ':', $sort->source());
    }

    function setfilter() {
        $filter = '';
        $boolean = '';
        if (!empty(UserRequest::$post['ACTR_idcode'])) {
            $filter .= $boolean . '  UPPER(IDCODE) LIKE ' . singleQuote(uppercase(UserRequest::$post['ACTR_idcode']) . '%');
            $boolean = 'AND';
        }
        if (!empty(UserRequest::$post['ACTR_idname'])) {
            $filter .= $boolean . '  UPPER(IDNAME) LIKE ' . singleQuote(uppercase(clx_change_especial_char_html (UserRequest::$post['ACTR_idname'])) . '%');
            $boolean = 'AND';
        }
        if (!empty(UserRequest::$post['ACTR_modulecode'])) {
            $filter .= $boolean . ' UPPER(MODULECODE) LIKE ' . singleQuote(uppercase(UserRequest::$post['ACTR_modulecode']) . '%');
            $boolean = 'AND';
        }
        $_SESSION['paging']['ACTRList']['sortby'] = UserRequest::$post['ACTR_sort'];
        $_SESSION['paging']['ACTRList']['criteria'] = $filter;
        $_SESSION['paging']['ACTRList']['startat'] = '0';
    }
	///////////////////////////////////////////////////
	///comentado 22/01/2019 para utilizar la clx_tool
	//////////////////////////////////////////////////
    /*function clearfilter() {
        $_SESSION['paging']['ACTRList']['criteria'] = '';
    }*/

    function loadToolRecord($recordid) {
        $values = ClxCommon::$db->getRecord($this->tablename, 'ID=' . $recordid);
        $record = array();
        $record['ACTR_id'] = $recordid;
        $record['ACTR_idcode'] = $values['IDCODE'];
        $record['ACTR_idname'] = $values['IDNAME'];
        $record['ACTR_version'] = $values['VERSION'];
        $record['ACTR_version_counter'] = $values['VERSION_COUNTER'];
        $record['ACTR_cdate'] = xFormat::date($values['CDATE']);
        $record['ACTR_mdate'] = xFormat::date($values['MDATE']);
        $record['ACTR_username'] = $values['USERNAME'];
        $record['ACTR_description'] = $values['DESCRIPTION'];
        $record['ACTR_modulecode'] = $values['MODULECODE'];
        $record['ACTR_format'] = $values['FORMAT'];
        $treecode = $record['ACTR_idcode'];
        $levels = ClxCommon::$db->getRecords('CLX_AC_TREE_LEVELS', 'UPPER(TREECODE)=' . singleQuote(uppercase($treecode)) . ' ORDER BY LEVELS');
        while (($level = ClxCommon::$db->getNextRecord($levels))) {
            $record['ACTR_levels' . $level['LEVELS']] = $level['NAME_SINGULAR'];
            $record['ACTR_levelp' . $level['LEVELS']] = $level['NAME_PLURAL'];
            $record['ACTR_format' . $level['LEVELS']] = $level['FORMAT'];
        }
        return $record;
    }

    function saveRecord() {
		apc_clear_cacheFct();
        $d = xFormat::getNow('Y-m-d');
        $record = array();
        $record['IDNAME'] = UserRequest::$post['ACTR_idname'];
        $record['VERSION'] = UserRequest::$post['ACTR_version'];
        $record['MDATE'] = $d;
        $record['USERNAME'] = $_SESSION['user_record']['username'];
        $record['DESCRIPTION'] = UserRequest::$post['ACTR_description'];
        $record['MODULECODE'] = UserRequest::$post['ACTR_modulecode'];
        ClxCommon::$db->updateRecord($this->tablename, $record, 'ID=' . UserRequest::$post['ACTR_id']);
        $this->versionIncrease('ID', UserRequest::$post['ACTR_id']);
        $this->saveLevels(UserRequest::$post['ACTR_idcode']);
		
    }

    function createRecordVoid_() {
        ClxFreeReturn($this->createRecord_());
    }

    function createRecord_() {
		apc_clear_cacheFct();
        $d = xFormat::getNow('Y-m-d');
        $record = array();
        $record['IDCODE'] = UserRequest::$post['ACTR_idcode'];
        $record['IDNAME'] = UserRequest::$post['ACTR_idname'];
        $record['VERSION'] = UserRequest::$post['ACTR_version'];
        $record['VERSION_COUNTER'] = '0';
        $record['CDATE'] = $d;
        $record['MDATE'] = $d;
        $record['USERNAME'] = $_SESSION['user_record']['username'];
        $record['DESCRIPTION'] = UserRequest::$post['ACTR_description'];
        $record['MODULECODE'] = UserRequest::$post['ACTR_modulecode'];
        $recordid = ClxCommon::$db->createRecord($this->tablename, $record, false);
        $DBMO = &getTool('DBMO');
        $DBMO->versionIncrease('IDCODE', $record['MODULECODE']);
        $this->saveLevels(UserRequest::$post['ACTR_idcode']);
        return $recordid;
    }

    function deleteRecordSk($id) {
        $treerec = ClxCommon::$db->getRecord($this->tablename, 'ID=' . $id);
        $treecode = uppercase($treerec['IDCODE']);
        ClxCommon::$db->deleteRecords('CLX_AC_TREE_CONTENTS', 'UPPER(TREECODE)=' . singleQuote($treecode));
        ClxCommon::$db->deleteRecords('CLX_AC_TREE_USERS', 'UPPER(TREECODE)=' . singleQuote($treecode));
        ClxCommon::$db->deleteRecords('CLX_AC_TREE_LEVELS', 'UPPER(TREECODE)=' . singleQuote($treecode));
        ClxCommon::$db->deleteRecords($this->tablename, 'ID=' . $id);
        $DBMO = &getTool('DBMO');
        $DBMO->versionIncrease('IDCODE', $treerec['MODULECODE']);
    }

    function defaultRecord() {
        $record = array();
        $record['ACTR_id'] = CLX_INIT_ZERO;
        $record['ACTR_idcode'] = '';
        $record['ACTR_idname'] = '';
        $record['ACTR_version'] = '1.0.0';
        $record['ACTR_cdate'] = '';
        $record['ACTR_mdate'] = '';
        $record['ACTR_username'] = $_SESSION['user_record']['username'];
        $record['ACTR_description'] = '';
        $record['ACTR_modulecode'] = '';
        $record['ACTR_format'] = '';
        return $record;
    }

    function versionIncrease($fieldname, $fieldvalue) {
        parent::versionIncrease($fieldname, $fieldvalue);
        $modulecode = ClxCommon::$db->getField($this->tablename, 'MODULECODE', $fieldname . '=' . singleQuote($fieldvalue));
        $DBMO = &getTool('DBMO');
        $DBMO->versionIncrease('IDCODE', $modulecode);
    }

    function placeChooseTree($msg, $prompt, $name, $default, &$table) {
        $searchIcon = getSkinIcon('view', 'javascript:openWindow(' . singleQuote(CLX_CHAR_EMPTY_STRING) . ', ' . singleQuote('pop.php?to=ACTR&ac=SEARCH&target=' . $name) . ', 500, 300);', 'Search for a tree');
        if (!empty($msg[$name]))
            $table->addCell($msg[$name], array(CLX_PROP_CLASS => CLX_CLASS_ALERT, CLX_PROP_COLSPAN => $table->width));
        $table->addCell($prompt, array(CLX_PROP_CLASS => CLX_CLASS_PROMPT));
        $c = new htmlField($name, $default, CLX_NUM_16, CLX_NUM_16);
        $table->addCell($c->source() . $searchIcon, array(CLX_PROP_CLASS => CLX_CLASS_DATA));
        return CLX_CHAR_EMPTY_STRING;
    }

    function treeExists(&$idcode) {
		$idcode=uppercase($idcode);
        $qty = ClxCommon::$db->oneExistsRecords($this->tablename, 'UPPER(IDCODE)=' . singleQuote($idcode));
        if ($qty > CLX_NUM_ZERO) {
            $idcode = ClxCommon::$db->getField($this->tablename, 'IDCODE', 'UPPER(IDCODE)=' . singleQuote($idcode));
        }
        return $qty > CLX_NUM_ZERO;
    }

    function saveLevels($treecode) {
		$treecode=uppercase($treecode);
        ClxCommon::$db->deleteRecords('CLX_AC_TREE_LEVELS', 'UPPER(TREECODE)=' . singleQuote($treecode));
        $level = CLX_NUM_ONE;
        for ($i = CLX_NUM_ONE; $i <= CLX_NUM_TEN; $i++) {
            $h1 = UserRequest::$post['ACTR_levels' . $i];
            $h2 = UserRequest::$post['ACTR_levelp' . $i];
            $h3 = UserRequest::$post['ACTR_format' . $i];
            if (!empty($h1) or ! empty($h2)) {
                $record = array();
                $record['TREECODE'] = $treecode;
                $record['NAME_SINGULAR'] = $h1;
                $record['NAME_PLURAL'] = $h2;
                $record['FORMAT'] = $h3;
                $record['LEVELS'] = $level;
                ClxCommon::$db->createRecord('CLX_AC_TREE_LEVELS', $record, false);
                $level++;
            }
        }
        $this->updateTree($treecode);
    }

    function updateTree($treecode) {
		$treecode=uppercase($treecode);
        $records = ClxCommon::$db->getRecords('CLX_AC_TREE_LEVELS', 'UPPER(TREECODE)=' . singleQuote($treecode) . ' ORDER BY LEVELS');
        $format = '';
        $delim = '';
        while (($record = ClxCommon::$db->getNextRecord($records, ASSOCIATIVE))) {
            $format .= $delim . $record['FORMAT'];
            $delim = '.';
        }
        $record = array();
        $record['FORMAT'] = $format;
        ClxCommon::$db->updateRecord($this->tablename, $record, 'UPPER(IDCODE)=' . singleQuote($treecode));
    }

    function getUserNodes($treecode, $nodes = array()) {
		$treecode=uppercase($treecode);
        $username = $_SESSION['user_record']['username'];
        $levelsrec = ClxCommon::$db->getRecords('CLX_AC_TREE_USERS', 'UPPER(TREECODE)=' . singleQuote($treecode), 'UPPER(USERNAME)=' . singleQuote(uppercase($username)), 'STATUS=1');
        while (($levelrec = ClxCommon::$db->getNextRecord($levelsrec))) {
            $nodes[] = $levelrec['TREENODE'];
        }
        return $nodes;
    }

    /**
     * Returns where sql part conteining nodes
     * @param $treecode
     * @param $target_tree_field
     */
    function getUserNodesCriteria($treecode, $target_tree_field) {
		$treecode=uppercase($treecode);		
        $nodes = array();
        $username = $_SESSION['user_record']['username'];
        $levelsrec = ClxCommon::$db->getRecords('CLX_AC_TREE_USERS', 'UPPER(TREECODE)=' . singleQuote($treecode), 'UPPER(USERNAME)=' . singleQuote(uppercase($username)), 'STATUS=1');
        while (($levelrec = ClxCommon::$db->getNextRecord($levelsrec))) {
            $nodes[] = $target_tree_field . ' LIKE "' . $levelrec['TREENODE'] . '%"';
        }
        if (!empty($nodes)) {
            return ' (' . implode(' OR ', $nodes) . ') ';
        } else {
            if ($_SESSION['user_record']['isadmin'] !== '1') {
                return '1=2';
            } else {
                return CLX_CHAR_EMPTY_STRING;
            }
        }
    }

}
