<?php

/**
 * Tool Class
 * For more information contact  support@alenet.com
 *
 * @package		Clientix.Tool.Interface
 * @abstract		System Chart Manager
 * @author       $Author: jclerma $
 * @copyright    Alenet, Inc.
 * @since        $Date: 2017-11-03 15:09:40 -0400 (Fri, 03 Nov 2017) $
 * @version        $Rev: 9615 $
 *
 * CHANGE LOG
 *  Included label
 */
class an_chart_properties extends clx_tool {

    // CONSTRUCTOR //
    function __construct() {
        $this->code = 'ANCP';
        $this->name = 'Chart Properties';
        $this->system = 'Analytics';
        $this->version = '0.2.2.0';
        $this->accesskey = 'ANCH';



        $this->tablename = 'CLX_AN_CHART_PROPERTIES';
        $this->record_name_singular = 'chart property';
        $this->record_name_plural = 'chart properties';
        $this->primary_field = 'IDCODE';
        $this->secondary_field = 'IDNAME';
    }

    function toolTableStructure() {//deprecated
        $fields = array();
        $fields['ID'] = array('type' => 'int(11) unsigned', 'null' => 'NO', 'properties' => 'auto_increment', 'index' => 'PRI');
        $fields['IDCODE'] = array('type' => 'varchar(4)', 'null' => 'NO', 'properties' => '');
        $fields['IDNAME'] = array('type' => 'varchar(64)', 'null' => 'NO', 'properties' => '');
        $fields['DESCRIPTION'] = array('type' => 'varchar(255)', 'null' => 'NO', 'properties' => '');
        $fields['STATUS'] = array('type' => 'tinyint(1) unsigned', 'null' => 'NO', 'properties' => '', 'default' => CLX_NUM_ZERO);
        $fields['CHARTCODE'] = array('type' => 'varchar(16)', 'null' => 'NO', 'properties' => '');
        $fields['SERIES_PROPERTIES'] = array('type' => 'mediumtext', 'null' => 'YES', 'properties' => '');
        $indexes = array();
        $ipf = 'CLX' . $this->code;
        $indexes[$ipf . '01'] = array('index' => 'PRI', 'fieldname' => 'ID');
        $indexes[$ipf . '02'] = array('index' => 'UNI', 'fieldname' => 'IDCODE,CHARTCODE');
        $indexes[$ipf . '03'] = array('index' => 'MUL', 'fieldname' => 'STATUS');
        return array('idcode' => $this->code, 'idname' => $this->name, 'name' => $this->tablename, 'type' => 'InnoDB',
            'fields' => $fields,
            'indexes' => $indexes);
    }

    function unpack($parameters) {
        $parameters = array_values(array_merge(array('package' => null, 'id' => null), $parameters));
        list($package, $chartcode) = $parameters;
        $record = $this->unpackRecord($package);
        ClxCommon::$db->createRecord($this->tablename, $record, false);
    }

    function makepanel($groupcode = null) {

        if ($this->getAccess()->canRead()) {
            $t = time();
            $to = $this->code;
            $here = 'tool.php?to=' . $to . '&t=' . $t;
            $status = array(dict('GEN_LBL_INACTIVE'), dict('GEN_LBL_ACTIVE'));
            $form = new htmlForm(CLX_PROP_ACTION, 'tool.php');
            $form->setProp(CLX_PROP_ID, 'ANCPPanel');
            $form->addArgument(CLX_PROP_TO, $this->code);
            $form->addArgument('CHARTCODE', $groupcode);
            $form->addArgument('ac', 'NOT_DEFINED');
            $selectclause = '*';
            $fromclause = 'CLX_AN_CHART_PROPERTIES';
            $whereclause = 'CHARTCODE=' . singleQuote($groupcode);
            $sortclause = '';
            $page = new xPaging();
            $page->setProp('rpp', CLX_NUM_TEN_THOUSAND);
            $page->setProp('printheaders', false);
            $page->pageInit($form->getProp(CLX_PROP_ID), $here, $selectclause, $fromclause, $whereclause, $sortclause, '');
            $table = new htmlTable(CLX_WIDTH_FOUR);
            $table->setProps(array(CLX_PROP_CLASS => CLX_CLASS_LIST, CLX_PROP_WIDTH => CLX_PROP_100PCT));
            $page->header($table);
            $table->addCells(CLX_CLASS_HEADERS, array(dict('GEN_LBL_SEL'), dict('GEN_LBL_NAME'), dict('GEN_LBL_DESCRIPTION'), dict('GEN_LBL_STATUS')));
            $i = CLX_INIT_ZERO;
            while ($record = $page->nextRecord()) {
                $recordid = $record['ID'];
                $select = new htmlCheckbox('selector_' . $recordid, false);
                $select->setProp('onclick', 'selectButtons(' . singleQuote($form->getProp(CLX_PROP_ID)) . ');');
                $table->addCell($select->source(), array(CLX_PROP_CLASS => CLX_CLASS_FOOTERS, CLX_PROP_WIDTH => '40', CLX_PROP_ALIGN => 'center'));
                $table->addCells(($i % CLX_NUM_TWO == CLX_NUM_ZERO) ? CLX_CLASS_DATA : CLX_CLASS_DATAALT, array(
                    $record['IDNAME'], $record['DESCRIPTION'], $status[$record['STATUS']]));
                $i++;
            }
            $buttons = '';

            if ($this->getAccess()->canModify()) {
                $buttons .=htmlButton::getButton('submit', dict('GEN_LBL_MODIFY'), array('action' => 'MODIFY', 'form' => $form->getProp(CLX_PROP_ID), CLX_PROP_ID => 'MULT'))->source();
            }
            $form->addContents($page->footers($table, $buttons));
            return buildPanelInterface(dict('TOOL_ANCP_TITLE_LIST'), 'TABLES', $form->source(), CLX_NUM_MINUS_1);
        } else {
            return CLX_CHAR_EMPTY_STRING;
        }
    }

    function editForm($ac, $idlist, $record, $msg) {
        $title = dict('TOOL_ANCP_TITLE_MODIFY');
        $form = new htmlForm(CLX_PROP_ACTION, 'tool.php');
        $form->addArgument(CLX_PROP_TO, $this->code);
        $form->addArgument('ac', $ac);
        $form->addArgument('confirm', '1');
        $form->addArgument('ANCP_id', $record['ANCP_id']);
        $form->addArgument('ANCP_idcode', $record['ANCP_idcode']);
        $form->addArgument('ANCP_idname', $record['ANCP_idname']);
        $form->addArgument('ANCP_description', $record['ANCP_description']);
        $form->addArgument('ANCP_chartcode', $record['ANCP_chartcode']);
        $form->addArgument('idlist', toString($idlist));
        $table = new htmlTable(CLX_WIDTH_TWO);
        $table->setProps(array(CLX_PROP_CLASS => CLX_CLASS_FORM, CLX_PROP_WIDTH => CLX_PROP_100PCT));
        $table->addCell($title, array(CLX_PROP_CLASS => CLX_CLASS_TITLE, CLX_PROP_COLSPAN => CLX_WIDTH_TWO));
        $table->addData(dict('GEN_LBL_PROPERTY') . ':', $record['ANCP_idname']);
        $table->addData(dict('GEN_LBL_DESCRIPTION') . ':', $record['ANCP_description']);
        // Status
        $pop = new htmlSelect('ANCP_status');
        $pop->addItem(dict('GEN_LBL_ACTIVE'), '1', ($record['ANCP_status'] === '1'));
        $pop->addItem(dict('GEN_LBL_INACTIVE'), '0', ($record['ANCP_status'] === '0'));
        $table->addData(dict('GEN_LBL_STATUS') . ':', $pop->source());
        $form->addContents($table->source());
        $o = getToolObject('CPTX', $record['ANCP_idcode']);
        $form->addContents($o->panel($msg, $record, true));
        $table = new htmlTable(CLX_WIDTH_TWO);
        $table->setProps(array(CLX_PROP_CLASS => CLX_CLASS_FORM, CLX_PROP_WIDTH => CLX_PROP_100PCT));
        $buttons = htmlButton::getButton('submit', dict('GEN_LBL_SAVE'), array('action' => 'MODIFY', 'form' => $form->getProp(CLX_PROP_ID)))->source();
        $c = new htmlButtonCancel();
        $buttons .= $c->source();
        $table->addCell($buttons, array(CLX_PROP_CLASS => CLX_CLASS_FOOTERS, CLX_PROP_COLSPAN => CLX_WIDTH_TWO));
        $form->addContents($table->source());
        return $form->source();
    }

    function checkForm(&$msg) {
        $msg = array();
        $o = getToolObject('CPTX', UserRequest::$post['ANCP_idcode']);
        $o->checkPanel($msg);
        return count($msg) == CLX_NUM_ZERO;
    }

    function loadToolRecord($recordid) {
        $values = ClxCommon::$db->getRecord($this->tablename, 'ID=' . $recordid);
        $record = array();
        $record['ANCP_id'] = $recordid;
        $record['ANCP_idcode'] = $values['IDCODE'];
        $record['ANCP_idname'] = $values['IDNAME'];
        $record['ANCP_description'] = $values['DESCRIPTION'];
        $record['ANCP_status'] = $values['STATUS'];
        $record['ANCP_chartcode'] = $values['CHARTCODE'];
        $record['ANCP_series_properties'] = $values['SERIES_PROPERTIES'];
        $record['ANCS_type'] = $record['ANCP_idcode'];
        $o = getToolObject('CPTX', $record['ANCP_idcode']);
        $o->loadPanel($record, $values);
        return $record;
    }

    function saveRecord() {
        $record = array();
        $record['STATUS'] = UserRequest::$post['ANCP_status'];
        $o = getToolObject('CPTX', UserRequest::$post['ANCP_idcode']);
        $o->savePanel($record);
        ClxCommon::$db->updateRecord($this->tablename, $record, 'ID=' . UserRequest::$post['ANCP_id']);
        $ANCH = &getTool('ANCH');
        $ANCH->versionIncrease('IDCODE', UserRequest::$post['ANCP_chartcode']);
    }

    function createChartProperties($chartcode) {
        $OBJS = &getTool('OBJS');
        $codes = $OBJS->objectList('CPTX');
        foreach ($codes as $code) {
            $o = $OBJS->getObject_('CPTX', $code);
            $record = array();
            $record['CHARTCODE'] = $chartcode;
            $record['IDCODE'] = $code;
            $record['IDNAME'] = $o->name;
            $record['DESCRIPTION'] = $o->description;
            $record['STATUS'] = '0';
            $record['SERIES_PROPERTIES'] = '';
            ClxCommon::$db->createRecord($this->tablename, $record, false);
        }
    }

}
