<?php

/**
 * Tool Class
 * For more information contact  support@alenet.com
 *
 * @author       $Author: jclerma $
 * @copyright    Alenet, Inc.
 * @since        $Date: 2017-11-03 15:09:40 -0400 (Fri, 03 Nov 2017) $
 * @version        $Rev: 9615 $
 *
 * CHANGE LOG
 *  Included label
 *  Oracle uppercase record changed
 */
class an_output_gen extends clx_tool {

    // CONSTRUCTOR //
    function __construct() {
        $this->code = 'ANOG';
        $this->name = 'Output Generator Manager';
        $this->system = 'Analytics';
        $this->version = '0.1.7.0';

        $this->has_access = true;
        $this->has_package = true;

        $this->tablename = 'CLX_AN_OUTPUT_GEN';
        $this->record_name_singular = 'template';
        $this->record_name_plural = 'templates';
        $this->primary_field = 'IDCODE';
        $this->secondary_field = 'IDNAME';

        $this->editform = array(CLX_CLASS_TITLE => 'TOOL_ANOG_TITLE_CREATE',
            'panels' => array(
                array('type' => 'normal',
                    'initial_state' => '1',
                    'entries' => array(
                        array('label' => 'GEN_LBL_CODE',
                            'fields' => array(
                                array('name' => 'idcode', 'type' => 'string', 'default' => '', 'size' => '16', 'displaysize' => '16', 'dontmodify' => '1',
                                    'validations' => array(
                                        array('type' => 'sqlobject', 'required' => '1'), array('type' => 'unique', 'fieldname' => 'idcode', 'tablename' => $this->tablename))
                                )
                            )
                        ), array('label' => 'GEN_LBL_NAME',
                            'fields' => array(
                                array('name' => 'idname', 'type' => 'string', 'default' => '', 'size' => '64', 'displaysize' => '48', 'validations' => array(
                                        array('type' => 'string', 'required' => '1'))
                                )
                            )
                        ), array('label' => 'GEN_LBL_VERSION',
                            'fields' => array(
                                array('name' => 'version', 'type' => 'string', 'default' => '1.0.0', 'size' => '32', 'displaysize' => '16', 'validations' => array(
                                        array('type' => 'string', 'required' => '1'))
                                )
                            )
                        ), array('label' => 'GEN_LBL_DESCRIPTION',
                            'fields' => array(
                                array('name' => 'description', 'type' => 'text', 'default' => '', 'cols' => '48', 'rows' => '4'))
                        ), array('label' => 'GEN_LBL_MODULECODE',
                            'fields' => array(
                                array('name' => 'modulecode', 'type' => 'lookup', 'default' => '', 'tool' => 'DBMO', 'validations' => array(
                                        array('type' => 'lookup', 'required' => '1', 'fieldname' => 'idcode', 'tablename' => 'CLX_DB_MODULES'))
                                )
                            )
                        ), array('label' => 'GEN_LBL_TABLECODE',
                            'fields' => array(
                                array('name' => 'tablecode', 'type' => 'lookup', 'default' => '', 'tool' => 'DBTB', 'validations' => array(
                                        array('type' => 'lookup', 'required' => '1', 'fieldname' => 'idcode', 'tablename' => 'CLX_DB_TABLES'))
                                )
                            )
                        ), array('label' => 'GEN_LBL_TYPE',
                            'fields' => array(
                                array('name' => 'type', 'type' => 'objects', 'default' => 'HTML', 'objects' => 'ANOG', 'validations' => array(
                                        array('type' => 'string', 'required' => '1'))
                                )
                            )
                        )
                    )
                )
            )
        );
    }

    function describe($primary_field_value) {
        $record = ClxCommon::$db->getRecord($this->tablename, $this->primary_field . '=' . singleQuote($primary_field_value));
        $name = $record['IDNAME'] . ' (' . $record['IDCODE'] . ')';
        $id = $record['ID'];
        $url = quote('window.php?to=' . $this->code . '&ac=VIEW&ID=' . $id);
        $title = quote('view_' . $id);
        $cmds = 'javaScript:openWindow(' . $title . ', ' . $url . ', 800, 600);' . CLX_CHAR_NL;
        $tag = '<a href="' . $cmds . '">' . $name . '</a>';
        return $tag;
    }

    function install() {
        $UIMI = getTool('UIMI');
        $UIMI->add($this->code, 'MENU_TOOLS:MENU_ANALYTICS:MENU_OUTPUT_GEN', 'TOOL', $this->code, 'ac=LIST');
    }

    function toolTableStructure() {
        $fields = array();
        $fields['ID'] = array('type' => 'int(11) unsigned', 'null' => 'NO', 'properties' => 'auto_increment', 'index' => 'PRI');
        $fields['IDCODE'] = array('type' => 'varchar(16)', 'null' => 'NO', 'properties' => '');
        $fields['IDNAME'] = array('type' => 'varchar(64)', 'null' => 'NO', 'properties' => '');
        $fields['VERSION'] = array('type' => 'varchar(32)', 'null' => 'NO', 'properties' => '');
        $fields['VERSION_COUNTER'] = array('type' => 'int(11) unsigned', 'null' => 'NO', 'properties' => '', 'default' => CLX_NUM_ZERO);
        $fields['CDATE'] = array('type' => 'datetime', 'null' => 'YES', 'properties' => '');
        $fields['MDATE'] = array('type' => 'datetime', 'null' => 'YES', 'properties' => '');
        $fields['USERNAME'] = array('type' => 'varchar(30)', 'null' => 'NO', 'properties' => '');
        $fields['DESCRIPTION'] = array('type' => 'mediumtext', 'properties' => '');
        $fields['MODULECODE'] = array('type' => 'varchar(4)', 'properties' => '');
        $fields['TABLECODE'] = array('type' => 'varchar(4)', 'properties' => '');
        $fields['TYPE'] = array('type' => 'varchar(4)', 'properties' => '');
        $fields['ERROR'] = array('type' => 'tinyint(1) unsigned', 'properties' => '');
        $fields['HAS_QUESTIONS'] = array('type' => 'tinyint(1) unsigned', 'properties' => '', 'default' => CLX_NUM_ZERO);
        $fields['ERRORMSG'] = array('type' => 'mediumtext', 'properties' => '');
        $indexes = array();
        $ipf = 'CLX' . $this->code;
        $indexes[$ipf . '01'] = array('index' => 'PRI', 'fieldname' => 'ID');
        $indexes[$ipf . '02'] = array('index' => 'UNI', 'fieldname' => 'IDCODE');
        $indexes[$ipf . '03'] = array('index' => 'MUL', 'fieldname' => 'IDNAME');
        $indexes[$ipf . '04'] = array('index' => 'MUL', 'fieldname' => 'USERNAME');
        $indexes[$ipf . '05'] = array('index' => 'MUL', 'fieldname' => 'MODULECODE');
        $indexes[$ipf . '06'] = array('index' => 'MUL', 'fieldname' => 'TABLECODE');
        return array('idcode' => $this->code, 'idname' => $this->name, 'name' => $this->tablename, 'type' => 'InnoDB', 'fields' => $fields, 'indexes' => $indexes);
    }

    function pack($record) {
        $o = $this->packRecord($record);
        $code = $record['IDCODE'];
        $objs = new XMLObject($this->code . '_OBJECTS');
        $this->packsub($objs, 'ANOF', 'SELECT * FROM CLX_AN_OUTPUT_GEN_FIELDS WHERE templatecode=' . singleQuote($code));
        $o->addObject($objs);
        return $o;
    }

    function packcheck($package, &$i, &$table) {
        $this->packdescribe($package, $i, $table);
        if ($package->hasSubObject($this->code . '_OBJECTS')) {
            $objs = $package->getObjectXml($this->code . '_OBJECTS');
            foreach ($objs->objects as $o) {
                $toolcode = $o->tag;
                $tool = &getTool($toolcode);
                $tool->packcheck($o, $i, $table);
            }
        }
    }

    function unpack($parameters) {
        $parameters = array_values(array_merge(array('package' => null), $parameters));
        list($package) = $parameters;
        $record = $this->unpackRecord($package);
        $idcode = $record['IDCODE'];
        ClxCommon::$db->deleteRecords($this->tablename, 'IDCODE=' . singleQuote($idcode));
        $templateid = ClxCommon::$db->createRecord($this->tablename, $record, false);
        unset($templateid);
        if ($package->hasSubObject($this->code . '_OBJECTS')) {
            $objs = $package->getObjectXml($this->code . '_OBJECTS');
            foreach ($objs->objects as $o) {
                $toolcode = $o->tag;
                $tool = &getTool($toolcode);
                $tool->unpack(array('package' => $o));
            }
        }
    }

    function action($ac, $idlist) {
        if ($this->getAccess()->canRead()) {
            switch ($ac) {
                case 'VIEW_EXECUTE':
                    $cmds = '';
                    foreach ($idlist as $id) {
                        $url = 'window.php?to=' . $this->code . '&ac=VIEW&id=' . $id;
                        $cmds .= 'openWindow(' . singleQuote(CLX_CHAR_EMPTY_STRING) . ', ' . singleQuote($url) . ' , 800, 600);' . CLX_CHAR_NL;
                    }
                    $contents = setJavaScriptTag($cmds, false);
                    $info = popHistory();
                    $contents .= executeHistory($info);
                    break;
                case 'VIEW':
                    $contents = $this->view_template(UserRequest::$request[CLX_PROP_ID]);
                    break;
                case 'PRINT_DIALOG':
                    if (isset(UserRequest::$post['confirm'])) {
                        $templatelist = $idlist;
                        $tablecode = UserRequest::$request['tc'];
                        $recs = toArray(UserRequest::$request['recs']);
                        $contents = $this->print_start($templatelist, $tablecode, $recs);
                    } else {
                        $tablecode = UserRequest::$request['tc'];
                        $recs = UserRequest::$request['recs'];
                        $contents = $this->print_dialog($tablecode, $recs);
                    }
                    break;
                case 'PRINT_QUESTION':
                    $printjobid = UserRequest::$request['pj'];
                    $contents = $this->print_question($printjobid);
                    break;
                case 'PRINT_ANSWER':
                    $printjobid = UserRequest::$request['pj'];
                    $answer = UserRequest::$request['record_chooser'];
                    $contents = $this->print_process_answer($printjobid, $answer);
                    break;
                case 'PRINT_EXECUTE':
                    $printjobid = UserRequest::$request['pj'];
                    $contents = '';
                    $this->print_execute($printjobid);
                    break;
                default:
                    $contents = parent::action($ac, $idlist);
            }
        } else {
            $contents = warning(dict('GEN_ERR_NO_ACCESS') . '[AN_OUTPUT_GEN-CAN_READ][' . $ac . '] ' . $this->getCode(false));
        }
        return $contents;
    }

    function makelist($listingcode = null) {
         //$t = time('U'); genera un warning
		$t = time();
        $to = $this->code;
        $here = 'tool.php?to=' . $to . '&t=' . $t;
        $yesno = array(dict('GEN_LBL_NO'), dict('GEN_LBL_YES'));
        clearHistory();
        pushHistory($this->code, 'LIST', array());
        $form = new htmlForm(CLX_PROP_ACTION, 'tool.php');
        $form->setProp(CLX_PROP_ID, 'ANOGList');
        $form->addArgument(CLX_PROP_TO, $this->code);
        $form->addArgument('ac', 'NOT_DEFINED');
        $selectclause = '*';
        $fromclause = 'CLX_AN_OUTPUT_GEN';
        $whereclause = '';
        $sortclause = 'IDNAME';
        $page = new xPaging();
        $page->pageInit($form->getProp(CLX_PROP_ID), $here, $selectclause, $fromclause, $whereclause, $sortclause, '');
        $table = new htmlTable(CLX_WIDTH_EIGHT);
        $table->setProps(array(CLX_PROP_CLASS => CLX_CLASS_LIST, CLX_PROP_WIDTH => CLX_PROP_100PCT));
        $table->addCell(dict('TOOL_ANOG_TITLE_LIST'), array(CLX_PROP_CLASS => CLX_CLASS_TITLE, CLX_PROP_COLSPAN => CLX_WIDTH_EIGHT));
        $page->header($table);
        $table->addCells(CLX_CLASS_HEADERS, array(dict('GEN_LBL_SELECT'), dict('GEN_LBL_CODE'), dict('GEN_LBL_NAME'), dict('GEN_LBL_MODULECODE'), dict('GEN_LBL_TYPE'), dict('GEN_LBL_ERRORS'), dict('GEN_LBL_VERSION'), dict('GEN_LBL_DESCRIPTION')));
        $i = CLX_INIT_ZERO;
        while ($record = $page->nextRecord()) {
            $recordid = $record['ID'];
            $select = new htmlCheckbox('selector_' . $recordid, false);
            $select->setProp('onclick', 'selectButtons(' . singleQuote($form->getProp(CLX_PROP_ID)) . ');');
            $table->addCell($select->source(), array(CLX_PROP_CLASS => CLX_CLASS_FOOTERS, CLX_PROP_WIDTH => '40', CLX_PROP_ALIGN => 'center'));
            if (empty($record['ERROR']))
                $record['ERROR'] = CLX_INIT_ZERO;
            $o = getToolObject($this->code, $record['TYPE']);
            $table->addCells(($i % CLX_NUM_TWO == CLX_NUM_ZERO) ? CLX_CLASS_DATA : CLX_CLASS_DATAALT, array($record['IDCODE'], $record['IDNAME'], $record['MODULECODE'], $o->name, $yesno[$record['ERROR']], $record['VERSION'] . '-' . $record['VERSION_COUNTER'], xFormat::text($record['DESCRIPTION'])));
            $i++;
        }
        $buttons = '';
        $buttons .= htmlButton::getButton('submit', dict('GEN_LBL_DISPLAY'), array('action' => 'DISPLAY', 'form' => $form->getProp(CLX_PROP_ID), CLX_PROP_ID => 'MULT'))->source();
        $b = new htmlFormSubmit(dict('GEN_LBL_VIEW'), 'VIEW_EXECUTE', $form->getProp(CLX_PROP_ID));
        $b->setProp(CLX_PROP_ID, 'MULT');
        $buttons .= $b->source();
        if ($this->getAccess()->canModify()) {
            $buttons .=htmlButton::getButton('submit', dict('GEN_LBL_MODIFY'), array('action' => 'MODIFY', 'form' => $form->getProp(CLX_PROP_ID), CLX_PROP_ID => 'MULT'))->source();
        }
        if ($this->getAccess()->canDelete()) {
            $buttons .=htmlButton::getButton('submit', dict('GEN_LBL_DELETE'), array('action' => 'DELETE', 'form' => $form->getProp(CLX_PROP_ID), CLX_PROP_ID => 'MULT'))->source();
        }
        if ($this->getAccess()->canDisplay()) {
            $b = new htmlFormSubmit(dict('GEN_LBL_IMPORT_SOMETHING', dict('GEN_LBL_TEMPLATE')), 'CREATE', $form->getProp(CLX_PROP_ID));
            $b->setProp(CLX_PROP_ID, 'ON');
            $buttons .= $b->source();
        }
        $buttons .=htmlButton::getButton('submit', dict('GEN_LBL_FILTER'), array('action' => 'FILTER', 'form' => $form->getProp(CLX_PROP_ID), CLX_PROP_ID => 'ON'))->source();
        $form->addContents($page->footers($table, $buttons));
        return $form->source();
    }

    function searchForm() {
        $msg = array();
        $contents = '';
        // Search Form
        $form = new htmlForm(CLX_PROP_ACTION, UserRequest::$server['PHP_SELF']);
        $form->addArgument(CLX_PROP_TO, $this->code);
        $form->addArgument('ac', 'SEARCH');
        $form->addArgument('confirm', '1');
        $form->addArgument(CLX_PROP_TARGET, UserRequest::$request[CLX_PROP_TARGET]);
        $table = new htmlTable(CLX_WIDTH_TWO);
        $table->setProps(array(CLX_PROP_CLASS => CLX_CLASS_FORM, CLX_PROP_WIDTH => CLX_PROP_100PCT));
        $table->addCell(dict('TOOL_ANOG_TITLE_SEARCH'), array(CLX_PROP_CLASS => CLX_CLASS_TITLE, CLX_PROP_COLSPAN => CLX_WIDTH_TWO));
        $table->addField($msg, dict('GEN_LBL_CODE') . ':', 'ANOG_idcode', (array_key_exists('ANOG_idcode', UserRequest::$post)) ? UserRequest::$post['ANOG_idcode'] : '', CLX_NUM_16, CLX_NUM_16);
        $table->addField($msg, dict('GEN_LBL_NAME') . ':', 'ANOG_idname', (array_key_exists('ANOG_idname', UserRequest::$post)) ? UserRequest::$post['ANOG_idname'] : '', CLX_NUM_32, CLX_NUM_64);
        $buttons = htmlButton::getButton('submit', dict('GEN_LBL_SEARCH'), array('action' => 'SEARCH', 'form' => $form->getProp(CLX_PROP_ID)))->source();
        $c = new htmlButtonCloseWindow(dict('GEN_LBL_CANCEL'));
        $buttons .= $c->source();
        $table->addCell($buttons, array(CLX_PROP_CLASS => CLX_CLASS_FOOTERS, CLX_PROP_COLSPAN => CLX_WIDTH_TWO));
        $form->setContents($table->source());
        $contents .= $form->source();
        // Search Procedure
        if (isset(UserRequest::$post['confirm'])) {
            $table = new htmlTable(CLX_WIDTH_TWO);
            $table->setProps(array(CLX_PROP_CLASS => CLX_CLASS_LIST, CLX_PROP_WIDTH => CLX_PROP_100PCT));
            $criteria = '';
            if (UserRequest::$post['ANOG_idcode'] <> '') {
                $criteria .= 'UPPER(IDCODE) LIKE ' . singleQuote(uppercase(UserRequest::$post['ANOG_idcode']) . '%');
            }
            if (UserRequest::$post['ANOG_idname'] <> '') {
                $criteria .= 'UPPER(IDNAME) LIKE ' . singleQuote(uppercase(clx_change_especial_char_html (UserRequest::$post['ANOG_idname'])) . '%');
            }
            $qty = ClxCommon::$db->oneExistsRecords($this->tablename, $criteria);
            if ($qty == CLX_NUM_ZERO) {
                $table->addCell(dict('TOOL_GEN_ERR_NOMATCH'), array(CLX_PROP_CLASS => CLX_CLASS_ALERT, CLX_PROP_COLSPAN => CLX_WIDTH_TWO));
            } else {
                $table->addCell(dict('TOOL_GEN_MSG_FOUNDRECORDS'), array(CLX_PROP_CLASS => CLX_CLASS_SUBTITLE, CLX_PROP_COLSPAN => CLX_WIDTH_TWO));
                $records = ClxCommon::$db->getRecords($this->tablename, $criteria);
                $i = CLX_INIT_ZERO;
                while ($record = ClxCommon::$db->getNextRecord($records, ASSOCIATIVE)) {
                    $js = 'javascript:setLookupValue(' . quote(UserRequest::$request[CLX_PROP_TARGET]) . ', ' . quote($record['IDCODE']) . ');';
                    $table->addCells(($i % CLX_NUM_TWO == CLX_NUM_ZERO) ? CLX_CLASS_DATA : CLX_CLASS_DATAALT, array('<page t=an_output_gen></page><a href=' . singleQuote($js) . '>' . $record['IDCODE'] . '</a>', $record['IDNAME']));
                    $i++;
                }
            }
            $contents .= $table->source();
        }
        return $contents;
    }

    function confirmDeleteFill(&$table, $record) {
        $table->addData(dict('GEN_LBL_CODE') . ':', $record['ANOG_idcode']);
        $table->addData(dict('GEN_LBL_NAME') . ':', $record['ANOG_idname']);
    }

    function viewFormFill($record) {
        addHistory('templatecode', $record['ANOG_idcode']);
        $contents = '';
        // Build the tree and save the errors...
        $tree = $this->createFieldTree($record['ANOG_idcode'], $record['ANOG_tablecode']);
        $r = array();
        $r['ERROR'] = $tree->hasErrors();
        $r['HAS_QUESTIONS'] = $tree->hasQuestion();
        ClxCommon::$db->updateRecord($this->tablename, $r, 'IDCODE=' . singleQuote($record['ANOG_idcode']));
        // Create the view form...
        $yesno = array(dict('GEN_LBL_NO'), dict('GEN_LBL_YES'));
        $table = new htmlTable(CLX_WIDTH_TWO);
        $table->setProps(array(CLX_PROP_CLASS => CLX_CLASS_FORM, CLX_PROP_WIDTH => CLX_PROP_100PCT));
        $table->addCell(dict('TOOL_ANOG_TITLE_VIEW') . ': ' . $record['ANOG_idname'], array(CLX_PROP_CLASS => CLX_CLASS_TITLE, CLX_PROP_COLSPAN => CLX_WIDTH_TWO));
        $table->addData(dict('GEN_LBL_CODE') . ':', $record['ANOG_idcode']);
        $table->addData(dict('GEN_LBL_NAME') . ':', $record['ANOG_idname']);
        $table->addData(dict('GEN_LBL_VERSION') . ':', $record['ANOG_version'] . '-' . $record['ANOG_version_counter']);
        $table->addData(dict('GEN_LBL_DESCRIPTION') . ':', xFormat::text($record['ANOG_description']));
        $table->addData(dict('GEN_LBL_MODULECODE') . ':', $record['ANOG_modulecode']);
        $table->addData(dict('GEN_LBL_TABLECODE') . ':', $record['ANOG_tablecode']);
        $table->addData(dict('GEN_LBL_ERRORS') . ':', $yesno[$r['ERROR']]);
        $contents .= $table->source();
        $ANOF = &getTool('ANOF');
        $contents .= $ANOF->makepanel($record['ANOG_idcode']);
        $table = new htmlTable(CLX_WIDTH_ONE);
        $table->setProps(array(CLX_PROP_CLASS => CLX_CLASS_FORM, CLX_PROP_WIDTH => CLX_PROP_100PCT));
        $table->addCell($tree->dump(), array(CLX_PROP_CLASS => CLX_CLASS_DATA));
        $contents .= buildPanelInterface(dict('GEN_LBL_MAP'), 'FORM_MAP', $table->source(), CLX_NUM_MINUS_1);
        $table = new htmlTable(CLX_WIDTH_TWO);
        $table->setProps(array(CLX_PROP_CLASS => CLX_CLASS_FORM, CLX_PROP_WIDTH => CLX_PROP_100PCT));
        $table->addData(dict('GEN_LBL_CDATE') . ':', $record['ANOG_cdate']);
        $table->addData(dict('GEN_LBL_MDATE') . ':', $record['ANOG_mdate']);
        $table->addData(dict('GEN_LBL_USERNAME') . ':', $record['ANOG_username']);
        $contents .= buildPanelInterface(dict('GEN_TTL_SYSTEMINFO'), 'SYSTEMINFO', $table->source(), CLX_NUM_MINUS_1);
        return $contents;
    }

    function filterFormFill($msg, &$table) {
        if (!array_key_exists('ANOG_idcode', UserRequest::$post))
            UserRequest::$post['ANOG_idcode'] = '';
        if (!array_key_exists('ANOG_idname', UserRequest::$post))
            UserRequest::$post['ANOG_idname'] = '';
        if (!array_key_exists('ANOG_modulecode', UserRequest::$post))
            UserRequest::$post['ANOG_modulecode'] = '';
        $table->addField($msg, dict('GEN_LBL_CODE'), 'ANOG_idcode', UserRequest::$post['ANOG_idcode'], CLX_NUM_16, CLX_NUM_16);
        $table->addField($msg, dict('GEN_LBL_NAME'), 'ANOG_idname', UserRequest::$post['ANOG_idname'], CLX_NUM_32, CLX_NUM_64);
        $table->addField($msg, dict('GEN_LBL_MODULECODE'), 'ANOG_modulecode', UserRequest::$post['ANOG_modulecode'], CLX_NUM_FOUR, CLX_NUM_FOUR);
        $sort = new HtmlSelect('ANOG_sort');
        $sort->addItem('', '', true);
        $sort->addItem(dict('GEN_LBL_MODULECODE'), 'modulecode', false);
        $sort->addItem(dict('GEN_LBL_CODE'), 'idcode', false);
        $sort->addItem(dict('GEN_LBL_NAME'), 'idname', false);
        $table->addData(dict('GEN_LBL_ORDERBY') . ':', $sort->source());
    }

    function setfilter() {
        $filter = '';
        $boolean = '';
        if (!empty(UserRequest::$post['ANOG_idcode'])) {
            $filter .= $boolean . ' UPPER(IDCODE) LIKE ' . singleQuote(uppercase(UserRequest::$post['ANOG_idcode']) . '%');
            $boolean = 'AND';
        }
        if (!empty(UserRequest::$post['ANOG_idname'])) {
            $filter .= $boolean . ' UPPER(IDNAME) LIKE ' . singleQuote(clx_change_especial_char_html (uppercase(UserRequest::$post['ANOG_idname'])) . '%');
            $boolean = 'AND';
        }
        if (!empty(UserRequest::$post['ANOG_modulecode'])) {
            $filter .= $boolean . ' UPPER(MODULECODE) LIKE ' . singleQuote(uppercase(UserRequest::$post['ANOG_modulecode']) . '%');
            $boolean = 'AND';
        }

        $this->setFilterKeys($filter, UserRequest::$post['ANOG_sort'], 0);
    }

    function loadToolRecord($recordid) {
        $values = ClxCommon::$db->getRecord($this->tablename, 'ID=' . $recordid);
        $record = array();
        $record['ANOG_id'] = $recordid;
        $record['ANOG_idcode'] = $values['IDCODE'];
        $record['ANOG_idname'] = $values['IDNAME'];
        $record['ANOG_version'] = $values['VERSION'];
        $record['ANOG_version_counter'] = $values['VERSION_COUNTER'];
        $record['ANOG_cdate'] = xFormat::date($values['CDATE']);
        $record['ANOG_mdate'] = xFormat::date($values['MDATE']);
        $record['ANOG_username'] = $values['USERNAME'];
        $record['ANOG_description'] = $values['DESCRIPTION'];
        $record['ANOG_modulecode'] = $values['MODULECODE'];
        $record['ANOG_tablecode'] = $values['TABLECODE'];
        $record['ANOG_type'] = $values['TYPE'];
        $record['ANOG_error'] = $values['ERROR'];
        $record['ANOG_errormsg'] = $values['ERRORMSG'];
        return $record;
    }

    function saveRecord() {
        $d = xFormat::getNow('Y-m-d');
        $record = array();
        $record['IDCODE'] = UserRequest::$post['ANOG_idcode'];
        $record['IDNAME'] = UserRequest::$post['ANOG_idname'];
        $record['VERSION'] = UserRequest::$post['ANOG_version'];
        $record['MDATE'] = $d;
        $record['USERNAME'] = $_SESSION['user_record']['username'];
        $record['DESCRIPTION'] = UserRequest::$post['ANOG_description'];
        $record['MODULECODE'] = UserRequest::$post['ANOG_modulecode'];
        $record['TABLECODE'] = UserRequest::$post['ANOG_tablecode'];
        $record['TYPE'] = UserRequest::$post['ANOG_type'];
        ClxCommon::$db->updateRecord($this->tablename, $record, 'ID=' . UserRequest::$post['ANOG_id']);
        $this->versionIncrease('ID', UserRequest::$post['ANOG_id']);
        $record['ID'] = UserRequest::$post['ANOG_id'];
        $this->processTemplate($record);
    }

    function createRecord_() {
        $d = xFormat::getNow('Y-m-d');
        $record = array();
        $record['IDCODE'] = UserRequest::$post['ANOG_idcode'];
        $record['IDNAME'] = UserRequest::$post['ANOG_idname'];
        $record['VERSION'] = UserRequest::$post['ANOG_version'];
        $record['VERSION_COUNTER'] = '0';
        $record['CDATE'] = $d;
        $record['MDATE'] = $d;
        $record['USERNAME'] = $_SESSION['user_record']['username'];
        $record['DESCRIPTION'] = UserRequest::$post['ANOG_description'];
        $record['MODULECODE'] = UserRequest::$post['ANOG_modulecode'];
        $record['TABLECODE'] = UserRequest::$post['ANOG_tablecode'];
        $record['TYPE'] = UserRequest::$post['ANOG_type'];
        $record['ERROR'] = '0';
        $record['ERRORMSG'] = '';
        $record['HAS_QUESTIONS'] = '0';
        $recordid = ClxCommon::$db->createRecord($this->tablename, $record, false);
        $DBMO = &getTool('DBMO');
        $DBMO->versionIncrease('IDCODE', $record['MODULECODE']);
        $record['ID'] = $recordid;
        $this->processTemplate($record);
        return $recordid;
    }

    function deleteRecordSk($id) {
        $rec = ClxCommon::$db->getRecord($this->tablename, 'ID=' . $id);
        $templatecode = $rec['IDCODE'];
        $object = getToolObject($this->code, $rec['TYPE']);
        $object->deletefiles($rec['ID']);
        ClxCommon::$db->deleteRecords('CLX_AN_OUTPUT_GEN_FIELDS', 'TEMPLATECODE=' . singleQuote($templatecode));
        ClxCommon::$db->deleteRecords($this->tablename, 'ID=' . $id);
        $object = getToolObject($this->code, $rec['TYPE']);
        $object->deletefiles($id);
        $DBMO = &getTool('DBMO');
        $DBMO->versionIncrease('IDCODE', $rec['MODULECODE']);
    }

    function defaultRecord() {
        $record = array();
        $record['ANOG_id'] = CLX_INIT_ZERO;
        $record['ANOG_idcode'] = '';
        $record['ANOG_idname'] = '';
        $record['ANOG_version'] = '1.0.0';
        $record['ANOG_cdate'] = '';
        $record['ANOG_mdate'] = '';
        $record['ANOG_username'] = $_SESSION['user_record']['username'];
        $record['ANOG_description'] = '';
        $record['ANOG_modulecode'] = '';
        $record['ANOG_tablecode'] = '';
        $record['ANOG_type'] = 'AHTM';
        return $record;
    }

    function versionIncrease($fieldname, $fieldvalue) {
        parent::versionIncrease($fieldname, $fieldvalue);
        $modulecode = ClxCommon::$db->getField($this->tablename, 'MODULECODE', $fieldname . '=' . singleQuote($fieldvalue));
        $DBMO = &getTool('DBMO');
        $DBMO->versionIncrease('IDCODE', $modulecode);
    }

    function processTemplate($templaterecord) {
        $object = getToolObject($this->code, $templaterecord['TYPE']);
        $object->extractFields($templaterecord);
        $this->checkTemplate($templaterecord);
    }

    function checkTemplate($templaterecord) {
        $templatecode = $templaterecord['IDCODE'];
        $tree = $this->createFieldTree($templatecode, '');
        $r = array();
        $r['ERROR'] = $tree->hasErrors();
        $r['HAS_QUESTIONS'] = $tree->hasQuestion();
        ClxCommon::$db->updateRecord($this->tablename, $r, 'IDCODE=' . singleQuote($templatecode));
    }

    function createFieldTree($templatecode, $template_tablecode) {
        require_once kPathToWebSite . 'classes/clx_datatree.php';
        require_once kPathToWebSite . 'classes/clx_dataleaf.php';
        $tree = new clx_datatree($template_tablecode);
        $leafs = ClxCommon::$db->getRecords('CLX_AN_OUTPUT_GEN_FIELDS', 'TEMPLATECODE=' . singleQuote($templatecode));
        while (($leaf = ClxCommon::$db->getNextRecord($leafs))) {
            $tree->parse($leaf['FIELDNAME'], $leaf['ID'], $leaf['MAP'], $leaf);
        }
        return $tree;
    }

    function print_dialog($tablecode, $recs) {

        $form = new htmlForm(CLX_PROP_ACTION, 'pop.php');
        $form->addArgument(CLX_PROP_TO, $this->code);
        $form->addArgument('ac', 'PRINT_DIALOG');
        $form->addArgument('confirm', 1);
        $form->addArgument('tc', $tablecode);
        $form->addArgument('recs', $recs);
        $table = new htmlTable(CLX_WIDTH_FOUR);
        $table->setProps(array(CLX_PROP_CLASS => CLX_CLASS_FORM, CLX_PROP_ALIGN => 'center'));
        $table->addCell(dict('TOOL_ANOG_CHOOSE_TEMPLATE'), array(CLX_PROP_CLASS => CLX_CLASS_TITLE, CLX_PROP_COLSPAN => CLX_WIDTH_FOUR));
        $table->addCell(dict('TOOL_ANOG_CHOOSE_TEMPLATE_EXPLAIN'), array(CLX_PROP_CLASS => CLX_CLASS_DATA, CLX_PROP_COLSPAN => CLX_WIDTH_FOUR));
        $table->addCells(CLX_CLASS_HEADERS, array(dict('GEN_LBL_SELECT'), dict('GEN_LBL_NAME'), dict('GEN_LBL_CODE'), dict('GEN_LBL_TYPE')));
        $criteria = 'TABLECODE = ' . singleQuote($tablecode) . ' ';
        $criteria .='AND (ERROR IS NULL OR ERROR != 1) ORDER BY IDNAME';
        $records = ClxCommon::$db->getRecords($this->tablename, 'TABLECODE= ' . singleQuote($tablecode) . ' ', $criteria);
        $i = CLX_INIT_ZERO;
        while ($record = ClxCommon::$db->getNextRecord($records, ASSOCIATIVE)) {
            $recordid = $record['ID'];
            $select = new htmlCheckbox('selector_' . $recordid, false);
            $table->addCell($select->source(), array(CLX_PROP_CLASS => CLX_CLASS_FOOTERS, CLX_PROP_WIDTH => '40', CLX_PROP_ALIGN => 'center'));
            $o = getToolObject($this->code, $record['TYPE']);
            $table->addCells(($i % CLX_NUM_TWO == CLX_NUM_ZERO) ? CLX_CLASS_DATA : CLX_CLASS_DATAALT, array($record['IDNAME'], $record['IDCODE'], $o->name));
            $i++;
        }
        $c = new htmlSubmit(dict('GEN_LBL_PRINT'), 'PRINT_DIALOG');
        $buttons = $c->source();
        $c = new htmlButtonCloseWindow(dict('GEN_LBL_CANCEL'));
        $buttons .= $c->source();
        $table->addCell($buttons, array(CLX_PROP_CLASS => CLX_CLASS_FOOTERS, CLX_PROP_COLSPAN => CLX_WIDTH_FOUR));
        $form->setContents($table->source());
        $contents = $form->source();
        return $contents;
    }

    function print_start($templatelist, $tablecode, $recs) {
        $_SESSION[$this->code] = array();
        $printjobid = xFormat::getNow('U');
        foreach ($templatelist as $templateid) {
            foreach ($recs as $recordid) {
                $printjobid++;
                $printjob = array();
                $printjob[CLX_PROP_ID] = $printjobid;
                $printjob['templateid'] = $templateid;
                $printjob['recordid'] = $recordid;
                $printjob['tablecode'] = $tablecode;
                $printjob['answers'] = array();
                $_SESSION[$this->code][$printjobid] = $printjob;
            }
        }
        return $this->print_process_all();
    }

    function print_process_all() {
        $cmds = '';
        foreach ($_SESSION[$this->code] as $printjobid => $printjob) {
            $cmds .= $this->print_process($printjobid);
        }
        $cmds .= CLX_CHAR_NL . '' . CLX_CHAR_NL . 'window.close();' . CLX_CHAR_NL . CLX_CHAR_NL;
        return setJavaScriptTag($cmds, false);
    }

    function print_process($printjobid) {
        $cmds = '';
        $templateid = $_SESSION[$this->code][$printjobid]['templateid'];
        $templaterec = ClxCommon::$db->getRecord($this->tablename, 'ID=' . $templateid);
        $tree = $this->createFieldTree($templaterec['IDCODE'], $templaterec['TABLECODE']);
        $question = $tree->load($_SESSION[$this->code][$printjobid]['recordid'], $_SESSION[$this->code][$printjobid]['answers']);
        if (strlen($question) > CLX_NUM_ZERO) {
            $_SESSION[$this->code][$printjobid]['question'] = $question;
            $url = 'pop.php?to=' . $this->code . '&ac=PRINT_QUESTION&pj=' . $printjobid;
            $cmds .= 'openWindow(' . singleQuote(CLX_CHAR_EMPTY_STRING) . ', ' . singleQuote($url) . '", 500, 500);' . CLX_CHAR_NL;
        } else {
            $url = 'window.php?to=' . $this->code . '&ac=PRINT_EXECUTE&pj=' . $printjobid;
            $cmds .= 'openWindow(' . singleQuote(CLX_CHAR_EMPTY_STRING) . ', ' . singleQuote($url) . ', 800, 600);' . CLX_CHAR_NL;
        }
        return $cmds;
    }

    function print_process_answer($printjobid, $answer) {
        $question = $_SESSION[$this->code][$printjobid]['question'];
        $_SESSION[$this->code][$printjobid]['answers'][$question] = $answer;
        $cmds = $this->print_process($printjobid);
        $cmds .= CLX_CHAR_NL . '' . CLX_CHAR_NL . 'window.close();' . CLX_CHAR_NL . CLX_CHAR_NL;
        return setJavaScriptTag($cmds,false);
    }

    function print_question($printjobid) {
        $recordid = $_SESSION[$this->code][$printjobid]['recordid'];
        $question = $_SESSION[$this->code][$printjobid]['question'];
        $contents = '';
        //-start addition
        if (strstr($question, ':') === true) {//added to set chooser option
            //tokens setup
            $tokens = explode(':', $question);
            $father_tablecode = $tokens[CLX_NUM_ZERO];
            $father_tablerec = ClxCommon::$db->getRecord('CLX_DB_TABLES', 'IDCODE=' . singleQuote($father_tablecode));
            $father_tablerecindex = ClxCommon::$db->getRecord('CLX_DB_TABLEINDEXES', 'TABLECODE=' . singleQuote($father_tablecode) . ' AND FIELDNAME!=' . singleQuote('I') . '');
            if (empty($father_tablerecindex['fieldname'])) {
                $father_tablerecindex['fieldname'] = '';
            }
            $father_tablename = $father_tablerec['MODULECODE'] . '_' . $father_tablerec['IDNAME'];
            $father_keyfield = $tokens[CLX_NUM_ONE];
            $father_keyfieldvalue = $tokens[CLX_NUM_TWO];
            //- setting form
            $form = new htmlForm(CLX_PROP_ACTION, 'pop.php');
            $form->addArgument(CLX_PROP_TO, $this->code);
            $form->addArgument('ac', 'PRINT_ANSWER');
            $form->addArgument('confirm', '1');
            $form->addArgument('pj', $printjobid);
            $table = new htmlTable(CLX_WIDTH_ONE);
            $table->setProps(array(CLX_PROP_CLASS => CLX_CLASS_FORM, CLX_PROP_ALIGN => 'center'));
            $table->addCell(dict('TOOL_ANOG_CHOOSE_RECORD'), array(CLX_PROP_CLASS => CLX_CLASS_TITLE, CLX_PROP_COLSPAN => CLX_WIDTH_ONE));
            $table->addCell(xFormat::text(dict('TOOL_ANOG_CHOOSE_RECORD_EXPLAIN', dict($father_tablerec['recordname_singular']))), array(CLX_PROP_CLASS => CLX_CLASS_DATA, CLX_PROP_COLSPAN => CLX_WIDTH_ONE));
            $i = CLX_INIT_ZERO;
            // Field Labels
            $father_field_info = ClxCommon::$db->getRecord('CLX_DB_FIELDS', 'FIELDNAME=' . singleQuote($father_tablerecindex['fieldname']));
            $table->addCells(CLX_CLASS_HEADERS, array(dict('GEN_LBL_SELECT'), dict('GEN_LBL_id'), dict($father_field_info['FIELDLISTLABEL'])));
            unset($father_field_info);
            $sql = 'SELECT * FROM ' . $father_tablename . ' WHERE ' . $father_keyfield . '=' . $father_keyfieldvalue . ' ORDER BY ID DESC';
            $records = ClxCommon::$db->executeQuery($sql);
            while (($record = ClxCommon::$db->getNextRecord($records, ASSOCIATIVE))) {
                $recordid = $record['ID'];
                $select = new htmlRadioButton('record_chooser', ($i == CLX_NUM_ZERO), $recordid);
                $table->addCell($select->source(), array(CLX_PROP_CLASS => CLX_CLASS_FOOTERS, CLX_PROP_WIDTH => '40', CLX_PROP_ALIGN => 'center'));
                $table->addCells(($i % CLX_NUM_TWO == CLX_NUM_ZERO) ? CLX_CLASS_DATA : CLX_CLASS_DATAALT, array($record['ID'], $record[$father_tablerecindex['fieldname']]));
                $i++;
            }
            $c = new htmlSubmit(dict('GEN_LBL_PRINT'), 'FRM_PRINT_QUE');
            $buttons = $c->source();
            $c = new htmlButtonCloseWindow(dict('GEN_LBL_CANCEL'));
            $buttons .= $c->source();
            $table->addCell($buttons, array(CLX_PROP_CLASS => CLX_CLASS_FOOTERS, CLX_PROP_COLSPAN => CLX_WIDTH_ONE));
            $form->setContents($table->source());
            $contents = $form->source();
        } else {
            //end addition continue on original dynamic
            //$question = ACCO,OPPO Father and son
            $tokens = explode(',', $question);
            $father_tablecode = $tokens[CLX_NUM_ZERO];
            $father_tablerec = ClxCommon::$db->getRecord('CLX_DB_TABLES', 'IDCODE=' . singleQuote($father_tablecode));
            $father_tablename = $father_tablerec['MODULECODE'] . '_' . $father_tablerec['IDNAME'];
            $son_tablecode = $tokens[CLX_NUM_ONE];
            $son_tablerec = ClxCommon::$db->getRecord('CLX_DB_TABLES', 'IDCODE=' . singleQuote($son_tablecode));
            $son_tablename = $son_tablerec['MODULECODE'] . '_' . $son_tablerec['IDNAME'];
            $form = new htmlForm(CLX_PROP_ACTION, 'pop.php');
            $form->addArgument(CLX_PROP_TO, $this->code);
            $form->addArgument('ac', 'PRINT_ANSWER');
            $form->addArgument('confirm', '1');
            $form->addArgument('pj', $printjobid);
            $table = new htmlTable(CLX_WIDTH_THREE);
            $table->setProps(array(CLX_PROP_CLASS => CLX_CLASS_FORM, CLX_PROP_ALIGN => 'center'));
            $table->addCell(dict('TOOL_ANOG_CHOOSE_RECORD'), array(CLX_PROP_CLASS => CLX_CLASS_TITLE, CLX_PROP_COLSPAN => CLX_WIDTH_THREE));
            $table->addCell(xFormat::text(dict('TOOL_ANOG_CHOOSE_RECORD_EXPLAIN', dict($son_tablerec['RECORDNAME_SINGULAR']))), array(CLX_PROP_CLASS => CLX_CLASS_DATA, CLX_PROP_COLSPAN => CLX_WIDTH_THREE));
            $i = CLX_INIT_ZERO;
            $rel_rec = ClxCommon::$db->getRecord('CLX_DB_DYNRELATIONS AND PARENT_TABLEDCODE=' . singleQuote($father_tablecode) . ' AND CHILD_TABLECODE=' . singleQuote($sonfather_tablecode) . ' AND RELATION_TYPE = 0');
            $rel_tablerec = ClxCommon::$db->getRecord('CLX_DB_TABLES', 'IDCODE=' . singleQuote($rel_rec['TABLECODE']));
            $rel_tablename = $rel_tablerec['MODULECODE'] . '_' . $rel_tablerec['IDNAME'];
            $f1 = ClxCommon::$db->getRecord('CLX_DB_FIELDS', 'FIELDNAME=' . singleQuote($rel_rec['PRIMARY_SEARCHFIELD']));
            $f2 = ClxCommon::$db->getRecord('CLX_DB_FIELDS', 'FIELDNAME=' . singleQuote($rel_rec['SECONDARY_SEARCHFIELD']));
            $table->addCells(CLX_CLASS_HEADERS, array(dict('GEN_LBL_SELECT'), dict($f1['fieldlistlabel']), dict($f2['fieldlistlabel'])));
            $sql = ' SELECT ' . $son_tablecode . '.* FROM ' . $rel_tablename . ' x, ' . $son_tablename . ' ' . $son_tablecode . ' WHERE x.PARENT_TABLECODE = ' . singleQuote($father_tablecode) . ' and x.PARENT_RECORDID = ' . $recordid . ' and CHILD_TABLECODE = ' . singleQuote($son_tablecode) . ' and x.CHILD_RECORDID = ' . $son_tablecode . ID . ' ';
            $records = ClxCommon::$db->executeQuery($sql);
            while (($record = ClxCommon::$db->getNextRecord($records, ASSOCIATIVE))) {
                $recordid = $record['ID'];
                $select = new htmlRadioButton('record_chooser', ($i == CLX_NUM_ZERO), $recordid);
                $table->addCell($select->source(), array(CLX_PROP_CLASS => CLX_CLASS_FOOTERS, CLX_PROP_WIDTH => '40', CLX_PROP_ALIGN => 'center'));
                $table->addCells(($i % CLX_NUM_TWO == CLX_NUM_ZERO) ? CLX_CLASS_DATA : CLX_CLASS_DATAALT, array($record[$f1['fieldname']], $record[$f2['fieldname']]));
                $i++;
            }
            $c = new htmlSubmit(dict('GEN_LBL_PRINT'), 'FRM_PRINT_QUE');
            $buttons .= $c->source();
            $c = new htmlButtonCloseWindow(dict('GEN_LBL_CANCEL'));
            $buttons .= $c->source();
            $table->addCell($buttons, array(CLX_PROP_CLASS => CLX_CLASS_FOOTERS, CLX_PROP_COLSPAN => CLX_WIDTH_THREE));
            $form->setContents($table->source());
            $contents = $form->source();
        }//end if strstr
        return $contents;
    }

    function print_execute($printjobid) {
        $contents = '';
        $headers = array();
	
        $templateid = $_SESSION[$this->code][$printjobid]['templateid'];
        $recordid = $_SESSION[$this->code][$printjobid]['recordid'];
        $tablecode = $_SESSION[$this->code][$printjobid]['tablecode'];
        $answers = $_SESSION[$this->code][$printjobid]['answers'];
        $templaterec = ClxCommon::$db->getRecord($this->tablename, 'ID=' . $templateid);
        $tree = $this->createFieldTree($templaterec['IDCODE'], $templaterec['TABLECODE']);
        $answer = $tree->load($recordid, $answers);

        $o = getToolObject($this->code, $templaterec['TYPE']);
        $o->printtemplate($templaterec, $tree, $contents, $headers);
        foreach ($headers as $key => $value) {
            header($key . ': ' . $value);
        }
        echo $contents;
        unset($_SESSION[$this->code][$printjobid]);
        $ACAD = getTool('ACAD');
        $ACAD->log($tablecode, $recordid, 'PRINT_REC', $templaterec['IDNAME'], $templaterec);
        unset($answer);
    }

    function merge($templatecode, $record, &$contents, &$headers) {
        $templaterec = ClxCommon::$db->getRecord($this->tablename, 'IDCODE=' . singleQuote($templatecode));
        $tree = $this->createFieldTree($templaterec['IDCODE'], $templaterec['TABLECODE']);
        $ignore = $tree->load($record['ID'], $templaterec['TABLECODE']);
        $o = getToolObject($this->code, $templaterec['TYPE']);
        $o->printtemplate($templaterec, $tree, $contents, $headers);
        unset($ignore);
    }

    function view_template($id) {
        $templaterec = ClxCommon::$db->getRecord($this->tablename, 'ID=' . $id);
        $o = getToolObject($this->code, $templaterec['TYPE']);
        return $o->view_template($templaterec);
    }

}
