<?php

/**
 * Tool Class
 * For more information contact  support@alenet.com
 *
 * @author       $Author: cramirez $
 * @copyright    Alenet, Inc.
 * @since        $Date: 2018-08-23 18:18:58 -0400 (Thu, 23 Aug 2018) $
 * @version        $Rev: 9902 $
 *
 * CHANGE LOG
 *  Oracle uppercase record changed
 *  Included label
 */
class an_questionary_questions extends clx_tool {

    var $question_types;
    var $data_types;
    var $field_required;

    // CONSTRUCTOR //
    function __construct() {
        $this->code = 'ANQQ';
        $this->name = 'Questionary Questions';
        $this->system = 'Interface';
        $this->version = '0.1.4.1';


        $this->accesskey = 'ANQN';

        $this->tablename = 'CLX_AN_QUESTIONARY_QUESTS';
        $this->record_name_singular = 'questionary question';
        $this->record_name_plural = 'questionary questions';
        $this->primary_field = 'ID';
        $this->secondary_field = 'QUESTION';
        $this->seqnum_field = 'SEQNUM';
        $this->question_types = array('SING' => 'GEN_LBL_QUESTION_SINGLE',
            'MULT' => 'GEN_LBL_QUESTION_MULTIPLE',
            'OPEN' => 'GEN_LBL_QUESTION_OPEN',
            'END' => 'TOOL_ANQO_PROMPT_END'
        );
        $this->data_types = array('BOOL' => 'GEN_LBL_BOOLEAN',
            'INTN' => 'GEN_LBL_INTEGER',
            'DECN' => 'GEN_LBL_DECIMAL',
            'CHAR' => 'GEN_LBL_CHARACTER',
            'DATE' => 'GEN_LBL_DATE',
            'DATI' => 'GEN_LBL_DATETIME',
            'TIME' => 'GEN_LBL_TIME',
            'LOOK' => 'GEN_LBL_CLXFIELD'
        );
        $this->display_types = array('' => '',
            'CHCK' => 'GEN_LBL_CHECKBOXES',
            'RADI' => 'GEN_LBL_RADIOBOTTON',
            'LIST' => 'GEN_LBL_SELECT',
            'CHAR' => 'GEN_LBL_CHARACTER'
        );
        $this->field_required = array('0' => 'GEN_LBL_NO',
            '1' => 'GEN_LBL_YES'
        );
        $this->yesno = array('0' => 'GEN_LBL_NO',
            '1' => 'GEN_LBL_YES'
        );
    }

    function toolTableStructure() {
        $fields = array();
        $fields['ID'] = array('type' => 'int(11) unsigned', 'null' => 'NO', 'properties' => 'auto_increment', 'index' => 'PRI');
        $fields['QUESTIONARYCODE'] = array('type' => 'varchar(16)', 'null' => 'NO', 'properties' => '');
        $fields['SEQNUM'] = array('type' => 'tinyint(4) unsigned', 'null' => 'NO', 'properties' => '', 'default' => CLX_NUM_ZERO);
        $fields['QUESTION'] = array('type' => 'mediumtext', 'properties' => '', 'dict' => true);
        $fields['TYPE'] = array('type' => 'varchar(4)', 'properties' => '');
        $fields['DATA_TYPE'] = array('type' => 'varchar(4)', 'properties' => '');
        $fields['DISPLAY_TYPE'] = array('type' => 'varchar(4)', 'properties' => '');
        $fields['FIELD_REQUIRED'] = array('type' => 'tinyint(1) unsigned', 'null' => 'NO', 'properties' => '', 'default' => CLX_NUM_ZERO);
        $fields['FIELD_READONLY'] = array('type' => 'tinyint(1) unsigned', 'null' => 'NO', 'properties' => '', 'default' => CLX_NUM_ZERO);
        $fields['FIELD_HIDDEN'] = array('type' => 'tinyint(1) unsigned', 'null' => 'NO', 'properties' => '', 'default' => CLX_NUM_ZERO);
        $fields['FIELDNAME'] = array('type' => 'varchar(64)', 'properties' => '');
        $fields['DEFAULT_ANSWER_ID'] = array('type' => 'int(11) unsigned', 'properties' => '');
        $fields['UPDATE_PARENT'] = array('type' => 'int(1) unsigned', 'properties' => '', 'default' => '0');
        $fields['NEXT_QUESTION'] = array('type' => 'int(11) unsigned', 'properties' => '');
        $indexes = array();
        $ipf = 'CLX' . $this->code;
        $indexes[$ipf . '01'] = array('index' => 'PRI', 'fieldname' => 'ID');
        $indexes[$ipf . '02'] = array('index' => 'MUL', 'fieldname' => 'QUESTIONARYCODE,SEQNUM');
        return array('idcode' => $this->code, 'idname' => $this->name, 'name' => $this->tablename, 'type' => 'InnoDB',
            'fields' => $fields,
            'indexes' => $indexes);
    }

//-- Q U E S T I O N A R Y     Q U E S T I O N S
    function pack($record) {
        $o = $this->packRecord($record);
        $id = $record['ID'];
        $objs = new XMLObject($this->code . '_OBJECTS');
        $this->packsub($objs, 'ANQO', 'SELECT * FROM CLX_AN_QUESTIONARY_OPTIONS WHERE QUESTIONID=' . $id);
        $o->addObject($objs);
        return $o;
    }

    function unpack($parameters) {
        $parameters = array_values(array_merge(array('package' => null), $parameters));
        list($package) = $parameters;
        $record = $this->unpackRecord($package);
        $id = ClxCommon::$db->createRecord($this->tablename, $record, false);
        if ($package->hasSubObject($this->code . '_OBJECTS')) {
            $objs = $package->getObjectXml($this->code . '_OBJECTS');
            foreach ($objs->objects as $o) {
                $toolcode = $o->tag;
                $tool = &getTool($toolcode);
                $tool->unpack(array('package' => $o, 'id' => $id));
            }
        }
    }

    function makepanel($groupcode = null) {
        if ($this->getAccess()->canRead()) {
            $t = time();
            $to = $this->code;
            $here = 'tool.php?to=' . $to . '&t=' . $t;
            $form = new htmlForm(CLX_PROP_ACTION, 'tool.php');
            $form->setProp(CLX_PROP_ID, 'ANQQPanel');
            $form->addArgument(CLX_PROP_TO, $this->code);
            $form->addArgument('questionarycode', $groupcode);
            $form->addArgument('ac', 'NOT_DEFINED');
            $selectclause = '*';
            $fromclause = 'CLX_AN_QUESTIONARY_QUESTS';
            $whereclause = 'QUESTIONARYCODE=' . singleQuote($groupcode);
            $sortclause = 'SEQNUM ASC';
            $page = new xPaging();
            $page->setProp('rpp', CLX_NUM_TEN_THOUSAND);
            $page->setProp('printheaders', false);
            $page->pageInit($form->getProp(CLX_PROP_ID), $here, $selectclause, $fromclause, $whereclause, $sortclause, '');
            $table = new htmlTable(CLX_WIDTH_12);
            $table->setProps(array(CLX_PROP_CLASS => CLX_CLASS_LIST));
            $page->header($table);
            //$table->addCells(CLX_CLASS_HEADERS,array( dict('GEN_LBL_SEL'),dict('GEN_LBL_SEQNUM'), dict('GEN_LBL_QUESTION'), dict('GEN_LBL_ID'), dict('GEN_LBL_TYPE'), dict('GEN_LBL_DATA_TYPE')));
            $table->addCells(CLX_CLASS_HEADERS, array(dict('GEN_LBL_SEL'), dict('GEN_LBL_SEQNUM'), dict('GEN_LBL_QUESTION'), dict('GEN_LBL_TYPE'), dict('GEN_LBL_DATA_TYPE'), dict('GEN_LBL_DISPLAY_TYPE'), dict('GEN_LBL_REQUIRED'), dict('GEN_LBL_READONLY'), dict('GEN_LBL_HIDDEN'), dict('GEN_LBL_UPDATE_PARENT'), dict('GEN_LBL_FIELDNAME'), dict('GEN_LBL_NEXTQUESTION')));
            $i = CLX_INIT_ZERO;
            while ($record = $page->nextRecord()) {
                $recordid = $record['ID'];
                $select = new htmlCheckbox('selector_' . $recordid, false);
                $select->setProp('onclick', 'selectButtons(' . singleQuote($form->getProp(CLX_PROP_ID)) . ');');
                $table->addCell($select->source(), array(CLX_PROP_CLASS => CLX_CLASS_FOOTERS));
                $table->addCells(($i % CLX_NUM_TWO == CLX_NUM_ZERO) ? CLX_CLASS_DATA : CLX_CLASS_DATAALT, array('<b>' . $record['SEQNUM'] . '</b>', '<b>' . dict($record['QUESTION']). '</b>', dict($this->question_types[$record['TYPE']]), dict($this->data_types[$record['DATA_TYPE']]), dict($this->display_types[$record['DISPLAY_TYPE']]), dict($this->field_required[$record['FIELD_REQUIRED']]), dict($this->yesno[$record['FIELD_READONLY']]), dict($this->yesno[$record['FIELD_HIDDEN']]), dict($this->yesno[$record['UPDATE_PARENT']]), $record['FIELDNAME'], (!empty($record['NEXT_QUESTION'])) ? $record['NEXT_QUESTION'] : ''));
                if (($record['TYPE'] === 'MULT') or ( $record['TYPE'] === 'SING')) {
                    $qty = ClxCommon::$db->oneExistsRecords('CLX_AN_QUESTIONARY_OPTIONS', 'QUESTIONID=' . $record['ID']);
                    if ($qty > CLX_NUM_ZERO) {
                        $letter = CLX_NUM_65;
                        $options = ClxCommon::$db->getRecords('CLX_AN_QUESTIONARY_OPTIONS', 'QUESTIONID=' . $record['ID'] . ' ORDER BY SEQNUM');
                        while (($option = ClxCommon::$db->getNextRecord($options))) {
                            $table->addCell(CLX_CHAR_HTML_NBSP, array(CLX_PROP_CLASS => CLX_CLASS_FOOTERS));
                            if ($record['TYPE'] === 'SING') {
                                $txt = '<page t=an_questionary_questions></page><a href="tool.php?to=ANQO&ac=MODIFY&idlist=' . $option['ID'] . '">' . dict($option['OPTION_TEXT']) . ' (' . dict('TOOL_ANQQ_IFSELECTEDGOTO') . ' ' . $option['NEXT_QUESTION'] . ')</a>';
                            } else {
                                $txt = '<page t=an_questionary_questions></page><a href="tool.php?to=ANQO&ac=MODIFY&idlist=' . $option['ID'] . '">' . dict($option['OPTION_TEXT']) . '</a>';
                            }
                            $table->addCell(CLX_CHAR_HTML_NBSP, array(CLX_PROP_CLASS => ($i % CLX_NUM_TWO == CLX_NUM_ZERO) ? CLX_CLASS_DATA : CLX_CLASS_DATAALT));
                            $table->addCell(chr($letter) . ') ' . $txt, array(CLX_PROP_CLASS => ($i % CLX_NUM_TWO == CLX_NUM_ZERO) ? CLX_CLASS_DATA : CLX_CLASS_DATAALT, CLX_PROP_COLSPAN => CLX_WIDTH_12 - CLX_NUM_ONE));
                            $letter++;
                        }
                    }
                }
                $i++;
            }

            $buttons = htmlButton::getButton('submit', dict('GEN_LBL_DISPLAY'), array('action' => 'DISPLAY', 'form' => $form->getProp(CLX_PROP_ID), CLX_PROP_ID => 'MULT'))->source();
            if ($this->getAccess()->canModify()) {
                $buttons .=htmlButton::getButton('submit', dict('GEN_LBL_MODIFY'), array('action' => 'MODIFY', 'form' => $form->getProp(CLX_PROP_ID), CLX_PROP_ID => 'MULT'))->source();
            }
            if ($this->getAccess()->canDelete()) {
                $buttons .=htmlButton::getButton('submit', dict('GEN_LBL_DELETE'), array('action' => 'DELETE', 'form' => $form->getProp(CLX_PROP_ID), CLX_PROP_ID => 'MULT'))->source();
            }
            if ($this->getAccess()->canDisplay()) {
                $buttons .=htmlButton::getButton('submit', dict('GEN_LBL_CREATE_SOMETHING', dict('GEN_LBL_QUESTION')), array('action' => 'CREATE', 'form' => $form->getProp(CLX_PROP_ID), CLX_PROP_ID => 'ON'))->source();
            }
            if (($page->getProp('qty') > CLX_NUM_ONE) && $this->getAccess()->canModify()) {
                $buttons .=htmlButton::getButton('submit', dict('GEN_LBL_REORDER'), array('action' => 'REORDER', 'form' => $form->getProp(CLX_PROP_ID), CLX_PROP_ID => 'ON'))->source();
            }
            $form->addContents($page->footers($table, $buttons));
            return buildPanelInterface(dict('TOOL_ANQQ_TITLE_LIST'), 'TABLES', $form->source(), CLX_NUM_MINUS_1);
        } else {
            return CLX_CHAR_EMPTY_STRING;
        }
    }

    function editForm($ac, $idlist, $record, $msg) {
        $modify = (UserRequest::$request['ac'] === 'MODIFY');
        $title = ($modify) ? dict('TOOL_ANQQ_TITLE_MODIFY') : dict('TOOL_ANQQ_TITLE_CREATE');
        $form = new htmlForm(CLX_PROP_ACTION, 'tool.php');
        $form->addArgument(CLX_PROP_TO, $this->code);
        $form->addArgument('ac', $ac);
        $form->addArgument('confirm', '1');
        $form->addArgument('ANQQ_id', $record['ANQQ_id']);
        $form->addArgument('idlist', toString($idlist));
        $callingpage = peekHistory();
        $questionarycode = $callingpage['questionarycode'];
        $form->addArgument('ANQQ_questionarycode', $questionarycode);
        $table = new htmlTable(CLX_WIDTH_TWO);
        $table->setProps(array(CLX_PROP_CLASS => CLX_CLASS_FORM, CLX_PROP_WIDTH => CLX_PROP_100PCT));
        $table->addCell($title, array(CLX_PROP_CLASS => CLX_CLASS_TITLE, CLX_PROP_COLSPAN => CLX_WIDTH_TWO));
        $DICT = &getTool('DICT');
        $DICT->editEntry($msg, dict('GEN_LBL_QUESTION') . ':', 'ANQQ_question', $record, $table, $form);
        //$table->addText($msg, dict('GEN_LBL_QUESTION') . ':', 'ANQQ_question', $record['ANQQ_question'], 4, 64);
        $pop = new HTMLSelect('ANQQ_type');
        foreach ($this->question_types as $key => $value) {
            $pop->addItem(dict($value), $key, ($record['ANQQ_type'] == $key));
        }
        $table->addData(dict('TOOL_ANQQ_TYPE') . ':', $pop->source());
        $pop = new HTMLSelect('ANQQ_data_type');
        foreach ($this->data_types as $key => $value) {
            $pop->addItem(dict($value), $key, ($record['ANQQ_data_type'] == $key));
        }
        $table->addData(dict('TOOL_ANQQ_DATA_TYPE') . ':', $pop->source());
        $pop = new HTMLSelect('ANQQ_display_type');
        foreach ($this->display_types as $key => $value) {
            $pop->addItem(dict($value), $key, ($record['ANQQ_display_type'] == $key));
        }
        $table->addData(dict('GEN_LBL_DISPLAY_TYPE') . ':', $pop->source());
        $pop = new HTMLSelect('ANQQ_field_required');
        foreach ($this->field_required as $key => $value) {
            $pop->addItem(dict($value), $key, ($record['ANQQ_field_required'] == $key));
        }
        $table->addData(dict('GEN_LBL_REQUIRED') . ':', $pop->source());
        $pop = new HTMLSelect('ANQQ_field_readonly');
        foreach ($this->yesno as $key => $value) {
            $pop->addItem(dict($value), $key, ($record['ANQQ_field_readonly'] == $key));
        }
        $table->addData(dict('GEN_LBL_READONLY') . ':', $pop->source());
        //Parent record on work table
        if (!empty($msg['ANQQ_update_parent']))
            $table->addCell($msg['ANQQ_update_parent'], array(CLX_PROP_CLASS => CLX_CLASS_ALERT, CLX_PROP_COLSPAN => CLX_WIDTH_TWO));
        $pop = new HTMLSelect('ANQQ_update_parent');
        foreach ($this->field_required as $key => $value) {
            $pop->addItem(dict($value), $key, ($record['ANQQ_update_parent'] == $key));
        }
        $table->addData(dict('GEN_LBL_UPDATE_PARENT') . ':', $pop->source());
        /*
          No meaning for now but is set. to use it uncomment this an comment	$form->addArgument('ANQQ_field_hidden', 0); */
        $pop = new HTMLSelect('ANQQ_field_hidden');
        foreach ($this->yesno as $key => $value) {
            $pop->addItem(dict($value), $key, ($record['ANQQ_field_hidden'] == $key));
        }
        $table->addData(dict('GEN_LBL_HIDDEN') . ':', $pop->source());
        $form->addArgument('ANQQ_field_hidden', CLX_NUM_ZERO);
        $DBFL = &getTool('DBFL');
        $DBFL->placeChooseField($msg, dict('GEN_LBL_FIELDNAME') . ':<br>' . dict('TOOL_ANQQ_LOOKUPORLISTONLY'), 'ANQQ_fieldname', $record['ANQQ_fieldname'], $table);
        $questions = ClxCommon::$db->getRecords('CLX_AN_QUESTIONARY_QUESTS', 'QUESTIONARYCODE=' . singleQuote($questionarycode) . ' ORDER BY SEQNUM');
        $pop = new HTMLSelect('ANQQ_next_question');
        $pop->addItem('', CLX_NUM_ZERO, false);
        while (($questionrec = ClxCommon::$db->getNextRecord($questions))) {
            $pop->addItem($questionrec['SEQNUM'] . '. ' . substr(dict($questionrec['QUESTION']), CLX_NUM_ZERO, CLX_NUM_50) . '...', $questionrec['SEQNUM'], ($record['ANQQ_next_question'] == $questionrec['SEQNUM']));
        }
        if (!empty($msg['ANQQ_next_question']))
            $table->addCell($msg['ANQQ_next_question'], array(CLX_PROP_CLASS => CLX_CLASS_ALERT, CLX_PROP_COLSPAN => CLX_WIDTH_TWO));
        $table->addData(dict('GEN_LBL_NEXTQUESTION'), $pop->source());
        $form->addContents($table->source());
        $table = new htmlTable(CLX_WIDTH_TWO);
        $table->setProps(array(CLX_PROP_CLASS => CLX_CLASS_FORM, CLX_PROP_WIDTH => CLX_PROP_100PCT));
        $buttons = '';
        if (!empty($modify)) {
            $buttons .= htmlButton::getButton('submit', dict('GEN_LBL_SAVE'), array('action' => $ac, 'form' => $form->getProp(CLX_PROP_ID)))->source();
        } else {
            $buttons .=htmlButton::getButton('submit', dict('GEN_LBL_CREATE'), array('action' => 'CREATE', 'form' => $form->getProp(CLX_PROP_ID)))->source();
        }
        $c = new htmlButtonCancel();
        $buttons .= $c->source();
        $table->addCell($buttons, array(CLX_PROP_CLASS => CLX_CLASS_FOOTERS, CLX_PROP_COLSPAN => CLX_WIDTH_TWO));
        $form->addContents($table->source());
        return $form->source();
    }

    function checkForm(&$msg) {
        $msg = array();
        xValidate::required($msg, 'ANQQ_question_DICT', UserRequest::$post['ANQQ_question_DICT']);
        xValidate::required($msg, 'ANQQ_data_type', UserRequest::$post['ANQQ_data_type']);
        if (UserRequest::$post['ANQQ_data_type'] === 'LOOK') {
            xValidate::required($msg, 'ANQQ_fieldname', UserRequest::$post['ANQQ_fieldname']);
        }
        if (!empty(UserRequest::$post['ANQQ_update_parent'])) {
            $count = ClxCommon::$db->getOne('SELECT COUNT(DBTF.ID) FROM CLX_DB_TABLEFIELDS DBTF, CLX_AN_QUESTIONARIES ANQN WHERE DBTF.TABLECODE = ANQN.TABLECODE AND DBTF.FIELDNAME = ' . singleQuote(UserRequest::$post['ANQQ_fieldname']) . ' AND 								   ANQN.IDCODE = ' . singleQuote(UserRequest::$post['ANQQ_questionarycode']));
            if ($count == CLX_NUM_ZERO)
                $msg['ANQQ_update_parent'] = dict('GEN_ERR_TABLEFIELDDONTEXIST');
        }
        if (UserRequest::$post['ANQQ_type'] === 'OPEN' && empty(UserRequest::$post['ANQQ_next_question'])) {
            // Es necesaria ajustar
        }
        if (UserRequest::$post['ANQQ_type'] <> 'OPEN' && !empty(UserRequest::$post['ANQQ_next_question'])) {
            $msg['ANQQ_next_question'] = dict('GEN_ERR_ONLYOPENQUESTIONS');
        }
        return count($msg) == CLX_NUM_ZERO;
    }

    function confirmDeleteFill(&$table, $record) {
        $table->addData(dict('GEN_LBL_QUESTION') . ':', xFormat::text($record['ANQQ_question']));
    }

    function viewFormFill($record) {
        addHistory('questionid', $record['ANQQ_id']);
        $contents = '';
        $table = new htmlTable(CLX_WIDTH_TWO);
        $table->setProps(array(CLX_PROP_CLASS => CLX_CLASS_FORM, CLX_PROP_WIDTH => CLX_PROP_100PCT));
        $table->addCell(dict('TOOL_ANQQ_TITLE_VIEW'), array(CLX_PROP_CLASS => CLX_CLASS_TITLE, CLX_PROP_COLSPAN => CLX_WIDTH_TWO));
        $table->addData(dict('GEN_LBL_QUESTIONARY_CODE') . ':', $record['ANQQ_questionarycode']);
        $table->addData(dict('GEN_LBL_QUESTION') . ':', dict($record['ANQQ_question']));
        $table->addData(dict('GEN_LBL_TYPE') . ':', dict($this->question_types[$record['ANQQ_type']]));
        $table->addData(dict('GEN_LBL_DISPLAY_TYPE') . ':', dict($this->display_types[$record['ANQQ_display_type']]));
        $table->addData(dict('GEN_LBL_RESPONSE_TYPE') . ':', dict($this->data_types[$record['ANQQ_data_type']]));
        $table->addData(dict('GEN_LBL_REQUIRED') . ':', dict($this->field_required[$record['ANQQ_field_required']]));
        $table->addData(dict('GEN_LBL_READONLY') . ':', dict($this->yesno[$record['ANQQ_field_readonly']]));
        $table->addData(dict('GEN_LBL_HIDDEN') . ':', dict($this->yesno[$record['ANQQ_field_hidden']]));
        $table->addData(dict('GEN_LBL_UPDATE_PARENT') . ':', dict($this->yesno[$record['ANQQ_update_parent']]));
        $table->addData(dict('GEN_LBL_FIELDNAME') . ':', $record['ANQQ_fieldname']);
        $table->addData(dict('GEN_LBL_NEXTQUESTION') . ':', $record['ANQQ_next_question']);
        $contents .= $table->source();
        if (($record['ANQQ_type'] === 'MULT') or ( $record['ANQQ_type'] === 'SING')) {
            $ANQO = &getTool('ANQO');
            $contents .= $ANQO->makepanel($record['ANQQ_id']);
        }
        return $contents;
    }

    function reorderRecords() {
        $records = ClxCommon::$db->getRecords($this->tablename, 'QUESTIONARYCODE=' . singleQuote(UserRequest::$post['questionarycode']) . ' ORDER BY SEQNUM');
        $list = array();
        while ($record = ClxCommon::$db->getNextRecord($records, ASSOCIATIVE)) {
            $t = str_replace(CLX_CHAR_NL, '', $record['QUESTION']);
            $t = str_replace(chr(CLX_NUM_13), '', $t);
            $list[$record['ID']] = dict($t);
        }
        return $list;
    }

    function reorderAction() {
        //Update Target
        $neworder = explode(',', UserRequest::$post['neworder']); //[neworder] => 16,15,17
        $newseq = array();
        $results = ClxCommon::$db->getRecords($this->tablename, 'ID IN (' . UserRequest::$post['neworder'] . ')');
        while ($result = ClxCommon::$db->getNextRecord($results, ASSOCIATIVE)) {
            $newseq[$result['SEQNUM']] = array_search($result['ID'], $neworder) + CLX_NUM_ONE;
        }
        //Reset Target Question on options
        $results = ClxCommon::$db->getRecords('CLX_AN_QUESTIONARY_OPTIONS', 'QUESTIONID IN (' . UserRequest::$post['neworder'] . ') AND NEXT_QUESTION<>' . singleQuote(CLX_CHAR_EMPTY_STRING));
        while ($result = ClxCommon::$db->getNextRecord($results, ASSOCIATIVE)) {
            ClxCommon::$db->updateRecord('CLX_AN_QUESTIONARY_OPTIONS', array('NEXT_QUESTION' => $newseq[$result['NEXT_QUESTION']]), 'ID = ' . singleQuote($result['ID']));
        }
        parent::reorderAction();
        //Adjusting Option Fields Tablefields
        $results = ClxCommon::$db->getRecords($this->tablename, 'ID IN (' . UserRequest::$post['neworder'] . ')');
        while ($result = ClxCommon::$db->getNextRecord($results, ASSOCIATIVE)) {
            $this->createQuestTableField($result);
        }
        $callingpage = peekHistory();
        $questionarycode = $callingpage['questionarycode'];
        $ANQN = &getTool('ANQN');
        $ANQN->versionIncrease('IDCODE', $questionarycode);
    }

    function loadToolRecord($recordid) {
        $values = ClxCommon::$db->getRecord($this->tablename, 'ID=' . $recordid);
        $record = array();
        $record['ANQQ_id'] = $recordid;
        $record['ANQQ_questionarycode'] = $values['QUESTIONARYCODE'];
        $DICT = &getTool('DICT');
        $DICT->loadEntry('ANQQ_question', $values['QUESTION'], $record);
        $record['ANQQ_type'] = $values['TYPE'];
        $record['ANQQ_data_type'] = $values['DATA_TYPE'];
        $record['ANQQ_display_type'] = $values['DISPLAY_TYPE'];
        $record['ANQQ_field_required'] = $values['FIELD_REQUIRED'];
        $record['ANQQ_field_readonly'] = $values['FIELD_READONLY'];
        $record['ANQQ_field_hidden'] = $values['FIELD_HIDDEN'];
        $record['ANQQ_fieldname'] = $values['FIELDNAME'];
        $record['ANQQ_seqnum'] = $values['SEQNUM'];
        $record['ANQQ_default_answer_id'] = $values['DEFAULT_ANSWER_ID'];
        $record['ANQQ_update_parent'] = $values['UPDATE_PARENT'];
        $record['ANQQ_next_question'] = $values['NEXT_QUESTION'];
        return $record;
    }

    function saveRecord() {
        $record = array();
        $DICT = &getTool('DICT');
        $record['QUESTIONARYCODE'] = UserRequest::$post['ANQQ_questionarycode'];
        $record['QUESTION'] = $DICT->saveEntry('ANQQ_question', UserRequest::$post);
        $record['TYPE'] = UserRequest::$post['ANQQ_type'];
        $record['DATA_TYPE'] = ($record['TYPE'] === 'END') ? 'CHAR' : UserRequest::$post['ANQQ_data_type'];
        $record['FIELD_REQUIRED'] = UserRequest::$post['ANQQ_field_required'];
        $record['FIELD_READONLY'] = UserRequest::$post['ANQQ_field_readonly'];
        $record['FIELD_HIDDEN'] = UserRequest::$post['ANQQ_field_hidden'];
        $record['DISPLAY_TYPE'] = UserRequest::$post['ANQQ_display_type'];
        $record['UPDATE_PARENT'] = UserRequest::$post['ANQQ_update_parent'];
        $record['NEXT_QUESTION'] = UserRequest::$post['ANQQ_next_question'];
        $record['FIELDNAME'] = UserRequest::$post['ANQQ_fieldname'];
        if (!empty($record['FIELDNAME'])) {
            $record['TYPE'] = 'SING';
            $record['DATA_TYPE'] = 'LOOK';
        }
        ClxCommon::$db->updateRecord($this->tablename, $record, 'ID=' . UserRequest::$post['ANQQ_id']);
        $this->createQuestTableField(ClxCommon::$db->getRecord($this->tablename, 'ID=' . singleQuote(UserRequest::$post['ANQQ_id'])));
        $ANQN = &getTool('ANQN');
        $ANQN->versionIncrease('IDCODE', $record['QUESTIONARYCODE']);
    }

    function createRecordVoid_() {
        ClxFreeReturn($this->createRecord_());
    }

    function createRecord_() {
        $record = array();
        $record['QUESTIONARYCODE'] = UserRequest::$post['ANQQ_questionarycode'];
        $DICT = &getTool('DICT');
        $record['QUESTION'] = $DICT->createEntry('ANQQ_question', UserRequest::$post);
        $record['TYPE'] = UserRequest::$post['ANQQ_type'];
        $record['DATA_TYPE'] = ($record['TYPE'] === 'END') ? 'CHAR' : UserRequest::$post['ANQQ_data_type'];
        $record['DISPLAY_TYPE'] = UserRequest::$post['ANQQ_display_type'];
        $record['FIELD_REQUIRED'] = UserRequest::$post['ANQQ_field_required'];
        $record['FIELD_READONLY'] = UserRequest::$post['ANQQ_field_readonly'];
        $record['FIELD_HIDDEN'] = UserRequest::$post['ANQQ_field_hidden'];
        $record['FIELDNAME'] = UserRequest::$post['ANQQ_fieldname'];
        $record['UPDATE_PARENT'] = UserRequest::$post['ANQQ_update_parent'];
        $record['NEXT_QUESTION'] = UserRequest::$post['ANQQ_next_question'];
        if (!empty($record['FIELDNAME'])) {
            $record['TYPE'] = 'SING';
            $record['DATA_TYPE'] = 'LOOK';
        }
        $record['SEQNUM'] = ClxCommon::$db->getField($this->tablename, 'SEQNUM', 'QUESTIONARYCODE=' . singleQuote($record['QUESTIONARYCODE']) . ' ORDER BY SEQNUM DESC') + CLX_NUM_ONE;
        $recordid = ClxCommon::$db->createRecord($this->tablename, $record, false);
        $ANQN = &getTool('ANQN');
        $ANQN->versionIncrease('IDCODE', $record['QUESTIONARYCODE']);
        //Creating Tablefields
        $this->createQuestTableField(ClxCommon::$db->getRecord($this->tablename, 'ID=' . singleQuote($recordid)));
        return $recordid;
    }

    function createQuestTableField($record) {
        $d = xformat::getNow();
        $DBFL = &getTool('DBFL');
        $ANQN = &getTool('ANQN');
        $DBTF = &getTool('DBTF');
        $DBTB = &getTool('DBTB');
        $ANQN = &getTool('ANQN');
        $fieldname = '';
        $options = $ANQN->getOptions($record['ID']);
        $options[] = array('SEGNUM' => '');
        //Delete All Fields to Recreate Table
        foreach ($options as $key => $val) {
            if (!empty($record['FIELDNAME'])) {
                $fieldrecord = ClxCommon::$db->getRecord($DBFL->tablename, 'FIELDNAME = ' . singleQuote($record['FIELDNAME']));
            } else {
                $fieldname = 'Q' . $record['QUESTIONARYCODE'] . '_' . $record['SEQNUM'];
            }
            if (array_key_exists('SEQNUM', $val) && strlen($val['SEQNUM'])) {
                $label = $val['OPTION_TEXT'];
                $fieldname .= '_' . $val['SEQNUM'];
            }
            if (strlen($fieldname) > CLX_NUM_ZERO) {
                $fieldid = ClxCommon::$db->getField($DBFL->tablename, 'ID', 'FIELDNAME = ' . singleQuote(uppercase($fieldname)));
                if (!empty($fieldid)) {
                    $DBFL->deleteRecordSk($fieldid);
                }
            }
        }
        foreach ($options as $key => $val) {
            if ($record['TYPE'] <> 'MULT' && (array_key_exists('SEQNUM', $val) && $val['SEQNUM'] <> '')) {
                continue;
            }
            if (!empty($record['FIELDNAME'])) {
                $fieldrecord = ClxCommon::$db->getRecord($DBFL->tablename, 'FIELDNAME = ' . singleQuote($record['FIELDNAME']));
            } else {
                $fieldname = 'Q' . $record['QUESTIONARYCODE'] . '_' . $record['SEQNUM'];
                $label = $record['QUESTION'];
                $fieldtype = $record['DATA_TYPE'];
                switch ($record['DATA_TYPE']) {
                    case 'DATE':
                        $fieldsize = '0';
                        $field_extra1 = '';
                        $field_extra2 = '';
                        $fieldsize = '0';
                        $field_format = $this->format=$format = $_SESSION['prefs']['DATE_FORMAT'];
                        break;
                    case 'DATI':
                        $field_extra1 = '';
                        $field_extra2 = '';
                        $fieldsize = '0';
                        $field_format = $this->format=$format = $_SESSION['prefs']['DATE_FORMAT'].' H:i:s';
                        break;
                    case 'TIME':
                        $field_extra1 = '';
                        $field_extra2 = '';
                        $fieldsize = '0';
                        $field_format = 'H:i:s';
                        break;
                    case 'DECN':
                        $field_extra1 = '';
                        $field_extra2 = '';
                        $fieldsize = '0';
                        $field_format = '25,2';
                        break;
                    case 'INTN':
                        $fieldsize = '3';
                        $field_extra1 = '0';
                        $field_extra2 = '0';
                        $field_format = '11';
                        break;
                    case 'BOOL':
                        $fieldtype = 'INTN';
                        $fieldsize = '3';
                        $field_extra1 = '0';
                        $field_extra2 = '0';
                        $field_format = '2';
                        break;
                    default:
                        $fieldtype = 'CHAR';
                        $fieldsize = '255';
                        $field_extra1 = '';
                        $field_extra2 = '';
                        $field_format = '';
                        break;
                }
                //Setting option Fields
                if (array_key_exists('SEQNUM', $val) && strlen($val['SEQNUM'])) {
                    $label = $val['OPTION_TEXT'];
                    $fieldname .= '_' . $val['SEQNUM'];
                }
                $fieldrecord = array('FIELDNAME' => uppercase($fieldname),
                    'VERSION' => '1.0.0',
                    'VERSION_COUNTER' => '0',
                    'CDATE' => $d, 'MDATE' => $d, 'USERNAME' => $_SESSION['user_record']['username'], 'DESCRIPTION' => 'Questionary Field',
                    'FIELDTYPE' => $fieldtype,
                    'FIELDLISTLABEL' => $label,
                    'FIELDFORMLABEL' => $label,
                    'FIELDPREFIX' => 'd41d8cd98f00b204e9800998ecf8427e',
                    'FIELDSUFIX' => 'd41d8cd98f00b204e9800998ecf8427e',
                    'FIELD_SIZE' => $fieldsize,
                    'FIELD_REQUIRED' => '0',
                    'FIELD_ALIGN' => '0',
                    'FIELD_DISPLAYSIZE' => '38',
                    'FIELD_FORMAT' => $field_format,
                    'FIELD_EXTRA1' => $field_extra1,
                    'FIELD_EXTRA2' => $field_extra2);
            }
            if (!$DBFL->fieldExists($fieldrecord['FIELDNAME'])) {
                ClxCommon::$db->createRecord($DBFL->tablename, $fieldrecord, false);
            }
            $fieldrecord['TABLECODE'] = ClxCommon::$db->getfield($ANQN->tablename, 'RESULT_TABLECODE', 'IDCODE =' . singleQuote($record['QUESTIONARYCODE']));
            if (!$DBTF->fieldExists($fieldrecord['TABLECODE'], $fieldrecord['FIELDNAME'])) {
                $rec = array();
                $rec['TABLECODE'] = $fieldrecord['TABLECODE'];
                $rec['FIELDNAME'] = $fieldrecord['FIELDNAME'];
                $rec['SEQNUM'] = ClxCommon::$db->getField($DBTF->tablename, 'SEQNUM', 'TABLECODE=' . singleQuote($fieldrecord['TABLECODE']) . ' ORDER BY SEQNUM DESC') + CLX_NUM_ONE;
                ClxCommon::$db->createRecord($DBTF->tablename, $rec, false);
                $DBTB->versionIncrease('IDCODE', $rec['TABLECODE']);
                $DBTF->createTableField($rec['TABLECODE'], $fieldrecord);
            }
        }
    }

    function deleteRecordSk($id) {
        $DICT = &getTool('DICT');
        $questionrec = ClxCommon::$db->getRecord($this->tablename, 'ID=' . singleQuote($id));
        $DBFL = &getTool('DBFL');
        //Deleting Field and Options Field first
        $fieldname = uppercase($questionrec['QUESTIONARYCODE'] . '_' . $questionrec['SEQNUM']);
        $results = ClxCommon::$db->getRecords('CLX_AN_QUESTIONARY_OPTIONS', 'QUESTIONID=' . singleQuote($id));
        while ($result = ClxCommon::$db->getNextRecord($results, ASSOCIATIVE)) {
            $optfieldname = $fieldname . '_' . $result['SEQNUM'];
            if ($DBFL->fieldExists($optfieldname)) {
                $DBFL->deleteRecordSk(ClxCommon::$db->getField($DBFL->tablename, 'ID', 'FIELDNAME=' . singleQuote($optfieldname)));
            }
        }
        if ($DBFL->fieldExists($fieldname)) {
            $DBFL->deleteRecordSk(ClxCommon::$db->getField($DBFL->tablename, 'ID', 'FIELDNAME=' . singleQuote($fieldname)));
        }
        $DICT->deleteEntry($questionrec['QUESTION']);
        ClxCommon::$db->deleteRecords('CLX_AN_QUESTIONARY_OPTIONS', 'QUESTIONID=' . $id);
        ClxCommon::$db->deleteRecords($this->tablename, 'ID=' . $id);
        $ANQN = &getTool('ANQN');
        $ANQN->versionIncrease('IDCODE', $questionrec['QUESTIONARYCODE']);
    }

    function defaultRecord() {
        $record = array();
        $record['ANQQ_id'] = '0';
        $record['ANQQ_questionarycode'] = '';
        $record['ANQQ_question'] = '';
        $record['ANQQ_type'] = 'OPEN';
        $record['ANQQ_data_type'] = 'TEXT';
        $record['ANQQ_display_type'] = '';
        $record['ANQQ_field_required'] = '0';
        $record['ANQQ_field_readonly'] = '0';
        $record['ANQQ_field_hidden'] = '0';
        $record['ANQQ_fieldname'] = '';
        $record['ANQQ_seqnum'] = '0';
        $record['ANQQ_default_answer_id'] = '0';
        $record['ANQQ_update_parent'] = '0';
        $record['ANQQ_next_question'] = '';
        return $record;
    }

    function versionIncrease($fieldname, $fieldvalue) {
        $questionarycode = ClxCommon::$db->getField($this->tablename, 'QUESTIONARYCODE', $fieldname . '=' . singleQuote($fieldvalue));
        $ANQN = &getTool('ANQN');
        $ANQN->versionIncrease('IDCODE', $questionarycode);
    }

}
