<?php

/**
 * Reports Class
 * For more information contact  support@alenet.com
 *
 * @author       $Author: jclerma $
 * @copyright    Alenet, Inc.
 * @since        $Date: 2017-11-03 15:09:40 -0400 (Fri, 03 Nov 2017) $
 * @version    	$Rev: 9615 $
 * @package		Clientix.Tool.Analytics
 * @abstract		Report Columns
 */
class an_report_columns extends clx_tool {

    /**
     * Constructor
     */
    function __construct() {
        $this->code = 'ANRC';
        $this->name = 'Report Columns';
        $this->system = 'Analytics';
        $this->version = '0.1.2.0';


        $this->accesskey = 'ANRP';

        $this->tablename = 'CLX_AN_REPORT_COLUMNS';
        $this->record_name_singular = 'report column';
        $this->record_name_plural = 'report columns';
        $this->primary_field = 'ID';
        $this->secondary_field = 'IDNAME';
        $this->seqnum_field = 'SEQNUM';
    }

    /**
     * Describes the tool table structure.
     * @return array ('type' => 'varchar(255)', 'properties'=>'auto increment',
     * 				'index'=>'', default = '', 'dict'=>true)
     * 				 dict true or false means that field holds dictionary entry
     */
    function toolTableStructure() {
        $fields = array();
        $fields['ID'] = array('type' => 'int(11) unsigned', 'null' => 'NO', 'properties' => 'auto_increment', 'index' => 'PRI');
        $fields['REPORTCODE'] = array('type' => 'varchar(16)', 'null' => 'NO', 'properties' => '');
        $fields['TABLECODE'] = array('type' => 'varchar(4)', 'null' => 'NO', 'properties' => '');
        $fields['FIELDNAME'] = array('type' => 'varchar(64)', 'null' => 'NO', 'properties' => '');
        $fields['SEQNUM'] = array('type' => 'int(4) unsigned', 'null' => 'NO', 'properties' => '', 'default' => CLX_NUM_ZERO);
        $fields['SORTING'] = array('type' => 'varchar(4)', 'null' => 'NO', 'properties' => '', 'default' => '' . singleQuote('NON') . '', 'default_raw' => 'NONE');
        $fields['FORMAT'] = array('type' => 'tinyint(1) unsigned', 'null' => 'NO', 'properties' => '', 'default' => CLX_NUM_ZERO);
        $fields['REPQRYCODE'] = array('type' => 'varchar(16)', 'null' => 'YES', 'properties' => '');
        $fields['ACTIVE'] = array('type' => 'tinyint(1) unsigned', 'null' => 'NO', 'default' => '1', 'properties' => '');

        $fields['FIELDLISTLABEL'] = array('type' => 'varchar(64)', 'properties' => '', 'dict' => true);
        $indexes = array();
        $ipf = 'CLX' . $this->code;
        $indexes[$ipf . '01'] = array('index' => 'PRI', 'fieldname' => 'ID');
        $indexes[$ipf . '02'] = array('index' => 'MUL', 'fieldname' => 'REPORTCODE,SEQNUM');
        return array('idcode' => $this->code, 'idname' => $this->name, 'name' => $this->tablename, 'type' => 'InnoDB',
            'fields' => $fields,
            'indexes' => $indexes);
    }

//-- R E P O R T    C O L U M N S
    /**
     * Pack tool record on xml Format
     * @param array record
     * @return obj
     */
    function pack($record) {
        $o = $this->packRecord($record);
        $code = $record['ID'];
        $objs = new XMLObject($this->code . '_OBJECTS');

        //Packing Dictionary
        $this->packsub($objs, 'DICT', 'SELECT  DICT.*  FROM  CLX_DICTIONARY DICT,  CLX_AN_REPORT_COLUMNS X WHERE  DICT.ENTRY=X.FIELDLISTLABEL AND X.ID=' . singleQuote($code));
        $o->addObject($objs);
        return $o;
    }

    /**
     * Loads package into system
     * @param string $package
     */
    function unpack($parameters) {
        $parameters = array_values(array_merge(array('package' => null, 'id' => CLX_NUM_ZERO), $parameters));
        list($package, $reportid) = $parameters;
        $record = $this->unpackRecord($package);
        ClxFreeReturn(ClxCommon::$db->createRecord($this->tablename, $record, false));
    }

    /**
     * Builds Panel listing on tool
     *
     * @param string $groupcode
     * @return string
     */
    function makepanel($reportcode = null) {
        if ($this->getAccess()->canRead()) {

            $DICT = &getTool('DICT');
            $t = time();
            $to = $this->code;
            $here = 'tool.php?to=' . $to . '&t=' . $t;
            $report_type = intval(ClxCommon::$db->getField('CLX_AN_REPORTS', 'REPORT_TYPE', 'IDCODE = ' . singleQuote($reportcode)));
            $form = new htmlForm(CLX_PROP_ACTION, 'tool.php');
            $form->setProp(CLX_PROP_ID, 'ANRCPanel');
            $form->addArgument(CLX_PROP_TO, $this->code);
            $form->addArgument('reportcode', $reportcode);
            $form->addArgument('ac', 'NOT_DEFINED');
            $selectclause = 'G.*, T.IDNAME TABLENAME, T.MODULECODE MODULECODE, Q.IDNAME QRYNAME ';
            $fromclause = 'CLX_DB_TABLES T, CLX_AN_REPORT_COLUMNS G  LEFT JOIN CLX_AN_REPORT_QUERIES Q ON Q.IDCODE = G.REPQRYCODE ';
            $whereclause = 'G.REPORTCODE=' . singleQuote($reportcode) . ' AND G.TABLECODE=T.IDCODE';
            $sortclause = 'G.SEQNUM';
            $page = new xPaging();
            $page->setProp('rpp', CLX_NUM_TEN_THOUSAND);
            $page->setProp('printheaders', false);
            $page->pageInit($form->getProp(CLX_PROP_ID), $here, $selectclause, $fromclause, $whereclause, $sortclause, '');
            $yesno = array();
            foreach ($this->yesno as $key => $val) {
                $yesno[$key] = dict($val);
            }

            $i = CLX_INIT_ZERO;
            if ($report_type === CLX_NUM_ONE) {//type template
                $table = new htmlTable(CLX_WIDTH_7);
                $table->setProps(array(CLX_PROP_CLASS => CLX_CLASS_LIST, CLX_PROP_WIDTH => CLX_PROP_100PCT));
                $page->header($table);

                /*
                 * GEN_LBL_FIELDNAME_WT
                 * Template Field Name
                 * Nombre Campo en Plantilla
                 */
                $table->addCells(CLX_CLASS_HEADERS, array(dict('GEN_LBL_SEL'), dict('GEN_LBL_REPQRYCODE'), dict('GEN_LBL_FIELDNAME'), dict('GEN_LBL_FIELDNAME_WT'), dict('GEN_LBL_DESCRIPTION'), dict('GEN_LBL_FORMAT'), dict('GEN_LBL_ACTIVE')));
                while ($record = $page->nextRecord()) {
                    $select = new htmlCheckbox('selector_' . $record['ID'], false);
                    $select->setProp('onclick', 'selectButtons(' . singleQuote($form->getProp(CLX_PROP_ID)) . ');');
                    $table->addCell($select->source(), array(CLX_PROP_CLASS => CLX_CLASS_FOOTERS, CLX_PROP_WIDTH => '40', CLX_PROP_ALIGN => 'center'));
                    $entry = ClxCommon::$db->getField('CLX_DB_FIELDS', 'FIELDFORMLABEL', 'FIELDNAME  = ' . singleQuote($record['FIELDNAME']));
                    $table->addCells(($i % CLX_NUM_TWO == CLX_NUM_ZERO) ? CLX_CLASS_DATA : CLX_CLASS_DATAALT, array($record['REPQRYCODE'], $record['TABLECODE'] . '.' . $record['FIELDNAME'], $record['REPQRYCODE'] . '.' . $record['TABLECODE'] . '_' . $record['FIELDNAME'], dict($entry), dict($this->format_types[$record['FORMAT']]), $yesno[$record['ACTIVE']]));
                    $i++;
                }
            } else {


                $sort = array();
                foreach ($this->sorting_types as $key => $val) {
                    $sort[$key] = dict($val);
                }
                $table = new htmlTable(CLX_WIDTH_7);
                $table->setProps(array(CLX_PROP_CLASS => CLX_CLASS_LIST, CLX_PROP_WIDTH => CLX_PROP_100PCT));
                $page->header($table);

                $table->addCells(CLX_CLASS_HEADERS, array(dict('GEN_LBL_SEL'), dict('GEN_LBL_FIELDNAME'), dict('GEN_LBL_COLLABEL_DEF'), dict('GEN_LBL_COLLABEL_EV'), dict('GEN_LBL_SORT'), dict('GEN_LBL_FORMAT'), dict('GEN_LBL_ACTIVE')));
                while ($record = $page->nextRecord()) {
                    $select = new htmlCheckbox('selector_' . $record['ID'], false);
                    $select->setProp('onclick', 'selectButtons(' . singleQuote($form->getProp(CLX_PROP_ID)) . ');');
                    $table->addCell($select->source(), array(CLX_PROP_CLASS => CLX_CLASS_FOOTERS, CLX_PROP_WIDTH => '40', CLX_PROP_ALIGN => 'center'));
                    $entry = ClxCommon::$db->getField('CLX_DB_FIELDS', 'FIELDFORMLABEL', 'FIELDNAME  = ' . singleQuote($record['FIELDNAME']));

                    if (is_null($record['FIELDLISTLABEL']) || $record['FIELDLISTLABEL'] === '') {
                        $collabel = '';
                    } else {
                        $collabel = $DICT->dictFastEdit($record['FIELDLISTLABEL']);
                    }
                    $table->addCells(($i % CLX_NUM_TWO == CLX_NUM_ZERO) ? CLX_CLASS_DATA : CLX_CLASS_DATAALT, array(
                        $record['TABLECODE'] . '.' . $record['FIELDNAME'], dict($entry), $collabel, $sort[$record['SORTING']], dict($this->format_types[$record['FORMAT']]), $yesno[$record['ACTIVE']]));
                    $i++;
                }
            }
            $buttons = '';
            if ($this->getAccess()->canModify()) {
                $buttons .=htmlButton::getButton('submit', dict('GEN_LBL_MODIFY'), array('action' => 'MODIFY', 'form' => $form->getProp(CLX_PROP_ID), CLX_PROP_ID => 'MULT'))->source();
            }
            if ($this->getAccess()->canDelete()) {
                $buttons .=htmlButton::getButton('submit', dict('GEN_LBL_EXCLUDE'), array('action' => 'DELETE', 'form' => $form->getProp(CLX_PROP_ID), CLX_PROP_ID => 'MULT'))->source();
            }
            if ($this->getAccess()->canDisplay()) {
                $b = new htmlFormSubmit(dict('GEN_LBL_INCLUDE_SOMETHING', dict('GEN_LBL_FIELD')), 'CREATE', $form->getProp(CLX_PROP_ID));
                $b->setProp(CLX_PROP_ID, 'ON');
                $buttons .= $b->source();
            }
            if (($page->getProp('qty') > CLX_NUM_ONE) and $this->getAccess()->canModify()) {
                $buttons .=htmlButton::getButton('submit', dict('GEN_LBL_REORDER'), array('action' => 'REORDER', 'form' => $form->getProp(CLX_PROP_ID), CLX_PROP_ID => 'ON'))->source();
            }
            $form->addContents($page->footers($table, $buttons));
            return buildPanelInterface(dict('TOOL_ANRC_TITLE_LIST'), $reportcode . 'COLUMNS', $form->source(), CLX_NUM_MINUS_1);
        } else {
            return CLX_CHAR_EMPTY_STRING;
        }
    }

    /**
     * Tool Edit Form
     * @param string $ac
     * @param string $idlist
     * @param array $record
     * @param array $msg
     * @return string
     */
    function editForm($ac, $idlist, $record, $msg) {
        $DICT = &getTool('DICT');
        $modify = (UserRequest::$request['ac'] === 'MODIFY');
        $title = ($modify) ? dict('TOOL_ANRC_TITLE_MODIFY') : dict('TOOL_ANRC_TITLE_CREATE');
        $reportrec = ClxCommon::$db->getRecord('CLX_AN_REPORTS', 'IDCODE=' . singleQuote($record['ANRC_reportcode']));
        $report_type = intval(ClxCommon::$db->getField('CLX_AN_REPORTS', 'REPORT_TYPE', 'IDCODE = ' . singleQuote($record['ANRC_reportcode'])));
        $jsscript = '';
        $form = new htmlForm(CLX_PROP_ACTION, 'tool.php');
        $form->addArgument(CLX_PROP_TO, $this->code);
        $form->addArgument('ac', $ac);
        $form->addArgument('confirm', '1');
        $form->addArgument('ANRC_id', $record['ANRC_id']);
        $form->addArgument('ANRC_reportcode', $record['ANRC_reportcode']);
        $form->addArgument('report_type', $reportrec['REPORT_TYPE']); //just to simplify validation on checkForm()
        $form->addArgument('idlist', toString($idlist));
        //-- Headers
        $table = new htmlTable(CLX_WIDTH_TWO);
        $table->setProps(array(CLX_PROP_CLASS => CLX_CLASS_FORM, CLX_PROP_WIDTH => CLX_PROP_100PCT));
        $table->addCell($title, array(CLX_PROP_CLASS => CLX_CLASS_TITLE, CLX_PROP_COLSPAN => CLX_WIDTH_TWO));
        // Query code
        if ($report_type === CLX_NUM_ONE) {//type template
            $jsscript = setJavaScriptTag(getJavaScriptFilename('ANRC_editform.js'), true);
            $ANRQ = &getTool('ANRQ');
            if (!empty($msg['ANRC_repqrycode'])) {
                $table->addCell($msg['ANRC_repqrycode'], array(CLX_PROP_CLASS => CLX_CLASS_ALERT, CLX_PROP_COLSPAN => $table->width));
            }
            $table->addCell(dict('GEN_LBL_REPQRYCODE'), array(CLX_PROP_CLASS => CLX_CLASS_PROMPTREQUIRED));
            $querycode = ClxCommon::$db->getField('CLX_AN_REPORT_QUERIES', 'QUERYCODE', 'IDCODE = ' . singleQuote($record['ANRC_repqrycode']));
            $table->addCell($ANRQ->chooseRecord('ANRC_repqrycode', $record['ANRC_repqrycode'], array('ANRC_reportcode' => $record['ANRC_reportcode'])), array(CLX_PROP_CLASS => CLX_CLASS_DATA));
            $form->addArgument('ANRC_tablecode', $record['ANRC_tablecode']);
            $form->addArgument('ANRC_fieldname', $record['ANRC_fieldname']);
        } else {
            $querycode = $reportrec['QUERYCODE'];
            $form->addArgument('ANRC_repqrycode', $record['ANRC_repqrycode']);
        }
        // Fieldname
        if (!empty($msg['ANRC_tablecode_fieldname'])) {
            $table->addCell($msg['ANRC_tablecode_fieldname'], array(CLX_PROP_CLASS => CLX_CLASS_ALERT, CLX_PROP_COLSPAN => $table->width));
        }
        $DBQT = &getTool('DBQT');
        $fields = $DBQT->getTableFieldsSql($querycode);
        $pop = new htmlSelect('ANRC_tablecode_fieldname');
        while (($field = ClxCommon::$db->getNextRecord($fields))) {
            $fieldname = $field['TABLECODE'] . '.' . $field['FIELDNAME'];
            $fieldcode = $field['TABLECODE'] . '.' . $field['FIELDNAME'];
            $entry = ClxCommon::$db->getField('CLX_DB_FIELDS', 'FIELDFORMLABEL', 'FIELDNAME  = ' . singleQuote($field['FIELDNAME']));
            $pop->addItem($fieldname . ' - ' . dict($entry), $fieldcode, ($fieldcode == ($record['ANRC_tablecode'] . '.' . $record['ANRC_fieldname'])));
        }
        $table->addData(dict('GEN_LBL_FIELDNAME') . ':', $pop->source());
        // Column Sort
        if ($report_type === CLX_NUM_ONE) {
            $form->addArgument('ANRC_sorting', 'NONE');
        } else {
            $pop = new htmlSelect('ANRC_sorting');
            foreach ($this->sorting_types as $key => $value) {
                $pop->addItem(dict($value), $key, ($record['ANRC_sorting'] == $key));
            }
            $table->addData(dict('GEN_LBL_SORT') . ':', $pop->source());
        }
        // Column Format
        //$table->addYesNo(dict('GEN_LBL_FORMAT'),'ANRC_format',$record['ANRC_format'],0);
        $table->addList(dict('GEN_LBL_FORMAT'), 'ANRC_format', $record['ANRC_format'], $this->format_types, false);
        // Column active/inactive
        $table->addYesNo(dict('GEN_LBL_ACTIVE'), 'ANRC_active', $record['ANRC_active'], CLX_NUM_ZERO);

        //-- Label
        if ($report_type === CLX_NUM_ONE) {
            $form->addArgument('ANRC_fieldlistlabel', '');
        } else {
            $table2 = new htmlTable(CLX_WIDTH_TWO);
            $table->setProps(array(CLX_PROP_WIDTH => CLX_PROP_100PCT));
            /*
             * GEN_LBL_COLLABEL_EV
             * Column Title (Specific)
             * Título Columna (Específico)
             */
            $DICT->editEntry($msg, dict('GEN_LBL_COLLABEL_EV') . ':', 'ANRC_fieldlistlabel', $record, $table2, $form);
            $table->addCell($table2->source(), array(CLX_PROP_COLSPAN => CLX_WIDTH_TWO));
        }
        $form->addContents($table->source());
        //-- Footers
        $buttons = '';
        $table = new htmlTable(CLX_WIDTH_TWO);
        $table->setProps(array(CLX_PROP_CLASS => CLX_CLASS_FORM, CLX_PROP_WIDTH => CLX_PROP_100PCT));
        if (!empty($modify)) {
            $buttons .= htmlButton::getButton('submit', dict('GEN_LBL_SAVE'), array('action' => 'MODIFY', 'form' => $form->getProp(CLX_PROP_ID)))->source();
        } else {
            $c = new htmlFormSubmit(dict('GEN_LBL_INCLUDE'), 'CREATE', $form->getProp(CLX_PROP_ID));
            $buttons .= $c->source();
        }
        $c = new htmlButtonCancel();
        $buttons .= $c->source();
        $table->addCell($buttons, array(CLX_PROP_CLASS => CLX_CLASS_FOOTERS, CLX_PROP_COLSPAN => CLX_WIDTH_TWO));
        $form->addContents($table->source());
        return $jsscript . $form->source();
    }

    /**
     * Verify Tool Edit Forms
     * @param array $msg
     * @return bool
     */
    function checkForm(&$msg) {
        $msg = array();
        if (intval(UserRequest::$post['report_type']) === CLX_NUM_ONE && empty(UserRequest::$post['ANRC_repqrycode'])) {
            $msg['ANRC_repqrycode'] = dict('GEN_LBL_FIELD_REQUIRED');
        }
        if (empty(UserRequest::$post['ANRC_tablecode_fieldname'])) {
            $msg['ANRC_tablecode_fieldname'] = dict('GEN_LBL_FIELD_REQUIRED');
        }

        if (array_key_exists('ANRC_fieldlistlabel_DICT', UserRequest::$post) && strlen(UserRequest::$post['ANRC_fieldlistlabel_DICT'])>0) {
            $DICT = &getTool('DICT');
            $DICT->validateEntry('ANRC_fieldlistlabel', UserRequest::$post, false, $msg);
        }
        return count($msg) == CLX_NUM_ZERO;
    }

    /**
     * Builds confirm delete form for tool
     * @param obj $table
     * @param array $record
     */
    function confirmDeleteFill(&$table, $record) {
        $table->addData(dict('GEN_LBL_REPQRYCODE') . ':', $record['ANRC_repqrycode']);
        $table->addData(dict('GEN_LBL_TABLECODE') . ':', $record['ANRC_tablecode']);
        $table->addData(dict('GEN_LBL_FIELDNAME') . ':', $record['ANRC_fieldname']);
    }

    /**
     * Creates reorder list
     * @return string
     */
    function reorderRecords() {
        $sql = 'SELECT (case R.REPORT_TYPE  WHEN 1 THEN	G.REPQRYCODE ELSE R.QUERYCODE END) QUERYCODE, G.TABLECODE, G.FIELDNAME, G.ID FROM CLX_AN_REPORTS R, CLX_AN_REPORT_COLUMNS G WHERE R.IDCODE = G.REPORTCODE AND G.REPORTCODE=' . singleQuote(UserRequest::$post['reportcode']) . ' ORDER BY G.SEQNUM';
        $list = array();
        $rs = ClxCommon::$db->executeQuery($sql);
        while ($r = ClxCommon::$db->getNextRecord($rs, ASSOCIATIVE)) {
            $list[$r['ID']] = $r['QUERYCODE'] . ': ' . $r['TABLECODE'] . '.' . $r['FIELDNAME'];
        }
        return $list;
    }

    /**
     * Performs reorder record action
     */
    function reorderAction() {
        parent::reorderAction();
        $callingpage = peekHistory();
        $ANRP = &getTool('ANRP');
        $ANRP->versionIncrease('IDCODE', $callingpage['reportcode']);
    }

    /**
     * Loads current record on selected id
     *
     * @param integer $id
     * @return array
     */
    function loadToolRecord($recordid) {
        $values = ClxCommon::$db->getRecord($this->tablename, 'ID=' . $recordid);
        $record = array();
        $record['ANRC_id'] = $recordid;
        $record['ANRC_reportcode'] = $values['REPORTCODE'];
        $record['ANRC_tablecode'] = $values['TABLECODE'];
        $record['ANRC_fieldname'] = $values['FIELDNAME'];
        $record['ANRC_seqnum'] = $values['SEQNUM'];
        $record['ANRC_sorting'] = $values['SORTING'];
        $record['ANRC_format'] = $values['FORMAT'];
        $record['ANRC_repqrycode'] = $values['REPQRYCODE'];
        $record['ANRC_active'] = $values['ACTIVE'];

        $DICT = &getTool('DICT');
        $DICT->loadEntry('ANRC_fieldlistlabel', $values['FIELDLISTLABEL'], $record);
        return $record;
    }

    /**
     * Update records
     *
     * @return integer
     */
    function saveRecord() {
        $data = UserRequest::$post['ANRC_tablecode_fieldname'];
        $pos = strpos($data, '.');
        $record = array();
        $record['REPORTCODE'] = UserRequest::$post['ANRC_reportcode'];
        $record['TABLECODE'] = substr($data, CLX_NUM_ZERO, $pos);
        $record['FIELDNAME'] = substr($data, $pos + CLX_NUM_ONE);
        $record['SORTING'] = UserRequest::$post['ANRC_sorting'];
        $record['FORMAT'] = UserRequest::$post['ANRC_format'];
        $record['REPQRYCODE'] = UserRequest::$post['ANRC_repqrycode'];
        $record['ACTIVE'] = UserRequest::$post['ANRC_active'];

        if (!array_key_exists('ANRC_fieldlistlabel_DICT', UserRequest::$post) || UserRequest::$post['ANRC_fieldlistlabel_DICT'] === '') {
            $record['FIELDLISTLABEL'] = 'null';
        } else {
            $DICT = &getTool('DICT');
            $record['FIELDLISTLABEL'] = $DICT->saveEntry('ANRC_fieldlistlabel', UserRequest::$post);
        }
        ClxCommon::$db->updateRecord($this->tablename, $record, 'ID=' . UserRequest::$post['ANRC_id']);
        $ANRP = &getTool('ANRP');
        $ANRP->versionIncrease('IDCODE', $record['REPORTCODE']);
        return UserRequest::$post['ANRC_id'];
    }

    /**
     * Creates record
     *
     * @return integer
     */
    function createRecordVoid_() {
        ClxFreeReturn($this->createRecord_());
    }

    function createRecord_() {
        $data = UserRequest::$post['ANRC_tablecode_fieldname'];
        $pos = strpos($data, '.');
        $record = array();
        $record['REPORTCODE'] = UserRequest::$post['ANRC_reportcode'];
        $record['TABLECODE'] = substr($data, CLX_NUM_ZERO, $pos);
        $record['FIELDNAME'] = substr($data, $pos + CLX_NUM_ONE);
        $record['SEQNUM'] = ClxCommon::$db->getField($this->tablename, 'SEQNUM', 'REPORTCODE=' . singleQuote($record['REPORTCODE']) . ' ORDER BY SEQNUM DESC') + CLX_NUM_ONE;
        $record['SORTING'] = UserRequest::$post['ANRC_sorting'];
        $record['FORMAT'] = UserRequest::$post['ANRC_format'];
        $record['REPQRYCODE'] = UserRequest::$post['ANRC_repqrycode'];
        $record['ACTIVE'] = UserRequest::$post['ANRC_active'];

        if (!array_key_exists('ANRC_fieldlistlabel_DICT', UserRequest::$post) || UserRequest::$post['ANRC_fieldlistlabel_DICT'] === '') {
            $record['FIELDLISTLABEL'] = 'null';
        } else {
            $DICT = &getTool('DICT');
            $record['FIELDLISTLABEL'] = $DICT->createEntry('ANRC_fieldlistlabel', UserRequest::$post);
        }
        $recordid = ClxCommon::$db->createRecord($this->tablename, $record, false);
        $ANRP = &getTool('ANRP');
        $ANRP->versionIncrease('IDCODE', $record['REPORTCODE']);
        return $recordid;
    }

    /**
     * Deletes record on selected id
     *
     * @param integer $id
     */
    function deleteRecordSk($id) {
        $grouprec = ClxCommon::$db->getRecord($this->tablename, 'ID=' . $id);
        ClxCommon::$db->deleteRecords($this->tablename, 'ID=' . $id);
        $ANRP = &getTool('ANRP');
        $ANRP->versionIncrease('IDCODE', $grouprec['REPORTCODE']);
    }

    /**
     * Defines default record on initializacion
     *
     * @return array
     */
    function defaultRecord() {
        $callingpage = peekHistory();
        $record = array();
        $record['ANRC_id'] = CLX_INIT_ZERO;
        $record['ANRC_reportcode'] = $callingpage['reportcode'];
        $record['ANRC_tablecode'] = '';
        $record['ANRC_fieldname'] = '';
        $record['ANRC_seqnum'] = '0';
        $record['ANRC_sorting'] = 'NONE';
        $record['ANRC_format'] = '0';
        $record['ANRC_repqrycode'] = '';
        $record['ANRC_active'] = '1';

        $record['ANRC_fieldlistlabel'] = false;
        $record['ANRC_fieldlistlabel_DICT'] = '';
        return $record;
    }

}
