<?php

/**
 * Reports Class
 * For more information contact  support@alenet.com
 *
 * @author       $Author: alenetdev $
 * @copyright    Alenet, Inc.
 * @since        $Date: 2019-01-22 19:45:22 +0000 (Tue, 22 Jan 2019) $
 * @version    	$Rev: 10496 $
 * @package		Clientix.Tool.Analytics
 * @abstract		Report Builder
 */

use Jaspersoft\Client\Client;
use Jaspersoft\Tool\Util;
use Jaspersoft\Dto\ImportExport\ImportTask;

class an_reports extends clx_tool {

    /**
     * List of report types
     * @var array
     */
    var $reportTypes;

    /**
     * List of report formats
     * @var array
     */
    var $reportFormatForm;

    /**
     * Query IDCODE qty of zero positions at IDCODE end (for automatic IDCODE generation on duplicate). For example: R<id>_XXX, where XXX is the IDCODE end, zero filled consecutive number
     * @var Integer
     */
    var $QryIdcodeEndPos = CLX_NUM_THREE;

    /**
     * Query IDCODE mask for code generation on duplicate
     * @var VarChar
     */
    var $QryCodeMask = 'R<!reportid>_<!consecutive>';
    var $jasperTypes = array('html' => 'GEN_LBL_HTML', 'pdf' => 'GEN_LBL_PDF', 'xls' => 'GEN_LBL_EXCELL', 'xlsx' => 'GEN_LBL_EXCELX', 'csv' => 'GEN_LBL_CSV_', 'rtf' => 'GEN_LBL_RTF', 'xml' => 'GEN_LBL_XML', 'pptx' => 'GEN_LBL_PPTX', 'odt' => 'GEN_LBL_ODT', 'ods' => 'GEN_LBL_ODS', 'docx' => 'GEN_LBL_DOCX');

	// 2018-12-10 CR Caso 1003486 - GB-Rep BATCH no arranca
	var $output_to_file = null;
	
    /**
     * Constructor
     */
    function __construct() {
	$this->code = 'ANRP';
	$this->name = 'Reports Manager';
	$this->system = 'Analytics';
	$this->version = '0.1.4.8';

	$this->has_preferences = false;
	$this->has_access = true;
	$this->has_package = true;
	$this->has_pagelet = true;
	$this->tablename = 'CLX_AN_REPORTS';
	$this->record_name_singular = 'report';
	$this->record_name_plural = 'reports';

	$this->primary_field = 'IDCODE';
	$this->secondary_field = 'IDNAME';
	$this->seqnum_field = '';
	$this->reportTypes = array('GEN_LBL_DEFAULT', 'GEN_LBL_TEMPLATE', 'GEN_LBL_JASPER');

	$this->editform = array('title' => 'TOOL_ANRP_TITLE_CREATE',
	    'panels' => array(
		array('type' => 'normal',
		    'entries' => array(
			array('label' => 'GEN_LBL_CODE', 'fields' => array(array('name' => 'idcode', 'type' => 'string', 'default' => '', 'size' => '16', 'displaysize' => '16', 'dontmodify' => '1', 'validations' => array(array('type' => 'sqlobject', 'required' => '1'), array('type' => 'unique', 'fieldname' => 'IDCODE', 'tablename' => $this->tablename))))),
			array('label' => 'GEN_LBL_NAME', 'fields' => array(array('name' => 'idname', 'type' => 'string', 'default' => '', 'size' => '64', 'displaysize' => '48', 'validations' => array(array('type' => 'string', 'required' => '1'))))),
			array('label' => 'GEN_LBL_VERSION', 'fields' => array(array('name' => 'version', 'type' => 'string', 'default' => '1.0.0', 'size' => '32', 'displaysize' => '16', 'validations' => array(array('type' => 'string', 'required' => '1'))))),
			array('label' => 'GEN_LBL_DESCRIPTION', 'fields' => array(array('name' => 'description', 'type' => 'text', 'default' => '', 'cols' => '48', 'rows' => '4'))),
			array('label' => 'GEN_LBL_MODULECODE', 'fields' => array(array('name' => 'modulecode', 'type' => 'lookup', 'default' => '', 'tool' => 'DBMO', 'validations' => array(array('type' => 'sqlobject', 'required' => '1'), array('type' => 'lookup', 'fieldname' => 'IDCODE', 'tablename' => 'CLX_DB_MODULES'))))),
			array('label' => 'GEN_LBL_ACTIVE', 'fields' => array(array('name' => 'active', 'type' => 'list', 'default' => '1', 'options' => $this->yesno, 'validations' => array()))),
			array('label' => 'GEN_LBL_REPORT_TYPE', 'fields' => array(array('name' => 'report_type', 'type' => 'dynamic', 'default' => '0', 'panels' => $this->reportTypes, 'validations' => array(array('type' => 'string', 'required' => '1'))))),
		    )
		),
		//new panel
		array('caption' => 'GEN_LBL_DEF_TYPE_OPTIONS', 'type' => 'dynamic', 'code' => '0',
		    'entries' => array(
			array('label' => 'GEN_LBL_QUERYCODE', 'fields' => array(array('name' => 'querycode', 'type' => 'lookup', 'default' => '', 'tool' => 'DBQY', 'validations' => array(array('type' => 'sqlobject', 'required' => '0'), array('type' => 'lookup', 'fieldname' => 'IDCODE', 'tablename' => 'CLX_DB_QUERIES')))))
		    )
		),
		//new panel
		array('type' => 'normal', 'title' => 'TOOL_ANHF_TITLE_CREATE', 'code' => '1',
		    // 'caption' => 'TOOL_ANHF_TITLE_CREATE',
		    'entries' => array(
			array('label' => 'GEN_LBL_HEADERCODE', 'fields' => array(array('name' => 'headercode', 'type' => 'lookup', 'default' => '', 'tool' => 'ANHF', 'validations' => array(array('type' => 'sqlobject', 'required' => '0'), array('type' => 'lookup', 'fieldname' => 'idcode', 'tablename' => 'CLX_AN_HEAD_FOOT'))))),
			array('label' => 'GEN_LBL_FOOTERCODE', 'fields' => array(array('name' => 'footercode', 'type' => 'lookup', 'default' => '', 'tool' => 'ANHF', 'validations' => array(array('type' => 'sqlobject', 'required' => '0'), array('type' => 'lookup', 'fieldname' => 'idcode', 'tablename' => 'CLX_AN_HEAD_FOOT')))))
		    )
		),
		//new panel tempalte
		array('caption' => 'GEN_LBL_DEF_TYPE_TEMPLATE', 'type' => 'dynamic', 'code' => '1',
		    'entries' => array(
			array('label' => 'GEN_LBL_TEMPLATE_REF', 'fields' => array(array('name' => 'template_ref', 'type' => 'string', 'default' => '', 'size' => '64', 'displaysize' => '48', 'readonly' => CLX_NUM_ONE, 'validations' => array(array('type' => 'string', 'required' => '0'))))),
			array('label' => 'GEN_LBL_TEMPLATE', 'fields' => array(array('name' => 'template', 'type' => 'file', 'validations' => array(array('type' => 'string', 'required' => '0')))))
		    )
		), //new panel jasper
		array(
		    'caption' => 'GEN_LBL_DEF_TYPE_JASPER',
		    'type' => 'dynamic',
		    'code' => '2',
		    'entries' => array(
			array(
			    'label' => 'GEN_LBL_JASPER_REF',
			    'fields' => array(
				array('name' => 'jasper_template_ref', 'type' => 'string', 'default' => '',
				    'size' => '64', 'displaysize' => '48', 'readonly' => CLX_NUM_ZERO,
				    'validations' => array(
					array('type' => 'string', 'required' => '0')
				    )
				)
			    )
			)/*,*/
			/*array('label' => 'GEN_LBL_TEMPLATE_REF', 'fields' => array(array('name' => 'template_ref', 'type' => 'string', 'default' => '', 'size' => '64', 'displaysize' => '48', 'readonly' => 1, 'validations' => array(array('type' => 'string', 'required' => '0'))))),*/
		/*	array('label' => 'GEN_LBL_TEMPLATE', 'fields' => array(array('name' => 'template', 'type' => 'file', 'validations' => array(array('type' => 'string', 'required' => '0')))))*/
		    )
		),
		//new panel
		array('type' => 'normal', 'title' => 'PDF_OPTIONS', 'code' => '1',
		    'entries' => array(
			array('label' => 'GEN_LBL_PDF_PRINT_ORIENTATION', 'fields' => array(array('name' => 'pdf_print_orientation', 'type' => 'list', 'default' => '1', 'options' => array('0' => 'GEN_LBL_PORTRAIT', '1' => 'GEN_LBL_LANDSCAPE')))),
			array('label' => 'GEN_LBL_PDF_PRINT_FORMAT', 'fields' => array(array('name' => 'pdf_print_format', 'type' => 'list', 'default' => '1', 'options' => array('2' => 'GEN_LBL_LETTER', '0' => 'GEN_LBL_A4', '1' => 'GEN_LBL_A5', '3' => 'GEN_LBL_LEGAL', '4' => 'GEN_LBL_OFFICE', '5' => 'GEN_LBL_FREE')))),
			array('label' => 'GEN_LBL_PDF_PRINT_HEIGHT', 'fields' => array(array('name' => 'pdf_print_height', 'type' => 'string', 'default' => '0', 'size' => '8', 'displaysize' => '16', 'sufix' => 'GEN_LBL_PIXEL', 'validations' => array(array('type' => 'integer', 'required' => '0'))))),
			array('label' => 'GEN_LBL_PDF_PRINT_WIDTH', 'fields' => array(array('name' => 'pdf_print_width', 'type' => 'string', 'default' => '0', 'size' => '8', 'displaysize' => '16', 'sufix' => 'GEN_LBL_PIXEL', 'validations' => array(array('type' => 'integer', 'required' => '0'))))),
			array('label' => 'GEN_LBL_PDF_PRINT_MARGIN_BREAK', 'fields' => array(array('name' => 'pdf_print_margin_break', 'type' => 'string', 'default' => '20', 'size' => '8', 'displaysize' => '16', 'sufix' => 'GEN_LBL_PIXEL', 'validations' => array(array('type' => 'integer', 'required' => '0'))))),
			array('label' => 'GEN_LBL_PDF_RUN_BATCH', 'fields' => array(array('name' => 'run_batch', 'type' => 'list', 'default' => '1', 'options' => $this->yesno)))
		    )
		),
		//new panel
		array(CLX_CLASS_TITLE => 'GEN_LBL_ADD_INFORMATION', 'type' => 'normal',
		    'entries' => array(
			array('label' => 'GEN_LBL_SHOWDETAIL', 'fields' => array(array('name' => 'show_detail', 'type' => CLX_CLASS_LIST, 'default' => '1', 'options' => $this->yesno))),
			array('label' => 'GEN_LBL_COLAPSABLE', 'fields' => array(array('name' => 'colapsable', 'type' => CLX_CLASS_LIST, 'default' => '1', 'options' => $this->yesno))),
			array('label' => 'GEN_LBL_HIDE_GRAN_TOTAL', 'fields' => array(array('name' => 'hide_gran_total', 'type' => CLX_CLASS_LIST, 'default' => '1', 'options' => $this->yesno))),
			array('label' => 'GEN_LBL_TITLE', 'fields' => array(array('name' => CLX_CLASS_TITLE, 'type' => 'dict', 'required' => '0'))),
			array('label' => 'GEN_LBL_SUBTITLE', 'fields' => array(array('name' => CLX_CLASS_SUBTITLE, 'type' => 'dict', 'required' => '0'))),
			array('label' => 'GEN_LBL_SEARCHFORM', 'fields' => array(array('name' => 'search_form', 'type' => 'lookup', 'default' => '', 'tool' => 'UIFM', 'validations' => array(array('type' => 'sqlobject', 'required' => '0'), array('type' => 'lookup', 'fieldname' => 'IDCODE', 'tablename' => 'CLX_UI_FORMS')))))
		    )
		)
	    )
	);


	$this->quickfilterform = array('panels' => array(
		array('type' => 'normal',
		    'entries' => array(
			array('label' => 'GEN_LBL_CODE',
			    'fields' => array(
				array('name' => 'idcode', 'type' => 'string', 'default' => '', 'size' => '16', 'displaysize' => '16', 'dontmodify' => '1',
				    'validations' => array()
				)
			    )
			), array('label' => 'GEN_LBL_NAME',
			    'fields' => array(
				array('name' => 'idname', 'type' => 'string', 'default' => '', 'size' => '64', 'displaysize' => '48', 'validations' => array()
				)
			    )
			), array('label' => 'GEN_LBL_MODULECODE',
			    'fields' => array(
				array('name' => 'modulecode', 'type' => 'lookup', 'default' => '', 'tool' => 'DBMO', 'validations' => array())
			    )
                        ), array('label' => 'GEN_LBL_REPORT_TYPE',
			    'fields' => array(
				array('name' => 'report_type', 'type' => CLX_CLASS_LIST, 'default' => '0', 
                                     'options' => array(''=>'GEN_LBL_ALL', '0'=>'GEN_LBL_DEFAULT', '1'=>'GEN_LBL_TEMPLATE', '2'=> 'GEN_LBL_JASPER')
                                    ,'validations' => array())
			    )
                        )
                        
		    )
		)
	    )
	);

	$this->reportFormatForm = array(CLX_CLASS_TITLE => 'TOOL_ANRP_TITLE_REPORTFORMAT',
	    'panels' => array(
		array('type' => 'normal',
		    'entries' => array(
			array('label' => 'GEN_LBL_REPORTID',
			    'fields' => array(
				array('name' => CLX_PROP_ID, 'type' => 'string', 'readonly' => '1'))
			), array('label' => 'GEN_LBL_FORMAT',
			    'fields' => array(
				array('name' => 'rf', 'type' => 'objects', 'objects' => 'RPFT'))
			)
		    )
		)
	    )
	);
    }

    /**
     * Install procedure
     */
    function install() {
	$UIMI = getTool('UIMI');
	$UIMI->add($this->code, 'MENU_TOOLS:MENU_ANALYTICS:MENU_REPORTS', 'TOOL', $this->code, 'ac=LIST');
    }

    /**
     * Describes the tool table structure.
     * @return array ('type' => 'varchar(255)', 'properties'=>'auto increment', 'index'=>'', default = '', 'dict'=>true) dict true or false means that field holds dictionary entry
     */
    function toolTableStructure() {
	$fields = array();
	$fields['ID'] = array('type' => 'int(11) unsigned', 'null' => 'NO', 'properties' => 'auto_increment', 'index' => 'PRI');
	$fields['IDCODE'] = array('type' => 'varchar(16)', 'null' => 'NO', 'properties' => '');
	$fields['IDNAME'] = array('type' => 'varchar(64)', 'null' => 'NO', 'properties' => '');
	$fields['VERSION'] = array('type' => 'varchar(32)', 'null' => 'NO', 'properties' => '');
	$fields['VERSION_COUNTER'] = array('type' => 'int(11) unsigned', 'null' => 'NO', 'properties' => '', 'default' => CLX_NUM_ZERO);
	$fields['CDATE'] = array('type' => 'datetime', 'null' => 'YES', 'properties' => '');
	$fields['MDATE'] = array('type' => 'datetime', 'null' => 'YES', 'properties' => '');
	$fields['USERNAME'] = array('type' => 'varchar(30)', 'null' => 'NO', 'properties' => '');
	$fields['DESCRIPTION'] = array('type' => 'mediumtext', 'properties' => '');
	$fields['REPORT_TYPE'] = array('type' => 'int(2) unsigned', 'null' => 'NO', 'default' => CLX_NUM_ZERO);
	$fields['QUERYCODE'] = array('type' => 'varchar(16)', 'null' => 'YES', 'properties' => '');
	$fields['MODULECODE'] = array('type' => 'varchar(4)', 'null' => 'NO', 'properties' => '');
	$fields['SHOW_DETAIL'] = array('type' => 'tinyint(1) unsigned', 'null' => 'NO', 'properties' => '', 'default' => CLX_NUM_ZERO);
	$fields['TITLE'] = array('type' => 'varchar(64)', 'null' => 'YES', 'properties' => '');
	$fields['SUBTITLE'] = array('type' => 'varchar(64)', 'null' => 'YES', 'properties' => '');
	$fields['COLAPSABLE'] = array('type' => 'tinyint(1) unsigned', 'null' => 'NO', 'properties' => '', 'default' => CLX_NUM_ZERO);
	$fields['HIDE_GRAN_TOTAL'] = array('type' => 'tinyint(1) unsigned', 'null' => 'NO', 'properties' => '', 'default' => CLX_NUM_ZERO);
	$fields['SEARCH_FORM'] = array('type' => 'varchar(16)', 'properties' => '');
	$fields['TEMPLATE_REF'] = array('type' => 'varchar(255)', 'properties' => '');
	$fields['TEMPLATE'] = array('type' => 'mediumblob', 'properties' => '');
	$fields['CONTROLS'] = array('type' => 'varchar(255)', 'properties' => '');
	//$fields['HEADERCODE'] = array('type' => 'varchar(16)', 'null' => 'NO', 'properties' => '');
	//$fields['FOOTERCODE'] = array('type' => 'varchar(16)', 'null' => 'NO', 'properties' => '');
	$fields['HEADERCODE'] = array('type' => 'varchar(16)', 'null' => 'YES', 'properties' => '');
	$fields['FOOTERCODE'] = array('type' => 'varchar(16)', 'null' => 'YES', 'properties' => '');
	$fields['PDFPRINTORIENTATION'] = array('type' => 'tinyint(1) unsigned', 'null' => 'YES', 'properties' => '', 'default' =>CLX_NUM_ZERO);
	$fields['PDFPRINTFORMAT'] = array('type' => 'tinyint(1) unsigned', 'null' => 'YES', 'properties' => '', 'default' => CLX_NUM_TWO);
	$fields['PDFPRINTHEIGHT'] = array('type' => 'int(6) unsigned', 'null' => 'YES', 'properties' => '', 'default' => CLX_NUM_ZERO);
	$fields['PDFPRINTWIDTH'] = array('type' => 'int(6) unsigned', 'null' => 'YES', 'properties' => '', 'default' => CLX_NUM_ZERO);
	$fields['PDFPRINTMARGINBREAK'] = array('type' => 'int(3) unsigned', 'null' => 'YES', 'properties' => '', 'default' => CLX_NUM_ZERO);
	$fields['PDFRUNBATCH'] = array('type' => 'tinyint(1) unsigned', 'null' => 'NO', 'properties' => '', 'default' => CLX_NUM_ZERO);
	$fields['ACTIVE'] = array('type' => 'tinyint(1) unsigned', 'null' => 'NO', 'default' => '1', 'properties' => '');
	$indexes = array();
	$ipf = 'CLX' . $this->code;
	$indexes[$ipf . '01'] = array('index' => 'PRI', 'fieldname' => 'ID');
	$indexes[$ipf . '02'] = array('index' => 'UNI', 'fieldname' => 'IDCODE');
	$indexes[$ipf . '03'] = array('index' => 'MUL', 'fieldname' => 'IDNAME');
	$indexes[$ipf . '04'] = array('index' => 'MUL', 'fieldname' => 'QUERYCODE');
	$indexes[$ipf . '05'] = array('index' => 'MUL', 'fieldname' => 'MODULECODE');
	return array('idcode' => $this->code, 'idname' => $this->name, 'name' => $this->tablename, 'type' => 'InnoDB',
	    'fields' => $fields,
	    'indexes' => $indexes);
    }

    /**
     * Pack tool record on xml Format
     * @param array record
     * @return obj
     */
    function pack($record) {
	$o = $this->packRecord($record);
	$code = $record['IDCODE'];
	$objs = new XMLObject($this->code . '_OBJECTS');
	$this->packsub($objs, 'ANRQ', 'SELECT * FROM CLX_AN_REPORT_QUERIES WHERE REPORTCODE=' . singleQuote($code));
	$this->packsub($objs, 'ANRC', 'SELECT * FROM CLX_AN_REPORT_COLUMNS WHERE REPORTCODE=' . singleQuote($code));
	$this->packsub($objs, 'ANRG', 'SELECT * FROM CLX_AN_REPORT_GROUPS WHERE REPORTCODE=' . singleQuote($code));
	$this->packsub($objs, 'DBQY', 'SELECT * FROM CLX_DB_QUERIES WHERE IDCODE=' . singleQuote($record['QUERYCODE']));
	$this->packsub($objs, 'UIFM', 'SELECT * FROM CLX_UI_FORMS WHERE IDCODE=' . singleQuote($record['SEARCH_FORM']));
	$this->packsub($objs, 'EVNT', 'SELECT * FROM CLX_EVENTS WHERE OWNERCODE=' . singleQuote($this->code) . ' AND OWNERID=' . $record ['ID']);
	// Packing Dictionary
	$this->packsub($objs, 'DICT', 'SELECT DICT.* FROM CLX_DICTIONARY DICT, ' . $this->tablename . ' X WHERE (DICT.ENTRY=X.TITLE OR DICT.ENTRY=X.SUBTITLE) AND X.IDCODE=' . singleQuote($code));
	$o->addObject($objs);
	return $o;
    }

    function packRecord($record) {

		$record['TEMPLATE'] = base64_encode($record['TEMPLATE']);
		return parent::packRecord($record);
    }

    /**
     * Loads package into system
     * @param string $package
     */
    function unpack($parameters) {
	$package = $parameters['package'];
	$record = $this->unpackRecord($package);
	$reportcode = $record['IDCODE'];
	$record['TEMPLATE'] = base64_decode($record['TEMPLATE']);
	$exists = (ClxCommon::$db->oneExistsRecords('CLX_AN_REPORTS', 'IDCODE=' . singleQuote($reportcode)) > CLX_NUM_ZERO) ? true : false;
	if ($exists === true) {
            //
            $rec_id = ClxCommon::$db->getField($this->tablename, 'ID', 'IDCODE=' . singleQuote($reportcode));
            ClxCommon::$db->deleteRecords('CLX_EVENTS', 'OWNERCODE=' . singleQuote($this->code) . ' AND OWNERID=' . singleQuote($rec_id));
            //
	    $sql = 'SELECT IDCODE FROM CLX_AN_REPORT_QUERIES WHERE REPORTCODE=' . singleQuote($reportcode);
	    $rs = ClxCommon::$db->executeQuery($sql);
	    while ($r = ClxCommon::$db->getNextRecord($rs, ASSOCIATIVE)) {
		ClxCommon::$db->deleteRecords('CLX_AN_REP_QRY_CRITS', 'REPQRYCODE=' . singleQuote($r['IDCODE']));
	    }
	    ClxCommon::$db->deleteRecords('CLX_AN_REPORT_QUERIES', 'REPORTCODE=' . singleQuote($reportcode));
	    $sql = 'DELETE FROM CLX_AN_REPORT_GROUP_FUNC ' . ' WHERE GROUPID IN ' . '(SELECT X.ID FROM CLX_AN_REPORT_GROUPS X ' . ' WHERE X.REPORTCODE = ' . singleQuote($reportcode) . ') ';
	    ClxCommon::$db->executeQuery($sql);
	    ClxCommon::$db->deleteRecords('CLX_AN_REPORT_GROUPS', 'REPORTCODE=' . singleQuote($reportcode));
	    ClxCommon::$db->deleteRecords('CLX_AN_REPORT_COLUMNS', 'REPORTCODE=' . singleQuote($reportcode));
	    ClxCommon::$db->deleteRecords('CLX_AN_REPORTS', 'IDCODE=' . singleQuote($reportcode));
	}
	$reportid = ClxCommon::$db->createRecord($this->tablename, $record, false);
	if ($package->hasSubObject($this->code . '_OBJECTS')) {
	    $objs = $package->getObjectXml($this->code . '_OBJECTS');
	    foreach ($objs->objects as $o) {
		$toolcode = $o->tag;
		$tool = &getTool($toolcode);
		$tool->unpack(array('package' => $o, 'id' => $reportid));
	    }
	}
    }

    /**
     * Action is called for every user interaction with the module...
     *
     * @param string $ac
     * @param string $idlist
     * @return string
     */
    function action($ac, $idlist) {
		if ( isset($_SESSION["_dEBUG"]) ) { 
			$tta=fopen(kPathToWebSite."/".kPathToTemp."archivo.txt","a+");
			
			fwrite($tta,chr(13).chr(10)."6-hora:".date("h:i:s d/m/Y").chr(13).chr(10));
			fwrite($tta,chr(13).chr(10)."action->ac: ".$ac.chr(13).chr(10));
			fwrite($tta,chr(13).chr(10)."Puede leer:".($this->getAccess()->canRead() ?"si":"no(no ejecuta accion de escritura formulario)").chr(13).chr(10)); 	
			if (isset(UserRequest::$request['ANRP_report_type']))
			fwrite($tta,chr(13).chr(10)."action->ANRP_report_type: ". intval(UserRequest::$request['ANRP_report_type']).chr(13).chr(10));	
			$te=debug_backtrace();

			for ($t=0;$t<12;$t++) {
				if (isset($te[$t]["file"]))
				fwrite($tta,substr($te[$t]["file"].str_repeat(" ",80),0,80).chr(9).$te[$t]["line"].chr(9).$te[$t]["function"].chr(13).chr(10));


			}
			fclose($tta);
		}	

		$msg = array();
		if ($this->getAccess()->canRead()) {
			if ( isset($_SESSION["_dEBUG"]) ) {
					$te=debug_backtrace();
					$tta=fopen(kPathToWebSite."/".kPathToTemp."archivo.txt","a+");
					fwrite($tta,chr(13).chr(10)."an2- hora:".date("h:i:s d/m/Y").chr(13).chr(10));
					fwrite($tta,chr(13).chr(10).'ac:'.var_export($ac,true).chr(13).chr(10));
					for ($t=0;$t<18;$t++) {
						if (isset($te[$t]["file"]))
							fwrite($tta,substr($te[$t]["file"].str_repeat(" ",80),0,80).chr(9).$te[$t]["line"].chr(9).$te[$t]["function"].chr(13).chr(10));


						
					}	
									fwrite($tta,chr(13).chr(10));
									fclose($tta);
			}

			switch ($ac) {
			case 'REPORT':
				$reportid = $idlist[CLX_NUM_ZERO];
				if (!defined('UTIL_VIEW_DIALOG') or UTIL_VIEW_DIALOG==1 ) {
					//pre(get_defined_constants());
					if (!defined("ANRP_WINDOW") OR ANRP_WINDOW==1) {
						$jscode = 'setTimeout("openWindow(' . singleQuote(CLX_CHAR_EMPTY_STRING) . ', ' . singleQuote('pop.php?to=' . $this->code . '&ac=REPORT_BUILD&ANRP_id=' . $reportid) . ', 640, 480);",1000)';
					}
					else {
						$jscode = 'setTimeout("window.open('.singleQuote('pop.php?to=' . $this->code . '&ac=REPORT_BUILD&ANRP_id=' . $reportid) .','.singlequote('Window_Clientix').','. singlequote('resizable,scrollbars,width=640,height=480').');",1000)';

					}
					$contents = setJavaScriptTag($jscode, false);
				}
				else { 
					$url='pop.php?to=' . $this->code . '&ac=REPORT_BUILD&ANRP_id=' . $reportid;
					
					$contents=windowsmodal('',$url,1);

				} 


				$info = popHistory();
				if (isset(UserRequest::$request['next'])) {
				unset(UserRequest::$request['next']);
				}
				$contents .= executeHistory($info);
				break;
			case 'REPORT_BUILD':
                if ($this->checkReportForm($msg)) {
				// Esto resuelve el asunto de CRM-Atencion al Cliente ()SAR/BNP) cuando los operadores con seguridad limitada
				// van a imprimir 'Reporte CRM > Atencion al Cliente > reporte plantilla SERV_REQS'
				// Notar que al imprimir el reporte si tiene algo en el stack de historico entonces aplica la seguridad
				clearHistory();
                if (intval(UserRequest::$request['ANRP_report_type']) === CLX_NUM_ONE) {
					$contents = $this->buildTemplateReport();
				} elseif (intval(UserRequest::$request['ANRP_report_type']) === CLX_NUM_TWO) {//jasper
					$contents = $this->buildJasperReport(UserRequest::$request);
				} else {
                    $contents = $this->buildReport();
				}
				} else {
                    $contents = $this->askReportFormat($msg);
				}
				break;
					case 'LIST':
				if (!empty(UserRequest::$get['menuid'])) {
					if (!empty(UserRequest::$request['mc'])) {//Adding parameter to the filter if mc is set on menu to display only mc.
						$_SESSION['paging']['ANRPList']['mc'] = UserRequest::$request['mc'];
					} else {
						unset($_SESSION['paging']['ANRPList']['mc']);
					}
				}
				if (!empty($_SESSION['paging']['ANRPList']['mc'])) {
				if (empty ($_SESSION['paging']['ANRPList']['criteria']))
								$_SESSION['paging']['ANRPList']['criteria'] = ' UPPER(MODULECODE) LIKE ' . singleQuote(UPPERCASE($_SESSION['paging']['ANRPList']['mc']) . '%');   
							else 
								$_SESSION['paging']['ANRPList']['criteria'] .= ' AND MODULECODE LIKE ' . singleQuote(UPPERCASE($_SESSION['paging']['ANRPList']['mc']) . '%');   
			
				if (empty($_SESSION['paging']['ANRPList']['sortby']))
					$_SESSION['paging']['ANRPList']['sortby'] = CLX_NUM_ZERO;
				if (empty($_SESSION['paging']['ANRPList']['startat']))
					$_SESSION['paging']['ANRPList']['startat'] = CLX_NUM_ZERO;
				} 
				if ( isset($_SESSION["_dEBUG"]) ) { 
					$te=debug_backtrace();
					$tta=fopen(kPathToWebSite."/".kPathToTemp."archivo.txt","a+");
					fwrite($tta,chr(13).chr(10)."an1- hora:".date("h:i:s d/m/Y").chr(13).chr(10));
					fwrite($tta,chr(13).chr(10).'ac:'.var_export($ac,true).chr(13).chr(10));
					if (function_Exists("get_parent_class"))
						fwrite($tta,chr(13).chr(10).'parent name:'.get_parent_class().chr(13).chr(10));
					
					for ($t=0;$t<18;$t++) {
						if (isset($te[$t]["file"]))
							fwrite($tta,substr($te[$t]["file"].str_repeat(" ",80),0,80).chr(9).$te[$t]["line"].chr(9).$te[$t]["function"].chr(13).chr(10));


					}	
					fwrite($tta,chr(13).chr(10));
					fclose($tta);
				}
				$contents = parent::action($ac, $idlist);
				break;
			case 'PUBLISH':
				$contents = $this->listReportstoJasper($idlist);
				break;
			case 'DL_TPL':
				$contents = $this->downloadTemplate(UserRequest::$request['rc']);
				break;
			default: 
				if ( isset($_SESSION["_dEBUG"]) ) { 
					$te=debug_backtrace();
					$tta=fopen(kPathToWebSite."/".kPathToTemp."archivo.txt","a+");
					fwrite($tta,chr(13).chr(10)."an3- hora:".date("h:i:s d/m/Y").chr(13).chr(10));
					fwrite($tta,chr(13).chr(10).'ac:'.var_export($ac,true).chr(13).chr(10));
					if (function_Exists("get_parent_class"))
						fwrite($tta,chr(13).chr(10).'parent name:'.get_parent_class().chr(13).chr(10));
					
					for ($t=0;$t<18;$t++) {
						if (isset($te[$t]["file"]))
							fwrite($tta,substr($te[$t]["file"].str_repeat(" ",80),0,80).chr(9).$te[$t]["line"].chr(9).$te[$t]["function"].chr(13).chr(10));


					}	
					fwrite($tta,chr(13).chr(10));
					fclose($tta);
				}
				$contents = parent::action($ac, $idlist);
				break;
			}
		} else {
			$contents = warning(dict('GEN_ERR_NO_ACCESS') . '[AN_REPORTS-CAN_READ][' . $ac . '] ' . $this->getCode(false));
		}
		return $contents;
    }

    /**
     * Builds listing on tool
     *
     * @param string $listingcode
     * @return string
     */
    function makelist($listingcode = null) {
    //$t = time('U'); genera un warning
	$t = time();
	$to = $this->code;
	$here = 'tool.php?to=' . $to . '&t=' . $t;
	clearHistory();
	pushHistory($this->code, 'LIST', array());
	$yesno = array();
	foreach ($this->yesno as $key => $val)
	    $yesno[$key] = dict($val);
	$form = new htmlForm(CLX_PROP_ACTION, 'tool.php');
	$form->setProp(CLX_PROP_ID, 'ANRPList');
	$form->addArgument(CLX_PROP_TO, $this->code);
	$form->addArgument('ac', 'NOT_DEFINED');
	$selectclause = '*';
	$fromclause = 'CLX_AN_REPORTS';
	$whereclause = '';
	$sortclause = '';
	$page = new xPaging();
	$page->pageInit($form->getProp(CLX_PROP_ID), $here, $selectclause, $fromclause, $whereclause, $sortclause, '');
	$table = new htmlTable(CLX_NUM_17);
	$table->setProps(array(CLX_PROP_CLASS => CLX_CLASS_LIST, CLX_PROP_WIDTH => CLX_PROP_100PCT));
	$table->addCell(dict('TOOL_ANRP_TITLE_LIST'), array(CLX_PROP_CLASS => CLX_CLASS_TITLE, CLX_PROP_COLSPAN => CLX_NUM_17));
	$page->header($table);
	$table->addCells(CLX_CLASS_HEADERS, array(dict('GEN_LBL_SEL'), dict('GEN_LBL_CODE'), dict('GEN_LBL_NAME'), dict('GEN_LBL_REPORT_TYPE'), dict('GEN_LBL_QUERYCODE'), dict('GEN_LBL_SEARCHFORM'), dict('GEN_LBL_MODULECODE'), dict('GEN_LBL_VERSION'), dict('GEN_LBL_ACTIVE'), dict('GEN_LBL_HEADERCODE'), dict('GEN_LBL_FOOTERCODE'), dict('GEN_LBL_PDF_PRINT_ORIENTATION'), dict('GEN_LBL_PDF_PRINT_FORMAT'), dict('GEN_LBL_PDF_PRINT_HEIGHT'), dict('GEN_LBL_PDF_PRINT_WIDTH'), dict('GEN_LBL_PDF_PRINT_MARGIN_BREAK'), dict('GEN_LBL_PDF_RUN_BATCH')));
	$i = CLX_INIT_ZERO;
	$orientation = array('0' => 'GEN_LBL_PORTRAIT', '1' => 'GEN_LBL_LANDSCAPE5');
	$formato = array('0' => 'GEN_LBL_A4', '1' => 'GEN_LBL_A5', '2' => 'GEN_LBL_LETTER', '3' => 'GEN_LBL_LEGAL', '4' => 'GEN_LBL_OFFICE', '5' => 'GEN_LBL_FREE');
	while ($record = $page->nextRecord()) {
	    $recordid = $record['ID'];
	    $select = new htmlCheckbox('selector_' . $recordid, false);
	    $select->setProp('onclick', 'selectButtons(' . singleQuote($form->getProp(CLX_PROP_ID)) . ');');
	    $table->addCell($select->source(), array(CLX_PROP_CLASS => CLX_CLASS_FOOTERS, CLX_PROP_WIDTH => '40', CLX_PROP_ALIGN => 'center'));
	    $idname = $record['IDNAME'];
	    if ($record['DESCRIPTION'] <> '' && $record['IDNAME'] <> $record['DESCRIPTION']) {
		$idname.= '<br><i>' . xFormat::text($record['DESCRIPTION']) . '</i>';
	    }
	    $table->addCells(($i % CLX_NUM_TWO == CLX_NUM_ZERO) ? CLX_CLASS_DATA : CLX_CLASS_DATAALT, array($record['IDCODE'], $idname,
		dict($this->reportTypes[$record['REPORT_TYPE']]), $record['QUERYCODE'], $record['SEARCH_FORM'], $record['MODULECODE'], $record['VERSION'] . '-' . $record['VERSION_COUNTER'],
		$yesno[$record['ACTIVE']], $record['HEADERCODE'], $record['FOOTERCODE'], dict($orientation[$record['PDFPRINTORIENTATION']]),
		dict($formato[$record['PDFPRINTFORMAT']]), $record['PDFPRINTHEIGHT'], $record['PDFPRINTWIDTH'],
		$record['PDFPRINTMARGINBREAK'], dict($this->yesno[$record['PDFRUNBATCH']])));
	    $i++;
	}
	$buttons = '';
	$buttons .= htmlButton::getButton('submit', dict('GEN_LBL_DISPLAY'), array('action' => 'DISPLAY', 'form' => $form->getProp(CLX_PROP_ID), CLX_PROP_ID => 'MULT'))->source();
	$b = new htmlFormSubmit(dict('GEN_LBL_VIEW'), 'REPORT', $form->getProp(CLX_PROP_ID));
	$b->setProp(CLX_PROP_ID, 'ONE');
	$buttons .= $b->source();
	if ($this->getAccess()->canModify()) {
	    $buttons .=htmlButton::getButton('submit', dict('GEN_LBL_MODIFY'), array('action' => 'MODIFY', 'form' => $form->getProp(CLX_PROP_ID), CLX_PROP_ID => 'MULT'))->source();
	}


	if ($this->getAccess()->canDelete()) {
	    $buttons .=htmlButton::getButton('submit', dict('GEN_LBL_DELETE'), array('action' => 'DELETE', 'form' => $form->getProp(CLX_PROP_ID), CLX_PROP_ID => 'MULT'))->source();
	}



	if ($this->getAccess()->canDisplay()) {
	    $buttons .=htmlButton::getButton('submit', dict('GEN_LBL_DUPLICATE'), array('action' => 'DUPLICATE', 'form' => $form->getProp(CLX_PROP_ID), CLX_PROP_ID => 'ONE'))->source();
	    $buttons .=htmlButton::getButton('submit', dict('GEN_LBL_CREATE_SOMETHING', dict('GEN_LBL_REPORT')), array('action' => 'CREATE', 'form' => $form->getProp(CLX_PROP_ID), CLX_PROP_ID => 'ON'))->source();
	}


	$buttons .=htmlButton::getButton('submit', dict('GEN_LBL_FILTER'), array('action' => 'FILTER', 'form' => $form->getProp(CLX_PROP_ID), CLX_PROP_ID => 'ON'))->source();



	$form->addContents($page->footers($table, $buttons));
	return $form->source();
    }

    /**
     * Verify Tool Edit Forms
     * @param array $msg
     * @return bool
     */
    function checkForm(&$msg) {
//print_r($_FILES);
	if ( isset($_SESSION["_dEBUG"])) {
		    
		$tta=fopen(kPathToWebSite."/".kPathToTemp."archivo.txt","a+");
		fwrite($tta,chr(13).chr(10)."an4-1 hora:".date("h:i:s d/m/Y").chr(13).chr(10));
		fwrite($tta,chr(13).chr(10).var_export($msg,true).chr(13).chr(10));
		fwrite($tta,chr(13).chr(10));
		fclose($tta);
    }
	$result = parent::checkForm($msg);
	if ( isset($_SESSION["_dEBUG"])) {
		    
		$tta=fopen(kPathToWebSite."/".kPathToTemp."archivo.txt","a+");
		fwrite($tta,chr(13).chr(10)."an4-2 hora:".date("h:i:s d/m/Y").chr(13).chr(10));
		fwrite($tta,chr(13).chr(10).var_export($msg,true).chr(13).chr(10));
		fclose($tta);
    }

	//Check file existance
	if (UserRequest::$request['ac'] === 'CREATE') {
	    if (UserRequest::$post['ANRP_report_type'] == CLX_NUM_ONE && empty(UserRequest::$files['ANRP_template']['tmp_name'])) {
		$msg['ANRP_template'] = dict('GEN_LBL_FIELD_REQUIRED');
		$result = false;
	    }
	}
	if (UserRequest::$post['ANRP_report_type'] == CLX_NUM_ZERO && empty(UserRequest::$post['ANRP_querycode'])) {
	    $msg['ANRP_querycode'] = dict('GEN_LBL_FIELD_REQUIRED');
	    $result = false;
	}
	//Test type word
	$sw_extsn='';
	if (!empty(UserRequest::$files['ANRP_template']['tmp_name'])) {
	    $sw_extsn = substr(UserRequest::$files['ANRP_template']['name'], -4);
		if ( UserRequest::$post['ANRP_report_type']==2 and UserRequest::$files['ANRP_template']['type']!='application/zip') {
			$msg['ANRP_template'] = dict('GEN_LBL_ZIP_REQUIRED');
			$result = false;
	    }
	    else if (stripos(UserRequest::$files['ANRP_template']['type'], 'vnd.openxmlformats-officedocument.wordprocessingml.document') === false && $sw_extsn !== 'docx') {
			$msg['ANRP_template'] = dict('GEN_LBL_DOCX_REQUIRED');
			$result = false;
	    }
	}
	if (!empty(UserRequest::$files['ANRP_template']['error']) && (UserRequest::$post['ANRP_report_type']==1 && UserRequest::$request['ac'] === 'CREATE') ) {
		$msg['ANRP_template'] = UserRequest::$file_upload_errcodes[UserRequest::$files['ANRP_template']['error']];
		$result = false;
	}
	if ( isset($_SESSION["_dEBUG"])) {
		    
		$tta=fopen(kPathToWebSite."/".kPathToTemp."archivo.txt","a+");
		fwrite($tta,chr(13).chr(10)."an4-4 hora:".date("h:i:s d/m/Y").chr(13).chr(10));
		fwrite($tta,chr(13).chr(10).var_export($msg,true).chr(13).chr(10));
		$te=debug_backtrace();
		for ($t=0;$t<18;$t++) {
		    if (isset($te[$t]["file"]))
			fwrite($tta,substr($te[$t]["file"].str_repeat(" ",80),0,80).chr(9).$te[$t]["line"].chr(9).$te[$t]["function"].chr(13).chr(10));
		}
		fwrite($tta,chr(13).chr(10));
		fclose($tta);
	}
	return $result;
    }

    /**
     * Builds Tool Search form
     * @return string
     */
    function searchForm() {
	$msg = array();
	$contents = '';
	if (!array_key_exists('ANRP_idcode', UserRequest::$post))
	    UserRequest::$post['ANRP_idcode'] = '';
	if (!array_key_exists('ANRP_idname', UserRequest::$post))
	    UserRequest::$post['ANRP_idname'] = '';
	if (!array_key_exists('ANRP_querycode', UserRequest::$post))
	    UserRequest::$post['ANRP_querycode'] = '';
	if (!array_key_exists('ANRP_modulecode', UserRequest::$post))
	    UserRequest::$post['ANRP_modulecode'] = '';
    if (!array_key_exists('ANRP_report_type', UserRequest::$post))
	    UserRequest::$post['ANRP_report_type'] = '';
	// Search Form
	$form = new htmlForm(CLX_PROP_ACTION, UserRequest::$server['PHP_SELF']);
	$form->addArgument(CLX_PROP_TO, $this->code);
	$form->addArgument('ac', 'SEARCH');
	$form->addArgument('confirm', '1');
	$form->addArgument(CLX_PROP_TARGET, UserRequest::$request[CLX_PROP_TARGET]);
	$table = new htmlTable(CLX_WIDTH_TWO);
	$table->setProps(array(CLX_PROP_CLASS => CLX_CLASS_FORM, CLX_PROP_WIDTH => CLX_PROP_100PCT));
	$table->addCell(dict('TOOL_ANRP_TITLE_SEARCH'), array(CLX_PROP_CLASS => CLX_CLASS_TITLE, CLX_PROP_COLSPAN => CLX_WIDTH_TWO));
	$table->addField($msg, dict('GEN_LBL_CODE') . ':', 'ANRP_idcode', UserRequest::$post['ANRP_idcode'], CLX_NUM_16, CLX_NUM_16);
	$table->addField($msg, dict('GEN_LBL_NAME') . ':', 'ANRP_idname', UserRequest::$post['ANRP_idname'], CLX_NUM_32, CLX_NUM_64);
	$table->addField($msg, dict('GEN_LBL_QUERYCODE') . ':', 'ANRP_querycode', UserRequest::$post['ANRP_querycode'], CLX_NUM_16, CLX_NUM_16);
	$table->addField($msg, dict('GEN_LBL_MODULECODE') . ':', 'ANRP_modulecode', UserRequest::$post['ANRP_modulecode'], CLX_NUM_FOUR, CLX_NUM_FOUR);
        $table->addField($msg, dict('GEN_LBL_REPORT_TYPE') . ':', 'ANRP_report_type', UserRequest::$post['ANRP_report_type'], CLX_NUM_FOUR, CLX_NUM_FOUR);
	$buttons = htmlButton::getButton('submit', dict('GEN_LBL_SEARCH'), array('action' => 'SEARCH', 'form' => $form->getProp(CLX_PROP_ID)))->source();
	$c = new htmlButtonCloseWindow(dict('GEN_LBL_CANCEL'));
	$buttons .= $c->source();
	$table->addCell($buttons, array(CLX_PROP_CLASS => CLX_CLASS_FOOTERS, CLX_PROP_COLSPAN => CLX_WIDTH_TWO));
	$form->setContents($table->source());
	$contents .= $form->source();

	// Search Procedure
	if (isset(UserRequest::$post['confirm'])) {
	    $table = new htmlTable(CLX_WIDTH_TWO);
	    $table->setProps(array(CLX_PROP_CLASS => CLX_CLASS_LIST, CLX_PROP_WIDTH => CLX_PROP_100PCT));
	    $criteria = '';
	    if (UserRequest::$post['ANRP_idcode'] <> '') {
		$criteria .= 'UPPER(IDCODE) LIKE ' . singleQuote(uppercase(UserRequest::$post['ANRP_idcode']) . '%');
	    }
	    if (UserRequest::$post['ANRP_idname'] <> '') {
		$criteria .= 'UPPER(IDNAME) LIKE ' . singleQuote(uppercase(clx_change_especial_char_html (UserRequest::$post['ANRP_idname'])) . '%');
	    }
	    if (UserRequest::$post['ANRP_modulecode'] <> '') {
		$criteria .= 'UPPER(MODULECODE) LIKE ' . singleQuote(uppercase(UserRequest::$post['ANRP_modulecode']) . '%');
	    }
	    if (UserRequest::$post['ANRP_querycode'] <> '') {
		$criteria .= 'UPPER(QUERYCODE) LIKE ' . singleQuote(uppercase(UserRequest::$post['ANRP_querycode']) . '%');
	    }
       	    if (UserRequest::$post['ANRP_report_type'] <> '') {
		$criteria .= 'REPORT_TYPE = ' . singleQuote(UserRequest::$post['ANRP_report_type'] );
	    }

	    $qty = ClxCommon::$db->oneExistsRecords($this->tablename, $criteria);
	    if ($qty == CLX_NUM_ZERO) {
		$table->addCell(dict('TOOL_GEN_ERR_NOMATCH'), array(CLX_PROP_CLASS => CLX_CLASS_ALERT, CLX_PROP_COLSPAN => CLX_WIDTH_TWO));
	    } else {
		$table->addCell(dict('TOOL_GEN_MSG_FOUNDRECORDS'), array(CLX_PROP_CLASS => CLX_CLASS_SUBTITLE, CLX_PROP_COLSPAN => CLX_WIDTH_TWO));
		$records = ClxCommon::$db->getRecords($this->tablename, $criteria);
		$i = CLX_INIT_ZERO;
		while ($record = ClxCommon::$db->getNextRecord($records, ASSOCIATIVE)) {
		    $js = 'javascript:setLookupValue(' . quote(UserRequest::$request[CLX_PROP_TARGET]) . ', ' . quote($record['IDCODE']) . ');';
		    $table->addCells(($i % CLX_NUM_TWO == CLX_NUM_ZERO) ? CLX_CLASS_DATA : CLX_CLASS_DATAALT, array('<page t=an_reports></page><a href=' . singleQuote($js) . '>' . $record['IDCODE'] . '</a>', $record['IDNAME']));
		    $i++;
		}
	    }
	    $contents .= $table->source();
	}
	return $contents;
    }

    /**
     * Builds confirm delete form for tool
     * @param obj $table
     * @param array $record
     */
    function confirmDeleteFill(&$table, $record) {
	$table->addData(dict('GEN_LBL_CODE') . ':', $record['ANRP_idcode']);
	$table->addData(dict('GEN_LBL_NAME') . ':', $record['ANRP_idname']);
    }

    /**
     * Build tool display form
     * @param $idlist
     * @return string
     */
    function viewForm($idlist) {
	$id = $idlist[count($idlist) - CLX_NUM_ONE]; // get the last one
	$record = $this->loadToolRecord($id);

	pushHistory($this->code, 'DISPLAY', $idlist);
	$contents = $this->viewFormFill($record);
	$form = new htmlForm(CLX_PROP_ACTION, 'tool.php');
	$form->setProp(CLX_PROP_ID, CLX_CLASS_FOOTERS);
	$form->addArgument('ac', 'NOT_DEFINED');
	$form->addArgument(CLX_PROP_TO, $this->code);
	$form->addArgument('idlist', toString($idlist));
	$form->addArgument('next', CLX_NUM_ONE);
	$table = new htmlTable(CLX_WIDTH_TWO);
	$table->setProps(array(CLX_PROP_CLASS => CLX_CLASS_FORM, CLX_PROP_WIDTH => CLX_PROP_100PCT));
	$c = new htmlButtonReturn();
	$buttons = $c->source();
	$c = new htmlFormSubmit(dict('GEN_LBL_VIEW'), 'REPORT', $form->getProp(CLX_PROP_ID));
	$buttons .= $c->source();
	if (count($idlist) > CLX_NUM_ONE) {
	    $c = new htmlFormSubmit(dict('GEN_VIEW_NEXT_SOMETHING', dict('GEN_LBL_RECORD')), 'DISPLAY', $form->getProp(CLX_PROP_ID));
	    $buttons .= $c->source();
	}
	$table->addCell($buttons, array(CLX_PROP_CLASS => CLX_CLASS_FOOTERS, CLX_PROP_COLSPAN => CLX_WIDTH_TWO));
	$form->setContents($table->source());
	$contents .= $form->source();
	return $contents;
    }

    /**
     * Build display form contents
     * @param array $record
     * @return string
     */
    function viewFormFill($record) {
		if (isset($record['ANRP_ID']))
            $record['ANRP_id']=$record['ANRP_ID'];
		
	addHistory('reportcode', $record['ANRP_idcode']);
	addHistory('ownercode', $this->code);
	addHistory('ownerid', $record ['ANRP_id']);
	$contents = '';
	$table = new htmlTable(CLX_WIDTH_TWO);
	$table->setProps(array(CLX_PROP_CLASS => CLX_CLASS_FORM, CLX_PROP_WIDTH => CLX_PROP_100PCT));
	$table->addCell(dict('TOOL_ANRP_TITLE_VIEW') . ': ' . $record['ANRP_idname'], array(CLX_PROP_CLASS => CLX_CLASS_TITLE, CLX_PROP_COLSPAN => CLX_WIDTH_TWO));
	$table->addData(dict('GEN_LBL_CODE') . ':', $record['ANRP_idcode']);
	$table->addData(dict('GEN_LBL_NAME') . ':', $record['ANRP_idname']);
	$table->addData(dict('GEN_LBL_VERSION') . ':', $record['ANRP_version'] . '-' . $record['ANRP_version_counter']);
	$table->addData(dict('GEN_LBL_DESCRIPTION') . ':', xFormat::text($record['ANRP_description']));
	$table->addData(dict('GEN_LBL_MODULECODE') . ':', $record['ANRP_modulecode']);
	$table->addData(dict('GEN_LBL_REPORT_TYPE') . ':', dict($this->reportTypes[$record['ANRP_report_type']]));
	$table->addData(dict('GEN_LBL_ACTIVE') . ':', dict($this->yesno[$record['ANRP_active']]));
	if ($record['ANRP_report_type'] == CLX_NUM_ZERO) {
	    $table->addData(dict('GEN_LBL_QUERYCODE') . ':', $record['ANRP_querycode']);
	} elseif ($record['ANRP_report_type'] == CLX_NUM_ONE) {
	    $table->addData(dict('GEN_LBL_TEMPLATE_REF') . ':', $record['ANRP_template_ref']);
	} else {
	    $table->addData(dict('GEN_LBL_TEMPLATE_REF'), '');
	}
	$table->addData(dict('GEN_LBL_HEADERCODE') . ':', $record['ANRP_headercode']);
	$table->addData(dict('GEN_LBL_FOOTERCODE') . ':', $record['ANRP_footercode']);
	$table->addData(dict('GEN_LBL_PDF_PRINT_ORIENTATION') . ':', $record['ANRP_pdf_print_orientation']);
	$table->addData(dict('GEN_LBL_PDF_PRINT_FORMAT') . ':', $record['ANRP_pdf_print_format']);
	$table->addData(dict('GEN_LBL_PDF_PRINT_HEIGHT') . ':', $record['ANRP_pdf_print_height']);
	$table->addData(dict('GEN_LBL_PDF_PRINT_WIDTH') . ':', $record['ANRP_pdf_print_width']);
	$table->addData(dict('GEN_LBL_PDF_PRINT_MARGIN_BREAK') . ':', $record['ANRP_pdf_print_margin_break']);
	$table->addData(dict('GEN_LBL_PDF_RUN_BATCH') . ':', $record['ANRP_pdf_run_batch']);
	if ($record['ANRP_report_type'] == CLX_NUM_TWO) {
	    $table->addData(dict('GEN_LBL_CONTROL') . ':', $this->getControlsJasper($record, false));
	}
	$contents .= $table->source();
	if (intval($record['ANRP_report_type']) === CLX_NUM_ONE) {
	    $ANRQ = &getTool('ANRQ');
	    $contents .= $ANRQ->makepanel($record['ANRP_idcode']);
	}
	if ($record['ANRP_report_type'] != CLX_NUM_TWO) {
	    $ANRG = &getTool('ANRG');
	    $contents .= $ANRG->makepanel($record['ANRP_idcode']);
	    $ANRC = &getTool('ANRC');
	    $contents .= $ANRC->makepanel($record['ANRP_idcode']);
	}
	//Attaching event options
	$EVNT = &getTool('EVNT');
	$contents .= $EVNT->makepanel($this->code, $record ['ANRP_id']);
	$table = new htmlTable(CLX_WIDTH_TWO);
	$table->setProps(array(CLX_PROP_CLASS => CLX_CLASS_FORM, CLX_PROP_WIDTH => CLX_PROP_100PCT));
	$table->addData(dict('GEN_LBL_CDATE') . ':', $record['ANRP_cdate']);
	$table->addData(dict('GEN_LBL_MDATE') . ':', $record['ANRP_mdate']);
	$table->addData(dict('GEN_LBL_USERNAME') . ':', $record['ANRP_username']);
	$contents .= buildPanelInterface(dict('GEN_TTL_SYSTEMINFO'), 'SYSTEMINFO', $table->source(), CLX_NUM_MINUS_1);
	return $contents;
    }

    /**
     * Adds Filter Form contents
     * @param array $msg
     * @param obj $table
     */
    function filterFormFill($msg, &$table) {
	if (!array_key_exists('ANRP_idcode', UserRequest::$post))
	    UserRequest::$post['ANRP_idcode'] = '';
	if (!array_key_exists('ANRP_idname', UserRequest::$post))
	    UserRequest::$post['ANRP_idname'] = '';
	if (!array_key_exists('ANRP_modulecode', UserRequest::$post))
	    UserRequest::$post['ANRP_modulecode'] = '';
	if (!array_key_exists('ANRP_querycode', UserRequest::$post))
	    UserRequest::$post['ANRP_querycode'] = '';

	$table->addField($msg, dict('GEN_LBL_CODE') . ':', 'ANRP_idcode', UserRequest::$post['ANRP_idcode'], CLX_NUM_16, CLX_NUM_16);
	$table->addField($msg, dict('GEN_LBL_NAME') . ':', 'ANRP_idname', UserRequest::$post['ANRP_idname'], CLX_NUM_32, CLX_NUM_64);
	$table->addField($msg, dict('GEN_LBL_MODULECODE') . ':', 'ANRP_modulecode', UserRequest::$post['ANRP_modulecode'], CLX_NUM_FOUR, CLX_NUM_FOUR);
	$table->addField($msg, dict('GEN_LBL_QUERYCODE') . ':', 'ANRP_querycode', UserRequest::$post['ANRP_querycode'], CLX_NUM_16, CLX_NUM_16);
	$sort = new HtmlSelect('ANRP_sort');
	$sort->addItem('', '', true);
	$sort->addItem('Module Code', 'modulecode', false);
	$sort->addItem('Query Code', 'querycode', false);
	$sort->addItem('Code', 'idcode', false);
	$sort->addItem('Name', 'idname', false);
	$table->addData(dict('GEN_LBL_ORDERBY') . ':', $sort->source());
    }

    /**
     * Register filter information on user session
     */
    function setfilter() {
	$filter = '';
	$boolean = '';
	if (!empty(UserRequest::$post['ANRP_idcode'])) {
	    $filter .= $boolean . ' UPPER(IDCODE) LIKE ' . singleQuote(uppercase(UserRequest::$post['ANRP_idcode']) . '%');
	    $boolean = 'AND';
	}
	if (!empty(UserRequest::$post['ANRP_idname'])) {
	    $filter .= $boolean . ' UPPER(IDNAME) LIKE ' . singleQuote(clx_change_especial_char_html (uppercase(UserRequest::$post['ANRP_idname'])) . '%');
	    $boolean = 'AND';
	}
	if (!empty(UserRequest::$post['ANRP_modulecode'])) {
	    $filter .= $boolean . ' UPPER(MODULECODE) LIKE ' . singleQuote(uppercase(UserRequest::$post['ANRP_modulecode']) . '%');
	    $boolean = 'AND';
	}
	if (!empty(UserRequest::$post['ANRP_querycode'])) {
	    $filter .= $boolean . ' UPPER(QUERYCODE) LIKE ' . singleQuote(uppercase(UserRequest::$post['ANRP_querycode']) . '%');
	    $boolean = 'AND';
	} 
       	if (!empty(UserRequest::$post['ANRP_report_type'])) { 
	    $filter .= $boolean . ' REPORT_TYPE = ' . singleQuote(UserRequest::$post['ANRP_report_type'] );
	    $boolean = 'AND';
	}
    	$sortby = (array_key_exists('ANRP_sort', UserRequest::$post)) ? UserRequest::$post['ANRP_sort'] : CLX_CHAR_EMPTY_STRING;
		$this->setFilterKeys($filter, $sortby, CLX_NUM_ZERO);
    }

    /**
     * Loads current record on selected id
     *
     * @param integer $id
     * @return array
     */
    function loadToolRecord($recordid) {
	$DICT = &getTool('DICT');
	$values = ClxCommon::$db->getRecord($this->tablename, 'ID=' . $recordid);
	$record = array();
	$record['ANRP_id'] = $recordid;
	$record['ANRP_idcode'] = $values['IDCODE'];
	$record['ANRP_idname'] = $values['IDNAME'];
	$record['ANRP_version'] = $values['VERSION'];
	$record['ANRP_version_counter'] = $values['VERSION_COUNTER'];
	$record['ANRP_cdate'] = xFormat::date($values['CDATE']);
	$record['ANRP_mdate'] = xFormat::date($values['MDATE']);
	$record['ANRP_username'] = $values['USERNAME'];
	$record['ANRP_description'] = $values['DESCRIPTION'];
	$record['ANRP_modulecode'] = $values['MODULECODE'];
	$record['ANRP_querycode'] = $values['QUERYCODE'];
	$record['ANRP_report_type'] = $values['REPORT_TYPE'];
	$record['ANRP_search_form'] = $values['SEARCH_FORM'];
	$record['ANRP_hide_gran_total'] = (empty($values['HIDE_GRAN_TOTAL'])) ? CLX_NUM_ZERO : CLX_NUM_ONE;
	$record['ANRP_show_detail'] = $values['SHOW_DETAIL'];
	$record['ANRP_colapsable'] = $values['COLAPSABLE'];
	$record['ANRP_headercode'] = $values['HEADERCODE'];
	$record['ANRP_footercode'] = $values['FOOTERCODE'];
	$record['ANRP_pdf_print_orientation'] = $values['PDFPRINTORIENTATION'];
	$record['ANRP_pdf_print_format'] = $values['PDFPRINTFORMAT'];
	$record['ANRP_pdf_print_height'] = $values['PDFPRINTHEIGHT'];
	$record['ANRP_pdf_print_width'] = $values['PDFPRINTWIDTH'];
	$record['ANRP_pdf_print_margin_break'] = ($values['PDFPRINTMARGINBREAK'] == '') ? $this->defaultRecord('ANRP_pdfprintmarginbreak') : $values['PDFPRINTMARGINBREAK'];
	$record['ANRP_pdf_run_batch'] = ($values['PDFRUNBATCH'] == '') ? $this->defaultRecord('ANRP_pdfrunbatch') : $values['PDFRUNBATCH'];
	if ($values['REPORT_TYPE'] == CLX_NUM_ZERO) {
	    $record['ANRP_template_ref'] = '';
	} else if ($values['REPORT_TYPE'] == CLX_NUM_ONE) {
	    if (empty($values['TEMPLATE_REF'])) {
		$record['ANRP_template_ref'] = dict('GEN_LBL_TEMPLATE') . ' ' . dict('GEN_ERR_FILE_NOT_FOUND');
	    } else {
		$url = 'pop.php?to=' . $this->code . '&ac=DL_TPL&rc=' . $values['IDCODE'] . '&no_menu=1&no_alerts=1';
		$record['ANRP_template_ref'] = '<page t=an_reports></page><a href=' . singleQuote($url) . ' target=' . singleQuote('blank') . '>' . strip_tags($values['TEMPLATE_REF']) . '</a>';
	    }
	} else {
	    if (empty($values['TEMPLATE_REF'])) {
		$record['ANRP_template_ref'] = dict('GEN_LBL_TEMPLATE') . ' ' . dict('GEN_ERR_FILE_NOT_FOUND');
	    } else {
		$url = 'pop.php?to=' . $this->code . '&ac=DL_TPL&rc=' . $values['IDCODE'] . '&no_menu=1&no_alerts=1';
		$record['ANRP_template_ref'] =  $url ;
	    }
	    $record['ANRP_jasper_template_ref'] = $values['TEMPLATE_REF'];
	}
	$record['ANRP_active'] = $values['ACTIVE'];
	$DICT->loadEntry('ANRP_title', (array_key_exists('TITLE', $values)) ? $values['TITLE'] : '', $record);
	$DICT->loadEntry('ANRP_subtitle', (array_key_exists('SUBTITLE', $values)) ? $values['SUBTITLE'] : '', $record);
	return $record;
    }

    /**
     * Update records
     *
     * @return integer
     */
    function saveRecord() {
	$DICT = &getTool('DICT');
	$d = xFormat::getNow('Y-m-d');
	$record = array();
	$record['IDNAME'] = UserRequest::$post['ANRP_idname'];
	$record['VERSION'] = UserRequest::$post['ANRP_version'];
	$record['MDATE'] = $d;
	$record['USERNAME'] = $_SESSION['user_record']['username'];
	$record['DESCRIPTION'] = UserRequest::$post['ANRP_description'];
	$record['MODULECODE'] = UserRequest::$post['ANRP_modulecode'];
	$record['SHOW_DETAIL'] = UserRequest::$post['ANRP_show_detail'];
	$record['COLAPSABLE'] = UserRequest::$post['ANRP_colapsable'];
	$record['QUERYCODE'] = UserRequest::$post['ANRP_querycode'];
	$record['REPORT_TYPE'] = UserRequest::$post['ANRP_report_type'];
	$record['SEARCH_FORM'] = UserRequest::$post['ANRP_search_form'];
	$record['HIDE_GRAN_TOTAL'] = (empty(UserRequest::$post['ANRP_hide_gran_total'])) ? CLX_NUM_ZERO : CLX_NUM_ONE;
	$record['TITLE'] = $DICT->saveEntry('ANRP_title', UserRequest::$post);
	$record['SUBTITLE'] = $DICT->saveEntry('ANRP_subtitle', UserRequest::$post);
	$record['TEMPLATE_REF'] = UserRequest::$post['ANRP_template_ref'];
	$record['ACTIVE'] = UserRequest::$post['ANRP_active'];
	$record['HEADERCODE'] = UserRequest::$post['ANRP_headercode'];
	$record['FOOTERCODE'] = UserRequest::$post['ANRP_footercode'];
	$record['PDFPRINTORIENTATION'] = UserRequest::$post['ANRP_pdf_print_orientation'];
	$record['PDFPRINTFORMAT'] = UserRequest::$post['ANRP_pdf_print_format'];
	$record['PDFPRINTHEIGHT'] = UserRequest::$post['ANRP_pdf_print_height'];
	$record['PDFPRINTWIDTH'] = UserRequest::$post['ANRP_pdf_print_width'];
	$record['PDFPRINTMARGINBREAK'] = UserRequest::$post['ANRP_pdf_print_margin_break'];
	$record['PDFRUNBATCH'] = UserRequest::$post['ANRP_run_batch'];
	if (UserRequest::$post['ANRP_report_type'] == CLX_NUM_TWO) {
	    $record['TEMPLATE_REF'] = UserRequest::$post['ANRP_jasper_template_ref'];
	    if (!empty($_FILES['ANRP_template']['name']))
		$record['TEMPLATE'] = file_get_contents($_FILES['ANRP_template']['tmp_name']);
	} else if (!empty(UserRequest::$files['ANRP_template']['tmp_name'])) {
	    $record['TEMPLATE_REF'] = substr(basename(UserRequest::$files['ANRP_template']['name']), CLX_NUM_ZERO, CLX_NUM_64);
	    $record['TEMPLATE'] = file_get_contents(UserRequest::$files['ANRP_template']['tmp_name']);
	} else {
	    // Solve problem when record is modified without loading a new template
	    $record['TEMPLATE_REF'] = ClxCommon::$db->getField($this->tablename, 'TEMPLATE_REF', 'ID=' . UserRequest::$post['ANRP_id']);
	}
		if (isset(UserRequest::$post['ANRP_ID']))
            UserRequest::$post['ANRP_id']=UserRequest::$post['ANRP_ID'];

	ClxCommon::$db->updateRecord($this->tablename, $record, 'ID=' . UserRequest::$post['ANRP_id']);
	$this->versionIncrease('ID', UserRequest::$post['ANRP_id']);
    }

    /**
     * Creates record
     *
     * @return integer
     */
    function createRecordVoid_() {
	ClxFreeReturn($this->createRecord_());
    }

    function createRecord_() {
	$DICT = &getTool('DICT');
	$d = xFormat::getNow('Y-m-d');
	$record = array();
	$record['IDCODE'] = UserRequest::$post['ANRP_idcode'];
	$record['IDNAME'] = UserRequest::$post['ANRP_idname'];
	$record['VERSION'] = UserRequest::$post['ANRP_version'];
	$record['VERSION_COUNTER'] = '0';
	$record['CDATE'] = $d;
	$record['MDATE'] = $d;
	$record['USERNAME'] = $_SESSION['user_record']['username'];
	$record['DESCRIPTION'] = UserRequest::$post['ANRP_description'];
	$record['REPORT_TYPE'] = UserRequest::$post['ANRP_report_type'];
	$record['QUERYCODE'] = UserRequest::$post['ANRP_querycode'];
	$record['SEARCH_FORM'] = UserRequest::$post['ANRP_search_form'];
	$record['MODULECODE'] = UserRequest::$post['ANRP_modulecode'];
	$record['SHOW_DETAIL'] = UserRequest::$post['ANRP_show_detail'];
	$record['HIDE_GRAN_TOTAL'] = (empty(UserRequest::$post['ANRP_hide_gran_total'])) ? CLX_NUM_ZERO : CLX_NUM_ONE;
	$record['COLAPSABLE'] = UserRequest::$post['ANRP_colapsable'];
	$record['TEMPLATE_REF'] = UserRequest::$post['ANRP_template_ref'];
	$record['ACTIVE'] = UserRequest::$post['ANRP_active'];
	$record['HEADERCODE'] = UserRequest::$post['ANRP_headercode'];
	$record['FOOTERCODE'] = UserRequest::$post['ANRP_footercode'];
	$record['PDFPRINTORIENTATION'] = UserRequest::$post['ANRP_pdf_print_orientation'];
	$record['PDFPRINTFORMAT'] = UserRequest::$post['ANRP_pdf_print_format'];
	$record['PDFPRINTHEIGHT'] = UserRequest::$post['ANRP_pdf_print_height'];
	$record['PDFPRINTWIDTH'] = UserRequest::$post['ANRP_pdf_print_width'];
	$record['PDFPRINTMARGINBREAK'] = UserRequest::$post['ANRP_pdf_print_margin_break'];
	$record['PDFRUNBATCH'] = UserRequest::$post['ANRP_run_batch'];
	if (UserRequest::$post['ANRP_report_type'] == 2) {
	    $record['TEMPLATE_REF'] = UserRequest::$post['ANRP_jasper_template_ref'];
	    if (!empty($_FILES['ANRP_template']['tmp_name']))
		$record['TEMPLATE'] = file_get_contents($_FILES['ANRP_template']['tmp_name']);
	} else if (!empty(UserRequest::$files['ANRP_template']['tmp_name'])) {
	    $record['TEMPLATE_REF'] = substr(basename(UserRequest::$files['ANRP_template']['name']), CLX_NUM_ZERO, CLX_NUM_64);
	    $record['TEMPLATE'] = file_get_contents(UserRequest::$files['ANRP_template']['tmp_name']);
	} else {
	    // Solve problem when record is modified without loading a new template
        if (isset(UserRequest::$post['ANRP_ID']))
            UserRequest::$post['ANRP_id']=UserRequest::$post['ANRP_ID'];

	    $record['TEMPLATE_REF'] = ClxCommon::$db->getField($this->tablename, 'TEMPLATE_REF', 'ID=' . UserRequest::$post['ANRP_id']);
	}

	$record['TITLE'] = $DICT->createEntry('ANRP_title', UserRequest::$post);
	$record['SUBTITLE'] = $DICT->createEntry('ANRP_subtitle', UserRequest::$post);
	$recordid = ClxCommon::$db->createRecord($this->tablename, $record, false);
	$DBMO = &getTool('DBMO');
	$DBMO->versionIncrease('IDCODE', $record['MODULECODE']);
	return $recordid;
    }

    /**
     * Duplicates record
     *
     * @return integer
     */
    function duplicateRecord() {
        if (isset(UserRequest::$post['ANRP_ID']))
            UserRequest::$post['ANRP_id']=UserRequest::$post['ANRP_ID'];

	$originalreportid = UserRequest::$post['ANRP_id'];
	$originalreportrec = ClxCommon::$db->getRecord($this->tablename, 'ID=' . $originalreportid);
	$originalreportcode = $originalreportrec['IDCODE'];
	$reportid = $this->createRecord_();
	$reportcode = UserRequest::$post['ANRP_idcode'];
	$qry_code_convertion = array();
	$records = ClxCommon::$db->getRecords('CLX_AN_REPORT_QUERIES', 'REPORTCODE=' . singleQuote($originalreportcode));
	while ($record = ClxCommon::$db->getNextRecord($records, ASSOCIATIVE)) {
	    $originalqryid = $record['IDCODE'];
	    unset($record['ID']);
	    $record['REPORTCODE'] = $reportcode;
	    $qryid = ClxCommon::$db->createRecord('CLX_AN_REPORT_QUERIES', $record, false);
	    $qryrec = ClxCommon::$db->getRecord('CLX_AN_REPORT_QUERIES', 'ID=' . $qryid);
	    $newqryid = $this->generate_qry_code($reportid);
	    $qry_code_convertion[$originalqryid] = $newqryid;
	    $qryrec['IDCODE'] = $newqryid;
	    ClxCommon::$db->updateRecord('CLX_AN_REPORT_QUERIES', $qryrec, 'ID=' . $qryid);
	    $funcs = ClxCommon::$db->getRecords('CLX_AN_REP_QRY_CRITS', 'REPQRYCODE=' . singleQuote($originalqryid));
	    while (($funcrec = ClxCommon::$db->getNextRecord($funcs))) {
		unset($funcrec['ID']);
		$funcrec['REPQRYCODE'] = $qry_code_convertion[$funcrec['REPQRYCODE']];
		$funcrec['REPQRYCODE2'] = (!empty($qry_code_convertion[$funcrec['REPQRYCODE2']])) ? $qry_code_convertion[$funcrec['REPQRYCODE2']] : '';
		ClxCommon::$db->createRecord('CLX_AN_REP_QRY_CRITS', $funcrec, false);
	    }
	}
	$records = ClxCommon::$db->getRecords('CLX_AN_REPORT_GROUPS', 'REPORTCODE=' . singleQuote($originalreportcode));
	while ($record = ClxCommon::$db->getNextRecord($records, ASSOCIATIVE)) {
	    $originalgroupid = $record['ID'];
	    unset($record['ID']);
	    $record['REPORTCODE'] = $reportcode;
	    $record['REPQRYCODE'] = (!empty($qry_code_convertion[$record['REPQRYCODE']])) ? $qry_code_convertion[$record['REPQRYCODE']] : '';
	    $groupid = ClxCommon::$db->createRecord('CLX_AN_REPORT_GROUPS', $record, false);
	    $funcs = ClxCommon::$db->getRecords('CLX_AN_REPORT_GROUP_FUNC', 'GROUPID=' . $originalgroupid);
	    while (($funcrec = ClxCommon::$db->getNextRecord($funcs))) {
		unset($funcrec['ID']);
		$funcrec['GROUPID'] = $groupid;
		ClxCommon::$db->createRecord('CLX_AN_REPORT_GROUP_FUNC', $funcrec, false);
	    }
	}
	$records = ClxCommon::$db->getRecords('CLX_AN_REPORT_COLUMNS', 'REPORTCODE=' . singleQuote($originalreportcode));
	while ($record = ClxCommon::$db->getNextRecord($records, ASSOCIATIVE)) {
	    unset($record['ID']);
	    $record['REPORTCODE'] = $reportcode;
	    $record['REPQRYCODE'] = (!empty($qry_code_convertion[$record['REPQRYCODE']])) ? $qry_code_convertion[$record['REPQRYCODE']] : '';
	    ClxCommon::$db->createRecord('CLX_AN_REPORT_COLUMNS', $record, false);
	}

	return $reportid;
    }

    /**
     * Generate CLX_AN_REPORT_QUERIES new idcode
     *
     * @param integer $qryid
     * @return string
     */
    function generate_qry_code($qryid) {
	$qryid000 = substr(str_repeat('0', $this->QryIdcodeEndPos) . $qryid, -$this->QryIdcodeEndPos);
	$qry_code_consecutive = CLX_INIT_ZERO;
	$newqryid = null;
	while ($qry_code_consecutive < CLX_NUM_THOUSAND) {
	    $qry_code_consecutive += CLX_NUM_ONE;
	    $newqryid = str_replace('<!reportid>', $qryid000, str_replace('<!consecutive>', substr(str_repeat('0', $this->QryIdcodeEndPos) . $qry_code_consecutive, -$this->QryIdcodeEndPos), $this->QryCodeMask)
	    );
	    if (ClxCommon::$db->oneExistsRecords('CLX_AN_REPORT_QUERIES', 'IDCODE=' . singleQuote($newqryid)) == CLX_NUM_ZERO)
		break;
	}
	return $newqryid;
    }

    /**
     * Deletes record on selected id
     *
     * @param integer $id
     */
    function deleteRecordSk($id) {
	$reportrec = ClxCommon::$db->getRecord($this->tablename, 'ID=' . $id);
	$reportcode = $reportrec['IDCODE'];
	$rs = ClxCommon::$db->getRecords('CLX_AN_REPORT_GROUPS', 'REPORTCODE=' . singleQuote($reportcode));
	while ($r = ClxCommon::$db->getNextRecord($rs, ASSOCIATIVE)) {
	    ClxCommon::$db->deleteRecords('CLX_AN_REPORT_GROUP_FUNC', 'GROUPID=' . $r['ID']);
	}
	$rs = ClxCommon::$db->getRecords('CLX_AN_REPORT_QUERIES', 'REPORTCODE=' . singleQuote($reportcode));
	while ($r = ClxCommon::$db->getNextRecord($rs, ASSOCIATIVE)) {
	    ClxCommon::$db->deleteRecords('CLX_AN_REP_QRY_CRITS', 'REPQRYCODE=' . singleQuote($r['IDCODE']));
	}
	$DICT = &getTool('DICT');
	$DICT->deleteEntry($reportrec['TITLE']);
	$DICT->deleteEntry($reportrec['SUBTITLE']);
	ClxCommon::$db->deleteRecords('CLX_AN_REPORT_QUERIES', 'REPORTCODE=' . singleQuote($reportcode));
	ClxCommon::$db->deleteRecords('CLX_AN_REPORT_GROUPS', 'REPORTCODE=' . singleQuote($reportcode));
	ClxCommon::$db->deleteRecords('CLX_AN_REPORT_COLUMNS', 'REPORTCODE=' . singleQuote($reportcode));
	ClxCommon::$db->deleteRecords('CLX_AN_REPORTS', 'IDCODE=' . singleQuote($reportcode));
	$DBMO = &getTool('DBMO');
	$DBMO->versionIncrease('IDCODE', $reportrec['MODULECODE']);
    }

    /**
     * Defines default record on initializacion
     *
     * @return array
     */
    function defaultRecord($value = '') {
	$record = array();
	$record['ANRP_id'] = CLX_INIT_ZERO;
	$record['ANRP_idcode'] = '';
	$record['ANRP_idname'] = '';
	$record['ANRP_version'] = '1.0.0';
	$record['ANRP_cdate'] = '';
	$record['ANRP_mdate'] = '';
	$record['ANRP_username'] = $_SESSION['user_record']['username'];
	$record['ANRP_description'] = '';
	$record['ANRP_modulecode'] = '';
	$record['ANRP_report_type'] = CLX_INIT_ZERO;
	$record['ANRP_querycode'] = '';
	$record['ANRP_searchform'] = '';
	$record['ANRP_show_detail'] = '1';
	$record['ANRP_hide_gran_total'] = '0';
	$record['ANRP_title'] = '';
	$record['ANRP_title_DICT'] = '';
	$record['ANRP_subtitle'] = '';
	$record['ANRP_subtitle_DICT'] = '';
	$record['ANRP_colapsable'] = '0';
	$record['ANRP_template_ref'] = '';
	$record['ANRP_jasper_template_ref'] = '';
	$record['ANRP_template'] = '1';
	$record['ANRP_headercode'] = '';
	$record['ANRP_footercode'] = '';
	$record['ANRP_pdfprintorientation'] = CLX_NUM_ZERO;
	$record['ANRP_pdfprintformat'] = CLX_NUM_TWO;
	$record['ANRP_pdfprintheight'] = CLX_NUM_ZERO;
	$record['ANRP_pdfprintwidth'] = CLX_NUM_ZERO;
	$record['ANRP_pdfprintmarginbreak'] = 20;
	$record['ANRP_pdfrunbatch'] = CLX_NUM_ONE;
	if ($value == '')
	    return $record;
	else {
	    return $record[$value];
	}
    }

    /**
     * Increase tool version
     * @param string $fieldname
     * @param string $fieldvalue
     */
    function versionIncrease($fieldname, $fieldvalue) {
	parent::versionIncrease($fieldname, $fieldvalue);
	$modulecode = ClxCommon::$db->getField($this->tablename, 'MODULECODE', $fieldname . '=' . singleQuote($fieldvalue));
	$DBMO = &getTool('DBMO');
	$DBMO->versionIncrease('IDCODE', $modulecode);
    }

    /**
     * Returns true if report exists
     * @param string $idcode
     */
    function reportExists(&$idcode) {
	$qty = ClxCommon::$db->oneExistsRecords($this->tablename, 'IDCODE=' . singleQuote($idcode));
	if ($qty > CLX_NUM_ZERO) {
	    $idcode = ClxCommon::$db->getField($this->tablename, 'IDCODE', 'IDCODE=' . singleQuote($idcode));
	}
	return $qty > CLX_NUM_ZERO;
    }

    /**
     * Prompts report format before build
     * @param array $msg
     */
    function askReportFormat(&$msg) {
	$content='';
	if (in_array('window.php', preg_split('/[\/\\\]/',$_SERVER['PHP_SELF']))) {
	    $content=getStyleHtml();
	}
    if (isset(UserRequest::$request['ANRP_ID']))
        UserRequest::$request['ANRP_id']=UserRequest::$request['ANRP_ID'];

	// Start new search form type
	if (isset(UserRequest::$request['ANRP_ID']))
        UserRequest::$request['ANRP_id']=UserRequest::$request['ANRP_ID'];

	if (empty(UserRequest::$request['ANRP_id'])) {pre(UserRequest::$request);
	    return note(dict('GEN_LBL_MISSING_REPORT_ID'));
	}

	$report_rec = ClxCommon::$db->getRecord($this->tablename, 'ID=' . UserRequest::$request['ANRP_id']);
	$access=$this->getReportAccess($report_rec['IDCODE']);
	if (!$access['canread']) {
	    return note(dict('GEN_LBL_REPORT_NOT_ACCESS'));
	}
	if ($report_rec['ACTIVE'] == CLX_NUM_ZERO) {
	    return note(dict('GEN_LBL_REPORT_INACTIVE'));
	}

	$UIFM = &getTool('UIFM');
	$action = /* (!empty(UserRequest::$server['PHP_SELF'])) ? UserRequest::$server['PHP_SELF'] : */ 'window.php';
	$form = new htmlForm(CLX_PROP_ACTION, $action);
	$form->setProp(CLX_PROP_ID, 'theForm');
	$form->addArgument(CLX_PROP_TO, $this->code);
	$form->addArgument('ac', 'REPORT_BUILD');
	$form->addArgument('qy', (array_key_exists('qy', UserRequest::$request)) ? UserRequest::$request['qy'] : CLX_CHAR_EMPTY_STRING);
	$form->addArgument('fm', (array_key_exists('fm', UserRequest::$request)) ? UserRequest::$request['fm'] : CLX_CHAR_EMPTY_STRING);
	$form->addArgument('li', (array_key_exists('li', UserRequest::$request)) ? UserRequest::$request['li'] : CLX_CHAR_EMPTY_STRING);
	$form->addArgument('nofr', (array_key_exists('nofr', UserRequest::$request)) ? UserRequest::$request['nofr'] : CLX_CHAR_EMPTY_STRING);
	$form->addArgument('pfid', (array_key_exists('pfid', UserRequest::$request)) ? UserRequest::$request['pfid'] : CLX_CHAR_EMPTY_STRING);
	$form->addArgument('rf', (array_key_exists('rf', UserRequest::$request)) ? UserRequest::$request['rf'] : CLX_CHAR_EMPTY_STRING);
	$form->addArgument('rfi', (array_key_exists('rfi', UserRequest::$request)) ? UserRequest::$request['rfi'] : CLX_CHAR_EMPTY_STRING);
    if (isset(UserRequest::$request['ANRP_ID']))
        UserRequest::$request['ANRP_id']=UserRequest::$request['ANRP_ID'];

	$form->addArgument('ANRP_id', UserRequest::$request['ANRP_id']);
	$form->addArgument('ANRP_report_type', $report_rec['REPORT_TYPE']);
	$form->addArgument('ANRP_search_form', $report_rec['SEARCH_FORM']);
	$form->addArgument('confirm', CLX_NUM_ONE);
        $form->addArgument(dict('GEN_LBL_SUBMIT2'), $form->getProp(CLX_PROP_ID));
	$form->addArgument('CODE_SIBLING', rand(1,100000000))  ;
        // 2017-11-01-CR-Caso 1003486 - GB-Rep BATCH no arranca
	//$form->addArgument(dict('GEN_LBL_BATCH_ACTION'),'SHOW_PANEL');
	$form->addArgument('BATCH_ACTION','SHOW_PANEL');
	if (!array_key_exists($this->code . '_rf', UserRequest::$request))
	    UserRequest::$request[$this->code . '_rf'] = '';
	$form->toolcode = $this->code;
	$title = 'GEN_LBL_REPORT';

	//Setting form
	$table = new htmlTable(CLX_WIDTH_TWO);
	$table->setProps(array(CLX_PROP_CLASS => CLX_CLASS_FORM, CLX_PROP_WIDTH => CLX_PROP_100PCT));
	$table->addCell(dict($title), array(CLX_PROP_CLASS => CLX_CLASS_TITLE, CLX_PROP_COLSPAN => CLX_WIDTH_TWO));
	$table->addCell(dict('TOOL_ANRP_TITLE_CHOOSE_FORMAT'), array(CLX_PROP_CLASS => CLX_CLASS_SUBTITLE, CLX_PROP_COLSPAN => CLX_WIDTH_TWO));
	if (isset($msg['ANRP_id'])) {
	    $table->addCell($msg['ANRP_id'], array(CLX_PROP_CLASS => CLX_CLASS_ALERT, CLX_PROP_COLSPAN => CLX_WIDTH_TWO));
	}
	$table->addData(dict('GEN_LBL_REPORT'), $report_rec['IDCODE']);
	$table->addData(dict('GEN_LBL_TITLE'), dict($report_rec['TITLE']));
	if (dict($report_rec['SUBTITLE']) <> '')
	    $table->addData(dict('GEN_LBL_SUBTITLE'), dict($report_rec['SUBTITLE']));
	$formatoptions = array();
	$OBJS = &getTool('OBJS');
	$formats = $OBJS->objectList('RPFT');
	if (intval($report_rec['REPORT_TYPE']) === CLX_NUM_ONE) {
	    $rpob = $OBJS->getObject_('RPFT', 'RPWX');
	    $formatoptions['RPWX'] = $rpob->name;
	} else if (intval($report_rec['REPORT_TYPE']) === CLX_NUM_TWO) {
	    $formatoptions = $this->jasperTypes;
	    /* 	foreach($jasper_formats as $val){
	      $rpob = $OBJS->getObject('RPFT', $val);
	      if(!$rpob->show_prompt_format || $val=='RPPP') { //donot place
	      $formatoptions[$val] = $rpob->name;
	      }
	      } */
	} else {
	    $formatoptions['RPHT'] = 'HTML'; //Setting html as first item in list
	    foreach ($formats as $val) {
		$rpob = $OBJS->getObject_('RPFT', $val);
		if (empty($rpob->show_prompt_format)) {
		    continue; //donot place
		}
		$formatoptions[$val] = $rpob->name;
	    }
	}
	$data = array('name' => 'rf', 'options' => $formatoptions);
	$f = new clx_field_list($form, $data);
	$table->addData(dict('GEN_LBL_FORMAT'), $f->editForm_($form, UserRequest::$request, $msg));
	unset($formats, $formatoptions);

	/*
	 * Caso 1002542 - GB-Reporte Fideicomiso Garantia Autos
	 * Caso 1002794 - GB-Reporte Saldo de Patrimonio
	 *
	 * Se agrega pregunta para que el reporte sea calculado en batch
	 */
	$data = array('name' => 'rbat', 'options' => $this->yesno);
	$f = new clx_field_list($form, $data);
	$table->addData(dict('GEN_LBL_BATCH_PRMT'), $f->editForm_($form, UserRequest::$request, $msg));
	if (intval($report_rec['REPORT_TYPE']) === CLX_NUM_TWO and
		$this->getAccess()->isAdmin()) {
	    $data = array('name' => 'rview', 'options' => $this->yesno);
	    $f = new clx_field_list($form, $data);
	    $table->addData(dict('GEN_LBL_VIEW_CTRL'), $f->editForm_($form, UserRequest::$request, $msg));
	}

	//search ask user fields
	$DBQY = &getTool('DBQY');
	$questions = $DBQY->getQuestions($this->getReportQueries($report_rec['IDCODE']));
	if (!empty($questions)) {
	    $table->addCell(dict('TOOL_ANRP_TITLE_SEARCH_FIELDS'), array(CLX_PROP_CLASS => CLX_CLASS_SUBTITLE, CLX_PROP_COLSPAN => CLX_WIDTH_TWO));
	    foreach ($questions as $key => $val) {
		$jsevents = array();
		$val['FIELD_READONLY'] = CLX_NUM_ZERO;
		$val['FIELD_HIDDEN'] = CLX_NUM_ZERO;
		$val['FIELD_FORM_REQUIRED'] = CLX_NUM_ZERO;
		$val['CAPTION'] = '';
		$val['CAPTION_CLASS'] = 'caption';
		$fieldinstance = getToolObject('DBFT', $val['FIELDTYPE']);
		$cell = $fieldinstance->placeEditField($msg, $val['TABLECODE'] . '_' . $val['FIELDNAME'], $val, UserRequest::$request, $form, $jsevents);
		$table->addCell(dict($val['OPERATOR_LBL']), array(CLX_PROP_CLASS => CLX_CLASS_PROMPT));
		$table->addCell($cell->source(), array(CLX_PROP_CLASS => CLX_CLASS_PROMPT));
	    }
	}
	$form->addContents($table->source() . '<br>');
	//handle search form if exists
	if ($report_rec['SEARCH_FORM'] <> '') {
	    $schformrec = ClxCommon::$db->getRecord('CLX_UI_FORMS', 'IDCODE=' . singleQuote($report_rec['SEARCH_FORM']));
	    $cell = $UIFM->userEditForm($schformrec, '', UserRequest::$request, $msg, $form);
	    $form->addContents($cell);
	}

	//Buttons
	//$b = new htmlFormSubmit(dict('GEN_LBL_SUBMIT'), 'REPORT_BUILD', $form->getProp(CLX_PROP_ID));
	// 2017-11-01-CR-Caso 1003486 - GB-Rep BATCH no arranca
	/*
	$code= ' $(\'<iframe frameborder="0" scrolling="no" width="3" height="3" name="formresponse" id="formresponse"></iframe>\').appendTo("body");
		function SubmitForm(){
		    if ($("#ANRP_rbat").val()=="1") {
			document.forms[0].target="formresponse";
			document.forms[0].'.dict('GEN_LBL_BATCH_ACTION').'.value="RUN_BATCH";
			document.forms[0].submit();
			document.forms[0].target="";
			document.forms[0].'.dict('GEN_LBL_BATCH_ACTION').'.value="SHOW_PANEL";
		    }
		    document.forms[0].submit();
		}
	    ';
	*/
	// 2018-12-05 CR - Caso 1003486 - GB-Rep BATCH no arranca
	$ANRJ = &getTool('ANRJ');
	$code= ' $(\'<iframe frameborder="0" scrolling="no" width="3" height="3" name="formresponse" id="formresponse"></iframe>\').appendTo("body");
		function SubmitForm(){
		    if ($("#ANRP_rbat").val()=="1") {' . ($ANRJ->run_in_cron() ? '' : '
//alert("1er submit");
			document.forms[0].target="formresponse";
			document.forms[0].BATCH_ACTION.value="RUN_BATCH";
			document.forms[0].submit();
'). '
//alert("2do submit");
			document.forms[0].target="";
			document.forms[0].BATCH_ACTION.value="SHOW_PANEL";
		    }
		    document.forms[0].submit();
		}
	';
	$content2=setJavaScriptTag($code);
	$b = new htmlButton(dict('GEN_LBL_SUBMIT2'));//, 'REPORT_BUILD', $form->getProp('id'));
	$b->setProp('onclick','SubmitForm()');
	$btn = $b->source();
	$b = new htmlButtonCloseWindow(dict('GEN_LBL_CANCEL'));
	$btn .= $b->source();
	$table = new htmlTable(CLX_WIDTH_ONE);
	$table->addCell($content2.$btn, array(CLX_PROP_CLASS => 'buttons', CLX_PROP_COLSPAN => CLX_WIDTH_ONE));
	$form->addContents($table->source());
	return $content.$form->source();
    }

    /**
     * Tool Edit Form
     * @param string $ac
     * @param string $idlist
     * @param array $record
     * @param array $msg
     * @return string
     */
    function editForm($ac, $idlist, $record, $msg) {

	if (UserRequest::$request['ac'] === 'MODIFY') {
	    $this->editform[CLX_CLASS_TITLE] = 'TOOL_' . $this->code . '_TITLE_MODIFY';
	} else {
	    $this->editform[CLX_CLASS_TITLE] = 'TOOL_' . $this->code . '_TITLE_CREATE';
	}
	$form = new clx_form($this->code, $this->editform);
	return $form->editForm($ac, $idlist, $record, $msg) . setJavaScriptTag('
		$("#ANRP_report_type").change(function() {
		    hideshowinput();
		});
		function hideshowinput() {
		    if($("#ANRP_report_type").val()==2 ) {
			$("#ANRP_show_detail").parent().parent().hide();
			$("#ANRP_colapsable").parent().parent().hide();
			$("#ANRP_hide_gran_total").parent().parent().hide();
			$("#ANRP_pdf_print_orientation").parent().parent().parent().hide();
			$("#ANRP_headercode").parent().parent().parent().hide();
			$( ".panel:eq(0)" ).hide()
			$( ".panel:eq(1)" ).hide()
		    }
		    else {
			$("#ANRP_show_detail").parent().parent().show();
			$("#ANRP_colapsable").parent().parent().show();
			$("#ANRP_hide_gran_total").parent().parent().show();
			$("#ANRP_pdf_print_orientation").parent().parent().parent().show();
			$("#ANRP_headercode").parent().parent().parent().show();
			$( ".panel:eq(0)" ).show();
			$( ".panel:eq(1)" ).show();

		    }
		}
		hideshowinput();
	        $("#ANRP_pdf_print_height").prop("readonly", true);
		$("#ANRP_pdf_print_width").prop("readonly", true);
		$("#ANRP_pdf_print_height").val(0);
		$("#ANRP_pdf_print_width").val(0);
		$("#ANRP_pdf_print_format").val(0);
	    	$("#ANRP_pdf_print_format").change( function() {
		    if ( this.value==5 ) {
		        $("#ANRP_pdf_print_height").prop("readonly", false);
			$("#ANRP_pdf_print_width").prop("readonly", false);
		        $("#ANRP_pdf_print_height").val(800);
			$("#ANRP_pdf_print_width").val(600);
		    }
		    else {
			$("#ANRP_pdf_print_height").prop("readonly", true);
			$("#ANRP_pdf_print_width").prop("readonly", true);
			$("#ANRP_pdf_print_height").val(0);
			$("#ANRP_pdf_print_width").val(0);
		    }
		});
		  ');
    }

    /**
     * Verify Report Questions
     * @param array &$msg
     * @return bool
     */
    function checkReportForm(&$msg) {
	if (empty(UserRequest::$request['confirm'])) {//just ask questions
	    return false;
	}
	if (!empty(UserRequest::$request['ANRP_search_form'])) {
	    $EDIT = &getTool('EDIT');
	    $formrec = ClxCommon::$db->getRecord('CLX_UI_FORMS', 'IDCODE = ' . singleQuote(UserRequest::$request['ANRP_search_form']));
	    $EDIT->checkForm2($msg, $formrec);
	}
	if (empty(UserRequest::$request['ANRP_rf'])) {
	    $msg['ANRP_rf'] = dict('GEN_LBL_MISSING_FORMAT');
	}
    if (isset(UserRequest::$request['ANRP_ID']))
        UserRequest::$request['ANRP_id']=UserRequest::$request['ANRP_ID'];

	if (empty(UserRequest::$request['ANRP_id'])) {
	    $msg['ANRP_id'] = dict('GEN_LBL_MISSING_REPORT_ID');
	}
	return (!empty($msg)) ? false : true;
    }

    /**
     * Builds report
     */
    function buildReport() {
        $reportid = UserRequest::$request['ANRP_id'];
		$reportformat = UserRequest::$request['ANRP_rf'];
        $report = getToolObject('RPFT', $reportformat);
		// 2018-12-05 CR - Caso 1003486 - GB-Rep BATCH no arranca
		$report->output_to_file = $this->output_to_file;
		/*
		 * Caso 1002542 - GB-Reporte Fideicomiso Garantia Autos
		 * Caso 1002794 - GB-Reporte Saldo de Patrimonio
		 *
		 * Se agrega pregunta para que el reporte sea calculado en batch
		 */
		$batchoption = (isset(UserRequest::$request['ANRP_rbat']) ? UserRequest::$request['ANRP_rbat'] : '');
		if ($batchoption === '1') {
			$reportrec = ClxCommon::$db->getRecord('CLX_AN_REPORTS', 'ID=' . singleQuote($reportid));
			$schedule_title = 'ANRE ' . $reportrec['IDNAME'];
			
			// 2018-11-16 CR - Caso 1003486 - GB-Rep BATCH no arranca
			//$report->output_buffer = true;

			$available = true;
			switch ($reportformat) {
				case 'RPPD':
				case 'RPWX':
					$available = false;
					break;

				case 'RPEX':
					$reptype = 'xls';
					$report->file_save = false;
					break;
				case 'RPPP':
					$reptype = 'pdf';
					break;
				case 'RPHT':
				default:
					$reptype = 'html';
					break;
			}
			if ($available === true) {
				$ANRJ = &getTool('ANRJ');
				$ANRJ->renameSourceFile = false;
				$ANRJ->setBatchReports($schedule_title, $reptype, UserRequest::$post);
			}
		}
		
		$this->intTemporaryTables($reportid);
		$this->runReportEvents($reportid);
		return $report->build($reportid);
	}

	private function buildJasperRequestParameters($record, $addcondition = '') {

	$OBJS = &getTool('OBJS');
	$fieldlist = $OBJS->getObject_('DBFT', 'LIST');
	$fieldlook = $OBJS->getObject_('DBFT', 'LOOK');
        switch (kDatabaseType) {
		case 'oci':
        $sql = 'SELECT
                    NVL(PF.TABLECODE,F.TABLECODE) TABLECODE2,
                    FL.*,
                    PF.FIELD_DEFAULT
                FROM CLX_AN_REPORTS R,
		CLX_UI_FORMS F,
		CLX_DB_FIELDS FL,
		CLX_UI_FORM_PANEL_FIELDS PF,
		CLX_UI_FORM_PANELS P
		WHERE
		    F.IDCODE = P.FORMCODE
		    AND FL.FIELDNAME = PF.FIELDNAME
		    AND P.IDCODE = PF.PANELID
		    AND PF.ACTIVE = 1
		    AND P.ACTIVE = 1
		    AND P.FORMCODE = R.SEARCH_FORM
		    AND R.IDCODE = ' .
		singleQuote($record['ANRP_idcode']) . $addcondition . ' ORDER BY PF.FIELD_HIDDEN '; // Oracle
            break;

		default: // MySQL Version all others
	$sql = 'SELECT
                    IFNULL(NULLIF(PF.TABLECODE,""), F.TABLECODE) TABLECODE,
                    FL.*,
                    PF.FIELD_DEFAULT
                FROM CLX_AN_REPORTS R,
		CLX_UI_FORMS F,
		CLX_DB_FIELDS FL,
		CLX_UI_FORM_PANEL_FIELDS PF,
		CLX_UI_FORM_PANELS P
		WHERE
		    F.IDCODE = P.FORMCODE
		    AND FL.FIELDNAME = PF.FIELDNAME
		    AND P.IDCODE = PF.PANELID
		    AND PF.ACTIVE = 1
		    AND P.ACTIVE = 1
		    AND P.FORMCODE = R.SEARCH_FORM
		    AND R.IDCODE = ' .
		singleQuote($record['ANRP_idcode']) . $addcondition . ' ORDER BY PF.FIELD_HIDDEN ';
            break;
	}
	

	$rs = ClxCommon::$db->executeQuery($sql);
	$controls = array();
	$controls['CLX_USERNAME'] = array('FIELDFORMLABEL' => dict('GEN_LBL_USERNAME'), 'FIELDNAME' => 'CLX_USERNAME', 'FIELD_DEFAULT' => $_SESSION['user_record']['username']);
	$controls['CLX_LANGUAGE'] = array('FIELDFORMLABEL' => dict('GEN_LBL_LANGUAGE'), 'FIELDNAME' => 'CLX_LANGUAGE', 'FIELD_DEFAULT' => getCurrentLanguage());
	$controls['CLX_TITLE'] = array('FIELDFORMLABEL' => dict('GEN_LBL_REP_TITLE'), 'FIELDNAME' => 'CLX_TITLE', 'FIELD_DEFAULT' => dict($record['ANRP_title']));
	$controls['CLX_SUBTITLE'] = array('FIELDFORMLABEL' => dict('GEN_LBL_REP_SUBTITLE'), 'FIELDNAME' => 'CLX_SUBTITLE', 'FIELD_DEFAULT' => dict($record['ANRP_subtitle']));
	$controls['CLX_IDCODE'] = array('FIELDFORMLABEL' => dict('GEN_LBL_REP_CODE'), 'FIELDNAME' => 'CLX_IDCODE', 'FIELD_DEFAULT' => $record['ANRP_idcode']);
	$controls['RTBL_REPORT_FORMAT_TYPE'] = array('FIELDFORMLABEL' => dict('RTBL_REPORT_FORMAT_TYPE'), 'FIELDNAME' => 'RTBL_REPORT_FORMAT_TYPE', 'FIELD_DEFAULT' => isset($record['ANRP_rf']) ? $record['ANRP_rf']:"");
	$controls['RTBL_URI_IMAGES'] = array('FIELDFORMLABEL' => dict('RTBL_URI_IMAGES'), 'FIELDNAME' => 'RTBL_URI_IMAGES', 'FIELD_DEFAULT' => @ANRP_JASPER_SERVER_URI_IMAGES."");
	while (($fieldrec = ClxCommon::$db->getNextRecord($rs)) == true) {

	    $fieldname = $fieldrec['TABLECODE'] . '_' . $fieldrec['FIELDNAME'];
	    $val = (array_key_exists($fieldname, $record)) ? $record[$fieldname] : $fieldrec['FIELD_DEFAULT'];
	    if ($fieldrec['FIELDTYPE'] == 'BOOL') {
		$val = (empty($val)) ? CLX_NUM_ZERO : CLX_NUM_ONE;
	    }

	    $controls[$fieldname] = array('FIELDFORMLABEL' => dict($fieldrec['FIELDFORMLABEL']),
		'FIELDNAME' => $fieldname,
		'FIELD_DEFAULT' => $val);
	    $record[$fieldname] = $val;

	    //Formato para
	    $fieldrec['DBQF_FIELD_FORMAT'] = CLX_NUM_ZERO;

	    $fieldrec['FIELD_DEFAULTREPORT'] = $fieldrec['FIELD_DEFAULT'];
	    if ($fieldrec['FIELDTYPE'] == 'LIST') {
		$controls[$fieldname . '_NAME'] = array(
		    'FIELDFORMLABEL' => dict($fieldrec['FIELDFORMLABEL']). ' ' . dict('GEN_LBL_NAME'),
		    'FIELDNAME' => $fieldname . '_NAME',
		    'FIELD_DEFAULT' => $fieldlist->formatField($fieldname, $fieldrec, $record));
		if (trim($controls[$fieldname . '_NAME']['FIELD_DEFAULT']) == '()') {
		    $controls[$fieldname . '_NAME']['FIELD_DEFAULT'] = '';
		}
	    }
		if ($fieldrec['FIELDTYPE'] == 'BOOL' &&  $controls[$fieldname ]['FIELD_DEFAULT'] == '') {
			 $controls[$fieldname ]['FIELD_DEFAULT'] = '0';
	    }
	    if ($fieldrec['FIELDTYPE'] == 'LOOK') {
		$controls[$fieldname . '_NAME'] = array(
		    'FIELDFORMLABEL' => dict($fieldrec['FIELDFORMLABEL']). ' ' . dict('GEN_LBL_NAME'),
		    'FIELDNAME' => $fieldname . '_NAME',
		    'FIELD_DEFAULT' => $fieldlook->formatField($fieldname, $fieldrec, $record));
                $controls[$fieldname . '_NAME']['FIELD_DEFAULT'] = trim(str_replace($controls[$fieldname]['FIELD_DEFAULT'].', ','',$controls[$fieldname . '_NAME']['FIELD_DEFAULT']));
		if (trim($controls[$fieldname . '_NAME']['FIELD_DEFAULT']) == '()') {
		    $controls[$fieldname . '_NAME']['FIELD_DEFAULT'] = '';
		}
	    }
	}
	return $controls;
    }

    /**
     *
     * @param type $record
     * @return type
     */
    private function getControlsJasper($record, $displaywindow = false) {
	$contents = '';
	$table = new htmlTable(CLX_NUM_THREE);
	//$table->setProp(CLX_PROP_CLASS, 'form');
	$table->addCells('headers', array(dict('GEN_LBL_FIELDNAME'), dict('GEN_LBL_PARAMETER'), $displaywindow ? dict('GEN_LBL_VALUE') : dict('GEN_LBL_DEFAULT')));
	$table->setProp(CLX_PROP_CLASS, CLX_CLASS_FORM);
	//$table->addCells('headers', array(dict('GEN_LBL_FIELDNAME'), dict('GEN_LBL_PARAMETER'), $displaywindow ? dict('GEN_LBL_VALUE') : dict('GEN_LBL_DEFAULT')));
	$table->addCells(CLX_CLASS_HEADERS, array(dict('GEN_LBL_FIELDNAME'), dict('GEN_LBL_PARAMETER'), $displaywindow ? dict('GEN_LBL_VALUE') : dict('GEN_LBL_DEFAULT')));
	$controls = $this->buildJasperRequestParameters($record, $addcondition = '');
	foreach ($controls as $key => $value) {
	    $table->addCell($value['FIELDFORMLABEL'], array(CLX_PROP_CLASS => CLX_CLASS_DATA));
	    $table->addCell($value['FIELDNAME'], array(CLX_PROP_CLASS => CLX_CLASS_DATA));
	    $table->addCell($value['FIELD_DEFAULT'], array(CLX_PROP_CLASS => CLX_CLASS_DATA));
	    if ($value['FIELDNAME'] == 'CLX_IDCODE')
		$table->addSeparator(CLX_CLASS_DATA);
	}
	if ($displaywindow) {
	    $contents = note(sprintf('<head><link href="%s" rel="stylesheet" type="text/css"></head>', kPathToSkins . $_SESSION['skin'] . '/styles.css') . $table->source());
	} else {
	    $div = new htmlDiv($this->code, 'height: 120px;overflow: scroll', $table->source());
	    $contents = note($div->source());
	}
	return $contents;
    }

    function buildJasperReport($record = array()) {
		if ( isset($_SESSION["_dEBUG"]) ) { 	
			$tta=fopen(kPathToWebSite."/".kPathToTemp."archivo.txt","a+");
			
			fwrite($tta,chr(13).chr(10)."12-hora:".date("h:i:s d/m/Y").chr(13).chr(10));
			fwrite($tta,chr(13).chr(10)."buildJasperReport=>record: ".var_export($record,true).chr(13).chr(10));
			fwrite($tta,chr(13).chr(10)."buildJasperReport=>UserRequest::request: ".var_export(UserRequest::$request,true).chr(13).chr(10));
			$te=debug_backtrace();

			for ($t=0;$t<18;$t++) {
				if (isset($te[$t]["file"]))
				fwrite($tta,substr($te[$t]["file"].str_repeat(" ",80),0,80).chr(9).$te[$t]["line"].chr(9).$te[$t]["function"].chr(13).chr(10));
				
					

		}
		fclose($tta);
	}	
	if (empty($record))
	    return '';
	if (isset($record['ANRP_ID']))
		$record['ANRP_id']=$record['ANRP_ID'];

	$formrec = ClxCommon::$db->getRecord('CLX_AN_REPORTS', 'ID = ' . singleQuote($record['ANRP_id']));
	$record['TEMPLATE_REF'] = $formrec['TEMPLATE_REF'];
	$record['ANRP_idcode'] = $formrec['IDCODE'];
	$record['ANRP_title'] = $formrec['TITLE'];
	$record['ANRP_subtitle'] = $formrec['SUBTITLE'];

	if (isset($record['ANRP_rview']) and $record['ANRP_rview']) {
	    return $this->getControlsJasper($record, true);
	} else {
	    $batchoption = (isset(UserRequest::$request['ANRP_rbat']) ? UserRequest::$request['ANRP_rbat'] : '');
	    if ($batchoption === '1') { 
			$schedule_title = 'ANRE ' . $formrec['IDNAME'];
			$ANRJ = &getTool('ANRJ');
			$ANRJ->renameSourceFile=false;
			$ANRJ->setBatchGeneral($schedule_title, $record['ANRP_rf'], UserRequest::$post);
	    } else {
			return $this->buildJasperReportprepare($record, false);
	    }
	}
    }

    /**
     * Prepares parameters
     * @param type $record
     * @return type array
     */
    private function buildJasperReportprepare($record = array()) {
		$controls = $this->buildJasperRequestParameters($record, '');
		foreach ($controls as $key => &$value) {
			$value = ($value['FIELD_DEFAULT'] == '') ? null : $value['FIELD_DEFAULT'];
		}
		if (isset($record['ANRP_ID']))
            $record['ANRP_id']=$record['ANRP_ID'];
        
		return $this->buildRunJasperReport($record['ANRP_id'], $record['ANRP_rf'], $record['TEMPLATE_REF'], $controls);
    }

    function buildRunJasperReport($id, $format = 'html', $report_name = '', $controls = array()) {
		if ( isset($_SESSION["_dEBUG"]) ) { 
			$tta=fopen(kPathToWebSite."/".kPathToTemp."archivo.txt","a+");
			
			fwrite($tta,chr(13).chr(10)."3-hora:".date("h:i:s d/m/Y").chr(13).chr(10));
			fwrite($tta,chr(13).chr(10).var_export($controls,true).chr(13).chr(10));	
			$te=debug_backtrace();

			for ($t=0;$t<18;$t++) {
				if (isset($te[$t]["file"]))
				fwrite($tta,substr($te[$t]["file"].str_repeat(" ",80),0,80).chr(9).$te[$t]["line"].chr(9).$te[$t]["function"].chr(13).chr(10));
			}
			fclose($tta);
		}	
        if (!defined('kPathToJasperPHP'))
			define('kPathToJasperPHP', '../extlibs/php/jasperPHP/');
        if (!defined('JASPERCLIENT_ROOT'))
			define('JASPERCLIENT_ROOT', kPathToWebSite . kPathToJasperPHP . 'src/');
	include (kPathToJasperPHP . 'autoload.dist.php');
	//if ($report_name == '')
	if (trim($report_name) === '')
	    return dict('Notnamereport');
	$server_path = ANRP_JASPER_SERVER_URI . '/' . ANRP_JASPER_SERVER_PATH;
    $report_name = ANRP_JASPER_FOLDER_REPORT . '/' . $report_name;
	$c = new Client($server_path, ANRP_JASPER_SERVER_USER, ANRP_JASPER_SERVER_PASS, '');
	$attachmentsPrefix = null;
	$report = $c->reportService()->runReport($report_name, $format, null, $attachmentsPrefix, $controls,ANRP_JASPER_CERTIFICATE);

	//pre($controls);
    //echo ANRP_JASPER_CERTIFICATE;        
	/// BUSCA TODOS LOS TIPOS DE SALIDA NO ENTRA EN EL IF CUANDO DEVUELVE 0 ESE ES HTML
	$keys = array_keys($this->jasperTypes);
	if (is_array($report)) {
	    //$table = new htmlTable(2);
	    //$table->setProp(CLX_PROP_CLASS, 'form');
	    $table = new htmlTable(CLX_WIDTH_TWO);
	    $table->setProp(CLX_PROP_CLASS, CLX_CLASS_FORM);
	    //$table->addCells('headers', array(dict('GEN_LBL_ERROR'), dict('GEN_LBL_VALUE')));
	    $table->addCells(CLX_CLASS_HEADERS, array(dict('GEN_LBL_ERROR'), dict('GEN_LBL_VALUE')));
	    $table->addCell(dict('GEN_LBL_MESSAGE'), array(CLX_PROP_CLASS=>CLX_CLASS_DATA));
            //if ($report[0]=='401') {
            if ($report[CLX_INIT_ZERO]=='401') {
                    $table->addCell('No autorizado', array(CLX_PROP_CLASS=>CLX_CLASS_DATA));
                    $table->addCell(dict('GEN_LBL_ERRORCODE'), array(CLX_PROP_CLASS=>CLX_CLASS_DATA));
                    $table->addCell('401', array(CLX_PROP_CLASS=>CLX_CLASS_DATA));
            }
            else {
//pre($report);
                //if(isset($report[2]->message)) {
                if(isset($report[CLX_WIDTH_TWO]->message)) {
                    //$table->addCell($report[2]->message, array(CLX_PROP_CLASS=>CLX_CLASS_DATA));
                    $table->addCell($report[CLX_WIDTH_TWO]->message, array(CLX_PROP_CLASS=>CLX_CLASS_DATA));
                    $table->addCell(dict('GEN_LBL_ERRORCODE'), array(CLX_PROP_CLASS=>CLX_CLASS_DATA));
                    //$table->addCell($report[2]->errorCode, array(CLX_PROP_CLASS=>CLX_CLASS_DATA));
                    $table->addCell($report[CLX_WIDTH_TWO]->errorCode, array(CLX_PROP_CLASS=>CLX_CLASS_DATA));
                //} elseif(isset($report[3])) {
                } elseif(isset($report[CLX_WIDTH_THREE])) {
                    $table->addCell('CURL Error', array(CLX_PROP_CLASS=>CLX_CLASS_DATA));
                    $table->addCell(dict('GEN_LBL_ERRORCODE'), array(CLX_PROP_CLASS=>CLX_CLASS_DATA));
                    //$table->addCell($report[3], array(CLX_PROP_CLASS=>CLX_CLASS_DATA));
                    $table->addCell($report[CLX_WIDTH_THREE], array(CLX_PROP_CLASS=>CLX_CLASS_DATA));
                } else {
                    $table->addCell('Unknown Error', array(CLX_PROP_CLASS=>CLX_CLASS_DATA));
                    $table->addCell(dict('GEN_LBL_ERRORCODE'), array(CLX_PROP_CLASS=>CLX_CLASS_DATA));
                    $table->addCell('[CR-010]', array(CLX_PROP_CLASS=>CLX_CLASS_DATA));
                }
            }    

			return note(sprintf('<head><link href="%s" rel="stylesheet" type="text/css"></head>', kPathToSkins . $_SESSION['skin'] . '/styles.css') . $table->source());
		
		}
		else if (array_search($format, $keys)) {
			if (isset($controls['view_display']) && $controls['view_display']){
				header('Content-type: application/' . $format);
				print $report;
			}else{
                //ob_clean();
				sendAnyFile('report', $format, 'application/' . $format, 'public', strlen($report), 'File Transfer', 'binary', $format);
				
			}
			return $report;
		} else if ($format == 'html') {
            $images = $c->reportService()->getReportImages();
			if (count($images) > CLX_NUM_ZERO) {
			foreach ($images as $key => $value) {
				$type = $this->getImgType($value);
				$report = str_replace($key, 'data:image/' . $type . ';base64,' . base64_encode($value), $report);
			}
			}
			//$matches[1]
			//echo '<textarea cols=40 rows=40>'.$report.'</textarea>';
			//$file = 'http://localhost:8080/'.$matches[1][CLX_INIT_ZERO];
			//$c->reportService()->getReportAttachments($file);
			//pre($c->reportService()->getReportAttachments($file));
			//preg_replace($pattern, 'data/files/logoweb_bancorp-01.png',$report);
			//$matches[1]
			//echo '<textarea cols=40 rows=40>'.$report.'</textarea>';
			//$file = 'http://localhost:8080/'.$matches[1][CLX_INIT_ZERO];
			//$c->reportService()->getReportAttachments($file);
			//pre($c->reportService()->getReportAttachments($file));
			return $report;
		}
    }
    function listReportstoJasper($idlist) {
	//define('kPathToJasperPHP', 'classes/jasperPHP/');
	//define('JASPERCLIENT_ROOT', kPathToWebSite . kPathToJasperPHP . 'src/');
        if (!defined('kPathToJasperPHP'))
            define('kPathToJasperPHP', '../extlibs/php/jasperPHP/');
        if (!defined('JASPERCLIENT_ROOT'))
	define('JASPERCLIENT_ROOT', kPathToWebSite . kPathToJasperPHP . 'src/');

	//$table = new htmlTable(2);
	//$table->setProp(CLX_PROP_CLASS, 'form');
	//$table->addCells('headers',array(dict('GEN_LBL_REPORTS'), dict('GEN_LBL_MESSAGE')));
	$table = new htmlTable(CLX_WIDTH_TWO);
	$table->setProp(CLX_PROP_CLASS, CLX_CLASS_FORM);
	$table->addCells('headers',array(dict('GEN_LBL_REPORTS'), dict('GEN_LBL_MESSAGE')));
	$arreglo=array();
	for ($t=0;$t<count($idlist);$t++) {

	    $id = $idlist[$t]; // get the last one
	    $record = $this->loadRecord($id);
	    if ($record['ANRP_report_type']=='2') {
	       $text= $this->ImportReportJasper($this->getTemplateContent($record['ANRP_idcode']));
	       $table->addCell($record['ANRP_idname'],array(CLX_PROP_CLASS=>CLX_CLASS_DATA));
	       $table->addCell($text, array(CLX_PROP_CLASS=>CLX_CLASS_DATA));
	    }
	    else {
		$table->addCell($record['ANRP_idname'], array(CLX_PROP_CLASS=>CLX_CLASS_DATA));
		$table->addCell(dict('GEN_LBL_NOJASPERREPORT'),array(CLX_PROP_CLASS=>CLX_CLASS_DATA));
	    }
	}
	return note(sprintf('<head><link href="%s" rel="stylesheet" type="text/css"></head>', kPathToSkins . $_SESSION['skin'] . '/styles.css') . $table->source());
    }
    function ImportReportJasper($zipcontent='',$sitezip='') {
	include (kPathToJasperPHP . 'autoload.dist.php');
	if (empty($zipcontent) and empty($sitezip))
	    return dict('GEN_LBL_ZIP_REQUIRED');
	$server_path = ANRP_JASPER_SERVER_URI . '/' . ANRP_JASPER_SERVER_PATH;
	$c = new Client($server_path, ANRP_JASPER_SERVER_USER, ANRP_JASPER_SERVER_PASS, '');

	$request = new ImportTask;
	$request->update = true;
	if (!empty($sitezip))
	    $zipcontent=file_get_contents($sitezip);//'n:\export.zip')
	$metadata = $c->importExportService()->startImportTask($request,$zipcontent );
	$continue = true;
	while($continue) {
	    $state = $c->importExportService()->getImportState($metadata->id);
	    if ($state->phase == 'finished') {
		$continue = false;

		return dict('GEN_LBL_PUBLISHREPORT');

	    } else {
		
		@ob_flush();
		@flush();
		sleep(10);
	    }
	}
    }
    /**
     * Builds Template report
     */
    function buildTemplateReport() {
	$report = getToolObject('RPFT', 'RPWX');
	//Load Temporary if needed
	if (isset(UserRequest::$request['ANRP_ID']))
        UserRequest::$request['ANRP_id']=UserRequest::$request['ANRP_ID'];
    
	$this->intTemporaryTables(UserRequest::$request['ANRP_id']);
	$this->runReportEvents(UserRequest::$request['ANRP_id']);
	return $report->buildTemplate(UserRequest::$request['ANRP_id']);
    }

    /**
     * Executes Report Events
     * @param unknown_type $reportid
     */
    private function runReportEvents($reportid) {
	$EVNT = &getTool('EVNT');

	//Test if event exists or just return
	if (!$EVNT->OwnerEventExists($this->code, $reportid, 'RPBS'))
	    return;
	$reportrec = ClxCommon::$db->getFields($this->tablename, 'SEARCH_FORM, QUERYCODE', 'ID = ' . $reportid);
	$prmts = UserRequest::$post;
	unset($prmts[CLX_PROP_TO], $prmts['ac'], $prmts['qy'], $prmts['fm'], $prmts['li'], $prmts['nofr'], $prmts['pfid'], $prmts['rf'], $prmts['rfi'], $prmts['confirm'], $prmts['submit1']);
	//Parsing parameters according to report format
	if (!empty($reportrec['QUERYCODE'])) {
	    $DBQY = &getTool('DBQY');
	    $prmts = array_merge($prmts, $DBQY->loadAnswersFromRecordNoTablecode($reportrec['QUERYCODE'], UserRequest::$post, $reportrec['SEARCH_FORM']));
	}
	$msg = array();
	$EVNT->execute5($this->code, $reportid, 'RPBS', $prmts, $msg);
    }

    /**
     * Loads temporary tables related to the report
     * @param int $reportid
     */
    private function intTemporaryTables($reportid) {
	if ($reportid === '')
	    $reportid = CLX_INIT_ZERO;
	$DBTB = &getTool('DBTB');
	$sql = 'SELECT DISTINCT T.IDCODE TABLECODE FROM CLX_DB_TABLES T, CLX_DB_QUERY_TABLES QT WHERE QT.TABLECODE = T.IDCODE AND T.TYPE = ' . singleQuote('TEMP') . ' AND QT.QUERYCODE IN (SELECT R.QUERYCODE FROM CLX_AN_REPORTS R WHERE R.ID = ' . $reportid . ' UNION SELECT RQ.QUERYCODE FROM CLX_AN_REPORT_QUERIES RQ, CLX_AN_REPORTS R WHERE RQ.REPORTCODE = R.IDCODE AND R.ID = ' . $reportid . ') ';
	$rs = ClxCommon::$db->executeQuery($sql);
	while ($r = ClxCommon::$db->getNextRecord($rs, ASSOCIATIVE)) {
	    $DBTB->buildTemporaryTable($r['TABLECODE']);
	}
    }

    /**
     * Search all queries asociated to the selected report
     * @param $reportcode
     * @return array
     */
    function getReportQueries($reportcode) {
	$queries = array();
	$sql = 'SELECT R.QUERYCODE FROM CLX_AN_REPORTS R WHERE R.IDCODE = ' . singleQuote($reportcode) . ' AND R.QUERYCODE IS NOT null AND R.REPORT_TYPE = 0 UNION SELECT Q.QUERYCODE FROM CLX_AN_REPORT_GROUPS G, CLX_AN_REPORT_QUERIES Q WHERE G.REPORTCODE = ' . singleQuote($reportcode) . ' AND G.REPQRYCODE = Q.IDCODE UNION SELECT Q.QUERYCODE FROM CLX_AN_REPORT_COLUMNS C, CLX_AN_REPORT_QUERIES Q WHERE Q.IDCODE = C.REPQRYCODE AND C.REPORTCODE = ' . singleQuote($reportcode);
	$rs = ClxCommon::$db->executeQuery($sql);
	while ($r = ClxCommon::$db->getNextRecord($rs, ASSOCIATIVE)) {
	    $queries[] = $r['QUERYCODE'];
	}
	return $queries;
    }

    /**
     * Builds Pagelet for Tool
     * @param string $code
     * @param string $ac
     * @param integer $objectid
     * @return string
     */
    function pagelet($code, $ac = null, $objectid = null) {
	$reportid = ClxCommon::$db->getField($this->tablename, 'ID', 'IDCODE=' . singleQuote($code));
	$report = getToolObject('RPFT', 'RPHT'); // RPHT = HTML Format
        $source = '';
        if(empty($reportid)){
            $BUGS = &getTool('BUGS');
            if (!empty($BUGS)) {
                $BUGS->report(E_USER_WARNING, 'Missing report on page: '.$code, 'tools/ui_pages.php', '611', UserRequest::$request, CLX_NUM_ZERO, '');
            }
            $source = note(dict('GEN_LBL_INACTIVE').' Ref: ANRP.'.$code);
        }else{
            $source = $report->build($reportid);
        }
	return $source;
    }

//end method

    /**
     * Sends template file download in case of existance. String warnign otherwise/
     * @param $reportcode
     * @return string
     */
    function getTemplateContent($reportcode) {
	$reportrec = ClxCommon::$db->getRecord($this->tablename, 'IDCODE = '.singleQuote($reportcode));
	if (empty($reportrec))
	    return note(dict('GEN_ERR_FILE_NOT_FOUND'));
	$filename = explode('.', $reportrec['TEMPLATE_REF']);
	$contents = $reportrec['TEMPLATE'];
	return $contents;
    }
    function downloadTemplate($reportcode) {
		$reportrec = ClxCommon::$db->getRecord($this->tablename, 'IDCODE = '.singleQuote($reportcode));
		if (empty($reportrec))
			return note(dict('GEN_ERR_FILE_NOT_FOUND'));
		ob_clean();
		sendDownloadFile($filename[CLX_INIT_ZERO], $filename[1]);
		return $reportrec['TEMPLATE'];
    }

    function getImgType($imagebinary) {
	$handle = @fopen($filename, 'r');
	$types = array('jpeg' => "\xFF\xD8\xFF", 'gif' => 'GIF', 'png' => "\x89\x50\x4e\x47\x0d\x0a", 'bmp' => 'BM', 'psd' => '8BPS', 'swf' => 'FWS');
	foreach ($types as $type => $header) {
	    if (strpos($imagebinary, $header) === CLX_NUM_ZERO) {
		$found = $type;
		break;
	    }
	}
	return $found;
    }

    function getReportAccess($reportcode) {
//    $callingpage = peekHistory(); // try to get security from query if defined
        $access = array();
        if (!$_SESSION ['hasUser']) {
            // do nothing
        } elseif ($_SESSION ['user_record'] ['isadmin']) {
          $access = array(
                'canread' => CLX_NUM_ONE,
                'canmodify' => CLX_NUM_ONE,
                'candelete' => CLX_NUM_ONE,
                'canappend' => CLX_NUM_ONE,
                'isadmin' => CLX_NUM_ONE,
                'report' => array()
            );
        } else {
            $username = $_SESSION ['user_record'] ['username'];
            $sql = '		SELECT  ACRP.ID,
					ACRP.REPORTSCODE,
					MAX(ACRP.ISADMIN) ISADMIN,
					MAX(ACRP.CANREAD) CANREAD,
					MAX(ACRP.CANMODIFY) CANMODIFY,
					MAX(ACRP.CANDELETE) CANDELETE,
					MAX(ACRP.CANAPPEND) CANAPPEND,
						ANRP.IDNAME REPORTSNAME,
						ANRP.IDCODE REPORTSCODE,
						ANRP.ID ID2,
						DBMO.IDNAME MODULENAME,
						DBMO.IDCODE MODULECODE
				FROM 	CLX_AN_REPORTS ANRP
					JOIN (SELECT  ACRP.ID,
						    ACRP.REPORTSCODE,
						    ACRP.ISADMIN ISADMIN,
						    ACRP.CANREAD CANREAD,
						    ACRP.CANMODIFY CANMODIFY,
						    ACRP.CANDELETE CANDELETE,
						    ACRP.CANAPPEND CANAPPEND
					      FROM  CLX_AC_REPORTS ACRP
					      WHERE ACRP.REPORTSCODE = '.singleQuote($reportcode).'
						       AND  ((  ACRP.OWNERCODE='.singleQuote('USER').' AND ACRP.OWNERREF='.singleQuote($username).' )
							OR ( ACRP.OWNERREF IN (SELECT G.GROUPCODE FROM CLX_AC_GROUP_MEMBERS G WHERE G.USERNAME='.singleQuote($username).')
							AND ACRP.OWNERCODE='.singleQuote('GRUP').'))
					      UNION
					      SELECT  0 ID,
						    '.singleQuote($reportcode).' REPORTSCODE,
						    1 ISADMIN,
						    1 CANREAD,
						    1 CANMODIFY,
						    1 CANDELETE,
						    1 CANAPPEND
					      FROM DUAL
					    WHERE
						  NOT EXISTS
						    (SELECT 1 FROM
						    CLX_AC_REPORTS ACRP2
						    WHERE ACRP2.REPORTSCODE = '.singleQuote($reportcode).'
							AND CANREAD=1))  ACRP
							ON	ACRP.REPORTSCODE = ANRP.IDCODE
								JOIN CLX_DB_MODULES DBMO
							ON	ANRP.MODULECODE = DBMO.IDCODE
							WHERE ANRP.IDCODE = '.singleQuote($reportcode);
            $records = ClxCommon::$db->executeQuery($sql);
            $access = array(
                'canread' => CLX_NUM_ZERO,
                'canmodify' => CLX_NUM_ZERO,
                'candelete' => CLX_NUM_ZERO,
                'canappend' => CLX_NUM_ZERO,
                'isadmin' => CLX_NUM_ZERO,
                'report' => array()
            );
            if (($record = ClxCommon::$db->getNextRecord($records)) == true) {
                $access ['canread'] = (empty($record ['CANREAD'])) ?  CLX_NUM_ZERO : CLX_NUM_ONE;
                $access ['canmodify'] = (empty($record ['CANMODIFY'])) ? CLX_NUM_ZERO : CLX_NUM_ONE;
                $access ['candelete'] = (empty($record ['CANDELETE'])) ? CLX_NUM_ZERO : CLX_NUM_ONE;
                $access ['canappend'] = (empty($record ['CANAPPEND'])) ? CLX_NUM_ZERO : CLX_NUM_ONE;
                $access ['isadmin'] = (empty($record ['ISADMIN'])) ? CLX_NUM_ZERO : CLX_NUM_ONE;
                $access ['report'][$record['REPORTSCODE']] = $record;
            }
        }

        return $access;
    }
}
