<?php

/**
 * Process Seccion Manager
 *
 * @package		Clientix.Tool.Business Process
 * @abstract		Business Process Manager
 * @author       $Author: cramirez $
 * @copyright    Alenet, Inc.
 * @since        $Date: 2018-06-21 08:03:11 -0400 (Thu, 21 Jun 2018) $
 * @version    	$Rev: 9841 $
 *
 */
class bp_categories extends clx_tool {

    /**
     * Constructor
     */
    function __construct() {
        $this->code = 'BPCT';
        $this->name = 'Business Process - Categories';
        $this->system = 'Business Process';
        $this->version = '5.5.0.1';
        $this->tablename = 'CLX_BP_CATEGORIES';
        $this->record_name_singular = 'GEN_LBL_BP_CATEGORY';
        $this->record_name_plural = 'GEN_LBL_BP_CATEGORIES';
        $this->primary_key = array('CATEGORIESCODE');
        $this->primary_field = 'CATEGORIESCODE';
        $this->secondary_field = 'IDNAME';
        $this->seqnum_field = 'SEQNUM';
        $this->editform = array(CLX_CLASS_TITLE => 'TOOL_BPCT_TITLE_CREATE',
            'panels' => array(
                array('type' => 'normal',
                    'entries' => array(
                        array('label' => 'GEN_LBL_CATEGORIESCODE',
                            'fields' => array(
                                array('name' => 'categoriescode',
                                    'type' => 'string',
                                    'default' => '',
                                    'size' => '16',
                                    'displaysize' => '16',
                                    'dontmodify' => '1',
                                    'validations' => array(
                                        array('type' => 'string',
                                            'required' => '1'
                                        ), array('type' => 'unique',
                                            'fieldname' => 'categoriescode',
                                            'tablename' => $this->tablename)
                                    )
                                )
                            )
                        ), array('label' => 'GEN_LBL_NAME',
                            'fields' => array(
                                array('name' => 'idname',
                                    'type' => 'string',
                                    'default' => '',
                                    'size' => '64',
                                    'displaysize' => '48',
                                    'validations' => array(
                                        array('type' => 'string',
                                            'required' => '1'
                                        )
                                    )
                                )
                            )
                        ), array('label' => 'GEN_LBL_PROCESSCODE',
                            'fields' => array(
                                array('name' => 'processcode',
                                    'type' => 'lookup',
                                    'default' => '',
                                    'readonly' => false,
                                    'tool' => 'BPPR',
                                    'validations' => array(
                                        array('type' => 'string',
                                            'required' => '1'
                                        ), array('type' => 'lookup',
                                            'fieldname' => 'processcode',
                                            'tablename' => 'CLX_BP_PROCESSES'
                                        )
                                    )
                                )
                            )
                        ), array('label' => 'GEN_LBL_DESCRIPTION',
                            'fields' => array(
                                array('name' => 'description',
                                    'type' => 'text',
                                    'default' => '',
                                    'cols' => '48',
                                    'rows' => '4'
                                )
                            )
                        ), array('label' => 'GEN_LBL_SEQNUM',
                            'fields' => array(
                                array('name' => 'seqnum',
                                    'type' => 'string',
                                    'default' => '1',
                                    'readonly' => true,
                                    'size' => '32',
                                    'displaysize' => '16',
                                    'validations' => array()
                                )
                            )
                        )
                    )
                )
            )
        );
        $this->viewform = $this->editform;
        $this->viewform [CLX_CLASS_TITLE] = 'TOOL_BPCT_TITLE_DISPLAY'; // Etiqueta listo
        $this->toolpanelkeys = array(array('name' => 'PROCESSCODE',
                'type' => 'string'
            )
        ); // PENDIENTE
        $this->toolpanel = array('name' => $this->tablename, CLX_CLASS_TITLE => $this->record_name_plural, 'has_reorder' => true,
            'fields' => array(
                array('name' => 'ID',
                    'label' => 'GEN_LBL_ID',
                    'type' => 'number'
                ), array('name' => 'CATEGORIESCODE',
                    'label' => 'GEN_LBL_CATEGORIESCODE',
                    'type' => 'string'
                ), array('name' => 'IDNAME',
                    'label' => 'GEN_LBL_IDNAME',
                    'type' => 'string'
                ), array('name' => 'SEQNUM',
                    'label' => 'GEN_LBL_SEQNUM',
                    'type' => 'number'
                ), array('name' => 'PROCESSCODE',
                    'label' => 'GEN_LBL_PROCESSCODE',
                    'type' => 'string'
                ), array('name' => 'DESCRIPTION',
                    'label' => 'GEN_LBL_DESCRIPTION',
                    'type' => 'string'
                )
            )
        );
    }

// End method
    /**
     * (non-PHPdoc)
     *
     * @see clx_tool::toolTableStructure()
     */
    function toolTableStructure() {
        $fields = array();
        $fields ['ID'] = array('type' => 'int(11) unsigned',
            'null' => 'NO',
            'properties' => 'auto_increment',
            'index' => 'PRI'
        );
        $fields ['CATEGORIESCODE'] = array('type' => 'varchar(16)',
            'null' => 'NO',
            'properties' => ''
        );
        $fields ['IDNAME'] = array('type' => 'varchar(64)',
            'null' => 'NO',
            'properties' => ''
        );
        $fields ['PROCESSCODE'] = array('type' => 'varchar(16)',
            'null' => 'NO',
            'properties' => ''
        );
        $fields ['DESCRIPTION'] = array('type' => 'mediumtext',
            'properties' => ''
        );
        $fields ['SEQNUM'] = array('type' => 'int(11) unsigned',
            'null' => 'NO',
            'properties' => '',
            'default' => CLX_NUM_ZERO
        );
        $indexes = array();
        $ipf = 'CLX' . $this->code;
        $indexes [$ipf . '01'] = array('index' => 'PRI',
            'fieldname' => 'ID'
        );
        $indexes [$ipf . '02'] = array('index' => 'UNI',
            'fieldname' => 'CATEGORIESCODE'
        );
        $indexes [$ipf . '03'] = array('index' => 'MUL',
            'fieldname' => 'PROCESSCODE'
        );
        return array('idcode' => $this->code, 'idname' => $this->name, 'name' => $this->tablename, 'type' => 'InnoDB',
            'fields' => $fields,
            'indexes' => $indexes
        );
    }

// end method
    /**
     * (non-PHPdoc)
     *
     * @see clx_tool::unpack()
     */
    function unpack($parameters) {
        $parameters = array_values(array_merge(array('package' => null), $parameters));
        list($package) = $parameters;
        $record = $this->unpackRecord($package);
        ClxCommon::$db->deleteRecords($this->tablename, $this->primary_field . '=' . singleQuote($record [$this->primary_field]));
        ClxCommon::$db->createRecord($this->tablename, $record, false);
    }

    /**
     * (non-PHPdoc)
     *
     * @see clx_tool::action()
     */
    function action($ac, $idlist) {
        //if ($this->access ['canread'] === true) {
        if ($this->getAccess()->canRead()) {
            switch ($ac) {
                default :
                    $contents = parent::action($ac, $idlist);
                    break;
            }
        } else {
            //$contents = warning ( dict ( 'GEN_ERR_NO_ACCESS' ) );
            $contents = warning(dict('GEN_ERR_NO_ACCESS') . '[BP_CATEGORIES-CAN_READ][' . $ac . '] ' . $this->getCode(false));
        }
        return $contents;
    }

// end method
    /**
     * (non-PHPdoc)
     *
     * @see clx_tool::versionIncrease()
     */
    function versionIncrease($fieldname, $fieldvalue) {
        $processcode = ClxCommon::$db->getField($this->tablename, 'PROCESSCODE', $fieldname . '=' . singleQuote($fieldvalue));
        $BPPR = &getTool('BPPR');
        $BPPR->versionIncrease('PROCESSCODE', $processcode);
        parent::versionIncrease($fieldname, $fieldvalue);
    }

// end method
    /**
     * (non-PHPdoc)
     *
     * @see clx_tool::defaultRecord()
     */
    function defaultRecord() {
        $callingpage = peekHistory();
        $record = parent::defaultRecord();
        $record ['BPCT_categoriescode'] = $callingpage ['processcode'] . '.' . (ClxCommon::$db->countRecords($this->tablename, 'PROCESSCODE = ' . singleQuote($callingpage ['processcode'])) + CLX_NUM_ONE);
        $record ['BPCT_version'] = '3.0.0';
        $record ['BPCT_status'] = CLX_NUM_ONE;
        $record ['BPCT_processcode'] = $callingpage ['processcode'];
        $record ['BPCT_seqnum'] = (ClxCommon::$db->countRecords($this->tablename, 'PROCESSCODE = ' . singleQuote($callingpage ['processcode'])) + CLX_NUM_ONE);
        return $record;
    }

// end method
    /**
     * (non-PHPdoc)
     *
     * @see clx_tool::reorderRecords()
     */
    function reorderRecords() {
        $callingpage = peekHistory();
        $records = ClxCommon::$db->getRecords($this->tablename, 'PROCESSCODE=' . singleQuote($callingpage ['processcode']) . ' ORDER BY SEQNUM');
        $list = array();
        while (($record = ClxCommon::$db->getNextRecord($records))) {
            $list [$record ['ID']] = htmlentities($record ['IDNAME']);
        }
        return $list;
    }

    /**
     * (non-PHPdoc)
     *
     * @see clx_tool::reorderAction()
     */
    function reorderAction() {
        parent::reorderAction();
        $callingpage = peekHistory();
        $BPPR = &getTool('BPPR');
        $BPPR->versionIncrease('PROCESSCODE', $callingpage ['processcode']);
    }

// end method
}

// End class
