<?php

/**
 * Process Seccion Manager
 *
 * @package		Clientix.Tool.Business Process
 * @abstract		Business Process Manager
 * @author       $Author: cramirez $
 * @copyright    Alenet, Inc.
 * @since        $Date: 2018-06-21 08:03:11 -0400 (Thu, 21 Jun 2018) $
 * @version    	$Rev: 9841 $
 *
 */
class bp_rules extends clx_tool {

    /**
     * Constructor
     */
    function __construct() {
        $this->code = 'BPRU';
        $this->name = 'Business Process - Rules';
        $this->system = 'Business Process';
        $this->version = '5.5.0.1';
        $this->tablename = 'CLX_BP_RULES';
        $this->record_name_singular = 'GEN_LBL_BP_RULE';
        $this->record_name_plural = 'GEN_LBL_BP_RULES';
        $this->primary_key = array('RULESCODE');
        $this->primary_field = 'RULESCODE';
        $this->secondary_field = 'IDNAME';
        $this->editform = array(CLX_CLASS_TITLE => 'TOOL_BPRU_TITLE_CREATE',
            'panels' => array(
                array('type' => 'normal',
                    'entries' => array(
                        array('label' => 'GEN_LBL_RULESCODE',
                            'fields' => array(
                                array('name' => 'rulescode',
                                    'type' => 'string',
                                    'default' => '',
                                    'size' => '16',
                                    'displaysize' => '16',
                                    'dontmodify' => '1',
                                    'validations' => array(
                                        array('type' => 'string',
                                            'required' => '1'
                                        ), array('type' => 'unique',
                                            'fieldname' => 'rulescode',
                                            'tablename' => $this->tablename)
                                    )
                                )
                            )
                        ), array('label' => 'GEN_LBL_NAME',
                            'fields' => array(
                                array('name' => 'idname',
                                    'type' => 'string',
                                    'default' => '',
                                    'size' => '64',
                                    'displaysize' => '48',
                                    'validations' => array(
                                        array('type' => 'string',
                                            'required' => '1'
                                        )
                                    )
                                )
                            )
                        ), array('label' => 'GEN_LBL_ACTIVITIESCODE',
                            'fields' => array(
                                array('name' => 'activitiescode',
                                    'type' => 'lookup',
                                    'default' => '',
                                    'readonly' => false,
                                    'tool' => 'BPAC',
                                    'validations' => array(
                                        array('type' => 'string',
                                            'required' => '1'
                                        ), array('type' => 'lookup',
                                            'fieldname' => 'activitiescode',
                                            'tablename' => 'CLX_BP_ACTIVITIES'
                                        )
                                    )
                                )
                            )
                        ), array('label' => 'GEN_LBL_DESCRIPTION',
                            'fields' => array(
                                array('name' => 'description',
                                    'type' => 'text',
                                    'default' => '',
                                    'cols' => '48',
                                    'rows' => '4'
                                )
                            )
                        )
                    )
                )
            )
        );
        $this->viewform = $this->editform;
        $this->viewform [CLX_CLASS_TITLE] = 'TOOL_BPRU_TITLE_DISPLAY';
        $this->toolpanelkeys = array(array('name' => 'ACTIVITIESCODE', 'type' => 'string'));
        $this->toolpanel = array('name' => $this->tablename, CLX_CLASS_TITLE => $this->record_name_plural, 'fields' => array(
                array('name' => 'ID',
                    'label' => 'GEN_LBL_ID',
                    'type' => 'number'
                ), array('name' => 'RULESCODE',
                    'label' => 'GEN_LBL_RULESCODE',
                    'type' => 'string'
                ), array('name' => 'IDNAME',
                    'label' => 'GEN_LBL_IDNAME',
                    'type' => 'string'
                ), array('name' => 'ACTIVITIESCODE',
                    'label' => 'GEN_LBL_ACTIVITIESCODE',
                    'type' => 'string'
                ), array('name' => 'DESCRIPTION',
                    'label' => 'GEN_LBL_DESCRIPTION',
                    'type' => 'string'
                )
            )
        );
    }

    /**
     * (non-PHPdoc)
     *
     * @see clx_tool::toolTableStructure()
     */
    function toolTableStructure() {
        $fields = array();
        $fields ['ID'] = array('type' => 'int(11) unsigned', 'null' => 'NO', 'properties' => 'auto_increment', 'index' => 'PRI');
        $fields ['RULESCODE'] = array('type' => 'varchar(16)', 'null' => 'NO', 'properties' => '');
        $fields ['IDNAME'] = array('type' => 'varchar(64)', 'null' => 'NO', 'properties' => '');
        $fields ['ACTIVITIESCODE'] = array('type' => 'varchar(16)', 'null' => 'NO', 'properties' => '');
        $fields ['DESCRIPTION'] = array('type' => 'mediumtext', 'properties' => '');
        $indexes = array();
        $ipf = 'CLX' . $this->code;
        $indexes [$ipf . '01'] = array('index' => 'PRI', 'fieldname' => 'ID');
        $indexes [$ipf . '02'] = array('index' => 'UNI', 'fieldname' => 'RULESCODE');
        $indexes [$ipf . '03'] = array('index' => 'MUL', 'fieldname' => 'ACTIVITIESCODE');
        return array('idcode' => $this->code, 'idname' => $this->name, 'name' => $this->tablename, 'type' => 'InnoDB', 'fields' => $fields, 'indexes' => $indexes);
    }

    /**
     * (non-PHPdoc)
     *
     * @see clx_tool::action()
     */
    function action($ac, $idlist) {
        //if ($this->access ['canread'] === true) {
        if ($this->getAccess()->canRead()) {
            switch ($ac) {
                default :
                    $contents = parent::action($ac, $idlist);
                    break;
            }
        } else {
            //$contents = warning ( dict ( 'GEN_ERR_NO_ACCESS' ) );
            $contents = warning(dict('GEN_ERR_NO_ACCESS') . '[BP_RULES-CAN_READ][' . $ac . '] ' . $this->getCode(false));
        }
        return $contents;
    }

    /**
     * (non-PHPdoc)
     *
     * @see clx_tool::unpack()
     */
    function unpack($parameters) {
        $parameters = array_values(array_merge(array('package' => null), $parameters));
        list($package) = $parameters;
        $record = $this->unpackRecord($package);
        ClxCommon::$db->deleteRecords($this->tablename, $this->primary_field . '=' . singleQuote($record [$this->primary_field]));
        ClxCommon::$db->createRecord($this->tablename, $record, false);
    }

    /**
     * (non-PHPdoc)
     *
     * @see clx_tool::viewFormFill()
     */
    function viewFormFill($record) {
        addHistory('rulesid', $record ['BPRU_id']);
        addHistory('rulescode', $record ['BPRU_rulescode']);
        addHistory('ownercode', $this->code);
        addHistory('ownerid', $record ['BPRU_rulescode']);
        addHistory('activitiescode', $record ['BPRU_activitiescode']);
        $EVNT = &getTool('EVNT');
        return $EVNT->makepanel($this->code, $record ['BPRU_rulescode']);
    }

    /**
     * (non-PHPdoc)
     *
     * @see clx_tool::versionIncrease()
     */
    function versionIncrease($fieldname, $fieldvalue) {
        $id = ClxCommon::$db->getField($this->tablename, 'ID', $fieldname . '=' . singleQuote($fieldvalue));
        $BPAC = &getTool('BPAC');
        $BPAC->versionIncrease('ID', $id);
        parent::versionIncrease($fieldname, $fieldvalue);
    }

    /**
     * (non-PHPdoc)
     *
     * @see clx_tool::defaultRecord()
     */
    function defaultRecord() {
        $callingpage = peekHistory(); // Buscar ojo
        $record = parent::defaultRecord();
        $record ['BPRU_rulescode'] = $callingpage ['activitiescode'] . '.' . (ClxCommon::$db->countRecords($this->tablename, 'ACTIVITIESCODE = ' . singleQuote($callingpage ['activitiescode'])) + CLX_NUM_ONE);
        $record ['BPRU_activitiescode'] = $callingpage ['activitiescode'];
        return $record;
    }

}

// End class
