<?php

/**
 * Tool Class
 * For more information contact  support@alenet.com
 *
 * @author       $Author: alenetdev $
 * @copyright    Alenet, Inc.
 * @since        $Date: 2019-01-22 15:45:22 -0400 (ma., 22 ene. 2019) $
 * @version        $Rev: 10496 $
 *
 * CHANGE LOG
 *  Oracle uppercase record changed
 *  Included label
 *  Added continue botton to report function
 */
class bugs extends clx_tool {

    var $debug = CLX_INIT_ZERO;
    var $alertlevel; // Alert the user
    var $savelevel; // Save to the database
    var $dielevel; // Die level
    var $status; // list of posible status strings
    var $types; // list of types
    var $priorities; // list of priorities

    // CONSTRUCTOR //

    function __construct() {
        $this->code = 'BUGS';
        $this->name = 'Error Trap, notes and comments';
        $this->system = 'System';
        $this->version = '7.0.0.0';

        $this->has_access = true;

        $this->tablename = 'CLX_BUGS';
        $this->record_name_singular = 'bug';
        $this->record_name_plural = 'bugs';
        $this->primary_field = 'ERRMSG';
        $this->secondary_field = 'ERRNO';

        $this->alertlevel = CLX_NUM_TWO; // 0=Errors, 1=Warnings, 2=Notice, 3=Unknowns, 4 or more=everything
        $this->savelevel = CLX_NUM_TWO;
        $this->dielevel = CLX_INIT_ZERO;
        $this->status = array('Closed', 'Opened');
        $this->types = array('Script Error', 'Database Error', 'JavaScript Error', 'Documentation Error', 'Cosmetic Error', 'PHP Error', 'Logic Error', 'Other', 'Network Error');
        $this->priorities = array('High', 'Medium', 'Low', 'None');
       // error_reporting(E_ALL);
		$this->mensajesizerror=1980;
    }

    function install() {
        $UIMI = getTool('UIMI');
        $UIMI->add($this->code, 'MENU_TOOLS:MENU_SYSTEM:MENU_BUGS', 'TOOL', $this->code, 'ac=LIST');
    }

    function toolTableStructure() {
        $fields = array();
        $fields['ID'] = array('type' => 'int(11) unsigned', 'null' => 'NO', 'properties' => 'auto_increment', 'index' => 'PRI');
        $fields['CDATE'] = array('type' => 'datetime', 'null' => 'YES', 'properties' => '');
        $fields['MDATE'] = array('type' => 'datetime', 'null' => 'YES', 'properties' => '');
        $fields['URL'] = array('type' => 'varchar(255)', 'null' => 'NO', 'properties' => '');
        $fields['USERNAME'] = array('type' => 'varchar(30)', 'null' => 'NO', 'properties' => '');
        $fields['DESCRIPTION'] = array('type' => 'mediumtext', 'properties' => '');
        $fields['CONTEXT'] = array('type' => 'mediumtext', 'properties' => '');
        $fields['PRIORITY'] = array('type' => 'tinyint(2) unsigned', 'null' => 'NO', 'default' => CLX_NUM_ZERO);
        $fields['TYPE'] = array('type' => 'tinyint(2) unsigned', 'null' => 'NO', 'properties' => '', 'default' => CLX_NUM_ZERO);
        $fields['STATUS'] = array('type' => 'tinyint(2) unsigned', 'null' => 'NO', 'properties' => '', 'default' => CLX_NUM_ZERO);
        $fields['ERRMSG'] = array('type' => 'varchar('.$this->mensajesizerror.')', 'properties' => '');
        $fields['ERRNUM'] = array('type' => 'varchar(32)', 'properties' => '');
        $indexes = array();
        $ipf = 'CLX' . $this->code;
        $indexes[$ipf . '01'] = array('index' => 'PRI', 'fieldname' => 'ID');
        $indexes[$ipf . '02'] = array('index' => 'MUL', 'fieldname' => 'CDATE');
        $indexes[$ipf . '03'] = array('index' => 'MUL', 'fieldname' => 'MDATE');
        return array('idcode' => $this->code, 'idname' => $this->name, 'name' => $this->tablename, 'type' => 'InnoDB',
            'fields' => $fields,
            'indexes' => $indexes);
    }

    function action($ac, $idlist) {
        $contents = '';
        if ($this->getAccess()->canRead()) {
            switch ($ac) {
                case 'CLOSE':
                    if ($this->getAccess()->canModify()) {
                        popHistory();
                        $bugid = $idlist[count($idlist) - CLX_NUM_ONE];
                        $record = array();
                        $record['STATUS'] = '0';
                        ClxCommon::$db->updateRecord('CLX_BUGS', $record, 'ID=' . $bugid);
                        $contents = $this->viewForm($idlist);
                    } else {
                        $contents = warning(dict('GEN_ERR_NO_ACCESS') . ' [BUGS-CAN_MODIFY_1][' . $ac . '] ' . $this->getCode(false));
                    }
                    break;
                case 'OPEN':
                    if ($this->getAccess()->canModify()) {
                        popHistory();
                        $bugid = $idlist[count($idlist) - CLX_NUM_ONE];
                        $record = array();
                        $record['STATUS'] = '1';
                        ClxCommon::$db->updateRecord('CLX_BUGS', $record, 'ID=' . $bugid);
                        $contents = $this->viewForm($idlist);
                    } else {
                        $contents = warning(dict('GEN_ERR_NO_ACCESS') . ' [BUGS-CAN_MODIFY_2][' . $ac . '] ' . $this->getCode(false));
                    }
                    break;
                default:
                    $contents = parent::action($ac, $idlist);
            }
        } else {
            $contents = warning(dict('GEN_ERR_NO_ACCESS') . ' [BUGS-CAN_READ][' . $ac . '] ' . $this->getCode(false));
        }
        return $contents;
    }

    /**
     * Displays tool listing
     * @param type $listingcode
     * @return type
     */
    function makelist($listingcode = null) {
        //$t = time('U'); genera un warning
        $t = time();
        $to = $this->code;
        $here = 'tool.php?to=' . $to . '&t=' . $t;
        clearHistory();
        pushHistory($this->code, 'LIST', array());
        $form = new htmlForm(CLX_PROP_ACTION, 'tool.php');
        $form->setProp(CLX_PROP_ID, 'buglist');
        $form->addArgument(CLX_PROP_TO, $this->code);
        $form->addArgument('ac', 'NOT_DEFINED');
        $selectclause = '*';
        $fromclause = 'CLX_BUGS';
        $whereclause = '';
        $sortclause = 'MDATE DESC';
        $page = new xPaging();
		if (isset(UserRequest::$request['startat'])) {
			$_SESSION['paging'][$form->getProp(CLX_PROP_ID)]['startat'] = UserRequest::$request['startat'];
		}
        $page->pageInit($form->getProp(CLX_PROP_ID), $here, $selectclause, $fromclause, $whereclause, $sortclause, '');

        $table = new htmlTable(CLX_WIDTH_EIGHT);
        $table->setProps(array(CLX_PROP_CLASS => CLX_CLASS_LIST, CLX_PROP_WIDTH => CLX_PROP_100PCT));
        $table->addCell(dict('TOOL_BUGS_TITLE_LIST'), array(CLX_PROP_CLASS => CLX_CLASS_TITLE, CLX_PROP_COLSPAN => CLX_WIDTH_EIGHT));
        $page->header($table);
        $table->addCells(CLX_CLASS_HEADERS, array(dict('GEN_LBL_SEL'), dict('GEN_LBL_REFERENCE_NO'),  dict('GEN_LBL_DATE'), dict('GEN_LBL_STATUS'), dict('GEN_LBL_PRIORITY'), dict('GEN_LBL_TYPE'), dict('GEN_LBL_USERNAME'), dict('GEN_LBL_ERROR')));
        $i = CLX_INIT_ZERO;
        while ($record = $page->nextRecord()) {
            $recordid = $record['ID'];
            $select = new htmlCheckbox('selector_' . $recordid, false);
            $select->setProp('onclick', 'selectButtons(' . singleQuote($form->getProp(CLX_PROP_ID)) . ');');
            $table->addCell($select->source(), array(CLX_PROP_CLASS => CLX_CLASS_FOOTERS, CLX_PROP_WIDTH => '40', CLX_PROP_ALIGN => 'center'));
            $table->addCells(($i % CLX_NUM_TWO == CLX_NUM_ZERO) ? CLX_CLASS_DATA : CLX_CLASS_DATAALT, array($record['ID'], xFormat::datetime($record['MDATE'],$_SESSION['prefs']['DATE_FORMAT'].' h:i:s'), $this->status[$record['STATUS']], $this->priorities[$record['PRIORITY']], $this->types[$record['TYPE']], $record['USERNAME'], xFormat::text($record['ERRMSG'])));
            $i++;
        }
        $buttons = '';
        $buttons .= htmlButton::getButton('submit', dict('GEN_LBL_DISPLAY'), array('action' => 'DISPLAY', 'form' => $form->getProp(CLX_PROP_ID), CLX_PROP_ID => 'MULT'))->source();
        if ($this->getAccess()->canDelete()) {
            $buttons .=htmlButton::getButton('submit', dict('GEN_LBL_DELETE'), array('action' => 'DELETE', 'form' => $form->getProp(CLX_PROP_ID), CLX_PROP_ID => 'MULT'))->source();
        }
        if ($this->getAccess()->canDisplay()) {
            $buttons .=htmlButton::getButton('submit', dict('GEN_LBL_CREATE'), array('action' => 'CREATE', 'form' => $form->getProp(CLX_PROP_ID), CLX_PROP_ID => 'ON'))->source();
        }
        $buttons .=htmlButton::getButton('submit', dict('GEN_LBL_FILTER'), array('action' => 'FILTER', 'form' => $form->getProp(CLX_PROP_ID), CLX_PROP_ID => 'ON'))->source();
        $form->addContents($page->footers($table, $buttons));
        return $form->source();
	
    }

    function confirmDeleteFill(&$table, $record) {
        $table->addData('Error msg.:', $record['BUG_errmsg']);
    }

    function editForm($ac, $idlist, $record, $msg) {
        $title = dict('TOOL_BUGS_TITLE_CREATE');
        $form = new htmlForm(CLX_PROP_ACTION, 'tool.php');
        $form->addArgument(CLX_PROP_TO, $this->code);
        $form->addArgument('ac', $ac);
        $form->addArgument('confirm', '1');
        $form->addArgument('BUGS_id', (!empty($record['BUGS_id'])) ? $record['BUGS_id'] : '');
        $form->addArgument('idlist', toString($idlist));
        $table = new htmlTable(CLX_WIDTH_TWO);
        $table->setProps(array(CLX_PROP_CLASS => CLX_CLASS_FORM, CLX_PROP_WIDTH => CLX_PROP_100PCT));
        $table->addCell($title, array(CLX_PROP_CLASS => CLX_CLASS_TITLE, CLX_PROP_COLSPAN => CLX_WIDTH_TWO));
        $table->addField($msg, dict('GEN_LBL_URL') . ':', 'BUGS_url', (!empty($record['BUGS_url'])) ? $record['BUGS_url'] : '', CLX_NUM_48, CLX_NUM_255);
        $table->addField($msg, dict('GEN_LBL_ERRORNUM') . ':', 'BUGS_errnum', (!empty($record['BUGS_errnum'])) ? $record['BUGS_errnum'] : '', CLX_NUM_16, CLX_NUM_32);
        $table->addField($msg, dict('GEN_LBL_ERRORMSG') . ':', 'BUGS_errmsg', (!empty($record['BUGS_errmsg'])) ? $record['BUGS_errmsg'] : '', CLX_NUM_48, CLX_NUM_255);
        $table->addText($msg, dict('GEN_LBL_DESCRIPTION') . ':', 'BUGS_description', (!empty($record['BUGS_description'])) ? $record['BUGS_description'] : '', CLX_NUM_FOUR, CLX_NUM_48);
        $pop = new HTMLSelect('BUGS_priority');
        foreach ($this->priorities as $key => $value) {
            $pop->addItem($value, $key, (!empty($record['BUGS_priority']) && $record['BUGS_priority'] == $key));
        }
        $table->addData(dict('GEN_LBL_PRIORITY') . ':', $pop->source());
        $pop = new HTMLSelect('BUGS_type');
        foreach ($this->types as $key => $value) {
            $pop->addItem($value, $key, (!empty($record['BUGS_type']) && $record['BUGS_type'] == $key));
        }
        $table->addData(dict('GEN_LBL_TYPE') . ':', $pop->source());
        $pop = new HTMLSelect('BUGS_status');
        foreach ($this->status as $key => $value) {
            $pop->addItem($value, $key, (!empty($record['BUGS_status']) && $record['BUGS_status'] == $key));
        }
        $table->addData(dict('GEN_LBL_STATUS') . ':', $pop->source());
        $table->addText($msg, dict('GEN_LBL_CONTEXT') . ':', 'BUGS_context', (!empty($record['BUGS_context'])) ? $record['BUGS_context'] : '', CLX_NUM_TEN, CLX_NUM_78);
        $form->addContents($table->source());
        $table = new htmlTable(CLX_WIDTH_TWO);
        $table->setProps(array(CLX_PROP_CLASS => CLX_CLASS_FORM, CLX_PROP_WIDTH => CLX_PROP_100PCT));
        $buttons .=htmlButton::getButton('submit', dict('GEN_LBL_CREATE'), array('action' => 'CREATE', 'form' => $form->getProp(CLX_PROP_ID), CLX_PROP_ID => 'ON'))->source();
        $c = new htmlButtonCancel();
        $buttons .= $c->source();
        $table->addCell($buttons, array(CLX_PROP_CLASS => CLX_CLASS_FOOTERS, CLX_PROP_COLSPAN => CLX_WIDTH_TWO));
	
        $form->addContents($table->source());
        return $form->source();
    }

    function checkForm(&$msg) {
        $msg = array();
        xValidate::required($msg, 'BUGS_url', UserRequest::$post['BUGS_url']);
        xValidate::required($msg, 'BUGS_description', UserRequest::$post['BUGS_description']);
        xValidate::required($msg, 'BUGS_errmsg', UserRequest::$post['BUGS_errmsg']);
        return count($msg) == CLX_NUM_ZERO;
    }

    /**
     * Display tools form
     * @param type $idlist
     * @return type
     */
    function viewForm($idlist) {

        $id = $idlist[count($idlist) - CLX_NUM_ONE]; // get the last one
        $record = $this->loadToolRecord($id);
        pushHistory($this->code, 'DISPLAY', $idlist);
        $contents = $this->viewFormFill($record);
        $form = new htmlForm(CLX_PROP_ACTION, 'tool.php');
        $form->setProp(CLX_PROP_ID, CLX_CLASS_FOOTERS);
        $form->addArgument('ac', 'NOT_DEFINED');
        $form->addArgument(CLX_PROP_TO, $this->code);
        $form->addArgument('idlist', toString($idlist));
        $form->addArgument('next', CLX_NUM_ONE);
        $buttons = '';
        $table = new htmlTable(CLX_WIDTH_TWO);
        $table->setProps(array(CLX_PROP_CLASS => CLX_CLASS_FORM, CLX_PROP_WIDTH => CLX_PROP_100PCT));
        $c = new htmlButtonReturn();
        $buttons .= $c->source();
        if ($record['BUG_status'] == CLX_NUM_ONE) {
            $c = new htmlFormSubmit(dict('TOOL_BUGS_BUTTON_CLOSE_BUG'), 'CLOSE', $form->getProp(CLX_PROP_ID));
            $buttons .= $c->source();
        } else {
            $c = new htmlFormSubmit(dict('TOOL_BUGS_BUTTON_OPEN_BUG'), 'OPEN', $form->getProp(CLX_PROP_ID));
            $buttons .= $c->source();
        }
        if (count($idlist) > CLX_NUM_ONE) {
            $c = new htmlFormSubmit(dict('GEN_LBL_VIEWNEXT_SOMETHING', 'record'), 'DISPLAY', $form->getProp(CLX_PROP_ID));
            $buttons .= $c->source();
        }
        $table->addCell($buttons, array(CLX_PROP_CLASS => CLX_CLASS_FOOTERS, CLX_PROP_COLSPAN => CLX_WIDTH_TWO));
        $form->setContents($table->source());
        $contents .= $form->source().		'<br>'.$te[1]["file"].",".$te[1]["line"].",".$te[1]["fucntion"]."<br>";

        return $contents;
    }

    function viewFormFill($record) {
        $contents = '';
        $table = new htmlTable(CLX_WIDTH_TWO);
        $table->setProps(array(CLX_PROP_CLASS => CLX_CLASS_FORM, CLX_PROP_WIDTH => CLX_PROP_100PCT));
        $table->addCell(dict('TOOL_BUGS_TITLE_LIST'), array(CLX_PROP_CLASS => CLX_CLASS_TITLE, CLX_PROP_COLSPAN => CLX_WIDTH_TWO));
        $table->addData(dict('GEN_LBL_REFERENCE_NO') . ':', $record['BUG_id']);
        $table->addData(dict('GEN_LBL_DATETIME') . ':', xFormat::datetime($record['BUG_cdate']));
        $table->addData(dict('GEN_LBL_URL') . ':', $record['BUG_url']);
        $table->addData(dict('GEN_LBL_USERNAME') . ':', $record['BUG_username']);
        $table->addData(dict('GEN_LBL_DESCRIPTION') . ':', $record['BUG_description']);
        $table->addData(dict('GEN_LBL_PRIORITY') . ':', $this->priorities[$record['BUGS_priority']]);
        $table->addData(dict('GEN_LBL_TYPE') . ':', $this->types[$record['BUG_type']]);
        $table->addData(dict('GEN_LBL_STATUS') . ':', $this->status[$record['BUG_status']]);
        $table->addData(dict('GEN_LBL_ERRORNUM') . ':', $record['BUG_errnum']);
        $table->addData(dict('GEN_LBL_ERRORMSG') . ':', $record['BUG_errmsg']);
        $contents .= $table->source();
        $text = new htmlTextArea('', $record['BUG_context'], CLX_NUM_30, CLX_NUM_120);
        $text->setProp('readonly', 'true');
        $contents .= buildPanelInterface(dict('GEN_LBL_CONTEXT'), 'context_panel', $text->source(), false);
        $NOTE = &getTool('NOTE');
        $contents .= $NOTE->makepanel('BUGS', $record['BUG_id']);
        return $contents;
    }

    function filterFormFill($msg, &$table) {
        // 	GEN_LBL_REFERENCE_NO
        $table->addField($msg, dict('GEN_LBL_REFERENCE_NO') . ':', 'BUGS_id', '', CLX_NUM_20, CLX_NUM_11);
        $pop = new htmlSelect('BUGS_priority');
        $pop->addItem('', '', true);
        foreach ($this->priorities as $key => $value) {
            $pop->addItem($value, $key, false);
        }
        $table->addData(dict('GEN_LBL_PRIORITY') . ':', $pop->source());
        $pop = new htmlSelect('BUGS_type');
        $pop->addItem('', '', true);
        foreach ($this->types as $key => $value) {
            $pop->addItem($value, $key, false);
        }
        $table->addData(dict('GEN_LBL_TYPE') . ':', $pop->source());
        $pop = new htmlSelect('BUGS_status');
        $pop->addItem('', '', true);
        foreach ($this->status as $key => $value) {
            $pop->addItem($value, $key, false);
        }
        $table->addData(dict('GEN_LBL_STATUS') . ':', $pop->source());
    }

    function setfilter() {
        $filter = '';
        $boolean = '';
        if (!empty(UserRequest::$post['BUGS_id'])) {
            $filter .= $boolean . ' ID=' . UserRequest::$post['BUGS_id'];
            $boolean = ' AND ';
        }
        if (UserRequest::$post['BUGS_status'] !== '') {
            $filter .= $boolean . ' STATUS=' . UserRequest::$post['BUGS_status'];
            $boolean = ' AND ';
        }
        if (UserRequest::$post['BUGS_priority'] !== '') {
            $filter .= $boolean . ' PRIORITY =' . UserRequest::$post['BUGS_priority'];
            $boolean = ' AND ';
        }
        if (UserRequest::$post['BUGS_type'] !== '') {
            $filter .= $boolean . ' TYPE=' . UserRequest::$post['BUGS_type'];
            $boolean = ' AND ';
        }
        $_SESSION['paging']['buglist']['criteria'] = $filter;
        $_SESSION['paging']['buglist']['startat'] = '0';
    }
	///////////////////////////////////////////////////
	///comentado 22/01/2019 para utilizar la clx_tool
	//////////////////////////////////////////////////
    /*function clearfilter() {
        $_SESSION['paging']['buglist']['criteria'] = '';
    }*/

    function loadToolRecord($recordid) {
        $values = ClxCommon::$db->getRecord('CLX_BUGS', 'ID= ' . $recordid);
        $record = array();
        $record['BUG_id'] = $values['ID'];
        $record['BUG_cdate'] = $values['CDATE'];
        $record['BUG_mdate'] = $values['MDATE'];
        $record['BUG_url'] = $values['URL'];
        $record['BUG_username'] = $values['USERNAME'];
        $record['BUG_description'] = $values['DESCRIPTION'];
        $record['BUG_context'] = $values['CONTEXT'];
        $record['BUGS_priority'] = $values['PRIORITY'];
        $record['BUG_type'] = $values['TYPE'];
        $record['BUG_status'] = $values['STATUS'];
        $record['BUG_errnum'] = $values['ERRNUM'];
        $record['BUG_errmsg'] = $values['ERRMSG'];
        return $record;
    }

    function createRecordVoid_() {
        ClxFreeReturn($this->createRecord_());
    }

    function createRecord_() {
        $d = xFormat::getNow('Y-m-d');
        $record = array();
        $record['CDATE'] = $d;
        $record['MDATE'] = $d;
        $record['USERNAME'] = $_SESSION['user_record']['username'];
        $record['URL'] = UserRequest::$post['BUGS_url'];
        $record['DESCRIPTION'] = UserRequest::$post['BUGS_description'];

        $record['CONTEXT'] = UserRequest::$post['BUGS_context'];
        $record['PRIORITY'] = (UserRequest::$post['BUGS_priority'] === '') ? CLX_NUM_ZERO : UserRequest::$post['BUGS_priority'];
        $record['TYPE'] = UserRequest::$post['BUGS_type'];
        $record['STATUS'] = UserRequest::$post['BUGS_status'];
        $record['ERRMSG'] = UserRequest::$post['BUGS_errmsg'];
        $record['ERRNUM'] = UserRequest::$post['BUGS_errnum'];
        $recordid = ClxCommon::$db->createRecord($this->tablename, $record, false);
        return $recordid;
    }

    function deleteRecordSk($id) {
        $NOTE = &getTool('NOTE');
        $notes = ClxCommon::$db->getRecords('CLX_NOTES', 'OWNERCODE=' . singleQuote($this->code), 'OWNERID=' . $id);
        while (($note = ClxCommon::$db->getNextRecord($notes))) {
            $NOTE->deleteRecordSk($note['ID']);
        }
        ClxCommon::$db->deleteRecords('CLX_BUGS', 'ID=' . $id);
    }

    function defaultRecord() {
        $d = xFormat::getNow('Y-m-d');
        $record = array();
        $record['BUG_id'] = '0';
        $record['BUG_cdate'] = $d;
        $record['BUG_mdate'] = $d;
        $record['BUG_url'] = '';
        $record['BUG_username'] = $_SESSION['user_record']['username'];
        $record['BUG_description'] = '';
        $record['BUG_context'] = '';
        $record['BUG_priority'] = '0';
        $record['BUG_type'] = '0';
        $record['BUG_status'] = '1';
        $record['BUG_errnum'] = '';
        $record['BUG_errmsg'] = '';
        return $record;
    }

    function autoreport($errno, $errmsg, $filename, $linenumber, $context) {
        $this->report($errno, $errmsg, $filename, $linenumber, $context, '0', 'Auto Report');
    }

    function report($errno, $errmsg, $filename, $linenumber, $context, $type, $description, $buttonGoto = 'index.php',$pipe=false,$showerror=true) {
		$GLOBALS['ERRORPROCESOBUG']='0';
	
		if(defined('BUGS_DEBUG')) {
			$this->debug = CLX_INIT_ZERO+BUGS_DEBUG;
		}
		$a_backtrace = array();
        $te=debug_backtrace(DEBUG_BACKTRACE_IGNORE_ARGS, CLX_NUM_TEN);
		for ($t=0;$t<18;$t++) { 
            if (isset($te[$t]['file']))	{
                $a_backtrace[] = str_replace(kPathToWebSite, '', $te[$t]['file']).' | '.$te[$t]['line'].' | '.$te[$t]['function'].'()';
            }
        }
		$siback=false;
		if (is_array($context)) {
			foreach($context as $xkey=>$xvalue){
				if (substr($xkey,0,4)=='DBAU') {
					$context=array($context,$a_backtrace );
	
					break;
				}	
			}	
		}
		


        if($this->debug != CLX_INIT_ZERO) {
            pre('$errno='.$errno);
            pre('$errmsg='.$errmsg);
            pre('$filename='.$filename);
            pre('$linenumber='.$linenumber);
            pre('$context');pre($context);
            pre('$type='.$type);
            pre('$description='.$description);
        }
        $contenidoHTML = '';
        $contents = '';
        switch ($errno) {
            case (E_USER_ERROR):
                $thislevel = CLX_INIT_ZERO;
                $leveltitle = 'ERROR';
                break;
            case (E_USER_WARNING):
                $thislevel = CLX_NUM_ONE;
                $leveltitle = 'WARNING';
                break;
            case (E_USER_NOTICE):
                $thislevel = CLX_NUM_TWO;
                $leveltitle = 'NOTICE';
                break;
            default:
                $thislevel = CLX_NUM_THREE;
                $leveltitle = 'UNKNOWN ALERT';
                break;
        }
        if($this->debug != CLX_INIT_ZERO) {
            pre('E_USER_ERROR='.E_USER_ERROR);
            pre('$thislevel='.$thislevel);
            pre('$leveltitle='.$leveltitle);
        }
        if($this->debug > CLX_NUM_ONE) {
			exit();
		}
        $text = new htmlTextArea('', print_r($context, true), CLX_NUM_30, CLX_NUM_120);
        $text->setProp('readonly', 'true');

        if ($this->alertlevel >= $thislevel) {
            $table = new htmlTable(CLX_WIDTH_TWO);
            $table->setProps(array(CLX_PROP_CLASS => CLX_CLASS_FORM));
            $table->addCell($leveltitle, array(CLX_PROP_CLASS => CLX_CLASS_TITLE, CLX_PROP_COLSPAN => CLX_NUM_TWO));
            $table->addData('Error number:', $errno);
            $table->addData('Error message:', $errmsg);
            $table->addData('File name:', $filename);
            $table->addData('Line number:', $linenumber);
            //$table->addData('Error context:', xDebug::dump($context));
            $table->addData('Error context:', $text->source());
            $contents .= $table->source();
            if (strlen($buttonGoto) > CLX_NUM_ZERO && !(isset($_SESSION['silent']) && $_SESSION['silent'] === true)) {
                $button = new htmlButtonGoto(dict('GEN_LBL_CONTINUE'), 'index.php');
                $table = new htmlTable(CLX_WIDTH_ONE);
                $table->addCell($button->source(), array());
                $contents .= $table->source();
            }
        }
        if ($this->savelevel >= $thislevel) {
            $d = xFormat::getNow('Y-m-d H:i:s');
            $record = array();
            $record['CDATE'] = $d;
            $record['MDATE'] = $d;
            $fl = htmlspecialchars($filename);
			if (is_array($linenumber)) {
				$linenumber=implode('',$linenumber);
			}
            $record['URL'] = ClxCommon::$db->addslashes($fl) . ' at line ' . $linenumber;
            $record['USERNAME'] = (!empty($_SESSION['user_record']['username'])) ? $_SESSION['user_record']['username'] : 'N/A';
            $fl = print_r($description, true);
            $record['DESCRIPTION'] = ClxCommon::$db->addslashes($fl); //$db->addslashes(htmlspecialchars($description));
            $fl = print_r($errmsg, true);
            $fl = print_r($context, true);
            $record['CONTEXT'] = ClxCommon::$db->addslashes($fl); //$db->addslashes(htmlspecialchars(xDebug::dump($context)));
            $record['TYPE'] = $type;
            $record['PRIORITY'] = (empty($thislevel)) ? CLX_NUM_ZERO : $thislevel;
            $record['STATUS'] = '1';
            $msg = substr(htmlspecialchars($errmsg), CLX_NUM_ZERO, $this->mensajesizerror-1);
            $record['ERRMSG'] = ClxCommon::$db->addslashes($msg);
            $record['ERRNUM'] = $errno;
            if (defined('CLX_ERROR_DONE')) {
                $record['CONTEXT'] = '';
                $record['URL'] = '';
                $record['DESCRIPTION'] = 'One or more error could not be stores. Verify request again';
                $record['ERRMSG'] = $record['DESCRIPTION'];
                return;
            } else {
                define('CLX_ERROR_DONE', CLX_NUM_ONE);
            }
            $error_id = ClxCommon::$db->createRecord('CLX_BUGS', $record, false);
            ClxCommon::$db->commit();
            addNotice(dict('GEN_ERR_CHECKTRAP'));
        }
		else {
		/*	$rra=fopen(kPathToWebSite."/".kPathToTemp."archivox3x.txt","a+");
			fwrite($rra,print_r($errmsg,true));
			fwrite($rra,print_r($context,true));
			
		
			fclose($rra);*/
			
		}
	$si_salir=CLX_NUM_ZERO;
	$si_content=true;
        if ($this->dielevel >= $thislevel) {
			if (defined('CLX_WEBSERVICEAPI_EXEC')) {
                echo utf8_encode($errmsg), CLX_NUM_500	;
				$si_content=false;
                $si_salir=CLX_NUM_ZERO;
			}	
            else if (defined('CLX_WEBSERVICE_EXEC')) {
				if ( !defined('CLX_WEBSERVICEAPI_EXEC'))
					throw new SOAPFault(utf8_encode($errmsg), CLX_NUM_500);
				else 
					echo utf8_encode($errmsg);
				$si_content=false;
                $si_salir=CLX_NUM_ZERO;
            } else {
                if (!empty($_SESSION['user_record'])) {
                    if (isset($_SESSION['silent']) && $_SESSION['silent'] === true) {
                        throw new Exception($contents);
                    } else {
                        if (kDefaultIsCron == CLX_NUM_ONE) {
                            $contenidoHTML.= dict('GEN_ERR_CHECKTRAP');
                            $PRSC = &getTool('PRSC');
                            $PRSC->setDieStatus();
                            ClxCommon::$db->commit();
                        } else {
                            $stime = microtime_float();
                            $PAGE_VARIABLES = getVariables();
                            $PAGE_VARIABLES['PAGE_TITLE'] = 'ERROR REPORT';
                            $PAGE_VARIABLES['CONTENTS'] = $contents;
                            //if not set is admin means that is not a log user then donot expose messages
                            if (!isset($_SESSION['user_record']['isadmin'])) {
                                $PAGE_VARIABLES['CONTENTS'] = note(dict('GEN_SYS_EXCEPTION', $error_id));
                            }
                            if (BUGS_DISPLAY_ADMIN_ONLY === CLX_NUM_ONE) {
                                if (empty($_SESSION['user_record']['isadmin'])) {
                                    $PAGE_VARIABLES['CONTENTS'] = note(dict('GEN_SYS_EXCEPTION', $error_id));
                                }
                            }
                            $PAGE_VARIABLES['PAGETIME'] = rounder((microtime_float() - $stime), CLX_NUM_FOUR);
                            $PAGE_VARIABLES['PAGEPEAKSIZE'] = rounder((memory_get_peak_usage(false) / CLX_NUM_1024), CLX_NUM_TWO) . ' Kb';
                            $contenidoHTML.= getSkinForm('pop.html', $PAGE_VARIABLES);
                        }
			$si_salir=CLX_NUM_ZERO;
                    }
                } else {
                    $contenidoHTML.= '<center><b>Internal Error</b><br>' . $errmsg . '</center>';
                    if (isset($_SESSION['silent']) && $_SESSION['silent'] === true) {
                        throw new Exception($contents);
                    } else {
                        $si_salir=CLX_NUM_ZERO;
                    }
                }
            }
        }
		if ($showerror) {
			if (isset($_SESSION["code_parameters"]["request_wind"]) ) {
				echo $contents;
			}
			else if ($si_content) {
	    echo $contenidoHTML;
		}	
		}
	//	$tta=fopen(kPathToWebSite."/".kPathToTemp."archivoX2.txt","a+");fwrite($tta,print_r($GLOBALS['PROCESS'] ,true));fwrite($tta," $this->savelevel ,, $thislevel  ".CHR(13).CHR(10));fwrite($tta," comte : ".strlen($contenidoHTML .$contents)  .CHR(13).CHR(10));fclose($tta);
		if ($si_salir===CLX_NUM_ZERO ){
			
			IF  (ISSET($GLOBALS['PROCESS']) && (strpos(' DBIM,MATC',$GLOBALS['PROCESS'])>0 )&& ( $contenidoHTML .$contents != '' ||$this->savelevel >= $thislevel)   ){
		//		$tta=fopen(kPathToWebSite."/".kPathToTemp."archivoX2.txt","a+");fwrite($tta,"S222222222".CHR(13).CHR(10));fclose($tta);
	    
				$GLOBALS['ERRORPROCESOBUG']='1';
			}	
	}    
		else if ($pipe===false || !(ISSET($GLOBALS['PROCESS']) && (strpos(' DBIM,MATC',$GLOBALS['PROCESS'])>0 )) ) {
			$tta=fopen(kPathToWebSite."/".kPathToTemp."archivoX2.txt","a+");fwrite($tta,"globals ".print_r($GLOBALS['PROCESS'] ,true));fwrite($tta," erroreslevel: $this->savelevel ,, $thislevel  ".CHR(13).CHR(10));fwrite($tta," contenido : ".strlen($contenidoHTML .$contents)  .CHR(13).CHR(10));fclose($tta);
			if ($showerror==true)
           salir($si_salir);
		}
		else {

		}
    }

}
