<?php

/**
 * Dynamic Relation Fields
 * For more information contact  support@alenet.com
 *
 * @author       $Author: jclerma $
 * @copyright    Alenet, Inc.
 * @since        $Date: 2017-11-03 15:09:40 -0400 (Fri, 03 Nov 2017) $
 * @version    	$Rev: 9615 $
 *
 * CHANGE LOG
 *   Initial load
 *
 */
class db_dyn_relation_columns extends clx_tool {

    /**
     * Constructor
     */
    function __construct() {
        $this->code = 'DBDC';
        $this->name = 'Relation Fields';
        $this->system = 'Database';
        $this->version = '0.1.1.0';


        $this->accesskey = 'DBDR';

        $this->tablename = 'CLX_DB_DYNRELATION_COLS';
        $this->record_name_singular = 'Relation Field';
        $this->record_name_plural = 'Relation fields';
        $this->primary_field = 'PARENT_TABLECODE';
        $this->secondary_field = 'CHILD_TABLECODE';
        $this->seqnum_field = 'SEQNUM';
        $this->primary_key = array('PARENT_TABLECODE', 'CHILD_TABLECODE', 'CHILD_FIELD', 'PARENT_FIELD');
    }

    /**
     * Describes the tool table structure.
     * @see clx_tool::toolTableStructure()
     */
    function toolTableStructure() {
        $fields = array();
        $fields['ID'] = array('type' => 'int(11) unsigned', 'null' => 'NO', 'properties' => 'auto_increment', 'index' => 'PRI');
        $fields['PARENT_TABLECODE'] = array('type' => 'varchar(4)', 'null' => 'NO', 'properties' => '');
        $fields['CHILD_TABLECODE'] = array('type' => 'varchar(4)', 'null' => 'NO', 'properties' => '');
        $fields['MN_TABLECODE'] = array('type' => 'varchar(4)', 'null' => 'YES', 'properties' => '');
        $fields['SEQNUM'] = array('type' => 'int(11) unsigned', 'null' => 'NO', 'properties' => '', 'default' => CLX_NUM_ZERO);
        $fields['PARENT_FIELD'] = array('type' => 'varchar(64)', 'null' => 'NO', 'properties' => '');
        $fields['CHILD_FIELD'] = array('type' => 'varchar(64)', 'null' => 'NO', 'properties' => '');
        $fields['COPY_FLAG'] = array('type' => 'tinyint(1) unsigned', 'null' => 'NO', 'properties' => '', 'default' => CLX_NUM_ZERO);
        $fields['DELETE_FLAG'] = array('type' => 'tinyint(1) unsigned', 'null' => 'NO', 'properties' => '', 'default' => CLX_NUM_ZERO);
        $indexes = array();
        $ipf = 'CLX' . $this->code;
        $indexes[$ipf . '01'] = array('index' => 'PRI', 'fieldname' => 'ID');
        $indexes[$ipf . '02'] = array('index' => 'UNI', 'fieldname' => 'PARENT_TABLECODE,CHILD_TABLECODE,PARENT_FIELD,CHILD_FIELD');
        return array('idcode' => $this->code, 'idname' => $this->name, 'name' => $this->tablename, 'type' => 'InnoDB',
            'fields' => $fields,
            'indexes' => $indexes);
    }

    /**
     * Unpack Record
     * @see clx_tool::unpack()
     */
    function unpack($parameters) {
        $parameters = array_values(array_merge(array('package' => null), $parameters));
        list($package) = $parameters;
        $record = $this->unpackRecord($package);
        ClxCommon::$db->createRecord($this->tablename, $record, false);
    }

//--  R E L A T I O N  F I E L D S
    /**
     * Returs array conteining fields on relation
     * @param string $parent_tablecode
     * @param string $child_tablecode
     * @return array
     */
    function getRelationFieldsArray($parent_tablecode, $child_tablecode) {
        $return = array();
        $sql = 'SELECT *  FROM ' . $this->tablename . '  WHERE PARENT_TABLECODE = ' . singleQuote($parent_tablecode) . ' AND CHILD_TABLECODE = ' . singleQuote($child_tablecode) . ' ORDER BY SEQNUM ASC';
        $rs = ClxCommon::$db->executeQuery($sql);
        while ($r = ClxCommon::$db->getNextRecord($rs, ASSOCIATIVE)) {
            $return[] = $r;
        }
        return $return;
    }

}
