<?php

/**
 * Tool Class
 * For more information contact  support@alenet.com
 *
 * @package		Clientix.Tool.Database
 * @abstract		Abstract Class for Clientix Tools
 * @author       $Author: cramirez $
 * @copyright    Alenet, Inc.
 * @since        $Date: 2018-09-24 07:14:14 -0400 (Mon, 24 Sep 2018) $
 * @version        $Rev: 10125 $
 *
 * CHANGE LOG
 *  Added getField Method.
 *  Oracle uppercase record changed
 *  Anchor field added
 *  Filter Removal Bug Fixed
 *  Added Version increase for tables when field is updated
 */
class db_fields extends clx_tool {

    /**
     * Constructor
     */
    function __construct() {
        $this->code = 'DBFL';
        $this->name = 'Field Manager';
        $this->system = 'Database';
        $this->version = '7.0.0.0';
        $this->has_access = true;
        $this->has_package = true;
        $this->tablename = 'CLX_DB_FIELDS';
        $this->record_name_singular = 'field definition';
        $this->record_name_plural = 'field definitions';
        $this->primary_field = 'FIELDNAME';
        $this->secondary_field = 'FIELDTYPE';
        $this->primary_key = array('FIELDNAME');

        $this->quickfilterform = array('panels' => array(
                array('type' => 'normal',
                    'entries' => array(
                        array('label' => 'GEN_LBL_FIELDNAME', 'fields' => array(array('name' => 'fieldname', 'type' => 'string', 'default' => '', 'size' => CLX_NUM_64, 'displaysize' => CLX_NUM_24))),
                		array('label' => 'GEN_LBL_FIELDFORMLABEL', 'fields' => array(array('name' => 'fieldformlabel', 'type' => 'string', 'default' => '', 'size' => CLX_NUM_64, 'displaysize' => CLX_NUM_24))),
                        array('label' => 'GEN_LBL_FIELDTYPE', 'fields' => array(array('name' => 'fieldtype', 'type' => 'string', 'default' => '', 'size' => CLX_NUM_FOUR, 'displaysize' => CLX_NUM_FOUR))),
                        array('label' => 'GEN_LBL_TABLECODE', 'fields' => array(array('name' => 'tablecode', 'type' => 'lookup', 'default' => '', 'tool' => 'DBTB')))
                    )
                )
            )
        );
    }

    /**
     * Install procedure
     *
     */
    function install() {
        $UIMI = &getTool('UIMI');
        $UIMI->add($this->code, 'MENU_TOOLS:MENU_DATABASE:MENU_FIELDS', 'TOOL', $this->code, 'ac=LIST');
        $d = xFormat::getNow('Y-m-d');
        $fieldrecord = array();
        $fieldrecord['VERSION'] = '1.0.2';
        $fieldrecord['VERSION_COUNTER'] = '0';
        $fieldrecord['CDATE'] = $d;
        $fieldrecord['MDATE'] = $d;
        $fieldrecord['USERNAME'] = 'admin';
        $fieldrecord['DESCRIPTION'] = 'This field works as a unique record identifier.';
        $fieldrecord['FIELDNAME'] = 'ID';
        $fieldrecord['FIELDTYPE'] = 'PRIM';
        $fieldrecord['FIELDPREFIX'] = '';
        $fieldrecord['FIELDSUFIX'] = '';
        $fieldrecord['FIELDLISTLABEL'] = 'GEN_LBL_ID';
        $fieldrecord['FIELDFORMLABEL'] = 'GEN_LBL_ID';
        $fieldrecord['FIELDFORMULA'] = '';
        $fieldrecord['FIELD_SIZE'] = CLX_INIT_ZERO;
        $fieldrecord['FIELD_REQUIRED'] = '1';
        $fieldrecord['FIELD_FORMAT'] = '';
        $fieldrecord['FIELD_ALIGN'] = '0';
        $fieldrecord['FIELD_DEFAULT'] = '';
        $fieldrecord['FIELD_MINVALUE'] = '';
        $fieldrecord['FIELD_MAXVALUE'] = '';
        $fieldrecord['FIELD_DISPLAYSIZE'] = '11';
        $fieldrecord['FIELD_ANCHOR'] = '';
        $fieldrecord['FIELD_EXTRA1'] = '';
        $fieldrecord['FIELD_EXTRA2'] = '';
        $fieldrecord['FIELD_EXTRA3'] = '';
        $fieldrecord['FIELD_EXTRA4'] = '';
        $fieldrecord['FIELD_EXTRA5'] = '';
        $fieldrecord['FIELD_EXTRA6'] = '';
        $fieldrecord['FIELD_EXTRA7'] = '';
        $fieldrecord['FIELD_EXTRA8'] = '';
        $fieldrecord['FIELD_EXTRA9'] = '';
        ClxCommon::$db->createRecordVoid($this->tablename, $fieldrecord, false);
    }

    /**
     * Describes the tool table structure.
     * @return array ('type' => 'varchar(255)', 'properties'=>'auto increment',
     * 				'index'=>'', default = '', 'dict'=>true)
     * 				 dict true or false means that field holds dictionary entry
     */
    function toolTableStructure() {
        $fields = array();
        $fields['ID'] = array('type' => 'int(11) unsigned', 'null' => 'NO', 'properties' => 'auto_increment', 'index' => 'PRI');
        $fields['VERSION'] = array('type' => 'varchar(32)', 'null' => 'NO', 'properties' => '');
        $fields['VERSION_COUNTER'] = array('type' => 'int(11) unsigned', 'null' => 'NO', 'properties' => '', 'default' => CLX_NUM_ZERO);
        $fields['CDATE'] = array('type' => 'datetime', 'null' => 'YES', 'properties' => '');
        $fields['MDATE'] = array('type' => 'datetime', 'null' => 'YES', 'properties' => '');
        $fields['USERNAME'] = array('type' => 'varchar(30)', 'null' => 'NO', 'properties' => '');
        $fields['DESCRIPTION'] = array('type' => 'mediumtext', 'properties' => '');
        $fields['FIELDNAME'] = array('type' => 'varchar(64)', 'null' => 'NO', 'properties' => '');
        $fields['FIELDFORMLABEL'] = array('type' => 'varchar(64)', 'properties' => '', 'dict' => true);
        $fields['FIELDLISTLABEL'] = array('type' => 'varchar(64)', 'properties' => '', 'dict' => true);
        $fields['FIELDPREFIX'] = array('type' => 'varchar(64)', 'properties' => '', 'dict' => true);
        $fields['FIELDSUFIX'] = array('type' => 'varchar(64)', 'properties' => '', 'dict' => true);
        $fields['FIELDTYPE'] = array('type' => 'varchar(4)', 'properties' => '');
        $fields['FIELDFORMULA'] = array('type' => 'mediumtext', 'properties' => '');
        // The next fields are controlled by the 'field' objects
        $fields['FIELD_SIZE'] = array('type' => 'int(11) unsigned', 'properties' => '');
        $fields['FIELD_REQUIRED'] = array('type' => 'tinyint(1) unsigned', 'null' => 'NO', 'properties' => '', 'default' => CLX_NUM_ZERO);
        $fields['FIELD_FORMAT'] = array('type' => 'varchar(255)', 'properties' => '');
        $fields['FIELD_ALIGN'] = array('type' => 'tinyint(1) unsigned', 'properties' => '');
        $fields['FIELD_DEFAULT'] = array('type' => 'varchar(255)', 'properties' => '');
        $fields['FIELD_MINVALUE'] = array('type' => 'varchar(255)', 'properties' => '');
        $fields['FIELD_MAXVALUE'] = array('type' => 'varchar(255)', 'properties' => '');
        $fields['FIELD_DISPLAYSIZE'] = array('type' => 'int(11) unsigned', 'null' => 'NO', 'properties' => '', 'default' => CLX_NUM_ZERO);
        $fields['FIELD_ANCHOR'] = array('type' => 'varchar(255)', 'properties' => '');
        $fields['FIELD_EXTRA1'] = array('type' => 'varchar(255)', 'properties' => '');
        $fields['FIELD_EXTRA2'] = array('type' => 'varchar(255)', 'properties' => '');
        $fields['FIELD_EXTRA3'] = array('type' => 'varchar(255)', 'properties' => '');
        $fields['FIELD_EXTRA4'] = array('type' => 'varchar(255)', 'properties' => '');
        $fields['FIELD_EXTRA5'] = array('type' => 'varchar(255)', 'properties' => '');
        $fields['FIELD_EXTRA6'] = array('type' => 'varchar(255)', 'properties' => '');
        $fields['FIELD_EXTRA7'] = array('type' => 'varchar(255)', 'properties' => '');
        $fields['FIELD_EXTRA8'] = array('type' => 'varchar(255)', 'properties' => '');
        $fields['FIELD_EXTRA9'] = array('type' => 'varchar(255)', 'properties' => '');
        $indexes = array();
        $ipf = 'CLX' . $this->code;
        $indexes[$ipf . '01'] = array('index' => 'PRI', 'fieldname' => 'ID');
        $indexes[$ipf . '02'] = array('index' => 'UNI', 'fieldname' => 'FIELDNAME');
        $indexes[$ipf . '03'] = array('index' => 'MUL', 'fieldname' => 'FIELD_REQUIRED');
        $indexes[$ipf . '04'] = array('index' => 'MUL', 'fieldname' => 'USERNAME');
        $indexes[$ipf . '05'] = array('index' => 'MUL', 'fieldname' => 'FIELDTYPE');
        return array('idcode' => $this->code, 'idname' => $this->name, 'name' => $this->tablename, 'type' => 'InnoDB',
            'fields' => $fields,
            'indexes' => $indexes);
    }

//-- F I E L D S    D E F I N I T I O N S
    /**
     * Pack tool record on xml Format
     * @param array record
     * @return obj
     */
  /*  function pack($record) {
        $o = $this->packRecord($record);
        $code = $record['FIELDNAME'];
        $objs = new XMLObject($this->code . '_OBJECTS');
        //Packing Dictionary
        $this->packsub($objs, 'DICT', 'SELECT DICT.* FROM CLX_DICTIONARY DICT, CLX_DB_FIELDS X WHERE (UPPER(DICT.ENTRY)=UPPER(X.FIELDLISTLABEL) OR UPPER(DICT.ENTRY)=UPPER(X.FIELDFORMLABEL) OR UPPER(DICT.ENTRY)=UPPER(X.FIELDPREFIX) OR UPPER(DICT.ENTRY)=UPPER(X.FIELDSUFIX)) AND X.FIELDNAME=' . singleQuote($code));
        $o->addObject($objs);
        return $o;
    }
*/

    function pack($record) {
        $o = $this->packRecord($record);
        $code = $record['FIELDNAME'];
        $objs = new XMLObject($this->code . '_OBJECTS');
        //Packing Dictionary
		
        $this->packsub($objs, 'DICT', '	SELECT X.fieldname,DICT.* FROM CLX_DICTIONARY DICT, CLX_DB_FIELDS X WHERE UPPER(DICT.ENTRY)=UPPER(X.FIELDLISTLABEL) UNION 
										SELECT X.fieldname,DICT.* FROM CLX_DICTIONARY DICT, CLX_DB_FIELDS X WHERE UPPER(DICT.ENTRY)=UPPER(X.FIELDFORMLABEL) UNION 
										SELECT X.fieldname,DICT.* FROM CLX_DICTIONARY DICT, CLX_DB_FIELDS X WHERE UPPER(DICT.ENTRY)=UPPER(X.FIELDPREFIX) UNION 
										SELECT X.fieldname,DICT.* FROM CLX_DICTIONARY DICT, CLX_DB_FIELDS X WHERE UPPER(DICT.ENTRY)=UPPER(X.FIELDSUFIX)', $code,true);
        $o->addObject($objs);
        return $o;
    }
	function packsub(&$objs, $toolcode, $query, $extrafield='',$cache=false) {
		if ($cache==false) {
			$records = ClxCommon::$db->executeQuery($query);
			$tool = &getTool($toolcode);
			while ($record = ClxCommon::$db->getNextRecord($records, ASSOCIATIVE)) {
				$o = $tool->pack($record);
				$objs->addObject($o);
			}
			unset($o);
		}
		else {
			if (!isset($GLOBALS['cache_fields'])) {
				$GLOBALS['cache_fields']=array();
				$records = ClxCommon::$db->executeQuery($query);
				while ($record = ClxCommon::$db->getNextRecord($records, ASSOCIATIVE)) {
					
					if (!isset($GLOBALS['cache_fields'][$record["fieldname"]]))
						$GLOBALS['cache_fields'][$record["fieldname"]]=array();
					$GLOBALS['cache_fields'][$record["fieldname"]][]=$record;
				
					
				}
				
			}
			$tool = &getTool($toolcode);
			$records=$GLOBALS['cache_fields'][$extrafield];
			foreach ($records as $record ) {
				$record2=$record;
				unset ($record2['fieldname']);
				$o = $tool->pack($record2);
				$objs->addObject($o);
			}
			unset($o);
			
		}
    }
    /**
     * Loads package into system
     * @param string $package
     */
    function unpack($parameters) {
        $parameters = array_values(array_merge(array('package' => null), $parameters));
        list($package) = $parameters;
        $record = $this->unpackRecord($package);
        $record['FIELDNAME'] = uppercase($record['FIELDNAME']);
        $record['FIELDFORMULA'] = stripslashes($record['FIELDFORMULA']);
        $record['FIELD_EXTRA1'] = uppercase($record['FIELD_EXTRA1']);
        $record['FIELD_EXTRA2'] = uppercase($record['FIELD_EXTRA2']);
        if ($record['FIELDTYPE'] <> 'FILE' && $record['FIELDTYPE'] <> 'PICT') {
            $record['FIELD_EXTRA3'] = uppercase($record['FIELD_EXTRA3']);
        }
        $record['FIELD_EXTRA4'] = uppercase($record['FIELD_EXTRA4']);
        $record['FIELD_EXTRA5'] = uppercase($record['FIELD_EXTRA5']);
        $record['FIELD_EXTRA6'] = uppercase($record['FIELD_EXTRA6']);
        $record['FIELD_EXTRA7'] = uppercase($record['FIELD_EXTRA7']);
        $record['FIELD_EXTRA8'] = uppercase($record['FIELD_EXTRA8']);
        $record['FIELD_EXTRA9'] = uppercase($record['FIELD_EXTRA9']);
        if (ClxCommon::$db->oneExistsRecords($this->tablename, $this->primary_field . '=' . singleQuote($record[$this->primary_field])) == CLX_NUM_ZERO) {
            ClxCommon::$db->createRecord($this->tablename, $record, false);
        } else {
            ClxCommon::$db->updateRecord($this->tablename, $record, $this->primary_field . '=' . singleQuote($record[$this->primary_field]));
        }
        if ($package->hasSubObject($this->code . '_OBJECTS')) {
            $objs = $package->getObjectXml($this->code . '_OBJECTS');
            foreach ($objs->objects as $o) {
                $toolcode = $o->tag;
                $tool = &getTool($toolcode);
                $tool->unpack(array('package' => $o));
            }
        }
    }

    /**
     * Action is called for every user interaction with the module...
     *
     * @param unknown_type $ac
     * @param unknown_type $idlist
     * @return unknown
     */
    function action($ac, $idlist) {
        if ($this->getAccess()->canRead()) {
            $contents = parent::action($ac, $idlist);
        } else {
            $contents = warning(dict('GEN_ERR_NO_ACCESS') . ' [DB_FIELDS-CAN_READ][' . $ac . '] ' . $this->getCode(false));
        }
        return $contents;
    }

    /**
     * Increase tool version
     * @param string $fieldname
     * @param string $fieldvalue
     */
    function versionIncrease($fieldname, $fieldvalue) {
        parent::versionIncrease($fieldname, $fieldvalue);
        $DBTB = &getTool('DBTB');
        $results = ClxCommon::$db->executeQuery('SELECT DBTB.IDCODE FROM CLX_DB_TABLES DBTB, CLX_DB_TABLEFIELDS DBTF, CLX_DB_FIELDS DBFL WHERE DBTF.TABLECODE = DBTB.IDCODE AND DBTF.FIELDNAME = DBFL.FIELDNAME AND DBFL.' . $fieldname . ' = ' . singleQuote($fieldvalue));
        while ($result = ClxCommon::$db->getNextRecord($results, ASSOCIATIVE)) {
            $DBTB->versionIncrease('IDCODE', $result['IDCODE']);
        }
    }

    /**
     * Builds listing on tool
     *
     * @param string $listingcode
     * @return string
     */
    function makelist($listingcode = null) {
        //$t = time('U'); genera un warning
        $t = time();
        $to = $this->code;
        $here = 'tool.php?to=' . $to . '&t=' . $t;
        clearHistory();
        pushHistory($this->code, 'LIST', array());
        $form = new htmlForm(CLX_PROP_ACTION, 'tool.php');
        $form->setProp(CLX_PROP_ID, $this->code.'List');
        $form->addArgument(CLX_PROP_TO, $this->code);
        $form->addArgument('ac', 'NOT_DEFINED');
        $selectclause = $this->code.'.*';
        $fromclause = 'CLX_DB_FIELDS '.$this->code;
        $whereclause = '';
        $sortclause = $this->code.'.FIELDNAME';
        $page = new xPaging();
        $page->pageInit($form->getProp(CLX_PROP_ID), $here, $selectclause, $fromclause, $whereclause, $sortclause, '');
        $table = new htmlTable(CLX_WIDTH_NINE);
        $table->setProps(array(CLX_PROP_CLASS => CLX_CLASS_LIST, CLX_PROP_WIDTH => CLX_PROP_100PCT));
        $table->addCell(dict('TOOL_DBFL_TITLE_LIST'), array(CLX_PROP_CLASS => CLX_CLASS_TITLE, CLX_PROP_COLSPAN => CLX_WIDTH_NINE));
        $page->header($table);
        $table->addCells(CLX_CLASS_HEADERS, array(dict('GEN_LBL_SEL'), dict('GEN_LBL_NAME'), dict('GEN_LBL_TYPE'), dict('GEN_LBL_VERSION'), dict('GEN_LBL_FORMLABEL'), dict('GEN_LBL_REQUIRED'), dict('GEN_LBL_DESCRIPTION'), dict('TOOL_DBTF_TITLE_FIELDS'), dict('TOOL_DBTD_TITLE_FIELDS')));
        $i = CLX_INIT_ZERO;
        $yesno = array(dict('GEN_LBL_NO'), dict('GEN_LBL_YES'));
        while ($record = $page->nextRecord()) {
            $recordid = $record['ID'];
            $select = new htmlCheckbox('selector_' . $recordid, false);
            $select->setProp('onclick', 'selectButtons(' . singleQuote($form->getProp(CLX_PROP_ID)) . ');');
            $table->addCell($select->source(), array(CLX_PROP_CLASS => CLX_CLASS_FOOTERS, CLX_PROP_WIDTH => '40', CLX_PROP_ALIGN => 'center'));
            if (empty($record['FIELD_REQUIRED']))
                $record['FIELD_REQUIRED'] = CLX_INIT_ZERO;
            $used = ClxCommon::$db->countRecords('CLX_DB_TABLEFIELDS', 'FIELDNAME=' . singleQuote($record['FIELDNAME']));
            $used_derived = ClxCommon::$db->countRecords('CLX_DB_TABLEDFIELDS', 'FIELDNAME=' . singleQuote($record['FIELDNAME']));
            $table->addCells(($i % CLX_NUM_TWO == CLX_NUM_ZERO) ? CLX_CLASS_DATA : CLX_CLASS_DATAALT, array($record['FIELDNAME'], $record['FIELDTYPE'], $record['VERSION'] . '-' . $record['VERSION_COUNTER'], dict($record['FIELDFORMLABEL']), $yesno[$record['FIELD_REQUIRED']], xFormat::text($record['DESCRIPTION']), $used, $used_derived));
            $i++;
        }
        $buttons = htmlButton::getButton('submit', dict('GEN_LBL_DISPLAY'), array('action' => 'DISPLAY', 'form' => $form->getProp(CLX_PROP_ID), CLX_PROP_ID => 'MULT'))->source();
        if ($this->getAccess()->canModify()) {
            $buttons .=htmlButton::getButton('submit', dict('GEN_LBL_MODIFY'), array('action' => 'MODIFY', 'form' => $form->getProp(CLX_PROP_ID), CLX_PROP_ID => 'MULT'))->source();
        }
        if ($this->getAccess()->canDelete()) {
            $buttons .=htmlButton::getButton('submit', dict('GEN_LBL_DELETE'), array('action' => 'DELETE', 'form' => $form->getProp(CLX_PROP_ID), CLX_PROP_ID => 'MULT'))->source();
        }
        if ($this->getAccess()->canDisplay()) {
            $buttons .=htmlButton::getButton('submit', dict('GEN_LBL_CREATE_SOMETHING', dict('GEN_LBL_FIELD')), array('action' => 'CREATE', 'form' => $form->getProp(CLX_PROP_ID), CLX_PROP_ID => 'ON'))->source();
        }
        $buttons .=htmlButton::getButton('submit', dict('GEN_LBL_FILTER'), array('action' => 'FILTER', 'form' => $form->getProp(CLX_PROP_ID), CLX_PROP_ID => 'ON'))->source();
        $form->addContents($page->footers($table, $buttons));
        return $form->source();
    }

    /**
     * Builds Tool Search form
     * @return string
     */
    function searchForm() {
        $msg = array();
        $contents = '';
        // Search Form
        $form = new htmlForm(CLX_PROP_ACTION, UserRequest::$server['PHP_SELF']);
        $form->addArgument(CLX_PROP_TO, $this->code);
        $form->addArgument('ac', 'SEARCH');
        $form->addArgument(CLX_PROP_TARGET, UserRequest::$request[CLX_PROP_TARGET]);
        $form->addArgument('confirm', '1');
        $table = new htmlTable(CLX_WIDTH_TWO);
        $table->setProps(array(CLX_PROP_CLASS => CLX_CLASS_FORM, CLX_PROP_WIDTH => CLX_PROP_100PCT));
        $table->addCell(dict('TOOL_DBFL_TITLE_SEARCH'), array(CLX_PROP_CLASS => CLX_CLASS_TITLE, CLX_PROP_COLSPAN => CLX_WIDTH_TWO));
        if (!array_key_exists('DBFL_fieldname', UserRequest::$post))
            UserRequest::$post['DBFL_fieldname'] = '';
        if (!array_key_exists('DBFL_fieldtype', UserRequest::$post))
            UserRequest::$post['DBFL_fieldtype'] = '';
        if (!array_key_exists('DBFL_fieldlistlabel', UserRequest::$post))
            UserRequest::$post['DBFL_fieldlistlabel'] = '';
        $table->addField($msg, dict('GEN_LBL_FIELDNAME') . ':', 'DBFL_fieldname', UserRequest::$post['DBFL_fieldname'], CLX_NUM_48, CLX_NUM_64);
        $table->addField($msg, dict('GEN_LBL_TYPE') . ':', 'DBFL_fieldtype', UserRequest::$post['DBFL_fieldtype'], CLX_NUM_FOUR, CLX_NUM_FOUR);
        $table->addField($msg, dict('GEN_LBL_LISTLABEL') . ':', 'DBFL_fieldlistlabel', UserRequest::$post['DBFL_fieldlistlabel'], CLX_NUM_32, CLX_NUM_32);
        $buttons = htmlButton::getButton('submit', dict('GEN_LBL_SEARCH'), array('action' => 'SEARCH', 'form' => $form->getProp(CLX_PROP_ID)))->source();
        $c = new htmlButtonGoto(dict('GEN_LBL_CANCEL'), 'javascript:closeWindow();');
        $buttons .= $c->source();
        $table->addCell($buttons, array(CLX_PROP_CLASS => CLX_CLASS_FOOTERS, CLX_PROP_COLSPAN => CLX_WIDTH_TWO));
        $form->setContents($table->source());
        $contents .= $form->source();
        // Search Procedure
        $table = new htmlTable(CLX_WIDTH_THREE);
        $table->setProps(array(CLX_PROP_CLASS => CLX_CLASS_LIST, CLX_PROP_WIDTH => CLX_PROP_100PCT));
        $criteria = '';
        $boolean = '';
        if (UserRequest::$post['DBFL_fieldname'] <> '') {
            $criteria .= $boolean . 'UPPER(FIELDNAME) LIKE ' . singleQuote(uppercase(UserRequest::$post['DBFL_fieldname']) . '%');
            $boolean = ' AND ';
        }
        if (UserRequest::$post['DBFL_fieldtype'] <> '') {
            $criteria .= $boolean . 'UPPER(FIELDTYPE) LIKE ' . singleQuote(uppercase(UserRequest::$post['DBFL_fieldtype']) . '%');
            $boolean = ' AND ';
        }
        if (UserRequest::$post['DBFL_fieldlistlabel'] <> '') {
            $criteria .= $boolean . 'UPPER(FIELDLISTLABEL) IN ( SELECT UPPER(X.ENTRY) FROM CLX_DICTIONARY X WHERE X.VALUE LIKE ' . singleQuote(UserRequest::$post['DBFL_fieldlistlabel']) . ' AND UPPER(X.LANGUAGE) = ' . singleQuote(uppercase($_SESSION ['prefs'] ['LANGUAGE'])) . ')';
            $boolean = ' AND ';
        }
        if (isset(UserRequest::$post['confirm'])) {
            $qty = ClxCommon::$db->oneExistsRecords($this->tablename, $criteria);
            if ($qty == CLX_NUM_ZERO) {
                $table->addCell(dict('TOOL_GEN_ERR_NOMATCH'), array(CLX_PROP_CLASS => CLX_CLASS_ALERT, CLX_PROP_COLSPAN => CLX_WIDTH_THREE));
            } else {
                $table->addCell(dict('TOOL_GEN_MSG_FOUNDRECORDS'), array(CLX_PROP_CLASS => CLX_CLASS_SUBTITLE, CLX_PROP_COLSPAN => CLX_WIDTH_THREE));
                $records = ClxCommon::$db->getRecords($this->tablename, $criteria);
                $i = CLX_INIT_ZERO;
                while ($record = ClxCommon::$db->getNextRecord($records, ASSOCIATIVE)) {
                    $js = 'javascript:setLookupValue(' . quote(UserRequest::$request[CLX_PROP_TARGET]) . ', ' . quote($record['FIELDNAME']) . ');';
                    $table->addCells(($i % CLX_NUM_TWO == CLX_NUM_ZERO) ? CLX_CLASS_DATA : CLX_CLASS_DATAALT, array('<page t=db_fields></page><a href=' . singleQuote($js) . '>' . $record['FIELDNAME'] . '</a>', $record['FIELDTYPE'], dict($record['FIELDLISTLABEL'])));
                    $i++;
                }
            }
        }
        $contents .= $table->source();
        return $contents;
    }

    /**
     * Tool Edit Form
     * @param string $ac
     * @param string $idlist
     * @param array $record
     * @param array $msg
     * @return string
     */
    function editForm($ac, $idlist, $record, $msg) {
        $DICT = &getTool('DICT');
        // Build/get the different javascripts...
        $OBJS = &getTool('OBJS');
        $fieldcodes = $OBJS->objectList('DBFT');
        $list = CLX_CHAR_TAB . 'panellist = new Array();' . CLX_CHAR_NL . CLX_CHAR_NL;
        $i = CLX_INIT_ZERO;
        foreach ($fieldcodes as $fieldcode) {
            $list .= sprintf(CLX_CHAR_TAB . 'panellist[%s] = ' . singleQuote('%s') . ';' . CLX_CHAR_NL, $i, $fieldcode);
            $i++;
        }

        $script = setJavaScriptTag($list, false);
        $variables = array();
        $variables['VISIBLEPANEL'] = 'DBFL_fieldtype';
        $topscript = setJavaScriptTag(getJavaScript('dynform_top.js', $variables));
        $topscript .= setJavaScriptTag(getJavaScript('DBFL_editform.js', $variables));
        $btmscript = setJavaScriptTag(getJavaScriptFilename('dynform_btm.js'), true);
        $btmscript .= setJavaScriptTag('initForm();', false);
        //Build the form
        $createfield = ($ac !== 'MODIFY') ? true : false;
        $title = ($createfield) ? dict('TOOL_DBFL_TITLE_CREATE') : dict('TOOL_DBFL_TITLE_MODIFY');
        $form = new htmlForm(CLX_PROP_ACTION, 'tool.php');
        $form->addArgument(CLX_PROP_TO, $this->code);
        $form->addArgument('ac', $ac);
        $form->addArgument('confirm', '1');
        $form->addArgument('DBFL_id', (!empty($record['DBFL_id'])) ? $record['DBFL_id'] : '');
        $form->addArgument('idlist', toString($idlist));
        //-- Headers
        $table = new htmlTable(CLX_WIDTH_TWO);
        $table->setProps(array(CLX_PROP_CLASS => CLX_CLASS_FORM, CLX_PROP_WIDTH => '90%'));
        $table->addCell($title, array(CLX_PROP_CLASS => CLX_CLASS_TITLE, CLX_PROP_COLSPAN => CLX_WIDTH_TWO));
        if (!empty($msg['DBFL_fieldname'])) {
            $table->addCell($msg['DBFL_fieldname'], array(CLX_PROP_CLASS => CLX_CLASS_ALERT, CLX_PROP_COLSPAN => CLX_WIDTH_TWO));
        }
        $f = new htmlField('DBFL_fieldname', $record['DBFL_fieldname'], CLX_NUM_64, CLX_NUM_64);
        $f->setProp('onchange', 'updateFields();');
        $table->addData(dict('GEN_LBL_NAME') . ':', $f->source());
        $table->addText($msg, dict('GEN_LBL_DESCRIPTION') . ':', 'DBFL_description', $record['DBFL_description'], CLX_NUM_FOUR, CLX_NUM_48);
        if (!empty($msg['DBFL_fieldtype'])) {
            $table->addCell($msg['DBFL_fieldtype'], array(CLX_PROP_CLASS => CLX_CLASS_ALERT, CLX_PROP_COLSPAN => CLX_WIDTH_TWO));
        }
        $fields = $OBJS->objectPop('DBFT', 'DBFL_fieldtype', $record['DBFL_fieldtype']);
        $fields->setProp('onchange', 'updateForm();');
        $table->addData(dict('GEN_LBL_TYPE'), $fields->source());
        $table->addField($msg, dict('GEN_LBL_VERSION') . ':', 'DBFL_version', $record['DBFL_version'], CLX_NUM_16, CLX_NUM_16);
        $form->addContents($table->source());
        //-- Field Types properties
        foreach ($fieldcodes as $fieldcode) {
            $fieldobject = getToolObject('DBFT', $fieldcode);
            $form->addContents($fieldobject->panel($msg, $record, $form));
        }
        if (UserRequest::$request['ac'] === 'MODIFY') {
            $table = new htmlTable(CLX_WIDTH_TWO);
            $table->setProps(array(CLX_PROP_CLASS => CLX_CLASS_FORM, CLX_PROP_WIDTH => '90%'));
            $table->addCell(dict('GEN_LBL_MASS_UPDATE_OPTIONS'), array(CLX_PROP_CLASS => CLX_CLASS_SUBTITLE, CLX_PROP_COLSPAN => CLX_WIDTH_TWO));
            $table->addYesNo(dict('GEN_LBL_PRESERVE_TFIELD'), 'DBFL_preserve_tablefield', CLX_NUM_ONE, CLX_NUM_ONE);
            $form->addContents($table->source());
        }
        //-- Labels
        $table = new htmlTable(CLX_WIDTH_FOUR, CLX_PROP_CLASS, CLX_CLASS_FORM, CLX_PROP_WIDTH, '90%');
        $table->addCell(dict('GEN_LBL_LABELS'), array(CLX_PROP_CLASS => CLX_CLASS_SUBTITLE, CLX_PROP_COLSPAN => CLX_WIDTH_FOUR));
        $table2 = new htmlTable(CLX_WIDTH_TWO);
        $table->setProps(array(CLX_PROP_WIDTH => CLX_PROP_100PCT));
        $DICT->editEntry($msg, dict('GEN_LBL_LISTLABEL') . ':', 'DBFL_fieldlistlabel', $record, $table2, $form);
        $table->addCell($table2->source(), array(CLX_PROP_COLSPAN => CLX_WIDTH_TWO));
        $table2 = new htmlTable(CLX_WIDTH_TWO);
        $table->setProps(array(CLX_PROP_WIDTH => CLX_PROP_100PCT));
        $DICT->editEntry($msg, dict('GEN_LBL_FORMLABEL') . ':', 'DBFL_fieldformlabel', $record, $table2, $form);
        $table->addCell($table2->source(), array(CLX_PROP_COLSPAN => CLX_WIDTH_TWO));
        $table2 = new htmlTable(CLX_WIDTH_TWO);
        $table->setProps(array(CLX_PROP_WIDTH => CLX_PROP_100PCT));
        $DICT->editEntry($msg, dict('GEN_LBL_PREFIX') . ':', 'DBFL_fieldprefix', $record, $table2, $form);
        $table->addCell($table2->source(), array(CLX_PROP_COLSPAN => CLX_WIDTH_TWO));
        $table2 = new htmlTable(CLX_WIDTH_TWO);
        $table->setProps(array(CLX_PROP_WIDTH => CLX_PROP_100PCT));
        $DICT->editEntry($msg, dict('GEN_LBL_SUFIX') . ':', 'DBFL_fieldsufix', $record, $table2, $form);
        $table->addCell($table2->source(), array(CLX_PROP_COLSPAN => CLX_WIDTH_TWO));
        $form->addContents($table->source());
        //-- Formula
        $table = new htmlTable(CLX_WIDTH_ONE);
        $table->setProps(array(CLX_PROP_CLASS => CLX_CLASS_FORM, CLX_PROP_WIDTH => '90%'));
        // Anchor
        $table->addCell(dict('GEN_LBL_ANCHOR'), array(CLX_PROP_CLASS => CLX_CLASS_SUBTITLE, CLX_PROP_COLSPAN => CLX_WIDTH_ONE));
        if (isset($msg['DBFL_field_anchor'])) {
            $this->addCell($msg['DBFL_field_anchor'], array(CLX_PROP_CLASS => CLX_CLASS_ALERT, CLX_PROP_COLSPAN => $this->width));
        }
        $c = new htmlField('DBFL_field_anchor', $record['DBFL_field_anchor'], CLX_NUM_HUNDRED, CLX_NUM_255);
        $table->addCell($c->source(), array(CLX_PROP_CLASS => CLX_CLASS_DATA));
        // Formula
        $table->addCell(dict('GEN_LBL_FORMULA'), array(CLX_PROP_CLASS => CLX_CLASS_SUBTITLE, CLX_PROP_COLSPAN => CLX_WIDTH_ONE));
        if (isset($msg['DBFL_fieldformula'])) {
            $this->addCell($msg['DBFL_fieldformula'], array(CLX_PROP_CLASS => CLX_CLASS_ALERT, CLX_PROP_COLSPAN => $this->width));
        }
        $c = new htmlTextArea('DBFL_fieldformula', $record['DBFL_fieldformula'], CLX_NUM_FOUR, CLX_NUM_72);
        $table->addCell($c->source(), array(CLX_PROP_CLASS => CLX_CLASS_DATA));
        $form->addContents($table->source());
        //-- Footers
        $buttons = '';
        $table = new htmlTable(CLX_WIDTH_TWO);
        ;
        $table->setProps(array(CLX_PROP_CLASS => CLX_CLASS_FORM, CLX_PROP_WIDTH => '90%'));
        if ($createfield === true) {
            $buttons .=htmlButton::getButton('submit', dict('GEN_LBL_CREATE'), array('action' => 'CREATE', 'form' => $form->getProp(CLX_PROP_ID)))->source();
        } else {
            $buttons .= htmlButton::getButton('submit', dict('GEN_LBL_SAVE'), array('action' => 'MODIFY', 'form' => $form->getProp(CLX_PROP_ID)))->source();
        }
        $c = new htmlButtonCancel();
        $buttons .= $c->source();
        $table->addCell($buttons, array(CLX_PROP_CLASS => CLX_CLASS_FOOTERS, CLX_PROP_COLSPAN => CLX_WIDTH_TWO));
        $form->addContents($table->source());
        return $script . $topscript . $form->source() . $btmscript;
    }

    /**
     * Verify Tool Edit Forms
     * @param array $msg
     * @return bool
     */
    function checkForm(&$msg) {
        $msg = array();
        $action = UserRequest::$post['ac'];
        UserRequest::$post['DBFL_fieldname'] = uppercase(UserRequest::$post['DBFL_fieldname']);
        xValidate::sqlobject($msg, 'DBFL_fieldname', UserRequest::$post['DBFL_fieldname']);
        if (!isset($msg['DBFL_fieldname'])) {
            if ($action !== 'MODIFY') {
                $qty = ClxCommon::$db->oneExistsRecords($this->tablename, 'FIELDNAME=' . singleQuote(UserRequest::$post['DBFL_fieldname']));
            } else {
                $qty = ClxCommon::$db->oneExistsRecords($this->tablename, 'FIELDNAME=' . singleQuote(UserRequest::$post['DBFL_fieldname']) . ' AND ID <> ' . UserRequest::$post['DBFL_id']);
            }
            if ($qty > CLX_NUM_ZERO) {
                $msg['DBFL_fieldname'] = dict('TOOL_DBFL_ERR_FIELDEXISTS');
            }
        }
        $DICT = &getTool('DICT');
        $DICT->validateEntry('DBFL_fieldlistlabel', UserRequest::$post, true, $msg);
        $DICT->validateEntry('DBFL_fieldformlabel', UserRequest::$post, true, $msg);
        $DICT->validateEntry('DBFL_fieldprefix', UserRequest::$post, false, $msg);
        $DICT->validateEntry('DBFL_fieldsufix', UserRequest::$post, false, $msg);
        $fieldobject = getToolObject('DBFT', UserRequest::$post['DBFL_fieldtype']);
        $fieldobject->checkPanel($msg);
        return count($msg) == CLX_NUM_ZERO;
    }

    /**
     * Adds Filter Form contents
     * @param array $msg
     * @param obj $table
     */
    function confirmDeleteFill(&$table, $record) {
        $qty = ClxCommon::$db->oneExistsRecords('CLX_DB_TABLEFIELDS', 'FIELDNAME=' . singleQuote($record['DBFL_fieldname']));
        if ($qty > CLX_NUM_ZERO) {
            $table->addData(dict('GEN_LBL_NAME') . ':', $record['DBFL_fieldname'] . ' ' . dict('TOOL_DBTF_MSG_USEDINTABLES', $qty));
        } else {
            $table->addData(dict('GEN_LBL_NAME') . ':', $record['DBFL_fieldname']);
        }
    }

    /**
     * Build display form contents
     * @param array $record
     * @return string
     */
    function viewFormFill($record) {
        $contents = '';
        // Headers
        $table = new htmlTable(CLX_WIDTH_TWO);
        $table->setProps(array(CLX_PROP_CLASS => CLX_CLASS_FORM, CLX_PROP_WIDTH => CLX_PROP_100PCT));
        $table->addCell(dict('GEN_LBL_FIELDNAME') . ': ' . $record['DBFL_fieldname'], array(CLX_PROP_CLASS => CLX_CLASS_TITLE, CLX_PROP_COLSPAN => CLX_WIDTH_TWO));
        $table->addData(dict('GEN_LBL_FIELDNAME') . ':', $record['DBFL_fieldname']);
        $table->addData(dict('GEN_LBL_VERSION') . ':', $record['DBFL_version'] . '-' . $record['DBFL_version_counter']);
        $table->addData(dict('GEN_LBL_DESCRIPTION') . ':', nl2br($record['DBFL_description']));
        $table->addData(dict('GEN_LBL_LISTLABEL') . ':', dict($record['DBFL_fieldlistlabel']));
        $table->addData(dict('GEN_LBL_FORMLABEL') . ':', dict($record['DBFL_fieldformlabel']));
        $table->addData(dict('GEN_LBL_PREFIX') . ':', dict($record['DBFL_fieldprefix']));
        $table->addData(dict('GEN_LBL_SUFIX') . ':', dict($record['DBFL_fieldsufix']));
        $contents .= $table->source();
        // Field Details
        $table = new htmlTable(CLX_WIDTH_TWO);
        $table->setProps(array(CLX_PROP_CLASS => CLX_CLASS_FORM, CLX_PROP_WIDTH => CLX_PROP_100PCT));
        $fieldinstance = getToolObject('DBFT', $record['DBFL_fieldtype']);
        $fieldinstance->viewForm($table, $record);
        $contents .= buildPanelInterface(dict('TOOL_DBFL_TITLE_FIELDDETAILS'), 'FIELDDETAILS', $table->source(), CLX_NUM_MINUS_1);
        // System Info
        $table = new htmlTable(CLX_WIDTH_TWO);
        $table->setProps(array(CLX_PROP_CLASS => CLX_CLASS_FORM, CLX_PROP_WIDTH => CLX_PROP_100PCT));
        $table->addData(dict('GEN_LBL_CDATE') . ':', $record['DBFL_cdate']);
        $table->addData(dict('GEN_LBL_MDATE') . ':', $record['DBFL_mdate']);
        $table->addData(dict('GEN_LBL_USERNAME') . ':', $record['DBFL_username']);
        $contents .= buildPanelInterface(dict('GEN_TTL_SYSTEMINFO'), 'SYSTEMINFO', $table->source(), CLX_NUM_MINUS_1);
        // Tables with this field
        $table = new htmlTable(CLX_WIDTH_FOUR);
        $table->setProps(array(CLX_PROP_CLASS => CLX_CLASS_FORM, CLX_PROP_WIDTH => CLX_PROP_100PCT));
        $table->addCells(CLX_CLASS_HEADERS, array(dict('GEN_LBL_MODULECODE'), dict('GEN_LBL_TABLECODE'), dict('GEN_LBL_TABLENAME'), dict('TOOL_DBTB_PROMPT_RECNAMEP')));
        $fieldname = $record['DBFL_fieldname'];
        $records = ClxCommon::$db->executeQuery(' 			SELECT t.MODULECODE, t.IDCODE, t.IDNAME, t.ID, t.RECORDNAME_PLURAL FROM CLX_DB_TABLES t, CLX_DB_TABLEFIELDS tf WHERE tf.FIELDNAME = ' . singleQuote($fieldname) . ' AND t.IDCODE = tf.TABLECODE ORDER BY t.MODULECODE ASC, t.IDCODE ASC');
        $i = CLX_INIT_ZERO;
        while ($record = ClxCommon::$db->getNextRecord($records, ASSOCIATIVE)) {
            $url = 'tool.php?to=DBTB&ac=DISPLAY&idlist=' . $record['ID'];
            $a = '<a href="' . $url . '">' . $record['IDCODE'] . '</a>';
            $table->addCells(($i % CLX_NUM_TWO == CLX_NUM_ZERO) ? CLX_CLASS_DATA : CLX_CLASS_DATAALT, array($record['MODULECODE'], $a, $record['IDNAME'], dict($record['RECORDNAME_PLURAL'])));
            $i++;
        }
        $contents .= buildPanelInterface(dict('TOOL_DBFL_TITLE_TABLEDETAILS'), 'TABLEDETAILS', $table->source(), CLX_NUM_MINUS_1);
        return $contents;
    }

    /**
     * Adds Filter Form contents
     * @param array $msg
     * @param obj $table
     */
    function filterFormFill($msg, &$table) {
        if (!array_key_exists('DBFL_fieldname', UserRequest::$post))
            UserRequest::$post['DBFL_fieldname'] = '';
        $table->addField($msg, dict('GEN_LBL_FIELDNAME') . ':', 'DBFL_fieldname', UserRequest::$post['DBFL_fieldname'], CLX_NUM_32, CLX_NUM_32);
        if (!array_key_exists('DBFL_fieldtype', UserRequest::$post))
            UserRequest::$post['DBFL_fieldtype'] = '';
		if (!array_key_exists('DBFL_fieldformlabel', UserRequest::$post))
            UserRequest::$post['DBFL_fieldformlabel'] = '';
        $table->addField($msg, dict('GEN_LBL_FIELDTYPE') . ':', 'DBFL_fieldtype', UserRequest::$post['DBFL_fieldtype'], CLX_NUM_FOUR, CLX_NUM_FOUR);
    }

    /**
     * Register filter information on user session
     */
    function setfilter() {
        $filter = '';
        $boolean = ' ';
        if (!empty(UserRequest::$post['DBFL_fieldname'])) {
            $filter .= $boolean . $this->code.'.FIELDNAME LIKE ' . singleQuote(UserRequest::$post['DBFL_fieldname'] . '%');
            $boolean = ' AND ';
        }
        if (!empty(UserRequest::$post['DBFL_fieldtype'])) {
            $filter .= $boolean . $this->code.'.FIELDTYPE LIKE ' . singleQuote(UserRequest::$post['DBFL_fieldtype'] . '%');
            $boolean = ' AND ';
        }

        if (!empty(UserRequest::$post['DBFL_tablecode'])) {
            $filter .= $boolean . 'EXISTS (SELECT T.ID FROM CLX_DB_TABLEFIELDS T WHERE T.TABLECODE = ' . singleQuote(UserRequest::$post['DBFL_tablecode']).' AND T.FIELDNAME = '.$this->code.'.FIELDNAME) ';
            $boolean = ' AND ';
        }
		if (!empty(UserRequest::$post['DBFL_listformlabel'])) {
            $filter .= $boolean . 'EXISTS (SELECT ENTRY FROM CLX_DICTIONARY T WHERE VALUE LIKE '. singleQuote('%'.UserRequest::$post['DBFL_listformlabel'].'%').'  AND T.ENTRY = '.$this->code.'.FIELDforLABEL)';
            $boolean = ' AND ';
        }
		if (!empty(UserRequest::$post['DBFL_fieldformlabel'])) {
            $filter .= $boolean . 'EXISTS (SELECT ENTRY FROM CLX_DICTIONARY T WHERE VALUE LIKE '. singleQuote('%'.UserRequest::$post['DBFL_fieldformlabel'].'%').'  AND T.ENTRY = '.$this->code.'.FIELDFORMLABEL)';
            $boolean = ' AND ';
        }
        $sortby = CLX_CHAR_EMPTY_STRING;
        $this->setFilterKeys($filter, $sortby, CLX_NUM_ZERO);
    }

    /**
     * Loads current record on selected id
     *
     * @param integer $id
     * @return array
     */
    function loadToolRecord($recordid) {
        $DICT = &getTool('DICT');
        $values = ClxCommon::$db->getRecord($this->tablename, 'ID= ' . $recordid);
        $record = array();
        $record['DBFL_id'] = $recordid;
        $record['DBFL_version'] = $values['VERSION'];
        $record['DBFL_version_counter'] = $values['VERSION_COUNTER'];
        $record['DBFL_cdate'] = xFormat::date($values['CDATE']);
        $record['DBFL_mdate'] = xFormat::date($values['MDATE']);
        $record['DBFL_username'] = $values['USERNAME'];
        $record['DBFL_description'] = $values['DESCRIPTION'];
        $record['DBFL_fieldname'] = $values['FIELDNAME'];
        $record['DBFL_fieldtype'] = $values['FIELDTYPE'];
        $DICT->loadEntry('DBFL_fieldprefix', $values['FIELDPREFIX'], $record);
        $DICT->loadEntry('DBFL_fieldsufix', $values['FIELDSUFIX'], $record);
        $DICT->loadEntry('DBFL_fieldlistlabel', $values['FIELDLISTLABEL'], $record);
        $DICT->loadEntry('DBFL_fieldformlabel', $values['FIELDFORMLABEL'], $record);
        $record['DBFL_field_anchor'] = $values['FIELD_ANCHOR'];
        $record['DBFL_fieldformula'] = $values['FIELDFORMULA'];
        $fieldobject = getToolObject('DBFT', $values['FIELDTYPE']);
        $fieldobject->loadPanel($record, $values);
        return $record;
    }

    /**
     * Update records
     *
     * @return integer
     */
    function saveRecord() {
        $d = xFormat::getNow('Y-m-d');
        $oldfieldrecord = ClxCommon::$db->getRecord($this->tablename, 'ID=' . UserRequest::$post['DBFL_id']);
        $fieldrecord = array();
        $fieldrecord['FIELDNAME'] = uppercase(UserRequest::$post['DBFL_fieldname']);
        $fieldrecord['VERSION'] = UserRequest::$post['DBFL_version'];
        $fieldrecord['MDATE'] = $d;
        $fieldrecord['DESCRIPTION'] = UserRequest::$post['DBFL_description'];
        $fieldrecord['FIELD_ANCHOR'] = UserRequest::$post['DBFL_field_anchor'];
        $fieldrecord['FIELDFORMULA'] = UserRequest::$post['DBFL_fieldformula'];
        $fieldrecord['FIELDTYPE'] = UserRequest::$post['DBFL_fieldtype'];
        $DICT = &getTool('DICT');
        $fieldrecord['FIELDLISTLABEL'] = $DICT->createEntry('DBFL_fieldlistlabel', UserRequest::$post);
        $fieldrecord['FIELDFORMLABEL'] = $DICT->createEntry('DBFL_fieldformlabel', UserRequest::$post);
        $fieldrecord['FIELDPREFIX'] = $DICT->createEntry('DBFL_fieldprefix', UserRequest::$post);
        $fieldrecord['FIELDSUFIX'] = $DICT->createEntry('DBFL_fieldsufix', UserRequest::$post);
        $fieldobject = getToolObject('DBFT', UserRequest::$post['DBFL_fieldtype']);
        $fieldobject->savePanel($fieldrecord);
        ClxCommon::$db->updateRecord($this->tablename, $fieldrecord, 'ID=' . UserRequest::$post['DBFL_id']);
        $this->versionIncrease('ID', UserRequest::$post['DBFL_id']);
        $this->updateField($oldfieldrecord, $fieldrecord);
        $this->clearHeap($fieldrecord['FIELDNAME']);
    }

    /**
     * Creates record
     *
     * @return integer
     */
    function createRecordVoid_() {
        ClxFreeReturn($this->createRecord_());
    }

    function createRecord_() {
		if (ClxCommon::$db->debugCF==true){
			echo "prepare create fields description fields";
			pre(UserRequest::$post);
			
		}	
        $d = xFormat::getNow('Y-m-d');
        $fieldrecord = array();
        $fieldrecord['FIELDNAME'] = UserRequest::$post['DBFL_fieldname'];
        $fieldrecord['VERSION'] = UserRequest::$post['DBFL_version'];
        $fieldrecord['VERSION_COUNTER'] = '0';
        $fieldrecord['CDATE'] = $d;
        $fieldrecord['MDATE'] = $d;
        $fieldrecord['USERNAME'] = $_SESSION['user_record']['username'];
        $fieldrecord['DESCRIPTION'] = UserRequest::$post['DBFL_description'];
        $fieldrecord['FIELDFORMULA'] = UserRequest::$post['DBFL_fieldformula'];
        $fieldrecord['FIELD_ANCHOR'] = UserRequest::$post['DBFL_field_anchor'];
        $fieldrecord['FIELDTYPE'] = UserRequest::$post['DBFL_fieldtype'];
        $DICT = &getTool('DICT');
        $fieldrecord['FIELDLISTLABEL'] = $DICT->createEntry('DBFL_fieldlistlabel', UserRequest::$post);
        $fieldrecord['FIELDFORMLABEL'] = $DICT->createEntry('DBFL_fieldformlabel', UserRequest::$post);
        $fieldrecord['FIELDPREFIX'] = $DICT->createEntry('DBFL_fieldprefix', UserRequest::$post);
        $fieldrecord['FIELDSUFIX'] = $DICT->createEntry('DBFL_fieldsufix', UserRequest::$post);
        $fieldobject = getToolObject('DBFT', UserRequest::$post['DBFL_fieldtype']);
        $fieldobject->savePanel($fieldrecord);
        $fieldrecordid = ClxCommon::$db->createRecord($this->tablename, $fieldrecord, false);
        $this->clearHeap($fieldrecord['FIELDNAME']);
        return $fieldrecordid;
    }

    /**
     * Deletes record on selected id
     *
     * @param integer $id
     */
    function deleteRecordSk($id) {
        $record = $this->loadToolRecord($id);
        $fieldobject = getToolObject('DBFT', $record['DBFL_fieldtype']);
        $fieldobject->deletePanel($record);
        switch (kDatabaseType) {
            case 'oci':
                $records = ClxCommon::$db->executeQuery('SELECT T.MODULECODE||' . singleQuote('_') . '||T.IDNAME TABLENAME, TF.FIELDNAME FIELDNAME FROM CLX_DB_TABLEFIELDS TF, CLX_DB_TABLES T WHERE T.IDCODE = TF.TABLECODE AND TF.FIELDNAME = ' . singleQuote($record['DBFL_fieldname']));
                break;
            default:
                $records = ClxCommon::$db->executeQuery('SELECT CONCAT(T.MODULECODE, ' . singleQuote('_') . ', T.IDNAME) TABLENAME, TF.FIELDNAME FIELDNAME FROM CLX_DB_TABLEFIELDS TF, CLX_DB_TABLES T WHERE T.IDCODE = TF.TABLECODE AND TF.FIELDNAME = ' . singleQuote($record['DBFL_fieldname']));
                break;
        }
        while ($tablefield = ClxCommon::$db->getNextRecord($records, ASSOCIATIVE)) {
            ClxCommon::$db->dropField($tablefield['TABLENAME'], $tablefield['FIELDNAME']);
        }
        ClxCommon::$db->deleteRecords('CLX_DB_TABLEFIELDS', 'FIELDNAME=' . singleQuote($record['DBFL_fieldname']));
        ClxCommon::$db->deleteRecords($this->tablename, 'ID=' . $id);
        $this->clearHeap($record['DBFL_fieldname']);
    }

    /**
     * Defines default record on initializacion
     *
     * @return array
     */
    function defaultRecord() {
        $record = array();
        $record['DBFL_fieldname'] = '';
        $record['DBFL_version'] = '1.0.0';
        $record['DBFL_cdate'] = '';
        $record['DBFL_mdate'] = '';
        $record['DBFL_username'] = $_SESSION['user_record']['username'];
        $record['DBFL_description'] = '';
        $record['DBFL_fieldformula'] = '';
        $record['DBFL_field_anchor'] = '';
        $record['DBFL_fieldtype'] = 'CHAR';
        $record['DBFL_fieldprefix'] = '';
        $record['DBFL_fieldprefix_DICT'] = '';
        $record['DBFL_fieldsufix'] = '';
        $record['DBFL_fieldsufix_DICT'] = '';
        $record['DBFL_fieldlistlabel'] = '';
        $record['DBFL_fieldlistlabel_DICT'] = '';
        $record['DBFL_fieldformlabel'] = '';
        $record['DBFL_fieldformlabel_DICT'] = '';
        $OBJS = &getTool('OBJS');
        $fieldcodes = $OBJS->objectList('DBFT');
        foreach ($fieldcodes as $fieldcode) {
            $fieldobject = getToolObject('DBFT', $fieldcode);
            $fieldobject->defaultPanel($record);
        }
        return $record;
    }

    /**
     * Builds field lookup object for tool form.
     * @param array $msg
     * @param string $prompt
     * @param string $name
     * @param string $default
     * @param obj $table
     * @return string
     */
    function placeChooseField($msg, $prompt, $name, $default, &$table) {

        $searchIcon = getSkinIconPop('view', dict('GEN_LBL_SEARCH'), 'DBFL', 'SEARCH', $name);
        if (!empty($msg[$name])) {
            $table->addCell($msg[$name], array(CLX_PROP_CLASS => CLX_CLASS_ALERT, CLX_PROP_COLSPAN => $table->width));
        }
        $table->addCell($prompt, array(CLX_PROP_CLASS => CLX_CLASS_PROMPT));
        $c = new htmlField($name, $default, CLX_NUM_32, CLX_NUM_64);
        $table->addCell($c->source() . $searchIcon, array(CLX_PROP_CLASS => CLX_CLASS_DATA));
        return CLX_CHAR_EMPTY_STRING;
    }

    /**
     * Validates if field exists
     * @param string $fieldname
     * @return bool
     */
    function fieldExists(&$fieldname) {
        $qty = ClxCommon::$db->oneExistsRecords($this->tablename, 'FIELDNAME=' . singleQuote($fieldname));
        if ($qty > CLX_NUM_ZERO) {
            $fieldname = ClxCommon::$db->getField($this->tablename, 'FIELDNAME', 'FIELDNAME=' . singleQuote($fieldname));
        }
        return ($qty > CLX_NUM_ZERO) ? true : false;
    }

    /**
     * Returns field record from tool metadata
     * @param string $fieldname
     * @return array
     */
    function getField($fieldname) {
        $fieldname = trim($fieldname);
        $fieldrec = array();
		$fetched = false;
		if (kApcTtl_7>0) $fieldrec = apc_fetchFct($findVar=kSpaceName . '.dbfl.' . $fieldname, $fetched);
		if (empty($fetched) or empty($fieldrec)) {
			$fieldrec = ClxCommon::$db->getRecord($this->tablename, 'FIELDNAME=' . singleQuote($fieldname));
			if (kApcTtl_7>0&&!empty($fieldrec)) {
				apc_addFct($findVar, $fieldrec,kApcTtl_7);
			}
		}
        
        return $fieldrec;
    }

    /**
     * Update field on database.
     * Method is called on field modification
     * @param array $oldfieldrecord
     * @param array $newfieldrecord
     */
    function updateField($oldfieldrecord, $newfieldrecord) {
        $table_altered_lst = array();
        $oldfieldname = $oldfieldrecord['FIELDNAME'];
        $newfieldname = $newfieldrecord['FIELDNAME'];
        if ($oldfieldname != $newfieldname) {
            $record = array();
            $record['FIELDNAME'] = $newfieldname;
            ClxCommon::$db->updateRecord('CLX_DB_TABLEFIELDS', $record, 'FIELDNAME=' . singleQuote($oldfieldname));
        }
        //updating required and table field options
        if (empty(UserRequest::$post['DBFL_preserve_tablefield'])) {
            $record = array();
            $record['FIELD_REQUIRED'] = $newfieldrecord['FIELD_REQUIRED'];
            $record['FIELD_DEFAULT'] = $newfieldrecord['FIELD_DEFAULT'];
            if ($newfieldrecord['FIELDTYPE'] === 'LOOK') {
                $record['CASCADE_DELETE'] = $newfieldrecord['FIELD_EXTRA5'];
            } else {
                $record['CASCADE_DELETE'] = CLX_INIT_ZERO;
            }
            ClxCommon::$db->updateRecord('CLX_DB_TABLEFIELDS', $record, 'FIELDNAME=' . singleQuote($newfieldname));
        }
        $field = getToolObject('DBFT', $newfieldrecord['FIELDTYPE']);
        switch (kDatabaseType) {
            case 'oci':
                $tables = ClxCommon::$db->executeQuery('SELECT t.MODULECODE||' . singleQuote('_') . '||t.IDNAME TABLENAME, f.FIELD_REQUIRED FROM CLX_DB_TABLES t, CLX_DB_TABLEFIELDS f WHERE t.IDCODE=f.TABLECODE AND t.TYPE <> ' . singleQuote('VIEW') . ' AND f.FIELDNAME=' . singleQuote($newfieldname));
                break;
            default:
                $tables = ClxCommon::$db->executeQuery('SELECT CONCAT(t.MODULECODE, ' . singleQuote('_') . ', t.IDNAME) TABLENAME, f.FIELD_REQUIRED FROM CLX_DB_TABLES t, CLX_DB_TABLEFIELDS f WHERE t.IDCODE=f.TABLECODE AND t.TYPE NOT IN(' . singleQuote('VIEW') . ',' . singleQuote('TEMP') . ') AND f.FIELDNAME=' . singleQuote($newfieldname));
                break;
        }
        while (($table = ClxCommon::$db->getNextRecord($tables))) {
            $newfieldrecord['FIELD_REQUIRED'] = $table['FIELD_REQUIRED'];
            if(ClxCommon::$db->tableExists($table['TABLENAME'])) {

                $field->modifyField($table['TABLENAME'], $oldfieldrecord, $table['TABLENAME'], $newfieldrecord);
                if(!isset($table_altered_lst[$table['TABLENAME']])) {
                    $table_altered_lst[$table['TABLENAME']] = 1;
                }
            }
        }
        // Elimina eventual tabla temporal (Oracle) de Audit
        foreach($table_altered_lst as $tablename => $val) {
            $pat_err_msg = ClxCommon::$db->postAlterTable($tablename);
            if($pat_err_msg !== '') {
                echo $pat_err_msg;
                return false;
            }
        }
    }

    /**
     * Create a physical field on database
     * called by the tablefield manager every time a field is included in a table
     * @param string $tablename
     * @param string $fieldname
     */
    function createPhysicalField($tablename, $fieldname) {
        if (!ClxCommon::$db->fieldExists($tablename, $fieldname)) {//Check Field Existance
            $fieldrec = ClxCommon::$db->getRecord($this->tablename, 'FIELDNAME=' . singleQuote($fieldname));
            $fieldobj = getToolObject('DBFT', $fieldrec['FIELDTYPE']);
            $fieldobj->createPhysicalFieldByType($tablename, $fieldrec);
        }
    }

    /**
     * Drops field on database
     * called by the tablefield manager every time a field is excluded from a table
     * @param string $tablename
     * @param string $fieldname
     */
    function deleteField($tablename, $fieldname) {
        $fieldrec = ClxCommon::$db->getRecord($this->tablename, 'FIELDNAME=' . singleQuote(uppercase($fieldname)));
        $fieldobj = getToolObject('DBFT', $fieldrec['FIELDTYPE']);
        $fieldobj->deleteField($tablename, $fieldrec);
    }

    /**
     * Clear field form heap
     * @param string $fieldname
     */
    private function clearHeap($fieldname) {
        apc_deleteFct(kSpaceName . '.dbfl.' . $fieldname);
    }

}
