<?php

/**
 * Tool Class
 * For more information contact  support@alenet.com
 *
 * @author       $Author: jclerma $
 * @copyright    Alenet, Inc.
 * @since        $Date: 2018-02-21 16:22:13 -0500 (Wed, 21 Feb 2018) $
 * @version        $Rev: 9738 $
 *
 * CHANGE LOG
 *  Oracle uppercase record changed
 *  Included label
 */
class db_list_items extends clx_tool {

    var $importform;

    function __construct() {
        $this->code = 'DBLI';
        $this->name = 'List Items';
        $this->system = 'Database';
        $this->version = '0.4.4.3';

        $this->accesskey = 'DBLT';


        $this->tablename = 'CLX_DB_LIST_ITEMS';
        $this->record_name_singular = 'list item';
        $this->record_name_plural = 'list items';
        $this->primary_field = 'IDCODE';
        $this->secondary_field = 'IDNAME';
        $this->seqnum_field = 'SEQNUM';
        $this->primary_key = array('LISTCODE', 'IDCODE');
        $this->importform = array(CLX_CLASS_TITLE => 'TOOL_DBLI_TITLE_IMPORT',
            CLX_CLASS_SUBTITLE => 'TOOL_DBLI_PROMPT_IMPORT',
            'panels' => array(
                array('type' => 'normal',
                    'entries' => array(
                        array('label' => 'GEN_LBL_FILE',
                            'fields' => array(
                                array('name' => 'filename', 'type' => 'file', 'validations' => array(
                                        array('type' => 'string', 'required' => '1'))
                                )
                            )
                        )
                    )
                )
            )
        );
    }

    function toolTableStructure() {
        $fields = array();
        $fields['ID'] = array('type' => 'int(11) unsigned', 'null' => 'NO', 'properties' => 'auto_increment', 'index' => 'PRI');
        $fields['LISTCODE'] = array('type' => 'varchar(4)', 'null' => 'NO', 'properties' => '');
        $fields['IDCODE'] = array('type' => 'varchar(10)', 'null' => 'YES', 'properties' => '');
        $fields['IDNAME'] = array('type' => 'varchar(255)', 'null' => 'NO', 'properties' => '', 'dict' => true);
        $fields['IDNAME2'] = array('type' => 'varchar(255)', 'null' => 'YES', 'properties' => '', 'dict' => true);
        $fields['PARENTITEM'] = array('type' => 'varchar(10)', 'null' => 'YES', 'properties' => '');
        $fields['SEQNUM'] = array('type' => 'int(11) unsigned', 'null' => 'NO', 'properties' => '', 'default' => CLX_NUM_ZERO);
        $fields['INACTIVE'] = array('type' => 'tinyint(1) unsigned', 'null' => 'NO', 'default' => '0', 'properties' => '');
        $indexes = array();
        $ipf = 'CLX' . $this->code;
        $indexes[$ipf . '01'] = array('index' => 'PRI', 'fieldname' => 'ID');
        $indexes[$ipf . '02'] = array('index' => 'UNI', 'fieldname' => 'LISTCODE,IDCODE');
        $indexes[$ipf . '03'] = array('index' => 'MUL', 'fieldname' => 'IDNAME');
        $indexes[$ipf . '04'] = array('index' => 'MUL', 'fieldname' => 'PARENTITEM');
        $indexes[$ipf . '05'] = array('index' => 'MUL', 'fieldname' => 'INACTIVE');
        $indexes[$ipf . '06'] = array('index' => 'MUL', 'fieldname' => 'LISTCODE');
        $indexes[$ipf . '07'] = array('index' => 'MUL', 'fieldname' => 'IDCODE');
        return array('idcode' => $this->code, 'idname' => $this->name, 'name' => $this->tablename, 'type' => 'InnoDB',
            'fields' => $fields,
            'indexes' => $indexes);
    }

    /**
     * (non-PHPdoc)
     * @see clx_tool::unpack()
     */
    function unpack($parameters) {
        $parameters = array_values(array_merge(array('package' => null), $parameters));
        list($package) = $parameters;
        $record = $this->unpackRecord($package);
        if (ClxCommon::$db->oneExistsRecords($this->tablename, 'UPPER(IDCODE)=' . singleQuote(uppercase($record['IDCODE'])) . ' AND LISTCODE=' . singleQuote($record['LISTCODE'])) == CLX_NUM_ZERO) {
            ClxCommon::$db->createRecord($this->tablename, $record, false);
        } else {
            ClxCommon::$db->updateRecord($this->tablename, $record, 'UPPER(IDCODE)=' . singleQuote(uppercase($record['IDCODE'])) . ' AND LISTCODE=' . singleQuote($record['LISTCODE']));
        }
    }

//-- L I S T    I T E M S
    function action($ac, $idlist) {
        if ($this->getAccess()->canRead()) {
            $msg = array();
            switch ($ac) {
                case 'IMPORT':
                    if (isset(UserRequest::$post['confirm'])) {
                        if ($this->checkImportForm($msg)) {
                            $contents = $this->import();
                            if (empty($contents)) {
                                $info = popHistory();
                                $contents = executeHistory($info);
                            }
                        } else {
                            $contents = $this->importForm($ac, array(), UserRequest::$post, $msg);
                        }
                    } else {
                        $record = array();
                        $contents = $this->importForm($ac, $idlist, $record, $msg);
                    }
                    break;
                default:
                    $contents = parent::action($ac, $idlist);
            }
        } else {
            $contents = warning(dict('GEN_ERR_NO_ACCESS') . ' [DB_LIST_ITEMS-CAN_READ][' . $ac . '] ' . $this->getCode(false));
        }
        return $contents;
    }

    function makepanel($listcode = null) {
        if ($this->getAccess()->canRead()) {
            $t = time();
            $to = $this->code;
            $here = 'tool.php?to=' . $to . '&t=' . $t;
            $form = new htmlForm(CLX_PROP_ACTION, 'tool.php');
            $form->setProp(CLX_PROP_ID, 'DBLIPanel');
            $form->addArgument(CLX_PROP_TO, $this->code);
            $form->addArgument('listcode', $listcode);
            $form->addArgument('ac', 'NOT_DEFINED');
            $selectclause = '*';
            $fromclause = 'CLX_DB_LIST_ITEMS';
            $whereclause = 'LISTCODE=' . singleQuote($listcode);
            $sortclause = 'SEQNUM';
            $page = new xPaging();
            $page->setProp('rpp', CLX_NUM_TEN_THOUSAND);
            $page->setProp('printheaders', false);
            $page->pageInit($form->getProp(CLX_PROP_ID), $here, $selectclause, $fromclause, $whereclause, $sortclause, '');
            $yesno = array(dict('GEN_LBL_NO'), dict('GEN_LBL_YES'));
            $DICT = getTool('DICT');
            $table = new htmlTable(CLX_WIDTH_SIX);
            $table->setProps(array(CLX_PROP_CLASS => CLX_CLASS_LIST, CLX_PROP_WIDTH => CLX_PROP_100PCT));
            $page->header($table);
            $table->addCells(CLX_CLASS_HEADERS, array(dict('GEN_LBL_SEL'), dict('GEN_LBL_CODE'), dict('GEN_LBL_NAME'), dict('GEN_LBL_NAME2'), dict('GEN_LBL_PARENTITEM'), dict('GEN_LBL_INACTIVE')));
            $i = CLX_INIT_ZERO;
            while ($record = $page->nextRecord()) {
                $recordid = $record['ID'];
                $select = new htmlCheckbox('selector_' . $recordid, false);
                $select->setProp('onclick', 'selectButtons(' . singleQuote($form->getProp(CLX_PROP_ID)) . ');');
                $table->addCell($select->source(), array(CLX_PROP_CLASS => CLX_CLASS_FOOTERS, CLX_PROP_WIDTH => '40', CLX_PROP_ALIGN => 'center'));
                $table->addCells(($i % CLX_NUM_TWO == CLX_NUM_ZERO) ? CLX_CLASS_DATA : CLX_CLASS_DATAALT, array($record['IDCODE'], $DICT->dictFastEdit($record['IDNAME']), $DICT->dictFastEdit($record['IDNAME2']), $record['PARENTITEM'], $yesno[$record['INACTIVE']]));
                $i++;
            }
            $buttons = '';
            $buttons .= htmlButton::getButton('submit', dict('GEN_LBL_DISPLAY'), array('action' => 'DISPLAY', 'form' => $form->getProp(CLX_PROP_ID), CLX_PROP_ID => 'MULT'))->source();
            if ($this->getAccess()->canModify()) {
                $buttons .=htmlButton::getButton('submit', dict('GEN_LBL_MODIFY'), array('action' => 'MODIFY', 'form' => $form->getProp(CLX_PROP_ID), CLX_PROP_ID => 'MULT'))->source();
            }
            if ($this->getAccess()->canDelete()) {
                $buttons .=htmlButton::getButton('submit', dict('GEN_LBL_DELETE'), array('action' => 'DELETE', 'form' => $form->getProp(CLX_PROP_ID), CLX_PROP_ID => 'MULT'))->source();
            }
            if ($this->getAccess()->canDisplay()) {
                $buttons .=htmlButton::getButton('submit', dict('GEN_LBL_CREATE_SOMETHING', dict('GEN_LBL_LISTITEM')), array('action' => 'CREATE', 'form' => $form->getProp(CLX_PROP_ID), CLX_PROP_ID => 'ON'))->source();
            }
            if ($this->getAccess()->canDisplay()) {
                $buttons .=htmlButton::getButton('submit', dict('GEN_LBL_IMPORT'), array('action' => 'IMPORT', 'form' => $form->getProp(CLX_PROP_ID), CLX_PROP_ID => 'ON'))->source();
            }
            if (($page->getProp('qty') > CLX_NUM_ONE) && ( $this->getAccess()->canModify())) {
                $buttons .=htmlButton::getButton('submit', dict('GEN_LBL_REORDER'), array('action' => 'REORDER', 'form' => $form->getProp(CLX_PROP_ID), CLX_PROP_ID => 'ON'))->source();
            }
            $form->addContents($page->footers($table, $buttons));
            $modulecode = '';
            return buildPanelInterface(dict('TOOL_DBLI_TITLE_LIST'), $modulecode . 'TABLES', $form->source(), CLX_NUM_MINUS_1);
        } else {
            return CLX_CHAR_EMPTY_STRING;
        }
    }

    /**
     * (non-PHPdoc)
     * @see clx_tool::editForm()
     */
    function editForm($ac, $idlist, $record, $msg) {
        $modify = (UserRequest::$request['ac'] === 'MODIFY');
        $title = ($modify) ? dict('TOOL_DBLI_TITLE_MODIFY') : dict('TOOL_DBLI_TITLE_CREATE');
        $form = new htmlForm(CLX_PROP_ACTION, 'tool.php');
        $form->addArgument(CLX_PROP_TO, $this->code);
        $form->addArgument('ac', $ac);
        $form->addArgument('confirm', '1');
        $form->addArgument('DBLI_id', $record['DBLI_id']);
        $form->addArgument('DBLI_listcode', $record['DBLI_listcode']);
        $form->addArgument('idlist', toString($idlist));
        $table = new htmlTable(CLX_WIDTH_TWO);
        $table->setProps(array(CLX_PROP_CLASS => CLX_CLASS_FORM, CLX_PROP_WIDTH => CLX_PROP_100PCT));
        $table->addCell($title, array(CLX_PROP_CLASS => CLX_CLASS_TITLE, CLX_PROP_COLSPAN => CLX_WIDTH_TWO));
        if (!empty($modify)) {
            $table->addData(dict('GEN_LBL_CODE') . ':', $record['DBLI_idcode']);
            $form->addArgument('DBLI_idcode', $record['DBLI_idcode']);
        } else {
            $table->addField($msg, dict('GEN_LBL_CODE') . ':', 'DBLI_idcode', $record['DBLI_idcode'], CLX_NUM_TEN, CLX_NUM_TEN);
        }
        $DICT = &getTool('DICT');
        $DICT->editEntry($msg, dict('GEN_LBL_NAME') . ':', 'DBLI_idname', $record, $table, $form);
        $DICT->editEntry($msg, dict('GEN_LBL_NAME2') . ':', 'DBLI_idname2', $record, $table, $form);
        $DBLT = &getTool('DBLT');
        $parentlist = ClxCommon::$db->getField($DBLT->tablename, 'PARENTLIST', 'IDCODE =' . singleQuote($record['DBLI_listcode']));
        $form->addArgument('DBLI_parentlist', $parentlist);
        $form->addContents($this->placeChooseListItem($msg, 'DBLI_parentlist', dict('GEN_LBL_PARENTITEM') . ':', 'DBLI_parentitem', $record['DBLI_parentitem'], $table));
        $pop = new HTMLSelect('DBLI_inactive');
        $pop->addItem(dict('GEN_LBL_NO'), '0', $record['DBLI_inactive']);
        $pop->addItem(dict('GEN_LBL_YES'), '1', $record['DBLI_inactive']);
        $table->addData(dict('GEN_LBL_INACTIVE') . ':', $pop->source());
        $form->addContents($table->source() . '<br>');
        $table = new htmlTable(CLX_WIDTH_TWO);
        $table->setProps(array(CLX_PROP_CLASS => CLX_CLASS_FORM, CLX_PROP_WIDTH => CLX_PROP_100PCT));
        $buttons = '';
        if (!empty($modify)) {
            $buttons .= htmlButton::getButton('submit', dict('GEN_LBL_SAVE'), array('action' => 'MODIFY', 'form' => $form->getProp(CLX_PROP_ID)))->source();
        } else {
            $buttons .=htmlButton::getButton('submit', dict('GEN_LBL_CREATE'), array('action' => 'CREATE', 'form' => $form->getProp(CLX_PROP_ID), CLX_PROP_ID => 'ON'))->source();
        }
        $c = new htmlButtonCancel();
        $buttons .= $c->source();
        $table->addCell($buttons, array(CLX_PROP_CLASS => CLX_CLASS_FOOTERS, CLX_PROP_COLSPAN => CLX_WIDTH_TWO));
        $form->addContents($table->source());
        return $form->source();
    }

    function importForm($ac, $idlist, $record, $msg) {
        $form = new clx_form($this->code, $this->importform);
        return $form->editForm($ac, $idlist, $record, $msg);
    }

    function checkImportForm(&$msg) {
        $form = new clx_form($this->code, $this->importform);
        $result = $form->checkForm1(UserRequest::$post['ac'], UserRequest::$post, $msg);
        return $result;
    }

    function checkForm(&$msg) {
        $msg = array();
        $action = UserRequest::$post['ac'];
        if ($action === 'CREATE') {
            xValidate::unique($msg, 'DBLI_idcode', 'IDCODE', $this->tablename, 'IDCODE=' . singleQuote(UserRequest::$post['DBLI_idcode']), 'LISTCODE=' . singleQuote(UserRequest::$post['DBLI_listcode']));
        }
        $DICT = &getTool('DICT');
        $DICT->validateEntry('DBLI_idname', UserRequest::$post, true, $msg);
        return count($msg) == CLX_NUM_ZERO;
    }

    function searchForm() {
        $DBLT = &getTool('DBLT');
        $contents = '';
        $msg = array();
        if (empty(UserRequest::$request['listcode'])) {
            $contents = note('Before selecting a list item, please select a list in the above field.');
        } elseif (!$DBLT->listExists(UserRequest::$request['listcode'])) {
            $contents = note('The selected list code does not exist: ' . UserRequest::$request['listcode']);
        } else {
            if (!array_key_exists('DBLI_idcode', UserRequest::$post))
                UserRequest::$post['DBLI_idcode'] = '';
            if (!array_key_exists('DBLI_idname', UserRequest::$post))
                UserRequest::$post['DBLI_idname'] = '';
            if (!array_key_exists('DBLI_idname2', UserRequest::$post))
                UserRequest::$post['DBLI_idname2'] = '';
            $listcode = UserRequest::$request['listcode'];
            // Search Form
            $form = new htmlForm(CLX_PROP_ACTION, UserRequest::$server['PHP_SELF']);
            $form->addArgument(CLX_PROP_TO, $this->code);
            $form->addArgument('ac', 'SEARCH');
            $form->addArgument(CLX_PROP_TARGET, UserRequest::$request[CLX_PROP_TARGET]);
            $form->addArgument('listcode', $listcode);
            $form->addArgument('confirm', '1');
            $table = new htmlTable(CLX_WIDTH_TWO);
            $table->setProps(array(CLX_PROP_CLASS => CLX_CLASS_FORM, CLX_PROP_WIDTH => CLX_PROP_100PCT));
            $table->addCell(dict('TOOL_DBLI_TITLE_SEARCH'), array(CLX_PROP_CLASS => CLX_CLASS_TITLE, CLX_PROP_COLSPAN => CLX_WIDTH_TWO));
            $table->addField($msg, dict('GEN_LBL_CODE') . ':', 'DBLI_idcode', UserRequest::$post['DBLI_idcode'], CLX_NUM_TEN, CLX_NUM_TEN);
            $table->addField($msg, dict('GEN_LBL_NAME') . ':', 'DBLI_idname', UserRequest::$post['DBLI_idname'], CLX_NUM_32, CLX_NUM_48);
            $table->addField($msg, dict('GEN_LBL_NAME2') . ':', 'DBLI_idname2', UserRequest::$post['DBLI_idname2'], CLX_NUM_32, CLX_NUM_48);
            $buttons = htmlButton::getButton('submit', dict('GEN_LBL_SEARCH'), array('action' => 'SEARCH', 'form' => $form->getProp(CLX_PROP_ID)))->source();
            $c = new htmlButtonGoto(dict('GEN_LBL_CANCEL'), 'javascript:closeWindow();');
            $buttons .= $c->source();
            $table->addCell($buttons, array(CLX_PROP_CLASS => CLX_CLASS_FOOTERS, CLX_PROP_COLSPAN => CLX_WIDTH_TWO));
            $form->setContents($table->source());
            $contents .= $form->source();
            // Search Procedure
            $table = new htmlTable(CLX_WIDTH_ONE);
            $table->setProps(array(CLX_PROP_CLASS => CLX_CLASS_LIST, CLX_PROP_WIDTH => CLX_PROP_100PCT));
            $criteria = 'INACTIVE = 0 AND LISTCODE=' . singleQuote($listcode);
            $boolean = ' and ';
            if (UserRequest::$post['DBLI_idcode'] <> '') {
                $criteria .= $boolean . 'UPPER(IDCODE) LIKE ' . singleQuote(uppercase(UserRequest::$post['DBLI_idcode']) . '%');
                $boolean = ' and ';
            }
            if (UserRequest::$post['DBLI_idname'] <> '') {
                $criteria .= $boolean . 'UPPER(IDNAME) LIKE ' . singleQuote(uppercase(clx_change_especial_char_html (UserRequest::$post['DBLI_idname'])) . '%');
                $boolean = ' and ';
            }
            $qty = ClxCommon::$db->oneExistsRecords($this->tablename, $criteria);
            if ($qty == CLX_NUM_ZERO) {
                $table->addCell('No records match your description.', array(CLX_PROP_CLASS => CLX_CLASS_ALERT, CLX_PROP_COLSPAN => CLX_WIDTH_ONE));
            } else {
                $table->addCell('Found list item(s)', array(CLX_PROP_CLASS => CLX_CLASS_SUBTITLE, CLX_PROP_COLSPAN => CLX_WIDTH_ONE));
                $records = ClxCommon::$db->getRecords($this->tablename, $criteria);
                $i = CLX_INIT_ZERO;
                while ($record = ClxCommon::$db->getNextRecord($records, ASSOCIATIVE)) {
                    $js = 'javascript:setLookupValue(' . quote(UserRequest::$request[CLX_PROP_TARGET]) . ', ' . quote($record['IDCODE']) . ');';
                    $table->addCells(($i % CLX_NUM_TWO == CLX_NUM_ZERO) ? CLX_CLASS_DATA : CLX_CLASS_DATAALT, array('<page t=db_list_items></page><a href=' . singleQuote($js) . '>' . dict($record['IDNAME']) . ' (' . $record['IDCODE'] . ')</a>'));
                    $i++;
                }
            }
            $contents .= $table->source();
        }
        return $contents;
    }

    function confirmDeleteFill(&$table, $record) {
        $table->addData(dict('GEN_LBL_CODE') . ':', $record['DBLI_idcode']);
        $table->addData(dict('GEN_LBL_NAME') . ':', dict($record['DBLI_idname']));
        $table->addData(dict('GEN_LBL_NAME2') . ':', dict($record['DBLI_idname2']));
    }

    function reorderRecords() {
        $records = ClxCommon::$db->getRecords($this->tablename, 'LISTCODE=' . singleQuote(UserRequest::$post['listcode']) . ' ORDER BY SEQNUM');
        $list = array();
        while ($record = ClxCommon::$db->getNextRecord($records, ASSOCIATIVE)) {
            $c = dict($record['IDNAME']) . ' (' . $record['IDCODE'] . ')';
            $c = str_replace(CLX_CHAR_NL, '', $c);
            $c = trim(str_replace(CLX_CHAR_CR, '', $c));
            $list[$record['ID']] = $c;
        }
        return $list;
    }

    function viewFormFill($record) {
        addHistory('tablecode', $record['DBLI_idcode']);
        $contents = '';
        $table = new htmlTable(CLX_WIDTH_TWO);
        $table->setProps(array(CLX_PROP_CLASS => CLX_CLASS_FORM, CLX_PROP_WIDTH => CLX_PROP_100PCT));
        $table->addCell(dict('TOOL_DBLI_TITLE_VIEW'), array(CLX_PROP_CLASS => CLX_CLASS_TITLE, CLX_PROP_COLSPAN => CLX_WIDTH_TWO));
        $table->addData(dict('GEN_LBL_LISTCODE') . ':', $record['DBLI_listcode']);
        $table->addData(dict('GEN_LBL_CODE') . ':', $record['DBLI_idcode']);
        $table->addData(dict('GEN_LBL_NAME') . ':', dict($record['DBLI_idname']));
        $table->addData(dict('GEN_LBL_NAME2') . ':', dict($record['DBLI_idname2']));
        $table->addData(dict('GEN_LBL_INACTIVE') . ':', dict($this->yesno[$record['DBLI_inactive']]));
        $contents .= $table->source();
        return $contents;
    }

    function loadToolRecord($recordid) {
        $DICT = &getTool('DICT');
        $values = ClxCommon::$db->getRecord($this->tablename, 'ID= ' . $recordid);
        $record = array();
        $record['DBLI_id'] = $recordid;
        $record['DBLI_listcode'] = $values['LISTCODE'];
        $record['DBLI_idcode'] = $values['IDCODE'];
        $DICT->loadEntry('DBLI_idname', $values['IDNAME'], $record);
        $DICT->loadEntry('DBLI_idname2', $values['IDNAME2'], $record);
        $record['DBLI_seqnum'] = $values['SEQNUM'];
        $record['DBLI_parentitem'] = $values['PARENTITEM'];
        $record['DBLI_inactive'] = $values['INACTIVE'];
        return $record;
    }

    function saveRecord() {
        $DICT = &getTool('DICT');
        $record = array();
        $record['IDNAME'] = $DICT->saveEntry('DBLI_idname', UserRequest::$post);
        $record['IDNAME2'] = $DICT->saveEntry('DBLI_idname2', UserRequest::$post);
        $record['PARENTITEM'] = UserRequest::$post['DBLI_parentitem'];
        $record['INACTIVE'] = UserRequest::$post['DBLI_inactive'];
        ClxCommon::$db->updateRecord($this->tablename, $record, 'ID=' . UserRequest::$post['DBLI_id']);
        $DBLT = &getTool('DBLT');
        $DBLT->versionIncrease('IDCODE', UserRequest::$post['DBLI_listcode']);
        $this->clearHeap();
    }

    function createRecord_() {
        $DICT = &getTool('DICT');
        $record = array();
        $record['LISTCODE'] = UserRequest::$post['DBLI_listcode'];
        $record['IDCODE'] = UserRequest::$post['DBLI_idcode'];
        $record['PARENTITEM'] = UserRequest::$post['DBLI_parentitem'];
        $record['IDNAME'] = $DICT->createEntry('DBLI_idname', UserRequest::$post);
        $record['IDNAME2'] = $DICT->createEntry('DBLI_idname2', UserRequest::$post);
        $record['SEQNUM'] = ClxCommon::$db->getOne('SELECT MAX(SEQNUM) FROM ' . $this->tablename . ' WHERE LISTCODE=' . singleQuote(UserRequest::$post['DBLI_listcode'])) + CLX_NUM_ONE;
        $record['INACTIVE'] = UserRequest::$post['DBLI_inactive'];
        $recordid = ClxCommon::$db->createRecord($this->tablename, $record, false);
        $DBLT = &getTool('DBLT');
        $DBLT->versionIncrease('IDCODE', UserRequest::$post['DBLI_listcode']);
        return $recordid;
    }

    /**
     * Creates list item
     * @param type $listcode
     * @param type $label
     * @param type $item_code
     * @param type $parent
     */
    function createItem($listcode, $label, $item_code = '', $parent = '') {
        $DICT = &getTool('DICT');
        $record = array();
        $record['LISTCODE'] = $listcode;
        $record['PARENTITEM'] = $parent;
        //Test if entry exists for list with sugested label to prevent duplicates if posible
        $sql = 'SELECT DBLI.IDCODE FROM CLX_DB_LIST_ITEMS DBLI, CLX_DICTIONARY DICT WHERE DBLI.IDNAME = DICT.ENTRY AND DBLI.LISTCODE = ' . singleQuote($listcode) . '  AND DICT.VALUE = ' . singleQuote($label) . '  ';
        if ($parent <> '')
            $sql .= 'AND  DBLI.PARENTITEM = ' . singleQuote($parent);
        $found = ClxCommon::$db->getOne($sql);
        if (!empty($found) && $found <> '0') {
            return $found; //retornando item que ya existe
        }
        $record['IDNAME'] = $DICT->createEntryByValue($label);
        $record['SEQNUM'] = ClxCommon::$db->getOne('SELECT MAX(SEQNUM) FROM ' . $this->tablename . '  WHERE LISTCODE=' . singleQuote($listcode)) + CLX_NUM_ONE;
        $record['INACTIVE'] = CLX_INIT_ZERO;
        $sw = false;
        $count = 0;
        if ($item_code === '') {
            do {
                $qty = ClxCommon::$db->getOne('SELECT COUNT(SEQNUM) FROM ' . $this->tablename . '  WHERE LISTCODE=' . singleQuote($listcode)) + CLX_NUM_ONE + $count;
                $record['IDCODE'] = $record['LISTCODE'] . '_' . $qty;
                $search = ClxCommon::$db->getOne('SELECT COUNT(SEQNUM) FROM ' . $this->tablename . '  WHERE IDCODE=' . singleQuote($record['IDCODE']));
                if($search == 0)
                    $sw = true;
                $count++;
            } while (!$sw);
        } else {
            $record['IDCODE'] = $item_code;
        }
        ClxCommon::$db->createRecord($this->tablename, $record, false);
        $DBLT = &getTool('DBLT');
        $DBLT->versionIncrease('IDCODE', $listcode);
        return $record['IDCODE'];
    }

    function deleteRecordSk($id) {
        $itemrec = ClxCommon::$db->getRecord($this->tablename, 'ID=' . $id);
        $DICT = &getTool('DICT');
        $DICT->deleteEntry($itemrec['IDNAME']);
        $DICT->deleteEntry($itemrec['IDNAME2']);
        ClxCommon::$db->deleteRecords($this->tablename, 'ID=' . $id);
        $DBLT = &getTool('DBLT');
        $DBLT->versionIncrease('IDCODE', $itemrec['LISTCODE']);
        $this->clearHeap();
    }

    function defaultRecord() {
        $callingpage = peekHistory();
        $listcode = $callingpage['listcode'];
        $record = array();
        $record['DBLI_listcode'] = $listcode;
        $record['DBLI_id'] = CLX_INIT_ZERO;
        $record['DBLI_idcode'] = '';
        $record['DBLI_idname'] = '';
        $record['DBLI_idname_DICT'] = '';
        $record['DBLI_idname2'] = '';
        $record['DBLI_idname2_DICT'] = '';
        $record['DBLI_seqnum'] = CLX_INIT_ZERO;
        $record['DBLI_parentitem'] = '';
        $record['DBLI_inactive'] = CLX_INIT_ZERO;
        return $record;
    }

    function placeChooseListItem($msg, $listfieldname, $prompt, $name, $default, &$table) {

        $contents = setJavaScriptTag(getJavaScriptFilename('DBLI_opensearchwindow.js'), true);
        $searchIcon = getSkinIcon('view', 'javascript:DBLI_opensearchwindow(' . singleQuote($listfieldname) . ' , ' . singleQuote($name) . ')', 'Search for a item');
        if (!empty($msg[$name])) {
            $table->addCell($msg[$name], array(CLX_PROP_CLASS => CLX_CLASS_ALERT, CLX_PROP_COLSPAN => $table->width));
        }
        $table->addCell($prompt, array(CLX_PROP_CLASS => CLX_CLASS_PROMPT));
        $c = new htmlField($name, $default, CLX_NUM_TEN, CLX_NUM_TEN);
        $table->addCell($c->source() . $searchIcon, array(CLX_PROP_CLASS => CLX_CLASS_DATA));
        return $contents;
    }

    function import() {
        require_once'classes/clx_file_delimited.php';
        $DICT = &getTool('DICT');
        $filename = UserRequest::$files['DBLI_filename']['tmp_name'];
        $callingpage = peekHistory();
        $listcode = $callingpage['listcode'];
        $contents = '';
        $fd = new clx_file_delimited($filename,CLX_CHAR_NL, CLX_CHAR_TAB );

        if ($fd->hasError()) {
            $contents = alert($fd->errorMsg, dict('GEN_LBL_RETURN'));
        } else {
            $record = array();
            $record['LISTCODE'] = $listcode;
            $i = CLX_INIT_ZERO;
            while ($buffer = $fd->getNextRecord2()) {
                $line = explode($fd->field_delim, $buffer);
                $record['IDCODE'] = encode_string($line[CLX_NUM_ZERO], kDefaultEncoding);
                $record['IDNAME'] = $DICT->createEntry(encode_string($line[CLX_NUM_ONE], kDefaultEncoding));
                $record['SEQNUM'] = $i;
                $record['PARENTITEM'] = str_replace(CLX_CHAR_CR, '', (array_key_exists(CLX_NUM_TWO, $line) ? $line[CLX_NUM_TWO] : ''));
                $record['INACTIVE'] = CLX_INIT_ZERO;
                ClxCommon::$db->createRecord($this->tablename, $record, false);
                $i++;
            }
            if ($i++)
                $this->clearHeap();
            $fd->close();
        }
        return $contents;
    }

    /**
     * Clear field form heap
     * @param string $fieldname
     */
    private function clearHeap() {
		apc_clear_cacheFct('user');
    }

}
