<?php

/**
 * Tool Class
 * For more information contact  support@alenet.com
 *
 * @author       $Author: jclerma $
 * @copyright    Alenet, Inc.
 * @since        $Date: 2017-11-03 15:09:40 -0400 (Fri, 03 Nov 2017) $
 * @version        $Rev: 9615 $
 *
 * CHANGE LOG
 *  Oracle uppercase record changed
 *  Included label
 */
class db_modules extends clx_tool {

    // CONSTRUCTOR //
    function __construct() {
        $this->code = 'DBMO';
        $this->name = 'Module Manager';
        $this->system = 'Database';
        $this->version = '0.4.5.0';

        $this->has_access = true;
        $this->has_package = true;

        $this->tablename = 'CLX_DB_MODULES';
        $this->record_name_singular = 'module';
        $this->record_name_plural = 'modules';
        $this->primary_field = 'IDCODE';
        $this->secondary_field = 'IDNAME';

        $this->primary_key = array('IDCODE');
        $this->editform = array(CLX_CLASS_TITLE => 'TOOL_DBMO_TITLE_CREATE',
            'panels' => array(
                array('type' => 'normal',
                    'entries' => array(
                        array('label' => 'GEN_LBL_CODE',
                            'fields' => array(
                                array('name' => 'idcode', 'type' => 'string', 'default' => '', 'size' => '4', 'displaysize' => '4', 'dontmodify' => '1',
                                    'validations' => array(
                                        array('type' => 'sqlobject', 'required' => '1'), array('type' => 'unique', 'fieldname' => 'idcode', 'tablename' => $this->tablename))
                                )
                            )
                        ), array('label' => 'GEN_LBL_NAME',
                            'fields' => array(
                                array('name' => 'idname', 'type' => 'string', 'default' => '', 'size' => '64', 'displaysize' => '48', 'validations' => array(
                                        array('type' => 'string', 'required' => '1'))
                                )
                            )
                        ), array('label' => 'GEN_LBL_VERSION',
                            'fields' => array(
                                array('name' => 'version', 'type' => 'string', 'default' => '1.0.0', 'size' => '32', 'displaysize' => '16', 'validations' => array(
                                        array('type' => 'string', 'required' => '1'))
                                )
                            )
                        ), array('label' => 'GEN_LBL_DESCRIPTION',
                            'fields' => array(
                                array('name' => 'description', 'type' => 'text', 'default' => '', 'cols' => '48', 'rows' => '4'))
                        ), array('label' => 'TOOL_DBMO_LABEL_AUDIT',
                            'fields' => array(
                                array('name' => 'audit', 'type' => CLX_CLASS_LIST, 'default' => '0', 'options' => array('0' => 'GEN_LBL_NO', '1' => 'GEN_LBL_YES')))
                        )
                    )
                )
            )
        );
    }

    function install() {
        $UIMI = getTool('UIMI');
        $UIMI->add($this->code, 'MENU_TOOLS:MENU_DATABASE:MENU_MODULES', 'TOOL', $this->code, 'ac=LIST');
        //Creating Initial Page  dict('GEN_LBL_PAGE')
        ClxCommon::$db->createRecord($this->tablename, array('IDCODE' => 'BASE', 'IDNAME' => 'Clientix Base Module', 'VERSION' => '1.0.0', 'VERSION_COUNTER' => CLX_NUM_ZERO, 'CDATE' => date('Ymdhis'), 'MDATE' => date('Ymdhis'), 'username' => 'admin', 'DESCRIPTION' => 'Clientix Base Module', 'AUDITU' => CLX_NUM_ZERO), false);
    }

    function toolTableStructure() {
        $fields = array();
        $fields['ID'] = array('type' => 'int(11) unsigned', 'null' => 'NO', 'properties' => 'auto_increment', 'index' => 'PRI');
        $fields['IDCODE'] = array('type' => 'varchar(4)', 'null' => 'NO', 'properties' => '');
        $fields['IDNAME'] = array('type' => 'varchar(64)', 'null' => 'NO', 'properties' => '');
        $fields['VERSION'] = array('type' => 'varchar(32)', 'null' => 'NO', 'properties' => '');
        $fields['VERSION_COUNTER'] = array('type' => 'int(11) unsigned', 'null' => 'NO', 'properties' => '', 'default' => CLX_NUM_ZERO);
        $fields['CDATE'] = array('type' => 'datetime', 'null' => 'YES', 'properties' => '');
        $fields['MDATE'] = array('type' => 'datetime', 'null' => 'YES', 'properties' => '');
        $fields['USERNAME'] = array('type' => 'varchar(30)', 'null' => 'NO', 'properties' => '');
        $fields['DESCRIPTION'] = array('type' => 'mediumtext', 'properties' => '');
        $fields['AUDITU'] = array('type' => 'tinyint(1) unsigned', 'properties' => '');
        $indexes = array();
        $ipf = 'CLX' . $this->code;
        $indexes[$ipf . '01'] = array('index' => 'PRI', 'fieldname' => 'ID');
        $indexes[$ipf . '02'] = array('index' => 'UNI', 'fieldname' => 'IDCODE');
        $indexes[$ipf . '03'] = array('index' => 'MUL', 'fieldname' => 'IDNAME');
        $indexes[$ipf . '04'] = array('index' => 'MUL', 'fieldname' => 'USERNAME');
        return array('idcode' => $this->code, 'idname' => $this->name, 'name' => $this->tablename, 'type' => 'InnoDB',
            'fields' => $fields,
            'indexes' => $indexes);
    }

//-- M O D U L E S
    /**
     * (non-PHPdoc)
     * @see clx_tool::pack()
     */
    function pack($record) {
        $o = $this->packRecord($record);
        $code = $record['IDCODE'];
        $objs = new XMLObject($this->code . '_OBJECTS');
        // Database Stuff
        $this->packsub($objs, 'DBTB', 'SELECT * FROM CLX_DB_TABLES WHERE MODULECODE=' . singleQuote($code));
        $this->packsub($objs, 'DBLT', 'SELECT * FROM CLX_DB_LISTS WHERE MODULECODE=' . singleQuote($code));
        $this->packsub($objs, 'DBQY', 'SELECT * FROM CLX_DB_QUERIES WHERE MODULECODE=' . singleQuote($code));
        $this->packsub($objs, 'DBIM', 'SELECT * FROM CLX_DB_IMPORTS WHERE MODULECODE=' . singleQuote($code));
        $this->packsub($objs, 'DBEX', 'SELECT * FROM CLX_DB_EXPORTS WHERE MODULECODE=' . singleQuote($code));
        $this->packsub($objs, 'DBPR', 'SELECT * FROM CLX_DB_PROCEDURES WHERE MODULECODE=' . singleQuote($code));
        // User Interface
        $this->packsub($objs, 'UIFM', 'SELECT * FROM CLX_UI_FORMS WHERE MODULECODE=' . singleQuote($code));
        $this->packsub($objs, 'UILG', 'SELECT * FROM CLX_UI_LISTINGS WHERE MODULECODE=' . singleQuote($code));
        $this->packsub($objs, 'UICA', 'SELECT * FROM CLX_UI_CALENDAR WHERE MODULECODE=' . singleQuote($code));
        $this->packsub($objs, 'UIPG', 'SELECT * FROM CLX_UI_PAGES WHERE MODULECODE=' . singleQuote($code));
        //Pack Processes
        $this->packsub($objs, 'PRPR', 'SELECT * FROM CLX_PR_PROCESS WHERE MODULECODE=' . singleQuote($code));
        // Analitics
        $this->packsub($objs, 'ANRP', 'SELECT * FROM CLX_AN_REPORTS WHERE MODULECODE=' . singleQuote($code));
        $this->packsub($objs, 'ANCH', 'SELECT * FROM CLX_AN_CHARTS WHERE MODULECODE=' . singleQuote($code));
        $this->packsub($objs, 'ANPL', 'SELECT * FROM CLX_AN_PLOT WHERE MODULECODE=' . singleQuote($code));
        // Other Stuff
        $this->packsub($objs, 'CODE', 'SELECT * FROM CLX_CODELIB WHERE MODULECODE=' . singleQuote($code));
        $this->packsub($objs, 'ACTR', 'SELECT * FROM CLX_AC_TREES WHERE MODULECODE=' . singleQuote($code));
        $o->addObject($objs);
        unset($objs);
        return $o;
    }

    /**
     * Packs module into separates file. use on zip pack only
     * @param obj $dbmo_rec
     * @param array $package_tool
     * @param obj $zip
     * @param array $packrec
     * @param int $i
     */
    function pack_split(&$dbmo_rec, &$package_tool, &$zip, &$packcontent, &$packrec, &$index) {
        //Pack Module
        $modulecode = $dbmo_rec['IDCODE'];
        $package = $package_tool->packRecord($packrec);
        $package_objs = new xmlObject($package_tool->code . '_OBJECTS');
        $package_objs->addObject($this->packRecord($dbmo_rec));
        $package->addObject($package_objs);
        //Add module item
        $pn = $packcontent ['PACKAGECODE'] . '_' . $this->code . '_' . $dbmo_rec ['ID'] . '.xml';
        $zip->addFromString($pn, $package->source_(array('level' => CLX_NUM_ZERO)));
        $index ++;
        //Adding dependants
        // Database Stuff
        $this->pack_split_sub($package_tool, $packrec, 'DBTB', 'MODULECODE = ' . singleQuote($modulecode), $packcontent ['PACKAGECODE'], $zip, $index);
        $this->pack_split_sub($package_tool, $packrec, 'DBLT', 'MODULECODE = ' . singleQuote($modulecode), $packcontent ['PACKAGECODE'], $zip, $index);
        $this->pack_split_sub($package_tool, $packrec, 'DBQY', 'MODULECODE = ' . singleQuote($modulecode), $packcontent ['PACKAGECODE'], $zip, $index);
        $this->pack_split_sub($package_tool, $packrec, 'DBIM', 'MODULECODE = ' . singleQuote($modulecode), $packcontent ['PACKAGECODE'], $zip, $index);
        $this->pack_split_sub($package_tool, $packrec, 'DBEX', 'MODULECODE = ' . singleQuote($modulecode), $packcontent ['PACKAGECODE'], $zip, $index);
        $this->pack_split_sub($package_tool, $packrec, 'DBPR', 'MODULECODE = ' . singleQuote($modulecode), $packcontent ['PACKAGECODE'], $zip, $index);
        // User Interface
        $this->pack_split_sub($package_tool, $packrec, 'UIFM', 'MODULECODE = ' . singleQuote($modulecode), $packcontent ['PACKAGECODE'], $zip, $index);
        $this->pack_split_sub($package_tool, $packrec, 'UILG', 'MODULECODE = ' . singleQuote($modulecode), $packcontent ['PACKAGECODE'], $zip, $index);
        $this->pack_split_sub($package_tool, $packrec, 'UICA', 'MODULECODE = ' . singleQuote($modulecode), $packcontent ['PACKAGECODE'], $zip, $index);
        $this->pack_split_sub($package_tool, $packrec, 'UIPG', 'MODULECODE = ' . singleQuote($modulecode), $packcontent ['PACKAGECODE'], $zip, $index);
        //Pack Processes
        $this->pack_split_sub($package_tool, $packrec, 'PRPR', 'MODULECODE = ' . singleQuote($modulecode), $packcontent ['PACKAGECODE'], $zip, $index);
        // Analitics
        $this->pack_split_sub($package_tool, $packrec, 'ANRP', 'MODULECODE = ' . singleQuote($modulecode), $packcontent ['PACKAGECODE'], $zip, $index);
        $this->pack_split_sub($package_tool, $packrec, 'ANCH', 'MODULECODE = ' . singleQuote($modulecode), $packcontent ['PACKAGECODE'], $zip, $index);
        $this->pack_split_sub($package_tool, $packrec, 'ANPL', 'MODULECODE = ' . singleQuote($modulecode), $packcontent ['PACKAGECODE'], $zip, $index);
        // Other Stuff
        $this->pack_split_sub($package_tool, $packrec, 'CODE', 'MODULECODE = ' . singleQuote($modulecode), $packcontent ['PACKAGECODE'], $zip, $index);
        $this->pack_split_sub($package_tool, $packrec, 'ACTR', 'MODULECODE = ' . singleQuote($modulecode), $packcontent ['PACKAGECODE'], $zip, $index);
    }

    /**
     * Adds dependant objects
     * @param unknown_type $toolcode
     * @param unknown_type $criteria
     * @param unknown_type $modulecode
     * @param unknown_type $zip
     * @param unknown_type $index
     */
    function pack_split_sub(&$package_tool, &$packrec, $toolcode, $criteria, $modulecode, &$zip, &$index) {
        $tool = &getTool($toolcode);
        //Adding dependants
        $objects = ClxCommon::$db->getRecords($tool->tablename, $criteria);
        while (($objectrec = ClxCommon::$db->getNextRecord($objects))) {
            $package = $package_tool->packRecord($packrec);
            $package_objs = new xmlObject($package_tool->code . '_OBJECTS');
            $package_objs->addObject($tool->pack($objectrec));
            $package->addObject($package_objs);
            // Package File Name
            $pn = $modulecode . '_' . $toolcode . '_' . $objectrec ['ID'] . '.xml';
            $zip->addFromString($pn, $package->source_(array('level' => CLX_NUM_ZERO)));
            $index ++;
        }
    }

    function packcheck($package, &$i, &$table) {
        $this->packdescribe($package, $i, $table);
        if ($package->hasSubObject($this->code . '_OBJECTS')) {
            $objs = $package->getObjectXml($this->code . '_OBJECTS');
            foreach ($objs->objects as $o) {
                $toolcode = $o->tag;
                $tool = &getTool($toolcode);
                $tool->packcheck($o, $i, $table);
            }
        }
    }

    function unpack($parameters) {
        $parameters = array_values(array_merge(array('package' => null), $parameters));
        list($package) = $parameters;
        $record = $this->unpackRecord($package);
        $modulecode = $record['IDCODE'];
        ClxCommon::$db->deleteRecords('CLX_DB_MODULES', 'IDCODE=' . singleQuote($modulecode));
        ClxCommon::$db->createRecordVoid($this->tablename, $record, false);
        if ($package->hasSubObject($this->code . '_OBJECTS')) {
            $objs = $package->getObjectXml($this->code . '_OBJECTS');
            foreach ($objs->objects as $o) {
                $toolcode = $o->tag;
                $tool = &getTool($toolcode);
                $tool->unpack(array('package' => $o));
            }
        }
    }

    function action($ac, $idlist) {
        if ($this->getAccess()->canRead()) {
            $contents = parent::action($ac, $idlist);
        } else {
            $contents = warning(dict('GEN_ERR_NO_ACCESS') . ' [DB_MODULES-CAN_READ][' . $ac . '] ' . $this->getCode(false));
        }
        return $contents;
    }

    function makelist($listingcode = null) {
        //$t = time('U'); genera un warning
		$t = time();
        $to = $this->code;
        $here = 'tool.php?to=' . $to . '&t=' . $t;
        clearHistory();
        pushHistory($this->code, 'LIST', array());
        $form = new htmlForm(CLX_PROP_ACTION, 'tool.php');
        $form->setProp(CLX_PROP_ID, 'DBMOList');
        $form->addArgument(CLX_PROP_TO, $this->code);
        $form->addArgument('ac', 'NOT_DEFINED');
        $selectclause = '*';
        $fromclause = 'CLX_DB_MODULES';
        $whereclause = '';
        $sortclause = 'IDNAME';
        $page = new xPaging();
        $page->pageInit($form->getProp(CLX_PROP_ID), $here, $selectclause, $fromclause, $whereclause, $sortclause, '');
        $table = new htmlTable(CLX_WIDTH_FIVE);
        $table->setProps(array(CLX_PROP_CLASS => CLX_CLASS_LIST, CLX_PROP_WIDTH => CLX_PROP_100PCT));
        $table->addCell(dict('TOOL_DBMO_TITLE_LIST'), array(CLX_PROP_CLASS => CLX_CLASS_TITLE, CLX_PROP_COLSPAN => CLX_WIDTH_FIVE));
        $page->header($table);
        $table->addCells(CLX_CLASS_HEADERS, array(dict('GEN_LBL_SEL'), dict('GEN_LBL_CODE'), dict('GEN_LBL_NAME'), dict('GEN_LBL_VERSION'), dict('GEN_LBL_DESCRIPTION')));
        $i = CLX_INIT_ZERO;
        while ($record = $page->nextRecord()) {
            $recordid = $record['ID'];
            $select = new htmlCheckbox('selector_' . $recordid, false);
            $select->setProp('onclick', 'selectButtons(' . singleQuote($form->getProp(CLX_PROP_ID)) . ');');
            $table->addCell($select->source(), array(CLX_PROP_CLASS => CLX_CLASS_FOOTERS, CLX_PROP_WIDTH => '40', CLX_PROP_ALIGN => 'center'));
            $table->addCells(($i % CLX_NUM_TWO == CLX_NUM_ZERO) ? CLX_CLASS_DATA : CLX_CLASS_DATAALT, array($record['IDCODE'], $record['IDNAME'], $record['VERSION'] . '-' . $record['VERSION_COUNTER'], xFormat::text($record['DESCRIPTION'])));
            $i++;
        }
        $buttons = '';
        $buttons .= htmlButton::getButton('submit', dict('GEN_LBL_DISPLAY'), array('action' => 'DISPLAY', 'form' => $form->getProp(CLX_PROP_ID), CLX_PROP_ID => 'MULT'))->source();
        if ($this->getAccess()->canModify()) {
            $buttons .=htmlButton::getButton('submit', dict('GEN_LBL_MODIFY'), array('action' => 'MODIFY', 'form' => $form->getProp(CLX_PROP_ID), CLX_PROP_ID => 'MULT'))->source();
        }
        if ($this->getAccess()->canDelete()) {
            $buttons .=htmlButton::getButton('submit', dict('GEN_LBL_DELETE'), array('action' => 'DELETE', 'form' => $form->getProp(CLX_PROP_ID), CLX_PROP_ID => 'MULT'))->source();
        }
        if ($this->getAccess()->canDisplay()) {
            $buttons .=htmlButton::getButton('submit', dict('GEN_LBL_CREATE_SOMETHING', dict('GEN_LBL_MODULE')), array('action' => 'CREATE', 'form' => $form->getProp(CLX_PROP_ID), CLX_PROP_ID => 'ON'))->source();
        }
        $buttons .=htmlButton::getButton('submit', dict('GEN_LBL_FILTER'), array('action' => 'FILTER', 'form' => $form->getProp(CLX_PROP_ID), CLX_PROP_ID => 'ON'))->source();
        $form->addContents($page->footers($table, $buttons));
        return $form->source();
    }

    function searchForm() {
        $msg = array();
        $contents = '';
        // Search Form
        $form = new htmlForm(CLX_PROP_ACTION, UserRequest::$server['PHP_SELF']);
        $form->addArgument(CLX_PROP_TO, $this->code);
        $form->addArgument('ac', 'SEARCH');
        $form->addArgument('confirm', '1');
        $form->addArgument(CLX_PROP_TARGET, UserRequest::$request[CLX_PROP_TARGET]);
        $table = new htmlTable(CLX_WIDTH_TWO);
        $table->setProps(array(CLX_PROP_CLASS => CLX_CLASS_FORM, CLX_PROP_WIDTH => CLX_PROP_100PCT));
        $table->addCell(dict('TOOL_DBMO_TITLE_SEARCH'), array(CLX_PROP_CLASS => CLX_CLASS_TITLE, CLX_PROP_COLSPAN => CLX_WIDTH_TWO));
        $table->addField($msg, dict('GEN_LBL_CODE') . ':', 'DBMO_idcode', (!empty(UserRequest::$post['DBMO_idcode'])) ? UserRequest::$post['DBMO_idcode'] : '', CLX_NUM_FOUR, CLX_NUM_FOUR);
        $table->addField($msg, dict('GEN_LBL_NAME') . ':', 'DBMO_idname', (!empty(UserRequest::$post['DBMO_idname'])) ? UserRequest::$post['DBMO_idname'] : '', CLX_NUM_32, CLX_NUM_64);
        $buttons = htmlButton::getButton('submit', dict('GEN_LBL_SEARCH'), array('action' => 'SEARCH', 'form' => $form->getProp(CLX_PROP_ID)))->source();
        $c = new htmlButtonCloseWindow(dict('GEN_LBL_CANCEL'));
        $buttons .= $c->source();
        $table->addCell($buttons, array(CLX_PROP_CLASS => CLX_CLASS_FOOTERS, CLX_PROP_COLSPAN => CLX_WIDTH_TWO));
        $form->setContents($table->source());
        $contents .= $form->source();
        // Search Procedure
        if (isset(UserRequest::$post['confirm'])) {
            $table = new htmlTable(CLX_WIDTH_TWO);
            $table->setProps(array(CLX_PROP_CLASS => CLX_CLASS_LIST, CLX_PROP_WIDTH => CLX_PROP_100PCT));
            $criteria = '';
            if (UserRequest::$post['DBMO_idcode'] <> '') {
                $criteria .= 'UPPER(IDCODE) LIKE ' . singleQuote(uppercase(UserRequest::$post['DBMO_idcode']) . '%');
            }
            if (UserRequest::$post['DBMO_idname'] <> '') {
                $criteria .= 'UPPER(IDNAME) LIKE ' . singleQuote(uppercase(clx_change_especial_char_html (UserRequest::$post['DBMO_idname'])) . '%');
            }
            $qty = ClxCommon::$db->oneExistsRecords('CLX_DB_MODULES', $criteria);
            if ($qty == CLX_NUM_ZERO) {
                $table->addCell(dict('TOOL_GEN_ERR_NOMATCH'), array(CLX_PROP_CLASS => CLX_CLASS_ALERT, CLX_PROP_COLSPAN => CLX_WIDTH_TWO));
            } else {
                $table->addCell(dict('TOOL_GEN_MSG_FOUNDRECORDS'), array(CLX_PROP_CLASS => CLX_CLASS_SUBTITLE, CLX_PROP_COLSPAN => CLX_WIDTH_TWO));
                $records = ClxCommon::$db->getRecords('CLX_DB_MODULES', $criteria);
                $i = CLX_INIT_ZERO;
                while ($record = ClxCommon::$db->getNextRecord($records, ASSOCIATIVE)) {
                    $js = 'javascript:setLookupValue(' . quote(UserRequest::$request[CLX_PROP_TARGET]) . ', ' . quote($record['IDCODE']) . ');';
                    $table->addCells(($i % CLX_NUM_TWO == CLX_NUM_ZERO) ? CLX_CLASS_DATA : CLX_CLASS_DATAALT, array('<page t=dbmodules></page><a href=' . singleQuote($js) . '>' . $record['IDCODE'] . '</a>', $record['IDNAME']));
                    $i++;
                }
            }
            $contents .= $table->source();
        }
        return $contents;
    }

    function confirmDeleteFill(&$table, $record) {
        $table->addData(dict('GEN_LBL_CODE') . ':', $record['DBMO_idcode']);
        $table->addData(dict('GEN_LBL_NAME') . ':', $record['DBMO_idname']);
    }

    function viewFormFill($record) {
        $yesno = array(dict('GEN_LBL_NO'), dict('GEN_LBL_YES'));
        $contents = '';
        $table = new htmlTable(CLX_WIDTH_TWO);
        $table->setProps(array(CLX_PROP_CLASS => CLX_CLASS_FORM, CLX_PROP_WIDTH => CLX_PROP_100PCT));
        $table->addCell(dict('TOOL_DBMO_TITLE_VIEW') . ': ' . $record['DBMO_idname'], array(CLX_PROP_CLASS => CLX_CLASS_TITLE, CLX_PROP_COLSPAN => CLX_WIDTH_TWO));
        $table->addData(dict('GEN_LBL_CODE') . ':', $record['DBMO_idcode']);
        $table->addData(dict('GEN_LBL_NAME') . ':', $record['DBMO_idname']);
        $table->addData(dict('GEN_LBL_VERSION') . ':', $record['DBMO_version'] . '-' . $record['DBMO_version_counter']);
        $table->addData(dict('GEN_LBL_DESCRIPTION') . ':', xFormat::text($record['DBMO_description']));
        $contents .= $table->source();
        $DBTB = &getTool('DBTB');
        addHistory('modulecode', $record['DBMO_idcode']);
        $contents .= $DBTB->makepanel($record['DBMO_idcode']);
        $table = new htmlTable(CLX_WIDTH_TWO);
        $table->setProps(array(CLX_PROP_CLASS => CLX_CLASS_FORM, CLX_PROP_WIDTH => CLX_PROP_100PCT));
        $table->addData(dict('GEN_LBL_AUDIT_SOMETHING', dict('GEN_LBL_TABLE')) . ':', $yesno[$record['DBMO_audit']]);
        $table->addData(dict('GEN_LBL_CDATE') . ':', $record['DBMO_cdate']);
        $table->addData(dict('GEN_LBL_MDATE') . ':', $record['DBMO_mdate']);
        $table->addData(dict('GEN_LBL_USERNAME') . ':', $record['DBMO_username']);
        $contents .= buildPanelInterface(dict('GEN_TTL_SYSTEMINFO'), 'SYSTEMINFO', $table->source(), CLX_NUM_MINUS_1);
        return $contents;
    }

    function filterFormFill($msg, &$table) {
        if (!array_key_exists('DBMO_idcode', UserRequest::$post))
            UserRequest::$post['DBMO_idcode'] = '';
        if (!array_key_exists('DBMO_idname', UserRequest::$post))
            UserRequest::$post['DBMO_idname'] = '';
        $table->addField($msg, dict('GEN_LBL_CODE'), 'DBMO_idcode', UserRequest::$post['DBMO_idcode'], CLX_NUM_FOUR, CLX_NUM_FOUR);
        $table->addField($msg, dict('GEN_LBL_NAME'), 'DBMO_idname', UserRequest::$post['DBMO_idname'], CLX_NUM_32, CLX_NUM_64);
    }

    function setfilter() {
        $filter = '';
        $boolean = '';
        if (!empty(UserRequest::$post['DBMO_idcode'])) {
            $filter .= $boolean . ' UPPER(IDCODE) LIKE ' . singleQuote(uppercase(UserRequest::$post['DBMO_idcode']) . '%');
            $boolean = 'AND';
        }
        if (!empty(UserRequest::$post['DBMO_idname'])) {
            $filter .= $boolean . ' UPPER(IDNAME) LIKE ' . singleQuote(uppercase(clx_change_especial_char_html (UserRequest::$post['DBMO_idname'])) . '%');
            $boolean = 'AND';
        }
        $_SESSION['paging']['DBMOList']['criteria'] = $filter;
        $_SESSION['paging']['DBMOList']['startat'] = '0';
    }

    function clearfilter() {
        $_SESSION['paging']['DBMOList']['criteria'] = '';
    }

    function loadToolRecord($recordid) {
        $values = ClxCommon::$db->getRecord('CLX_DB_MODULES', 'ID= ' . $recordid);
        $record = array();
        $record['DBMO_id'] = $recordid;
        $record['DBMO_idcode'] = $values['IDCODE'];
        $record['DBMO_idname'] = $values['IDNAME'];
        $record['DBMO_version'] = $values['VERSION'];
        $record['DBMO_version_counter'] = $values['VERSION_COUNTER'];
        $record['DBMO_cdate'] = xFormat::date($values['CDATE']);
        $record['DBMO_mdate'] = xFormat::date($values['MDATE']);
        $record['DBMO_username'] = $values['USERNAME'];
        $record['DBMO_description'] = $values['DESCRIPTION'];
        $record['DBMO_audit'] = $values['AUDITU'];
        return $record;
    }

    function saveRecord() {
        $d = xFormat::getNow('Y-m-d');
        $record = array();
        $record['IDCODE'] = UserRequest::$post['DBMO_idcode'];
        $record['IDNAME'] = UserRequest::$post['DBMO_idname'];
        $record['VERSION'] = UserRequest::$post['DBMO_version'];
        $record['MDATE'] = $d;
        $record['USERNAME'] = $_SESSION['user_record']['username'];
        $record['DESCRIPTION'] = UserRequest::$post['DBMO_description'];
        $record['AUDITU'] = UserRequest::$post['DBMO_audit'];
        ClxCommon::$db->updateRecord('CLX_DB_MODULES', $record, 'ID=' . UserRequest::$post['DBMO_id']);
        $this->versionIncrease(CLX_PROP_ID, UserRequest::$post['DBMO_id']);
    }

    function createRecord_() {
        $d = xFormat::getNow('Y-m-d');
        $record = array();
        $record['IDCODE'] = UserRequest::$post['DBMO_idcode'];
        $record['IDNAME'] = UserRequest::$post['DBMO_idname'];
        $record['VERSION'] = UserRequest::$post['DBMO_version'];
        $record['VERSION_COUNTER'] = '0';
        $record['CDATE'] = $d;
        $record['MDATE'] = $d;
        $record['USERNAME'] = $_SESSION['user_record']['username'];
        $record['DESCRIPTION'] = UserRequest::$post['DBMO_description'];
        $record['AUDITU'] = UserRequest::$post['DBMO_audit'];
        $recordid = ClxCommon::$db->createRecord('CLX_DB_MODULES', $record, false);
        return $recordid;
    }

    function deleteRecordSk($id) {
        $DBTB = &getTool('DBTB');
        $modulerec = ClxCommon::$db->getRecord('CLX_DB_MODULES', 'ID=' . $id);
        $modulecode = $modulerec['IDCODE'];
        $tables = ClxCommon::$db->getRecords('CLX_DB_TABLES', 'MODULECODE=' . singleQuote($modulecode));
        while (($table = ClxCommon::$db->getNextRecord($tables))) {
            $DBTB->deleteRecordSk($table['ID']);
        }
        ClxCommon::$db->deleteRecords('CLX_DB_MODULES', 'ID=' . $id);
    }

    function defaultRecord() {
        $record = array();
        $record['DBMO_id'] = CLX_INIT_ZERO;
        $record['DBMO_idcode'] = '';
        $record['DBMO_idname'] = '';
        $record['DBMO_version'] = '1.0.0';
        $record['DBMO_cdate'] = '';
        $record['DBMO_mdate'] = '';
        $record['DBMO_username'] = $_SESSION['user_record']['username'];
        $record['DBMO_description'] = '';
        $record['DBMO_audit'] = '0';
        return $record;
    }

// DEPRECATED in favor of chooseRecord($name, $default)
    function placeChooseModule($msg, $prompt, $name, $default, &$table) {
        if (!empty($msg[$name]))
            $table->addCell($msg[$name], array(CLX_PROP_CLASS => CLX_CLASS_ALERT, CLX_PROP_COLSPAN => $table->width));
        $table->addCell($prompt, array(CLX_PROP_CLASS => CLX_CLASS_PROMPT));
        $table->addCell($this->chooseRecord($name, $default, null, CLX_NUM_TEN, CLX_NUM_FOUR), array(CLX_PROP_CLASS => CLX_CLASS_DATA));
        return CLX_CHAR_EMPTY_STRING;
    }


    function moduleExists(&$idcode) {
        $qty = ClxCommon::$db->oneExistsRecords($this->tablename, 'IDCODE = ' . singleQuote($idcode));
        if ($qty > CLX_NUM_ZERO) {
            $idcode = ClxCommon::$db->getField($this->tablename, 'IDCODE', 'IDCODE =' . singleQuote($idcode));
        }
        return ($qty > CLX_NUM_ZERO)?true:false;
    }

}
