<?php

/**
 * Tool Class
 * For more information contact  support@alenet.com
 *
 * @author       $Author: alenetdev $
 * @copyright    Alenet, Inc.
 * @since        $Date: 2019-02-28 19:18:52 +0000 (Thu, 28 Feb 2019) $
 * @version        $Rev: 10563 $
 *
 * CHANGE LOG
 *  Oracle uppercase record changed
 *  Included label
 *  Oracle uppercase record changed
 *  Incorporate null on tablefield on upgrade
  UPDATE CLX_DB_TABLEFIELDS TF SET TF.FIELD_REQUIRED = (SELECT X.FIELD_REQUIRED FROM CLX_DB_FIELDS X WHERE TF.FIELDNAME = X.FIELDNAME);
 */
class db_tablefields extends clx_tool {

    function __construct() {
        $this->code = 'DBTF';
        $this->name = 'Table Fields';
        $this->system = 'Database';
        $this->version = '0.4.9.1';
        $this->accesskey = 'DBTB';
        $this->tablename = 'CLX_DB_TABLEFIELDS';
        $this->record_name_singular = 'table field';
        $this->record_name_plural = 'table fields';
        $this->primary_field = 'TABLECODE';
        $this->secondary_field = 'FIELDNAME';
        $this->seqnum_field = 'SEQNUM';
        $this->primary_key = array('TABLECODE', 'FIELDNAME');
		if (!isset(ClxCommon::$db->debugCF))
			ClxCommon::$db->debugCF=false;
    }

    function toolTableStructure() {
        $fields = array();
        $fields['ID'] = array('type' => 'int(11) unsigned', 'null' => 'NO', 'properties' => 'auto_increment', 'index' => 'PRI');
        $fields['TABLECODE'] = array('type' => 'varchar(4)', 'null' => 'NO', 'properties' => '');
        $fields['FIELDNAME'] = array('type' => 'varchar(64)', 'null' => 'NO', 'properties' => '');
        $fields['SEQNUM'] = array('type' => 'int(11) unsigned', 'null' => 'NO', 'properties' => '', 'default' => CLX_NUM_ZERO);
        $fields['FIELD_REQUIRED'] = array('type' => 'tinyint(1) unsigned', 'null' => 'NO', 'properties' => '', 'default' => CLX_NUM_ZERO);
        $fields['FIELD_DEFAULT'] = array('type' => 'varchar(255)', 'properties' => '');
        $fields['CASCADE_DELETE'] = array('type' => 'tinyint(1) unsigned', 'null' => 'NO', 'properties' => '', 'default' => CLX_NUM_ZERO);
        $fields['DISABLE_AUTOINCR'] = array('type' => 'tinyint(1) unsigned', 'null' => 'NO', 'properties' => '', 'default' => CLX_NUM_ZERO);
        $indexes = array();
        $ipf = 'CLX' . $this->code;
        $indexes[$ipf . '01'] = array('index' => 'PRI', 'fieldname' => 'ID');
        $indexes[$ipf . '02'] = array('index' => 'UNI', 'fieldname' => 'TABLECODE,FIELDNAME');
        return array('idcode' => $this->code, 'idname' => $this->name, 'name' => $this->tablename, 'type' => 'InnoDB', 'fields' => $fields, 'indexes' => $indexes);
    }

    function unpack($parameters) {
        $parameters = array_values(array_merge(array('package' => null), $parameters));
        list($package) = $parameters;
        $record = $this->unpackRecord($package);
        $record['FIELDNAME'] = uppercase($record['FIELDNAME']);
        ClxCommon::$db->createRecord($this->tablename, $record, false);
    }

    /**
     * Tool Action method
     * @param type $ac
     * @param type $idlist
     * @return type
     */
    function action($ac, $idlist) {
        $contents = '';
        $msg = array();
        if ($this->getAccess()->canRead()) {
            switch ($ac) {
                case 'INCLUDE':
                    if ($this->getAccess()->canModify()) {
                        if (isset(UserRequest::$post['confirm'])) {
                            if ($this->checkForm($msg)) {
                                $this->createRecordVoid_();
                                $info = peekHistory();
                                $to = &getTool($info[CLX_PROP_TO]);
                                $contents = $to->action($info['ac'], toArray($info['idlist']));
                            } else {
                                if (array_key_exists('ERROR', $msg) && $msg['ERROR'] === 'NO_FIELD') {
                                    pushHistory($this->code, 'INCLUDE_NEW', $idlist, 'tablecode', UserRequest::$post['DBTF_tablecode']);
                                    $DBFL = &getTool('DBFL');
                                    $record = $DBFL->defaultRecord();
                                    $record['DBFL_fieldname'] = UserRequest::$post['DBTF_fieldname'];
                                    $contents = $DBFL->editForm('CREATE', array(), $record, array());
                                } else {
                                    $contents = $this->editForm($ac, array(), UserRequest::$post, $msg);
                                }
                            }
                        } else {
                            $record = $this->defaultRecord();
                            $contents = $this->editForm($ac, array(), $record, $msg);
                        }
                    } else {
                        $contents = warning(dict('GEN_ERR_NO_ACCESS') . ' [DB_TABLEFIELDS-CAN_MODIFY_1][' . $ac . '] ' . $this->getCode(false));
                    }
                    break;
                case 'TBF_LIST_XML':
                    $fields = 'FIELDNAME';
                    $criteria = array('TABLECODE' => UserRequest::$get['tc']);
                    sendXmlHeader();
                    $contents = $this->getListXml($fields, $criteria)->saveXml();
                    break;
                case 'INCLUDE_NEW':
                    if ($this->getAccess()->canModify()) {
                        if (!empty(UserRequest::$request['DBFL_fieldname'])) {
                            UserRequest::$post['DBTF_fieldname'] = UserRequest::$request['DBFL_fieldname'];
                            //$info = popHistory();
                            $info = peekHistory();
                            UserRequest::$post['DBTF_tablecode'] = $info['tablecode'];
                            $this->createRecordVoid_();
                        } else {
                            //popHistory();
                        }
                        //$info = popHistory();
                        $info = peekHistory();
                        $to = &getTool($info[CLX_PROP_TO]);
                        $contents = $to->action($info['ac'], toArray($info['idlist']));
                    } else {
                        $contents = warning(dict('GEN_ERR_NO_ACCESS') . ' [DB_TABLEFIELDS-CAN_MODIFY_2][' . $ac . '] ' . $this->getCode(false));
                    }
                    break;
                default:
                    $contents = parent::action($ac, $idlist);
            }
        } else {
            $contents = warning(dict('GEN_ERR_NO_ACCESS') . ' [DB_TABLEFIELDS-CAN_READ][' . $ac . '] ' . $this->getCode(false));
        }
        return $contents;
    }

    function makepanel($groupcode = null) {
        $tablecode = $groupcode;
        if ($this->getAccess()->canRead()) {
            $t = time();
            $to = $this->code;
            $here = 'tool.php?to=' . $to . '&t=' . $t;
            $form = new htmlForm(CLX_PROP_ACTION, 'tool.php');
            $form->setProp(CLX_PROP_ID, 'tablefieldspanel');
            $form->addArgument(CLX_PROP_TO, $this->code);
            $form->addArgument('ac', 'NOT_DEFINED');
            $form->addArgument('DBTF_tablecode', $tablecode);
            $selectclause = '*';
            $fromclause = 'CLX_DB_TABLEFIELDS';
            $whereclause = 'TABLECODE=' . singleQuote($tablecode);
            $sortclause = 'SEQNUM ASC';
            $page = new xPaging();
            $page->setProp('rpp', CLX_NUM_TEN_THOUSAND);
            $page->setProp('printheaders', false);
            $page->pageInit($form->getProp(CLX_PROP_ID), $here, $selectclause, $fromclause, $whereclause, $sortclause, '');
            $table = new htmlTable(CLX_WIDTH_NINE);
            $table->setProps(array(CLX_PROP_CLASS => CLX_CLASS_LIST));
            $page->header($table);
            $table->addCells(CLX_CLASS_HEADERS, array(dict('GEN_LBL_SEL'), dict('GEN_LBL_NAME'), dict('GEN_LBL_TYPE'), dict('GEN_LBL_VERSION'), dict('GEN_LBL_LABELS'), dict('GEN_LBL_REQUIRED'), dict('GEN_LBL_DEFAULT'), dict('GEN_LBL_CASCADE_DELETE'), dict('GEN_LBL_DESCRIPTION')));
            $i = CLX_INIT_ZERO;
            $DICT = getTool('DICT');
            $yesno = array();
            foreach ($this->yesno as $key => $val)
                $yesno[$key] = dict($val);
            while ($record = $page->nextRecord()) {
                $recordid = $record['ID'];
                $select = new htmlCheckbox('selector_' . $recordid, false);
                $select->setProp('onclick', 'selectButtons(' . singleQuote($form->getProp(CLX_PROP_ID)) . ');');
                $table->addCell($select->source(), array(CLX_PROP_CLASS => CLX_CLASS_FOOTERS, CLX_PROP_WIDTH => '40', CLX_PROP_ALIGN => 'center'));
                $fieldrec = ClxCommon::$db->getRecord('CLX_DB_FIELDS', 'FIELDNAME=' . singleQuote($record['FIELDNAME']));
                if ($record['FIELDNAME'] !== 'ID') {
                    $fieldname = sprintf('<a href="tool.php?to=DBFL&ac=MODIFY&idlist=%s">%s</a>', $fieldrec['ID'], $record['FIELDNAME']);
                } else {
                    $fieldname = $record['FIELDNAME'];
                }
                if (empty($fieldrec['FIELD_REQUIRED'])) {
                    $fieldrec['FIELD_REQUIRED'] = CLX_INIT_ZERO;
                }
                $table->addCells(($i % CLX_NUM_TWO == CLX_NUM_ZERO) ? CLX_CLASS_DATA : CLX_CLASS_DATAALT, array($fieldname, $fieldrec['FIELDTYPE'], $fieldrec['VERSION'] . '-' . $fieldrec['VERSION_COUNTER'], '<b>' . dict('GEN_LBL_LISTLABEL') . ':</b> ' . $DICT->dictFastEdit($fieldrec['FIELDLISTLABEL']) .
                    '<BR><b>' . dict('GEN_LBL_FORMLABEL') . ':</b> ' . $DICT->dictFastEdit($fieldrec['FIELDFORMLABEL']), $yesno[$record['FIELD_REQUIRED']] .
                    ' [' . dict('GEN_LBL_FIELD_DEFAULT') . ': ' . $yesno[$fieldrec['FIELD_REQUIRED']] . ']', $record['FIELD_DEFAULT'], $yesno[$record['CASCADE_DELETE']], xFormat::text($fieldrec['DESCRIPTION'])));
                $i++;
            }
            $buttons = '';
            if ($this->getAccess()->canModify()) {
                $buttons .=htmlButton::getButton('submit', dict('GEN_LBL_MODIFY'), array('action' => 'MODIFY', 'form' => $form->getProp(CLX_PROP_ID), CLX_PROP_ID => 'MULT'))->source();
            }
            if ($this->getAccess()->canDelete()) {
                $buttons .=htmlButton::getButton('submit', dict('GEN_LBL_EXCLUDE'), array('action' => 'DELETE', 'form' => $form->getProp(CLX_PROP_ID), CLX_PROP_ID => 'MULT'))->source();
            }
            if ($this->getAccess()->canDisplay()) {
                $b = new htmlFormSubmit(dict('GEN_LBL_INCLUDE', dict('GEN_LBL_FIELD')), 'INCLUDE', $form->getProp(CLX_PROP_ID));
                $b->setProp(CLX_PROP_ID, 'ON');
                $buttons .= $b->source();
            }
            if ($page->getProp('qty') > CLX_NUM_ONE && $this->getAccess()->canModify()) {
                $buttons .=htmlButton::getButton('submit', dict('GEN_LBL_REORDER'), array('action' => 'REORDER', 'form' => $form->getProp(CLX_PROP_ID), CLX_PROP_ID => 'ON'))->source();
            }
            $form->addContents($page->footers($table, $buttons));
            $recordcode = ''; // Verify why recordcode?
            return buildPanelInterface(dict('TOOL_DBTF_TITLE_FIELDS'), $recordcode . '_FIELDS', $form->source(), CLX_NUM_MINUS_1);
        } else {
            return CLX_CHAR_EMPTY_STRING;
        }
    }

    function editForm($ac, $idlist, $record, $msg) {
        $contents = '';
        $modify = (UserRequest::$request['ac'] === 'MODIFY');
        $title = ($modify) ? dict('TOOL_DBTF_TITLE_MODIFY') : dict('TOOL_DBTF_TITLE_CREATE');
        $form = new htmlForm(CLX_PROP_ACTION, 'tool.php');
        $form->addArgument(CLX_PROP_TO, $this->code);
        $form->addArgument('ac', $ac);
        $form->addArgument('confirm', '1');
        $form->addArgument('DBTF_id', (!empty($record['DBTF_id'])) ? $record['DBTF_id'] : '');
        $form->addArgument('DBTF_tablecode', $record['DBTF_tablecode']);
		$form->addArgument('DBTF_cascade_delete', $record['DBTF_cascade_delete']);
        $form->addArgument('idlist', toString($idlist));
        $table = new htmlTable(CLX_WIDTH_TWO);
        $table->setProps(array(CLX_PROP_CLASS => CLX_CLASS_FORM, CLX_PROP_WIDTH => CLX_PROP_100PCT));
        $table->addCell($title, array(CLX_PROP_CLASS => CLX_CLASS_TITLE, CLX_PROP_COLSPAN => CLX_WIDTH_TWO));
        if (!empty($modify)) {
            $form->addArgument('DBTF_fieldname', $record['DBTF_fieldname']);
            $table->addData(dict('GEN_LBL_TABLECODE') . ':', $record['DBTF_tablecode']);
            if (!empty($msg['DBTF_fieldname'])) {
                $table->addCell($msg['DBTF_fieldname'], array(CLX_PROP_CLASS => CLX_CLASS_ALERT, CLX_PROP_COLSPAN => CLX_WIDTH_TWO));
            }
            $table->addData(dict('GEN_LBL_FIELDNAME') . ':', $record['DBTF_fieldname']);
            $table->addCell(dict('GEN_LBL_OPTIONS'), array(CLX_PROP_CLASS => CLX_CLASS_SUBTITLE, CLX_PROP_COLSPAN => CLX_WIDTH_TWO));
            if (!empty($msg['DBTF_field_required'])) {
                $table->addCell($msg['DBTF_field_required'], array(CLX_PROP_CLASS => CLX_CLASS_ALERT, CLX_PROP_COLSPAN => CLX_WIDTH_TWO));
            }
            $table->addYesNo(dict('GEN_LBL_REQUIRED'), 'DBTF_field_required', $record['DBTF_field_required'], CLX_NUM_ZERO);
            //$table->addYesNo(dict('GEN_LBL_DELWHENPARENT'),'DBTF_cascade_delete',$record['DBTF_cascade_delete'],0);
            $table->addField($msg, dict('GEN_LBL_DEFAULT') . ':', 'DBTF_field_default', $record['DBTF_field_default'], CLX_NUM_32, CLX_NUM_64);
            //Disable autoincrement if exists
            $table->addYesNo(dict('GEN_LBL_DISABLE_AUTOINCR'), 'DBTF_disable_autoincr', $record['DBTF_disable_autoincr'], CLX_NUM_ZERO);
        } else {
            $DBFL = &getTool('DBFL');
            $contents .= $DBFL->placeChooseField($msg, dict('GEN_LBL_FIELDNAME') . ':', 'DBTF_fieldname', $record['DBTF_fieldname'], $table);
        }
        $form->addContents($table->source() . '<br>');
        $table = new htmlTable(CLX_WIDTH_TWO);
        $table->setProps(array(CLX_PROP_CLASS => CLX_CLASS_FORM, CLX_PROP_WIDTH => CLX_PROP_100PCT));
        $buttons = '';
        if (!empty($modify)) {
            $buttons .= htmlButton::getButton('submit', dict('GEN_LBL_SAVE'), array('action' => 'MODIFY', 'form' => $form->getProp(CLX_PROP_ID)))->source();
        } else {
            $c = new htmlFormSubmit(dict('GEN_LBL_INCLUDE', dict('GEN_LBL_FIELD')), 'INCLUDE', $form->getProp(CLX_PROP_ID));
            $buttons .= $c->source();
        }
        $c = new htmlButtonCancel();
        $buttons .= $c->source();
        $table->addCell($buttons, array(CLX_PROP_CLASS => CLX_CLASS_FOOTERS, CLX_PROP_COLSPAN => CLX_WIDTH_TWO));
        $form->addContents($table->source());
        $contents .= $form->source();
        return $contents;
    }

    function checkForm(&$msg) {
        $msg = array();
        $DBFL = &getTool('DBFL');
        UserRequest::$post['DBTF_fieldname'] = uppercase(UserRequest::$post['DBTF_fieldname']);
        if (UserRequest::$request['ac'] <> 'MODIFY') {
            if (!$DBFL->fieldexists(UserRequest::$post['DBTF_fieldname'])) {
                $msg['DBTF_fieldname'] = dict('TOOL_DBTF_ERR_NOFIELD');
                $msg['ERROR'] = 'NO_FIELD';
            } else {
                $qty = ClxCommon::$db->oneExistsRecords($this->tablename, 'TABLECODE=' . singleQuote(UserRequest::$post['DBTF_tablecode']) . ' AND FIELDNAME=' . singleQuote(UserRequest::$post['DBTF_fieldname']));
                if ($qty > CLX_NUM_ZERO) {
                    $msg['DBTF_fieldname'] = dict('TOOL_DBTF_ERR_ALREADYINCLUDED');
                }
            }
        }
        //Test that not exists a relation that may set null
        if (UserRequest::$request['ac'] <> 'INCLUDE' && UserRequest::$post['DBTF_field_required'] == CLX_NUM_ONE) {
            $sql = 'SELECT COUNT(DBDC.ID) ID FROM CLX_DB_DYNRELATIONS DBDR, CLX_DB_DYNRELATION_COLS DBDC WHERE DBDR.PARENT_TABLECODE = DBDC.PARENT_TABLECODE AND DBDR.CHILD_TABLECODE = DBDC.CHILD_TABLECODE AND DBDR.ACTIVE = 1 AND DBDR.RELATION_TYPE = 1 AND DBDR.CHILD_TABLECODE = ' . singleQuote(UserRequest::$post['DBTF_tablecode']) . ' AND DBDC.CHILD_FIELD = ' . singleQuote(UserRequest::$post['DBTF_fieldname']) . ' AND (DBDR.DELETE_ACTION = 2 OR DBDR.UPDATE_ACTION = 2) ';
            if (ClxCommon::$db->getOne($sql) > CLX_NUM_ZERO) {
                $msg['DBTF_field_required'] = dict('TOOL_DBTF_RELATION_REQ_null');
            }
        }
        return count($msg) == CLX_NUM_ZERO;
    }

    function searchForm() {
        $DBTB = &getTool('DBTB');
        $contents = '';
        $msg = array();
        if (empty(UserRequest::$request['tablecode'])) {
            $contents = note(dict('TOOL_DBTF_ERR_MUSTSELECTFIELD'));
        } elseif (!$DBTB->tableExists(UserRequest::$request['tablecode'])) {
            $contents = note(dict('TOOL_DBTF_ERR_TABLECODESELNOTFOUND', UserRequest::$request['tablecode']));
        } else {
            $tablecode = UserRequest::$request['tablecode'];
            // Search Form
            $form = new htmlForm(CLX_PROP_ACTION, UserRequest::$server['PHP_SELF']);
            $form->addArgument(CLX_PROP_TO, $this->code);
            $form->addArgument('ac', 'SEARCH');
            $form->addArgument(CLX_PROP_TARGET, UserRequest::$request[CLX_PROP_TARGET]);
            $form->addArgument('tablecode', $tablecode);
            $form->addArgument('confirm', '1');
            $table = new htmlTable(CLX_WIDTH_TWO);
            $table->setProps(array(CLX_PROP_CLASS => CLX_CLASS_FORM, CLX_PROP_WIDTH => CLX_PROP_100PCT));
            $table->addCell(dict('TOOL_DBTF_TITLE_SEARCH'), array(CLX_PROP_CLASS => CLX_CLASS_TITLE, CLX_PROP_COLSPAN => CLX_WIDTH_TWO));
            if (!array_key_exists('DBTF_fieldname', UserRequest::$post))
                UserRequest::$post['DBTF_fieldname'] = '';
            $table->addField($msg, dict('GEN_LBL_FIELDNAME') . ':', 'DBTF_fieldname', UserRequest::$post['DBTF_fieldname'], CLX_NUM_32, CLX_NUM_48);
            $buttons = htmlButton::getButton('submit', dict('GEN_LBL_SEARCH'), array('action' => 'SEARCH', 'form' => $form->getProp(CLX_PROP_ID)))->source();
            $c = new htmlButtonGoto(dict('GEN_LBL_CANCEL'), 'javascript:closeWindow();');
            $buttons .= $c->source();
            $table->addCell($buttons, array(CLX_PROP_CLASS => CLX_CLASS_FOOTERS, CLX_PROP_COLSPAN => CLX_WIDTH_TWO));
            $form->setContents($table->source());
            $contents .= $form->source();
            // Search Procedure
            $table = new htmlTable(CLX_WIDTH_ONE);
            $table->setProps(array(CLX_PROP_CLASS => CLX_CLASS_LIST, CLX_PROP_WIDTH => CLX_PROP_100PCT));
            $criteria = 'TABLECODE=' . singleQuote($tablecode);
            $boolean = ' AND ';
            if (!empty(UserRequest::$post['DBTF_fieldname'])) {
                $criteria .= $boolean . 'FIELDNAME LIKE ' . singleQuote(UserRequest::$post['DBTF_fieldname'] . '%');
                $boolean = ' and ';
            }
            $qty = ClxCommon::$db->oneExistsRecords($this->tablename, $criteria);
            if ($qty == CLX_NUM_ZERO) {
                $table->addCell(dict('TOOL_GEN_ERR_NOMATCH'), array(CLX_PROP_CLASS => CLX_CLASS_ALERT, CLX_PROP_COLSPAN => CLX_WIDTH_ONE));
            } else {
                $table->addCell(dict('TOOL_GEN_MSG_FOUNDRECORDS'), array(CLX_PROP_CLASS => CLX_CLASS_SUBTITLE, CLX_PROP_COLSPAN => CLX_WIDTH_ONE));
                $records = ClxCommon::$db->getRecords($this->tablename, $criteria);
                $i = CLX_INIT_ZERO;
                while ($record = ClxCommon::$db->getNextRecord($records, ASSOCIATIVE)) {
                    $js = 'javascript:setLookupValue(' . quote(UserRequest::$request[CLX_PROP_TARGET]) . ', ' . quote($record['FIELDNAME']) . ');';
                    $table->addCells(($i % CLX_NUM_TWO == CLX_NUM_ZERO) ? CLX_CLASS_DATA : CLX_CLASS_DATAALT, array('<page t=db_tablefileds></page><a href=' . singleQuote($js) . '>' . $record['FIELDNAME'] . '</a>'));
                    $i++;
                }
            }
            $contents .= $table->source();
        }
        return $contents;
    }

    function confirmDeleteFill(&$table, $record) {
        $table->addData(dict('GEN_LBL_FIELDNAME') . ':', $record['DBTF_fieldname']);
    }

    function reorderRecords() {
        $records = ClxCommon::$db->getRecords($this->tablename, 'TABLECODE=' . singleQuote(UserRequest::$post['DBTF_tablecode']) . ' ORDER BY SEQNUM');
        $list = array();
        while ($record = ClxCommon::$db->getNextRecord($records, ASSOCIATIVE)) {
            $list[$record['ID']] = $record['FIELDNAME'];
        }
        return $list;
    }

    function reorderAction() {
        parent::reorderAction();
        $callingpage = peekHistory();
        $tablename = tableName($callingpage['tablecode']);
        //Reorder physical table
        $qty = ClxCommon::$db->oneExistsRecords('CLX_DB_TABLES', 'IDCODE = ' . singleQuote($callingpage['tablecode']) . ' AND TYPE IN (' . singleQuoteArray(array('VIEW', 'TEMP')) . ')');
        if ($qty === CLX_NUM_ZERO && ClxCommon::$db->tableExists($tablename)) {
            //reorder if table exists
            $fields = array();
            $sql = 'SELECT DBTF.FIELDNAME, DBFL.FIELDTYPE FROM ' . $this->tablename . ' DBTF, CLX_DB_FIELDS DBFL WHERE DBTF.FIELDNAME = DBFL.FIELDNAME AND DBTF.TABLECODE =' . singleQuote($callingpage['tablecode']) . ' ORDER BY DBTF.SEQNUM ASC';
            $rs = ClxCommon::$db->executeQuery($sql);
            while ($r = ClxCommon::$db->getNextRecord($rs, ASSOCIATIVE)) {
                if ($r['FIELDTYPE'] === 'RECU') {
                    $fields[] = $r['FIELDNAME'] . '_R1';
                    $fields[] = $r['FIELDNAME'] . '_R2';
                    $fields[] = $r['FIELDNAME'] . '_R3';
                    $fields[] = $r['FIELDNAME'] . '_R4';
                    $fields[] = $r['FIELDNAME'] . '_R5';
                    $fields[] = $r['FIELDNAME'] . '_R6';
                    $fields[] = $r['FIELDNAME'] . '_R7';
                    $fields[] = $r['FIELDNAME'] . '_R8';
                    $fields[] = $r['FIELDNAME'] . '_R9';
                    $fields[] = $r['FIELDNAME'] . '_R0';
                } else {
                    $fields[] = $r['FIELDNAME'];
                }
            }
            if (!empty($fields)) {
                ClxCommon::$db->reorderFields($tablename, $fields);
            }
        }
        $DBTB = &getTool('DBTB');
        $DBTB->versionIncrease('IDCODE', $callingpage['tablecode']);
    }

    function loadToolRecord($recordid) {
        $values = ClxCommon::$db->getRecord($this->tablename, 'ID= ' . $recordid);
        $record = array();
        $record['DBTF_id'] = $recordid;
        $record['DBTF_tablecode'] = $values['TABLECODE'];
        $record['DBTF_fieldname'] = $values['FIELDNAME'];
        $record['DBTF_field_required'] = $values['FIELD_REQUIRED'];
        $record['DBTF_field_default'] = $values['FIELD_DEFAULT'];
        $record['DBTF_cascade_delete'] = $values['CASCADE_DELETE'];
        $record['DBTF_seqnum'] = $values['SEQNUM'];
        $record['DBTF_disable_autoincr'] = $values['DISABLE_AUTOINCR'];
        return $record;
    }

    function saveRecord() {
		if (ClxCommon::$db->debugCF==true){
			pre('inicio saveRecord de db_tablefields');
			pre('post');
			pre(UserRequest::$post);
			pre('this tablename');
			pre($this->tablename);
			
		}	
        $record = array();
        $record['TABLECODE'] = UserRequest::$post['DBTF_tablecode'];
        $record['FIELDNAME'] = UserRequest::$post['DBTF_fieldname'];
        $record['FIELD_REQUIRED'] = UserRequest::$post['DBTF_field_required'];
        $record['FIELD_DEFAULT'] = UserRequest::$post['DBTF_field_default'];
        $record['CASCADE_DELETE'] = UserRequest::$post['DBTF_cascade_delete'];
        $record['DISABLE_AUTOINCR'] = UserRequest::$post['DBTF_disable_autoincr'];
        ClxCommon::$db->updateRecord($this->tablename, $record, 'ID=' . singleQuote(UserRequest::$post['DBTF_id']));
        $this->clearHeap(UserRequest::$post['DBTF_tablecode']);
        $DBTB = &getTool('DBTB');
        $DBTB->versionIncrease('IDCODE', UserRequest::$post['DBTF_tablecode']);
		$result=ClxCommon::$db->executequery($sql='SELECT X.* FROM CLX_DB_TABLE_HIST X JOIN CLX_DB_TABLES Y ON X.TABLE_CODE_HIST=Y.IDCODE WHERE X.TABLE_CODE='.singleQuote($record['TABLECODE'])) ;
		$info_hist_exist = ClxCommon::$db->getNextRecord($result, ASSOCIATIVE) ;
		if (ClxCommon::$db->debugCF==true){
			pre('info history');
			pre($info_hist_exist);
			pre('sql');
			pre($sql);
		}
		if (!empty($info_hist_exist['TABLE_CODE']) && $info_hist_exist['TABLE_CODE']==$record['TABLECODE']) {
			$record['TABLECODE']=$info_hist_exist['TABLE_CODE_HIST'];
			ClxCommon::$db->updateRecord($record['TABLECODE'], $record, 'ID=' . singleQuote($info_hist_exist['id']));

			$DBTB->versionIncrease('IDCODE', $info_hist_exist['TABLE_CODE_HIST']);
		}
    }

    function createRecordVoid_() {
        ClxFreeReturn($this->createRecord_());
    }

    function createRecord_() {
		if (ClxCommon::$db->debugCF==true){
			pre(UserRequest::$post);
		}	
        $record = array();
        $record['TABLECODE'] = UserRequest::$post['DBTF_tablecode'];
        $record['FIELDNAME'] = uppercase(UserRequest::$post['DBTF_fieldname']);
        if (!array_key_exists('DBFT_field_required', UserRequest::$post)) {
            $field = ClxCommon::$db->getFields('CLX_DB_FIELDS', 'FIELD_REQUIRED, FIELD_DEFAULT, FIELDTYPE, FIELD_EXTRA5', 'FIELDNAME=' . singleQuote(uppercase(UserRequest::$post['DBTF_fieldname'])));
            $record['FIELD_REQUIRED'] = $field['FIELD_REQUIRED'];
            $record['FIELD_DEFAULT'] = $field['FIELD_DEFAULT'];
            if ($field['FIELDTYPE'] === true) {
                $record['CASCADE_DELETE'] = $field['FIELD_EXTRA5'];
            } else {
                $record['CASCADE_DELETE'] = CLX_INIT_ZERO;
            }
        } else {
            $record['FIELD_REQUIRED'] = UserRequest::$post['DBFT_field_required'];
            $record['FIELD_DEFAULT'] = UserRequest::$post['DBFT_field_default'];
            $record['CASCADE_DELETE'] = UserRequest::$post['DBFT_cascade_delete'];
        }
		if (empty( $record['FIELD_REQUIRED'])){
			 $record['FIELD_REQUIRED']='0';
		}	
        $record['DISABLE_AUTOINCR'] = CLX_INIT_ZERO;
        $record['CASCADE_DELETE'] = (!empty($record['CASCADE_DELETE'])) ? CLX_NUM_ONE : CLX_NUM_ZERO;
        $record['SEQNUM'] = ClxCommon::$db->getField($this->tablename, 'SEQNUM', 'TABLECODE=' . singleQuote(UserRequest::$post['DBTF_tablecode']) . ' ORDER BY SEQNUM DESC') + CLX_NUM_ONE;
        $recordid = ClxCommon::$db->createRecord($this->tablename, $record, false);
        $record['ID'] = $recordid;
        //Set Relation if delete when parent is deleted
        $this->createRelation($record['TABLECODE'], $record['FIELDNAME']);


        //Phisically create field
        $this->createTableField($record['TABLECODE'], $record);
		$info_hist_exist=ClxCommon::$db->getRecord('CLX_DB_TABLE_HIST', ' TABLE_CODE='.singleQuote($record['TABLECODE'])) ;
		/*if (!empty($info_hist_exist['TABLE_CODE']) && $info_hist_exist['TABLE_CODE']==$record['TABLECODE']) {
			$this->createTableField($info_hist_exist['TABLE_CODE_HIST'], $record);
		}*/
        $DBTB = &getTool('DBTB');
        $DBTB->versionIncrease('IDCODE', $record['TABLECODE']);
		
		$info_hist_exist=ClxCommon::$db->getRecord('CLX_DB_TABLE_HIST', ' TABLE_CODE='.singleQuote($record['TABLECODE'])) ;
		
		if (!empty($info_hist_exist['TABLE_CODE']) && $info_hist_exist['TABLE_CODE']==$record['TABLECODE']) {
			$record['TABLECODE']=$info_hist_exist['TABLE_CODE_HIST'];
			unset($record['ID']);
			$recordid = ClxCommon::$db->createRecord($this->tablename, $record, false);
        	$this->createTableField($info_hist_exist['TABLE_CODE_HIST'], $record);
			$DBTB->versionIncrease('IDCODE', $info_hist_exist['TABLE_CODE_HIST']);
		}
        return $recordid;
    }

    /**
     * Deletes tablefield records
     * @see clx_tool::deleteRecord()
     */
    function deleteRecordSk($id) {
        $record = ClxCommon::$db->getRecord($this->tablename, 'ID=' . $id);
		if($this->deleteRecordSk_validation($record) === false) {
			return;
		}
        $tablerec = ClxCommon::$db->getFields('CLX_DB_TABLES', 'TYPE', 'IDCODE = ' . singleQuote($record['TABLECODE']));
        $DBTB = &getTool('DBTB');
        $DBTB->versionIncrease('IDCODE', $record['TABLECODE']);
        ClxCommon::$db->deleteRecords($this->tablename, 'ID=' . $id);
        //Deleting relation
        $this->deleteRelation($record['TABLECODE'], $record['FIELDNAME']);
		
		$info_hist_exist=ClxCommon::$db->getRecord('CLX_DB_TABLE_HIST', ' TABLE_CODE='.singleQuote($record['TABLECODE'])) ;
		if (!empty($info_hist_exist['TABLE_CODE']) && $info_hist_exist['TABLE_CODE']==$record['TABLECODE']) {
			ClxCommon::$db->deleteRecords($this->tablename,' TABLECODE= '.singleQuote($info_hist_exist['TABLE_CODE_HIST']). ' AND FIELDNAME=' . singleQuote($record['FIELDNAME']));
			$reco['TABLECODE']=$info_hist_exist['TABLE_CODE'];
        	$this->deleteRelation($info_hist_exist['TABLE_CODE_HIST'], $record['FIELDNAME']);
		}
        if ($tablerec['TYPE'] <> 'VIEW') {
            ClxFreeReturn($this->deleteField_($record));
        }
    }
	
    /**
     * Verify if tablefield record can be deleted
     */
    function deleteRecordSk_validation($fieldrec) {
		$errmsg = CLX_CHAR_EMPTY_STRING;
		if(!defined('DBTF_FLDS_LOCK_DELETE')) {
			define('DBTF_FLDS_LOCK_DELETE', CLX_CHAR_EMPTY_STRING.CLX_NUM_ZERO);
		}
		if(!defined('DBTF_FLDS_LOCK_DELETE_BY_ADM')) {
			define('DBTF_FLDS_LOCK_DELETE_BY_ADM', CLX_CHAR_EMPTY_STRING.CLX_NUM_ZERO);
		}
		$sql = 'SELECT DBTI.IDCODE NDX_NAME, DBIC.FIELDNAME, DBTI.TYPE NDX_TYPE FROM CLX_DB_TABLEINDEXES DBTI, CLX_DB_INDEX_COLUMNS DBIC WHERE DBTI.IDCODE = DBIC.INDEX_CODE AND DBTI.TABLECODE = ' . singleQuote($fieldrec['TABLECODE']) . ' /* AND DBTI.TYPE = ' . singleQuote('PRI') . ' */ AND DBIC.FIELDNAME = ' . singleQuote($fieldrec['FIELDNAME']);
		$ndx_rs = ClxCommon::$db->executeQuery($sql);
		$ndx_names = CLX_CHAR_EMPTY_STRING;
		$sep = '';
		while ($ndx_record = ClxCommon::$db->getNextRecord($ndx_rs)) {
			$ndx_names .= $sep . $ndx_record['NDX_NAME'] . ' [' . $ndx_record['NDX_TYPE'] . ']';
			$sep = ', ';
		}
		//if (strpos(','.DBTF_FLDS_LOCK_DELETE.',', ','.$fieldrec['FIELDNAME'].',') !== false && empty($_SESSION['user_record']['isadmin'])) {
		if (strpos(','.str_replace(' ', CLX_CHAR_EMPTY_STRING, DBTF_FLDS_LOCK_DELETE).',', ','.$fieldrec['FIELDNAME'].',') !== false) {
			// No se puede eliminar de ninguna manera, a menos que sea Administrador
			if(empty($_SESSION['user_record']['isadmin']) || (!empty($_SESSION['user_record']['isadmin']) && DBTF_FLDS_LOCK_DELETE_BY_ADM === CLX_CHAR_EMPTY_STRING.CLX_NUM_ZERO)) {
				/*
				 * TOOL_DBTF_ERR_FLDLOCKED
				 * Column ^1 cannot be deleted because is a reserved column (see DBTF_FLDS_LOCK_DELETE* parameter(s))
				 * La columna ^1 No se puede eliminar porque es una columna de uso reservado (ver parámetro(s) DBTF_FLDS_LOCK_DELETE*)
				 */
				$errmsg .= ($errmsg === CLX_CHAR_EMPTY_STRING ? CLX_CHAR_EMPTY_STRING : '<BR>') . dict('TOOL_DBTF_ERR_FLDLOCKED', $fieldrec['FIELDNAME']);
			}
		}
		if($ndx_names !== CLX_CHAR_EMPTY_STRING) {
			// No se puede eliminar porque está en algún indice
			/*
			 * TOOL_DBTF_ERR_INANYINDEX
			 * Column ^1 cannot be deleted because is in index(es): ^2
			 * La columna ^1 No se puede eliminar porque está en el o los indices: ^2
			 */
			$errmsg .= ($errmsg === CLX_CHAR_EMPTY_STRING ? CLX_CHAR_EMPTY_STRING : '<BR>') . dict('TOOL_DBTF_ERR_INANYINDEX', $fieldrec['FIELDNAME'], $ndx_names);
		}
        if($errmsg !== CLX_CHAR_EMPTY_STRING) {
			addMessage($errmsg);
        }
		return ($errmsg === CLX_CHAR_EMPTY_STRING);
	}
	

    /**
     * Include Fields on table
     * @param string $tablecode
     * @param string $fieldname
     * @return number
     */
    function includeField($tablecode, $fieldname) {

        $record = array();
        $record['TABLECODE'] = $tablecode;
        $record['FIELDNAME'] = uppercase($fieldname);
        $DBFL = &getTool('DBFL');
        $field = $DBFL->getField($record['FIELDNAME']);
        $record['FIELD_REQUIRED'] = $field['FIELD_REQUIRED'];
        $record['FIELD_DEFAULT'] = $field['FIELD_DEFAULT'];
        $record['CASCADE_DELETE'] = (!empty($field['FIELDTYPE'])) ? (( $field['FIELD_EXTRA5'] =='') ?'0': $field['FIELD_EXTRA5'] ) : CLX_INIT_ZERO;
        $record['DISABLE_AUTOINCR'] = CLX_INIT_ZERO;
        $record['SEQNUM'] = ClxCommon::$db->getField($this->tablename, 'SEQNUM', 'TABLECODE=' . singleQuote($tablecode) . ' ORDER BY SEQNUM DESC') + CLX_NUM_ONE;
        $recordid = ClxCommon::$db->loadDbRecord($this->tablename, $record, false, 'TABLECODE = ' . singleQuote($record['TABLECODE']) . ' AND FIELDNAME = ' . singleQuote($record['FIELDNAME']));

        $record['ID'] = $recordid;

        $this->createTableField($tablecode, $record);
        $this->clearHeap($tablecode);

        $DBTB = &getTool('DBTB');
        $DBTB->versionIncrease('IDCODE', $record['TABLECODE']);
		
		$info_hist_exist=ClxCommon::$db->getRecord('CLX_DB_TABLE_HIST', ' TABLE_CODE='.singleQuote($tablecode)) ;
		if (!empty($info_hist_exist['TABLE_CODE']) && $info_hist_exist['TABLE_CODE']==$tablecode) {
			$record['TABLECODE']=$info_hist_exist['TABLE_CODE_HIST'];
			$this->createTableField($info_hist_exist['TABLE_CODE_HIST'], $record);
			$this->clearHeap($info_hist_exist['TABLE_CODE_HIST']);

			$DBTB = &getTool('DBTB');
			$DBTB->versionIncrease('IDCODE', $info_hist_exist['TABLE_CODE_HIST']);
		}
        
		
		
        return $recordid;
    }

    /**
     * Creates Relation when lookup is added to the table
     * @param unknown_type $tablecode
     * @param unknown_type $fieldname
     */
    function createRelation($tablecode, $fieldname) {
        $DBFL = &getTool('DBFL');
        $field = $DBFL->getField($fieldname);
        //Adding Relation if lookup
        if ($field['FIELDTYPE'] === 'LOOK') {
            //Adding Relation
            $rec = array('IDNAME' => $field['FIELD_EXTRA1'] . '-' . $tablecode,
                'PARENT_TABLECODE' => $field['FIELD_EXTRA1'],
                'PARENT_FIELDNAME' => 'ID',
                'CHILD_TABLECODE' => $tablecode,
                'CHILD_FIELDNAME' => 'ID',
                'ACTIVE' => CLX_NUM_ONE,
                'RELATION_TYPE' => CLX_NUM_ONE,
                'DELETE_ACTION' => $field['FIELD_EXTRA5'],
                'UPDATE_ACTION' => CLX_NUM_ZERO,
                'DUPLICATE_ACTION' => CLX_NUM_ZERO,
                'RELT_PFIELD' => 'PARENT_RECORDID',
                'RELT_CFIELD' => 'CHILD_RECORDID',
                'TABLECODE' => 'DYRL',
                'SEQNUM' => CLX_NUM_ZERO);
            $rec['ID'] = ClxCommon::$db->loadDbRecord('CLX_DB_DYNRELATIONS', $rec, true, 'PARENT_TABLECODE=' . singleQuote($field['FIELD_EXTRA1']) . ' AND CHILD_TABLECODE=' . singleQuote($tablecode));
            //Adding Relation Columns
            $recc = array('PARENT_TABLECODE' => $field['FIELD_EXTRA1'], 'CHILD_TABLECODE' => $tablecode, 'SEQNUM' => CLX_NUM_ONE, 'PARENT_FIELD' => $field['FIELD_EXTRA2'], 'CHILD_FIELD' => $fieldname, 'DELETE_FLAG' => CLX_NUM_ZERO, 'COPY_FLAG' => CLX_NUM_ONE);
            ClxCommon::$db->loadDbRecord('CLX_DB_DYNRELATION_COLS', $recc, true, 'PARENT_TABLECODE=' . singleQuote($field['FIELD_EXTRA1']) . ' AND CHILD_TABLECODE=' . singleQuote($tablecode) . ' AND SEQNUM=1');
        }
    }

    /**
     * Deletes Relation on lookup
     * @param string $tablecode
     * @param string $fieldname
     */
    function deleteRelation($tablecode, $fieldname) {
        $DBFL = &getTool('DBFL');
        $field = $DBFL->getField($fieldname);
        //Adding Relation if lookup
        if ($field['FIELDTYPE'] === 'LOOK') {
            $sql = 'DELETE FROM CLX_DB_DYNRELATION_COLS WHERE PARENT_TABLECODE=' . singleQuote($field['FIELD_EXTRA1']) . ' AND CHILD_TABLECODE=' . singleQuote($tablecode) . ' AND EXISTS (SELECT ID FROM CLX_DB_DYNRELATIONS WHERE PARENT_TABLECODE=' . singleQuote($field['FIELD_EXTRA1']) . ' AND CHILD_TABLECODE=' . singleQuote($tablecode) . ' AND RELATION_TYPE = 1)';
            ClxCommon::$db->executeQuery($sql);
            $sql = 'DELETE FROM CLX_DB_DYNRELATIONS WHERE PARENT_TABLECODE=' . singleQuote($field['FIELD_EXTRA1']) . '        	       AND CHILD_TABLECODE=' . singleQuote($tablecode) . ' AND RELATION_TYPE = 1';
            ClxCommon::$db->executeQuery($sql);
        }
    }

    function defaultRecord() {
        $record = array();
        $record['DBTF_tablecode'] = UserRequest::$post['DBTF_tablecode'];
        $record['DBTF_fieldname'] = '';
        $record['DBTF_seqnum'] = '0';
        $record['DBTF_field_required'] = CLX_INIT_ZERO;
        $record['DBTF_field_default'] = '';
        $record['DBTF_cascade_delete'] = CLX_INIT_ZERO;
        $record['DBTF_disable_autoincr'] = CLX_INIT_ZERO;
        return $record;
    }

    function placeChooseTableField($msg, $tablefieldname, $prompt, $name, $default, &$table) {

        $contents = setJavaScriptTag(getJavaScriptFilename('DBTF_searchwindow.js'), true);
        $searchIcon = getSkinIcon('view', 'javascript:dbtf_searchwindow(' . singleQuote($tablefieldname) . ', ' . singleQuote($name) . ')', 'Search for a field');
        if (!empty($msg[$name])) {
            $table->addCell($msg[$name], array(CLX_PROP_CLASS => CLX_CLASS_ALERT, CLX_PROP_COLSPAN => $table->width));
        }
        $table->addCell($prompt, array(CLX_PROP_CLASS => CLX_CLASS_PROMPT));
        $c = new htmlField($name, $default, CLX_NUM_32, CLX_NUM_64);
        $table->addCell($contents . $c->source() . $searchIcon, array(CLX_PROP_CLASS => CLX_CLASS_DATA));
        return $contents;
    }

    /**
     * Test if field exists on table
     * @param type $tablecode
     * @param type $fieldname
     * @return boolean
     */
    function fieldExists(&$tablecode, &$fieldname) {
        $DBTB = &getTool('DBTB');
        if ($DBTB->tableExists($tablecode)) {
            $qty = ClxCommon::$db->oneExistsRecords($this->tablename, 'TABLECODE=' . singleQuote($tablecode) . ' AND FIELDNAME=' . singleQuote(uppercase($fieldname)));
            if ($qty > CLX_NUM_ZERO) {
                $fieldname = ClxCommon::$db->getField($this->tablename, 'FIELDNAME', 'TABLECODE=' . singleQuote($tablecode) . ' AND FIELDNAME=' . singleQuote(uppercase($fieldname)));
            }
        } else {
            $qty = CLX_INIT_ZERO;
        }
        return ($qty > CLX_NUM_ZERO) ? true : false;
    }

    /**
     * Build Tool Lookup field in forms.
     * @param string $name
     * @param string $default
     * @param array $data
     * @param int $display_size
     * @param int $maxlength
     * @return string
     */
    function chooseRecord($name, $default, $data = array(), $display_size = CLX_NUM_32, $maxlength = CLX_NUM_64) {

        $contents = setJavaScriptTag(getJavaScriptFilename('DBTF_searchwindow.js'), true);
        $code = $this->code;
        $alt = 'TOOL_' . $code . '_ALT_SEARCH';
        $tablecode = $data['tablecode'];
        $searchIcon = getSkinIcon('view', 'javascript:dbtf_searchwindow(' . singleQuote($tablecode) . ', ' . singleQuote($name) . ');', dict($alt));
        $c = new htmlField($name, $default, CLX_NUM_32, CLX_NUM_64);
        return $contents . $c->source() . $searchIcon;
    }

    function getTableFields($tablecode) {
		$fetched = false;
		if (kApcTtl_13>0) $fls = apc_fetchFct($findVar=kSpaceName . '.dbtf.' . $tablecode, $fetched);
		if (empty($fetched)) {
			$fls = array();
        
			// send deletemsg to fields
			$sql = 'SELECT DBFL.ID, DBFL.ID TABLE_FIELD_ID, DBFL.FIELDNAME, DBFL.FIELDFORMLABEL, DBFL.FIELDLISTLABEL, DBFL.FIELDPREFIX, DBFL.FIELDSUFIX, DBFL.FIELDTYPE, DBFL.FIELD_ANCHOR, DBFL.FIELDFORMULA, DBFL.FIELD_SIZE, DBTF.FIELD_REQUIRED, DBFL.FIELD_FORMAT, DBFL.FIELD_ALIGN, DBTF.FIELD_DEFAULT, DBFL.FIELD_MINVALUE, DBFL.FIELD_MAXVALUE, DBFL.FIELD_DISPLAYSIZE, DBFL.FIELD_EXTRA1, DBFL.FIELD_EXTRA2, DBFL.FIELD_EXTRA3, DBFL.FIELD_EXTRA4, DBFL.FIELD_EXTRA5, DBFL.FIELD_EXTRA6, DBFL.FIELD_EXTRA7, DBFL.FIELD_EXTRA8, DBFL.FIELD_EXTRA9, DBTF.CASCADE_DELETE FROM CLX_DB_FIELDS DBFL, CLX_DB_TABLEFIELDS DBTF WHERE DBTF.FIELDNAME = DBFL.FIELDNAME AND DBTF.TABLECODE = ' . singleQuote($tablecode) . ' ORDER BY DBTF.SEQNUM ASC';
			$fields = ClxCommon::$db->executeQuery($sql);
			while (($fieldrecord = ClxCommon::$db->getNextRecord($fields))) {
				$fls[] = $fieldrecord;
			}
			if (kApcTtl_13>0 && count($fls) > CLX_NUM_ZERO) {
				apc_addFct($findVar, $fls, kApcTtl_13);
			}
		}
        return $fls;
    }

    function createTable($tablerec) { // called by the table manager every time a table is created.
        $record = array();
        $record['TABLECODE'] = $tablerec['IDCODE'];
        $record['FIELDNAME'] = 'ID';
        $record['SEQNUM'] = CLX_NUM_ONE;
        $record['FIELD_REQUIRED'] = CLX_INIT_ZERO;
        $record['CASCADE_DELETE'] = CLX_INIT_ZERO;
        $record['DISABLE_AUTOINCR'] = CLX_INIT_ZERO;
        ClxCommon::$db->loadDbRecord($this->tablename, $record, false, 'TABLECODE = ' . singleQuote($record['TABLECODE']) . ' AND FIELDNAME = ' . singleQuote($record['FIELDNAME']));
    }

    /**
     * Creates field on table
     * @param type $tablecode
     * @param type $fieldrecord
     */
    function createTableField($tablecode, $fieldrecord) {
        $tablerec = ClxCommon::$db->getFields('CLX_DB_TABLES', 'MODULECODE,IDNAME,TYPE,STATUS', 'IDCODE=' . singleQuote($tablecode));
        $tablename = $tablerec['MODULECODE'] . '_' . $tablerec['IDNAME'];
        $tableExists = ClxCommon::$db->tableExists($tablename);
        if (!empty($tablerec['STATUS']) && !($tablerec['TYPE'] === 'VIEW' || ($tablerec['TYPE'] === 'TEMP' && kDatabaseType <> 'oci')) && $tableExists === true) {
            $DBFL = &getTool('DBFL');
            $DBFL->createPhysicalField($tablename, $fieldrecord['FIELDNAME']);
        }
        // Elimina eventual tabla temporal (Oracle) de Audit
        $pat_err_msg = ClxCommon::$db->postAlterTable($tablename);
        if($pat_err_msg !== '') {
            //echo $pat_err_msg;
			addMessage($pat_err_msg);
            return false;
        }
		//$info_hist_exist=ClxCommon::$db->getRecord('CLX_DB_TABLE_HIST', ' TABLE_CODE='.singleQuote($tablecode)) ;
		//if (!empty($info_hist_exist['TABLE_CODE']) &&$info_hist_exist['TABLE_CODE']==$tablecode) {
		//	$fieldrecord['TABLECODE']=$info_hist_exist['TABLE_CODE_HIST'];
		//	$this->deleteField_($info_hist_exist['TABLE_CODE_HIST'],$fieldrecord);
		//}	

	}

    function deleteField_($fieldrec) {
        $tablerec = ClxCommon::$db->getFields('CLX_DB_TABLES', 'MODULECODE, IDNAME, STATUS, TYPE', 'IDCODE=' . singleQuote($fieldrec['TABLECODE']));
        $tablename = $tablerec['MODULECODE'] . '_' . $tablerec['IDNAME'];
        if (empty($tablerec['STATUS']) || ($tablerec['TYPE'] === 'TEMP' && kDatabaseType <> 'oci')) {
            //do noting
        } else {
			if (ClxCommon::$db->fieldExists($tablename, $fieldrec['FIELDNAME'])) {
				// Elimina la columna de la tabla
				$DBFL = &getTool('DBFL');
				$DBFL->deleteField($tablename, $fieldrec['FIELDNAME']);
			}
        }
        // Elimina eventual tabla temporal (Oracle) de Audit
        $pat_err_msg = ClxCommon::$db->postAlterTable($tablename);
        if($pat_err_msg !== CLX_CHAR_EMPTY_STRING) {
            //echo $pat_err_msg;
			addMessage($pat_err_msg);
			return false;
		}
		$info_hist_exist=ClxCommon::$db->getRecord('CLX_DB_TABLE_HIST', ' TABLE_CODE='.singleQuote($fieldrec['TABLECODE'])) ;
		if (!empty($info_hist_exist['TABLE_CODE'])  && $info_hist_exist['TABLE_CODE']==$fieldrec['TABLECODE']) {
			$fieldrec['TABLECODE']=$info_hist_exist['TABLE_CODE_HIST'];
			$this->deleteField_($fieldrec);
		}	

        return true;
    }

    /**
     * Clear field form heap
     * @param string $tablecode
     */
    private function clearHeap($tablecode) {
		apc_deleteFct(kSpaceName . '.dbtf.' . $tablecode);
		apc_deleteFct(kSpaceName . '.dbtb.dbfl.' . $tablecode);
    }

}
