<?php

/**
 * Tool Class
 * For more information contact  support@alenet.com
 *
 * @author       $Author: alenetdev $
 * @copyright    Alenet, Inc.
 * @since        $Date: 2019-01-22 15:45:22 -0400 (ma., 22 ene. 2019) $
 * @version        $Rev: 10496 $
 *
 * CHANGE LOG
 *  Oracle uppercase record changed
 *  Included label
 */
class ma_mail_accounts extends clx_tool {

    var $debug = false;

    function __construct() {
        $this->code = 'MAAC';
        $this->name = 'The Clientix Webmail - Accounts';
        $this->system = 'Mail';
        $this->version = '1.0.0.0';

        $this->has_access = true;


        $this->tablename = 'CLX_MA_MAILACCOUNTS';
        $this->record_name_singular = 'ma mail mail';
        $this->record_name_plural = 'ma ma mails';
        $this->primary_field = 'IDNAME';
        $this->secondary_field = 'EMAIL';

        $this->editform = array(CLX_CLASS_TITLE => 'TOOL_MAAC_TITLE_CREATE',
            'panels' => array(
                array('caption' => 'TOOL_MAAC_ACCOUNTINFO',
                    'type' => 'normal',
                    'entries' => array(
                        array('label' => 'GEN_LBL_IDNAME',
                            'fields' => array(
                                array('name' => 'idname', 'type' => 'string', 'default' => '', 'size' => CLX_NUM_HUNDRED, 'displaysize' => '64', 'dontmodify' => '1',
                                    'validations' => array(
                                        array('type' => 'string', 'required' => '1'), array('type' => 'unique', 'fieldname' => 'idname', 'tablename' => $this->tablename))
                                )
                            )
                        ), array('label' => 'GEN_LBL_FULLNAME',
                            'fields' => array(
                                array('name' => 'fullname', 'type' => 'string', 'default' => '', 'size' => CLX_NUM_HUNDRED, 'displaysize' => '64', 'readonly' => false, 'validations' => array(
                                        array('type' => 'string', 'required' => '1'))
                                )
                            )
                        ), array('label' => 'GEN_LBL_EMAIL',
                            'fields' => array(
                                array('name' => 'email', 'type' => 'string', 'default' => '', 'size' => CLX_NUM_HUNDRED, 'displaysize' => '64', 'readonly' => false, 'validations' => array(
                                        array('type' => 'string', 'required' => '1'), array('type' => 'mail')
                                    )
                                )
                            )
                        ), array('label' => 'GEN_LBL_MAILTYPE',
                            'fields' => array(
                                array('name' => 'email_type', 'type' => CLX_CLASS_LIST, 'default' => 'POP3', 'options' => array(
                                        'POP3' => 'GEN_LBL_POP3',
                                        'IMAP' => 'GEN_LBL_IMAP'
                                    )
                                )
                            )
                        ), array('label' => 'GEN_LBL_MAILSIGNATURE',
                            'fields' => array(
                                array('name' => 'signature', 'type' => 'text'))
                        ), array('label' => 'GEN_LBL_USERNAME',
                            'fields' => array(
                                array('name' => 'username', 'type' => 'lookup', 'default' => '', 'tool' => 'ACUS', 'readonly' => true, 'validations' => array(
                                        array('type' => 'lookup', 'fieldname' => 'username', 'tablename' => 'CLX_AC_USERS'))
                                )
                            )
                        ), array('label' => 'GEN_LBL_SYSTEM_DEFAULT',
                            'fields' => array(
                                array('name' => 'system_default', 'type' => CLX_CLASS_LIST, 'readonly' => true, 'options' => $this->yesno))
                        )
                    )
                ), array('caption' => 'TOOL_MAAC_INSERVERINFO',
                    'type' => 'normal',
                    'entries' => array(
                        array('label' => 'GEN_LBL_INSERVER',
                            'fields' => array(
                                array('name' => 'in_hostname', 'type' => 'string', 'default' => '', 'size' => CLX_NUM_HUNDRED, 'displaysize' => '64', 'readonly' => false, 'validations' => array(
                                        array('type' => 'string', 'required' => '1'))
                                )
                            )
                        ), array('label' => 'GEN_LBL_INPORT',
                            'fields' => array(
                                array('name' => 'in_port', 'type' => 'string', 'default' => '', 'size' => '6', 'displaysize' => '10', 'readonly' => false, 'validations' => array(
                                        array('type' => 'string', 'required' => '1'), array('type' => 'integer')
                                    )
                                )
                            )
                        ), array('label' => 'GEN_LBL_INSERVERUSER',
                            'fields' => array(
                                array('name' => 'in_username', 'type' => 'string', 'default' => '', 'size' => CLX_NUM_HUNDRED, 'displaysize' => '64', 'readonly' => false))
                        ), array('label' => 'GEN_LBL_INSERVERPASS',
                            'fields' => array(
                                array('name' => 'in_password', 'type' => 'password', 'default' => '', 'size' => CLX_NUM_HUNDRED, 'displaysize' => '64', 'readonly' => false))
                        ), array('label' => 'GEN_LBL_INSERVERDELMSG',
                            'fields' => array(
                                array('name' => 'in_deletemsg', 'type' => CLX_CLASS_LIST, 'default' => '1', 'options' => array(
                                        '0' => 'GEN_LBL_NO',
                                        '1' => 'GEN_LBL_YES'
                                    )
                                )
                            )
                        )
                    )
                ), array('caption' => 'TOOL_MAAC_OUTSERVERINFO',
                    'type' => 'normal',
                    'entries' => array(
                        array('label' => 'GEN_LBL_OUTSERVER',
                            'fields' => array(
                                array('name' => 'out_hostname', 'type' => 'string', 'default' => '', 'size' => CLX_NUM_HUNDRED, 'displaysize' => '64', 'readonly' => false, 'validations' => array(
                                        array('type' => 'string', 'required' => '1'))
                                )
                            )
                        ), array('label' => 'GEN_LBL_OUTPORT',
                            'fields' => array(
                                array('name' => 'out_port', 'type' => 'string', 'default' => '', 'size' => '6', 'displaysize' => '10', 'readonly' => false, 'validations' => array(
                                        array('type' => 'string', 'required' => '1'), array('type' => 'integer')
                                    )
                                )
                            )
                        ), array('label' => 'GEN_LBL_OUTSERVERAUTH',
                            'fields' => array(
                                array('name' => 'out_authenticate', 'type' => CLX_CLASS_LIST, 'default' => '1', 'options' => array(
                                        '0' => 'GEN_LBL_NO',
                                        '1' => 'GEN_LBL_YES'
                                    )
                                )
                            )
                        ), array('label' => 'GEN_LBL_OUTSERVERUSER',
                            'fields' => array(
                                array('name' => 'out_username', 'type' => 'string', 'default' => '', 'size' => CLX_NUM_HUNDRED, 'displaysize' => '64', 'readonly' => false))
                        ), array('label' => 'GEN_LBL_OUTSERVERPASS',
                            'fields' => array(
                                array('name' => 'out_password', 'type' => 'password', 'default' => '', 'size' => CLX_NUM_HUNDRED, 'displaysize' => '64', 'readonly' => false))
                        )
                    )
                )
            )
        );
        $this->viewform = array(CLX_CLASS_TITLE => 'TOOL_MAAC_TITLE_DISPLAY',
            'panels' => array(
                array('caption' => 'TOOL_MAAC_ACCOUNTINFO',
                    'type' => 'normal',
                    'entries' => array(
                        array('label' => 'GEN_LBL_IDNAME',
                            'fields' => array(
                                array('name' => 'idname', 'type' => 'string', 'default' => '', 'size' => CLX_NUM_HUNDRED, 'displaysize' => '64', 'readonly' => false))
                        ), array('label' => 'GEN_LBL_FULLNAME',
                            'fields' => array(
                                array('name' => 'fullname', 'type' => 'string', 'default' => '', 'size' => CLX_NUM_HUNDRED, 'displaysize' => '64', 'readonly' => false))
                        ), array('label' => 'GEN_LBL_EMAIL',
                            'fields' => array(
                                array('name' => 'email', 'type' => 'string', 'default' => '', 'size' => CLX_NUM_HUNDRED, 'displaysize' => '64', 'readonly' => false))
                        ), array('label' => 'GEN_LBL_MAILTYPE',
                            'fields' => array(
                                array('name' => 'email_type', 'type' => CLX_CLASS_LIST, 'default' => 'POP3', 'options' => array(
                                        'POP3' => 'GEN_LBL_POP3',
                                        'IMAP' => 'GEN_LBL_IMAP'
                                    )
                                )
                            )
                        ), array('label' => 'GEN_LBL_MAILSIGNATURE',
                            'fields' => array(
                                array('name' => 'signature', 'type' => 'text'))
                        ), array('label' => 'GEN_LBL_USERNAME',
                            'fields' => array(
                                array('name' => 'username', 'type' => 'string', 'default' => '', 'size' => CLX_NUM_HUNDRED, 'displaysize' => '64', 'readonly' => true))
                        ), array('label' => 'GEN_LBL_SYSTEM_DEFAULT',
                            'fields' => array(
                                array('name' => 'system_default', 'type' => CLX_CLASS_LIST, 'readonly' => false, 'options' => $this->yesno)
                            )
                        )
                    )
                ), array('caption' => 'TOOL_MAAC_INSERVERINFO',
                    'type' => 'normal',
                    'entries' => array(
                        array('label' => 'GEN_LBL_INSERVER',
                            'fields' => array(
                                array('name' => 'in_hostname', 'type' => 'string', 'default' => '', 'size' => CLX_NUM_HUNDRED, 'displaysize' => '64', 'readonly' => false))
                        ), array('label' => 'GEN_LBL_INPORT',
                            'fields' => array(
                                array('name' => 'in_port', 'type' => 'string', 'default' => '', 'size' => '6', 'displaysize' => '10', 'readonly' => false))
                        ), array('label' => 'GEN_LBL_INSERVERUSER',
                            'fields' => array(
                                array('name' => 'in_username', 'type' => 'string', 'default' => '', 'size' => CLX_NUM_HUNDRED, 'displaysize' => '64', 'readonly' => false))
                        ),
                        array('label' => 'GEN_LBL_INSERVERDELMSG',
                            'fields' => array(
                                array('name' => 'in_deletemsg', 'type' => CLX_CLASS_LIST, 'options' => $this->yesno)
                            )
                        )
                    )
                ), array('caption' => 'TOOL_MAAC_OUTSERVERINFO',
                    'type' => 'normal',
                    'entries' => array(
                        array('label' => 'GEN_LBL_OUTSERVER',
                            'fields' => array(
                                array('name' => 'out_hostname', 'type' => 'string', 'default' => '', 'size' => CLX_NUM_HUNDRED, 'displaysize' => '64', 'readonly' => false))
                        ), array('label' => 'GEN_LBL_OUTPORT',
                            'fields' => array(
                                array('name' => 'out_port', 'type' => 'string', 'default' => '', 'size' => '6', 'displaysize' => '10', 'readonly' => false))
                        ), array('label' => 'GEN_LBL_OUTSERVERAUTH',
                            'fields' => array(
                                array('name' => 'out_authenticate', 'type' => CLX_CLASS_LIST, 'default' => '1', 'options' => array(
                                        '0' => 'GEN_LBL_NO',
                                        '1' => 'GEN_LBL_YES'
                                    )
                                )
                            )
                        ), array('label' => 'GEN_LBL_OUTSERVERUSER',
                            'fields' => array(
                                array('name' => 'out_username', 'type' => 'string', 'default' => '', 'size' => CLX_NUM_HUNDRED, 'displaysize' => '64', 'readonly' => false))
                        )
                    )
                )
            )
        );
    }

    function toolTableStructure() {
        $fields = array();
        $fields['ID'] = array('type' => 'int(11) unsigned', 'null' => 'NO', 'properties' => 'auto_increment', 'index' => 'PRI');
        $fields['IDNAME'] = array('type' => 'varchar(100)', 'null' => 'NO', 'properties' => '');
        $fields['EMAIL'] = array('type' => 'varchar(100)', 'null' => 'NO', 'properties' => '');
        $fields['FULLNAME'] = array('type' => 'varchar(100)', 'null' => 'NO', 'properties' => '');
        $fields['EMAIL_TYPE'] = array('type' => 'varchar(4)', 'null' => 'YES', 'properties' => '');
        $fields['IN_HOSTNAME'] = array('type' => 'varchar(255)', 'null' => 'YES', 'properties' => '');
        $fields['IN_PORT'] = array('type' => 'int(6)', 'null' => 'YES', 'properties' => '');
        $fields['IN_USERNAME'] = array('type' => 'varchar(255)', 'null' => 'YES', 'properties' => '');
        $fields['IN_PASSWORD'] = array('type' => 'varchar(255)', 'null' => 'YES', 'properties' => '');
        $fields['IN_DELETEMSG'] = array('type' => 'tinyint(1) unsigned', 'null' => 'YES', 'properties' => '');
        $fields['OUT_HOSTNAME'] = array('type' => 'varchar(255)', 'null' => 'YES', 'properties' => '');
        $fields['OUT_PORT'] = array('type' => 'int(6)', 'null' => 'YES', 'properties' => '');
        $fields['OUT_USERNAME'] = array('type' => 'varchar(255)', 'null' => 'YES', 'properties' => '');
        $fields['OUT_PASSWORD'] = array('type' => 'varchar(255)', 'null' => 'YES', 'properties' => '');
        $fields['OUT_AUTHENTICATE'] = array('type' => 'tinyint(1) unsigned', 'null' => 'YES', 'properties' => '');
        $fields['USERNAME'] = array('type' => 'varchar(255)', 'null' => 'YES', 'properties' => '');
        $fields['SIGNATURE'] = array('type' => 'tinytext', 'null' => 'YES', 'properties' => '');
        $fields['SYSTEM_DEFAULT'] = array('type' => 'tinyint(1) unsigned', 'null' => 'YES', 'properties' => '', 'default' => CLX_NUM_ZERO);
        $indexes = array();
        $ipf = 'CLX' . $this->code;
        $indexes[$ipf . '01'] = array('index' => 'PRI', 'fieldname' => 'ID');
        $indexes[$ipf . '02'] = array('index' => 'UNI', 'fieldname' => 'EMAIL');
        $indexes[$ipf . '03'] = array('index' => 'MUL', 'fieldname' => 'IDNAME');
        return array('idcode' => $this->code, 'idname' => $this->name, 'name' => $this->tablename, 'type' => 'InnoDB',
            'fields' => $fields,
            'indexes' => $indexes);
    }

//-- M A I L
    /**
     * Action handler
     *
     * @param string $ac
     * @param array $idlist
     * @return string $contents
     */
    function action($ac, $idlist) {
        $contents = '';
        if ($this->getAccess()->canRead()) {
            switch ($ac) {
                case 'LIST_ADM':
                    $this->setAc($ac);
                    $contents = parent::action('LIST', $idlist);
                    break;
                default:
                    $contents = parent::action($ac, $idlist);
                    break;
            }
        } else {
            $contents = warning(dict('GEN_ERR_NO_ACCESS') . ' [MA_ACCOUNTS-CAN_READ][' . $ac . '] ' . $this->getCode(false));
        }
        return $contents;
    }

    function makelist($listingcode = null) {
	ClxFreeReturn($listingcode);
         //$t = time('U'); genera un warning
		$t = time();
        $to = $this->code;
        $here = 'tool.php?to=' . $to . '&t=' . $t;
        //clearHistory();
        pushHistory($this->code, UserRequest::$request['ac'], array());
        $form = new htmlForm(CLX_PROP_ACTION, 'tool.php');
        $form->setProp(CLX_PROP_ID, 'MAACList');
        $form->addArgument(CLX_PROP_TO, $this->code);
        $form->addArgument('ac', 'NOT_DEFINED');
        $selectclause = '*';
        $fromclause = $this->tablename;
        if (UserRequest::$request['ac'] <> 'LIST_ADM') {
            $whereclause = ' UPPER(USERNAME) = ' . singleQuote($_SESSION['user_record']['username']) . ' ';
        } else {
            $whereclause = '';
        }
        $sortclause = '';
        $page = new xPaging();
        $page->pageInit($form->getProp(CLX_PROP_ID), $here, $selectclause, $fromclause, $whereclause, $sortclause, '');

        $table = new htmlTable(CLX_WIDTH_FOUR);
        $table->setProps(array(CLX_PROP_CLASS => CLX_CLASS_LIST, CLX_PROP_WIDTH => CLX_PROP_100PCT));
        $table->addCell(dict('TOOL_MAAC_TITLE_LIST'), array(CLX_PROP_CLASS => CLX_CLASS_TITLE, CLX_PROP_COLSPAN => CLX_WIDTH_FOUR));
        $page->header($table);
        $table->addCells(CLX_CLASS_HEADERS, array(dict('GEN_LBL_SEL'), dict('GEN_LBL_MAILACCOUNT'), dict('GEN_LBL_FULLNAME'), dict('GEN_LBL_EMAIL')));
        $i = CLX_INIT_ZERO;
        while ($record = $page->nextRecord()) {
            $recordid = $record['ID'];
            $select = new htmlCheckbox('selector_' . $recordid, false);
            $select->setProp('onclick', 'selectButtons(' . singleQuote($form->getProp(CLX_PROP_ID)) . ');');
            $table->addCell($select->source(), array(CLX_PROP_CLASS => CLX_CLASS_FOOTERS, CLX_PROP_WIDTH => '40', CLX_PROP_ALIGN => 'center'));
            $table->addCells(($i % CLX_NUM_TWO == CLX_NUM_ZERO) ? CLX_CLASS_DATA : CLX_CLASS_DATAALT, array($record['IDNAME'], $record['FULLNAME'], $record['EMAIL'])); //Add attachment note
            $i++;
        }
        $buttons = '';
        $buttons .= htmlButton::getButton('submit', dict('GEN_LBL_DISPLAY'), array('action' => 'DISPLAY', 'form' => $form->getProp(CLX_PROP_ID), CLX_PROP_ID => 'MULT'))->source();
        if ($this->getAccess()->canModify()) {
            $buttons .=htmlButton::getButton('submit', dict('GEN_LBL_MODIFY'), array('action' => 'MODIFY', 'form' => $form->getProp(CLX_PROP_ID), CLX_PROP_ID => 'MULT'))->source();
        }
        if ($this->getAccess()->canDelete()) {
            $buttons .=htmlButton::getButton('submit', dict('GEN_LBL_DELETE'), array('action' => 'DELETE', 'form' => $form->getProp(CLX_PROP_ID), CLX_PROP_ID => 'MULT'))->source();
        }
        if ($this->getAccess()->canDisplay()) {
            $buttons .=htmlButton::getButton('submit', dict('GEN_LBL_CREATE_SOMETHING', dict('GEN_LBL_MAAC_CREATEMSG')), array('action' => 'CREATE', 'form' => $form->getProp(CLX_PROP_ID), CLX_PROP_ID => 'ON'))->source();
        }
        $buttons .=htmlButton::getButton('submit', dict('GEN_LBL_FILTER'), array('action' => 'FILTER', 'form' => $form->getProp(CLX_PROP_ID), CLX_PROP_ID => 'ON'))->source();
        $form->addContents($page->footers($table, $buttons));
        return $form->source();
    }

    /**
     * Edit form
     * @param type $ac
     * @param type $idlist
     * @param type $record
     * @param type $msg
     * @return type
     */
    function editForm($ac, $idlist, $record, $msg) {
        $calling = peekHistory();
        if ($calling['ac'] === 'LIST_ADM') {
            //Permitir que administrador vea el usuario
            $this->editform['panels'][CLX_NUM_ZERO]['entries'][CLX_NUM_FIVE]['fields'][CLX_NUM_ZERO]['readonly'] = false;
            $this->editform['panels'][CLX_NUM_ZERO]['entries'][CLX_NUM_SIX]['fields'][CLX_NUM_ZERO]['readonly'] = false;
        }
        $contents = parent::editForm($ac, $idlist, $record, $msg);
        $this->editform['panels'][CLX_NUM_ZERO]['entries'][CLX_NUM_FIVE]['fields'][CLX_NUM_ZERO]['readonly'] = true;
        $this->editform['panels'][CLX_NUM_ZERO]['entries'][CLX_NUM_SIX]['fields'][CLX_NUM_ZERO]['readonly'] = true;
        return $contents;
    }

    function searchForm() {
        $msg = array();
        $contents = '';
        // Search Form
        $form = new htmlForm(CLX_PROP_ACTION, UserRequest::$server['PHP_SELF']);
        $form->addArgument(CLX_PROP_TO, $this->code);
        $form->addArgument('ac', 'SEARCH');
        $form->addArgument('confirm', '1');
        $form->addArgument(CLX_PROP_TARGET, UserRequest::$request[CLX_PROP_TARGET]);
        $table = new htmlTable(CLX_WIDTH_TWO);
        $table->setProps(array(CLX_PROP_CLASS => CLX_CLASS_FORM, CLX_PROP_WIDTH => CLX_PROP_100PCT));
        $table->addCell(dict('TOOL_MAAC_TITLE_SEARCH'), array(CLX_PROP_CLASS => CLX_CLASS_TITLE, CLX_PROP_COLSPAN => CLX_WIDTH_TWO));
        $table->addField($msg, dict('GEN_LBL_IDNAME'), 'MAAC_idname', UserRequest::$post['MAAC_idname'], CLX_NUM_HUNDRED, CLX_NUM_64);
        $table->addField($msg, dict('GEN_LBL_EMAIL'), 'MAAC_email', UserRequest::$post['MAAC_email'], CLX_NUM_32, CLX_NUM_64);
        $buttons .=htmlButton::getButton('submit', dict('GEN_LBL_SEARCH'), array('action' => 'SEARCH', 'form' => $form->getProp(CLX_PROP_ID)))->source();
        $c = new htmlButtonCloseWindow(dict('GEN_LBL_CANCEL'));
        $buttons .= $c->source();
        $table->addCell($buttons, array(CLX_PROP_CLASS => CLX_CLASS_FOOTERS, CLX_PROP_COLSPAN => CLX_WIDTH_TWO));
        $form->setContents($table->source());
        $contents .= $form->source();
        // Search Procedure
        if (isset(UserRequest::$post['confirm'])) {
            $table = new htmlTable(CLX_WIDTH_TWO);
            $table->setProps(array(CLX_PROP_CLASS => CLX_CLASS_LIST, CLX_PROP_WIDTH => CLX_PROP_100PCT));
            $criteria = '';
            if (UserRequest::$post['MAAC_idname'] <> '') {
                $criteria .= 'UPPER(IDNAME) LIKE ' . singleQuote(uppercase(clx_change_especial_char_html (UserRequest::$post['MAAC_idname'])) . '%');
            }
            if (UserRequest::$post['MAAC_email'] <> '') {
                $criteria .= 'EMAIL LIKE ' . singleQuote(UserRequest::$post['MAAC_email'] . '%');
            }
            $qty = ClxCommon::$db->oneExistsRecords($this->tablename, $criteria);
            if ($qty == CLX_NUM_ZERO) {
                $table->addCell(dict('TOOL_GEN_ERR_NOMATCH'), array(CLX_PROP_CLASS => CLX_CLASS_ALERT, CLX_PROP_COLSPAN => CLX_WIDTH_TWO));
            } else {
                $table->addCell(dict('TOOL_GEN_MSG_FOUNDRECORDS'), array(CLX_PROP_CLASS => CLX_CLASS_SUBTITLE, CLX_PROP_COLSPAN => CLX_WIDTH_TWO));
                $records = ClxCommon::$db->getRecords($this->tablename, $criteria);
                $i = CLX_INIT_ZERO;
                while ($record = ClxCommon::$db->getNextRecord($records, ASSOCIATIVE)) {
                    $js = 'javascript:setLookupValue(' . quote(UserRequest::$request[CLX_PROP_TARGET]) . ', ' . quote($record['SUBJECT']) . ');';
                    $table->addCells(($i % CLX_NUM_TWO == CLX_NUM_ZERO) ? CLX_CLASS_DATA : CLX_CLASS_DATAALT, array('<page t=ma_mail_accounts></page><a href=' . singleQuote($js) . '>' . $record['SUBJECT'] . '</a>', $record['FROM_EMAIL']));
                    $i++;
                }
            }
            $contents .= $table->source();
        }
        return $contents;
    }

    function confirmDeleteFill(&$table, $record) {
        $table->addData(dict('GEN_LBL_IDNAME') . ':', $record['MAAC_idname']);
        $table->addData(dict('GEN_LBL_EMAIL') . ':', $record['MAAC_email']);
    }

    function filterFormFill($msg, &$table) {
        if (!array_key_exists('MAAC_idname', UserRequest::$post))
            UserRequest::$post['MAAC_idname'] = '';
        if (!array_key_exists('MAAC_email', UserRequest::$post))
            UserRequest::$post['MAAC_email'] = '';
        $table->addField($msg, dict('GEN_LBL_IDNAME'), 'MAAC_idname', UserRequest::$post['MAAC_idname'], CLX_NUM_40, CLX_NUM_64);
        $table->addField($msg, dict('GEN_LBL_EMAIL'), 'MAAC_email', UserRequest::$post['MAAC_email'], CLX_NUM_40, CLX_NUM_64);
    }

    function setfilter() {
        $filter = '';
        $boolean = '';
        if (!empty(UserRequest::$post['MAAC_idname'])) {
            $filter .= $boolean . ' UPPER(IDNAME) LIKE ' . singleQuote(uppercase(clx_change_especial_char_html (UserRequest::$post['MAAC_idname'])). '%');
            $boolean = 'AND';
        }
        if (!empty(UserRequest::$post['MAAC_email'])) {
            $filter .= $boolean . ' UPPER(EMAIL) LIKE ' . singleQuote(uppercase(UserRequest::$post['MAAC_email']) . '%');
            $boolean = 'AND';
        }
        $_SESSION['paging']['MAACList']['sortby'] = '';
        $_SESSION['paging']['MAACList']['criteria'] = $filter;
        $_SESSION['paging']['MAACList']['startat'] = '0';
    }
	///////////////////////////////////////////////////
	///comentado 22/01/2019 para utilizar la clx_tool
	//////////////////////////////////////////////////
    /*function clearfilter() {
        $_SESSION['paging']['MAACList']['criteria'] = '';
    }*/

    function loadToolRecord($recordid) {
        $values = ClxCommon::$db->getRecord($this->tablename, 'ID=' . $recordid);
        $record = array();
        $record['MAAC_id'] = $values['ID'];
        $record['MAAC_idname'] = $values['IDNAME'];
        $record['MAAC_fullname'] = $values['FULLNAME'];
        $record['MAAC_email'] = $values['EMAIL'];
        $record['MAAC_email_type'] = $values['EMAIL_TYPE'];
        $record['MAAC_in_hostname'] = $values['IN_HOSTNAME'];
        $record['MAAC_in_port'] = $values['IN_PORT'];
        $record['MAAC_in_username'] = $values['IN_USERNAME'];
        $record['MAAC_in_password'] = $values['IN_PASSWORD'];
        $record['MAAC_in_deletemsg'] = $values['IN_DELETEMSG'];
        $record['MAAC_out_hostname'] = $values['OUT_HOSTNAME'];
        $record['MAAC_out_port'] = $values['OUT_PORT'];
        $record['MAAC_out_username'] = $values['OUT_USERNAME'];
        $record['MAAC_out_password'] = $values['OUT_PASSWORD'];
        $record['MAAC_out_authenticate'] = $values['OUT_AUTHENTICATE'];
        $record['MAAC_username'] = $values['USERNAME'];
        $record['MAAC_signature'] = $values['SIGNATURE'];
        $record['MAAC_system_default'] = $values['SYSTEM_DEFAULT'];
        return $record;
    }

    function saveRecord() {
        $record = array();
        $record['IDNAME'] = UserRequest::$post['MAAC_idname'];
        $record['FULLNAME'] = UserRequest::$post['MAAC_fullname'];
        $record['EMAIL'] = UserRequest::$post['MAAC_email'];
        $record['EMAIL_TYPE'] = UserRequest::$post['MAAC_email_type'];
        $record['IN_HOSTNAME'] = UserRequest::$post['MAAC_in_hostname'];
        $record['IN_PORT'] = UserRequest::$post['MAAC_in_port'];
        $record['IN_USERNAME'] = UserRequest::$post['MAAC_in_username'];
        $record['IN_PASSWORD'] = UserRequest::$post['MAAC_in_password'];
        $record['IN_DELETEMSG'] = UserRequest::$post['MAAC_in_deletemsg'];
        $record['OUT_HOSTNAME'] = UserRequest::$post['MAAC_out_hostname'];
        $record['OUT_PORT'] = UserRequest::$post['MAAC_out_port'];
        $record['OUT_USERNAME'] = UserRequest::$post['MAAC_out_username'];
        $record['OUT_PASSWORD'] = UserRequest::$post['MAAC_out_password'];
        $record['OUT_AUTHENTICATE'] = UserRequest::$post['MAAC_out_authenticate'];
        $record['USERNAME'] = UserRequest::$post['MAAC_username'];
        $record['SIGNATURE'] = UserRequest::$post['MAAC_signature'];
        $record['SYSTEM_DEFAULT'] = (empty(UserRequest::$post['MAAC_system_default'])) ? CLX_NUM_ZERO : CLX_NUM_ONE;
        ClxCommon::$db->updateRecord($this->tablename, $record, 'ID=' . UserRequest::$post['MAAC_id']);
    }

    function createRecord_() {
        $record = array();
        $record['IDNAME'] = UserRequest::$post['MAAC_idname'];
        $record['FULLNAME'] = UserRequest::$post['MAAC_fullname'];
        $record['EMAIL'] = UserRequest::$post['MAAC_email'];
        $record['EMAIL_TYPE'] = UserRequest::$post['MAAC_email_type'];
        $record['IN_HOSTNAME'] = UserRequest::$post['MAAC_in_hostname'];
        $record['IN_PORT'] = UserRequest::$post['MAAC_in_port'];
        $record['IN_USERNAME'] = UserRequest::$post['MAAC_in_username'];
        $record['IN_PASSWORD'] = UserRequest::$post['MAAC_in_password'];
        $record['IN_DELETEMSG'] = UserRequest::$post['MAAC_in_deletemsg'];
        $record['OUT_HOSTNAME'] = UserRequest::$post['MAAC_out_hostname'];
        $record['OUT_PORT'] = UserRequest::$post['MAAC_out_port'];
        $record['OUT_USERNAME'] = UserRequest::$post['MAAC_out_username'];
        $record['OUT_PASSWORD'] = UserRequest::$post['MAAC_out_password'];
        $record['OUT_AUTHENTICATE'] = UserRequest::$post['MAAC_out_authenticate'];
        $record['SIGNATURE'] = UserRequest::$post['MAAC_signature'];
        $record['USERNAME'] = $_SESSION['user_record']['username'];
        $record['SYSTEM_DEFAULT'] = (empty(UserRequest::$post['MAAC_system_default'])) ? CLX_NUM_ZERO : CLX_NUM_ONE;
        $recordid = ClxCommon::$db->createRecord($this->tablename, $record, false);
        return $recordid;
    }

    function deleteRecordSk($id) {
        ClxCommon::$db->deleteRecords($this->tablename, 'ID=' . $id);
    }

    function defaultRecord() {
        $record = array();
        $record['MAAC_id'] = CLX_INIT_ZERO;
        $record['MAAC_fullname'] = '';
        $record['MAAC_idname'] = '';
        $record['MAAC_email'] = '';
        $record['MAAC_email_type'] = 'POP3';
        $record['MAAC_in_hostname'] = '';
        $record['MAAC_in_port'] = '110';
        $record['MAAC_in_username'] = '';
        $record['MAAC_in_password'] = '';
        $record['MAAC_in_deletemsg'] = CLX_INIT_ZERO;
        $record['MAAC_out_hostname'] = '';
        $record['MAAC_out_port'] = '25';
        $record['MAAC_out_username'] = '';
        $record['MAAC_out_password'] = '';
        $record['MAAC_out_authenticate'] = CLX_INIT_ZERO;
        $record['MAAC_username'] = '';
        $record['MAAC_signature'] = '';
        $record['MAAC_username'] = $_SESSION['user_record']['username'];
        $record['MAAC_system_default'] = CLX_INIT_ZERO;
        return $record;
    }

    function versionIncrease($fieldname, $fieldvalue) {
	ClxFreeReturn($fieldname);
	ClxFreeReturn($fieldvalue);

    }

}

// End class
