<?php

/**
 * Tool Class
 * For more information contact  support@alenet.com
 *
 * @author       $Author: alenetdev $
 * @copyright    Alenet, Inc.
 * @since        $Date: 2019-03-15 18:54:41 +0000 (Fri, 15 Mar 2019) $
 * @version        $Rev: 10572 $
 *
 * CHANGE LOG
 *  Oracle uppercase record changed
 *  Included label
 */
class objects extends clx_tool {

    var $loadedObjects;

    function __construct() {
        $this->code = 'OBJS';
        $this->name = 'Object Manager';
        $this->system = 'System';
        $this->version = '7.0.0.0';

        $this->has_access = true;


        $this->tablename = 'CLX_OBJECTS';
        $this->record_name_singular = 'object';
        $this->record_name_plural = 'objects';
        $this->primary_field = 'IDCODE';
        $this->secondary_field = 'IDNAME';

        $this->loadedObjects = array();
		IF (ISSET($_SESSION['ADDRDEBUGEDIT']) && $_SESSION['ADDRDEBUGEDIT']==$_SERVER['REMOTE_ADDR']) {
			$this->debug=true;
		}
    }

    function install() {
        ClxFreeReturn($this->scanObjects(''));
        $UIMI = getTool('UIMI');
        $UIMI->add($this->code, 'MENU_TOOLS:MENU_SYSTEM:MENU_SYSOBJECTS', 'TOOL', $this->code, 'ac=LIST');
    }

    function toolTableStructure() {
        $fields = array();
        $fields['ID'] = array('type' => 'int(11) unsigned', 'null' => 'NO', 'properties' => 'auto_increment', 'index' => 'PRI');
        $fields['IDCODE'] = array('type' => 'varchar(4)', 'null' => 'NO', 'properties' => '');
        $fields['IDNAME'] = array('type' => 'varchar(64)', 'null' => 'NO', 'properties' => '');
        $fields['TYPE'] = array('type' => 'varchar(4)', 'null' => 'NO', 'properties' => '');
        $fields['VERSION'] = array('type' => 'varchar(32)', 'null' => 'NO', 'properties' => '');
        $fields['IDATE'] = array('type' => 'datetime', 'null' => 'YES', 'properties' => '');
        $fields['DESCRIPTION'] = array('type' => 'varchar(255)', 'null' => 'NO', 'properties' => '');
        $fields['CLASS'] = array('type' => 'varchar(255)', 'null' => 'NO', 'properties' => '');
        $indexes = array();
        $ipf = 'CLX' . $this->code;
        $indexes[$ipf . '01'] = array('index' => 'PRI', 'fieldname' => 'ID');
        $indexes[$ipf . '02'] = array('index' => 'UNI', 'fieldname' => 'IDCODE');
        $indexes[$ipf . '03'] = array('index' => 'MUL', 'fieldname' => 'IDNAME');
        $indexes[$ipf . '04'] = array('index' => 'MUL', 'fieldname' => 'TYPE');
        $indexes[$ipf . '05'] = array('index' => 'MUL', 'fieldname' => 'CLASS');
        return array('idcode' => $this->code, 'idname' => $this->name, 'name' => $this->tablename, 'type' => 'MyISAM',
            'fields' => $fields,
            'indexes' => $indexes);
        ;
    }

    function action($ac, $idlist) {
        $contents = '';
        if ($this->getAccess()->canRead()) {
            switch ($ac) {
                case 'SCAN':
                    if ($this->getAccess()->canAppend()) {
                        $list = $this->scanObjects('');
                        if (count($list) > 0) {
                            addMessage(dict('TOOL_MAGR_NEWTOOLS') . ':<br>' . implode('<br>', $list));
                        }
                        $contents = $this->makelist();
                    } else {
                        $contents = warning(dict('GEN_ERR_NO_ACCESS') . ' [OBJECTS-CAN_APPEND][' . $ac . '] ' . $this->getCode(false));
                    }
                    break;
                default:
                    $contents = parent::action($ac, $idlist);
            }
        } else {
            $contents = warning(dict('GEN_ERR_NO_ACCESS') . ' [OBJECTS-CAN_READ][' . $ac . '] ' . $this->getCode(false));
        }
        return $contents;
    }

    function makelist($listingcode = null) {
        $t = time();
        $to = $this->code;
        $here = 'tool.php?to=' . $to . '&t=' . $t;
        clearHistory();
        pushHistory($this->code, 'LIST', array());
        $form = new htmlForm(CLX_PROP_ACTION, 'tool.php');
        $form->setProp(CLX_PROP_ID, 'OBJSList');
        $form->addArgument(CLX_PROP_TO, $this->code);
        $form->addArgument('ac', 'NOT_DEFINED');
        $selectclause = '*';
        $fromclause = 'CLX_OBJECTS';
        $whereclause = '';
        $sortclause = '';
        $page = new xPaging();
        $page->pageInit($form->getProp(CLX_PROP_ID), $here, $selectclause, $fromclause, $whereclause, $sortclause, '');
        $table = new htmlTable(CLX_WIDTH_7);
        $table->setProps(array(CLX_PROP_CLASS => CLX_CLASS_LIST, CLX_PROP_WIDTH => CLX_PROP_100PCT));
        $table->addCell(dict('TOOL_OBJS_TITLE_LIST'), array(CLX_PROP_CLASS => CLX_CLASS_TITLE, CLX_PROP_COLSPAN => CLX_WIDTH_7));
        $page->header($table);
        $table->addCells(CLX_CLASS_HEADERS, array(dict('GEN_LBL_SEL'), dict('GEN_LBL_CODE'), dict('GEN_LBL_NAME'), dict('GEN_LBL_TYPE'), dict('GEN_LBL_VERSION'), dict('GEN_LBL_IDATE'), dict('GEN_LBL_DESCRIPTION')));
        $i = CLX_INIT_ZERO;
        while ($record = $page->nextRecord()) {
            $recordid = $record['ID'];
            $select = new htmlCheckbox('selector_' . $recordid, false);
            $select->setProp('onclick', 'selectButtons(' . singleQuote($form->getProp(CLX_PROP_ID)) . ');');
            $table->addCell($select->source(), array(CLX_PROP_CLASS => CLX_CLASS_FOOTERS, CLX_PROP_WIDTH => '40', CLX_PROP_ALIGN => 'center'));
            $table->addCells(($i % CLX_NUM_TWO == CLX_NUM_ZERO) ? CLX_CLASS_DATA : CLX_CLASS_DATAALT, array($record['IDCODE'], $record['IDNAME'], $record['TYPE'], $record['VERSION'], xFormat::date($record['IDATE'],$_SESSION['prefs']['DATE_FORMAT']), xFormat::text($record['DESCRIPTION'])));
            $i++;
        }
        $buttons = '';
        $buttons .= htmlButton::getButton('submit', dict('GEN_LBL_DISPLAY'), array('action' => 'DISPLAY', 'form' => $form->getProp(CLX_PROP_ID), CLX_PROP_ID => 'MULT'))->source();
        if ($this->getAccess()->canDelete()) {
            $buttons .=htmlButton::getButton('submit', dict('GEN_LBL_DELETE'), array('action' => 'DELETE', 'form' => $form->getProp(CLX_PROP_ID), CLX_PROP_ID => 'MULT'))->source();
        }
        if ($this->getAccess()->canDisplay()) {
            $b = new htmlFormSubmit(dict('GEN_LBL_SCAN', dict('GEN_LBL_OBJECTS')), 'SCAN', $form->getProp(CLX_PROP_ID));
            $b->setProp(CLX_PROP_ID, 'ON');
            $buttons .= $b->source();
        }
        $buttons .=htmlButton::getButton('submit', dict('GEN_LBL_FILTER'), array('action' => 'FILTER', 'form' => $form->getProp(CLX_PROP_ID), CLX_PROP_ID => 'ON'))->source();
        $form->addContents($page->footers($table, $buttons));
        $sqlstatement = $page->getProp('statement');
        $records = ClxCommon::$db->executeQuery($sqlstatement);
        $totals = array(CLX_NUM_ZERO, CLX_NUM_ZERO, CLX_NUM_ZERO, CLX_NUM_ZERO);
        while (($record = ClxCommon::$db->getNextRecord($records, ASSOCIATIVE))) {
            $pieces = explode('.', $record['VERSION']);
            for ($i = CLX_INIT_ZERO; $i < CLX_NUM_FOUR; $i++) {
                $totals[$i]+= $pieces[$i];
            }
        }
        $table = new htmlTable(CLX_WIDTH_TWO);
        $table->setProps(array(CLX_PROP_CLASS => CLX_CLASS_LIST, CLX_PROP_WIDTH => CLX_PROP_100PCT));
        $table->addCell(dict('TOOL_OBJS_STATISTICS'), array(CLX_PROP_CLASS => CLX_CLASS_TITLE, CLX_PROP_COLSPAN => CLX_WIDTH_TWO));
        $table->addData(dict('TOOL_OBJS_VERSION_TOTAL'), $totals[CLX_NUM_ZERO] . '.' . $totals[CLX_NUM_ONE] . '.' . $totals[CLX_NUM_TWO] . '.' . $totals[CLX_NUM_THREE]);
        return $form->source() . $table->source();
    }

    function viewFormFill($record) {
        $contents = '';
        // Headers
        $table = new htmlTable(CLX_WIDTH_TWO);
        $table->setProps(array(CLX_PROP_CLASS => CLX_CLASS_FORM, CLX_PROP_WIDTH => CLX_PROP_100PCT));
        $table->addCell(dict('GEN_LBL_OBJECT') . ': ' . $record['OBJS_idname'], array(CLX_PROP_CLASS => CLX_CLASS_TITLE, CLX_PROP_COLSPAN => CLX_WIDTH_TWO));
        $table->addData(dict('GEN_LBL_CODE') . ':', $record['OBJS_idcode']);
        $table->addData(dict('GEN_LBL_NAME') . ':', $record['OBJS_idname']);
        $table->addData(dict('GEN_LBL_TYPE') . ':', $record['OBJS_type']);
        $table->addData(dict('GEN_LBL_VERSION') . ':', $record['OBJS_version']);
        $table->addData(dict('GEN_LBL_CLASS') . ':', $record['OBJS_class']);
        $table->addData(dict('GEN_LBL_IDATE') . ':', xFormat::date($record['OBJS_idate']));
        $table->addData(dict('GEN_LBL_DESCRIPTION') . ':', xFormat::text($record['OBJS_description']));
        $contents .= $table->source();
        return $contents;
    }

    function filterFormFill($msg, &$table) {
        if (!array_key_exists('OBJS_idcode', UserRequest::$post))
            UserRequest::$post['OBJS_idcode'] = '';
        if (!array_key_exists('OBJS_idname', UserRequest::$post))
            UserRequest::$post['OBJS_idname'] = '';
        if (!array_key_exists('OBJS_type', UserRequest::$post))
            UserRequest::$post['OBJS_type'] = '';
        $table->addField($msg, dict('GEN_LBL_CODE') . ':', 'OBJS_idcode', UserRequest::$post['OBJS_idcode'], CLX_NUM_FOUR, CLX_NUM_FOUR);
        $table->addField($msg, dict('GEN_LBL_NAME') . ':', 'OBJS_idname', UserRequest::$post['OBJS_idname'], CLX_NUM_32, CLX_NUM_32);
        $table->addField($msg, dict('GEN_LBL_TYPE') . ':', 'OBJS_type', UserRequest::$post['OBJS_type'], CLX_NUM_FOUR, CLX_NUM_FOUR);
        $sorts = new HTMLSelect('OBJS_sort');
        $sorts->addItem('', '', true);
        $sorts->addItem('Code', 'idcode', false);
        $sorts->addItem('Name', 'idname', false);
        $sorts->addItem('Type', 'type', false);
        $sorts->addItem('Instalation Date (Asc)', 'IDATE ASC', false);
        $sorts->addItem('Instalation Date (Desc)', 'IDATE DESC', false);
        $table->addData(dict('GEN_LBL_ORDERBY') . ':', $sorts->source());
    }

    function setfilter() {
        $filter = '';
        $boolean = '';
        if (!empty(UserRequest::$post['OBJS_idcode'])) {
            $filter .= $boolean . ' UPPER(IDCODE) LIKE ' . singleQuote(uppercase(UserRequest::$post['OBJS_idcode']) . '%');
            $boolean = 'AND';
        }
        if (!empty(UserRequest::$post['OBJS_idname'])) {
            $filter .= $boolean . ' UPPER(IDNAME) LIKE ' . singleQuote(uppercase(clx_change_especial_char_html (UserRequest::$post['OBJS_idname'])) . '%');
            $boolean = 'AND';
        }
        if (!empty(UserRequest::$post['OBJS_type'])) {
            $filter .= $boolean . ' TYPE LIKE ' . singleQuote(UserRequest::$post['OBJS_type'] . '%');
            $boolean = 'AND';
        }
        $_SESSION['paging']['OBJSList']['sortby'] = UserRequest::$post['OBJS_sort'];
        $_SESSION['paging']['OBJSList']['criteria'] = $filter;
        $_SESSION['paging']['OBJSList']['startat'] = '0';
    }
	///////////////////////////////////////////////////
	///comentado 22/01/2019 para utilizar la clx_tool
	//////////////////////////////////////////////////
    /*function clearfilter() {
        $_SESSION['paging']['OBJSList']['criteria'] = '';
    }*/

    function confirmDeleteFill(&$table, $record) {
        $table->addData(dict('GEN_LBL_CODE') . ':', $record['OBJS_idcode']);
        $table->addData(dict('GEN_LBL_NAME') . ':', $record['OBJS_idname']);
    }

    /**
     * Scan and install objects
     * @param type $sourcepath
     * @return array
     */
    private function scanObjects($sourcepath = '') {
        $newobjects = array();
        $pathname = ($sourcepath === '') ? (kPathToWebSite . kPathToObjects) : $sourcepath;

        if (is_readable($pathname)) {
            $dir = opendir($pathname);
            while ($file = readdir($dir)) {
                if (!is_dir($pathname . $file) && ($file !== '.') && ($file !== '..') && stristr($file, '.php')) {
                    $pos = strpos($file, '.');

                    $class = substr($file, CLX_NUM_ZERO, $pos);
                    require_once $pathname . $file;
                    if (!class_exists($class)) {
                        addMessage('Class not exists or fail include: ' . $class);
                        continue;
                    }

                    $object = new $class;
                    if (ClxCommon::$db->oneExistsRecords('CLX_OBJECTS', 'IDCODE=' . singleQuote($object->code)) === CLX_NUM_ZERO) {
                        $this->installObject($class, $object);
                        $newobjects[] = $object->code;
                    } else {
                        $this->updateObject($class, $object);
                    }
                }
            }
            closedir($dir);
        } else {
            addMessage('Can' . CLX_CHAR_SINGLE_QUOTE . 't access the tools directory: ' . $pathname);
        }
        return $newobjects;
    }

    /**
     * Scan Object for bath installer.
     */
    function scanObjectsinstaller($pathname) {
        return $this->scanObjects($pathname);
    }

    function installObject($class, $object) {
        $record = array();
        $record['IDCODE'] = $object->code;
        $record['IDNAME'] = $object->name;
        $record['TYPE'] = $object->type;
        $record['VERSION'] = $object->version;
        $record['IDATE'] = xFormat::getNow('Y-m-d H:i:s');
        $record['DESCRIPTION'] = $object->description;
        $record['CLASS'] = $class;
        ClxCommon::$db->createRecord('CLX_OBJECTS', $record, false);
    }

    function updateObject($class, $object) {
        $record = ClxCommon::$db->getRecord('CLX_OBJECTS', 'IDCODE=' . singleQuote($object->code));
        if ($record['VERSION'] != $object->version) {
            $record = array();
            $record['IDCODE'] = $object->code;
            $record['IDNAME'] = $object->name;
            $record['TYPE'] = $object->type;
            $record['VERSION'] = $object->version;
            $record['IDATE'] = xFormat::getNow('Y-m-d H:i:s');
            $record['DESCRIPTION'] = $object->description;
            $record['CLASS'] = $class;
            ClxCommon::$db->updateRecord('CLX_OBJECTS', $record, 'IDCODE=' . singleQuote($object->code));
        }
    }

    function loadToolRecord($recordid) {
        
		 
		$fetched = false;
		if (kApcTtl_26>0) $record = apc_fetchFct($findVar=kSpaceName . '_CLX_OBJECTS_' . $recordid, $fetched);
		if (empty($fetched)) {
			$values = ClxCommon::$db->getRecord('CLX_OBJECTS', 'ID= ' . $recordid);
			$record = array();
			$record['OBJS_id'] = $recordid;
			$record['OBJS_idcode'] = $values['IDCODE'];
			$record['OBJS_idname'] = $values['IDNAME'];
			$record['OBJS_type'] = $values['TYPE'];
			$record['OBJS_version'] = $values['VERSION'];
			$record['OBJS_idate'] = $values['IDATE'];
			$record['OBJS_description'] = $values['DESCRIPTION'];
			$record['OBJS_class'] = $values['CLASS'];
			if (kApcTtl_26>0) {
				apc_addFct($findVar, $record,kApcTtl_26);
			}
		}
        
		
       
        
        return $record;
    }

    function deleteRecordSk($id) {
        ClxCommon::$db->deleteRecords('CLX_OBJECTS', 'ID=' . $id);
    }

    function getObject_($type, $code) {if ($this->debug==true )  {$posx=0;$pos=1639;$tta=fopen(kPathToWebSite."/".kPathToTemp."archivo.txt","a+");$te=debug_backtrace(); for ($t=0;$t<12;$t++) {if (isset($te[$t]["file"])){if ($t==$posx)fwrite($tta,chr(13).chr(10).$pos."_".$te[$posx]["function"]."_17201 db_export-hora:".date("h:i:s d/m/Y").chr(13).chr(10));fwrite($tta,substr($te[$t]["file"].str_repeat(" ",80),0,80).chr(9).$te[$t]["line"].chr(9).$te[$t]["function"].chr(13).chr(10));}}fclose($tta);}
        $return = null;
        $info = $this->getObjectRecord($type, $code);
        if (empty($info)) {
	    $BUGS = &getTool('BUGS');
            $BUGS->report(E_USER_ERROR, 'No object of type ' . singleQuote($type) . ' with code ' . singleQuote($code), 'tools/objects.php', '315', UserRequest::$request, 0, '');
        } else {
            $pos = strpos($info['CLASS'], '_');
		    if (!empty($pos))  {if ( isset($_SESSION["_dEBUG"]) && $_SESSION["_dEBUG"]==99 ) {$posx=0;$pos=1639;$tta=fopen(kPathToWebSite."/".kPathToTemp."archivo.txt","a+");  fwrite($tta,chr(13).chr(10).kPathToWebSite . kPathToObjects . substr($info['CLASS'], CLX_NUM_ZERO, $pos) . '.php'.chr(13).chr(10)); $te=debug_backtrace(); for ($t=0;$t<12;$t++) {if (isset($te[$t]["file"])){if ($t==$posx)fwrite($tta,chr(13).chr(10).$pos."_".$te[$posx]["function"]."_17201 db_export-hora:".date("h:i:s d/m/Y").chr(13).chr(10));fwrite($tta,substr($te[$t]["file"].str_repeat(" ",80),0,80).chr(9).$te[$t]["line"].chr(9).$te[$t]["function"].chr(13).chr(10));}}fclose($tta);}
                include_once kPathToWebSite . kPathToObjects . substr($info['CLASS'], CLX_NUM_ZERO, $pos) . '.php';
            }
			if ( isset($_SESSION["_dEBUG"]) && $_SESSION["_dEBUG"]==99 ) {$posx=0;$pos=1639;$tta=fopen(kPathToWebSite."/".kPathToTemp."archivo.txt","a+");  fwrite($tta,chr(13).chr(10).kPathToWebSite . kPathToObjects . $info['CLASS'] . '.php'.chr(13).chr(10)); $te=debug_backtrace(); for ($t=0;$t<12;$t++) {if (isset($te[$t]["file"])){if ($t==$posx)fwrite($tta,chr(13).chr(10).$pos."_".$te[$posx]["function"]."_17201 db_export-hora:".date("h:i:s d/m/Y").chr(13).chr(10));fwrite($tta,substr($te[$t]["file"].str_repeat(" ",80),0,80).chr(9).$te[$t]["line"].chr(9).$te[$t]["function"].chr(13).chr(10));}}fclose($tta);}
            include_once kPathToWebSite . kPathToObjects . $info['CLASS'] . '.php';
            $return = new $info['CLASS'];
			}
        return $return;
    }

    function getObjectRecord($type, $code) {
        $fetched = false;
        if (kApcTtl_17>0) $info = apc_fetchFct($findVar=kSpaceName . 'objs.' . $type . '.' . $code, $fetched);
        if (empty($fetched)) {
            $info = ClxCommon::$db->getRecord('CLX_OBJECTS', 'TYPE=' . singleQuote($type) . ' AND IDCODE=' . singleQuote($code));
            if (kApcTtl_17>0&&!empty($info)) {
                apc_addFct($findVar, $info, kApcTtl_17);
            }
        }
        return $info;
    }

    /**
     * Returns list of objects asociate to type
     * @param string $type
     * @return array
     */
    function objectList($type) {
        $list = array();
        $sql = 'SELECT IDCODE FROM CLX_OBJECTS WHERE TYPE = ' . singleQuote($type);
        $rs = ClxCommon::$db->executeQuery($sql);
        while (($r = ClxCommon::$db->getNextRecord($rs, NUMERIC))) {
            $list[] = $r[CLX_NUM_ZERO];
        }
        return $list;
    }

    function objectPop($type, $name, $default, $event = false, $parent = '') {
        $records = ClxCommon::$db->getRecords('CLX_OBJECTS', 'TYPE=' . singleQuote($type) . ' ORDER BY IDNAME');
        $pop = new htmlSelect($name);
        while (($record = ClxCommon::$db->getNextRecord($records, ASSOCIATIVE))) {
            if ($event == false || ($record['IDCODE'] !== 'SQL' || $parent === 'PRSE')) {
                $pop->addItem($record['IDNAME'], $record['IDCODE'], ($record['IDCODE'] == $default));
            }
        }
        return $pop;
    }

}
