<?php

/**
 * Process Schedules Manager
 *
 * @package		Clientix.Tool.Process
 * @abstract		Process Engine Manager
 * @author       $Author: cramirez $
 * @copyright    Alenet, Inc.
 * @since        $Date: 2018-09-11 06:19:25 -0400 (Tue, 11 Sep 2018) $
 * @version    	$Rev: 10042 $
 *
 */
class pr_schedules extends clx_tool {

    /**
     * Recurrence Types
     * @var array
     */
    var $recu_types;

    /**
     * Constructor
     */
    function __construct() {
        $this->code = 'PRSC';
        $this->name = 'Process Schedules Manager';
        $this->system = 'Process';
        $this->version = '5.5.0.1';

        $this->has_access = true;
        $this->has_package = true;

        $this->tablename = 'CLX_PR_SCHEDULES';
        $this->record_name_singular = 'GEN_LBL_PR_SCHEDULE';
        $this->record_name_plural = 'GEN_LBL_PR_SCHEDULES';
        $this->primary_field = 'IDCODE';
        $this->secondary_field = 'IDNAME';
        $this->terciary_field = 'PROCESSCODE';

        $this->log_method = array('GEN_LBL_ONEPEREXE', 'GEN_LBL_ONEPERPROC');
        $this->recu_types = array(CLX_NUM_ZERO => 'GEN_LBL_NORECURRENCE', CLX_NUM_ONE => 'GEN_LBL_MINUTES', CLX_NUM_60 => 'GEN_LBL_HOURS', CLX_NUM_1440 => 'GEN_LBL_DAYS');
        $this->editform = array(CLX_CLASS_TITLE => 'TOOL_PRST_TITLE_CREATE',
            'panels' => array(
                array('type' => 'normal',
                    'entries' => array(
                        array('label' => 'GEN_LBL_CODE',
                            'fields' => array(
                                array('name' => 'idcode',
                                    'type' => 'string',
                                    'default' => '',
                                    'size' => '20',
                                    'displaysize' => '35',
                                    'dontmodify' => '1',
                                    'validations' => array(
                                        array('type' => 'sqlobject',
                                            'required' => '1'
                                        ), array('type' => 'unique',
                                            'fieldname' => 'idcode',
                                            'tablename' => $this->tablename)
                                    )
                                )
                            )
                        ), array('label' => 'GEN_LBL_SHORT_NAME',
                            'fields' => array(
                                array('name' => 'idname',
                                    'type' => 'string',
                                    'default' => '',
                                    'size' => '64',
                                    'displaysize' => '35',
                                    'validations' => array(
                                        array('type' => 'string',
                                            'required' => '1'
                                        )
                                    )
                                )
                            )
                        ), array('label' => 'GEN_LBL_PROCESSCODE',
                            'fields' => array(
                                array('name' => 'processcode',
                                    'type' => 'lookup',
                                    'default' => '',
                                    'readonly' => false,
                                    'tool' => 'PRPR',
                                    'validations' => array(
                                        array('type' => 'sqlobject',
                                            'required' => '1'
                                        ), array('type' => 'lookup',
                                            'fieldname' => 'idcode',
                                            'tablename' => 'CLX_PR_PROCESS'
                                        )
                                    )
                                )
                            )
                        ), array('label' => 'GEN_LBL_OWNERCODE',
                            'fields' => array(
                                array('name' => 'ownercode',
                                    'type' => 'lookup',
                                    'default' => '',
                                    'readonly' => false,
                                    'tool' => 'ACUS',
                                    'validations' => array(
                                        array('type' => 'lookup',
                                            'fieldname' => 'username',
                                            'tablename' => 'CLX_AC_USERS',
                                            'required' => '1'
                                        )
                                    )
                                )
                            )
                        ), array('label' => 'GEN_LBL_STATUS',
                            'fields' => array(
                                array('name' => 'status',
                                    'type' => CLX_CLASS_LIST,
                                    'default' => '1',
                                    'options' => $this->status)
                            )
                        ), array('label' => 'GEN_LBL_NOTES',
                            'fields' => array(
                                array('name' => 'notes',
                                    'type' => 'text',
                                    'default' => '',
                                    'cols' => '48',
                                    'rows' => '4'
                                )
                            )
                        )
                    )
                ), array('type' => 'normal',
                    CLX_CLASS_TITLE => 'GEN_LBL_SCH_OPTIONS',
                    'entries' => array(
                        array('label' => 'GEN_LBL_PR_VALUE',
                            'fields' => array(
                                array('name' => 'pr_value',
                                    'type' => 'string',
                                    'default' => '0',
                                    'size' => '10',
                                    'displaysize' => '35',
                                    'readonly' => false,
                                    'validations' => array(
                                        array('type' => 'string',
                                            'required' => '1'
                                        )
                                    )
                                )
                            )
                        ), array('label' => 'GEN_LBL_PR_RECURRENCE',
                            'fields' => array(
                                array('name' => 'pr_recurrence',
                                    'type' => CLX_CLASS_LIST,
                                    'default' => 'ONE',
                                    'options' => $this->recu_types, 'validations' => array(
                                        array('type' => 'string',
                                            'required' => '1'
                                        )
                                    )
                                )
                            )
                        ), array('label' => 'GEN_LBL_NEXT_RUN_TIME',
                            'fields' => array(
                                array('name' => 'next_run_time',
                                    'type' => 'dati',
                                    'displaysize' => '35',
                                    'validations' => array(
                                        array('type' => 'string',
                                            'required' => '0'
                                        )
                                    )
                                )
                            )
                        )
                    )
                ), array('type' => 'normal',
                    CLX_CLASS_TITLE => 'GEN_LBL_ADITIONAL_OPTIONS',
                    'entries' => array(
                        array('label' => 'GEN_LBL_IS_DEBUG',
                            'fields' => array(
                                array('name' => 'is_debug',
                                    'type' => CLX_CLASS_LIST,
                                    'default' => '1',
                                    'options' => $this->yesno)
                            )
                        ), array('label' => 'GEN_LBL_IS_DISPLAY',
                            'fields' => array(
                                array('name' => 'is_display',
                                    'type' => CLX_CLASS_LIST,
                                    'default' => '1',
                                    'options' => $this->yesno)
                            )
                        ), array('label' => 'GEN_LBL_LOG_METHOD',
                            'fields' => array(
                                array('name' => 'log_method',
                                    'type' => CLX_CLASS_LIST,
                                    'default' => '1',
                                    'options' => $this->log_method)
                            )
                        ), array('label' => 'GEN_LBL_PROC_STATUS',
                            'fields' => array(
                                array('name' => 'proc_status',
                                    'type' => CLX_CLASS_LIST,
                                    'default' => '',
                                    'options' => $this->notify_status
                                )
                            )
                        ), array('label' => 'GEN_LBL_BATCH_PID',
                            'fields' => array(
                                array('name' => 'batch_pid',
                                    'readonly' => true,
                                    'type' => 'string',
                                    'default' => '0',
                                    'size' => '10',
                                    'displaysize' => '35'
                                )
                            )
                        )
                    )
                ), array('type' => 'normal',
                    CLX_CLASS_TITLE => 'GEN_LBL_MSG_OPTIONS',
                    'entries' => array(
                        array('label' => 'GEN_LBL_NOTIFY_STATUS',
                            'fields' => array(
                                array('name' => 'notify_status',
                                    'type' => CLX_CLASS_LIST,
                                    'default' => '',
                                    'options' => $this->notify_status
                                )
                            )
                        ), array('label' => 'GEN_LBL_TOUSER',
                            'fields' => array(
                                array('name' => 'touser',
                                    'type' => 'lookup',
                                    'default' => '',
                                    'readonly' => false,
                                    'tool' => 'ACUS',
                                    'validations' => array(
                                        array('type' => 'lookup',
                                            'fieldname' => 'username',
                                            'tablename' => 'CLX_AC_USERS'
                                        )
                                    )
                                )
                            )
                        ), array('label' => 'GEN_LBL_CREATE_TASK',
                            'fields' => array(
                                array('name' => 'create_task',
                                    'type' => 'radiobutton',
                                    'checked' => false,
                                    CLX_CLASS_PROMPT => '',
                                    'value' => '0',
                                    'options' => $this->yesno, 'validations' => array(
                                        array('type' => 'number',
                                            'required' => '0'
                                        )
                                    )
                                )
                            )
                        ), array('label' => 'GEN_LBL_CREATE_ALERT',
                            'fields' => array(
                                array('name' => 'create_alert',
                                    'type' => 'radiobutton',
                                    'checked' => false,
                                    CLX_CLASS_PROMPT => '',
                                    'value' => '0',
                                    'options' => $this->yesno, 'validations' => array(
                                        array('type' => 'number',
                                            'required' => '0'
                                        )
                                    )
                                )
                            )
                        ), array('label' => 'GEN_LBL_SEND_EMAIL',
                            'fields' => array(
                                array('name' => 'send_email',
                                    'type' => 'radiobutton',
                                    'checked' => false,
                                    CLX_CLASS_PROMPT => '',
                                    'value' => '0',
                                    'options' => $this->yesno, 'validations' => array(
                                        array('type' => 'number',
                                            'required' => '0'
                                        )
                                    )
                                )
                            )
                        ), array('label' => 'GEN_LBL_SUBJECT',
                            'fields' => array(
                                array('name' => 'subject',
                                    'type' => 'string',
                                    'default' => '',
                                    'size' => '255',
                                    'displaysize' => '35',
                                    'validations' => array(
                                        array('type' => 'string',
                                            'required' => '0'
                                        )
                                    )
                                )
                            )
                        ), array('label' => 'GEN_LBL_MESSAGE',
                            'fields' => array(
                                array('name' => 'message',
                                    'type' => 'text',
                                    'default' => '',
                                    'cols' => '48',
                                    'rows' => '4'
                                )
                            )
                        )
                    )
                ), array('type' => 'normal',
                    CLX_CLASS_TITLE => 'GEN_LBL_RECORD_OPTIONS',
                    'cols' => CLX_NUM_FOUR,
                    'entries' => array(
                        array('label' => 'GEN_LBL_CDATE',
                            'fields' => array(
                                array('name' => 'cdate',
                                    'type' => 'dati',
                                    'dontmodify' => '1',
                                    'displaysize' => '35',
                                    'readonly' => true,
                                    'validations' => array(
                                        array('type' => 'string'
                                        )
                                    )
                                )
                            )
                        ), array('label' => 'GEN_LBL_CREATEDBY',
                            'fields' => array(
                                array('name' => 'createdby',
                                    'type' => 'lookup',
                                    'default' => '',
                                    'readonly' => true,
                                    'tool' => 'ACUS',
                                    'validations' => array(
                                        array('type' => 'lookup',
                                            'fieldname' => 'username',
                                            'tablename' => 'CLX_AC_USERS'
                                        )
                                    )
                                )
                            )
                        ), array('label' => 'GEN_LBL_MDATE',
                            'fields' => array(
                                array('name' => 'mdate',
                                    'type' => 'dati',
                                    'displaysize' => '35',
                                    'dontmodify' => '1',
                                    'readonly' => true,
                                    'validations' => array(
                                        array('type' => 'string'
                                        )
                                    )
                                )
                            )
                        ), array('label' => 'GEN_LBL_MODIFIEDBY',
                            'fields' => array(
                                array('name' => 'modifiedby',
                                    'type' => 'lookup',
                                    'default' => '',
                                    'readonly' => true,
                                    'tool' => 'ACUS',
                                    'validations' => array(
                                        array('type' => 'lookup',
                                            'fieldname' => 'username',
                                            'tablename' => 'CLX_AC_USERS'
                                        )
                                    )
                                )
                            )
                        )
                    )
                )
            )
        );
        $this->viewform = $this->editform;
        $this->quickfilterform = array(CLX_CLASS_TITLE => 'TOOL_PRST_TITLE_CREATE',
            'panels' => array(
                array('type' => 'normal',
                    'entries' => array(
                        array('label' => 'GEN_LBL_CODE',
                            'fields' => array(
                                array('name' => 'idcode',
                                    'type' => 'string',
                                    'size' => '20',
                                    'displaysize' => '35',
                                    'validations' => array()
                                )
                            )
                        ), array('label' => 'GEN_LBL_SHORT_NAME',
                            'fields' => array(
                                array('name' => 'idname',
                                    'type' => 'string',
                                    'default' => '',
                                    'size' => '64',
                                    'displaysize' => '35',
                                    'validations' => array()
                                )
                            )
                        ), array('label' => 'GEN_LBL_PROCESSCODE',
                            'fields' => array(
                                array('name' => 'processcode',
                                    'type' => 'lookup',
                                    'default' => '',
                                    'tool' => 'PRPR',
                                    'validations' => array()
                                )
                            )
                        ), array('label' => 'GEN_LBL_OWNERCODE',
                            'fields' => array(
                                array('name' => 'ownercode',
                                    'type' => 'lookup',
                                    'default' => '',
                                    'readonly' => false,
                                    'tool' => 'ACUS'
                                )
                            )
                        )
                    )
                )
            )
        );
        $this->toollist = array('name' => $this->tablename, CLX_CLASS_TITLE => $this->record_name_plural, 'has_filter' => true,
            'fields' =>
            array(array('name' => 'ID', 'type' => 'number', 'label' => 'GEN_LBL_ID'),
                array('name' => 'IDCODE', 'type' => 'string', 'label' => 'GEN_LBL_IDCODE'),
                array('name' => 'IDNAME', 'type' => 'string', 'label' => 'GEN_LBL_IDNAME'),
                array('name' => 'PROCESSCODE', 'type' => 'string', 'label' => 'GEN_LBL_PROCESSCODE'),
                array('name' => 'PROC_STATUS', 'type' => CLX_CLASS_LIST, 'label' => 'GEN_LBL_PROC_STATUS', 'options' => $this->notify_status),
                array('name' => 'STATUS', 'type' => CLX_CLASS_LIST, 'label' => 'GEN_LBL_STATUS', 'options' => $this->status),
                array('name' => 'PR_RECURRENCE', 'type' => 'fieldgroup', 'label' => 'GEN_LBL_PR_RECURRENCE', 'fields' => array(array('name' => 'PR_RECURRENCE', 'label' => 'GEN_LBL_PR_RECURRENCE', 'type' => CLX_CLASS_LIST, 'options' => $this->recu_types), array('name' => 'PR_VALUE', 'label' => 'GEN_LBL_PR_VALUE', 'type' => 'number', 'prefix' => '  '))),
                array('name' => 'BATCH_PID', 'type' => 'string', 'label' => 'GEN_LBL_BATCH_PID')),
            'buttons' => array(array('access' => 'canread', 'tool' => 'has_display', 'library' => 'GEN_LBL_RUN_TEST', 'name' => 'RUN_TEST', 'prop' => 'ONE', 'window' => 'POP'))
        );
		IF (ISSET($_SESSION['ADDRDEBUGSCHEDULER']) && (empty($_SERVER['REMOTE_ADDR']) || $_SESSION['ADDRDEBUGSCHEDULER']==$_SERVER['REMOTE_ADDR'])) {
			$this->debug=true;
		}	
    }

// End method
    /**
     * Tool Table Structure
     * @return array
     */
    function toolTableStructure() {
        $fields = array();
        $fields['ID'] = array('type' => 'int(11) unsigned', 'null' => 'NO', 'properties' => 'auto_increment', 'index' => 'PRI');
        $fields['IDCODE'] = array('type' => 'varchar(20)', 'null' => 'NO');
        $fields['IDNAME'] = array('type' => 'varchar(64)', 'null' => 'NO');
        $fields['PROCESSCODE'] = array('type' => 'varchar(20)', 'null' => 'NO');
        $fields['NOTES'] = array('type' => 'tinytext');
        $fields['OWNERCODE'] = array('type' => 'varchar(32)', 'null' => 'NO');
        $fields['PROC_STATUS'] = array('type' => 'varchar(10)', 'null' => 'NO', 'default' => '' . singleQuote('IDLE') . '', 'default_raw' => 'IDLE');
        $fields['STATUS'] = array('type' => 'tinyint(1) unsigned', 'null' => 'NO', 'default' => CLX_NUM_ONE);
        $fields['IS_DEBUG'] = array('type' => 'tinyint(1) unsigned', 'null' => 'NO', 'default' => CLX_NUM_ZERO);
        $fields['IS_DISPLAY'] = array('type' => 'tinyint(1) unsigned', 'null' => 'NO', 'default' => CLX_NUM_ZERO);
        $fields['LOG_METHOD'] = array('type' => 'tinyint(1) unsigned', 'null' => 'NO', 'default' => CLX_NUM_ZERO);
        $fields['PR_VALUE'] = array('type' => 'int(10) unsigned', 'null' => 'NO', 'default' => CLX_NUM_ZERO);
        $fields['TOUSER'] = array('type' => 'varchar(30)', 'null' => 'YES');
        $fields['NOTIFY_STATUS'] = array('type' => 'varchar(10)', 'null' => 'YES');
        $fields['SUBJECT'] = array('type' => 'varchar(255)', 'null' => 'YES');
        $fields['MESSAGE'] = array('type' => 'tinytext');
        $fields['CREATE_TASK'] = array('type' => 'tinyint(1) unsigned', 'null' => 'NO', 'default' => CLX_NUM_ZERO);
        $fields['CREATE_ALERT'] = array('type' => 'tinyint(1) unsigned', 'null' => 'NO', 'default' => CLX_NUM_ZERO);
        $fields['SEND_EMAIL'] = array('type' => 'tinyint(1) unsigned', 'null' => 'NO', 'default' => CLX_NUM_ZERO);
        $fields['PR_RECURRENCE'] = array('type' => 'varchar(10)', 'null' => 'NO', 'default' => SINGLEQUOTE('NONE'), 'default_raw' => 'NONE');
        //$fields['BATCH_PID'] = array('type' => 'varchar(20)', 'null' => 'NO', 'default' => CLX_NUM_ZERO);
        $fields['BATCH_PID'] = array('type' => 'varchar(35)', 'null' => 'YES', 'default' => CLX_NUM_ZERO);
        $fields['NEXT_RUN_TIME'] = array('type' => 'datetime', 'null' => 'YES');
        $fields['CDATE'] = array('type' => 'datetime', 'null' => 'YES');
        $fields['MDATE'] = array('type' => 'datetime', 'null' => 'YES');
        $fields['CREATEDBY'] = array('type' => 'varchar(30)', 'null' => 'NO');
        $fields['MODIFIEDBY'] = array('type' => 'varchar(30)', 'null' => 'NO');

        $ipf = 'CLX' . $this->code;
        $indexes = array($ipf . '01' => array('index' => 'PRI', 'fieldname' => 'ID'), $ipf . '02' => array('index' => 'UNI', 'fieldname' => 'IDCODE'));
        return array('idcode' => $this->code, 'idname' => $this->name, 'name' => $this->tablename, 'type' => 'InnoDB', 'fields' => $fields, 'indexes' => $indexes);
    }

    /**
     * Tool Install Method
     */
    function install() {
        $UIMI = &getTool('UIMI');
        $UIMI->add($this->code, 'MENU_TOOLS:MENU_PROCESS:MENU_' . $this->code, 'TOOL', $this->code, 'ac=LIST');
    }


    /**
     * Creates an xml object from record
     * @param array $record
     * @return obj
     */
    function pack($record) {
        $o = $this->packRecord($record);
        $objs = new XMLObject($this->code . '_OBJECTS');
        $this->packsub($objs, 'PRSP', 'SELECT * FROM CLX_PR_SCHED_PRMTS WHERE SCHEDULECODE =' . singleQuote($record['IDCODE']));
        $o->addObject($objs);
        return $o;
    }

    /**
     * Unpack Record into Metadata
     * @param obj $o
     * @return array
     */
    function unpack($parameters) {
        $parameters = array_values(array_merge(array('package' => null), $parameters));
        list($package) = $parameters;
        $record = $this->unpackRecord($package);
        $record['PROC_STATUS'] = 'NOT';
        $exists = (ClxCommon::$db->oneExistsRecords($this->tablename, 'UPPER(IDCODE)=' . singleQuote(uppercase($record['IDCODE']))));
        if (!empty($exists)) {
            unset($record['ID'], $record['CREATEDBY'], $record['OWNERCODE']);
            $record['MODIFIEDBY'] = $_SESSION['user_record']['username'];
            $id = ClxCommon::$db->updateRecord($this->tablename, $record, 'UPPER(IDCODE)=' . singleQuote(uppercase($record['IDCODE'])));
            $record['ID'] = $id;
        } else {
            $record['OWNERCODE'] = $_SESSION['user_record']['username'];
            $record['CREATEDBY'] = $_SESSION['user_record']['username'];
            $record['MODIFIEDBY'] = $_SESSION['user_record']['username'];
            $id = ClxCommon::$db->createRecord($this->tablename, $record, false);
        }
        ClxCommon::$db->deleteRecords('CLX_PR_SCHED_PRMTS', 'SCHEDULECODE=' . singleQuote($record['IDCODE']));
        if ($package->hasSubObject($this->code . '_OBJECTS')) {
            $objs = $package->getObjectXml($this->code . '_OBJECTS');
            foreach ($objs->objects as $o) {
                $toolcode = $o->tag;
                $tool = getTool($toolcode);
                $tool->unpack(array('package' => $o, 'id' => $id));
            }
        }
    }

    /**
     * Tool Action
     * @param string $ac Action code
     * @param string $idlist Record(s) affected by action
     * @return string
     */
    function action($ac, $idlist) {
		$tta=fopen(kPathToWebSite."/".kPathToTemp."/process_executions.txt","a+");
		fwrite($tta,chr(13).chr(10)."action ".chr(13).chr(10));
		fclose($tta);
        $contents = '';
        if ($this->getAccess()->canRead()) {
            switch ($ac) {
                case 'LIST':
                    if (!empty(UserRequest::$get['menuid'])) {
                        if (!empty(UserRequest::$request['mc'])) {//Adding parameter to the filter if mc is set on menu to display only mc.
                            $_SESSION['paging']['PRSCList']['mc'] = UserRequest::$request['mc'];
                        } else {
                            unset($_SESSION['paging']['PRSCList']['mc']);
                        }
                    }
                    if (!empty($_SESSION['paging']['PRSCList']['mc'])) {
                        $_SESSION['paging']['PRSCList']['criteria'] = ' EXISTS(SELECT PRPR.ID FROM CLX_PR_PROCESS PRPR WHERE PRPR.IDCODE = PROCESSCODE AND PRPR.MODULECODE LIKE ' . singleQuote($_SESSION['paging']['PRSCList']['mc'] . '%') . ')';
                        if (empty($_SESSION['paging']['PRSCList']['sortby']))
                            $_SESSION['paging']['PRSCList']['sortby'] = CLX_INIT_ZERO;
                        if (empty($_SESSION['paging']['PRSCList']['startat']))
                            $_SESSION['paging']['PRSCList']['startat'] = CLX_INIT_ZERO;
                    }
                    $contents = parent::action($ac, $idlist);
                    break;
                case 'RUN_TEST':

                    $this->runSchedule();

                    break;
                case 'MON':
                    $contents = $this->checkProcessStatusConsole();
                    $contents .= $this->checkProcessStatus();
                    break;
                case 'CHECK_SCHEDULE':
                    $contents = $this->check_schedule();
                    break;
                default:
					if ($this->debug==true) {
						pre('action pr_schedules');
						pre('idlist');
						pre($idlist);
					}
                    $contents = parent::action($ac, $idlist);
                    break;
            }
        } else {
            $contents = warning(dict('GEN_ERR_NO_ACCESS') . ' [PR_SCHEDULES-CAN_READ][' . $ac . '] ' . $this->getCode(false));
        }
        return $contents;
    }

    /**
     * Prepares tool default record
     * @return array
     */
    function defaultRecord() {
        $record = parent::defaultRecord();
        $record['PRSC_idname'] = '';
        $record['PRSC_ownercode'] = $_SESSION['user_record']['username'];
        $record['PRSC_proc_status'] = 'NOT';
        return $record;
    }

    /**
     * Creates Tool Record
     * @return int
     */
    function createRecord_() {
        $id = parent::createRecord_();
        $PRSP = &getTool('PRSP');
        $PRSP->createParameters($id);
        return $id;
    }

    /**
     * Saves Tool Record
     * @return int
     */
    function saveRecord() {
        $id = parent::saveRecord();
        $PRSP = &getTool('PRSP');
        $PRSP->createParameters($id);
        return $id;
    }

    /**
     * Builds tool display form
     * @param array $record
     * @return string
     */
    function viewFormFill($record) {
        addHistory('schedulecode', $record['PRSC_idcode']);
        // Elements
        $PRSP = &getTool('PRSP');
        $contents = $PRSP->makepanel($record['PRSC_idcode']);
        return $contents;
    }

    /**
     * Deletes record on selected id
     *
     * @param integer $id
     */
    function deleteRecordSk($id) {
        $idcode = ClxCommon::$db->getField($this->tablename, 'IDCODE', 'ID=' . $id);
        //Process
        $PRSP = &getTool('PRSP');
        $rs = ClxCommon::$db->getRecords($PRSP->tablename, 'SCHEDULECODE=' . singleQuote($idcode));
        while ($r = ClxCommon::$db->getNextRecord($rs, ASSOCIATIVE)) {
            $PRSP->deleteRecordSk($r['ID']);
        }
        //Schedules Logs
        $PRLO = &getTool('PRLO');
        $rs = ClxCommon::$db->getRecords($PRLO->tablename, 'SCHEDULECODE=' . singleQuote($idcode));
        while ($r = ClxCommon::$db->getNextRecord($rs, ASSOCIATIVE)) {
            $PRLO->deleteRecordSk($r['ID']);
        }
        ClxCommon::$db->deleteRecords($this->tablename, 'ID=' . $id);
    }


    /**
     * Run Process from Schedule code
     * @see pr_processes::execute_process($processcode, $schedulecode)
     */
    function runSchedule() {

        $PRLO = &getTool('PRLO');
        $PRPR = &getTool('PRPR');
		$tta=fopen(kPathToWebSite."/".kPathToTemp."/process_executions.txt","a+");
		fwrite($tta,chr(13).chr(10)."schedule run ".chr(13).chr(10));
		fclose($tta);
			
        UserResponse::setRunTimeLimit(CLX_NUM_60 * CLX_NUM_60 * CLX_NUM_48);
        $idlist = getIdlist();
		
        $rs = ClxCommon::$db->getRecords($this->tablename, 'STATUS=1 AND ID IN (' . toString($idlist) . ')');
        while ($r = ClxCommon::$db->getNextRecord($rs, ASSOCIATIVE)) {
            $this->scheduleCode = $r['IDCODE'];
            $display = (!empty($r['IS_DISPLAY'])) ? true : false;
            $debug = (!empty($r['IS_DEBUG'])) ? true : false;
            $html = true;
            $this->setNextRun($r);

            //if ($r['LOG_METHOD'] == CLX_NUM_ONE) {
            if (strval($r['LOG_METHOD']) == CLX_NUM_ONE) {
                $logid = ClxCommon::$db->getField($PRLO->tablename, 'ID', 'PROCESSCODE = ' . singleQuote($r['PROCESSCODE']).' AND SCHEDULECODE = ' . singleQuote($r['IDCODE']));
                if (!empty($logid)) {
                    $PRLO->deleteChildRecord($logid);
                }
            }

            if (!empty($r['PROCESSCODE'])) {
                $PRPR->execute_process($r['PROCESSCODE'], $r['IDCODE'], $display, $debug, $html);
            }
        }

    }

    /**
     * Transforms parameters according format into array required by the process
     * @param string $parameters
     * @return array
     */
    function cronParseBatchArgumentParameters($argument) {
        $parameters = array();
        $a = explode('&', $argument);
        foreach ($a as $v) {
            $v = explode('=', $v);
            if (!empty($v)) {
                $parameters[$v[CLX_NUM_ZERO]] = $v[CLX_NUM_ONE];
            }
        }
        return $parameters;
    }

    /**
     * Runs cron for demand only
     * @param string $schedulecode
     * @param string $startat
     * @param string $parameters
     */
    function cronOnDemandRun($schedulecode, $startat, $parameters) {
		$tta=fopen(kPathToWebSite."/".kPathToTemp."/process_executions.txt","a+");
		fwrite($tta,chr(13).chr(10)."cronOnDemandRun ".chr(13).chr(10));
		fclose($tta);
        $return = true;
        if ($this->now === '')
            $this->now = adodb_date('Y-m-d H:i:s');
        $this->setPid();
        $this->cron_welcome_sign('Process Engine Events');
        //First add schedule configuration
        ClxFreeReturn($this->setSchedule($schedulecode, 'ONE', date('Y-m-d'), date('H:i:s'), $parameters, 'NOT'));
        ClxCommon::$db->commit();
        //Now run as in check schedule
        $sql = 'UPDATE ' . $this->tablename . '  PRSC SET PROC_STATUS = ' . singleQuote('RUN') . ', BATCH_PID = ' . singleQuote($this->batch_pid) . ' WHERE PRSC.STATUS = 1 AND PRSC.IDCODE = ' . singleQuote($schedulecode);
        ClxCommon::$db->executeQuery($sql);
        ClxCommon::$db->commit();
        $ACUS = &getTool('ACUS');
        $sql = 'SELECT ID, IDCODE, IDNAME, NEXT_RUN_TIME, OWNERCODE, MODIFIEDBY FROM ' . $this->tablename . '  WHERE PROC_STATUS =' . singleQuote('RUN') . ' AND BATCH_PID = ' . singleQuote($this->batch_pid) . ' AND IDCODE = ' . singleQuote($schedulecode) . ' ORDER BY NEXT_RUN_TIME ASC';
        $results = ClxCommon::$db->executeQuerySilent($sql);
        if (!empty($results)) {
            while ($result = ClxCommon::$db->getNextRecord($results, ASSOCIATIVE)) {
                //Setting user for process
                $init_session = $_SESSION;
                $this->executedby = $result['MODIFIEDBY'];
                if (!is_null($result['OWNERCODE']) && $result['OWNERCODE'] !== '') {
                    $this->executedby = $result['OWNERCODE'];
                }
                $ACUS->process_engine_login($this->executedby);
                ClxCommon::$db->commit();
                echolog('Starting Process: ' . $result['IDCODE'] . ', ' . $result['IDNAME']);
                $this->cronRun($result['IDCODE']);
                echolog('Ending Process: ' . $result['IDCODE'] . ', ' . $result['IDNAME']);
                $sql = 'UPDATE ' . $this->tablename . '  SET PROC_STATUS = ' . singleQuote('END') . ' WHERE NEXT_RUN_TIME IS NULL AND ID = ' . $result['ID'];
                ClxCommon::$db->executeQuery($sql);
                $sql = 'UPDATE ' . $this->tablename . '  SET PROC_STATUS = ' . singleQuote('IDLE') . ' WHERE NEXT_RUN_TIME IS NOT null AND ID = ' . $result['ID'];
                ClxCommon::$db->executeQuery($sql);
                ClxCommon::$db->commit();
                $ACUS->logout();
                $_SESSION = $init_session;
            }
        }
        return $return;
    }

    /**
     * used on cron process to run schedule
     * @todo needs revision
     */
    function cronRun($schedulecode) {
		$tta=fopen(kPathToWebSite."/".kPathToTemp."/process_executions.txt","a+");
		fwrite($tta,chr(13).chr(10)."schedule cronrun ".chr(13).chr(10));
		fclose($tta);
        $PRLO = &getTool('PRLO');
        $PRPR = getNewTool('PRPR');
        UserResponse::setRunTimeLimit(CLX_NUM_60 * CLX_NUM_60 * CLX_NUM_48);
		if ( isset($_SESSION["_dEBUG"]) and $_SESSION["_dEBUG"]==99 ) { 
					$tta=fopen(kPathToWebSite."/".kPathToTemp."archivo.txt","a+");	
					fwrite($tta,"1- fin-hora:".date("h:i:s d/m/Y").chr(13).chr(10));
					fwrite($tta,"iniciando cronRUN " .chr(13).chr(10));fclose($tta);
		}
        $rs = ClxCommon::$db->getRecords($this->tablename, 'STATUS=1', 'IDCODE IN (' . singleQuote($schedulecode) . ')');
        if (($r = ClxCommon::$db->getNextRecord($rs))) {
            $this->scheduleCode = $r['IDCODE'];
            $display = (!empty($r['IS_DISPLAY'])) ? true : false;
            $debug = (!empty($r['IS_DEBUG'])) ? true : false;
            $html = false;
            $this->setNextRun($r);
            //if ($r['LOG_METHOD'] == CLX_NUM_ONE) {
            if (strval($r['LOG_METHOD']) == CLX_NUM_ONE) {
                $logid = ClxCommon::$db->getField($PRLO->tablename, 'ID', 'PROCESSCODE = ' . singleQuote($r['PROCESSCODE']), 'SCHEDULECODE = ' . singleQuote($r['IDCODE']));
                if (!empty($logid)) {
                    $PRLO->deleteChildRecord($logid);
                }
            }
            if (!empty($r['PROCESSCODE'])) {
                $PRPR->execute_process($r['PROCESSCODE'], $r['IDCODE'], $display, $debug, $html);
            }
        } else {
            $BUGS = &getTool('BUGS');
            $BUGS->report(E_USER_WARNING, 'No schedule with code: ' . $schedulecode, UserRequest::$server['SCRIPT_NAME'], 'unknown line', UserRequest::$request, CLX_NUM_ZERO, 'No schedule with code: ' . $schedulecode);
        }
    }

    /**
     * Display welcome sign on schedule executed by cron
     */
    function cron_welcome_sign($msg = '') {
		$tta=fopen(kPathToWebSite."/".kPathToTemp."/process_executions.txt","a+");
		fwrite($tta,chr(13).chr(10)."cron_welcome_sign ".chr(13).chr(10));
		fclose($tta);
        $this->now = adodb_date('Y-m-d H:i:s');
        $this->setPid();
        $d = CLX_NUM_79;
        $rec = array(CLX_NUM_ZERO => 'CLX PROCESS ENGINE',
            CLX_NUM_ONE => 'Version: ' . $this->version,
            CLX_NUM_TWO => 'System: ' . kDatabaseName . '@' . kDatabaseURL,
            CLX_NUM_THREE => 'Batch ID: ' . $this->batch_pid,
            CLX_NUM_FOUR => 'Next run: ' . $this->now);
        if ($msg <> '')
            $rec[CLX_NUM_FIVE] = $msg;
        echolog('');
        echolog(str_repeat('*', $d));
        foreach ($rec as $val) {
            $l = $d - strlen($val) - CLX_NUM_THREE;
            echolog('* ' . $val . str_repeat(' ', $l) . '*');
        }
        echolog(str_repeat('*', $d));
      //  ob_flush();
    }

    /**
     * Use by cron to verify the next schedule to be run
     */
    function check_schedule() {
		$tta=fopen(kPathToWebSite."/".kPathToTemp."/process_executions.txt","a+");
		fwrite($tta,chr(13).chr(10)."check_schedule ".chr(13).chr(10));
		fclose($tta);
        $return = true;
        $this->now = adodb_date('Y-m-d H:i:s');
        $this->setPid();
        $this->cron_welcome_sign();
        //Lock item
        switch (kDatabaseType) {
            case 'oci':
                ClxCommon::$db->executeQuery('SET TRANSACTION NAME '.singleQuote('check_schedule_'.adodb_date('YmdHis').'_trn'));
                break;
            default:
                ClxCommon::$db->executeQuery('START TRANSACTION WITH CONSISTENT SNAPSHOT');
                break;
        }
        $sql = 'UPDATE ' . $this->tablename . '  PRSC SET PROC_STATUS = ' . singleQuote('RUN') . ', BATCH_PID = ' . singleQuote($this->batch_pid) . ' WHERE PRSC.NEXT_RUN_TIME <= ' . singleQuote($this->now) . ' AND PRSC.PROC_STATUS =' . singleQuote('IDLE') . ' AND PRSC.STATUS = 1 AND PRSC.PR_RECURRENCE <> ' . singleQuote('NEV') . ' ';
        ClxCommon::$db->executeQuery($sql);
        ClxCommon::$db->commit();
        $ACUS = &getTool('ACUS');
        $contadorPROC=1;
        $sql = 'SELECT ID, IDCODE, IDNAME, NEXT_RUN_TIME, OWNERCODE, MODIFIEDBY FROM ' . $this->tablename . '  WHERE PROC_STATUS =' . singleQuote('RUN') . '  AND BATCH_PID = ' . singleQuote($this->batch_pid) . '  ORDER BY NEXT_RUN_TIME ASC';
        $results = ClxCommon::$db->executeQuerySilent($sql);
		if ( isset($_SESSION["_dEBUG"]) and $_SESSION["_dEBUG"]==99 ) { 
			$tta=fopen(kPathToWebSite."/".kPathToTemp."archivo.txt","a+");	
			fwrite($tta,"1- fin-hora:".date("h:i:s d/m/Y").chr(13).chr(10));
			fwrite($tta,"EMPTY_RESULTS".(empty($results)?1:0) .chr(13).chr(10));fclose($tta);
		}
        if (!empty($results)) {
            //Setting user for process
            $init_session = $_SESSION;
            $login = false;
            while ($result = ClxCommon::$db->getNextRecord($results, ASSOCIATIVE)) {
				$sql = 'UPDATE ' . $this->tablename . ' SET BATCH_PID = ' . singleQuote($this->batch_pid.$contadorPROC). ' WHERE BATCH_PID = ' . singleQuote($this->batch_pid).' AND ID = ' . $result['ID'];
        		ClxCommon::$db->executeQuery($sql);
				if ( isset($_SESSION["_dEBUG"]) and $_SESSION["_dEBUG"]==99 ) { 
					$tta=fopen(kPathToWebSite."/".kPathToTemp."archivo.txt","a+");	
					fwrite($tta,"1- fin-hora:".date("h:i:s d/m/Y").chr(13).chr(10));
					fwrite($tta,"ciclo while process " .chr(13).chr(10));fclose($tta);
				}
                $this->executedby = $result['MODIFIEDBY'];
                if (!is_null($result['OWNERCODE']) && $result['OWNERCODE'] !== '') {
                    $this->executedby = $result['OWNERCODE'];
                }
				if ( isset($_SESSION["_dEBUG"]) and $_SESSION["_dEBUG"]==99 ) { 
					$tta=fopen(kPathToWebSite."/".kPathToTemp."archivo.txt","a+");	
					fwrite($tta,"1- fin-hora:".date("h:i:s d/m/Y").chr(13).chr(10));
					fwrite($tta,"process_engine_login " .chr(13).chr(10));fclose($tta);
				}
				
                if (empty($login)) {
                    $login = true;
                    $ACUS->process_engine_login($this->executedby);
                }
                ClxCommon::$db->commit();
                echolog('Starting Process: ' . $result['IDCODE'] . ', ' . $result['IDNAME']);
                //Init Transaction
				if ( isset($_SESSION["_dEBUG"]) and $_SESSION["_dEBUG"]==99 ) { 
					$tta=fopen(kPathToWebSite."/".kPathToTemp."archivo.txt","a+");	
					fwrite($tta,"1- fin-hora:".date("h:i:s d/m/Y").chr(13).chr(10));
					fwrite($tta,"starting message process, call cronrun " .chr(13).chr(10));fclose($tta);
				}
                ClxCommon::$db->setAutocommit(false);
                $this->cronRun($result['IDCODE']);
                //Commit if apply
                ClxCommon::$db->commit();
                ClxCommon::$db->setAutocommit(true);
                echolog('Ending Process: ' . $result['IDCODE'] . ', ' . $result['IDNAME']);
                $sql = 'UPDATE ' . $this->tablename . '  SET PROC_STATUS = ' . singleQuote('END') . ' WHERE NEXT_RUN_TIME IS NULL AND ID = ' . $result['ID'];
                ClxCommon::$db->executeQuery($sql);
                $sql = 'UPDATE ' . $this->tablename . ' SET PROC_STATUS =' . singleQuote('IDLE') . ' WHERE NEXT_RUN_TIME IS NOT null AND ID = ' . $result['ID'];
                ClxCommon::$db->executeQuery($sql);
                ClxCommon::$db->commit();
				if ( isset($_SESSION["_dEBUG"]) and $_SESSION["_dEBUG"]==99 ) { 
					$tta=fopen(kPathToWebSite."/".kPathToTemp."archivo.txt","a+");	
					fwrite($tta,"1- fin-hora:".date("h:i:s d/m/Y").chr(13).chr(10));
					fwrite($tta,"re-ciclo while process " .chr(13).chr(10));fclose($tta);
				}
				$contadorPROC++;
            }
            $ACUS->logout();
            $_SESSION = $init_session;
        } else {
            echolog('Error on database. cannot select schedules. please check database files');
            $return = false;
        }
        return $return;
    }

// End method
    /**
     * Marks logs on error in case of failure. Called by tool bugs
     */
    function setDieStatus() {
		if (empty($this->batch_pid)) {
			$this->batch_pid=getPid();
		}	
        $sql = 'UPDATE ' . $this->tablename . '  SET PROC_STATUS = ' . singleQuote('ERRO') . ' WHERE PROC_STATUS =' . singleQuote('RUN') . ' AND BATCH_PID = ' . singleQuote($this->batch_pid);
        if ( isset($_SESSION["_dEBUG"]) && $_SESSION["_dEBUG"]==99 ) {$posx=0;$pos=1610;$tta=fopen(kPathToWebSite."/".kPathToTemp."archivo.txt","a+");fwrite($tta,chr(13).chr(10)."DIE STATUS SCHEDULE".chr(13).chr(10));$te=debug_backtrace(); for ($t=0;$t<12;$t++) {if (isset($te[$t]["file"])){if ($t==$posx)fwrite($tta,chr(13).chr(10).$pos."_".$te[$posx]["function"]."_db_import-hora:".date("h:i:s d/m/Y").chr(13).chr(10));fwrite($tta,substr($te[$t]["file"].str_repeat(" ",80),0,80).chr(9).$te[$t]["line"].chr(9).$te[$t]["function"].chr(13).chr(10));}}fclose($tta);}

        ClxCommon::$db->executeQuery($sql);
    }

    /**
     * Verify if exchedule exists
     * @param string $schedule_code
     * @return bool
     */
    function schedule_exists($schedule_code) {
	$qty = ClxCommon::$db->oneExistsRecords($this->tablename, 'IDCODE = ' . singleQuote($schedule_code));
        return ($qty > CLX_NUM_ZERO) ? true : false;
    }

    /**
     * Set btach process
     * @param string $pid
     */
    function setPid($pid = '') {
        if ($pid <> '') {
            $this->batch_pid = $pid;
        } else {
			usleep(20);
            $this->batch_pid = getPid();
        }
    }

    /**
     * Set the next Schedule Code
     */
    function setNextRun($schedule_rec) {
		$tta=fopen(kPathToWebSite."/".kPathToTemp."/process_executions.txt","a+");
		fwrite($tta,chr(13).chr(10)."setNextRun ".chr(13).chr(10));
		fclose($tta);
        $next_run_date = '';
        if (empty($schedule_rec['PR_RECURRENCE']))
            $schedule_rec['PR_RECURRENCE'] = CLX_INIT_ZERO;
        if (empty($schedule_rec['PR_VALUE']))
            $schedule_rec['PR_VALUE'] = CLX_INIT_ZERO;
        $last_run_date = adodb_getdate(adodb_strtotime($schedule_rec['NEXT_RUN_TIME']));
        $interval_minutes = $schedule_rec['PR_RECURRENCE'] * $schedule_rec['PR_VALUE'];
        //e cholog('Next execution ' . $interval_minutes);
        if ($last_run_date['hours'] == CLX_NUM_18 && $last_run_date['minutes'] == CLX_NUM_59 && $last_run_date['seconds'] == CLX_NUM_59 && $last_run_date['mon'] == CLX_NUM_12 && $last_run_date['mday'] == CLX_NUM_31 && $last_run_date['year'] == CLX_NUM_1969) {
            $last_run_date = adodb_getdate();
        }
        if ($interval_minutes == CLX_NUM_ZERO) {
            $sql = 'UPDATE ' . $this->tablename . '  SET NEXT_RUN_TIME = NULL WHERE IDCODE = ' . singleQuote($schedule_rec['IDCODE']);
        } else {
            $last_run_date['minutes'] = $last_run_date['minutes'] + $interval_minutes;
            $next_run_date = adodb_date('Y-m-d H:i:s', adodb_mktime($last_run_date['hours'], $last_run_date['minutes'], $last_run_date['seconds'], $last_run_date['mon'], $last_run_date['mday'], $last_run_date['year']));
            $sql = 'UPDATE ' . $this->tablename . '  SET NEXT_RUN_TIME = ' . singleQuote($next_run_date) . '  WHERE IDCODE = ' . singleQuote($schedule_rec['IDCODE']);
            echoLog('Next run date: ' . $next_run_date);
        }
        ClxCommon::$db->executeQuery($sql);
		ClxCommon::$db->executeQuery('commit');
    }

    /**
     * Creates schedule
     * @param string $scheduleCode
     * @param string $scheduleName
     * @param string $processCode
     * @param string $recurrence
     * @param int $interval
     * @param string $start_date
     * @param string $start_time
     * @param array $parameters
     */
    function createSchedule($scheduleCode, $scheduleName, $processCode, $recurrence, $interval, $start_date, $start_time, $parameters = array(), $display = CLX_NUM_ONE, $debug = CLX_NUM_ZERO) {
        $tta=fopen(kPathToWebSite."/".kPathToTemp."/process_executions.txt","a+");
		fwrite($tta,chr(13).chr(10)."createSchedule ".chr(13).chr(10));
		fclose($tta);
        $this->translateRecurrence($recurrence, $interval);
        $schedule = $this->defaultTableRecord();
        unset($schedule['ID']);
        $schedule['IDCODE'] = $scheduleCode;
        $schedule['IDNAME'] = $scheduleName;
        $schedule['PROCESSCODE'] = $processCode;
        $schedule['STATUS'] = CLX_NUM_ONE;
        $schedule['IS_DEBUG'] = $debug;
        $schedule['IS_DISPLAY'] = $display;
        $schedule['LOG_METHOD'] = CLX_INIT_ZERO;
        $schedule['PROC_STATUS'] = 'NOT';
        $schedule['PR_RECURRENCE'] = $recurrence;
        $schedule['PR_VALUE'] = $interval;
        $schedule['NEXT_RUN_TIME'] = $start_date . ' ' . $start_time;
        $schedule['MDATE'] = getNow('Y-m-d H:i:s');
        $schedule['CDATE'] = $schedule['MDATE'];
        $schedule['OWNERCODE'] = $_SESSION['user_record']['username'];
        $schedule['CREATEDBY'] = $_SESSION['user_record']['username'];
        $schedule['MODIFIEDBY'] = $_SESSION['user_record']['username'];
        //Create schedule
        $id = ClxCommon::$db->createRecord($this->tablename, $schedule, false);
        if (empty($id)) {
            return false;
        }
        //Setting paramters
        $PRSP = &getTool('PRSP');
        $PRSP->createParameters($id);
        $PRSP->setScheduleParameters($scheduleCode, $parameters);
        ClxCommon::$db->updateRecord($this->tablename, array('PROC_STATUS' => 'IDLE'), 'ID = ' . $id);
        return $scheduleCode;
    }

    /**
     * Set the new Schedule Execution
     * For Recurrence options please check dblist
     * @param unknown_type $scheduleCode
     * @param unknown_type $recurrence
     * @param unknown_type $start_date
     * @param unknown_type $start_time
     * @param unknown_type $parameters
     * @param unknown_type $status
     * @return boolean
     */
    function setSchedule($scheduleCode, $recurrence, $start_date, $start_time, $parameters, $status = 'IDLE') {
		$tta=fopen(kPathToWebSite."/".kPathToTemp."/process_executions.txt","a+");
		fwrite($tta,chr(13).chr(10)."setSchedule ".chr(13).chr(10));
		fclose($tta);
        $interval = CLX_NUM_ONE;
        $this->translateRecurrence($recurrence, $interval);
        $PRSP = &getTool('PRSP');
        //Create Process Schedule Schedule
        foreach ($parameters as $key => $val) {
            ClxCommon::$db->updateRecord($PRSP->tablename, array('PARAMETER_VALUE' => $val), 'FIELDNAME = ' . singleQuote($key) . ' AND SCHEDULECODE =' . singleQuote($scheduleCode));
        }
		if ( isset($_SESSION["_dEBUG"]) && $_SESSION["_dEBUG"]==99 ) {$posx=0;$pos=1610;$tta=fopen(kPathToWebSite."/".kPathToTemp."archivo.txt","a+");fwrite($tta,chr(13).chr(10)."STATUS schedule:".$status.chr(13).chr(10));$te=debug_backtrace(); for ($t=0;$t<12;$t++) {if (isset($te[$t]["file"])){if ($t==$posx)fwrite($tta,chr(13).chr(10).$pos."_".$te[$posx]["function"]."_db_import-hora:".date("h:i:s d/m/Y").chr(13).chr(10));fwrite($tta,substr($te[$t]["file"].str_repeat(" ",80),0,80).chr(9).$te[$t]["line"].chr(9).$te[$t]["function"].chr(13).chr(10));}}fclose($tta);}

        ClxCommon::$db->updateRecord($this->tablename, array('PR_RECURRENCE' => $recurrence, 'PR_VALUE' => $interval, 'PROC_STATUS' => $status, 'NEXT_RUN_TIME' => $start_date . ' ' . $start_time), 'IDCODE =' . singleQuote($scheduleCode));
        return true;
    }

    /**
     * Create schedule for execution one time.
     */
    function createScheduleOneTimeNow($scheduleCode, $scheduleName, $processCode, $parameters = array()) {
        return $this->createSchedule($scheduleCode, $scheduleName, $processCode, CLX_NUM_ZERO, CLX_NUM_ZERO, date('Y-m-d'), date('H:i:s'), $parameters);
    }

    /**
     * Creates an schedule with no recurrence and run the process
     * @param string $processcode
     * @param array $parameters
     */
    function runProcessNow($processcode, $parameters, $processname = '') {
		$tta=fopen(kPathToWebSite."/".kPathToTemp."/process_executions.txt","a+");
		fwrite($tta,chr(13).chr(10)."runProcessNow ".chr(13).chr(10));
		fclose($tta);
        if (ClxCommon::$db->oneExistsRecords('CLX_PR_PROCESS', 'IDCODE =' . singleQuote($processcode)) == CLX_NUM_ZERO) {
            return false;
        }
        $scheduleCode = uniqid('TMP_');
        if ($processname === '') {
            $processname = ClxCommon::$db->getField('CLX_PR_PROCESS', 'IDNAME', 'IDCODE=' . singleQuote($processcode));
        }
        //Create schedule
        $schedule = $this->createSchedule($scheduleCode, $processname, $processcode, CLX_NUM_ZERO, CLX_NUM_ZERO, date('Y-m-d'), date('H:i:s'), $parameters, CLX_NUM_ZERO, CLX_NUM_ZERO);
        //Run schedule
        $this->cronRun($schedule);
        //End
		if ( isset($_SESSION["_dEBUG"]) && $_SESSION["_dEBUG"]==99 ) {$posx=0;$pos=1610;$tta=fopen(kPathToWebSite."/".kPathToTemp."archivo.txt","a+");fwrite($tta,chr(13).chr(10)."STATUS schedule FIJO:END".chr(13).chr(10));$te=debug_backtrace(); for ($t=0;$t<12;$t++) {if (isset($te[$t]["file"])){if ($t==$posx)fwrite($tta,chr(13).chr(10).$pos."_".$te[$posx]["function"]."_db_import-hora:".date("h:i:s d/m/Y").chr(13).chr(10));fwrite($tta,substr($te[$t]["file"].str_repeat(" ",80),0,80).chr(9).$te[$t]["line"].chr(9).$te[$t]["function"].chr(13).chr(10));}}fclose($tta);}

        $sql = 'UPDATE ' . $this->tablename . ' SET PROC_STATUS = ' . singleQuote('END') . ' WHERE IDCODE = ' . singleQuote($scheduleCode);
        ClxCommon::$db->executeQuery($sql);
        return true;
    }

    /**
     * Set schedule for execution one time.
     */
    function runScheduleOneTimeNow($scheduleCode, $parameters = array()) {
        return $this->setSchedule($scheduleCode, 'ONE', date('Y-m-d'), date('H:i:s'), $parameters);
    }

    /**
     * Translation from old recurrent list for compatibility
     * @param string $recurrence
     * @param int $interval
     */
    function translateRecurrence(&$recurrence, &$interval) {
        switch ($recurrence) {
            case 'ONE':
                $recurrence = CLX_INIT_ZERO;
                $interval = CLX_INIT_ZERO;
                break;
            case 'DAY':
                $recurrence = CLX_NUM_1440;
                $interval = CLX_NUM_ONE;
                break;
            case 'WEEK':
                $recurrence = CLX_NUM_1440;
                $interval = CLX_NUM_7;
                break;
            case 'MONT':
                $recurrence = CLX_NUM_1440;
                $interval = CLX_NUM_30;
                break;
            case 'YEAR':
                $recurrence = CLX_NUM_1440;
                $interval = CLX_365_DAYS;
                break;
            case 'HOUR':
                $recurrence = CLX_NUM_60;
                $interval = CLX_NUM_ONE;
                break;
            case 'NEVE':
                $recurrence = CLX_INIT_ZERO;
                $interval = CLX_INIT_ZERO;
                break;
            default:
                if (empty($recurrence))
                    $recurrence = CLX_INIT_ZERO;
                if (empty($interval))
                    $interval = CLX_INIT_ZERO;
                break;
        }
    }

    function checkProcessStatusConsole() {
	if (is_windows()) {
            $cmd = 'tasklist';
        } else {
            $cmd = 'ps -Al';
        }
        $output = null;
        exec($cmd, $output);
        $a = new htmlTextArea('', implode(CLX_CHAR_NL, $output), CLX_NUM_20, CLX_NUM_90);
        $b = new htmlTable(CLX_WIDTH_ONE);
        $b->setProps(array(CLX_PROP_CLASS => CLX_CLASS_FORM));

        $b->addCell(dict('GEN_LBL_CONSOLE_MONITOR'), array(CLX_PROP_CLASS => CLX_CLASS_TITLE));
        $b->addCell($a->source(), array(CLX_PROP_CLASS => CLX_CLASS_DATA));
        return $b->source();
    }

    function checkProcessStatus() {
		$tta=fopen(kPathToWebSite."/".kPathToTemp."/process_executions.txt","a+");
		fwrite($tta,chr(13).chr(10)."checkProcessStatus ".chr(13).chr(10));
		fclose($tta);
        $title = dict('GEN_LBL_DB_PROCESSLIST');
        $tableFrame = new htmlTable(CLX_WIDTH_ONE);
        $tableFrame->setProps(array(CLX_PROP_CLASS => 'page'));
        $tableFrame->addCell($title, array(CLX_PROP_CLASS => CLX_CLASS_TITLE));
        $table = new htmlTable(CLX_WIDTH_EIGHT);
        $table->setProps(array(CLX_PROP_CLASS => CLX_CLASS_FORM, CLX_PROP_WIDTH => CLX_PROP_100PCT));
        if (kDatabaseType === 'oci') {
            $table->addCells(CLX_CLASS_HEADERS, array(dict('GEN_LBL_OS_PROCESS_ID'), dict('GEN_LBL_OS_USERNAME'), dict('GEN_LBL_PROGRAM'), dict('GEN_LBL_LOGONTIME'), dict('GEN_LBL_SQL_COMMAND'), dict('GEN_LBL_USED_MEMORY'), dict('GEN_LBL_ALLOC_MEMORY'), dict('GEN_LBL_FREEABLE_MEMORY')));
            /*
              v$process p PGA_USED_MEM NUMBER PGA memory currently used by the process PGA_ALLOC_MEM NUMBER PGA memory currently allocated by the process (including free PGA memory not yet released to the operating system by the server process) PGA_FREEABLE_MEM NUMBER Allocated PGA memory which can be freed PGA_MAX_MEM NUMBER Maximum PGA memory ever allocated by the process */
            $this->check_batch = true;
            //$sql = 'SELECT s.process, s.osuser, s.program, s.logon_time, st.sql_text, p.PGA_USED_MEM, p.PGA_ALLOC_MEM, p.PGA_FREEABLE_MEM, P.PGA_MAX_MEM FROM v' . $session . ' s, v' . $sql . ' st, v' . $process . ' p WHERE s.sql_hash_value = st.hash_value AND s.sql_address = st.address AND s.paddr = p.addr AND s.status = ' . singleQuote('ACTIV') . '';
            $sql = 'SELECT s.process, s.osuser, s.program, s.logon_time, st.sql_text, p.PGA_USED_MEM, p.PGA_ALLOC_MEM, p.PGA_FREEABLE_MEM, P.PGA_MAX_MEM FROM v$session s, v$sql st, v$process p WHERE s.sql_hash_value = st.hash_value AND s.sql_address = st.address AND s.paddr = p.addr AND s.status = ' . singleQuote('ACTIV') . '';
            //
            // Guarda valores de variables de sesion que deben ser asignadas para evitar el throw normal del error
            $prev_disable_execute_query_error_trap = (isset($_SESSION['disable_execute_query_error_trap']) ? $_SESSION['disable_execute_query_error_trap'] : null);
            $_SESSION['disable_execute_query_error_trap'] = true; 
            ClxCommon::$db->clearErrorMsg();
            // Ejecuta SQL, si hay error, reporta que no hay acceso tipo "SYS"
            $results = ClxCommon::$db->executeQuery($sql);
            // Restaura variables de sesion asignadas para evitar el throw normal del error
            if(is_null($disable_execute_query_error_trap)) {
                unset($_SESSION['disable_execute_query_error_trap']);
            } else {
                $_SESSION['disable_execute_query_error_trap'] = $disable_execute_query_error_trap; // Restaura variable de session
            }
            // Verifica eventual error
            if(ClxCommon::$db->errorMsg() !== '') {
                // No hay acceso "sys" a esas tablas
                $class = ($i % CLX_NUM_TWO == CLX_NUM_ZERO) ? CLX_CLASS_DATA : CLX_CLASS_DATAALT;
                $table->addCells($class, array('', '', '', '', 'No [SYS] access<br>' . ClxCommon::$db->errorMsg(), '', '', ''));
                ClxCommon::$db->clearErrorMsg();
            } else {
            //
                $i = CLX_INIT_ZERO;
                while (($result = ClxCommon::$db->getNextRecord($results, NUMERIC))) {
                    $class = ($i % CLX_NUM_TWO == CLX_NUM_ZERO) ? CLX_CLASS_DATA : CLX_CLASS_DATAALT;
                    $i++;
                    $mem_pct = ($result[CLX_NUM_FIVE] / $result[CLX_NUM_8]) * CLX_NUM_HUNDRED;
                    $mem = xFormat::number($result[CLX_NUM_FIVE]) . ' / ' .
                            xFormat::number($result[CLX_NUM_8]);
                    $textarea = new htmlTextArea('', $result[CLX_NUM_FOUR], CLX_NUM_FIVE, CLX_NUM_40);
                    $textarea->setProp('readonly', CLX_NUM_ONE);
                    $textarea->setProp(CLX_PROP_CLASS, CLX_CLASS_DATA);
                    $table->addCells($class, array($result[CLX_NUM_ZERO], $result[CLX_NUM_ONE], $result[CLX_NUM_TWO], $result[CLX_NUM_THREE], $textarea->source(), $mem . ' (' . xFormat::number($mem_pct) . '%)', xFormat::number($result[CLX_NUM_SIX]), xFormat::number($result[CLX_NUM_7])));
                    unset($textarea);
                }
            }
        } else {//mysql
            $this->check_batch = false;
            $table->addCells(CLX_CLASS_HEADERS, array(dict('GEN_LBL_MYSQL_ID'), dict('GEN_LBL_USER'), dict('GEN_LBL_HOST'), dict('GEN_LBL_DB'), dict('GEN_LBL_COMMAND'), dict('GEN_LBL_LOGONTIME'), dict('GEN_LBL_STATE'), dict('GEN_LBL_SQL_COMMAND')));
            $sql = 'show full processlist';
            $results = ClxCommon::$db->executeQuery($sql);
            $i = CLX_INIT_ZERO;
            while (($result = ClxCommon::$db->getNextRecord($results, NUMERIC))) {
                $class = ($i % CLX_NUM_TWO == CLX_NUM_ZERO) ? CLX_CLASS_DATA : CLX_CLASS_DATAALT;
                $i++;
                $textarea = new htmlTextArea('', $result[CLX_NUM_7], CLX_NUM_FIVE, CLX_NUM_40);
                $textarea->setProp('readonly', CLX_NUM_ONE);
                $textarea->setProp(CLX_PROP_CLASS, CLX_CLASS_DATA);
                $table->addCells($class, array($result[CLX_NUM_ZERO], $result[CLX_NUM_ONE], $result[CLX_NUM_TWO], $result[CLX_NUM_THREE], $result[CLX_NUM_FOUR], $result[CLX_NUM_FIVE], $result[CLX_NUM_SIX], $textarea->source()));
                unset($textarea);
            }
        }
        $tableFrame->addCell($table->source(), array());
        return $tableFrame->source();
    }

// PROMPT PARAMETERS TO DO
    /**
     * Prompt for parameters to run
     * @param $scheduleCode
     * @todo Enable funtionality to allow parameters to be promped before excecution
     */
    function askScheduleParameters($scheduleCode) {
        // Create the Form
        $form = new htmlForm(CLX_PROP_ACTION, 'tool.php');
        $form->addArgument(CLX_PROP_TO, UserRequest::$request[CLX_PROP_TO]);
        $form->addArgument('ac', $ac);
        $form->addArgument('fm', '');
        $form->addArgument('confirm', '1');
        $form->addArgument('idlist', '');
        $form->addArgument('li', '');
        // Buttons Table
        $buttons = '';
        $table = new htmlTable(CLX_WIDTH_ONE);
        $table->setProps(array(CLX_PROP_CLASS => 'buttons', CLX_PROP_WIDTH => CLX_PROP_100PCT));
        if (UserRequest::$request['ac'] === 'MODIFY') {
            $buttons .= htmlButton::getButton('submit', dict('GEN_LBL_SAVE'), array('action' => $ac, 'form' => $form->getProp(CLX_PROP_ID)))->source();
        } else {
            $buttons .=htmlButton::getButton('submit', dict('GEN_LBL_CREATE'), array('action' => 'CREATE', 'form' => $form->getProp(CLX_PROP_ID)))->source();
        }
        $c = new htmlButtonCancel(null, '');
        $buttons .= $c->source();
        $table->addCell($buttons, array(CLX_PROP_CLASS => 'buttons'));
        $btable = $table->source();
        $form->addContents($btable);
        return $form;
    }

}
