<?php

/**
 * Process Steps Logs
 *
 * @package		Clientix.Tool.Process
 * @abstract		Process Engine Manager
 * @author       $Author: cramirez $
 * @copyright    Alenet, Inc.
 * @since        $Date: 2018-08-25 14:48:10 -0400 (Sat, 25 Aug 2018) $
 * @version    	$Rev: 9909 $
 *
 */
class pr_step_logs extends clx_tool {

    /**
     * List of code types
     * @var array
     */
    var $code_type = array();

    /**
     *
     * @var unknown_type
     */
    var $do_section_type = array('' => 'GEN_LBL_NONE',
        'NOT' => 'GEN_LBL_NONE',
        'DS' => 'GEN_LBL_DO_SELECT',
        'DW' => 'GEN_LBL_DO_WHILE',
        'DWH' => 'GEN_LBL_DO_WHEN',
        'DU' => 'GEN_LBL_DO_UNTIL');

    /**
     * Constructor
     */
    function __construct() {
        $this->code = 'PRSL';
        $this->name = 'Process Step Logs Manager';
        $this->system = 'Process';
        $this->version = '5.5.0.0';
        $this->set_code_type();
        $this->tablename = 'CLX_PR_STEP_LOGS';
        $this->record_name_singular = 'GEN_LBL_PR_STEP_LOG';
        $this->record_name_plural = 'GEN_LBL_PR_STEPS_LOGS';
        $this->primary_field = 'ID';
        $this->secondary_field = 'LOG_RECID';
        $this->terciary_field = 'SECTIONCODE';
        $this->seqnum_field = 'SEQNUM';
        $this->viewform = array(CLX_CLASS_TITLE => 'TOOL_PRSL_TITLE_CREATE',
            'panels' => array(
                array('type' => 'normal',
                    'entries' => array(
                        array('label' => 'GEN_LBL_CODE',
                            'fields' => array(
                                array('name' => CLX_PROP_ID,
                                    'type' => 'string',
                                    'default' => '',
                                    'size' => '16',
                                    'displaysize' => '16',
                                    'dontmodify' => '1',
                                    'validations' => array(
                                        array('type' => 'sqlobject',
                                            'required' => '1'
                                        ), array('type' => 'unique',
                                            'fieldname' => 'idcode',
                                            'tablename' => $this->tablename)
                                    )
                                )
                            )
                        ), array('label' => 'GEN_LBL_LOG_RECID',
                            'fields' => array(
                                array('name' => 'log_recid',
                                    'type' => 'lookup',
                                    'default' => '',
                                    'readonly' => false,
                                    'tool' => 'PRLO',
                                    'validations' => array(
                                        array('type' => 'sqlobject',
                                            'required' => '1'
                                        ), array('type' => 'lookup',
                                            'fieldname' => 'idcode',
                                            'tablename' => 'CLX_PR_PROCESS_LOG'
                                        )
                                    )
                                )
                            )
                        )
                    )
                ), array('type' => 'normal',
                    CLX_CLASS_TITLE => 'GEN_LBL_STEP_INFO',
                    'entries' => array(
                        array('label' => 'GEN_LBL_SECTIONCODE',
                            'fields' => array(
                                array('name' => 'sectioncode',
                                    'type' => 'lookup',
                                    'readonly' => true,
                                    'tool' => 'PRSE',
                                    'validations' => array(
                                        array('type' => 'sqlobject',
                                            'required' => '1'
                                        ), array('type' => 'lookup',
                                            'fieldname' => 'idcode',
                                            'tablename' => 'CLX_PR_SECTIONS'
                                        )
                                    )
                                )
                            )
                        ), array('label' => 'GEN_LBL_SEQNUM',
                            'fields' => array(
                                array('name' => 'seqnum',
                                    'type' => 'string',
                                    'default' => '1',
                                    'readonly' => true,
                                    'size' => '10',
                                    'displaysize' => '10'
                                )
                            )
                        ), array('label' => 'GEN_LBL_STEP_MSG',
                            'fields' => array(
                                array('name' => 'step_msg',
                                    'type' => 'text',
                                    'default' => '',
                                    'readonly' => true
                                )
                            )
                        ), array('label' => 'GEN_LBL_STEP_STATUS',
                            'fields' => array(
                                array('name' => 'step_status',
                                    'default' => 'IDLE',
                                    'type' => CLX_CLASS_LIST,
                                    'options' => $this->notify_status
                                )
                            )
                        ), array('label' => 'GEN_LBL_CODE_TYPE',
                            'fields' => array(
                                array('name' => 'code_type',
                                    'type' => CLX_CLASS_LIST,
                                    'default' => '0',
                                    'options' => $this->code_type)
                            )
                        ), array('label' => 'GEN_LBL_CODE_TEXT',
                            'fields' => array(
                                array('name' => 'code_text', 'type' => 'text', 'default' => '', 'cols' => '80', 'rows' => '20', 'validations' => array(
                                        array('type' => 'string',
                                            'required' => '1'
                                        )
                                    )
                                )
                            )
                        ), array('label' => 'GEN_LBL_DO_SECTION_TYPE',
                            'fields' => array(
                                array('name' => 'do_section_type',
                                    'type' => CLX_CLASS_LIST,
                                    'default' => '0',
                                    'options' => $this->do_section_type)
                            )
                        ), array('label' => 'GEN_LBL_DO_SECTION',
                            'fields' => array(
                                array('name' => 'do_section',
                                    'type' => 'lookup',
                                    'tool' => 'PRSE',
                                    'validations' => array(
                                        array('type' => 'sqlobject',
                                            'required' => CLX_NUM_ZERO
                                        )
                                    )
                                )
                            )
                        )
                    )
                ), array('type' => 'normal',
                    CLX_CLASS_TITLE => 'GEN_LBL_RUN_INFO',
                    'entries' => array(
                        array('label' => 'GEN_LBL_PARAMETERS_IN',
                            'fields' => array(
                                array('name' => 'parameters_in',
                                    'type' => 'text',
                                    'default' => '',
                                    'readonly' => true
                                )
                            )
                        ), array('label' => 'GEN_LBL_PARAMETERS_OUT',
                            'fields' => array(
                                array('name' => 'parameters_out',
                                    'type' => 'text',
                                    'default' => '',
                                    'readonly' => true
                                )
                            )
                        )
                    )
                ), array('type' => 'normal',
                    CLX_CLASS_TITLE => 'GEN_LBL_RECORD_INFO',
                    'entries' => array(
                        array('label' => 'GEN_LBL_SDATE',
                            'fields' => array(
                                array('name' => 'sdate',
                                    'type' => 'date',
                                    'displaysize' => '35',
                                    'validations' => array(
                                        array('type' => 'string',
                                            'required' => '0'
                                        )
                                    )
                                )
                            )
                        ), array('label' => 'GEN_LBL_EDATE',
                            'fields' => array(
                                array('name' => 'edate',
                                    'type' => 'date',
                                    'displaysize' => '35',
                                    'validations' => array(
                                        array('type' => 'string',
                                            'required' => '0'
                                        )
                                    )
                                )
                            )
                        )
                    )
                )
            )
        );
        $this->viewform[CLX_CLASS_TITLE] = 'TOOL_PRSL_TITLE_DISPLAY';
        $this->editform = $this->viewform;
        $this->toollist = array(
            'name' => $this->tablename, 
            CLX_CLASS_TITLE => $this->record_name_plural, 
            'has_modify' => false,
            'has_create' => false,
            'has_duplicate' => false,
            'fields' => array(
                array('name' => 'ID', 'type' => 'number'), 
                array('name' => 'LOG_RECID', 'type' => 'lookup', 'tool' => 'PRLO'), 
                array('name' => 'STEP_ID', 'type' => 'fieldgroup', 
                    'fields' => array(
                        array('name' => 'SECTIONCODE', 'type' => 'string', 'sufix' => 'GEN_LBL_SEQUENCE', 'prefix' => 'GEN_LBL_PR_SECTION'), 
                        array('name' => 'SEQNUM', 'type' => 'number')
                    )
                ), 
                array('name' => 'STEP_STATUS', 'type' => CLX_CLASS_LIST, 'options' => $this->notify_status),
                array('name' => 'SDATE', 'type' => 'dati'), array('name' => 'EDATE', 'type' => 'dati'))
        );
        $this->toolpanelkeys = array(array('name' => 'LOG_RECID', 'type' => 'string'));
        $this->toolpanel = $this->toollist;
    }

// End method
    /**
     * Tool Table Structure
     * @return array
     */
    function toolTableStructure() {
        $fields = array();
        $fields['ID'] = array('type' => 'int(11) unsigned', 'null' => 'NO', 'properties' => 'auto_increment', 'index' => 'PRI');
        $fields['LOG_RECID'] = array('type' => 'int(11) unsigned', 'null' => 'NO', 'default' => CLX_NUM_ZERO);
        //$fields['SECTIONCODE'] = array('type' => 'varchar(16)', 'null' => 'NO', 'default' => '' . singleQuote('N') . '', 'default_raw' => 'NA');
        $fields['SECTIONCODE'] = array('type' => 'varchar(16)', 'null' => 'YES', 'default' => '' . singleQuote('N') . '', 'default_raw' => 'NA');
        $fields['SEQNUM'] = array('type' => 'int(8) unsigned', 'null' => 'NO', 'default' => CLX_NUM_ONE);
        $fields['CODE_TYPE'] = array('type' => 'varchar(10)', 'null' => 'NO', 'properties' => '');
        $fields['CODE_TEXT'] = array('type' => 'longtext', 'null' => 'YES', 'properties' => '');
        $fields['DO_SECTION'] = array('type' => 'varchar(20)', 'null' => 'YES', 'properties' => '');
        $fields['DO_SECTION_TYPE'] = array('type' => 'varchar(10)', 'null' => 'YES', 'properties' => '');
        $fields['STEP_MSG'] = array('type' => 'mediumtext', 'null' => 'YES');
        $fields['PARAMETERS_IN'] = array('type' => 'text', 'null' => 'YES');
        $fields['PARAMETERS_OUT'] = array('type' => 'text', 'null' => 'YES');
        $fields['SDATE'] = array('type' => 'datetime', 'null' => 'YES');
        $fields['EDATE'] = array('type' => 'datetime', 'null' => 'YES');
        $fields['STEP_STATUS'] = array('type' => 'varchar(10)', 'null' => 'NO', 'default' => '' . singleQuote('IDLE') . '', 'default_raw' => 'IDLE');
        $indexes = array();
        $ipf = 'CLX' . $this->code;
        $indexes[$ipf . '01'] = array('index' => 'PRI', 'fieldname' => 'ID');
        $indexes[$ipf . '02'] = array('index' => 'MUL', 'fieldname' => 'LOG_RECID');
        return array('idcode' => $this->code, 'idname' => $this->name, 'name' => $this->tablename, 'type' => 'InnoDB',
            'fields' => $fields,
            'indexes' => $indexes);
    }

    /**
     * Tool Install Method
     */
    function install() {

    }

//-- M E S A G E S
    /**
     * Tool Action
     * @param string $ac Action code
     * @param string $idlist Record(s) affected by action
     * @return string
     */
    function action($ac, $idlist) {
        if ($this->getAccess()->canRead()) {
            $contents = parent::action($ac, $idlist);
        } else {
            $contents = warning(dict('GEN_ERR_NO_ACCESS') . ' [PR_STEP_LOGS-CAN_READ][' . $ac . '] ' . $this->getCode(false));
        }
        return $contents;
    }

    function set_code_type() {
        $EVNT = getTool('EVNT');
        $this->code_type = $EVNT->set_action_list();
    }

    function toolArchive($parameters = null) {
	if(is_null($parameters)) {
            $parameters = array();
	}
	$extra_where = '';
	$extra_from = '';
	$extra_delete = '';
	$col_prefix = '';
	if(isset($parameters['EXCLUDED'])) {
            $extra_where = explode(',', $parameters['EXCLUDED']);
            foreach($extra_where as $key => $val) {
                    $extra_where[$key] = trim($extra_where[$key]);
            }
            $extra_from = ' PRSL, CLX_PR_SECTIONS PRSE ';
            $extra_where = ' AND PRSL.SECTIONCODE = PRSE.IDCODE AND PRSE.PROCESSCODE NOT IN (\'' . implode('\',\'', $extra_where) . '\')';
            $extra_delete = ' PRSL.*';
            $col_prefix = 'PRSL.';
	}
        if (!defined('UTIL_ARCHIVE_MODE'))
            define('UTIL_ARCHIVE_MODE', CLX_NUM_12);
        //Archive
        $sql = 'SELECT * FROM ' . $this->tablename;
        if (UTIL_ARCHIVE_MODE == CLX_NUM_ONE) {
            $archive_tbl = $this->tablename . '_' . getNow('m') . '_' . getNow('Y');
        } else {
            $archive_tbl = $this->tablename . '_' . getNow('Y');
        }
        $tableinfo = $this->toolTableStructure();
        unset($tableinfo['fields']['ID']);
        $tableinfo['name'] = $archive_tbl;
        $tableinfo['type'] = 'ARCHIVE';
        $tableinfo['indexes'] = array();
        $structure = ClxCommon::$db->translateTableStructure($tableinfo);
        //Create table if not exists
        if (!ClxCommon::$db->tableExists($structure['name'])) {
            //Create table if not exists
            ClxCommon::$db->createTable($structure['name'], $structure['type'], $structure['fields'], $structure['case_sensitive']);
        }
        $comma = '';
        $select = '';
	$select_w_prefix = '';
        foreach ($structure['fields'] as $key => $val) {
            $select .= $comma . $key;
            $select_w_prefix .= $comma . $col_prefix . $key;
            $comma = ',';
        }
        //Lock table to archive
        $sql = 'SELECT MAX(ID) FROM ' . $this->tablename;
        $maxid = ClxCommon::$db->getOne($sql);
        if (empty($maxid))
            return;
        //Backup
        $sql = 'INSERT INTO ' . $structure['name'] . '(' . $select . ') SELECT '. $select_w_prefix . ' FROM ' . $this->tablename . $extra_from.' WHERE ' . $col_prefix . 'ID <=' . $maxid . $extra_where;
        ClxCommon::$db->executeQuery($sql);
        //Unlock table
        $sql = 'DELETE ' . $extra_delete . ' FROM ' . $this->tablename . $extra_from . ' WHERE ' . $col_prefix . 'ID<=' . $maxid . $extra_where;
        ClxCommon::$db->executeQuery($sql);
        //Restore stats if apply
        ClxCommon::$db->buldStats($this->tablename);
    }

    function toolArchive_batch($parameters) {
        $contenidoHTML = '';
        //e cho 'TOOLARCHIVE_BATCH parameters:<br>'.CLX_CHAR_NL; print_r($parameters);
        //   - Caso 1002598 - BNP-Limpieza Proccess Logs en Batch
        // Method 'get_bkp_time_limit' implemented to avoid schedule CLX_PRLO_CLEAN
        // to remain un 'In progress(RUN) status due to PHP timeout.
        $DBBK = getToolObject('UTIL', 'DBBK');
        $bkp_time_limit = $DBBK->get_bkp_time_limit();
        // Set it to twice the time limit of backup, just in case
        UserResponse::setRunTimeLimit($bkp_time_limit * CLX_NUM_TWO * CLX_NUM_60 * CLX_NUM_60);
        //   - Caso 1002598 - BNP-Limpieza Audit en Batch
        // Se agrega este método para realizar el archival en batch,
        // ya que en los casos que CLX_PR_PROC_LOGS tenga muchos registros,
        // se cae cuando se hace en el interactivo.
        // Realiza la llamada al método $this->toolArchive()
	$this->toolArchive($parameters);
        // Extrae la lista de las tablas con los archives existentes
        //   - Caso 1002670 - BNP-Limpieza Proc Log en batch
        // Fix size of SUBSTR with calculated strlen($this->tablename)+1
        // Substitutes table name prefix with $this->tablename
        /*
         * Para saber los nombres de los Archives de PROC_LOGS en la base de datos corriente
         */
        $sql = 'SELECT TABLE_NAME FROM INFORMATION_SCHEMA.TABLES WHERE SUBSTR(TABLE_NAME, 1, ' . (strlen($this->tablename) + CLX_NUM_ONE) . ') = ' . singleQuote($this->tablename . '_') . ' AND TABLE_SCHEMA = ' . singleQuote(kDatabaseName);
        $results = ClxCommon::$db->executeQuery($sql);
        $tables = array();
        while (($rec = ClxCommon::$db->getNextRecord($results, ASSOCIATIVE))) {
            $tables[] = $rec['TABLE_NAME'];
        }
        // Verifica su hay archives
        if (count($tables) > CLX_NUM_ZERO) {
            // List of archive tables to backup (separated by '/')
            $parameters['TABLES'] = implode('/', $tables);
            //   - Caso 1002670 - BNP-Limpieza Proc Log en batch
            // Get ACAD tool to make ACAD_ARC_NAME_SUFFIX constant available
            // Eventually adds a suffix to backup file name
            // to avoid Recycler to delete those bkp files
            // Le agrega lo necesario para que el respaldo se conserve del reciclaje
            if (defined('ACAD_ARC_NAME_SUFFIX')) {
                $archive_name_suffix = ACAD_ARC_NAME_SUFFIX;
            } else {
                $archive_name_suffix = '';
            }
            if ($archive_name_suffix === '' || $archive_name_suffix === 'null' || is_null($archive_name_suffix)) {
                $archive_name_suffix = '';
            }
            // Backup file name
            $parameters['NAME_SUFFIX'] = '_' . $this->tablename . $archive_name_suffix;
            // No Drop Table option
            $parameters['DUMP_OPTIONS'] = '--skip-add-drop-table';
            // Local Backup Script optional parameters: Don't stop minute-by-minute process
            $parameters['LBS_OPT_PAR'] = '-ncss 0';
            $contenidoHTML.= '<br>' . 'Calling backup procedure<br>' . 'Tables: ' . str_replace('/', ', ', $parameters['TABLES']) . '<br>' . 'File name suffix: ' . $parameters['NAME_SUFFIX'] . '<br>';
            // Llama al respaldo pasandole como parametro los nombres de los archives
            $results = $DBBK->run($parameters);
            if ($results === '') {
                // Si el respaldo fue existoso, elimina los archives de la BDD
                $contenidoHTML.= '<br>';
                foreach ($tables as $key => $val) {
                    $contenidoHTML.= 'Dropping Table: ' . $val . '<br>';
                    $sql = 'DROP TABLE IF EXISTS ' . $val;
                    ClxCommon::$db->executeQuery($sql);
                }
            } else {
                $contenidoHTML.= '<br>' . '<br>' . 'ERROR: excecuting backup<br>' . $results . '<br>';
            }
        }
        echo $contenidoHTML;
    }

}