<?php

/**
 * Tool Class
 * For more information contact  support@alenet.com
 *
 * @author       $Author: alenetdev $
 * @copyright    Alenet, Inc.
 * @since        $Date: 2019-01-22 15:45:22 -0400 (ma., 22 ene. 2019) $
 * @version        $Rev: 10496 $
 *
 * CHANGE LOG
 *  Oracle uppercase record changed
 *  Included label
 */
class sd_files extends clx_tool {

    var $importform = array();

    function __construct() {
        $this->code = 'SDFL';
        $this->name = 'File Manager';
        $this->system = 'System Data';
        $this->version = '0.2.0.0';

        $this->has_access = true;


        $this->tablename = 'CLX_SD_FILES';
        $this->record_name_singular = 'File';
        $this->record_name_plural = 'Files';
        $this->primary_field = 'ID';
        $this->secondary_field = 'FILENAME';

        // not used!
        $this->editform = array(CLX_CLASS_TITLE => 'TOOL_SDFL_TITLE_CREATE',
            'panels' => array(
                array('type' => 'normal',
                    'entries' => array(
                        array('label' => 'GEN_LBL_CODE',
                            'fields' => array(
                                array('name' => 'idcode', 'type' => 'string', 'default' => '', 'size' => '64', 'displaysize' => '32', 'dontmodify' => '1',
                                    'validations' => array(
                                        array('type' => 'sqlobject', 'required' => '1'), array('type' => 'unique', 'fieldname' => 'idcode', 'tablename' => $this->tablename))
                                )
                            )
                        ), array('label' => 'GEN_LBL_NAME',
                            'fields' => array(
                                array('name' => 'idname', 'type' => 'string', 'default' => '', 'size' => '64', 'displaysize' => '48', 'validations' => array(
                                        array('type' => 'string', 'required' => '1'))
                                )
                            )
                        )
                    )
                )
            )
        );
        // not used!
        $this->importForm = array(CLX_CLASS_TITLE => 'TOOL_SDFL_TITLE_IMPORT',
            'panels' => array(
                array('type' => 'normal',
                    'entries' => array(
                        array('label' => 'GEN_LBL_FILE',
                            'fields' => array(
                                array('name' => 'filename', 'type' => 'file', 'validations' => array(
                                        array('type' => 'string', 'required' => '1'))
                                )
                            )
                        ), array('label' => 'GEN_LBL_NAME',
                            'fields' => array(
                                array('name' => 'idname', 'type' => 'string', 'default' => '', 'size' => '64', 'displaysize' => '48', 'validations' => array(
                                        array('type' => 'string', 'required' => '1'))
                                )
                            )
                        )
                    )
                )
            )
        );

        //Directory load
        $this->importDirForm = array(CLX_CLASS_TITLE => 'TOOL_SDFL_TITLE_IMPORT_DIR',
            'panels' => array(
                array('type' => 'normal',
                    'entries' => array(
                        array('label' => 'TOOL_PACK_PROMPT_DIRONSERVER',
                            'fields' => array(
                                array('name' => 'localdir', 'type' => 'string', 'default' => '', 'size' => '255', 'displaysize' => '48', 'readonly' => CLX_NUM_ONE, 'validations' => array(
                                        array('type' => 'string', 'required' => '0'))
                                )
                            )
                        ), array('label' => 'GEN_LBL_TGT_TABLECODE',
                            'fields' => array(
                                array('name' => 'tablecode', 'type' => 'lookup', 'default' => '', 'tool' => 'DBTB', 'validations' => array(
                                        array('type' => 'string', 'required' => '1'), array('type' => 'lookup', 'fieldname' => 'IDCODE', 'tablename' => 'CLX_DB_TABLES'))
                                )
                            )
                        ), array('label' => 'GEN_LBL_TGT_IMGFIELDNAME',
                            'fields' => array(
                                array('name' => 'fieldname', 'type' => 'lookup', 'tool' => 'DBTF', 'tablecode' => 'SDFL_tablecode', 'validations' => array(
                                        array('type' => 'sqlobject', 'required' => '1'), array('type' => 'query', 'msg' => 'SDFL_MUST_BE_FILE_FIELD', 'query' => 'SELECT X.FIELDNAME SDFL_FIELDNAME FROM CLX_DB_TABLEFIELDS X, CLX_DB_FIELDS F WHERE X.TABLECODE=' . singleQuote('SDFL_tablecode') . ' AND X.FIELDNAME=' . singleQuote('SDFL_fieldname') . ' AND X.FIELDNAME = F.FIELDNAME AND F.FIELDTYPE = ' . singleQuote('FILE'))))
                            )
                        ), array('label' => 'GEN_LBL_FORM',
                            'fields' => array(
                                array('name' => 'formcode', 'type' => 'lookup', 'tool' => 'UIFM', 'tablecode' => 'SDFL_tablecode', 'validations' => array(
                                        array('type' => 'sqlobject', 'required' => '1'), array('type' => 'query', 'msg' => 'SDFL_MUST_FORM_ON_TABLE', 'query' => 'SELECT F.IDCODE SDFL_FORMCODE FROM CLX_UI_FORMS F WHERE F.IDCODE = ' . singleQuote('SDFL_formcode') . ' AND F.TABLECODE=' . singleQuote('SDFL_tablecode'))))
                            )
                        )
                    )
                )
            )
        );
    }

    function action($ac, $idlist) {
        $contents = '';
        if ($this->getAccess()->canRead()) {
            $msg = array();
            switch ($ac) {
                case 'LOAD_DIR':
                    if ($this->getAccess()->canDisplay()) {
                        $form = new clx_form($this->code, $this->importDirForm);
                        if (isset(UserRequest::$post['confirm'])) {
                            if ($form->checkForm1(UserRequest::$post['ac'], UserRequest::$post, $msg)) {
                                $contents = $this->action('LOAD_DIR_FASE2', $idlist);
                            } else {
                                $contents = $form->editForm($ac, $idlist, UserRequest::$post, $msg);
                            }
                        } else {
                            UserRequest::$post['SDFL_localdir'] = kPathToImport;
                            $contents = $form->editForm($ac, $idlist, UserRequest::$post, $msg);
                        }
                    } else {
                        $contents = warning(dict('GEN_ERR_NO_ACCESS') . ' [SD_FILES-CAN_APPEND_1][' . $ac . '] ' . $this->getCode(false));
                    }
                    break;
                case 'LOAD_DIR_FASE2':
                    if ($this->getAccess()->canDisplay()) {
                        $contents = $this->importDirectoryForm();
                    } else {
                        $contents = warning(dict('GEN_ERR_NO_ACCESS') . ' [SD_FILES-CAN_APPEND_2][' . $ac . '] ' . $this->getCode(false));
                    }
                    break;
                case 'SHOW_FILE':
                    $contents = $this->showFileCall(UserRequest::$request['ff']);
                    break;
                case 'CLEAN_FILES':
                    $contents = $this->removeMissingFiles();
                    break;
                default:
                    $contents = parent::action($ac, $idlist);
                    break;
            }
        } else {
            $contents = warning(dict('GEN_ERR_NO_ACCESS') . ' [SD_FILES-CAN_READ][' . $ac . '] ' . $this->getCode(false));
        }
        return $contents;
    }

    function install() {
        $UIMI = getTool('UIMI');
        $UIMI->add($this->code, 'MENU_TOOLS:MENU_SYSTEMDATA:MENU_FILES', 'TOOL', $this->code, 'ac=LIST');
    }

    function toolTableStructure() {
        $fields = array();
        $fields['ID'] = array('type' => 'int(11) unsigned', 'null' => 'NO', 'properties' => 'auto_increment', 'index' => 'PRI');
        $fields['NAME'] = array('type' => 'varchar(255)', 'null' => 'NO', 'properties' => '');
        $fields['FILENAME'] = array('type' => 'varchar(255)', 'properties' => '');
        $fields['TYPE'] = array('type' => 'varchar(100)', 'null' => 'NO', 'properties' => '');
        $fields['SIZES'] = array('type' => 'int(11) unsigned', 'null' => 'NO', 'properties' => '', 'default' => CLX_NUM_ZERO);
        $fields['CDATE'] = array('type' => 'datetime', 'null' => 'YES', 'properties' => '');
        $fields['USERNAME'] = array('type' => 'varchar(30)', 'null' => 'NO', 'properties' => '');
        $fields['CONTENTS'] = array('type' => 'longblob', 'properties' => '');
        $fields['DESCRIPTION'] = array('type' => 'varchar(255)', 'properties' => '');
        $fields['ENCODING'] = array('type' => 'varchar(255)', 'properties' => '');
        $indexes = array();
        $ipf = 'CLX' . $this->code;
        $indexes[$ipf . '01'] = array('index' => 'PRI', 'fieldname' => 'ID');
        $indexes[$ipf . '03'] = array('index' => 'MUL', 'fieldname' => 'NAME');
        $indexes[$ipf . '04'] = array('index' => 'MUL', 'fieldname' => 'TYPE');
        return array('idcode' => $this->code, 'idname' => $this->name, 'name' => $this->tablename, 'type' => 'InnoDB',
            'fields' => $fields,
            'indexes' => $indexes);
    }

//-- F I L E S
    function makelist($listingcode = null) {
	ClxFreeReturn($listingcode);
        $t = time();
        $to = $this->code;
        $here = 'tool.php?to=' . $to . '&t=' . $t;
        clearHistory();
        pushHistory($this->code, 'LIST', array());
        $form = new htmlForm(CLX_PROP_ACTION, 'tool.php');
        $form->setProp(CLX_PROP_ID, 'SDFLList');
        $form->addArgument(CLX_PROP_TO, $this->code);
        $form->addArgument('ac', 'NOT_DEFINED');
        $selectclause = '*';
        $fromclause = 'CLX_SD_FILES';
        $whereclause = '';
        //$sortclause = 'NAME';
        $sortclause = 'CDATE DESC';
        $page = new xPaging();
        $page->pageInit($form->getProp(CLX_PROP_ID), $here, $selectclause, $fromclause, $whereclause, $sortclause, '');
        $table = new htmlTable(CLX_WIDTH_SIX);
        $table->setProps(array(CLX_PROP_CLASS => CLX_CLASS_LIST, CLX_PROP_WIDTH => CLX_PROP_100PCT));
        $table->addCell(dict('TOOL_SDFL_TITLE_LIST'), array(CLX_PROP_CLASS => CLX_CLASS_TITLE, CLX_PROP_COLSPAN => CLX_WIDTH_SIX));
        $page->header($table);
        $table->addCells(CLX_CLASS_HEADERS, array(dict('GEN_LBL_SEL'), dict('GEN_LBL_NAME'), dict('GEN_LBL_TYPE'), dict('GEN_LBL_SIZE'), dict('GEN_LBL_CDATE'), dict('GEN_LBL_FILENAME')));
        $i = CLX_INIT_ZERO;
        while ($record = $page->nextRecord()) {
            $recordid = $record['ID'];
            $select = new htmlCheckbox('selector_' . $recordid, false);
            $select->setProp('onclick', 'selectButtons(' . singleQuote($form->getProp(CLX_PROP_ID)) . ');');
            $table->addCell($select->source(), array(CLX_PROP_CLASS => CLX_CLASS_FOOTERS, CLX_PROP_WIDTH => '40', CLX_PROP_ALIGN => 'center'));
            
            $url = 'window.php?to=SDFL&ac=SHOW_FILE&no_menu=1&no_alerts=1&ff='.  $record['ID'];
            //E.C se cambio para que fuera compatible con rutas del ambiente o externas 
            //$link = new htmlAnchor($record['FILENAME'], 'javascript: openWindow(' . singleQuote('fi' . $record['ID'] ) . ', ' . singleQuote($record['FILENAME']) . ', 400, 400);');
            //$link = new htmlAnchor($record['FILENAME'], $url);
            $link = new htmlAnchor($record['FILENAME'], 'javascript: openWindow(' . singleQuote('fi' . $record['ID'] ) . ' , ' . singleQuote($url)  . ',800,600);');
            $table->addCells(($i % CLX_NUM_TWO == CLX_NUM_ZERO) ? CLX_CLASS_DATA : CLX_CLASS_DATAALT, array($record['NAME'], $record['TYPE'], xFormat::filesize($record['SIZES']), xFormat::date($record['CDATE'],$_SESSION['prefs']['DATE_FORMAT']), $link->source()));
            $i++;
        }
        $buttons = '';
        $buttons .= htmlButton::getButton('submit', dict('GEN_LBL_DISPLAY'), array('action' => 'DISPLAY', 'form' => $form->getProp(CLX_PROP_ID), CLX_PROP_ID => 'MULT'))->source();
        if ($this->getAccess()->canDelete()) {
            $buttons .=htmlButton::getButton('submit', dict('GEN_LBL_DELETE'), array('action' => 'DELETE', 'form' => $form->getProp(CLX_PROP_ID), CLX_PROP_ID => 'MULT'))->source();
        }
        if ($this->getAccess()->canDisplay()) {
            $b = new htmlFormSubmit(dict('GEN_LBL_LOAD_DIR'), 'LOAD_DIR', $form->getProp(CLX_PROP_ID));
            $b->setProp(CLX_PROP_ID, 'ON');
            $buttons .= $b->source();
        }
        $buttons .=htmlButton::getButton('submit', dict('GEN_LBL_FILTER'), array('action' => 'FILTER', 'form' => $form->getProp(CLX_PROP_ID), CLX_PROP_ID => 'ON'))->source();
        $form->addContents($page->footers($table, $buttons));
        return $form->source();
    }

    function confirmDeleteFill(&$table, $record) {
        $table->addData(dict('GEN_LBL_NAME') . ':', $record['SDFL_name']);
        $table->addData(dict('GEN_LBL_TYPE') . ':', $record['SDFL_type']);
    }

    function viewFormFill($record) {
        $contents = '';
        $table = new htmlTable(CLX_WIDTH_TWO);
        $table->setProps(array(CLX_PROP_CLASS => CLX_CLASS_FORM, CLX_PROP_WIDTH => CLX_PROP_100PCT));
        $table->addCell(dict('TOOL_SDFL_TITLE_VIEW') . ': ' . $record['SDFL_name'], array(CLX_PROP_CLASS => CLX_CLASS_TITLE, CLX_PROP_COLSPAN => CLX_WIDTH_TWO));
        $table->addData(dict('GEN_LBL_NAME') . ':', $record['SDFL_name']);
        $table->addData(dict('GEN_LBL_SIZE') . ':', xFormat::filesize($record['SDFL_size']));
        $table->addData(dict('GEN_LBL_TYPE') . ':', $record['SDFL_type']);
        $table->addData(dict('GEN_LBL_FILENAME') . ':', $record['SDFL_filename']);
        $table->addData(dict('GEN_LBL_DESCRIPTION') . ':', $record['SDFL_description']);
        $table->addData(dict('GEN_LBL_ENCODING') . ':', $record['SDFL_encoding']);
        $contents .= $table->source();
        return $contents;
    }

    function filterFormFill($msg, &$table) {
        if (!array_key_exists('SDFL_name', UserRequest::$post))
            UserRequest::$post['SDFL_name'] = '';
        if (!array_key_exists('SDFL_filename', UserRequest::$post))
            UserRequest::$post['SDFL_filename'] = '';
        $table->addField($msg, dict('GEN_LBL_NAME'), 'SDFL_name', UserRequest::$post['SDFL_name'], CLX_NUM_32, CLX_NUM_64);
        $table->addField($msg, dict('GEN_LBL_FILENAME'), 'SDFL_filename', UserRequest::$post['SDFL_filename'], CLX_NUM_32, CLX_NUM_64);
        //  cjrd
        if (!array_key_exists('SDFL_type', UserRequest::$post))
            UserRequest::$post['SDFL_type'] = '';
        if (!array_key_exists('SDFL_cdate', UserRequest::$post))
            UserRequest::$post['SDFL_cdate'] = '';
        if (!array_key_exists('SDFL_filename', UserRequest::$post))
            UserRequest::$post['SDFL_filename'] = '';
        $table->addField($msg, dict('GEN_LBL_TYPE'), 'SDFL_type', UserRequest::$post['SDFL_type'], CLX_NUM_32, CLX_NUM_64);
        $table->addField($msg, dict('GEN_LBL_CDATE'), 'SDFL_cdate', UserRequest::$post['SDFL_cdate'], CLX_NUM_32, CLX_NUM_64);
        $table->addField($msg, dict('GEN_LBL_FILENAME'), 'SDFL_filename', UserRequest::$post['SDFL_filename'], CLX_NUM_32, CLX_NUM_64);
    }

    function setfilter() {
        $filter = '';
        $boolean = '';
        if (!empty(UserRequest::$post['SDFL_name'])) {
            $filter .= $boolean . ' NAME LIKE ' . singleQuote(UserRequest::$post['SDFL_name']);
            $boolean = 'AND';
        }
        if (!empty(UserRequest::$post['SDFL_filename'])) {
            $filter .= $boolean . ' FILENAME LIKE ' . singleQuote(UserRequest::$post['SDFL_filename']);
            $boolean = 'AND';
        }
        //  cjrd
        if (!empty(UserRequest::$post['SDFL_type'])) {
            $filter .= $boolean . ' TYPE LIKE ' . singleQuote(UserRequest::$post['SDFL_type'] . '%');
            $boolean = 'AND';
        }
        if (!empty(UserRequest::$post['SDFL_cdate'])) {
            $filter .= $boolean . ' CDATE LIKE ' . singleQuote(UserRequest::$post['SDFL_cdate'] . '%');
            $boolean = 'AND';
        }
        if (!empty(UserRequest::$post['SDFL_filename'])) {
            $filter .= $boolean . ' FILENAME LIKE ' . singleQuote(UserRequest::$post['SDFL_filename'] . '%');
            $boolean = 'AND';
        }
        $_SESSION['paging']['SDFLList']['criteria'] = $filter;
        $_SESSION['paging']['SDFLList']['startat'] = '0';
    }
	///////////////////////////////////////////////////
	///comentado 22/01/2019 para utilizar la clx_tool
	//////////////////////////////////////////////////
    /*function clearfilter() {
        $_SESSION['paging']['SDFLList']['criteria'] = '';
    }*/

    function loadToolRecord($recordid) {
        $values = ClxCommon::$db->getRecord($this->tablename, 'ID=' . singleQuote($recordid));
        $record = array();
        $record['SDFL_id'] = $recordid;
        $record['SDFL_name'] = $values['NAME'];
        $record['SDFL_size'] = $values['SIZES'];
        $record['SDFL_type'] = $values['TYPE'];
        $record['SDFL_filename'] = $values['FILENAME'];
        $record['SDFL_description'] = $values['DESCRIPTION'];
        $record['SDFL_encoding'] = $values['ENCODING'];
        return $record;
    }

    function deleteRecordSk($id) {

        $filerec = ClxCommon::$db->getRecord($this->tablename, 'ID=' . singleQuote($id));

        $sdfl_clx_root_sd_files = (SDFL_CLX_ROOT_SD_FILES!='' ? trim(SDFL_CLX_ROOT_SD_FILES) : 'null');
        $kPathToRoot='';
        if ($sdfl_clx_root_sd_files!='null' && $sdfl_clx_root_sd_files!=''){
            //le asignamos la ruta alterna definida en el parametro
            $kPathToRoot = $sdfl_clx_root_sd_files;
        }else{
            $kPathToRoot = '';
        }

        if (!empty($filerec['FILENAME']) && !$this->isRemoteFile($filerec['FILENAME']) && file_exists($kPathToRoot.$filerec['FILENAME']) && is_writable($kPathToRoot.$filerec['FILENAME'])) {
            //E.C AJUSTES PARA QUE LOS ARCHIVOS ATTACHMENT SE PUEDAN BORRAR EN UNA RUTA DISTINTAS A LA POR DEFECTO 
            unlink($kPathToRoot.$filerec['FILENAME']);
        }
        ClxCommon::$db->deleteRecords($this->tablename, 'ID=' . singleQuote($id));
    }

    /**
     * Performs file import
     * @return string
     */
    function importDirectory() {
        $contents = '';
        $pathname = kPathToWebSite . UserRequest::$post['SDFL_localdir'];
        if (($dir = opendir($pathname)) === true) {
            $contents .= '<b>' . dict('GEN_LBL_IMPORT_RESULTS') . '</b>';
            $status = true;
            $EVNT = &getTool('EVNT');
            $DBFL = &getTool('DBFL');
            $DBTB = &getTool('DBTB');
            $tablerec = $DBTB->getTable(UserRequest::$post['SDFL_tablecode']);
            $tgttablerec = $DBTB->tableDefaultRecord(UserRequest::$post['SDFL_tablecode']);
            $fieldrec = $DBFL->getField(UserRequest::$post['SDFL_fieldname']);
            //[tc] => TEST
            foreach ($tgttablerec as $key => $val) {
                if (array_key_exists(UserRequest::$post['tc'] . '_' . $key, UserRequest::$post)) {
                    if (is_array(UserRequest::$post[UserRequest::$post['tc'] . '_' . $key])) {
                        $tgttablerec[$key] = implode(',', UserRequest::$post[UserRequest::$post['tc'] . '_' . $key]);
                    } else {
                        $tgttablerec[$key] = UserRequest::$post[UserRequest::$post['tc'] . '_' . $key];
                    }
                }
            }
            //Load Table Information
            $maxfilesize = $fieldrec['FIELD_MAXVALUE']; // size in megabytes => 2
            $groupcode = $fieldrec['FIELD_EXTRA1'];
            $storein = $fieldrec['FIELD_EXTRA2'];
            $savepath = $fieldrec['FIELD_EXTRA3'];
            //perform some file checks before loading. all or nothing
            $i = CLX_INIT_ZERO;
            while (($file = readdir($dir)) !== false) {
                //No php can be loaded
                if (is_dir($pathname . $file) || $file === '.' || $file === '..' || stristr($file, '.php') || stristr($file, '.clxdone'))
                    continue;
                $contents .= '<li>' . $file . ' ';
                $fl = new clx_file($pathname . $file, 'r');
                //e cho $fl->size().' ' . $fl->getMimeType().' ' . $fl->getFileExtension().'<Br>';
                //check size
                $filesize = ($fl->size() / CLX_NUM_1024) / CLX_NUM_1024; // we need this value in MB
                if ($filesize > $maxfilesize) {
                    $contents .= ' - ' . dict('GEN_ERR_FILETOOBIG') . ' Cur:' . number_format($filesize, CLX_NUM_TWO) . ' Mb, Max:' . $maxfilesize;
                    $status = false;
                }
                //Check Mime
                if (strlen($groupcode) > CLX_NUM_ZERO) {
                    if (ClxCommon::$db->oneExistsRecords('CLX_SD_MIMEGROUP_MEMBERS', 'GROUPCODE = ' . singleQuote($groupcode) . ' AND MIMECODE = ' . singleQuote($fl->getMimeType())) == CLX_NUM_ZERO) {
                        $contents .= ' - ' . dict('GEN_ERR_FILENOTHERGHTFMT', $fl->getMimeType());
                        $status = false;
                    }
                }
                $fl->close();
                $contents .= (($status) ? ' - Ok ' : ' - Fail') . '</li>';
                $i++;
            }
            if ($i == CLX_NUM_ZERO) {
                $status = false;
                $contents .= '<br>' . dict('GEN_LBL_NOTHINGTODO');
            }
            if ($status === true) {
                //pre($tgttablerec);
                rewinddir($dir);
                while (($file = readdir($dir)) !== false) {
                    //No php can be loaded
                    if (is_dir($pathname . $file) || $file === '.' || $file === '..' || stristr($file, '.php') || stristr($file, '.clxdone'))
                        continue;
                    $fl = new clx_file($pathname . $file, 'r');
                    $fl->open();
                    //Load File on record
                    $mime = $fl->getMimeType();
                    $flcontents = $fl->read();
                    $fileid = $this->saveFile($file, $mime, $flcontents, $storein, $savepath);
                    $fl->close();
                    //Executing Events
                    $msg = array();
                    $record = $tgttablerec;
                    $record[UserRequest::$post['SDFL_fieldname']] = $fileid;
                    $EVNT->execute5('DBTB', $tablerec['ID'], 'CREP', $record, $msg); //  Create Record (pre database commit)
                    ClxCommon::$db->createRecord($tablerec['MODULECODE'] . '_' . $tablerec['IDNAME'], $record, false);
                    $EVNT->execute5('DBTB', $tablerec['ID'], 'CREC', $record, $msg); //  Create Record (post database commit)
                    //Rename file
                    $fl->rename($pathname . $file, $pathname . $file . '.clxdone');
                }
            }
            closedir($dir);
            $contents = note($contents);
        } else {
            $contents = warning(dict('CODE_ERR_FILE_INVALIDDIR', kPathToImport . UserRequest::$post['SDFL_localdir']));
        }
        return $contents;
    }

    function importDirectoryForm() {
        $contents = '';
        $UIFM = &getTool('UIFM');
        $DBTB = &getTool('DBTB');
        $EDIT = &getTool('EDIT');
        $formrec = ClxCommon::$db->getRecord('CLX_UI_FORMS', 'IDCODE=' . singleQuote(UserRequest::$request['SDFL_formcode']));
        $tablerec = $DBTB->getTable(UserRequest::$request['SDFL_tablecode']);
        $msg = array();
        if (UserRequest::$request['ac'] <> 'LOAD_DIR_FASE2') {
            UserRequest::$post = $EDIT->defaultRecord($tablerec, UserRequest::$request['SDFL_formcode']);
        } else {
            //Validate to add errors on forms if found. Messages will be added at the end
            $UIFM->userCheckForm($formrec, UserRequest::$post, $msg);
        }
        // The Search Form
        $form = new htmlForm(CLX_PROP_ACTION, UserRequest::$server['PHP_SELF']);
        $form->addArgument(CLX_PROP_TO, $this->code);
        $form->addArgument('ac', 'LOAD_DIR_FASE2');
        $form->addArgument('confirm', '1');
        $form->addArgument('confirm_fase2', '1');
        $form->addArgument('tc', $tablerec['IDCODE']);
        $form->addArgument('SDFL_formcode', UserRequest::$request['SDFL_formcode']);
        $form->addArgument('SDFL_tablecode', UserRequest::$request['SDFL_tablecode']);
        $form->addArgument('SDFL_fieldname', UserRequest::$request['SDFL_fieldname']);
        $form->addArgument('SDFL_localdir', UserRequest::$request['SDFL_localdir']);
        $form->addArgument('fid', '');
        $form->addArgument('rf', ''); //Related Field Value
        $form->addArgument('rfi', ''); //Related Formfield Value
        // Buttons
        $table = new htmlTable(CLX_WIDTH_TWO);
        $table->setProps(array(CLX_PROP_CLASS => 'buttons', CLX_PROP_WIDTH => CLX_PROP_100PCT));
        $buttons = '';
        $buttons .=htmlButton::getButton('submit', dict('GEN_LBL_IMPORT'), array('action' => 'LOAD_DIR_FASE2', 'form' => $form->getProp(CLX_PROP_ID)))->source();
        $table->addCell($buttons, array(CLX_PROP_CLASS => 'buttons', CLX_PROP_COLSPAN => CLX_WIDTH_TWO));
        $buttons = $table->source();
        $title = dict($tablerec['RECORDNAME_PLURAL']);
        $form->addContents($UIFM->userEditForm($formrec, $title, UserRequest::$post, $msg, $form));
        $form->addContents($buttons);
        $contents .= $form->source();
        if (UserRequest::$request['ac'] === 'LOAD_DIR_FASE2' && empty($msg)) {//means no error show import
            $contents .= '<br>' . $this->importDirectory();
        }
        return $contents;
    }

    function saveFile($name, $type, $contents, $saveindatabase = true, $path = '', $encoding = '', $description = '') {
        if (strlen($name) == CLX_NUM_ZERO && strlen($type) == CLX_NUM_ZERO && strlen($contents) == CLX_NUM_ZERO)
            return CLX_NUM_ZERO;
        $cdate = xFormat::getNow('Y-m-d H:i:s');
        $username = $_SESSION['user_record']['username'];
        $mimeexists = (ClxCommon::$db->oneExistsRecords('CLX_SD_MIMETYPES', 'IDCODE=' . singleQuote($type)) > CLX_NUM_ZERO);
        if ($mimeexists === true) {
            $mimerec = ClxCommon::$db->getRecord('CLX_SD_MIMETYPES', 'IDCODE=' . singleQuote($type));
        } else {
            $mimerec = array();
            $mimerec['EXTENSION'] = '';
        }
        $filerecord = array();
        $filerecord['NAME'] = $name;
        $filerecord['TYPE'] = $type;
        $filerecord['SIZES'] = strlen($contents);
        $filerecord['CDATE'] = $cdate;
        $filerecord['USERNAME'] = $username;
        $filerecord['ENCODING'] = $encoding;
        $filerecord['DESCRIPTION'] = $description;
        if ($saveindatabase == CLX_NUM_TWO) {
            $filerecord['FILENAME'] = $name;
            $filerecord['CONTENTS'] = 'null';
        } elseif ($saveindatabase == CLX_NUM_ONE) {
            $filerecord['FILENAME'] = 'null';
            $filerecord['CONTENTS'] = addslashes($contents);
        } else {
            //$u = time();error puede juntar 2 archivos.

            $u = str_replace('.', '', microtime(true));
            
            //E.C ajuste para que el sistema admita una ruta de archivos adjuntos distinta a la por defecto
            $sdfl_clx_root_sd_files = (SDFL_CLX_ROOT_SD_FILES!='' ? trim(SDFL_CLX_ROOT_SD_FILES) : 'null');
            $kPathToRoot='';
            if ($sdfl_clx_root_sd_files!='null' && $sdfl_clx_root_sd_files!=''){
                //le asignamos la ruta alterna definida en el parametro
                $kPathToRoot = $sdfl_clx_root_sd_files;
            }else{
                $kPathToRoot = '';
            }

            $filename = $path . $u . '.' . $mimerec['EXTENSION'];
            $fd = new clx_file($kPathToRoot.$filename, 'w');
            $fd->open();
            if (!$fd->hasError()) {
                $fd->write($contents);
            } else {
                addMessage(dict('CLX_ERR_FILE_PERMIT'));
            }
            $fd->close();

            $filerecord['FILENAME'] = $filename;
            $filerecord['CONTENTS'] = 'null';
        }
        $fileid = ClxCommon::$db->createRecord($this->tablename, $filerecord, false);
        return $fileid;
    }

    function isRemoteFile($filename) {
        return (strpos(lowercase($filename), 'http') !== false) ? true : false;
    }

    function removeMissingFiles() {
        $contents = '<p class=CLX_CLASS_TITLE>' . dict('GEN_LBL_CLEAN_FIELD') . '</p>';
        $sql = 'SELECT ID, FILENAME FROM ' . $this->tablename . ' WHERE CONTENTS IS NULL AND FILENAME IS NOT null ORDER BY ID ASC';
        $rs = ClxCommon::$db->executeQuery($sql);
        while (($r = ClxCommon::$db->getNextRecord($rs, NUMERIC))) {
            if ($this->isRemoteFile($r[CLX_NUM_ONE]))
                continue;
            $filename = kPathToWebSite . $r[CLX_NUM_ONE];
            if (!file_exists($filename)) {
                $contents .= dict('GEN_ERR_FILE_NOT_FOUND') . ': ' . $r[CLX_NUM_ONE] . '<br>';
                $sql = 'DELETE FROM ' . $this->tablename . ' WHERE ID = ' . $r[CLX_NUM_ZERO];
                ClxCommon::$db->executeQuery($sql);
            }
        }
        return alert($contents);
    }

    function getFileLink($fileid) {
        $windowname = 'Download';
        $contents = '';
        if (empty($fileid)) {
            return $contents;
        }
        $filerecord = ClxCommon::$db->getRecord($this->tablename, 'ID = ' . $fileid);
        if (!empty($fileid)) {
            $mimerec = ClxCommon::$db->getRecord('CLX_SD_MIMETYPES', 'IDCODE=' . singleQuote($filerecord['TYPE']));
            if (empty($mimerec['HTML_TAG'])) {
                $windowname = 'window' . $fileid;
                $url = 'javaScript:openWindow("' . $windowname . '", "window.php?to=SDFL&ac=SHOW_FILE&ff=' . $fileid . '", 800, 600);';
                $contents = '<page t=sproc_class_sd></page><a href=' . singleQuote($url) . '>' . $filerecord['NAME'] . '</a>';
            } else {
                $contents = $mimerec['HTML_TAG'];
                $contents = str_replace('[PATHNAME]', 'window.php?to=SDFL&ac=SHOW_FILE&ff=' . $fileid, $contents);
            }
        }
        return $contents;
    }

    /**
     * Dump to file to screen
     */
    function showFile($fileid) {

        if (empty($fileid) || !is_numeric($fileid)) {
            throw new Exception('Invalid file identification');
        }
        $filerec = ClxCommon::$db->getRecord($this->tablename, 'ID=' . $fileid);
        if (empty($filerec)) {
            throw new Exception('File not found');
        }

        $mimerec = ClxCommon::$db->getRecord('CLX_SD_MIMETYPES', 'IDCODE=' . singleQuote($filerec['TYPE']));

        if (ob_get_contents())
			ob_clean();
        
        UserResponse::setCustomHeaders(true);
        UserResponse::setContentType($mimerec['IDCODE']);
        UserResponse::setContentDisposition('inline');
        UserResponse::setFilename($filerec['NAME']);
        UserResponse::setPragma('no-cache');
        UserResponse::setExpires('0');
        UserResponse::setCacheControl('private');
        $contents = '';

        //E:C AJUSTE PARA QUE EL SISTEMA PUEDA MOSTRAR ARCHIVOS ADJUNTOS FUERA DE LA RUTA POR DEFECTO
        $sdfl_clx_root_sd_files = (SDFL_CLX_ROOT_SD_FILES!='' ? trim(SDFL_CLX_ROOT_SD_FILES) : 'null');
        $PathToWebSite='';
        if ($sdfl_clx_root_sd_files!='null' && $sdfl_clx_root_sd_files!=''){
            //le asignamos la ruta alterna definida en el parametro
            //forzamos la descarga del archivo cuando el mismo esta fuera de la ruta publica  del servidor
            UserResponse::setContentDisposition('attachment');
            $PathToWebSite = $sdfl_clx_root_sd_files;
        }else{
            $PathToWebSite = kPathToWebSite;
        }        
        if ($mimerec['IDCODE']=='application/pdf'){
            echo ' ';
        }
        if (empty($filerec['FILENAME'])) {
            $contents = stripslashes($filerec['CONTENTS']);
        } elseif (strpos(lowercase($filerec['NAME']), 'http') !== false) {
            $contents = file_get_contents($filerec['FILENAME']);
        } else {
            $fd = new clx_file($PathToWebSite . $filerec['FILENAME'], 'r');
            $fd->open();
            if (!$fd->hasError()) {
                $contents = $fd->read();
                $fd->close();
            } else {
                $BUGS = &getTool('BUGS');
                $BUGS->report(E_USER_WARNING, 'Unable to read file, check folder or file permissions', 'tools/sd_files.php', '601', array('filename' => kPathToWebSite . $filerec['FILENAME']), CLX_NUM_ONE, '');
            }
        }
        return $contents;
    }

    function showFileCall($ff) {
        try {
            $contents = $this->showFile($ff);
        } catch (Exception $ex) {
            header('HTTP/1.0 404 ' + $ex->getMessage());
            $contents = '';
        }
        return $contents;
    }

}
