<?php

/**
 * Tool Class
 * For more information contact  support@alenet.com
 *
 * @author       $Author: mchirino $
 * @copyright    Alenet, Inc.
 * @since        $Date: 2019-01-28 15:40:31 +0000 (Mon, 28 Jan 2019) $
 * @version        $Rev: 10506 $
 *
 * CHANGE LOG
 *  - Hidden field added property
 *  - Oracle uppercase record changed
 *  - Added 0 as default create display size
 *  Included label. Tool code changed from UIFC->UISC due to conflict
 */
class ui_srch_list_crits extends clx_tool {

    // CONSTRUCTOR //
    function __construct() {
        $this->code = 'UISC';
        $this->name = 'Form Search listing criterias';
        $this->system = 'Interface';
        $this->version = '0.5.5.0';


        $this->accesskey = 'UIFM';

        $this->tablename = 'CLX_UI_SRCH_LIST_CRITS';
        $this->record_name_singular = 'panel field';
        $this->record_name_plural = 'panel fields';
        $this->primary_field = 'PFID';
        $this->secondary_field = 'QRY_FIELDNAME';
        $this->primary_key = array('PFID', 'QRY_FIELDNAME');
        $this->operators = array(
            'EQ' => array('GEN_OPE_EQUAL', '=VALUE'),
            'NQ' => array('GEN_OPE_NOTEQUAL', '<>VALUE'),
            'LS' => array('GEN_OPE_LESS', '<VALUE'),
            'LQ' => array('GEN_OPE_LESS_EQUAL', '<=VALUE'),
            'GQ' => array('GEN_OPE_GREATER_EQUAL', '>=VALUE'),
            'GT' => array('GEN_OPE_GREATER', '>VALUE'),
            'IN' => array('GEN_OPE_IN', 'IN (VALUE) '),
            'NI' => array('GEN_OPE_NOTIN', 'NOT IN (VALUE)'),
            'SW' => array('GEN_OPE_STARTS_WITH', 'LIKE ' . singleQuote('VALUE%')),
            'EW' => array('GEN_OPE_ENDS_WITH', 'LIKE ' . singleQuote('%VALUE')),
            'CT' => array('GEN_OPE_CONTAINS', 'LIKE ' . singleQuote('%VALUE%')),
            'EX' => array('GEN_OPE_EXPRESSIONS', '(VALUE)'),
            'IS' => array('GEN_OPE_IS', 'IS (VALUE)'),
            'IT' => array('GEN_OPE_ISNOT', 'IS NOT (VALUE)')
        );
    }

    function toolTableStructure() {
        $fields = array();
        $fields['ID'] = array('type' => 'int(11) unsigned', 'null' => 'NO', 'properties' => 'auto_increment', 'index' => 'PRI');
        $fields['PFID'] = array('type' => 'int(11) unsigned', 'null' => 'NO', 'properties' => '', 'default' => CLX_NUM_ZERO);
        $fields['QRY_FIELDNAME'] = array('type' => 'varchar(64)', 'null' => 'NO', 'properties' => '');
        $fields['FRM_FIELDNAME'] = array('type' => 'varchar(64)', 'null' => 'YES', 'properties' => '');
        $fields['OPERATOR'] = array('type' => 'varchar(4)', 'null' => 'NO', 'default' => '' . singleQuote('E') . '', 'default_raw' => 'EQ', 'properties' => '');
        $fields['VALUE'] = array('type' => 'varchar(255)', 'null' => 'YES', 'properties' => '');
        $indexes = array();
        $ipf = 'CLX' . $this->code;
        $indexes[$ipf . '01'] = array('index' => 'PRI', 'fieldname' => 'ID');
        $indexes[$ipf . '02'] = array('index' => 'MUL', 'fieldname' => 'PFID');
        $indexes[$ipf . '03'] = array('index' => 'MUL', 'fieldname' => 'QRY_FIELDNAME');
        $indexes[$ipf . '04'] = array('index' => 'MUL', 'fieldname' => 'FRM_FIELDNAME');
        return array('idcode' => $this->code,
            'idname' => $this->name,
            'name' => $this->tablename,
            'type' => 'InnoDB',
            'fields' => $fields,
            'indexes' => $indexes);
    }

//-- F O R M    P A N E L    F I E L D S
    function unpack($parameters) {
        $parameters = array_values(array_merge(array('package' => null, 'id' => ''), $parameters));
        list($package, $panel_field_id) = $parameters;
        $record = $this->unpackRecord($package);
        $record['PFID'] = $panel_field_id;
        $record['QRY_FIELDNAME'] = uppercase($record['QRY_FIELDNAME']);
        $record['FRM_FIELDNAME'] = uppercase($record['FRM_FIELDNAME']);
        ClxCommon::$db->createRecordVoid($this->tablename, $record, false);
    }

    function makepanel($panelid = null, $idlist = null) {
        if ($this->getAccess()->canRead()) {
            $t = time();
            $to = $this->code;
            $here = 'tool.php?to=' . $to . '&t=' . $t;
            $form = new htmlForm(CLX_PROP_ACTION, 'tool.php');
            $form->setProp(CLX_PROP_ID, 'UISCPanel');
            $form->addArgument(CLX_PROP_TO, $this->code);
            $form->addArgument('panelid', $panelid);
            $form->addArgument('ac', 'NOT_DEFINED');
            $selectclause = 'FC.*, PF.FIELDNAME, PF.SRCH_LISTING ';
            $fromclause = $this->tablename . ' FC, CLX_UI_FORM_PANEL_FIELDS PF';
            $whereclause = 'PF.ID = FC.PFID AND FC.PFID IN (' . $idlist . ')';
            $sortclause = '';
            $page = new xPaging();
            $page->setProp('rpp', CLX_NUM_TEN_THOUSAND);
            $page->setProp('printheaders', false);
            $page->pageInit($form->getProp(CLX_PROP_ID), $here, $selectclause, $fromclause, $whereclause, $sortclause, '');
            $table = new htmlTable(CLX_WIDTH_7);
            $table->setProps(array(CLX_PROP_CLASS => CLX_CLASS_LIST, CLX_PROP_WIDTH => CLX_PROP_100PCT));
            $page->header($table);
            $table->addCells(CLX_CLASS_HEADERS, array(dict('GEN_LBL_SEL'), dict('GEN_LBL_FIELDNAME'), dict('GEN_LBL_SRCH_LISTING'), dict('GEN_LBL_QRY_FIELDNAME'), dict('GEN_LBL_OPERATOR'), dict('GEN_LBL_FRM_FIELDNAME'), dict('GEN_LBL_VALUE')));
            $i = CLX_INIT_ZERO;
            while ($record = $page->nextRecord()) {
                $recordid = $record['ID'];
                $select = new htmlCheckbox('selector_' . $recordid, false);
                $select->setProp('onclick', 'selectButtons(' . singleQuote($form->getProp(CLX_PROP_ID)) . ');');
                $table->addCell($select->source(), array(CLX_PROP_CLASS => CLX_CLASS_FOOTERS, CLX_PROP_WIDTH => '40', CLX_PROP_ALIGN => 'center'));
                $table->addCells(($i % CLX_NUM_TWO == CLX_NUM_ZERO) ? CLX_CLASS_DATA : CLX_CLASS_DATAALT, array(
                    $record['FIELDNAME'],
                    $record['SRCH_LISTING'],
                    $record['QRY_FIELDNAME'],
                    dict($this->operators[$record['OPERATOR']][CLX_NUM_ZERO]),
                    $record['FRM_FIELDNAME'],
                    $record['VALUE']));
                $i++;
            }
            $buttons = '';
            if ($this->getAccess()->canModify()) {
                $buttons .=htmlButton::getButton('submit', dict('GEN_LBL_MODIFY'), array('action' => 'MODIFY', 'form' => $form->getProp(CLX_PROP_ID), CLX_PROP_ID => 'MULT'))->source();
            }
            if ($this->getAccess()->canDisplay()) {
                $buttons .=htmlButton::getButton('submit', dict('GEN_LBL_CREATE'), array('action' => 'CREATE', 'form' => $form->getProp(CLX_PROP_ID), CLX_PROP_ID => 'ON'))->source();
            }
            if ($this->getAccess()->canDelete()) {
                $buttons .=htmlButton::getButton('submit', dict('GEN_LBL_DELETE'), array('action' => 'DELETE', 'form' => $form->getProp(CLX_PROP_ID), CLX_PROP_ID => 'MULT'))->source();
            }
            $form->addContents($page->footers($table, $buttons));
            $modulecode = '';
            return buildPanelInterface(dict('TOOL_UISC_TITLE_LIST'), $modulecode . 'TABLES', $form->source(), CLX_NUM_MINUS_1);
        } else {
            return CLX_CHAR_EMPTY_STRING;
        }
    }

    function editForm($ac, $idlist, $record, $msg) {
        $modify = (UserRequest::$request['ac'] === 'MODIFY');
        if (empty($modify)) {
            $callingpage = peekHistory();
            $record['UISC_pfid'] = $callingpage['ownerid'];
            unset($callingpage);
            if (ClxCommon::$db->getField('CLX_UI_FORM_PANEL_FIELDS', 'SRCH_LISTING', 'ID = ' . $record['UISC_pfid']) === '') {
                return alert(dict('GEN_ERR_SRCH_LISTING_NOT_FOUND'), dict('GEN_LBL_RETURN'));
            }
        }
        $sql = 'SELECT UIFP.FORMCODE FROM CLX_UI_FORM_PANELS UIFP, CLX_UI_FORM_PANEL_FIELDS UIFF WHERE UIFF.PANELID = UIFP.IDCODE AND UIFF.ID = ' . $record['UISC_pfid'];
        $formcode = ClxCommon::$db->getOne($sql);
        // Query Fields -------------------------
        $sql = 'SELECT CONCAT(CONCAT(CONCAT(CONCAT(T.MODULECODE, ' . singleQuote('_') . '), T.IDNAME), ' . singleQuote('.') . '), TF.FIELDNAME) FIELDNAME, CONCAT(CONCAT(T.IDCODE, ' .singleQuote('.'). '), TF.FIELDNAME) FIELDCODE FROM CLX_DB_QUERIES Q, CLX_DB_QUERY_TABLES QT, CLX_DB_TABLEFIELDS TF, CLX_UI_LISTINGS UILG, CLX_UI_FORM_PANEL_FIELDS UIFF, CLX_DB_TABLES T  WHERE UILG.IDCODE = UIFF.SRCH_LISTING AND UILG.QUERYCODE = Q.IDCODE  AND Q.IDCODE = QT.QUERYCODE AND QT.TABLECODE = TF.TABLECODE AND TF.TABLECODE = T.IDCODE AND UIFF.ID = ' . $record['UISC_pfid'] . ' ORDER BY QT.TABLECODE ASC, TF.SEQNUM ASC';
        $fields = ClxCommon::$db->executeQuery($sql);
        $queryfields = new htmlSelect($this->code . '_qry_fieldname');
        while ($field = ClxCommon::$db->getNextRecord($fields)) {
            $queryfields->addItem($field['FIELDNAME'], $field['FIELDCODE'], ($field['FIELDCODE'] == $record[$this->code . '_qry_fieldname']));
        }

        // Form Fields -------------------------
        switch (kDatabaseType) {
		case 'oci': // Oracle
                $sql = 'SELECT CONCAT(CONCAT(F.IDCODE, ' .singleQuote('.'). '), TB.FIELDNAME) FIELDNAME, CONCAT(CONCAT(IFNULL(UIFF.TABLECODE, TB.TABLECODE), ' . singleQuote('_') . '), TB.FIELDNAME) FIELDCODE FROM CLX_UI_FORMS F, CLX_UI_FORM_PANELS UIFP, CLX_UI_FORM_PANEL_FIELDS UIFF, CLX_DB_TABLES T, CLX_DB_TABLEFIELDS TB, CLX_DB_MODULES M WHERE F.IDCODE=UIFP.FORMCODE AND UIFF.PANELID = UIFP.IDCODE AND F.TABLECODE = TB.TABLECODE AND TB.FIELDNAME = UIFF.FIELDNAME AND TB.TABLECODE = T.IDCODE AND T.MODULECODE = M.IDCODE AND F.IDCODE = ' . singleQuote($formcode) . ' UNION ALL SELECT CONCAT(CONCAT(F.IDCODE, ' .singleQuote('.'). '), TB.FIELDNAME) FIELDNAME, CONCAT(CONCAT(IFNULL(UIFF.TABLECODE, TB.TABLECODE), ' . singleQuote('_') . '), TB.FIELDNAME) FIELDCODE FROM CLX_UI_FORMS F, CLX_UI_FORM_PANELS UIFP, CLX_UI_FORM_PANEL_FIELDS UIFF, CLX_DB_TABLES T, CLX_DB_TABLEDFIELDS TB, CLX_DB_MODULES M WHERE F.IDCODE=UIFP.FORMCODE AND UIFF.PANELID = UIFP.IDCODE AND F.TABLECODE = TB.TABLECODE AND TB.FIELDNAME = UIFF.FIELDNAME AND TB.TABLECODE = T.IDCODE AND T.MODULECODE = M.IDCODE AND F.IDCODE = ' . singleQuote($formcode);
                //$sql = 'SELECT CONCAT(CONCAT(F.IDCODE, '.'), TB.FIELDNAME) FIELDNAME, CONCAT(CONCAT(IFNULL(UIFF.TABLECODE, TB.TABLECODE),'_' ), TB.FIELDNAME) FIELDCODE FROM CLX_UI_FORMS F, CLX_UI_FORM_PANELS UIFP, CLX_UI_FORM_PANEL_FIELDS UIFF, CLX_DB_TABLES T, CLX_DB_TABLEFIELDS TB, CLX_DB_MODULES M WHERE F.IDCODE=UIFP.FORMCODE AND UIFF.PANELID = UIFP.IDCODE AND F.TABLECODE = TB.TABLECODE AND TB.FIELDNAME = UIFF.FIELDNAME AND TB.TABLECODE = T.IDCODE AND T.MODULECODE = M.IDCODE AND F.IDCODE = ' . singleQuote($formcode) . ' UNION ALL SELECT CONCAT(CONCAT(F.IDCODE, '.'), TB.FIELDNAME) FIELDNAME, CONCAT(CONCAT(IFNULL(UIFF.TABLECODE, TB.TABLECODE), '_'), TB.FIELDNAME) FIELDCODE FROM CLX_UI_FORMS F, CLX_UI_FORM_PANELS UIFP, CLX_UI_FORM_PANEL_FIELDS UIFF, CLX_DB_TABLES T, CLX_DB_TABLEDFIELDS TB, CLX_DB_MODULES M WHERE F.IDCODE=UIFP.FORMCODE AND UIFF.PANELID = UIFP.IDCODE AND F.TABLECODE = TB.TABLECODE AND TB.FIELDNAME = UIFF.FIELDNAME AND TB.TABLECODE = T.IDCODE AND T.MODULECODE = M.IDCODE AND F.IDCODE = ' . singleQuote($formcode);
                    break;
                default: // MySQL Version all others
                $sql = 'SELECT CONCAT(CONCAT(F.IDCODE, ' .singleQuote('.'). '), TB.FIELDNAME) FIELDNAME, CONCAT(CONCAT(IFNULL(NULLIF(UIFF.TABLECODE,' . singleQuote(CLX_CHAR_EMPTY_STRING) . ' ), TB.TABLECODE), ' . singleQuote('_') . '), TB.FIELDNAME) FIELDCODE FROM CLX_UI_FORMS F, CLX_UI_FORM_PANELS UIFP, CLX_UI_FORM_PANEL_FIELDS UIFF, CLX_DB_TABLES T, CLX_DB_TABLEFIELDS TB, CLX_DB_MODULES M WHERE F.IDCODE=UIFP.FORMCODE AND UIFF.PANELID = UIFP.IDCODE AND F.TABLECODE = TB.TABLECODE AND TB.FIELDNAME = UIFF.FIELDNAME AND TB.TABLECODE = T.IDCODE AND T.MODULECODE = M.IDCODE AND F.IDCODE = ' . singleQuote($formcode) . ' UNION ALL SELECT CONCAT(CONCAT(F.IDCODE, ' .singleQuote('.'). '), TB.FIELDNAME) FIELDNAME, CONCAT(CONCAT(IFNULL(NULLIF(UIFF.TABLECODE,' . singleQuote(CLX_CHAR_EMPTY_STRING) . '), TB.TABLECODE), ' . singleQuote('_') . '), TB.FIELDNAME) FIELDCODE FROM CLX_UI_FORMS F, CLX_UI_FORM_PANELS UIFP, CLX_UI_FORM_PANEL_FIELDS UIFF, CLX_DB_TABLES T, CLX_DB_TABLEDFIELDS TB, CLX_DB_MODULES M WHERE F.IDCODE=UIFP.FORMCODE AND UIFF.PANELID = UIFP.IDCODE AND F.TABLECODE = TB.TABLECODE AND TB.FIELDNAME = UIFF.FIELDNAME AND TB.TABLECODE = T.IDCODE AND T.MODULECODE = M.IDCODE AND F.IDCODE = ' . singleQuote($formcode);
                    break;
	}
        $fields = ClxCommon::$db->executeQuery($sql);
        $formfields = new htmlSelect($this->code . '_frm_fieldname');
        $formfields->addItem('', '', (empty($record[$this->code . '_frm_fieldname'])));
        while ($field = ClxCommon::$db->getNextRecord($fields)) {
            $formfields->addItem($field['FIELDNAME'], $field['FIELDCODE'], ($field['FIELDCODE'] == $record[$this->code . '_frm_fieldname']));
        }
        //The listing
        $title = ($modify) ? dict('TOOL_UISC_TITLE_MODIFY') : dict('TOOL_UISC_TITLE_CREATE');
        $form = new htmlForm(CLX_PROP_ACTION, 'tool.php');
        $form->addArgument(CLX_PROP_TO, $this->code);
        $form->addArgument('ac', $ac);
        $form->addArgument('confirm', '1');
        $form->addArgument($this->code . '_id', $record[$this->code . '_id']);
        $form->addArgument('idlist', toString($idlist));
        $form->addArgument($this->code . '_pfid', $record[$this->code . '_pfid']);
        $table = new htmlTable(CLX_WIDTH_TWO);
        $table->setProps(array(CLX_PROP_CLASS => CLX_CLASS_FORM, CLX_PROP_WIDTH => CLX_PROP_100PCT));
        $table->addCell($title, array(CLX_PROP_CLASS => CLX_CLASS_TITLE, CLX_PROP_COLSPAN => CLX_WIDTH_TWO));
        if (!empty($msg[$this->code . '_qry_fieldname']))
            $table->addCell($msg[$this->code . '_qry_fieldname'], array(CLX_PROP_CLASS => CLX_CLASS_ALERT, CLX_PROP_COLSPAN => $table->width));
        $table->addData(dict('TOOL_UIFI_PROMPT_QUERYFIELD') . ':', $queryfields->source());
        // The operators
        $pop = new HTMLSelect($this->code . '_operator');
        foreach ($this->operators as $key => $value) {
            $pop->addItem(dict($value[CLX_NUM_ZERO]), $key, ($key == $record[$this->code . '_operator']));
        }
        $table->addData(dict('GEN_LBL_OPERATORS') . ':', $pop->source());
        $field = new htmlTextArea('UISC_value', $record['UISC_value'], CLX_NUM_7, CLX_NUM_50);
        if (!empty($msg[$this->code . '_frm_fieldname']))
            $table->addCell($msg[$this->code . '_frm_fieldname'], array(CLX_PROP_CLASS => CLX_CLASS_ALERT, CLX_PROP_COLSPAN => $table->width));
        $table->addData(dict('TOOL_UIFI_PROMPT_FORMFIELDORVALUE') . ':', $formfields->source() . '<br>' . dict('GEN_LBL_OR') . '<br>' . $field->source());
        $form->addContents($table->source());
        $buttons = '';
        $table = new htmlTable(CLX_WIDTH_TWO);
        $table->setProps(array(CLX_PROP_CLASS => CLX_CLASS_FORM, CLX_PROP_WIDTH => CLX_PROP_100PCT));
        $buttons .= htmlButton::getButton('submit', dict('GEN_LBL_SAVE'), array('action' => $ac, 'form' => $form->getProp(CLX_PROP_ID)))->source();
        $c = new htmlButtonCancel();
        $buttons .= $c->source();
        $table->addCell($buttons, array(CLX_PROP_CLASS => CLX_CLASS_FOOTERS, CLX_PROP_COLSPAN => CLX_WIDTH_TWO));
        $form->addContents($table->source());
        return $form->source();
    }

    function checkForm(&$msg) {
        $msg = array();
        if (strlen(UserRequest::$post[$this->code . '_qry_fieldname']) == CLX_NUM_ZERO) {
            $msg[$this->code . '_qry_fieldname'] = dict('GEN_ERR_REQUIRED');
        }
        if (strlen(UserRequest::$post[$this->code . '_frm_fieldname']) == CLX_NUM_ZERO && strlen(UserRequest::$post[$this->code . '_value']) == CLX_NUM_ZERO) {
            $msg[$this->code . '_frm_fieldname'] = dict('GEN_ERR_REQUIRED');
        }
        if (strlen(UserRequest::$post[$this->code . '_pfid']) == CLX_NUM_ZERO) {
            $msg[$this->code . '_pfid'] = dict('GEN_ERR_REQUIRED');
        }
        if (ClxCommon::$db->oneExistsRecords($this->tablename, 'ID <> ' . singleQuote(UserRequest::$post[$this->code . '_id']) . '  AND PFID = ' . singleQuote(UserRequest::$post[$this->code . '_pfid']) . ' AND QRY_FIELDNAME = ' . singleQuote(UserRequest::$post[$this->code . '_qry_fieldname']) . ' AND FRM_FIELDNAME = ' . singleQuote(UserRequest::$post[$this->code . '_frm_fieldname'])) <> CLX_NUM_ZERO) {
            $msg[$this->code . '_qry_fieldname'] = dict('GEN_ERR_ALREADY_EXISTS');
        }
        return count($msg) == CLX_NUM_ZERO;
    }

    function confirmDeleteFill(&$table, $record) {
        $table->addData(dict('TOOL_UIFI_PROMPT_QUERYFIELD') . ':', $record[$this->code . '_qry_fieldname']);
        $table->addData(dict('TOOL_UIFI_PROMPT_FORMFIELD') . ':', $record[$this->code . '_frm_fieldname']);
    }

    function loadToolRecord($recordid) {
        $values = ClxCommon::$db->getRecord($this->tablename, 'ID= ' . $recordid);
        $record = array();
        $record['UISC_id'] = $recordid;
        $record['UISC_pfid'] = $values['PFID'];
        $record['UISC_qry_fieldname'] = $values['QRY_FIELDNAME'];
        $record['UISC_operator'] = $values['OPERATOR'];
        $record['UISC_frm_fieldname'] = $values['FRM_FIELDNAME'];
        $record['UISC_value'] = $values['VALUE'];
        return $record;
    }

    function createRecord_() {
        $record = array();
        $record['PFID'] = UserRequest::$post['UISC_pfid'];
        $record['QRY_FIELDNAME'] = UserRequest::$post['UISC_qry_fieldname'];
        $record['FRM_FIELDNAME'] = UserRequest::$post['UISC_frm_fieldname'];
        $record['OPERATOR'] = UserRequest::$post['UISC_operator'];
        $record['VALUE'] = UserRequest::$post['UISC_value'];
        $id = ClxCommon::$db->createRecord($this->tablename, $record, false);
        $this->versionIncrease('ID', $id);
    }

    function saveRecord() {
        $record = array();
        $record['QRY_FIELDNAME'] = UserRequest::$post['UISC_qry_fieldname'];
        $record['FRM_FIELDNAME'] = UserRequest::$post['UISC_frm_fieldname'];
        $record['OPERATOR'] = UserRequest::$post['UISC_operator'];
        $record['VALUE'] = UserRequest::$post['UISC_value'];
        ClxCommon::$db->updateRecord($this->tablename, $record, 'ID=' . UserRequest::$post['UISC_id']);
        $this->versionIncrease('ID', UserRequest::$post['UISC_id']);
    }

    function deleteRecordSk($id) {
        $this->versionIncrease('ID', $id);
        ClxCommon::$db->deleteRecords($this->tablename, 'ID=' . $id);
    }

    function defaultRecord() {
        $record = array();
        $record['UISC_id'] = CLX_INIT_ZERO;
        $record['UISC_uifp_id'] = CLX_INIT_ZERO;
        $record['UISC_qry_fieldname'] = '';
        $record['UISC_frm_fieldname'] = '';
        $record['UISC_operator'] = 'EQ';
        $record['UISC_value'] = '';
        return $record;
    }

    function versionIncrease($fieldname, $fieldvalue) {
        $pfid = ClxCommon::$db->getField($this->tablename, 'PFID', $fieldname . '=' . singleQuote($fieldvalue));
        $formcode = ClxCommon::$db->getField('CLX_UI_FORM_PANELS', 'FORMCODE', 'ID=' . $pfid);
        $UIFM = &getTool('UIFM');
        $UIFM->versionIncrease('IDCODE', $formcode);
    }

    function buildOperator($operator, $fieldname, $value, $is_numeric = false) {
        ClxFreeReturn($is_numeric);
        if ($operator === 'EX') {
            return '(' . $value . ')';
        }
        foreach ($this->operators as $key => $val) {
            if ($operator == $key) {
                if ($operator === 'IN' || $operator === 'NI') {
                    $value = str_replace(',', '' . singleQuote(CLX_CHAR_EMPTY_STRING) . '', $value);
                }
                if (is_numeric($value)) {
                    $value = str_replace('VALUE', $value, $val[CLX_NUM_ONE]);
                } else {
                    $value = str_replace('VALUE', singleQuote($value), $val[CLX_NUM_ONE]);
                }
                return $fieldname . ' ' . $value;
            }
        }
        return $fieldname . ' IS NULL ';
    }

}
